@@ -11,6 +11,11 @@ $g_examplebgcolor="#F5F5F5";
 $g_notetablebgcolor="#F0F0F0";
 $g_notetextcolor="#909090";
 $g_syntaxcolor="#802000";
+
+$g_kvssyntaxcolor="#802000";
+$g_kvstypenamecolor="#808080";
+$g_kvstypedelimiterscolor="#8080a0";
+
 $g_bodybgcolor="#FFFFFF";
 $g_bodytextcolor="#000000";
 $g_bodylinkcolor="#000000";
@@ -33,6 +38,7 @@ $g_fileextension=".html";
 #$g_headerborderdarkcolor="#000000";
 $g_headerbgcolor="#FFFFFF";
 
+
 $g_prefixes{'command'}="cmd";
 $g_prefixes{'function'}="fnc";
 $g_prefixes{'event'}="event";
@@ -41,7 +47,7 @@ $g_prefixes{'class'}="class";
 $g_prefixes{'module'}="module";
 $g_prefixes{'widget'}="widget";
 
-$g_version = "3.1.0";
+$g_version = "3.2.0.99";
 $g_filehandle="";
 $g_shortsIdx{"keyterms"}=0;
 $g_directory = "";
@@ -145,7 +151,7 @@ sub print_footer
 
 sub print_tablestart
 {
-	print $g_filehandle "<table width=\"100%\" cellpadding=\"1\" cellspacing=\"1\" border=\"0\"\n";
+	print $g_filehandle "<table width=\"100%\" cellpadding=\"3\" cellspacing=\"1\" border=\"0\"\n";
 	print $g_filehandle "<tr><td bgcolor=\"$g_tablebgcolor\">\n";
 }
 
@@ -244,10 +250,31 @@ sub make_single_token
 
 sub make_single_line
 {
-	$_[0] =~ s/^[ 	]*//g;
+	$_[0] =~ s/^[ 	\n]*//g;
 	$_[0] =~ s/[	 \n]*$//gs;
 }
 
+sub make_syntax
+{
+	$_[0] =~ s/^[ 	\n]*//g;
+	$_[0] =~ s/[	 \n]*$//gs;
+	my @arry;
+	@arry = split('\n',$_[0]);
+	$_[0] = "";
+	foreach(@arry)
+	{
+		$_ =~ s/^[ 	\n]*//g;
+		if($_ ne "")
+		{
+			if($_[0] ne "")
+			{
+				$_[0] = "$_[0]\n$_";
+			} else {
+				$_[0] = "$_";
+			}
+		}
+	}
+}
 
 sub extract_keyterms
 {
@@ -276,6 +303,7 @@ sub extract_keyterms
 			$docfilename=~ s/\@doc://g;
 			$docfilename=~ s/\n//g;
 			$docfilename=~ s/\r//g;
+			$docfilename=~ s/([a-zA-Z_]*)/\L\1/g;
 
 			undef %parts;
 			$part = "";
@@ -398,83 +426,6 @@ sub extract_keyterms
 	close(CPPFILE);
 }
 
-sub substitute_keyterms_old_unused_to_be_removed_sooner_or_later_oh_what_a_long_function_name
-{
-	my(@lines);
-	my(@tmp);
-	my($left);
-	my($right);
-	my($line);
-	my($term);
-
-	# Kinda complex keyword substitution routine
-
-
-	# For each keyterm we have
-	for $term (@g_keytermsSorted)
-	{
-		# If the doc we're scanning isn't the keyword target
-		if($_[1] ne $g_keyterms{$term})
-		{
-			$termclean=$g_keytermsclean{$term};
-
-			# If the doc matches the keyterm at least once
-			if($_[0] =~ /$termclean/)
-			{
-				# First we split the doc in lines
-				@lines = split(/\n/,$_[0]);
-				$_[0]="";
-	
-				$ref=$g_keyterms{$term};
-
-				$skipIt = 0;
-
-				for(@lines)
-				{
-					if($_ ne "")
-					{
-						@tmp = split(/</,$_);
-						$line = "";
-						$first=1;
-						for(@tmp)
-						{
-							if($skipIt)
-							{
-								if(/[A-Za-z0-9\"]>/)
-								{
-									$skipIt=0 if /^[ ]*\/a[ ]*>/;
-								}
-							}
-
-							if($skipIt)
-							{
-								$line="$line<$_";
-							} else {
-								if(/[A-Za-z0-9\"]>/)
-								{
-									if(/^[ ]*a[ 	][ 	]*h/)
-									{
-										$skipIt=1;
-										$first ? $line="$line$_" : $line="$line<$_";
-									} else {
-										($left,$right) = split(/>/,$_);
-										$right=~s/([^A-Za-z0-9])($termclean)([^A-Za-z0-9])/$1<a href=\"$ref\">$2<\/a>$3/ig;
-										$line="$line<$left>$right";
-									}
-								} else {
-									$_=~s/([^A-Za-z0-9])($termclean)([^A-Za-z0-9])/$1<a href=\"$ref\">$2<\/a>$3/ig;
-									$line="$line$_";
-								}
-								$first=0;
-							}
-						}
-						$_[0]="$_[0]$line\n";
-					}
-				}
-			}
-		}
-	}
-}
 
 
 sub substitute_keyterms
@@ -501,6 +452,8 @@ sub substitute_keyterms
 			if($_[0] =~ /$termclean/)
 			{
 				$ref=$g_keyterms{$term};
+				$ref =~ s/([a-zA-Z_]*)/\L\1/g;
+
 				@tmp = split(/</,$_[0]);
 				$_[0]="";
 				$first=1;
@@ -531,7 +484,7 @@ sub substitute_keyterms
 								$_[0] .= "<$left>$right";
 							}
 						} else {
-							$_=~s/([^A-Za-z0-9<>\+\-\=_])($termclean)([^A-Za-z0-9<>\+\-\=_])/$1<a href=\"$ref\">$2<\/a>$3/ig;
+							$_=~s/([^A-Za-z0-9<>\+\-\=_\.])($termclean)([^A-Za-z0-9<>\+\-\=_\.])/$1<a href=\"$ref\">$2<\/a>$3/ig;
 							$_[0] .= $_;
 						}
 						$first=0;
@@ -543,6 +496,64 @@ sub substitute_keyterms
 }
 
 
+sub build_usage_from_kvs_syntax_line
+{
+	$_[0] =~ s/\&lt\;([A-Za-z0-9_]+):([A-Za-z0-9_]+)\&gt\;/\&lt\;\1\&gt\;/g;
+	$_[0] =~ s/\[([A-Za-z0-9_]+):([A-Za-z0-9_]+)\]/\[;\1\]/g;
+	$_[0] =~ s/^\&lt\;([A-Za-z0-9_]+)\&gt\;//g;
+}
+
+sub build_usage_from_kvs_syntax
+{
+	my @arry;
+	@arry = split('\n',$_[0]);
+	$tmp = "";
+	foreach(@arry)
+	{
+		build_usage_from_kvs_syntax_line($_);
+		if($tmp ne "")
+		{
+			$tmp = "$tmp\n$_";
+		} else {
+			$tmp = "$_";
+		}
+	}
+	return $tmp;
+}
+
+sub process_kvs_syntax_line
+{
+	$_[0] =~ s/\&lt\;([A-Za-z0-9_]+):([A-Za-z0-9_]+)\&gt\;/\&lt\;\1\<font color=\"$g_kvstypenamecolor\"\>:\2\<\/font\>\&gt\;/g;
+	$_[0] =~ s/\[([A-Za-z0-9_]+):([A-Za-z0-9_]+)\]/\[;\1\<font color=\"$g_kvstypenamecolor\"\>:\2\<\/font\>\]/g;
+	$_[0] =~ s/^\&lt\;([A-Za-z0-9_]+)\&gt\;/\&lt\;\<font color=\"$g_kvstypenamecolor\"\>\1\<\/font\>\&gt\;/g;
+	$_[0] =~ s/^([A-Za-z0-9_\.]+)/\<b\>\1\<\/b\>/g;
+	$_[0] =~ s/(\$[A-Za-z0-9_\.]*)/\<b\>\1\<\/b\>/g;
+	$_[0] =~ s/\(/\<b\>\(<\/b\>/g;
+	$_[0] =~ s/\)/\<b\>\)<\/b\>/g;
+	$_[0] =~ s/\&lt\;/\<font color=\"$g_kvstypedelimiterscolor\"\>\&lt\;\<\/font\>/g;
+	$_[0] =~ s/\&gt\;/\<font color=\"$g_kvstypedelimiterscolor\"\>\&gt\;\<\/font\>/g;
+	$_[0] =~ s/\[/\<font color=\"$g_kvstypedelimiterscolor\"\>\[\<\/font\>/g;
+	$_[0] =~ s/\]/\<font color=\"$g_kvstypedelimiterscolor\"\>\]\<\/font\>/g;
+}
+
+sub process_kvs_syntax
+{
+	my @arry;
+	@arry = split('\n',$_[0]);
+	$_[0] = "";
+	foreach(@arry)
+	{
+		process_kvs_syntax_line($_);
+		if($_[0] ne "")
+		{
+			$_[0] = "$_[0]\n$_";
+		} else {
+			$_[0] = "$_";
+		}
+	}
+}
+
+
 
 
 sub process_body_line
@@ -577,23 +588,24 @@ sub process_body_line
 	$_[0] =~ s/\[\/example\]/\<\/font\>\<\/code\>\<\/pre\>\<\/td\>\<\/tr\>\<\/table\>/g;
 	$_[0] =~ s/\[comment\]/\<font color=\"$g_commentcolor\"\>/g;
 	$_[0] =~ s/\[\/comment\]/\<\/font\>/g;
-	$_[0] =~ s/\[doc\]([a-zA-Z0-9_]*)\[\/doc\]/\<a href=\"doc_\1$g_fileextension">\1\<\/a\>/g;
-	$_[0] =~ s/\[doc:([a-zA-Z0-9_]*)\]([a-zA-Z0-9_\-\&\;\. ]*)\[\/doc\]/\<a href=\"doc_\1$g_fileextension"\>\2\<\/a\>/g;
-	$_[0] =~ s/\[cmd\]([a-zA-Z0-9_\.]*)\[\/cmd\]/\<a href=\"cmd_\1$g_fileextension">\1\<\/a\>/g;
-	$_[0] =~ s/\[cmd:([a-zA-Z0-9_\.]*)\]([a-zA-Z0-9_\-\&\;\. ]*)\[\/cmd\]/\<a href=\"cmd_\1$g_fileextension"\>\2\<\/a\>/g;
-	$_[0] =~ s/\[fnc\]\$([a-zA-Z0-9_\.]*)\[\/fnc\]/\<a href=\"fnc_\1$g_fileextension">\$\1\<\/a\>/g;
-	$_[0] =~ s/\[fnc\]([a-zA-Z0-9_\.]*)\[\/fnc\]/\<a href=\"fnc_\1$g_fileextension">\$\1\<\/a\>/g;
-	$_[0] =~ s/\[fnc:\$([a-zA-Z0-9_\.]*)\]\$([a-zA-Z0-9_\-\&\;\. ]*)\[\/fnc\]/\<a href=\"fnc_\1$g_fileextension"\>\$\2\<\/a\>/g;
-	$_[0] =~ s/\[event\]\$([a-zA-Z0-9_]*)\[\/event\]/\<a href=\"event_\1$g_fileextension">\$\1\<\/a\>/g;
-	$_[0] =~ s/\[event:([a-zA-Z0-9_]*)\]([a-zA-Z0-9_]*)\[\/event\]/\<a href=\"event_\1$g_fileextension">\2\<\/a\>/g;
-	$_[0] =~ s/\[class\]([a-zA-Z0-9_]*)\[\/class\]/\<a href=\"class_\1$g_fileextension">\1\<\/a\>/g;
-	$_[0] =~ s/\[class:([a-zA-Z0-9_]*)\]([a-zA-Z0-9_ ]*)\[\/class\]/\<a href=\"class_\1$g_fileextension">\2\<\/a\>/g;
-	$_[0] =~ s/\[module:([a-zA-Z0-9_]*)\]([a-zA-Z0-9_ ]*)\[\/module\]/\<a href=\"module_\1$g_fileextension">\2\<\/a\>/g;
-	$_[0] =~ s/\[widget:([a-zA-Z0-9_]*)\]([a-zA-Z0-9_ ]*)\[\/widget\]/\<a href=\"widget_\1$g_fileextension">\2\<\/a\>/g;
-	$_[0] =~ s/\[classfnc:([a-zA-Z0-9_]*)\]\$([a-zA-Z0-9_]*)\[\/classfnc\]/\<a href=\"class_\1$g_fileextension#\2">\$\2\<\/a\>/g;
+	$_[0] =~ s/\[doc\]([a-zA-Z0-9_]*)\[\/doc\]/\<a href=\"doc_\L\1$g_fileextension">\1\<\/a\>/g;
+	$_[0] =~ s/\[doc:([a-zA-Z0-9_]*)\]([a-zA-Z0-9_\-\&\;\. ]*)\[\/doc\]/\<a href=\"doc_\L\1$g_fileextension"\>\2\<\/a\>/g;
+	$_[0] =~ s/\[cmd\]([a-zA-Z0-9_\.]*)\[\/cmd\]/\<a href=\"cmd_\L\1$g_fileextension">\1\<\/a\>/g;
+	$_[0] =~ s/\[cmd:([a-zA-Z0-9_\.]*)\]([a-zA-Z0-9_\-\&\;\. ]*)\[\/cmd\]/\<a href=\"cmd_\L\1$g_fileextension"\>\2\<\/a\>/g;
+	$_[0] =~ s/\[fnc\]\$([a-zA-Z0-9_\.]*)\[\/fnc\]/\<a href=\"fnc_\L\1$g_fileextension">\$\1\<\/a\>/g;
+	$_[0] =~ s/\[fnc\]\$([a-zA-Z0-9_\.]*)\(\)\[\/fnc\]/\<a href=\"fnc_\L\1$g_fileextension">\$\1()\<\/a\>/g;
+	$_[0] =~ s/\[fnc\]([a-zA-Z0-9_\.]*)\[\/fnc\]/\<a href=\"fnc_\L\1$g_fileextension">\$\1\<\/a\>/g;
+	$_[0] =~ s/\[fnc:\$([a-zA-Z0-9_\.]*)\]\$([a-zA-Z0-9_\-\&\;\. ]*)\[\/fnc\]/\<a href=\"fnc_\L\1$g_fileextension"\>\$\2\<\/a\>/g;
+	$_[0] =~ s/\[event\]\$([a-zA-Z0-9_]*)\[\/event\]/\<a href=\"event_\L\1$g_fileextension">\$\1\<\/a\>/g;
+	$_[0] =~ s/\[event:([a-zA-Z0-9_]*)\]([a-zA-Z0-9_]*)\[\/event\]/\<a href=\"event_\L\1$g_fileextension">\2\<\/a\>/g;
+	$_[0] =~ s/\[class\]([a-zA-Z0-9_]*)\[\/class\]/\<a href=\"class_\L\1$g_fileextension">\1\<\/a\>/g;
+	$_[0] =~ s/\[class:([a-zA-Z0-9_]*)\]([a-zA-Z0-9_ ]*)\[\/class\]/\<a href=\"class_\L\1$g_fileextension">\2\<\/a\>/g;
+	$_[0] =~ s/\[module:([a-zA-Z0-9_]*)\]([a-zA-Z0-9_ ]*)\[\/module\]/\<a href=\"module_\L\1$g_fileextension">\2\<\/a\>/g;
+	$_[0] =~ s/\[widget:([a-zA-Z0-9_]*)\]([a-zA-Z0-9_ ]*)\[\/widget\]/\<a href=\"widget_\L\1$g_fileextension">\2\<\/a\>/g;
+	$_[0] =~ s/\[classfnc:([a-zA-Z0-9_]*)\]\$([a-zA-Z0-9_]*)\[\/classfnc\]/\<a href=\"class_\L\1$g_fileextension#\2">\$\2\<\/a\>/g;
 	$_[0] =~ s/\[classfnc\]\$([a-zA-Z0-9_]*)\[\/classfnc\]/\<a href=\"#\1">\$\1\<\/a\>/g;
-	$_[0] =~ s/\[classsignal:([a-zA-Z0-9_]*)\]([a-zA-Z0-9_]*)\[\/classsignal\]/\<a href=\"class_\1$g_fileextension#\2">\2\<\/a\>/g;
-	$_[0] =~ s/\[classsignal\]([a-zA-Z0-9_]*)\[\/classsignal\]/\<a href=\"#\1">\1\<\/a\>/g;
+	$_[0] =~ s/\[classsignal:([a-zA-Z0-9_]*)\]([a-zA-Z0-9_]*)\[\/classsignal\]/\<a href=\"class_\L\1$g_fileextension#\2">\2\<\/a\>/g;
+	$_[0] =~ s/\[classsignal\]([a-zA-Z0-9_]*)\[\/classsignal\]/\<a href=\"#\L\1">\1\<\/a\>/g;
 	$_[0] =~ s/\[anchorlink:([a-zA-Z0-9_]*)\]/\<a href=\"#\1"\>/g;
 	$_[0] =~ s/\[\/anchorlink\]/\<\/a\>/g;
 	$_[0] =~ s/\[anchor:([a-zA-Z0-9_]*)\]/\<a name=\"\1"\>/g;
@@ -629,6 +641,7 @@ sub process_file
 			$docfilename=~ s/[ 	]*//g;
 			$docfilename=~ s/\@doc://g;
 			$docfilename=~ s/\n//g;
+			$docfilename=~ s/([a-zA-Z_]*)/\L\1/g;
 
 			undef %parts;
 			$part = "";
@@ -702,7 +715,15 @@ sub process_file
 			}
 
 			make_single_token($parts{'title'});
-			make_single_line($parts{'syntax'});
+			if($parts{'syntax'} ne "")
+			{
+				if($parts{'usage'} eq "")
+				{
+					$parts{'usage'} = build_usage_from_kvs_syntax($parts{'syntax'})
+				}
+			}
+			make_syntax($parts{'usage'});
+			make_syntax($parts{'syntax'});
 			make_single_line($parts{'parameters'});
 			make_single_line($parts{'inherits'});
 			make_single_token($parts{'short'});
@@ -748,7 +769,6 @@ sub process_file
 			{
 				substitute_keyterms($parts{'switches'},"$docfilename$g_fileextension");
 			}
-
 			if(open(DOCFILE,">$g_directory/$docfilename$g_fileextension"))
 			{
 				$g_filehandle=DOCFILE;
@@ -757,13 +777,13 @@ sub process_file
 				print_tablestart();
 
 				print_title($parts{'title'},$parts{'short'});
-				if($parts{'syntax'} ne "")
+				if($parts{'usage'} ne "")
 				{
-					print_entry("Syntax","<font color=\"$g_syntaxcolor\"><br><pre><code>$parts{'syntax'}</code></pre><br></font>");
+					print_entry("Usage","<font color=\"$g_syntaxcolor\"><pre><code>$parts{'usage'}</code></pre></font>");
 				}
 				if($parts{'parameters'} ne "")
 				{
-					print_entry("Parameters","<font color=\"$g_syntaxcolor\"><br><pre><code>$parts{'parameters'}</code></pre><br></font>");
+					print_entry("Parameters","<font color=\"$g_syntaxcolor\"><pre><code>$parts{'parameters'}</code></pre></font>");
 				}
 
 				print_entry("Inherits","$parts{'inherits'}");
@@ -910,6 +930,12 @@ sub process_file
 					print DOCFILE "  </tr>\n";
 				}
 
+				if($parts{'syntax'} ne "")
+				{
+					process_kvs_syntax($parts{'syntax'});
+					print_entry("Syntax Specification","<font color=\"$g_kvssyntaxcolor\"><pre><code>$parts{'syntax'}</code></pre></font>");
+				}
+
 				print_entry("Examples","$parts{'examples'}");
 				print_entry("See also","$parts{'seealso'}");
 				print_tableend();
@@ -979,7 +1005,7 @@ sub generate_indexes
 	######################################################
 	# generate some helper stuff (alphabetic index links)
 
-	@chars=("\$","a","b","c","d","e","f","g","h","i","j","k","l","m","n","o","p","q","r","s","t","u","w","v","x","y","z");
+	@chars=("\$","a","b","c","d","e","f","g","h","i","j","k","l","m","n","o","p","q","r","s","t","u","v","w","x","y","z");
 
 	$i = 0;
 
@@ -1075,6 +1101,7 @@ sub generate_indexes
 		while($i < $count)
 		{
 			($cmd,$short,$link) = split("<!>",$sortedCommands[$i]);
+			$link =~ s/([a-zA-Z_]*)/\L\1/g;
 			print_twocolumnbody("<a href=\"$link\">$cmd</a>",$short);
 			$i++;
 		}
@@ -1131,6 +1158,8 @@ sub generate_indexes
 					#$right=~ s/^$left//;
 					$left = substr($cmd,0,$number);
 					$right = substr($cmd,$number);
+					$link =~ s/([a-zA-Z_]*)/\L\1/g;
+
 
 					#print("RIGHT=$right,_=$_,LEFT=$left,UPCASE=$upcase\n");
 
@@ -1147,6 +1176,7 @@ sub generate_indexes
 				while($i < $count)
 				{
 					($cmd,$short,$link) = split("<!>",$sortedCommands[$i]);
+					$link =~ s/([a-zA-Z_]*)/\L\1/g;
 					if(($cmd =~ /^$_/) || ($cmd =~ /^$upcase/))
 					{
 						$j++;
@@ -1249,18 +1279,15 @@ if(open(DOCINDEX,">$g_directory/index$g_fileextension"))
 	print_twocolumnbody("<a href=\"doc_kvircintro$g_fileextension\">Introduction to KVIrc</a>","A couple of words about KVIrc");
 	
 	print_twocolumnsubtitle("Scripting Concepts: The KVS Manual");
-#	print_twocolumnbody("<a href=\"doc_kvs_introduction$g_fileextension\">KVS Overview</a>","Introduction to the KVIrc Scripting Language");
-#	print_twocolumnbody("<a href=\"doc_kvs_basicconcepts$g_fileextension\">Basic Concepts</a>","The first steps in the KVS world");
-#	print_twocolumnbody("<a href=\"doc_kvs_aliasesandfunctions$g_fileextension\">Aliases and Functions</a>","How to write aliases/functions");
-#	print_twocolumnbody("<a href=\"doc_kvs_codingtips$g_fileextension\">Coding Tips</a>","Some tips that may help you");
-#	print_twocolumnsubtitle("Old Scripting Concepts (Versions prior to 3.0.0 final)");
-	
-	print_twocolumnbody("<a href=\"doc_language_overview$g_fileextension\">Language overview</a>","Introduction to the KVIrc Scripting Language");
-	print_twocolumnbody("<a href=\"doc_datatypes$g_fileextension\">Variables</a>","Which types of variables are available and how to handle them");
+	print_twocolumnbody("<a href=\"doc_kvs_introduction$g_fileextension\">Introduction to KVS</a>","Introduction to the KVIrc Scripting Language");
+	print_twocolumnbody("<a href=\"doc_kvs_basicconcepts$g_fileextension\">Basic KVS Concepts</a>","The first steps in the KVS world");
+	print_twocolumnbody("<a href=\"doc_kvs_aliasesandfunctions$g_fileextension\">Aliases and Functions</a>","How to write aliases/functions");
+	print_twocolumnbody("<a href=\"doc_kvs_datatypes$g_fileextension\">Variables</a>","Which types of variables are available and how to handle them");
 	print_twocolumnbody("<a href=\"doc_operators$g_fileextension\">Operators</a>","Describes simple operations with variables");
-	print_twocolumnbody("<a href=\"doc_aliases$g_fileextension\">Aliases</a>","How to write functions in KVS ?");
 	print_twocolumnbody("<a href=\"doc_events$g_fileextension\">Events</a>","How to handle network events in KVS ?");
 	print_twocolumnbody("<a href=\"doc_objects$g_fileextension\">Objects</a>","Object oriented scripting");
+	print_twocolumnbody("<a href=\"doc_kvs_addons$g_fileextension\">Addons</a>","How to write nice addons for KVIrc");
+	print_twocolumnbody("<a href=\"doc_kvs_codingtips$g_fileextension\">Coding Tips</a>","Some tips that may help you");
 	print_twocolumnbody("<a href=\"doc_language_index_all$g_fileextension\">All the language documents</a>","All the documents related to KVS");
 
 	print_twocolumnsubtitle("Scripting Reference");
@@ -1282,6 +1309,24 @@ if(open(DOCINDEX,">$g_directory/index$g_fileextension"))
 	print "Can't open $g_directory/index$g_fileextension for writing\n";
 }
 
+if(open(NOHELP,">$g_directory/nohelpavailable$g_fileextension"))
+{
+	$g_filehandle=NOHELP;
+	print_header("No help available");
+
+	print_tablestart();
+	print_title("No help available","");
+	print_tableend();
+
+	print $g_filehandle "<center><h3>Sorry :(<br>Unfortunately there is no documentation related to the item you have selected</h3><br>\n";
+	print $g_filehandle "Please try a different search term or take a look at the <a href=\"index$g_fileextension\">documentation index</a> ";
+	print $g_filehandle "and try to locate the topic manually.</center><br><br>";
+	print_footer();
+	close(NOHELP);
+} else {
+	printf "Can't open $g_directory/nohelpavailable$g_fileextension for writing\n";
+}
+
 print "\n";
 print "*\n";
 print "* Done! (Processed $g_files files)\n";
