@@ -4,7 +4,7 @@
 //   Creation date : Sun Jun 18 2000 18:26:27 CEST by Szymon Stefanek
 //
 //   This file is part of the KVirc irc client distribution
-//   Copyright (C) 1999-2001 Szymon Stefanek (pragma at kvirc dot net)
+//   Copyright (C) 1999-2006 Szymon Stefanek (pragma at kvirc dot net)
 //
 //   This program is FREE software. You can redistribute it and/or
 //   modify it under the terms of the GNU General Public License
@@ -28,7 +28,7 @@
 // kvi_malloc , kvi_free and kvi_realloc are macros (see kvi_malloc.h)
 //=============================================================================
 
-#define __LIBKVILIB__
+#define __KVILIB__
 
 #define _KVI_MALLOC_CPP_
 #include "kvi_malloc.h"
