@@ -4,7 +4,7 @@
 //   Creation date : Sun Jun 18 2000 18:27:50 CEST by Szymon Stefanek
 //
 //   This file is part of the KVirc irc client distribution
-//   Copyright (C) 1999-2001 Szymon Stefanek (pragma at kvirc dot net)
+//   Copyright (C) 1999-2006 Szymon Stefanek (pragma at kvirc dot net)
 //
 //   This program is FREE software. You can redistribute it and/or
 //   modify it under the terms of the GNU General Public License
@@ -22,7 +22,7 @@
 //
 //=============================================================================
 
-#define __LIBKVILIB__
+#define __KVILIB__
 
 #define _KVI_DEBUG_CHECK_RANGE_
 #include "kvi_debug.h"
