@@ -4,7 +4,7 @@
 //   Creation date : Mon Aug 04 2003 13:36:33 CEST by Szymon Stefanek
 //
 //   This file is part of the KVirc irc client distribution
-//   Copyright (C) 2003 Szymon Stefanek (pragma at kvirc dot net)
+//   Copyright (C) 2003-2006 Szymon Stefanek (pragma at kvirc dot net)
 //
 //   This program is FREE software. You can redistribute it and/or
 //   modify it under the terms of the GNU General Public License
@@ -28,7 +28,8 @@
 //
 //=============================================================================
 
-#define __LIBKVILIB__
+#define __KVILIB__
+
 
 #include "kvi_qstring.h"
 #include "kvi_string.h"
@@ -36,6 +37,11 @@
 
 #include <ctype.h> // for tolower()
 #include <stdio.h> // for sprintf()
+#include <qregexp.h> 
+
+// kvi_string.cpp
+extern unsigned char iso88591_toLower_map[256];
+extern unsigned char iso88591_toUpper_map[256];
 
 namespace KviQString
 {
@@ -117,6 +123,26 @@ namespace KviQString
 		return (c1 == c1e);
 	}
 
+	// sz2 is assumed to be null terminated, sz1 is not!
+	bool equalCIN(const QString &sz1,const QChar *sz2,unsigned int len)
+	{
+		if(len == 0)return true; // assume equal
+		const QChar * c1 = sz1.unicode();
+		if(sz1.length() < len)return false;
+		const QChar * c1e = c1 + len;
+
+		if(!sz2)return !c1;
+		if(!c1)return !(sz2->unicode());
+
+		while((c1 < c1e) && (sz2->unicode()))
+		{
+			if(c1->lower().unicode() != sz2->lower().unicode())return false;
+			c1++;
+			sz2++;
+		}
+		return (c1 == c1e);
+	}
+
 	bool equalCS(const QString &sz1,const QString &sz2)
 	{
 		if(sz1.length() != sz2.length())return false;
@@ -155,6 +181,24 @@ namespace KviQString
 		return (c1 == c1e);
 	}
 
+	// sz2 is assumed to be null terminated, sz1 is not!
+	bool equalCI(const QString &sz1,const QChar *sz2)
+	{
+		const QChar * c1 = sz1.unicode();
+		const QChar * c1e = c1 + sz1.length();
+
+		if(!c1 || !sz2)return (c1 == sz2);
+
+		while(c1 < c1e)
+		{
+			if(!sz2->unicode())return false; // sz1 has at least another character
+			if(c1->lower().unicode() != sz2->lower().unicode())return false;
+			c1++;
+			sz2++;
+		}
+		return (c1 == c1e) && (!sz2->unicode());
+	}
+
 	bool equalCS(const QString &sz1,const char * sz2)
 	{
 		const QChar * c1 = sz1.unicode();
@@ -194,7 +238,13 @@ namespace KviQString
 		const QChar * c1e = c1 + sz1.length();
 		const QChar * c2e = c2 + sz2.length();
 
-		if(!c1 || !c2)return (c1 == c2);
+		if(!c1)
+		{
+			if(!c2)return 0;
+			return -1;
+		}
+		if(!c2)return 1;
+
 
 		for(;;)
 		{
@@ -222,7 +272,12 @@ namespace KviQString
 		const QChar * c1e = c1 + sz1.length();
 		const QChar * c2e = c2 + sz2.length();
 
-		if(!c1 || !c2)return (c1 == c2);
+		if(!c1)
+		{
+			if(!c2)return 0;
+			return -1;
+		}
+		if(!c2)return 1;
 
 		for(;;)
 		{
@@ -253,7 +308,13 @@ namespace KviQString
 		const QChar * c2 = sz2.unicode();
 		const QChar * c1e = c1 + l;
 
-		if(!c1 || !c2)return (c1 == c2);
+		if(!c1)
+		{
+			if(!c2)return 0;
+			return -1;
+		}
+		if(!c2)return 1;
+
 		int diff = 0;
 
 		while((c1 < c1e) && !(diff = (c1->lower().unicode() - c2->lower().unicode())))
@@ -270,10 +331,36 @@ namespace KviQString
 		if(!lastCharIs(szString,c))szString.append(c);
 	}
 
+	QString getToken(QString &szString,const QChar &sep)
+	{
+		int i=0;
+		while(i < szString.length())
+		{
+			if(szString[i] == sep)break;
+			i++;
+		}
+		QString ret;
+		if(i == szString.length())
+		{
+			ret = szString;
+			szString = "";
+		} else {
+			ret = szString.left(i);
+			while(i < szString.length())
+			{
+				if(szString[i] != sep)break;
+				i++;
+			}
+			if(i == szString.length())szString = "";
+			else szString.remove(0,i);
+		}
+		return ret;
+	}
+
 	void stripRightWhiteSpace(QString &s)
 	{
 		int iRemove = 0;
-		while(s.length() > 0)
+		while(iRemove < s.length())
 		{
 			if(s.at(s.length() - (iRemove + 1)).isSpace())iRemove++;
 			else break;
@@ -281,6 +368,30 @@ namespace KviQString
 		if(iRemove > 0)s.remove(s.length() - iRemove,iRemove);
 	}
 
+	void stripRight(QString &s,const QChar &c)
+	{
+		int iRemove = 0;
+		while(iRemove < s.length())
+		{
+			if(s.at(s.length() - (iRemove + 1)) == c)iRemove++;
+			else break;
+		}
+		if(iRemove > 0)s.remove(s.length() - iRemove,iRemove);
+	}
+
+	void stripLeft(QString &s,const QChar &c)
+	{
+		int iRemove = 0;
+		while(iRemove < s.length())
+		{
+			if(s[iRemove] == c)
+				iRemove++;
+			else
+				break;
+		}
+		if(iRemove > 0)s.remove(0,iRemove);
+	}
+
 	void detach(QString &sz)
 	{
 		sz.setLength(sz.length());
@@ -294,21 +405,35 @@ namespace KviQString
 	
 	void appendNumber(QString &s,double dReal)
 	{
-		char buffer[64];
+		char buffer[512];
 		::sprintf(buffer,"%f",dReal);
 		s.append(buffer);
 	}
 	
 	void appendNumber(QString &s,int iInteger)
 	{
-		char buffer[32];
+		char buffer[64];
 		::sprintf(buffer,"%d",iInteger);
 		s.append(buffer);
 	}
+
+	void appendNumber(QString &s,kvi_i64_t iInteger)
+	{
+		char buffer[64];
+		::sprintf(buffer,"%ld",iInteger);
+		s.append(buffer);
+	}
+	
+	void appendNumber(QString &s,kvi_u64_t uInteger)
+	{
+		char buffer[64];
+		::sprintf(buffer,"%lu",uInteger);
+		s.append(buffer);
+	}
 	
 	void appendNumber(QString &s,unsigned int uInteger)
 	{
-		char buffer[32];
+		char buffer[64];
 		::sprintf(buffer,"%u",uInteger);
 		s.append(buffer);
 	}
@@ -385,9 +510,12 @@ namespace KviQString
 					}
 					//check for space...
 					if((allocsize - reallen) < argValue)INCREMENT_MEM_BY(argValue)
-
-					while(*argString)*p++ = QChar(*argString++);
-					reallen += argValue;
+					QString str(argString);
+					if(str.isEmpty())continue;
+					int len = str.length();
+					const QChar * ch = str.unicode();
+					while(len--)*p++ = *ch++;
+					reallen += str.length();
 					continue;
 				}
 				case 'S': // KviStr * string
@@ -579,31 +707,31 @@ namespace KviQString
 		//      m1
 		//      s1
 		//
-		const QChar * m1 = nullTerminatedArray(szM1);
-		const QChar * m2 = nullTerminatedArray(szM2);
 
-
-		if(!(m1 && m2 && (*m1)))return false;
+		const QChar * m1 = (const QChar *)szM1.ucs2();
+		const QChar * m2 = (const QChar *)szM2.ucs2();
+		
+		if(!(m1 && m2 && (m1->unicode())))return false;
 		const QChar * savePos1 = 0;
 		const QChar * savePos2 = m2;
-		while(*m1)
+		while(m1->unicode())
 		{
 			//loop managed by m1 (initially first mask)
-			if(*m1=='*')
+			if(m1->unicode()=='*')
 			{
 				//Found a wildcard in m1
 				savePos1 = ++m1;            //move to the next char and save the position...this is our jolly
-				if(!*savePos1)return true;  //last was a wildcard , matches everything ahead...
+				if(!savePos1->unicode())return true;  //last was a wildcard , matches everything ahead...
 				savePos2 = m2+1;            //next return state for the second string
 				continue;                   //and return
 			}
-			if(!(*m2))return false;         //m2 finished and we had something to match here!
+			if(!m2->unicode())return false;         //m2 finished and we had something to match here!
 			if(m1->lower()==m2->lower())
 			{
 				//chars matched
 				m1++;                       //Go ahead in the two strings
 				m2++;                       //
-				if((!(*m1)) && *m2 && savePos1)
+				if((!(m1->unicode())) && m2->unicode() && savePos1)
 				{
 					//m1 finished , but m2 not yet and we have a savePosition for m1 (there was a wildcard)...
 					//retry matching the string following the * from the savePos2 (one char ahead last time)
@@ -612,7 +740,7 @@ namespace KviQString
 					savePos2++;             //next savePos2 will be next char
 				}
 			} else {
-				if(*m2 == '*')
+				if(m2->unicode() == '*')
 				{
 					//A wlidcard in the second string
 					//Invert the game : mask1 <-> mask2
@@ -625,11 +753,11 @@ namespace KviQString
 					continue;               //...and again
 				}
 				// m1 != m2 , m1 != * , m2 != *
-				if((*m1 == '?') || (*m2 == '?'))
+				if((m1->unicode() == '?') || (m2->unicode() == '?'))
 				{
 					m1++;
 					m2++;
-					if((!(*m1)) && *m2 && savePos1)
+					if((!(m1->unicode())) && m2->unicode() && savePos1)
 					{
 						//m1 finished , but m2 not yet and we have a savePosition for m1 (there was a wildcard)...
 						//retry matching the string following the * from the savePos2 (one char ahead last time)
@@ -648,71 +776,51 @@ namespace KviQString
 				}
 			}
 		}
-		return (!(*m2));                     //m1 surely finished , so for the match , m2 must be finished too
+		return (!(m2->unicode()));           //m1 surely finished , so for the match , m2 must be finished too
 	}
 
-
-	bool matchStringCI(const QString &szExp,const QString &szStr)
+	bool matchStringCI(const QString &szExp,const QString &szStr,bool bIsRegExp,bool bExact)
 	{
-		//               a
-		//               .
-		// exp = a*x?mem*a
-		// str = arexoxmexamemizazv
-		//                         .
-		//                         n
-		const QChar * exp = nullTerminatedArray(szExp);
-		const QChar * str = nullTerminatedArray(szStr);
-		const QChar * afterWild = 0;
-		const QChar * nextStrToCheck = 0;
-
-		if(!(exp && str))return false;
-	
-		while(*exp)
+		QString szWildcard;
+		QChar* ptr=(QChar*)szExp.ucs2();
+		if(!ptr) return 0;
+		while(ptr->unicode())
 		{
-			if(*exp == '*')
+			if((ptr->unicode()=='[') || (ptr->unicode()==']'))
 			{
-				// exp is a wildcard...
-				afterWild = ++exp;
-				nextStrToCheck = str + 1;
-				if(!(*exp))return true; // and it's the last char in the string: matches everything ahead
-				continue;
+				szWildcard.append("[");
+				szWildcard.append(*ptr);
+				szWildcard.append("]");
+			} else {
+				szWildcard.append(*ptr);
 			}
-	
-			if(!(*str))return false; // str finished but we had something to match :(
-	
-			if(exp->lower() == str->lower())
-			{
-				// chars matched
-				++exp;
-				++str;
-				if((!(*exp)) && *str)goto check_recovery;
-				continue;
-			}
-	
-			if(*exp == '?')
-			{
-				// any-char wildcard
-				++exp;
-				++str;
-				continue;
-			}
-	
-	check_recovery:
-			// chars unmatched!!!
-			if(afterWild)
-			{
-				// we had a wildcard in exp...
-				// let's use this jolly then
-				exp = afterWild;
-				str = nextStrToCheck;
-				nextStrToCheck++;
-				// and try to compare now
-				continue;
-			}
-	
-			return false; // no match :(
+			ptr++;
 		}
-		return (!(*str));
+		QRegExp re(szWildcard,false,!bIsRegExp);
+		if(bExact) return re.exactMatch(szStr);
+		return re.search(szStr) != -1;
+	}
+
+	bool matchStringCS(const QString &szExp,const QString &szStr,bool bIsRegExp,bool bExact)
+	{
+		QString szWildcard;
+		QChar* ptr=(QChar*)szExp.ucs2();
+		if(!ptr) return 0;
+		while(ptr->unicode())
+		{
+			if((ptr->unicode()=='[')) // <-- hum ?
+			{
+				szWildcard.append("[");
+				szWildcard.append(*ptr);
+				szWildcard.append("]");
+			} else {
+				szWildcard.append(*ptr);
+			}
+			ptr++;
+		}
+		QRegExp re(szWildcard,true,!bIsRegExp);
+		if(bExact) return re.exactMatch(szStr);
+		return re.search(szStr) != -1;
 	}
 
 	void cutFromFirst(QString &s,const QChar &c,bool bIncluded)
@@ -787,4 +895,89 @@ namespace KviQString
 		s.remove(0,bIncluded ? idx + c.length() : idx);
 	}
 
+	QString upperISO88591(const QString &szSrc)
+	{
+		const QChar * c = nullTerminatedArray(szSrc);
+		if(!c)
+		{
+			QString ret;
+			return ret;
+		}
+		QChar * buffer = (QChar *)kvi_malloc(sizeof(QChar) * szSrc.length());
+		QChar * b = buffer;
+		unsigned short us = c->unicode();
+		while(us)
+		{
+			if(us < 256)
+				*b=QChar((unsigned short)iso88591_toUpper_map[us]);
+			else
+				*b = *c;
+			c++;
+			b++;
+			us = c->unicode();
+		}
+		QString ret(buffer,szSrc.length());
+		kvi_free(buffer);
+		return ret;
+	}
+	
+	QString lowerISO88591(const QString &szSrc)
+	{
+		const QChar * c = nullTerminatedArray(szSrc);
+		if(!c)
+		{
+			QString ret;
+			return ret;
+		}
+		QChar * buffer = (QChar *)kvi_malloc(sizeof(QChar) * szSrc.length());
+		QChar * b = buffer;
+		unsigned short us = c->unicode();
+		while(us)
+		{
+			if(us < 256)
+			{
+				*b=QChar((unsigned short)iso88591_toLower_map[us]);
+			} else
+				*b = *c;
+			c++;
+			b++;
+			us = c->unicode();
+		}
+		QString ret(buffer,szSrc.length());
+		kvi_free(buffer);
+		return ret;
+	}
+
+	void transliterate(QString &s,const QString &szToFind,const QString &szReplacement)
+	{
+		int i=0;
+		int il = QMIN(szToFind.length(),szReplacement.length());
+		while(i < il)
+		{
+			int k=0;
+			int kl = s.length();
+			while(k < kl)
+			{
+				if(s[k] == szToFind[i])s[k] = szReplacement[i];
+				k++;
+			}
+			i++;
+		}
+	}
+	
+	static char hexdigits[16] = { '0','1','2','3','4','5','6','7','8','9','a','b','c','d','e','f' };
+	
+	void bufferToHex(QString &szRetBuffer,const unsigned char * buffer,unsigned int len)
+	{
+		szRetBuffer.setLength(len * 2);
+		unsigned int i=0;
+		while(i < (len*2))
+		{
+			szRetBuffer[int(i)] = QChar( (unsigned int) hexdigits[(*buffer) / 16] );
+			i++;
+			szRetBuffer[int(i)] = QChar( (unsigned int)hexdigits[(*buffer) % 16] );
+			i++;
+			buffer++;
+		}
+	}
 };
