@@ -200,6 +200,8 @@ public:
 	// upper and lower case copies
 	KviStr upper() const;
 	KviStr lower() const;
+	KviStr upperISO88591() const;
+	KviStr lowerISO88591() const;
 
 	// left , right & co.
 	// all parameters are safety-checked
@@ -244,7 +246,9 @@ public:
 	void ensureLastCharIs(char ch)              { if(!lastCharIs(ch))append(ch);        };
 
 	// Change THIS string to uppercase or lowercase
-	void toUpper();
+	void toUpperISO88591();
+	void toUpper(); // this is LOCALE AWARE (in Turkish it maps i to Ý!)
+	void toLowerISO88591();
 	void toLower();
 
 	// Assignment
