@@ -22,12 +22,14 @@
 //
 //=================================================================================================
 
-#define __LIBKVILIB__
+
+#define __KVILIB__
 
 
 #include "kvi_debug.h"
 #include "kvi_regusersdb.h"
 #include "kvi_config.h"
+#include "kvi_locale.h"
 
 /*
 	@doc: registered_users
@@ -229,6 +231,7 @@ void KviRegisteredUser::setProperty(const QString & name,const QString & value)
 bool KviRegisteredUser::getProperty(const QString & name,QString &value)
 {
 	if(!m_pPropertyDict)return false;
+	if(name.isEmpty()) return false;
 	QString * pValue = m_pPropertyDict->find(name);
 	if(pValue)value = *pValue;
 	else return false;
@@ -237,7 +240,8 @@ bool KviRegisteredUser::getProperty(const QString & name,QString &value)
 
 const QString & KviRegisteredUser::getProperty(const QString & name)
 {
-	if(!m_pPropertyDict)return QString::null;
+	if(!m_pPropertyDict) return QString::null;
+	if(name.isEmpty()) return QString::null;
 	QString * pValue = m_pPropertyDict->find(name);
 	if(pValue)return *pValue;
 	return QString::null;
@@ -246,6 +250,7 @@ const QString & KviRegisteredUser::getProperty(const QString & name)
 bool KviRegisteredUser::getBoolProperty(const QString & name)
 {
 	if(!m_pPropertyDict)return false;
+	if(name.isEmpty()) return false;
 	QString * pValue = m_pPropertyDict->find(name);
 	if(pValue)
 	{
@@ -257,6 +262,21 @@ bool KviRegisteredUser::getBoolProperty(const QString & name)
 	return false;
 }
 
+//============================================================================================================
+//
+// KviRegisteredUserGroup
+//
+
+KviRegisteredUserGroup::KviRegisteredUserGroup(const QString &name)
+{
+	setName(name);
+}
+
+KviRegisteredUserGroup::~KviRegisteredUserGroup()
+{
+}
+
+
 //============================================================================================================
 //
 // KviRegisteredUserDb
@@ -272,6 +292,9 @@ KviRegisteredUserDataBase::KviRegisteredUserDataBase()
 
 	m_pMaskDict = new QDict<KviRegisteredMaskList>(49,false); // copy keys here!
 	m_pMaskDict->setAutoDelete(true);
+	
+	m_pGroupDict = new  QDict<KviRegisteredUserGroup>(5,false); // copy keys here!
+	m_pGroupDict->setAutoDelete(true);
 }
 
 KviRegisteredUserDataBase::~KviRegisteredUserDataBase()
@@ -279,18 +302,30 @@ KviRegisteredUserDataBase::~KviRegisteredUserDataBase()
 	delete m_pUserDict;
 	delete m_pWildMaskList;
 	delete m_pMaskDict;
+	delete m_pGroupDict;
 }
 
 KviRegisteredUser * KviRegisteredUserDataBase::addUser(const QString & name)
 {
+	if(name.isEmpty()) return false;
 	if(m_pUserDict->find(name))return 0;
 	KviRegisteredUser * u = new KviRegisteredUser(name);
 	m_pUserDict->replace(u->name(),u); //u->name() because we're NOT copying keys!
 	return u;
 }
 
+KviRegisteredUserGroup * KviRegisteredUserDataBase::addGroup(const QString & name)
+{
+	if(name.isEmpty()) return false;
+	if(m_pGroupDict->find(name))return 0;
+	KviRegisteredUserGroup * pGroup = new KviRegisteredUserGroup(name);
+	m_pGroupDict->replace(pGroup->name(),pGroup); //u->name() because we're NOT copying keys!
+	return pGroup;
+}
+
 KviRegisteredUser * KviRegisteredUserDataBase::getUser(const QString & name)
 {
+	if(name.isEmpty()) return 0;
 	KviRegisteredUser * u = m_pUserDict->find(name);
 	if(!u)
 	{
@@ -318,6 +353,7 @@ static void append_mask_to_list(KviRegisteredMaskList *l,KviRegisteredUser *u,Kv
 
 KviRegisteredUser * KviRegisteredUserDataBase::addMask(KviRegisteredUser * u,KviIrcMask * mask)
 {
+	if(!u || !mask) return 0;
 	__range_valid(u == m_pUserDict->find(u->name()));
 
 	KviRegisteredMaskList * l;
@@ -382,6 +418,7 @@ void KviRegisteredUserDataBase::copyFrom(KviRegisteredUserDataBase * db)
 	m_pUserDict->clear();
 	m_pWildMaskList->clear();
 	m_pMaskDict->clear();
+	m_pGroupDict->clear();
 
 	QDictIterator<KviRegisteredUser> it(*(db->m_pUserDict));
 
@@ -406,23 +443,38 @@ void KviRegisteredUserDataBase::copyFrom(KviRegisteredUserDataBase * db)
 				++pdi;
 			}
 		}
+		u->setGroup(theCur->group());
 		++it;
 	}
+	
+	QDictIterator<KviRegisteredUserGroup> git(*db->m_pGroupDict);
+	while(git.current())
+	{
+		addGroup(git.currentKey());	
+		++git;
+	}
 }
 
 
 bool KviRegisteredUserDataBase::removeUser(const QString & name)
 {
+	if(name.isEmpty()) return false;
 	KviRegisteredUser * u = m_pUserDict->find(name);
 	if(!u)return false;
 	while(KviIrcMask * mask = u->maskList()->first())
 	{
 		if(!removeMaskByPointer(mask))
-			debug("Ops... removeMaskByPointer(%s) failed ?",name.latin1());
+			debug("Ops... removeMaskByPointer(%s) failed ?",name.utf8().data());
 	}
 	m_pUserDict->remove(name);
 	return true;
 }
+bool KviRegisteredUserDataBase::removeGroup(const QString & name)
+{
+	if(name.isEmpty()) return false;
+	m_pGroupDict->remove(name);
+	return true;
+}
 
 bool KviRegisteredUserDataBase::removeMask(const KviIrcMask &mask)
 {
@@ -435,6 +487,7 @@ bool KviRegisteredUserDataBase::removeMask(const KviIrcMask &mask)
 
 bool KviRegisteredUserDataBase::removeMaskByPointer(KviIrcMask * mask)
 {
+	if(!mask) return 0;
 	if(mask->hasWildNick())
 	{
 		// remove from the wild list
@@ -504,6 +557,7 @@ KviRegisteredUser * KviRegisteredUserDataBase::findMatchingUser(const QString &
 KviRegisteredMask * KviRegisteredUserDataBase::findMatchingMask(const QString & nick,const QString &user,const QString & host)
 {
 	// first lookup the nickname in the maskDict
+	if(nick.isEmpty()) return false;
 	KviRegisteredMaskList * l = m_pMaskDict->find(nick);
 	if(l)
 	{
@@ -530,6 +584,7 @@ KviRegisteredUser * KviRegisteredUserDataBase::findUserWithMask(const KviIrcMask
 KviRegisteredMask * KviRegisteredUserDataBase::findExactMask(const KviIrcMask &mask)
 {
 	// first lookup the nickname in the maskDict
+	if(mask.nick()=="") return 0;
 	KviRegisteredMaskList * l = m_pMaskDict->find(mask.nick());
 	if(l)
 	{
@@ -555,32 +610,46 @@ bool KviRegisteredUserDataBase::isIgnoredUser(const QString & nick,const QString
 */
 void KviRegisteredUserDataBase::load(const QString & filename)
 {
+	QString szCurrent;
 	KviConfig cfg(filename,KviConfig::Read);
 
 	KviConfigIterator it(*cfg.dict());
 	while(it.current())
 	{
-		KviRegisteredUser * u = addUser(it.currentKey());
-		if(u)
+		szCurrent=it.currentKey();
+		if(KviQString::equalCSN("#Group ",szCurrent,7))
 		{
-			KviConfigGroupIterator sdi(*(it.current()));
-			while(sdi.current())
+			szCurrent.remove(0,7);
+			addGroup(szCurrent);
+		} else {
+			KviRegisteredUser * u = addUser(szCurrent);
+			
+			if(u)
 			{
-				QString tmp = sdi.currentKey();
-				if(KviQString::equalCSN("prop_",tmp,5))
+				KviConfigGroupIterator sdi(*(it.current()));
+				while(sdi.current())
 				{
-					tmp.remove(0,5);
-					u->setProperty(tmp,*(sdi.current()));
-				} else if(KviQString::equalCSN("mask_",tmp,5))
-				{
-					KviIrcMask * mask = new KviIrcMask(*(sdi.current()));
-					addMask(u,mask);
+					QString tmp = sdi.currentKey();
+					if(KviQString::equalCSN("prop_",tmp,5))
+					{
+						tmp.remove(0,5);
+						u->setProperty(tmp,*(sdi.current()));
+					} else if(KviQString::equalCSN("mask_",tmp,5))
+					{
+						KviIrcMask * mask = new KviIrcMask(*(sdi.current()));
+						addMask(u,mask);
+					} else if(KviQString::equalCI(tmp,"Group"))
+					{
+						u->setGroup(*(sdi.current()));
+					}
+					++sdi;
 				}
-				++sdi;
 			}
 		}
 		++it;
 	}
+	if(!m_pGroupDict->find(__tr("Default")))
+		addGroup(__tr("Default"));
 }
 
 
@@ -618,6 +687,17 @@ void KviRegisteredUserDataBase::save(const QString & filename)
 			cfg.writeEntry(tmp,mask);
 			++idx;
 		}
+		cfg.writeEntry("Group",it.current()->group());
 		++it;
 	}
+	
+	QDictIterator<KviRegisteredUserGroup> git(*m_pGroupDict);
+	QString szTmp;
+	while(git.current())
+	{
+		KviQString::sprintf(szTmp,"#Group %Q",&(git.current()->name()));
+		cfg.setGroup(szTmp);
+		++git;
+	}
+	
 }
