@@ -22,7 +22,8 @@
 //
 //=============================================================================
 
-#define __LIBKVILIB__
+#define __KVILIB__
+
 
 #include "kvi_debug.h"
 #include "kvi_ircmask.h"
@@ -555,11 +556,11 @@ bool KviIrcMask::matches(const KviIrcMask &mask) const
 
 bool KviIrcMask::matchesFixed(const KviIrcMask &mask) const
 {
-	if(KviQString::matchStringCI(m_szNick,mask.m_szNick))
+	if(KviQString::matchStringCI(m_szNick,mask.m_szNick,0,1))
 	{
-		if(KviQString::matchStringCI(m_szUser,mask.m_szUser))
+		if(KviQString::matchStringCI(m_szUser,mask.m_szUser,0,1))
 		{
-			if(KviQString::matchStringCI(m_szHost,mask.m_szHost))return true;
+			if(KviQString::matchStringCI(m_szHost,mask.m_szHost,0,1))return true;
 		}
 	}
 	return false;
@@ -588,9 +589,9 @@ bool KviIrcMask::matches(const char * nick,const char * user,const char * host)
 
 bool KviIrcMask::matchesFixed(const QString &nick,const QString &user,const QString &host) const
 {
-	if(!KviQString::matchStringCI(m_szNick,nick))return false;
-	if(!KviQString::matchStringCI(m_szUser,user))return false;
-	if(!KviQString::matchStringCI(m_szHost,host))return false;
+	if(!KviQString::matchStringCI(m_szNick,nick,0,1))return false;
+	if(!KviQString::matchStringCI(m_szUser,user,0,1))return false;
+	if(!KviQString::matchStringCI(m_szHost,host,0,1))return false;
 	return true;
 }
 
