@@ -22,7 +22,8 @@
 //
 //=============================================================================
 
-#define __LIBKVILIB__
+#define __KVILIB__
+
 
 #include <qdir.h>
 #include <qtimer.h>
@@ -155,7 +156,7 @@ bool KviHttpRequest::start()
 	if(!kvi_strEqualCI(m_url.protocol().ptr(),"http"))
 	{
 		resetInternalStatus();
-		m_szLastError.sprintf(__tr2qs("Unsupported protocol %s"),m_url.protocol().ptr());
+		m_szLastError=__tr2qs("Unsupported protocol %1").arg(m_url.protocol().ptr());
 		return false;
 	}
 
@@ -227,7 +228,8 @@ void KviHttpRequest::haveServerIp()
 						uPort,
 						m_url.path().ptr(),
 						m_uContentOffset,
-						(m_eProcessingType == HeadersOnly) ? KviHttpRequestThread::Head : KviHttpRequestThread::Get);
+						(m_eProcessingType == HeadersOnly) ? KviHttpRequestThread::Head : (m_szPostData.isEmpty() ? KviHttpRequestThread::Get : KviHttpRequestThread::Post),
+						m_szPostData);
 
 	if(!m_pThread->start())
 	{
@@ -424,7 +426,7 @@ bool KviHttpRequest::openFile()
 	if(!m_pFile->open(iOpenFlags))
 	{
 		resetInternalStatus();
-		m_szLastError.sprintf(__tr2qs("Can't open file \"%s\" for writing"),m_szFileName.latin1());
+		m_szLastError.sprintf(__tr2qs("Can't open file \"%s\" for writing"),m_szFileName.utf8().data());
 		emit terminated(false);
 		return false;
 	}
@@ -474,7 +476,7 @@ bool KviHttpRequest::processHeader(KviStr &szHeader)
 	{
 		// the response is invalid ?
 		resetInternalStatus();
-		m_szLastError.sprintf(__tr2qs("Invalid HTTP response: %s"),szResponse.ptr());
+		m_szLastError=__tr2qs("Invalid HTTP response: %s").arg(szResponse.ptr());
 		emit terminated(false);
 		return false;
 	}
@@ -587,7 +589,7 @@ bool KviHttpRequest::processHeader(KviStr &szHeader)
 	if((m_uMaxContentLength > 0) && (m_uTotalSize > ((unsigned int)m_uMaxContentLength)))
 	{
 		resetInternalStatus();
-		m_szLastError.sprintf(__tr2qs("Stream exceeding maximum length"));
+		m_szLastError=__tr2qs("Stream exceeding maximum length");
 		emit terminated(false);
 		return false;
 	}
@@ -625,7 +627,7 @@ void KviHttpRequest::processData(KviDataBuffer * data)
 		if(((m_uTotalSize > 0) && (m_uReceivedSize > m_uTotalSize)) || ((m_uMaxContentLength > 0) && (m_uReceivedSize > m_uMaxContentLength)))
 		{
 			resetInternalStatus();
-			m_szLastError.sprintf(__tr2qs("Stream exceeded expected length"));
+			m_szLastError=__tr2qs("Stream exceeded expected length");
 			emit terminated(false);
 		}
 		return;
@@ -691,7 +693,7 @@ void KviHttpRequest::processData(KviDataBuffer * data)
 
 
 
-KviHttpRequestThread::KviHttpRequestThread(KviHttpRequest * r,const char * szHost,const char * szIp,unsigned short uPort,const char * szPath,unsigned int uContentOffset,RequestMethod m)
+KviHttpRequestThread::KviHttpRequestThread(KviHttpRequest * r,const char * szHost,const char * szIp,unsigned short uPort,const char * szPath,unsigned int uContentOffset,RequestMethod m,const QString &szPostData)
 : KviSensitiveThread()
 {
 	m_pRequest = r;
@@ -701,6 +703,7 @@ KviHttpRequestThread::KviHttpRequestThread(KviHttpRequest * r,const char * szHos
 	m_uPort = uPort > 0 ? uPort : 80;
 	m_uContentOffset = uContentOffset;
 	m_eRequestMethod = m;
+	m_szPostData = szPostData;
 }
 
 KviHttpRequestThread::~KviHttpRequestThread()
@@ -789,7 +792,7 @@ bool KviHttpRequestThread::selectForWrite(int iTimeoutInSecs)
 #endif
 				{
 					KviStr tmp(KviStr::Format,__tr2qs("Select error: %s (errno=%d)"),
-						KviError::getDescription(KviError::translateSystemError(err)).latin1(),err);
+						KviError::getDescription(KviError::translateSystemError(err)).utf8().data(),err);
 					return failure(tmp.ptr());
 				}
 			}
@@ -827,7 +830,7 @@ bool KviHttpRequestThread::connectToRemoteHost()
 		if(!kvi_socket_recoverableConnectError(err))
 		{
 			KviStr tmp(KviStr::Format,__tr2qs("Connect error: %s (errno=%d)"),
-				KviError::getDescription(KviError::translateSystemError(err)).latin1(),err);
+				KviError::getDescription(KviError::translateSystemError(err)).utf8().data(),err);
 			return failure(tmp.ptr());
 		}
 	}
@@ -847,7 +850,7 @@ bool KviHttpRequestThread::connectToRemoteHost()
 		if(sockError > 0)sockError = KviError::translateSystemError(sockError);
 		else sockError = KviError_unknownError;
 		KviStr tmp(KviStr::Format,__tr2qs("Connect error: %s (errno=%d)"),
-			KviError::getDescription(sockError).latin1(),sockError);
+			KviError::getDescription(sockError).utf8().data(),sockError);
 		return failure(tmp.ptr());
 	}
 
@@ -885,7 +888,7 @@ bool KviHttpRequestThread::sendBuffer(const char * buffer,int bufLen,int iTimeou
 #endif
 				{
 					KviStr tmp(KviStr::Format,__tr2qs("Write error: %s (errno=%d)"),
-							KviError::getDescription(KviError::translateSystemError(err)).latin1(),err);
+							KviError::getDescription(KviError::translateSystemError(err)).utf8().data(),err);
 					return failure(tmp.ptr());
 				}
 			}
@@ -939,7 +942,7 @@ int KviHttpRequestThread::selectForReadStep()
 #endif
 			{
 				KviStr tmp(KviStr::Format,__tr2qs("Select error: %s (errno=%d)"),
-					KviError::getDescription(KviError::translateSystemError(err)).latin1(),err);
+					KviError::getDescription(KviError::translateSystemError(err)).utf8().data(),err);
 				failure(tmp.ptr());
 				return -1;
 			}
@@ -1003,7 +1006,7 @@ bool KviHttpRequestThread::readData()
 			{
 				// yes...read error
 				KviStr tmp(KviStr::Format,__tr2qs("Read error: %s (errno=%d)"),
-						KviError::getDescription(KviError::translateSystemError(err)).latin1(),err);
+						KviError::getDescription(KviError::translateSystemError(err)).utf8().data(),err);
 				return failure(tmp.ptr());
 			}
 			return true; // EINTR or EAGAIN...transient problem
@@ -1027,18 +1030,42 @@ void KviHttpRequestThread::run()
 
 	// FIXME: Other headers ?
 
+	KviStr szMethod;
+	switch(m_eRequestMethod)
+	{
+		case Head: szMethod = "HEAD"; break;
+		case Post: szMethod = "POST"; break;
+		case Get: szMethod = "GET"; break;
+	}
+	
 	KviStr szRequest(KviStr::Format,"%s %s HTTP/1.0\r\n" \
 				"Host: %s\r\n" \
 				"Connection: Close\r\n" \
 				"User-Agent: KVIrc-http-slave/1.0.0\r\n" \
 				"Accept: */*\r\n",
-				(m_eRequestMethod == Head) ? "HEAD" : "GET",m_szPath.ptr(),m_szHost.ptr());
+				szMethod.ptr(),m_szPath.ptr(),m_szHost.ptr());
 
 	if(m_uContentOffset > 0)
 		szRequest.append(KviStr::Format,"Range: bytes=%u-\r\n",m_uContentOffset);
 
+	QCString szPostData = m_szPostData.utf8();
+
+	if(m_eRequestMethod == Post)
+	{
+		szRequest.append(KviStr::Format,"Content-Type: application/x-www-form-urlencoded\r\n" \
+				"Content-Length: %u\r\n" \
+				"Pragma: no-cache\r\n",szPostData.length());
+	}
+
 	szRequest += "\r\n";
 
+	if(m_eRequestMethod == Post)
+	{
+		if(!szPostData.isEmpty())
+			szRequest.append(szPostData.data());
+		szRequest += "\r\n";
+	}
+
 	//debug("SENDING REQUEST:\n%s",szRequest.ptr());
 
 	if(!sendBuffer(szRequest.ptr(),szRequest.len(),60))return;
