@@ -22,7 +22,8 @@
 //
 //=============================================================================
 
-#define __LIBKVILIB__
+#define __KVILIB__
+
 
 #include "kvi_ssl.h"
 #include "kvi_locale.h"
