@@ -22,7 +22,8 @@
 //
 //=============================================================================
 
-#define __LIBKVILIB__
+#define __KVILIB__
+
 
 //#define _KVI_DEBUG_CHECK_RANGE_
 #include "kvi_debug.h"
@@ -35,6 +36,7 @@
 #include <qglobal.h> //for debug()
 #include <qasciidict.h>
 #include <qtextcodec.h>
+#include <qdir.h>
 
 #include "kvi_string.h"
 #include "kvi_env.h"
@@ -350,7 +352,7 @@ bool KviMessageCatalogue::load(const char * name)
 	KviStr szCatalogueFile = name;
 
 	// Try to load the header
-	QFile f(szCatalogueFile.ptr());
+	QFile f(QString::fromUtf8(szCatalogueFile.ptr()));
 	if(!f.open(IO_ReadOnly))
 	{
 		debug("[KviLocale]: Failed to open the messages file %s: probably doesn't exist",szCatalogueFile.ptr());
@@ -494,7 +496,7 @@ bool KviMessageCatalogue::load(const char * name)
 			m_pTextCodec = KviLocale::codecForName(szHeader.ptr());
 			if(!m_pTextCodec)
 			{
-				debug("Can't find the codec for charser=%s",szHeader.ptr());
+				debug("Can't find the codec for charset=%s",szHeader.ptr());
 				debug("Falling back to codecForLocale()");
 				m_pTextCodec = QTextCodec::codecForLocale();
 			}
@@ -540,7 +542,7 @@ const QString & KviMessageCatalogue::translateToQString(const char *text)
 namespace KviLocale
 {
 #ifndef QT_NO_BIG_CODECS
-	#define NUM_ENCODINGS 69
+	#define NUM_ENCODINGS 73
 #else
 	#define NUM_ENCODINGS 57
 #endif
@@ -576,7 +578,9 @@ namespace KviLocale
 		{ "Big5"                 , 0 , "Chinese Traditional" },
 		{ "Big5-HKSCS"           , 0 , "Chinese Traditional, Hong Kong" },
 		{ "GB18030"              , 0 , "Chinese Simplified" },
-		{ "Shift-JIS"            , 0 , "Japanese" },
+		{ "JIS7"                 , 0 , "Japanese (JIS7)" },
+		{ "Shift-JIS"            , 0 , "Japanese (Shift-JIS)" },
+		{ "EUC-JP"               , 0 , "Japanese (EUC-JP)" },
 		{ "EUC-KR"               , 0 , "Korean" },
 		{ "TSCII"                , 0 , "Tamil" },
 #endif
@@ -613,7 +617,9 @@ namespace KviLocale
 		{ "UTF-8/Big5"           , 1 , "UTF-8? -> Chinese Traditional" },
 		{ "UTF-8/Big5-HKSCS"     , 1 , "UTF-8? -> Chinese Traditional, Hong Kong" },
 		{ "UTF-8/GB18030"        , 1 , "UTF-8? -> Chinese Simplified" },
-		{ "UTF-8/Shift-JIS"      , 1 , "UTF-8? -> Japanese" },
+		{ "UTF-8/JIS7"           , 1 , "UTF-8? -> Japanese (JIS7)" },
+		{ "UTF-8/Shift-JIS"      , 1 , "UTF-8? -> Japanese (Shift-JIS)" },
+		{ "UTF-8/EUC-JP"         , 1 , "UTF-8? -> Japanese (EUC-JP)" },
 		{ "UTF-8/EUC-KR"         , 1 , "UTF-8? -> Korean" },
 		{ "UTF-8/TSCII"          , 1 , "UTF-8? -> Tamil" },
 #endif
@@ -693,7 +699,7 @@ namespace KviLocale
 
 		szBuffer.sprintf("%s%s_%s.mo",szLocDir.ptr(),name,szLocale.ptr());
 		//debug("Lookup %s",szBuffer.ptr());
-		if(kvi_fileExists(szBuffer.ptr()))return true;
+		if(KviFileUtils::fileExists(szBuffer.ptr()))return true;
 
 		if(szLocale.findFirstIdx('.') != -1)
 		{
@@ -701,7 +707,7 @@ namespace KviLocale
 			// kill them
 			szLocale.cutFromFirst('.');
 			szBuffer.sprintf("%s%s_%s.mo",szLocDir.ptr(),name,szLocale.ptr());
-			if(kvi_fileExists(szBuffer.ptr()))return true;
+			if(KviFileUtils::fileExists(szBuffer.ptr()))return true;
 		}
 
 		if(szLocale.findFirstIdx('@') != -1)
@@ -710,7 +716,7 @@ namespace KviLocale
 			// kill them
 			szLocale.cutFromFirst('@');
 			szBuffer.sprintf("%s%s_%s.mo",szLocDir.ptr(),name,szLocale.ptr());
-			if(kvi_fileExists(szBuffer.ptr()))return true;
+			if(KviFileUtils::fileExists(szBuffer.ptr()))return true;
 		}
 
 		if(szLocale.findFirstIdx('_') != -1)
@@ -719,13 +725,13 @@ namespace KviLocale
 			// kill them
 			szLocale.cutFromFirst('_');
 			szBuffer.sprintf("%s%s_%s.mo",szLocDir.ptr(),name,szLocale.ptr());
-			if(kvi_fileExists(szBuffer.ptr()))return true;
+			if(KviFileUtils::fileExists(szBuffer.ptr()))return true;
 		}
 
 		// try the lower case version too
 		szLocale.toLower();
 		szBuffer.sprintf("%s%s_%s.mo",szLocDir.ptr(),name,szLocale.ptr());
-		if(kvi_fileExists(szBuffer.ptr()))return true;
+		if(KviFileUtils::fileExists(szBuffer.ptr()))return true;
 
 		return false;
 	}
@@ -743,7 +749,15 @@ namespace KviLocale
 	void init(QApplication * app,const char * localeDir)
 	{
 		// first of all try to find out the current locale
-		g_szLang = kvi_getenv("KVIRC_LANG");
+		g_szLang="";
+		QString szLangFile=QString("%1/.kvirc_force_locale").arg(QDir::homeDirPath());
+		if(KviFileUtils::fileExists(szLangFile))
+		{
+			QString szTmp;
+			KviFileUtils::readFile(szLangFile,szTmp);
+			g_szLang=szTmp;
+		}
+		if(g_szLang.isEmpty())g_szLang = kvi_getenv("KVIRC_LANG");
 		if(g_szLang.isEmpty())g_szLang = QTextCodec::locale();
 		if(g_szLang.isEmpty())g_szLang = kvi_getenv("LC_MESSAGES");
 		if(g_szLang.isEmpty())g_szLang = kvi_getenv("LANG");
@@ -811,6 +825,12 @@ namespace KviLocale
 		}
 	}
 
+	KviMessageCatalogue * getLoadedCatalogue(const char * name)
+	{
+		return g_pCatalogueDict->find(name);
+	}
+
+
 	const char * translate(const char * text,const char * context)
 	{
 		if(context)
