@@ -20,7 +20,8 @@
 //   Inc. ,59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 //
 
-#define __LIBKVILIB__
+#define __KVILIB__
+
 
 #include "kvi_tal_menubar.h"
 
