@@ -31,7 +31,6 @@
 #include "kvi_frame.h"
 #include "kvi_msgbox.h"
 #include "kvi_iconmanager.h"
-#include "kvi_ircview.h"
 #include "kvi_input.h"
 #include "kvi_config.h"
 #include "kvi_colorwin.h"
@@ -43,13 +42,11 @@
 #include "kvi_options.h"
 #include "kvi_sparser.h"
 #include "kvi_modulemanager.h"
-#include "kvi_event.h"
 #include "kvi_mediatype.h"
 #include "kvi_scriptobject.h"
 #include "kvi_regusersdb.h"
 #include "kvi_thread.h"
 #include "kvi_sharedfiles.h"
-#include "kvi_popupmanager.h"
 #include "kvi_confignames.h"
 #include "kvi_taskbar.h"
 #include "kvi_defaults.h"
@@ -57,7 +54,7 @@
 #include "kvi_out.h"
 #include "kvi_splash.h"
 #include "kvi_nickserv.h"
-#include "kvi_timermanager.h"
+//#include "kvi_timermanager.h"
 #include "kvi_xlib.h"
 #include "kvi_garbage.h"
 #include "kvi_texticonmanager.h"
@@ -71,15 +68,24 @@
 #include "kvi_kvs.h"
 #include "kvi_ircurl.h"
 #include "kvi_avatarcache.h"
-#include "kvi_processmanager.h"
+//#include "kvi_processmanager.h"
 #include "kvi_actionmanager.h"
 #include "kvi_customtoolbarmanager.h"
 #include "kvi_fileutils.h"
+#include "kvi_time.h"
+#include "kvi_doublebuffer.h"
 
 #ifndef COMPILE_NO_IPC
 	#include "kvi_ipc.h"
 #endif
 
+#include "kvi_kvs_script.h"
+#include "kvi_kvs_popupmanager.h"
+#include "kvi_kvs_kernel.h"
+#include "kvi_kvs_object_controller.h"
+#include "kvi_kvs_eventtriggers.h"
+#include "kvi_sourcesdate.h"
+
 #include <qpopupmenu.h>
 #include <qsplitter.h>
 #include <qstringlist.h>
@@ -89,6 +95,7 @@
 #include <qclipboard.h>
 #include <qmessagebox.h>
 #include <qtextcodec.h>
+#include <qmetaobject.h>
 
 #ifdef COMPILE_SSL_SUPPORT
 	#include "kvi_ssl.h"
@@ -102,14 +109,6 @@ KviConfig                              * g_pWinPropertiesConfig    = 0;
 KVIRC_API KviIrcServerDataBase         * g_pIrcServerDataBase      = 0;
 KVIRC_API KviProxyDataBase             * g_pProxyDataBase          = 0;
 
-// Graphic stuff
-
-
-QPixmap                                * g_pIrcViewMemBuffer       = 0;
-KviPtrList<KviIrcView>                 * g_pIrcViewWidgetList      = 0;
-QPixmap                                * g_pInputMemBuffer         = 0;
-KviPtrList<KviInputEditor>             * g_pInputWidgetList        = 0;
-
 // Global windows
 
 KVIRC_API KviColorWindow               * g_pColorWindow               = 0;
@@ -122,7 +121,7 @@ KVIRC_API KviMediaManager              * g_pMediaManager              = 0;
 KVIRC_API KviRegisteredUserDataBase    * g_pRegisteredUserDataBase    = 0;
 KVIRC_API KviSharedFilesManager        * g_pSharedFilesManager        = 0;
 KVIRC_API KviNickServRuleSet           * g_pNickServRuleSet           = 0;
-KVIRC_API KviTimerManager              * g_pTimerManager              = 0;
+//KVIRC_API KviTimerManager              * g_pTimerManager              = 0;
 KVIRC_API KviGarbageCollector          * g_pGarbageCollector          = 0;
 KVIRC_API KviCtcpPageDialog            * g_pCtcpPageDialog            = 0;
 KVIRC_API KviRegisteredChannelDataBase * g_pRegisteredChannelDataBase = 0;
@@ -132,9 +131,8 @@ KVIRC_API KviHistoryWindow             * g_pHistoryWindow             = 0;
 
 KVIRC_API bool                           g_bIdentDaemonRunning     = false; // this is eventually set by libkviident
 
-#ifdef COMPILE_SPLASH_SCREEN
-	KVIRC_API KviSplashScreen       * g_pSplashScreen           = 0;
-#endif
+KVIRC_API KviSplashScreen       * g_pSplashScreen           = 0;
+
 
 // Loaded and destroyed by KviIconManager
 QPixmap                             * g_pUserChanStatePixmap    = 0;
@@ -170,22 +168,24 @@ KviApp::KviApp(int argc,char ** argv)
 {
 	// Ok...everything begins here
 	g_pApp                  = this;
-	m_pConfigFile           = 0;
+	m_szConfigFile          = QString::null;
 	m_bCreateConfig         = false;
-#ifdef COMPILE_SPLASH_SCREEN
 	m_bShowSplashScreen     = true;
-#endif
 	m_bUpdateGuiPending     = false;
 	m_pPendingAvatarChanges = 0;
-#ifndef COMPILE_NO_IPC
+	m_pRecentChannelsDict   = 0;
 	m_pIpcSentinel          = 0;
-#endif
 	// next step is setup()
 	m_bSetupDone = false;
 }
 
 void KviApp::setup()
 {
+	// Here we start up all the substystems.
+	// This is a really critical phase since in general subsystems depend
+	// on each other and we must activate them in the right order.
+	// Don't move stuff around unless you really know what you're doing.
+
 #ifdef COMPILE_ON_WINDOWS
 	// setup winsock.dll
 	WSADATA w;
@@ -194,6 +194,7 @@ void KviApp::setup()
 
 #ifndef COMPILE_NO_IPC
 	// Create this very early
+	// FIXME: this is still not early enough...
 	createIpcSentinel();
 #endif
 
@@ -207,59 +208,73 @@ void KviApp::setup()
 	// Setup our filesystem and initalize locale
 	loadDirectories();
 
-#ifdef COMPILE_SPLASH_SCREEN
+	// check if we want to permanently disable the splash screen
+	// we do it once for every version: the user should see the new splash screens at least once
+	
+	QString szSplashDisableFile;
+	getLocalKvircDirectory(szSplashDisableFile,Pics,"disable-splash." KVI_VERSION);
+	if(KviFileUtils::fileExists(szSplashDisableFile))
+		m_bShowSplashScreen = false;
+
 	// Now we can create the splash screen (we can locate the splash image)
 	if(m_bShowSplashScreen)
 		createSplashScreen();
-#endif
 
 	KVI_SPLASH_SET_PROGRESS(0)
 
-	// make sure that the C strings are translated to utf8: this
-	// will make our old parser work a bit better
+	// Make sure that the C strings are translated to utf8
+	// This is a fallback solution anyway: we should use the appropriate
+	// encoding every time.
 	QTextCodec::setCodecForCStrings(KviLocale::codecForName("UTF-8"));
 
-
 	// Set the default help files search path
 	QStringList list;
 	KviStr tmp;
-	getLocalKvircDirectory(tmp,Help);
+	getLocalKvircDirectory(tmp,Help); // localized help/lang or help if help/lang doesn't exist
+	list.append(tmp.ptr());
+	getLocalKvircDirectory(tmp,HelpEN); // help/en
+	list.append(tmp.ptr());
+	getLocalKvircDirectory(tmp,HelpNoIntl); // just help/
 	list.append(tmp.ptr());
 	getGlobalKvircDirectory(tmp,Help);
 	list.append(tmp.ptr());
 	getGlobalKvircDirectory(tmp,HelpEN);
 	list.append(tmp.ptr());
+	getGlobalKvircDirectory(tmp,HelpNoIntl);
+	list.append(tmp.ptr());
 	QMimeSourceFactory::defaultFactory()->setFilePath(list);
 
 	KVI_SPLASH_SET_PROGRESS(1)
-	KVI_SPLASH_SET_TEXT(__tr2qs("Loading options..."))
-
-	// Load the configuration
-	loadOptions();
 
 	// Initialize the scripting engine
 	KviKvs::init();
 
+	KVI_SPLASH_SET_PROGRESS(2)
+
 	// Initialize the action manager
 	KviActionManager::init();
 
+	KVI_SPLASH_SET_PROGRESS(3)
+
 	// Create the module manager early: so the other managers can load modules
 	g_pModuleExtensionManager = new KviModuleExtensionManager();
 	g_pModuleManager = new KviModuleManager();
 
-	KVI_SPLASH_SET_PROGRESS(40)
-	KVI_SPLASH_SET_TEXT(__tr2qs("Loading actions..."));
-	if(getReadOnlyConfigPath(tmp,KVI_CONFIGFILE_USERACTIONS))
-		KviActionManager::instance()->load(tmp.ptr());
-	
-	KVI_SPLASH_SET_PROGRESS(42);
-	KVI_SPLASH_SET_TEXT(__tr2qs("Loading icons..."))
+	KVI_SPLASH_SET_PROGRESS(4);
 
-	// Load the icons (needed before loading options)
 	g_pIconManager = new KviIconManager();
 
-	KVI_SPLASH_SET_PROGRESS(43)
-	KVI_SPLASH_SET_TEXT(__tr2qs("Performing early initializations..."))
+	KVI_SPLASH_SET_PROGRESS(5)
+
+	if(getReadOnlyConfigPath(tmp,KVI_CONFIGFILE_USERACTIONS))
+		KviActionManager::instance()->load(tmp.ptr());
+
+	KVI_SPLASH_SET_PROGRESS(8);
+
+	// Load the configuration
+	loadOptions();
+
+	KVI_SPLASH_SET_PROGRESS(42)
 
 	// set the global font if needed
 	updateApplicationFont();
@@ -268,15 +283,13 @@ void KviApp::setup()
 	updatePseudoTransparency();
 #endif
 
-	KVI_SPLASH_SET_PROGRESS(45)
-	KVI_SPLASH_SET_TEXT(__tr2qs("Loading window configuration database..."))
+	KVI_SPLASH_SET_PROGRESS(44)
 
 	// Load the win properties config
 	getLocalKvircDirectory(tmp,Config,KVI_CONFIGFILE_WINPROPERTIES);
 	g_pWinPropertiesConfig = new KviConfig(tmp.ptr(),KviConfig::ReadWrite);
 
 	KVI_SPLASH_SET_PROGRESS(48)
-	KVI_SPLASH_SET_TEXT(__tr2qs("Loading server & proxy database..."))
 
 	// Load the server database
 	g_pIrcServerDataBase   = new KviIrcServerDataBase();
@@ -290,54 +303,53 @@ void KviApp::setup()
 	if(getReadOnlyConfigPath(tmp,KVI_CONFIGFILE_PROXYDB))
 		g_pProxyDataBase->load(tmp.ptr());
 
-	KVI_SPLASH_SET_TEXT(__tr2qs("Loading event handlers..."))
 	KVI_SPLASH_SET_PROGRESS(54)
 
 	// Event manager
-
 	if(getReadOnlyConfigPath(tmp,KVI_CONFIGFILE_EVENTS))
 		KviKvs::loadAppEvents(tmp.ptr());
 
-	KVI_SPLASH_SET_TEXT(__tr2qs("Loading raw event handlers..."))
 	KVI_SPLASH_SET_PROGRESS(59)
 
 	if(getReadOnlyConfigPath(tmp,KVI_CONFIGFILE_RAWEVENTS))
 		KviKvs::loadRawEvents(tmp.ptr());
 
-	KVI_SPLASH_SET_TEXT(__tr2qs("Loading popups..."))
 	KVI_SPLASH_SET_PROGRESS(62)
 
 	// Popup manager
-	g_pPopupManager = new KviPopupManager();
 	if(getReadOnlyConfigPath(tmp,KVI_CONFIGFILE_POPUPS))
-		g_pPopupManager->load(tmp.ptr());
+		KviKvs::loadPopups(tmp.ptr());
 
-	KVI_SPLASH_SET_TEXT(__tr2qs("Loading toolbars..."))
-	KVI_SPLASH_SET_PROGRESS(66)
+	KVI_SPLASH_SET_PROGRESS(67)
 
 	KviCustomToolBarManager::init();
 	if(getReadOnlyConfigPath(tmp,KVI_CONFIGFILE_CUSTOMTOOLBARS))
 		KviCustomToolBarManager::instance()->load(tmp.ptr());
 	// THIS IS A COMPATIBILITY ENTRY ADDED AT 3.0.2 TIME THAT SHOULD BE DROPPED IN A COUPLE OF VERSION BUMPS!
 	getLocalKvircDirectory(tmp,Config,KVI_CONFIGFILE_SCRIPTTOOLBARS);
-	if(kvi_fileExists(tmp.ptr()))
+	if(KviFileUtils::fileExists(tmp.ptr()))
 		KviCustomToolBarManager::instance()->loadScripttoolbarsCompat(tmp.ptr());
 	// EOF COMPATIBILITY
 
-	KVI_SPLASH_SET_TEXT(__tr2qs("Loading aliases..."))
 	KVI_SPLASH_SET_PROGRESS(70)
 
 	// Alias manager
 	if(getReadOnlyConfigPath(tmp,KVI_CONFIGFILE_ALIASES))
 		KviKvs::loadAliases(tmp.ptr());
 
+	KVI_SPLASH_SET_PROGRESS(75)
+
+	// Script addons manager (this in fact has delayed loading, so we don't even care
+	// about showing up an entry in the splash screen)
+	if(getReadOnlyConfigPath(tmp,KVI_CONFIGFILE_SCRIPTADDONS))
+		KviKvs::loadScriptAddons(tmp.ptr());
+
 	KVI_SPLASH_SET_PROGRESS(77)
-	KVI_SPLASH_SET_TEXT(__tr2qs("Loading text icon associations..."))
+
 	g_pTextIconManager = new KviTextIconManager();
 	g_pTextIconManager->load();
 
-	KVI_SPLASH_SET_PROGRESS(78)
-	KVI_SPLASH_SET_TEXT(__tr2qs("Loading recent entries..."))
+	KVI_SPLASH_SET_PROGRESS(80)
 
 	// load the recent data lists
 	g_pRecentTopicList = new QStringList();
@@ -345,7 +357,6 @@ void KviApp::setup()
 	loadRecentEntries();
 
 	KVI_SPLASH_SET_PROGRESS(81)
-	KVI_SPLASH_SET_TEXT(__tr2qs("Loading media types..."))
 
 	// media manager
 	g_pMediaManager = new KviMediaManager();
@@ -355,57 +366,46 @@ void KviApp::setup()
 	g_pMediaManager->unlock();
 
 
-	KVI_SPLASH_SET_TEXT(__tr2qs("Loading registered users..."))
-	KVI_SPLASH_SET_PROGRESS(84)
+	KVI_SPLASH_SET_PROGRESS(82)
 
 	// registered user data base
 	g_pRegisteredUserDataBase = new KviRegisteredUserDataBase();
 	if(getReadOnlyConfigPath(tmp,KVI_CONFIGFILE_REGUSERDB))
 		g_pRegisteredUserDataBase->load(tmp.ptr());
 
-	KVI_SPLASH_SET_TEXT(__tr2qs("Loading registered channels..."))
-	KVI_SPLASH_SET_PROGRESS(86)
+	KVI_SPLASH_SET_PROGRESS(84)
 
 	// registered channel data base
 	g_pRegisteredChannelDataBase = new KviRegisteredChannelDataBase();
 	if(getReadOnlyConfigPath(tmp,KVI_CONFIGFILE_REGCHANDB))
 		g_pRegisteredChannelDataBase->load(tmp.ptr());
 
-
-	KVI_SPLASH_SET_TEXT(__tr2qs("Loading file trader configuration..."))
-	KVI_SPLASH_SET_PROGRESS(88)
+	KVI_SPLASH_SET_PROGRESS(85)
 
 	// file trader
 	g_pSharedFilesManager = new KviSharedFilesManager();
 	if(getReadOnlyConfigPath(tmp,KVI_CONFIGFILE_SHAREDFILES))
 		g_pSharedFilesManager->load(tmp.ptr());
 
-	KVI_SPLASH_SET_TEXT(__tr2qs("Loading nick serv service configuration..."))
-	KVI_SPLASH_SET_PROGRESS(90)
+	KVI_SPLASH_SET_PROGRESS(86)
 
 	// nick serv data base
 	g_pNickServRuleSet = new KviNickServRuleSet();
 	if(getReadOnlyConfigPath(tmp,KVI_CONFIGFILE_NICKSERVDATABASE))
 		g_pNickServRuleSet->load(tmp.ptr());
 
-	KVI_SPLASH_SET_TEXT(__tr2qs("Loading history entries..."))
-	KVI_SPLASH_SET_PROGRESS(91)
+	KVI_SPLASH_SET_PROGRESS(88)
 
 	g_pInputHistory = new KviInputHistory();
 	if(getReadOnlyConfigPath(tmp,KVI_CONFIGFILE_INPUTHISTORY))
 		g_pInputHistory->load(tmp.ptr());
 
-	KVI_SPLASH_SET_TEXT(__tr2qs("Loading avatar cache..."))
-	KVI_SPLASH_SET_PROGRESS(92)
+	KVI_SPLASH_SET_PROGRESS(89)
 	KviAvatarCache::init();
 	if(getReadOnlyConfigPath(tmp,KVI_CONFIGFILE_AVATARCACHE))
 		KviAvatarCache::instance()->load(tmp.ptr());
 
-	KVI_SPLASH_SET_TEXT(__tr2qs("Creating internal structures..."))
-	KVI_SPLASH_SET_PROGRESS(93)
-
-	// Init some other globals
-	createSharedGraphicGlobals();
+	KVI_SPLASH_SET_PROGRESS(90)
 
 	// Eventually initialize the crypt engine manager
 #ifdef COMPILE_CRYPT_SUPPORT
@@ -414,22 +414,14 @@ void KviApp::setup()
 
 	g_pGarbageCollector = new KviGarbageCollector();
 
-	// Create the color windoww
-	//g_pColorWindow = new KviColorWindow();
-	//g_pColorWindow->hide();
 	// and the input popup
 	g_pInputPopup = new QPopupMenu();
 
-
 	// create the parsers
 	g_pUserParser = new KviUserParser();
 	g_pServerParser = new KviServerParser();
-	// And the timer manager
-	g_pTimerManager = new KviTimerManager();
 
-	KviProcessManager::init();
-
-	KVI_SPLASH_SET_PROGRESS(95)
+	KVI_SPLASH_SET_PROGRESS(91)
 
 	// Global window dictionary
 	g_pGlobalWindowDict = new QAsciiDict<KviWindow>(41);
@@ -437,12 +429,16 @@ void KviApp::setup()
 	// Script object controller
 	g_pScriptObjectController = new KviScriptObjectController();
 
-	KVI_SPLASH_SET_TEXT(__tr2qs("Starting up the engine..."))
-	KVI_SPLASH_SET_PROGRESS(98)
+	KVI_SPLASH_SET_PROGRESS(92)
 
-    // create the frame window, we're almost up and running...
-    createFrame();
+	KviDoubleBuffer::init();
 
+	// create the frame window, we're almost up and running...
+	createFrame();
+
+	// ok, we also have an UI now
+
+	// check if this is the first time this version of KVIrc runs...
 	if(firstTimeRun())
 	{
 		// Finish the setup...
@@ -458,60 +454,80 @@ void KviApp::setup()
 	// kill the splash screen
 
 	KVI_SPLASH_SET_PROGRESS(100)
-	KVI_SPLASH_SET_TEXT(__tr2qs("Have fun! :)"))
+	//KVI_SPLASH_SET_TEXT(__tr2qs("Have fun! :)"))
 
-#ifdef COMPILE_SPLASH_SCREEN
 	destroySplashScreen();
-#endif
 
 	// check if we're in trouble...
 	checkSuggestRestoreDefaultScript();
+
+	// start our heartbeat now
+	m_iHeartbeatTimerId = startTimer(1000);
+}
+
+void KviApp::frameDestructorCallback()
+{
+	// here we should kill anything that depends on g_pFrame , g_pActiveWindow and related being non zero...
+
+	// kill all the objects, while we have windows...
+	if(KviKvsKernel::instance())
+	{
+		if(KviKvsKernel::instance()->objectController())
+		{
+			KviKvsKernel::instance()->objectController()->clearInstances();
+		}
+	}
 }
 
 
 
 KviApp::~KviApp()
 {
+	// Another critical phase.
+	// We shutdown our subsystems in the right order here.
 
 #ifndef COMPILE_NO_IPC
 	destroyIpcSentinel();
 #endif
 
-#ifdef COMPILE_SPLASH_SCREEN
 	if(g_pSplashScreen)delete g_pSplashScreen;
-#endif
-
 	if(g_pCtcpPageDialog)delete g_pCtcpPageDialog;
 
-	delete g_pGarbageCollector;
-
 	// if we still have a frame: kill it
 	if(g_pFrame)delete g_pFrame;
 	g_pActiveWindow = 0; // .. but it should be already 0 anyway
 
+	// execute pending deletes (this may still contain some UI elements)
+	delete g_pGarbageCollector;
+
 	if(!m_bSetupDone)return; // killed with IPC (nothing except the m_pFrameList was created yet)
 
 	KviFileTransferManager::cleanup();
 
-	// NO MORE EVENTS ARE TRIGGERED FROM NOW ON: NO COMMANDS ARE EXECUTED
-	// dangerous part.... we're unloading all the modules
+	// No more events are triggered from now on and no KVS commands are executed
+
+	// Dangerous part.... we're unloading all the modules
+	// We need to unload them early since they may use other subsystems
+	// that we are going to kill now.
 	delete g_pModuleManager;
 	delete g_pModuleExtensionManager;
-	// NO MORE EXTERNAL MODULES EXIST: NO "UNKNOWN" ACTIONS ARE PERFORMED
-	// we are safe here...
+	// No more external modules exist: all that happens from now is generated
+	// from inside the kvirc core.
+
+	// We should have almost no UI here: only certain dialogs or popup windows may
+	// still exist: they should be harmless tough.
 	saveOptions();
+	if(m_pRecentChannelsDict) delete m_pRecentChannelsDict;
 	// now kill the stuff that the frame depends on
 	saveIrcServerDataBase();
 	delete g_pIrcServerDataBase;
 	saveProxyDataBase();
 	delete g_pProxyDataBase;
 	delete g_pWinPropertiesConfig;
-	if(g_pTextIconWindow)delete g_pTextIconWindow;
 	saveTextIcons();
+	if(g_pTextIconWindow)delete g_pTextIconWindow;
 	delete g_pTextIconManager;
 	delete g_pIconManager;
-	KviProcessManager::done();
-	delete g_pTimerManager;
 	delete g_pUserParser;
 	delete g_pServerParser;
 	saveMediaTypes();
@@ -534,11 +550,10 @@ KviApp::~KviApp()
 	saveToolBars();
 	KviCustomToolBarManager::done();
 	savePopups();
-	delete g_pPopupManager;
 	saveAliases();
 	delete g_pGlobalWindowDict;
+	saveScriptAddons();
 	// kill the remaining resources
-	destroySharedGraphicGlobals();
 	delete g_pColorWindow;
 	if(g_pHistoryWindow)delete g_pHistoryWindow;
 	saveInputHistory();
@@ -551,6 +566,7 @@ KviApp::~KviApp()
 #ifdef COMPILE_PSEUDO_TRANSPARENCY
 	destroyPseudoTransparency();
 #endif
+	KviDoubleBuffer::done();
 	if(m_pPendingAvatarChanges)delete m_pPendingAvatarChanges;
 	// Kill the thread manager.... all the slave threads should have been already terminated ...
 #ifdef COMPILE_SSL_SUPPORT
@@ -580,8 +596,6 @@ int KviApp::getGloballyUniqueId()
 	return g_iGloballyUniqueId;
 }
 
-//#include "kvi_module.h"
-
 typedef struct _NotifierMessageSupaDupaParameterStruct
 {
 	KviWindow * pWindow;
@@ -619,7 +633,31 @@ QTextCodec * KviApp::defaultTextCodec()
 	return c;
 }
 
-
+void KviApp::contextSensitiveHelp()
+{
+	// the F1 Key has been pressed
+	// try to pass it to the active widget or one of its parents
+	QWidget * w = g_pApp->focusWidget();
+	while(w)
+	{
+		//QVariant v = w->property("contextSensitiveHelp"); <-- this prints a lot of warnings: this is a QT BUG
+		QMetaObject * o = w->metaObject();
+		if(o)
+		{
+			int i = o->findProperty("contextSensitiveHelp",true);
+			if(i != -1)
+			{
+				QVariant v = w->property("contextSensitiveHelp");
+				if(v.isValid() && v.toBool())
+					return; // done!
+			}
+		}
+		w = w->parentWidget();
+	}
+	// no way
+	// FIXME: just show up simple plain online help
+	//debug("No way: found no focus widget for that...");
+}
 
 
 void KviApp::collectGarbage(QObject * garbage)
@@ -630,14 +668,14 @@ void KviApp::collectGarbage(QObject * garbage)
 
 void KviApp::loadDefaultScript()
 {
-	KviStr cmd;
+	QString cmd;
 	getGlobalKvircDirectory(cmd,KviApp::DefScript,"default.kvs");
-	cmd.prepend("/parse \"");
+	cmd.prepend("parse \"");
 	cmd.append("\"");
 #ifdef COMPILE_ON_WINDOWS
-	cmd.replaceAll("\\","\\\\");
+	cmd.replace("\\","\\\\");
 #endif
-	g_pUserParser->parseUserCommand(cmd,g_pFrame->firstConsole());
+	KviKvsScript::run(cmd,g_pFrame->firstConsole());
 	// now line up the toolbars (they may get messed while loading the script)
 	g_pFrame->lineUpDockWindows();
 }
@@ -670,7 +708,7 @@ void KviApp::checkSuggestRestoreDefaultScript()
 	int score = 0;
 
 	if(KviCustomToolBarManager::instance()->descriptorCount() < 1)score += 100; // straight suggest
-	else if(g_pPopupManager->popupCount() < 1)score += 100; // straight suggest
+	else if(KviKvsPopupManager::instance()->popupCount() < 1)score += 100; // straight suggest
 	else {
 		// other stuff is not that critical
 		if(!KviKvsEventManager::instance()->hasAppHandlers(KviEvent_OnChannelNickPopupRequest))score += 80;
@@ -681,7 +719,7 @@ void KviApp::checkSuggestRestoreDefaultScript()
 		if(!KviKvsEventManager::instance()->hasAppHandlers(KviEvent_OnQueryPopupRequest))score += 20;
 		if(!KviKvsEventManager::instance()->hasAppHandlers(KviEvent_OnQueryNickPopupRequest))score += 20;
 		if(KviCustomToolBarManager::instance()->descriptorCount() < 4)score += 20;
-		if(g_pPopupManager->popupCount() < 3)score += 20;
+		if(KviKvsPopupManager::instance()->popupCount() < 3)score += 20;
 		if(KviCustomToolBarManager::instance()->visibleToolBarCount() < 2)
 		{
 			score += 20;
@@ -738,7 +776,10 @@ void KviApp::restoreDefaultScript()
 	KviKvs::clearAliases();
 	KviKvs::clearAppEvents();
 	KviKvs::clearRawEvents();
-	g_pPopupManager->clear();
+	KviKvs::clearPopups();
+	KviKvs::clearScriptAddons();
+	
+	KviActionManager::instance()->killAllKvsUserActions();
 	KviCustomToolBarManager::instance()->clear();
 
 	loadDefaultScript();
@@ -774,12 +815,11 @@ void KviApp::ipcMessage(char * message)
 		if(idx != -1)cmd.cutRight(cmd.len() - (idx + 1));
 		cns->output(KVI_OUT_SYSTEMMESSAGE,__tr2qs("Remote command received (%s ...)"),cmd.ptr());
 	}
-	g_pUserParser->parseCommandBuffer(message,cns);
+	KviKvsScript::run(message,cns);
 }
 
 #endif // COMPILE_NO_IPC
 
-#ifdef COMPILE_SPLASH_SCREEN
 
 void KviApp::createSplashScreen()
 {
@@ -792,7 +832,6 @@ void KviApp::destroySplashScreen()
 	if(g_pSplashScreen)g_pSplashScreen->die();
 }
 
-#endif
 
 QString KviApp::getClipboardText()
 {
@@ -912,13 +951,49 @@ KviPendingAvatarChange * KviApp::findPendingAvatarChange(KviConsole * pConsole,c
 	return 0;
 }
 
-void KviApp::fileDownloadTerminated(bool bSuccess,const QString &szRemoteUrl,const QString &szLocalFileName,const QString &szNick,const QString &szError)
+void KviApp::fileDownloadTerminated(bool bSuccess,const QString &szRemoteUrl,const QString &szLocalFileName,const QString &szNick,const QString &szError,bool bQuiet)
 {
-	if(!m_pPendingAvatarChanges)return;
 
-	KviPendingAvatarChange * p = findPendingAvatarChange(0,szNick,szRemoteUrl);
+	KviPendingAvatarChange * p;
 
-	if(!p)return;
+	if(m_pPendingAvatarChanges)p = findPendingAvatarChange(0,szNick,szRemoteUrl);
+	else p = 0;
+
+	if(!p)
+	{
+		// signal dcc completion only for NON-avatars
+		if(KVI_OPTION_BOOL(KviOption_boolNotifyDccSendSuccessInNotifier) && (!bQuiet))
+		{
+			if(!g_pActiveWindow)return;
+			if(g_pActiveWindow->hasAttention())return;
+			QString szMsg;
+			int iIconId;
+			if(!bSuccess)
+			{
+				iIconId = KVI_SMALLICON_DCCERROR;
+				if(szNick.isEmpty())
+					szMsg = __tr2qs("File download failed");
+				else
+					szMsg = __tr2qs("File download from %1 failed").arg(szNick);
+				szMsg += ": ";
+				szMsg += szError;
+				szMsg += " (";
+				szMsg += szLocalFileName;
+				szMsg += ")";
+			} else {
+				iIconId = KVI_SMALLICON_DCCMSG;
+				if(szNick.isEmpty())
+					szMsg = __tr2qs("File download successfully complete");
+				else
+					szMsg = __tr2qs("File download from %1 successfully complete").arg(szNick);
+				szMsg += " (";
+				szMsg += szLocalFileName;
+				szMsg += ")";
+			}
+			notifierMessage(0,iIconId,szMsg,30);
+		}
+		return;
+	}
 
 	if(bSuccess)
 	{
@@ -928,7 +1003,7 @@ void KviApp::fileDownloadTerminated(bool bSuccess,const QString &szRemoteUrl,con
 				szLocalFileName,KviQString::equalCIN("http://",szRemoteUrl,7) ? szRemoteUrl : QString::null);
 		}
 	} else {
-		if(!_OUTPUT_MUTE)
+		if((!_OUTPUT_MUTE) && (!bQuiet))
 		{
 			p->pConsole->output(KVI_OUT_AVATAR,__tr2qs("Avatar download failed for %Q!%Q@%Q and url %Q: %Q"),
 				&(p->szNick),&(p->szUser),&(p->szHost),&(szRemoteUrl),&(szError));
@@ -1207,21 +1282,6 @@ void KviApp::updatePseudoTransparency()
 }
 
 
-
-void KviApp::createSharedGraphicGlobals()
-{
-	// The memory buffer for KviIrcView
-	g_pIrcViewMemBuffer = new QPixmap(1,1);
-	// The global list of KviIrcView widgets
-	g_pIrcViewWidgetList = new KviPtrList<KviIrcView>;
-	g_pIrcViewWidgetList->setAutoDelete(false);
-	// The memory buffer for KviInput
-	g_pInputMemBuffer = new QPixmap(1,1);
-	// The global list of KviInput widgets
-	g_pInputWidgetList = new KviPtrList<KviInputEditor>;
-	g_pInputWidgetList->setAutoDelete(false);
-}
-
 void KviApp::triggerUpdateGui()
 {
 	if(m_bUpdateGuiPending)return;
@@ -1245,19 +1305,6 @@ void KviApp::updateApplicationFont()
 	// FIXME: #warning "And what if this option is turned off ?...a reboot only"
 }
 
-void KviApp::destroySharedGraphicGlobals()
-{
-	// Kill the shared graphic globals
-	delete g_pIrcViewWidgetList;
-	delete g_pIrcViewMemBuffer;
-	delete g_pInputWidgetList;
-	delete g_pInputMemBuffer;
-    g_pIrcViewWidgetList = 0;
-    g_pIrcViewMemBuffer = 0;
-    g_pInputWidgetList = 0;
-    g_pInputMemBuffer = 0;
-}
-
 void KviApp::loadRecentEntries()
 {
 	KviStr tmp;
@@ -1294,14 +1341,17 @@ void KviApp::savePopups()
 {
 	KviStr tmp;
 	getLocalKvircDirectory(tmp,Config,KVI_CONFIGFILE_POPUPS);
-	g_pPopupManager->save(tmp.ptr());
+	KviKvs::savePopups(tmp.ptr());
 }
 
 void KviApp::saveInputHistory()
 {
-	KviStr tmp;
-	getLocalKvircDirectory(tmp,Config,KVI_CONFIGFILE_INPUTHISTORY);
-	g_pInputHistory->save(tmp.ptr());
+	if(!KVI_OPTION_BOOL(KviOption_boolDisableInputHistory))
+	{
+		KviStr tmp;
+		getLocalKvircDirectory(tmp,Config,KVI_CONFIGFILE_INPUTHISTORY);
+		g_pInputHistory->save(tmp.ptr());
+	}
 }
 
 
@@ -1312,6 +1362,13 @@ void KviApp::saveAliases()
 	KviKvs::saveAliases(tmp.ptr());
 }
 
+void KviApp::saveScriptAddons()
+{
+	KviStr tmp;
+	getLocalKvircDirectory(tmp,Config,KVI_CONFIGFILE_SCRIPTADDONS);
+	KviKvs::saveScriptAddons(tmp.ptr());
+}
+
 void KviApp::saveTextIcons()
 {
 	g_pTextIconManager->save();
@@ -1411,28 +1468,39 @@ void KviApp::saveConfiguration()
 	saveAliases();
 	saveTextIcons();
 	saveInputHistory();
+	saveScriptAddons();
+	KviKvs::flushUserClasses();
 	g_pWinPropertiesConfig->sync();
 }
 
 void KviApp::autoConnectToServers()
 {
 	KviPtrList<KviIrcServer> * l = g_pIrcServerDataBase->autoConnectOnStartupServers();
-	if(!l)return;
-
-	for(KviIrcServer * s = l->first();s;s = l->next())
+	if(l)
 	{
-		KviStr szCommand = "server -u ";
-		if(s->isIpV6())szCommand += "-i ";
-		if(s->useSSL())szCommand += "-s ";
-		szCommand += s->hostName();
-		QString tmp;
-		tmp.setNum(s->port());
-		szCommand += " ";
-		szCommand += tmp;
-		g_pUserParser->parseCommandBuffer(szCommand.ptr(),activeConsole());
+		for(KviIrcServer * s = l->first();s;s = l->next())
+		{
+			QString szCommand = "server -u \"id:";
+			if(s->id().isEmpty())s->generateUniqueId();
+			szCommand += s->id();
+			szCommand += "\"";
+			KviKvsScript::run(szCommand,activeConsole());
+		}
+		g_pIrcServerDataBase->clearAutoConnectOnStartupServers();
+	}
+	
+	KviPtrList<KviIrcServerDataBaseRecord> * lr = g_pIrcServerDataBase->autoConnectOnStartupNetworks();
+	if(lr)
+	{
+		for(KviIrcServerDataBaseRecord * r = lr->first();r;r = lr->next())
+		{
+			QString szCommandx = "server -u \"net:";
+			szCommandx += r->network()->name();
+			szCommandx += "\"";
+			KviKvsScript::run(szCommandx,activeConsole());
+		}
+		g_pIrcServerDataBase->clearAutoConnectOnStartupNetworks();
 	}
-
-	g_pIrcServerDataBase->clearAutoConnectOnStartupServers();
 }
 
 
@@ -1441,22 +1509,34 @@ void KviApp::createFrame()
 	if(g_pFrame)debug("WARNING: Creating the main frame twice!");
 	g_pFrame = new KviFrame();
 	g_pFrame->createNewConsole(true);
-	g_pFrame->show();
 
 	if(m_szExecAfterStartup.hasData())
 	{
-		g_pUserParser->parseCommandBuffer(m_szExecAfterStartup.ptr(),g_pFrame->firstConsole(),0);
+		// FIXME , this should be a QString
+		KviKvsScript::run(m_szExecAfterStartup.ptr(),g_pFrame->firstConsole());
 		m_szExecAfterStartup = "";
 	}
 	
 	// auto connect to servers if needed
-	if(g_pIrcServerDataBase->autoConnectOnStartupServers())
+	if(g_pIrcServerDataBase->autoConnectOnStartupServers() || g_pIrcServerDataBase->autoConnectOnStartupNetworks())
 	{
 		autoConnectToServers();
 	}
 
 	if(KVI_OPTION_BOOL(KviOption_boolShowDockExtension))
 		g_pFrame->executeInternalCommand(KVI_INTERNALCOMMAND_DOCKWIDGET_SHOW);
+	
+	if(KVI_OPTION_BOOL(KviOption_boolStartupMinimized))
+	{
+		if(KVI_OPTION_BOOL(KviOption_boolMinimizeInTray) && KVI_OPTION_BOOL(KviOption_boolShowDockExtension))
+		{
+			g_pFrame->hide();
+		} else {
+			g_pFrame->showMinimized();
+		}
+	} else {
+		g_pFrame->show();
+	}
 }
 
 void KviApp::destroyFrame()
@@ -1496,7 +1576,7 @@ unsigned int KviApp::windowCount()
 	return g_pGlobalWindowDict->count();
 }
 
-KviConsole * KviApp::findConsole(KviStr &server,KviStr &nick)
+KviConsole * KviApp::findConsole(QString &server,QString &nick)
 {
 	QAsciiDictIterator<KviWindow> it(*g_pGlobalWindowDict);
 
@@ -1506,22 +1586,22 @@ KviConsole * KviApp::findConsole(KviStr &server,KviStr &nick)
 		{
 			if(((KviConsole *)it.current())->isConnected())
 			{
-				if(server.hasData())
+				if(!server.isEmpty())
 				{
-					if(kvi_strEqualCI(server.ptr(),
+					if(KviQString::equalCI(server,
 						((KviConsole *)it.current())->connection()->currentServerName()))
 					{
-						if(nick.hasData())
+						if(!nick.isEmpty())
 						{
-							if(KviQString::equalCI(nick.ptr(),
+							if(KviQString::equalCI(nick,
 								((KviConsole *)it.current())->connection()->currentNickName()))
 									return ((KviConsole *)it.current());
 						} else return ((KviConsole *)it.current());
 					}
 				} else {
-					if(nick.hasData())
+					if(!nick.isEmpty())
 					{
-						if(KviQString::equalCI(nick.ptr(),
+						if(KviQString::equalCI(nick,
 							((KviConsole *)it.current())->connection()->currentNickName()))
 								return ((KviConsole *)it.current());
 					}
@@ -1533,6 +1613,14 @@ KviConsole * KviApp::findConsole(KviStr &server,KviStr &nick)
 	return 0;
 }
 
+KviConsole * KviApp::findConsole(KviStr &server,KviStr &nick)
+{
+	// DEPRECATED: TO BE KILLED (if nobody is using it)
+	QString s = server.ptr();
+	QString n = nick.ptr();
+	return findConsole(s,n);
+}
+
 void KviApp::restartLagMeters()
 {
 	QAsciiDictIterator<KviWindow> it(*g_pGlobalWindowDict);
@@ -1593,21 +1681,7 @@ KviConsole * KviApp::findConsole(unsigned int ircContextId)
 	return 0;
 }
 
-bool KviApp::isAnyConsoleConnected()
-{
-	QAsciiDictIterator<KviWindow> it(*g_pGlobalWindowDict);
 
-	while(it.current())
-	{
-		if(it.current()->type() == KVI_WINDOW_TYPE_CONSOLE)
-		{
-			if(((KviConsole *)it.current())->isConnected())
-				return true;
-		}
-		++it;
-	}
-	return false;
-}
 
 KviConsole * KviApp::topmostConnectedConsole()
 {
@@ -1676,7 +1750,7 @@ KviConsole * KviApp::activeConsole()
 /////////////////// RECENT STUFF LISTS & POPUPS HANDLING
 
 // Helper for KviApp::addRecent*()
-static void merge_to_stringlist_option(QString &item,int iOption,int iMaxEntries)
+static void merge_to_stringlist_option(const QString &item,int iOption,int iMaxEntries)
 {
 	for(QStringList::Iterator it = KVI_OPTION_STRINGLIST(iOption).begin();
 			it != KVI_OPTION_STRINGLIST(iOption).end(); ++it)
@@ -1703,12 +1777,86 @@ void KviApp::addRecentNickname(const char * newNick)
 	merge_to_stringlist_option(nk,KviOption_stringlistRecentNicknames,KVI_MAX_RECENT_NICKNAMES);
 }
 
-void KviApp::addRecentChannel(const char * chan)
+void KviApp::addRecentNickname(const QString& newNick)
 {
-	QString ch(chan);
-	merge_to_stringlist_option(ch,KviOption_stringlistRecentChannels,KVI_MAX_RECENT_CHANNELS);
+	
+	merge_to_stringlist_option(newNick,KviOption_stringlistRecentNicknames,KVI_MAX_RECENT_NICKNAMES);
 }
 
+void KviApp::addRecentChannel(const QString& szChan,const QString& net)
+{
+	if(!m_pRecentChannelsDict)
+		buildRecentChannels();
+	QStringList* pList=m_pRecentChannelsDict->find(net.utf8().data());
+	if(pList)
+	{
+		if(!pList->contains(szChan)) pList->append(szChan);
+	}
+	else
+	{
+		pList=new QStringList(szChan);
+		m_pRecentChannelsDict->insert(net.utf8().data(),pList);
+	}
+}
+
+void KviApp::buildRecentChannels()
+{
+	if(m_pRecentChannelsDict)
+		delete m_pRecentChannelsDict;
+	m_pRecentChannelsDict = new QAsciiDict<QStringList>;
+	m_pRecentChannelsDict->setAutoDelete(TRUE);
+	QString szChan,szNet;
+	for ( 
+		QStringList::Iterator it = KVI_OPTION_STRINGLIST(KviOption_stringlistRecentChannels).begin();
+		it != KVI_OPTION_STRINGLIST(KviOption_stringlistRecentChannels).end();
+		++it
+	) 
+	{
+		if(!(*it).isEmpty())
+		{
+			szChan = (*it).section( KVI_RECENT_CHANNELS_SEPARATOR, 0, 0 );
+			szNet = (*it).section( KVI_RECENT_CHANNELS_SEPARATOR, 1 );
+			if(!szNet.isEmpty())
+			{
+				QStringList* pList=m_pRecentChannelsDict->find(szNet.utf8().data());
+				if(pList)
+				{
+					if(!pList->contains(szChan)) pList->append(szChan);
+				}
+				else
+				{
+					pList=new QStringList(szChan);
+					m_pRecentChannelsDict->insert(szNet.utf8().data(),pList);
+				}
+			}
+		}
+	}
+}
+
+void KviApp::saveRecentChannels()
+{
+	if(!m_pRecentChannelsDict) return;
+	QString szTmp;
+	KVI_OPTION_STRINGLIST(KviOption_stringlistRecentChannels).clear();
+	QAsciiDictIterator<QStringList> it( *m_pRecentChannelsDict );
+	for( ; it.current(); ++it )
+	{
+		for ( QStringList::Iterator it_str = it.current()->begin(); it_str != it.current()->end(); ++it_str ) {
+			szTmp=*it_str;
+			szTmp.append(KVI_RECENT_CHANNELS_SEPARATOR);
+			szTmp.append(it.currentKey());
+			KVI_OPTION_STRINGLIST(KviOption_stringlistRecentChannels).append(szTmp);
+		}
+        }
+}
+
+QStringList* KviApp::getRecentChannels(const QString& net)
+{
+	if(!m_pRecentChannelsDict) buildRecentChannels();
+	return m_pRecentChannelsDict->find(net.utf8().data());
+}
+
+
 void KviApp::addRecentServer(const char * server)
 {
 	QString sr(server);
@@ -1749,12 +1897,16 @@ void KviApp::fillRecentChannelsPopup(QPopupMenu * m,KviConsole * pConsole)
 {
 	m->clear();
 	int id;
-	for(QStringList::Iterator it = KVI_OPTION_STRINGLIST(KviOption_stringlistRecentChannels).begin(); it != KVI_OPTION_STRINGLIST(KviOption_stringlistRecentChannels).end(); ++it)
+	QStringList* pList=getRecentChannels(pConsole->currentNetworkName());
+	if(pList)
 	{
-		id = m->insertItem(*(g_pIconManager->getSmallIcon(KVI_SMALLICON_CHANNEL)),*it);
-		if(!pConsole->isConnected())m->setItemEnabled(id,false);
-		else {
-			m->setItemEnabled(id,!(pConsole->connection()->findChannel(*it)));
+		for(QStringList::Iterator it = pList->begin(); it != pList->end(); ++it)
+		{
+			id = m->insertItem(*(g_pIconManager->getSmallIcon(KVI_SMALLICON_CHANNEL)),*it);
+			if(!pConsole->isConnected())m->setItemEnabled(id,false);
+			else {
+				m->setItemEnabled(id,!(pConsole->connection()->findChannel(*it)));
+			}
 		}
 	}
 }
@@ -1783,10 +1935,10 @@ bool KviApp::playFile(const char * filename,KviStr &error,KviWindow * w)
 		g_pMediaManager->unlock();
 		if(szCommandline.hasData())
 		{
-			KviParameterList * l = new KviParameterList;
-			l->append(new KviStr(filename));
+			KviKvsVariantList l;
+			l.append(new KviKvsVariant(QString(filename)));
 
-			if(!g_pUserParser->parseCommandBuffer(szCommandline.ptr(),w ? w : g_pActiveWindow,l))
+			if(!KviKvsScript::run(szCommandline.ptr(),w ? w : g_pActiveWindow,&l))
 			{
 				error.sprintf(__tr("The commandline for media type '%s' seems to be broken"),szDescription.ptr());
 				return false;
@@ -1803,6 +1955,34 @@ bool KviApp::playFile(const char * filename,KviStr &error,KviWindow * w)
 	return true;
 }
 
+void KviApp::heartbeat(kvi_time_t tNow)
+{
+	if(g_pApp->topmostConnectedConsole())
+	{
+		// FIXME: this has huge precision problems...
+		KVI_OPTION_UINT(KviOption_uintTotalConnectionTime)++;
+	}
+	
+	// the line below is an approximation of (tNow / 120) == 0
+	// we don't need a really great precision here, so 128 is still ok
+	if(!(tNow & 0x7f))
+		KviDoubleBuffer::heartbeat();
+}
+
+void KviApp::timerEvent(QTimerEvent *e)
+{
+	if(e->timerId() != m_iHeartbeatTimerId)
+	{
+		QObject::timerEvent(e);
+		return;
+	}
+
+	// our heartbeat
+	
+	kvi_time_t tNow = kvi_unixTime();
+
+	heartbeat(tNow);
+}
 
 // qvariant.h uses this , and it is included by the qt generated moc file for Qt >= 3.0.0
 #ifdef Bool
@@ -1810,5 +1990,4 @@ bool KviApp::playFile(const char * filename,KviStr &error,KviWindow * w)
 #endif
 
 
-
 #include "kvi_app.moc"
