@@ -41,25 +41,10 @@ extern KviMediaManager * g_pMediaManager;
 
 //============ getGlobalKvircDirectory ================//
 
-void KviApp::getGlobalKvircDirectory(QString &szData,KvircSubdir dir,const char *appendFile)
+void KviApp::getGlobalKvircDirectory(QString &szData,KvircSubdir dir,const QString &appendFile)
 {
-	KviStr szBuf;
-	getGlobalKvircDirectory(szBuf,dir,appendFile);
-	szData = szBuf.ptr();
-}
-
-void KviApp::getLocalKvircDirectory(QString &szData,KvircSubdir dir,const QString & appendFile,bool bCreateIfNeeded)
-{
-	KviStr szBuf;
-	getLocalKvircDirectory(szBuf,dir,appendFile.isNull() ? 0 : appendFile.latin1(),bCreateIfNeeded);
-	szData = szBuf.ptr();
-}
-
-
-void KviApp::getGlobalKvircDirectory(KviStr &szData,KvircSubdir dir,const char *appendFile)
-{
-	szData=m_szGlobalKvircDir.ptr();
-	szData.ensureLastCharIs(KVI_PATH_SEPARATOR_CHAR);
+	szData=m_szGlobalKvircDir;
+	KviQString::ensureLastCharIs(szData,KVI_PATH_SEPARATOR_CHAR);
 
 	switch(dir)
 	{
@@ -67,23 +52,23 @@ void KviApp::getGlobalKvircDirectory(KviStr &szData,KvircSubdir dir,const char *
 		case Pics          : szData.append("pics");                                                    break;
 		case Modules       :
 		case Plugins       : szData.append("modules");                                                 break;
-		case ConfigPlugins : szData.append(KviStr::Format,"config%smodules",KVI_PATH_SEPARATOR);       break;
-		case ConfigScripts : szData.append(KviStr::Format,"config%sscripts",KVI_PATH_SEPARATOR);       break;
+		case ConfigPlugins : KviQString::appendFormatted(szData,"config%smodules",KVI_PATH_SEPARATOR);       break;
+		case ConfigScripts : KviQString::appendFormatted(szData,"config%sscripts",KVI_PATH_SEPARATOR);       break;
 		case Help          :
 			szData.append("help");
 			{
 				// Localized help
-				KviStr tmp = KviLocale::localeName();
+				QString tmp(KviLocale::localeName().ptr());
 				tmp.prepend(KVI_PATH_SEPARATOR_CHAR);
 				tmp.prepend(szData);
-				if(kvi_directoryExists(tmp.ptr()))szData = tmp;
+				if(KviFileUtils::directoryExists(tmp))szData = tmp;
 				else {
-					tmp = szData; tmp.append(KviStr::Format,"%sen",KVI_PATH_SEPARATOR);
-					if(kvi_directoryExists(tmp.ptr()))szData = tmp;
+					tmp = szData; KviQString::appendFormatted(tmp,"%sen",KVI_PATH_SEPARATOR);
+					if(KviFileUtils::directoryExists(tmp))szData = tmp;
 				}
 			}
 		break;
-		case HelpEN        : szData.append(KviStr::Format,"help%sen",KVI_PATH_SEPARATOR);              break;
+		case HelpEN        : KviQString::appendFormatted(szData,"help%sen",KVI_PATH_SEPARATOR);              break;
 		case HelpNoIntl    : szData.append("help");                                                    break;
 		case Log           : debug("WARNING Global log directory requested!");                         break;
 		case Incoming      : debug("WARNING Global incoming directory requested!");                    break;
@@ -100,38 +85,38 @@ void KviApp::getGlobalKvircDirectory(KviStr &szData,KvircSubdir dir,const char *
 		case Locale        : szData.append("locale");                                                  break;
 		case Tmp           : debug("WARNING Global tmp directory requested!");                         break;
 		case Themes        : szData.append("themes");                                                  break;
+		case Classes       : szData.append("classes");                                                 break;
 	}
 
-	if(appendFile)
+	if(!appendFile.isEmpty())
 	{
-		szData.ensureLastCharIs(KVI_PATH_SEPARATOR_CHAR);
+		KviQString::ensureLastCharIs(szData,KVI_PATH_SEPARATOR_CHAR);
 		szData.append(appendFile);
 	}
 
-	kvi_adjustFilePath(szData);
+	KviFileUtils::adjustFilePath(szData);
 }
 
-//=============== getLocalKvircDirectory ================//
-
-void KviApp::getLocalKvircDirectory(KviStr &szData,KvircSubdir dir,const char *appendFile,bool bCreateIfNeeded)
+void KviApp::getLocalKvircDirectory(QString &szData,KvircSubdir dir,const QString & appendFile,bool bCreateIfNeeded)
 {
-	szData=m_szLocalKvircDir.ptr();
-	szData.ensureLastCharIs(KVI_PATH_SEPARATOR_CHAR);
+	szData=m_szLocalKvircDir;
+	KviQString::ensureLastCharIs(szData,KVI_PATH_SEPARATOR_CHAR);
 
 	switch(dir)
 	{
 		case None          :                                                                     break;
 		case Pics          : szData.append("pics");                                              break;
 		case Config        : szData.append("config");                                            break;
-		case ConfigPlugins : szData.append(KviStr::Format,"config%smodules",KVI_PATH_SEPARATOR); break;
-		case ConfigScripts : szData.append(KviStr::Format,"config%sscripts",KVI_PATH_SEPARATOR); break;
+		case ConfigPlugins : szData.append("config"KVI_PATH_SEPARATOR"modules");                 break;
+		case ConfigScripts : szData.append("config"KVI_PATH_SEPARATOR"scripts");                 break;
 		case Log           : szData.append("log");                                               break;
 		case Incoming      :
 			szData = KVI_OPTION_STRING(KviOption_stringIncomingPath);
 			// we should take special care here
 			// the user is likely to mess the path behind our back
 			// try to recover from most common problems
-			while(szData.lastCharIs(KVI_PATH_SEPARATOR_CHAR))szData.cutRight(1);
+			while(KviQString::lastCharIs(szData,KVI_PATH_SEPARATOR_CHAR))
+				szData = szData.remove(szData.length()-1,1);
 			if(szData.isEmpty())
 			{
 				// unset ?
@@ -142,7 +127,20 @@ void KviApp::getLocalKvircDirectory(KviStr &szData,KvircSubdir dir,const char *a
 			}
 		break;
 		case Help          :
-		case HelpEN        :
+			szData.append("help");
+			{
+				// Localized help
+				QString tmp = KviLocale::localeName().ptr();
+				tmp.prepend(KVI_PATH_SEPARATOR_CHAR);
+				tmp.prepend(szData);
+				if(KviFileUtils::directoryExists(tmp))szData = tmp;
+				else {
+					tmp = szData; tmp.append(KVI_PATH_SEPARATOR"en");
+					if(KviFileUtils::directoryExists(tmp))szData = tmp;
+				}
+			}
+		break;
+		case HelpEN        : szData.append("help"KVI_PATH_SEPARATOR"en");                        break;
 		case HelpNoIntl    : szData.append("help");                                              break;
 		case Audio         : szData.append("audio");                                             break;
 		case Scripts       : szData.append("scripts");                                           break;
@@ -158,16 +156,35 @@ void KviApp::getLocalKvircDirectory(KviStr &szData,KvircSubdir dir,const char *a
 		case Locale        : szData.append("locale");                                            break;
 		case Themes        : szData.append("themes");                                            break;
 		case DefScript     : szData.append("defscript");  /* should never happen! */             break;
+		case Classes       : szData.append("classes");                                           break;
 	}
-	if(bCreateIfNeeded)kvi_makeDir(szData.ptr());
+	if(bCreateIfNeeded)KviFileUtils::makeDir(szData);
 
-	if(appendFile)
+	if(!appendFile.isEmpty())
 	{
-		szData.ensureLastCharIs(KVI_PATH_SEPARATOR_CHAR);
+		KviQString::ensureLastCharIs(szData,KVI_PATH_SEPARATOR_CHAR);
 		szData.append(appendFile);
 	}
 
-	kvi_adjustFilePath(szData);
+	KviFileUtils::adjustFilePath(szData);
+}
+
+
+void KviApp::getGlobalKvircDirectory(KviStr &szData,KvircSubdir dir,const QString &appendFile)
+{
+	QString szBuffer = szData.ptr();
+	getGlobalKvircDirectory(szBuffer,dir,appendFile);
+	szData=szBuffer;
+	
+}
+
+//=============== getLocalKvircDirectory ================//
+
+void KviApp::getLocalKvircDirectory(KviStr &szData,KvircSubdir dir,const QString &appendFile,bool bCreateIfNeeded)
+{
+	QString szBuffer = szData.ptr();
+	getLocalKvircDirectory(szBuffer,dir,appendFile,bCreateIfNeeded);
+	szData=szBuffer;
 }
 
 void KviApp::getTmpFileName(KviStr &szBuffer,const char * pattern)
@@ -182,7 +199,7 @@ void KviApp::getTmpFileName(KviStr &szBuffer,const char * pattern)
 	do {
 		szBuffer.append(KviStr::Format,"kvirc_%d_%s",tmv.tv_usec,pattern ? pattern : "file.tmp");
 		tmv.tv_usec++;
-	} while(kvi_fileExists(szBuffer.ptr()));
+	} while(KviFileUtils::fileExists(szBuffer.ptr()));
 }
 
 //====================== trashFile ====================//
@@ -193,8 +210,8 @@ void KviApp::getTmpFileName(KviStr &szBuffer,const char * pattern)
 bool KviApp::trashFile(const char *filename)
 {
 	// Exists ?
-	QFileInfo fi(filename);
-	if(!fi.exists())return false; // nothing to trash
+	QFileInfo fi = QFileInfo(QString::fromUtf8(filename));
+	if( !fi.exists() ) return false; // nothing to trash
 
 	// Build the filename
 	KviStr lastPart = filename;
@@ -215,7 +232,7 @@ bool KviApp::trashFile(const char *filename)
 		fi.setFile(trashFile.ptr());
 	}
 	// rename the file
-	return kvi_renameFile(filename,trashFile.ptr());
+	return KviFileUtils::renameFile(filename,trashFile.ptr());
 }
 
 void KviApp::completeDirectory(const QString &word,KviPtrList<QString> * matches)
@@ -293,7 +310,7 @@ void KviApp::getChannelDumpLogFileName(KviStr &str)
 			fPrefix+="x";
 			logN = 0;
 		}
-	} while(kvi_fileExists(str.ptr()));
+	} while(KviFileUtils::fileExists(str.ptr()));
 	kvi_adjustFilePath(str);
 }
 
@@ -309,7 +326,7 @@ bool KviApp::findFileByMediaType(KviStr &szRetPath,const char * filename)
 			szRetPath = m->szSavePath;
 			szRetPath.ensureLastCharIs(KVI_PATH_SEPARATOR_CHAR);
 			szRetPath.append(filename);
-			if(kvi_fileExists(szRetPath.ptr()))
+			if(KviFileUtils::fileExists(szRetPath.ptr()))
 			{
 				// check again the media type... check the magic too
 				KviMediaType * mt = g_pMediaManager->findMediaType(szRetPath.ptr());
@@ -343,7 +360,7 @@ bool KviApp::findUserFile(KviStr &szRetPath,const char *filename)
 	{
 		szRetPath=filename;
 		kvi_adjustFilePath(szRetPath);
-		return kvi_fileExists(filename);
+		return KviFileUtils::fileExists(filename);
 	}
 
 	// FIXME: #warning "Use the user defined path ?"
@@ -357,13 +374,13 @@ bool KviApp::findUserFile(KviStr &szRetPath,const char *filename)
 	for(i= 0;i<5; i++)
 	{
 		getLocalKvircDirectory(szRetPath,localsubdirs[i],filename);
-		if(kvi_fileExists(szRetPath.ptr()))return true;
+		if(KviFileUtils::fileExists(szRetPath.ptr()))return true;
 	}
 
 	for(i= 0;i<3; i++)
 	{
 		getGlobalKvircDirectory(szRetPath,globalsubdirs[i],filename);
-		if(kvi_fileExists(szRetPath.ptr()))return true;
+		if(KviFileUtils::fileExists(szRetPath.ptr()))return true;
 	}
 
 	if(findImageInImageSearchPath(szRetPath,filename))return true;
@@ -372,7 +389,7 @@ bool KviApp::findUserFile(KviStr &szRetPath,const char *filename)
 	szRetPath = QDir::homeDirPath();
 	szRetPath.ensureLastCharIs(KVI_PATH_SEPARATOR_CHAR);
 	szRetPath.append(filename);
-	if(kvi_fileExists(szRetPath.ptr()))return true;;
+	if(KviFileUtils::fileExists(szRetPath.ptr()))return true;;
 
 	szRetPath = filename;
 	//if(retMediaType)*retMediaType = m;
@@ -404,7 +421,23 @@ bool KviApp::findImageInImageSearchPath(KviStr &szRetPath,const char * filename)
 		szRetPath.ensureLastCharIs(KVI_PATH_SEPARATOR_CHAR);
 		szRetPath.append(filename);
 		//debug("LOOK FOR %s",szRetPath.ptr());
-		if(kvi_fileExists(szRetPath.ptr()))return true;
+		if(KviFileUtils::fileExists(szRetPath.ptr()))return true;
+	}
+
+	return false;
+}
+
+bool KviApp::findImageInImageSearchPath(QString &szRetPath,const char * filename)
+{
+	// first lookup the user defined paths
+	for(QStringList::Iterator it = KVI_OPTION_STRINGLIST(KviOption_stringlistImageSearchPaths).begin();
+			it != KVI_OPTION_STRINGLIST(KviOption_stringlistImageSearchPaths).end();++it)
+	{
+		szRetPath = *it;
+		KviQString::ensureLastCharIs(szRetPath,KVI_PATH_SEPARATOR_CHAR);
+		szRetPath.append(filename);
+		//debug("LOOK FOR %s",szRetPath.ptr());
+		if(KviFileUtils::fileExists(szRetPath))return true;
 	}
 
 	return false;
@@ -413,24 +446,31 @@ bool KviApp::findImageInImageSearchPath(KviStr &szRetPath,const char * filename)
 static KviApp::KvircSubdir pics_localsubdirs[2]={ KviApp::Pics , KviApp::Avatars };
 static KviApp::KvircSubdir pics_globalsubdirs[2]={ KviApp::Pics , KviApp::Avatars };
 
-
 bool KviApp::mapImageFile(KviStr &szRetPath,const char * filename)
+{
+	QString buff;
+	bool ret=mapImageFile(buff,filename);
+	szRetPath=buff;
+	return ret;
+}
+
+bool KviApp::mapImageFile(QString &szRetPath,const char * filename)
 {
 	szRetPath = filename;
 	// can't map non absolute paths
 	if(!kvi_isAbsolutePath(filename))return false;
 	// can't map non existing paths
-	if(!kvi_fileExists(filename))return false;
+	if(!KviFileUtils::fileExists(filename))return false;
 
-	QFileInfo fi(filename);
+	QFileInfo fi = QFileInfo(QString::fromUtf8(filename));
 
 	unsigned int size = fi.size();
 
 	KviStr szBestMatch;
 
-	while(szRetPath.findFirstIdx(KVI_PATH_SEPARATOR) != -1)
+	while(szRetPath.find(KVI_PATH_SEPARATOR) != -1)
 	{
-		szRetPath.cutToFirst(KVI_PATH_SEPARATOR);
+		KviQString::cutToFirst(szRetPath,KVI_PATH_SEPARATOR);
 
 		if(szRetPath.isEmpty())break;
 
@@ -440,10 +480,10 @@ bool KviApp::mapImageFile(KviStr &szRetPath,const char * filename)
 
 		for(i=0;i<2;i++)
 		{
-			getLocalKvircDirectory(szBuffer,pics_localsubdirs[i],szRetPath.ptr());
-			if(kvi_fileExists(szBuffer.ptr()))
+			getLocalKvircDirectory(szBuffer,pics_localsubdirs[i],szRetPath);
+			if(KviFileUtils::fileExists(szBuffer.ptr()))
 			{
-				QFileInfo fi2(szBuffer.ptr());
+				QFileInfo fi2(QString::fromUtf8(szBuffer.ptr()));
 				if(size == fi2.size())
 				{
 					// probably the same file
@@ -454,11 +494,11 @@ bool KviApp::mapImageFile(KviStr &szRetPath,const char * filename)
 	
 		for(i=0;i<2;i++)
 		{
-			getGlobalKvircDirectory(szBuffer,pics_globalsubdirs[i],szRetPath.ptr());
+			getGlobalKvircDirectory(szBuffer,pics_globalsubdirs[i],szRetPath);
 			//debug("CHECK %s",szBuffer.ptr());
-			if(kvi_fileExists(szBuffer.ptr()))
+			if(KviFileUtils::fileExists(szBuffer.ptr()))
 			{
-				QFileInfo fi2(szBuffer.ptr());
+				QFileInfo fi2(QString::fromUtf8(szBuffer.ptr()));
 				if(size == fi2.size())
 				{
 					// probably the same file
@@ -471,9 +511,9 @@ bool KviApp::mapImageFile(KviStr &szRetPath,const char * filename)
 		szBuffer = QDir::homeDirPath();
 		szBuffer.ensureLastCharIs(KVI_PATH_SEPARATOR_CHAR);
 		szBuffer.append(szRetPath);
-		if(kvi_fileExists(szBuffer.ptr()))
+		if(KviFileUtils::fileExists(szBuffer.ptr()))
 		{
-			QFileInfo fi2(szBuffer.ptr());
+			QFileInfo fi2(QString::fromUtf8(szBuffer.ptr()));
 			if(size == fi2.size())
 			{
 				// prolly the same file
@@ -494,27 +534,36 @@ bool KviApp::mapImageFile(KviStr &szRetPath,const char * filename)
 	return false;
 }
 
+
 bool KviApp::findImage(KviStr &szRetPath,const char *filename)
+{
+	QString buff;
+	bool ret=findImage(buff,filename);
+	szRetPath=buff;
+	return ret;
+}
+
+bool KviApp::findImage(QString &szRetPath,const char *filename)
 {
 	// Find an user file...
 	if(kvi_isAbsolutePath(filename))
 	{
 		szRetPath=filename;
-		return kvi_fileExists(filename);
+		return KviFileUtils::fileExists(filename);
 	}
 
 	// if we have a theme, look it up as first
 	if(!KVI_OPTION_STRING(KviOption_stringIconThemeSubdir).isEmpty())
 	{
-		KviStr szTmp = KVI_OPTION_STRING(KviOption_stringIconThemeSubdir);
+		QString szTmp = KVI_OPTION_STRING(KviOption_stringIconThemeSubdir);
 		szTmp.append(KVI_PATH_SEPARATOR_CHAR);
 		szTmp.append(filename);
 
-		getLocalKvircDirectory(szRetPath,KviApp::Themes,szTmp.ptr());
-		if(kvi_fileExists(szRetPath.ptr()))return true;
+		getLocalKvircDirectory(szRetPath,KviApp::Themes,szTmp);
+		if(KviFileUtils::fileExists(szRetPath))return true;
 
-		getGlobalKvircDirectory(szRetPath,KviApp::Themes,szTmp.ptr());
-		if(kvi_fileExists(szRetPath.ptr()))return true;
+		getGlobalKvircDirectory(szRetPath,KviApp::Themes,szTmp);
+		if(KviFileUtils::fileExists(szRetPath))return true;
 	} //else {
 		//debug("NO PATH YET");
 	//}
@@ -524,22 +573,22 @@ bool KviApp::findImage(KviStr &szRetPath,const char *filename)
 	for(i= 0;i<2; i++)
 	{
 		getLocalKvircDirectory(szRetPath,pics_localsubdirs[i],filename);
-		if(kvi_fileExists(szRetPath.ptr()))return true;
+		if(KviFileUtils::fileExists(szRetPath))return true;
 	}
 
 	for(i= 0;i<2; i++)
 	{
 		getGlobalKvircDirectory(szRetPath,pics_globalsubdirs[i],filename);
-		if(kvi_fileExists(szRetPath.ptr()))return true;
+		if(KviFileUtils::fileExists(szRetPath))return true;
 	}
 
 	if(findImageInImageSearchPath(szRetPath,filename))return true;
 
 	// Last resort
 	szRetPath = QDir::homeDirPath();
-	szRetPath.ensureLastCharIs(KVI_PATH_SEPARATOR_CHAR);
+	KviQString::ensureLastCharIs(szRetPath,KVI_PATH_SEPARATOR_CHAR);
 	szRetPath.append(filename);
-	if(kvi_fileExists(szRetPath.ptr()))return true;
+	if(KviFileUtils::fileExists(szRetPath))return true;
 
 	szRetPath = filename;
 
@@ -556,12 +605,13 @@ bool KviApp::getReadOnlyConfigPath(KviStr &buffer,const char *config_name,KvircS
 {
 	// Take a look in the local directory....
 	getLocalKvircDirectory(buffer,sbd,config_name);
-	if(!kvi_fileExists(buffer.ptr()))
+	//debug("%s %s %i |%s| %i",__FILE__,__FUNCTION__,__LINE__,buffer.ptr(),KviFileUtils::fileExists(buffer.ptr()));
+	if(!KviFileUtils::fileExists(buffer.ptr()))
 	{
 		// No saved config yet... check for defaults
 		KviStr tmp;
 		getGlobalKvircDirectory(tmp,sbd,config_name);
-		if(!kvi_fileExists(tmp.ptr()))
+		if(!KviFileUtils::fileExists(tmp.ptr()))
 		{
 			// No defaults...no such config file at all.
 			if(bNoFail)getLocalKvircDirectory(buffer,sbd,config_name);
