@@ -37,11 +37,15 @@
 #include "kvi_mdimanager.h"
 #include "kvi_irctoolbar.h"
 #include "kvi_locale.h"
-#include "kvi_uparser.h"
+//#include "kvi_uparser.h"
 #include "kvi_ircurl.h"
 #include "kvi_ircconnection.h"
 #include "kvi_ircconnectionuserinfo.h"
 #include "kvi_coreactionnames.h"
+#include "kvi_ircconnectionserverinfo.h"
+#include "kvi_styled_controls.h"
+
+#include "kvi_kvs_script.h"
 
 #include <qpopupmenu.h>
 #include <qobject.h>
@@ -52,6 +56,7 @@
 // Ctrl+Alt+S : Servers dialog
 // Ctrl+Alt+I : Identity dialog
 // Ctrl+Alt+O : General options
+// Ctrl+Alt+T : Theme options
 // Ctrl+Alt+J : Join Channels
 // Ctrl+Alt+X : Execute script
 // Ctrl+Alt+N : New irc context
@@ -63,6 +68,7 @@
 // Ctrl+Alt+6 : Customize toolbars
 // Ctrl+Alt+H : Show help index
 // Ctrl+Alt+R : Edit regusers
+// Ctrl+Alt+M : Manage Script Addons
 
 void register_core_actions(KviActionManager * m)
 {
@@ -116,6 +122,17 @@ void register_core_actions(KviActionManager * m)
 		0,
 		"Ctrl+Alt+S");
 
+	SCRIPT_ACTION(
+		KVI_COREACTION_MANAGEADDONS,
+		"addon.dialog",
+		__tr2qs("Manage Addons..."),
+		__tr2qs("Allows you to manage the script-based addons"),
+		KviActionManager::categorySettings(),
+		"kvi_bigicon_addons.png",
+		KVI_SMALLICON_ADDONS,
+		0,
+		"Ctrl+Alt+M");
+
 	SCRIPT_ACTION(
 		KVI_COREACTION_EDITREGUSERS,
 		"reguser.edit -t",
@@ -182,6 +199,17 @@ void register_core_actions(KviActionManager * m)
 		0,
 		"Ctrl+Alt+O");
 
+	SCRIPT_ACTION(
+		KVI_COREACTION_THEMEOPTIONS,
+		"options.dialog theme",
+		__tr2qs("Configure Theme..."),
+		__tr2qs("Shows the theme options dialog"),
+		KviActionManager::categorySettings(),
+		"kvi_bigicon_themeoptions.png",
+		KVI_SMALLICON_THEMEOPTIONS,
+		0,
+		"Ctrl+Alt+T");
+
 	SCRIPT_ACTION(
 		KVI_COREACTION_TOOLBAREDITOR,
 		"toolbareditor.open",
@@ -274,7 +302,7 @@ void register_core_actions(KviActionManager * m)
 
 	SCRIPT_ACTION(
 		KVI_COREACTION_EXECUTEKVS,
-		"dialog.file(open,$tr(\"Select a Script File\")){ if(\"$0\" != \"\")parse $0; }",
+		"dialog.file(open,$tr(\"Select a Script File\")){ if(!$str.isEmpty($0))parse $0; }",
 		__tr2qs("Execute Script..."),
 		__tr2qs("Allows executing a KVS script from file"),
 		KviActionManager::categoryScripting(),
@@ -315,6 +343,17 @@ void register_core_actions(KviActionManager * m)
 		KVI_SMALLICON_KVIRC,
 		0,
 		QString::null);
+	
+	SCRIPT_ACTION(
+		KVI_COREACTION_KVIRCRUHOMEPAGE,
+		"openurl http://www.kvirc.ru",
+		__tr2qs("KVIrc Russian WWW"),
+		__tr2qs("Opens the KVIrc homepage in russian"),
+		KviActionManager::categoryGeneric(),
+		"kvi_bigicon_kvircwww.png",
+		KVI_SMALLICON_KVIRC,
+		0,
+		QString::null);
 
 
 	SLOT_ACTION(
@@ -613,7 +652,7 @@ bool KviConnectAction::addToPopupMenu(QPopupMenu *p)
 QWidget * KviConnectAction::addToCustomToolBar(KviCustomToolBar *t)
 {
 	if(!setupDone())setup();
-	QToolButton * b = new QToolButton(
+	QToolButton * b = new KviStyledToolButton(
 			*(g_pIconManager->getBigIcon(KVI_BIGICON_DISCONNECTED)),
 			m_szConnectString,
 			m_szConnectString,
@@ -686,13 +725,14 @@ bool KviSubmenuAction::addToPopupMenu(QPopupMenu *p)
 QWidget * KviSubmenuAction::addToCustomToolBar(KviCustomToolBar *t)
 {
 	if(!setupDone())setup();
-	QToolButton * b = new QToolButton(
+	KviStyledToolButton * b = new KviStyledToolButton(
 			*(bigIcon()),
 			visibleName(),
 			visibleName(),
 			this,SLOT(activate()),t,name());
 	b->setPopup(m_pPopup);
 	b->setPopupDelay(1);
+	b->setShowSubmenuIndicator(true);
 	registerWidget(b);
 	if(!isEnabled())b->setEnabled(false);
 	return b;
@@ -735,13 +775,13 @@ void KviJoinChannelAction::popupActivated(int id)
 	QString text = m_pPopup->text(id);
 	if(!text.isEmpty())
 	{
-		KviStr szText;
+		QString szText;
 		if(__tr2qs("Other...") == text)szText = "channelsjoin.open";
 		else {
 			szText = text;
 			szText.prepend("join ");
 		}
-		g_pUserParser->parseCommandBuffer(szText.ptr(),c);
+		KviKvsScript::run(szText,c);
 	}
 }
 
@@ -750,7 +790,7 @@ void KviJoinChannelAction::popupActivated(int id)
 KviChangeNickAction::KviChangeNickAction(QObject * pParent)
 : KviSubmenuAction(pParent,
 	QString(KVI_COREACTION_NICKNAMEMENU),
-	QString("dialog.textinput(\"Change Nickname\",\"Please enter the new nickname\",\"Cancel\",\"OK\"){ if($0 == 1)nick $1; }"),
+	QString("dialog.textinput(\"Change Nickname\",\"Please enter the new nickname\",\"Cancel\",\"default=OK\"){ if($0 == 1)nick $1; }"),
 	__tr2qs("Change Nickname"),
 	__tr2qs("Shows a popup menu that allows quickly changing the nickname"),
 	KviActionManager::categoryIrc(),
@@ -779,14 +819,14 @@ void KviChangeNickAction::popupActivated(int id)
 	QString text = m_pPopup->text(id);
 	if(!text.isEmpty())
 	{
-		KviStr szText;
+		QString szText;
 		if(__tr2qs("Other...") == text)
 		{
 			activate();
 		} else {
 			szText = text;
 			szText.prepend("nick ");
-			g_pUserParser->parseCommandBuffer(szText.ptr(),c);
+			KviKvsScript::run(szText,c);
 		}
 	}
 }
@@ -832,7 +872,7 @@ void KviConnectToServerAction::popupActivated(int id)
 			KviStr szText = text;
 			if(parseIrcUrl(szText.ptr(),szCommand,KVI_IRCURL_CONTEXT_THIS))
 			{
-				g_pUserParser->parseCommandBuffer(szCommand.ptr(),c);
+				KviKvsScript::run(szCommand.ptr(),c);
 			}
 		}
 	}
@@ -862,12 +902,25 @@ void KviChangeUserModeAction::popupAboutToShow()
 
 	m_pPopup->clear();
 	int id;
+	QString szModes = g_pActiveWindow->console()->connection()->serverInfo()->supportedUserModes();
+	
 	id = m_pPopup->insertItem(*(g_pIconManager->getSmallIcon(KVI_SMALLICON_WALLOPS)),__tr2qs("Wallops (+w)"));
 	m_pPopup->setItemChecked(id,c->connection()->userInfo()->hasUserMode('w'));
 	id = m_pPopup->insertItem(*(g_pIconManager->getSmallIcon(KVI_SMALLICON_SERVERNOTICE)),__tr2qs("Server Notices (+s)"));
 	m_pPopup->setItemChecked(id,c->connection()->userInfo()->hasUserMode('s'));
 	id = m_pPopup->insertItem(*(g_pIconManager->getSmallIcon(KVI_SMALLICON_INVISIBLE)),__tr2qs("Invisible (+i)"));
 	m_pPopup->setItemChecked(id,c->connection()->userInfo()->hasUserMode('i'));
+	
+	szModes.replace("w","");
+	szModes.replace("s","");
+	szModes.replace("i","");
+	while(!szModes.isEmpty())
+	{
+		QChar ccc = szModes[0];
+		szModes.remove(0,1);
+		id = m_pPopup->insertItem(*(g_pIconManager->getSmallIcon(KVI_SMALLICON_MODE)),QString("+%1 Mode").arg(ccc));
+		m_pPopup->setItemChecked(id,c->connection()->userInfo()->hasUserMode(ccc.latin1()));
+	}
 }
 
 void KviChangeUserModeAction::popupActivated(int id)
@@ -875,23 +928,22 @@ void KviChangeUserModeAction::popupActivated(int id)
 	KviConsole * c = g_pActiveWindow->console();
 	if(!c)return;
 
-	KviStr text = m_pPopup->text(id);
+	QString text = m_pPopup->text(id);
 	if(!c->isConnected())return;
-	if(text.hasData())
+	if(!text.isEmpty())
 	{
-		int idx = text.findFirstIdx('+');
+		int idx = text.find('+');
 		if(idx != -1)
 		{
 			idx++;
-			if(idx < text.len())
+			if(idx < text.length())
 			{
-				QChar mm = *(text.ptr() + idx);
-				QChar m = mm.lower();
+				QChar m = text[idx];
 				QString command = "mode ";
 				command.append(c->connection()->userInfo()->nickName());
 				command.append(c->connection()->userInfo()->hasUserMode(m) ? " -" : " +");
 				command.append(m);
-				g_pUserParser->parseCommandBuffer(command.utf8().data(),c);
+				KviKvsScript::run(command,c);
 			}
 		}
 	}
@@ -957,13 +1009,13 @@ void KviGoAwayAction::activeContextStateChanged()
 			b->setPixmap(*p);
 			b->setTextLabel(txt);
 		}
-		m_uFlags |= KVI_ACTION_FLAG_ENABLED;
+		m_uInternalFlags |= KVI_ACTION_FLAG_ENABLED;
 	} else {
 		for(QToolButton * b = (QToolButton *)bl->first();b;b = (QToolButton *)bl->next())
 		{
 			if(b->isEnabled())b->setEnabled(false);
 		}
-		m_uFlags &= ~KVI_ACTION_FLAG_ENABLED;
+		m_uInternalFlags &= ~KVI_ACTION_FLAG_ENABLED;
 	}
 }
 
@@ -1013,7 +1065,7 @@ bool KviGoAwayAction::addToPopupMenu(QPopupMenu *p)
 QWidget * KviGoAwayAction::addToCustomToolBar(KviCustomToolBar *t)
 {
 	if(!setupDone())setup();
-	QToolButton * b = new QToolButton(
+	QToolButton * b = new KviStyledToolButton(
 			*(g_pIconManager->getBigIcon("kvi_bigicon_nokeyboard.png")),
 			m_szAwayString,
 			m_szAwayString,
