@@ -41,6 +41,7 @@
 #include <qdragobject.h>
 #include <qcursor.h>
 
+
 // kvi_app.cpp
 extern QPixmap * g_pUserChanStatePixmap;
 extern QPixmap * g_pActivityMeterPixmap;
@@ -322,11 +323,24 @@ static const char * g_szIconNames[KVI_NUM_SMALL_ICONS]=
 	"tearsmile", // 270
 	"shy", // 271
 	"servererror", // 272
-	"cafe" // 273
+	"cafe", // 273
+	"addons", // 274
+	"chanowneraway", // 275
+	"opaway", // 276
+	"voiceaway", // 277
+	"chanadminaway", // 278
+	"useropaway", // 279
+	"halfopaway", // 280
+	"aliashighlighted", // 281
+	"demoralized", // 282
+	"slurp", // 283
+	"namespace",  // 284
+	"saysmile", // 285
+	"saykvs" // 286
 };
 
 KviIconWidget::KviIconWidget()
-: QWidget(0,"global_icon_widget",WType_TopLevel | WStyle_Customize | WStyle_Title | WStyle_StaysOnTop | WStyle_DialogBorder)
+: QWidget(0,"global_icon_widget",WType_TopLevel | WStyle_Customize | WStyle_Title | WStyle_StaysOnTop | WStyle_DialogBorder | WStyle_SysMenu | WStyle_Minimize)
 {
 	init();
 }
@@ -567,11 +581,11 @@ KviCachedPixmap * KviIconManager::getPixmapWithCache(const QString &szName)
 
 	QPixmap * pix = 0;
 
-	KviStr szRetPath;
+	QString szRetPath;
 
-	if(g_pApp->findImage(szRetPath,szName.latin1()))
+	if(g_pApp->findImage(szRetPath,szName))
 	{
-		pix = new QPixmap(szRetPath.ptr());
+		pix = new QPixmap(szRetPath);
 		if(pix->isNull())
 		{
 			delete pix; // it is not an valid image!!! (really bad situation...)
@@ -582,7 +596,7 @@ KviCachedPixmap * KviIconManager::getPixmapWithCache(const QString &szName)
 		return 0;
 	}
 
-	p = new KviCachedPixmap(pix,QString(szRetPath.ptr()));
+	p = new KviCachedPixmap(pix,QString(szRetPath));
 	addToCache(szName,p);
 
 	return p;
@@ -596,13 +610,13 @@ void KviIconManager::addToCache(const QString &szName,KviCachedPixmap * p)
 	m_uCacheTotalSize += p->size();
 }
 
-QPixmap * KviIconManager::getImage(const char * id,bool bCanBeNumber,KviStr * pRetPath)
+QPixmap * KviIconManager::getImage(const char * id,bool bCanBeNumber,QString* pRetPath)
 {
 	if(!id)return 0;
 	if(bCanBeNumber)
 	{
 		bool bOk;
-		KviStr tmp = id;
+		QString tmp = id;
 		int idx = tmp.toInt(&bOk);
 		if(bOk)
 		{
@@ -662,6 +676,7 @@ void KviIconManager::urlToCachedFileName(KviStr &szFName)
 	szFName.replaceAll("\'","_");
 	szFName.replaceAll("(","_");
 	szFName.replaceAll(")","_");
+	szFName.replaceAll("?","_");
 	szFName.replaceAll("___",".");
 }
 
@@ -675,6 +690,7 @@ void KviIconManager::urlToCachedFileName(QString &szFName)
 	szFName.replace("\'","_");
 	szFName.replace("(","_");
 	szFName.replace(")","_");
+	szFName.replace("?","_");
 	szFName.replace("___",".");
 }
 
