@@ -57,13 +57,13 @@
 #include "kvi_notifylist.h"
 #include "kvi_dns.h"
 #include "kvi_defaults.h"
-#include "kvi_uparser.h"
-#include "kvi_event.h"
-#include "kvi_uparser.h"
 #include "kvi_sparser.h"
 #include "kvi_ircdatastreammonitor.h"
 #include "kvi_databuffer.h"
 #include "kvi_lagmeter.h"
+#include "kvi_kvs_eventtriggers.h"
+
+#include "kvi_kvs_script.h"
 
 #include <qtimer.h>
 #include <qtextcodec.h>
@@ -170,6 +170,7 @@ void KviIrcConnection::setEncoding(const QString &szEncoding)
 		}
 	}
 	m_pTextCodec = c;
+	m_pConsole->setTextEncoding(szEncoding);
 }
 
 void KviIrcConnection::setupTextCodec()
@@ -179,7 +180,7 @@ void KviIrcConnection::setupTextCodec()
 	if(!m_pTarget->server()->encoding().isEmpty())
 	{
 		m_pTextCodec = KviLocale::codecForName(m_pTarget->server()->encoding().latin1());
-		if(!m_pTextCodec)debug("KviIrcConnection: can't find QTextCodec for encoding %s",m_pTarget->server()->encoding().latin1());
+		if(!m_pTextCodec)debug("KviIrcConnection: can't find QTextCodec for encoding %s",m_pTarget->server()->encoding().utf8().data());
 	}
 	if(!m_pTextCodec)
 	{
@@ -187,18 +188,19 @@ void KviIrcConnection::setupTextCodec()
 		if(!m_pTarget->network()->encoding().isEmpty())
 		{
 			m_pTextCodec = KviLocale::codecForName(m_pTarget->network()->encoding().latin1());
-			if(!m_pTextCodec)debug("KviIrcConnection: can't find QTextCodec for encoding %s",m_pTarget->network()->encoding().latin1());
+			if(!m_pTextCodec)debug("KviIrcConnection: can't find QTextCodec for encoding %s",m_pTarget->network()->encoding().utf8().data());
 		}
 	}
 	if(!m_pTextCodec)
 	{
 		m_pTextCodec = KviApp::defaultTextCodec();
 	}
+	m_pConsole->setTextEncoding(QString(m_pTextCodec->name()));
 }
 
 QCString KviIrcConnection::encodeText(const QString &szText)
 {
-	if(!m_pTextCodec)return QCString(szText.latin1());
+	if(!m_pTextCodec)return szText.utf8();
 	return m_pTextCodec->fromUnicode(szText);
 }
 
@@ -436,7 +438,8 @@ KviQuery * KviIrcConnection::findQuery(const QString &name)
 void KviIrcConnection::registerChannel(KviChannel * c)
 {
 	m_pChannelList->append(c);
-	g_pApp->addRecentChannel(c->windowName());
+	if(KVI_OPTION_BOOL(KviOption_boolLogChannelHistory))
+		g_pApp->addRecentChannel(c->windowName(),m_pTarget->networkName());
 }
 
 void KviIrcConnection::unregisterChannel(KviChannel * c)
@@ -631,7 +634,7 @@ void KviIrcConnection::restartLagMeter()
 
 void KviIrcConnection::resolveLocalHost()
 {
-	KviStr szIp;
+	QString szIp;
 
 	if(!socket()->getLocalHostIp(szIp,server()->isIpV6()))
 	{
@@ -641,17 +644,17 @@ void KviIrcConnection::resolveLocalHost()
 #ifdef COMPILE_IPV6_SUPPORT
 			if(server()->isIpV6())
 			{
-				if(kvi_isValidStringIp_V6(KVI_OPTION_STRING(KviOption_stringLocalHostIp).latin1()))bGotIp = true;
+				if(KviNetUtils::isValidStringIp_V6(KVI_OPTION_STRING(KviOption_stringLocalHostIp)))bGotIp = true;
 			} else {
 #endif
-				if(kvi_isValidStringIp(KVI_OPTION_STRING(KviOption_stringLocalHostIp).latin1()))bGotIp = true;
+				if(KviNetUtils::isValidStringIp(KVI_OPTION_STRING(KviOption_stringLocalHostIp)))bGotIp = true;
 #ifdef COMPILE_IPV6_SUPPORT
 			}
 #endif
 		}
 		if(bGotIp)
 		{
-			m_pUserInfo->setLocalHostIp(KVI_OPTION_STRING(KviOption_stringLocalHostIp).latin1());
+			m_pUserInfo->setLocalHostIp(KVI_OPTION_STRING(KviOption_stringLocalHostIp));
 			if(!_OUTPUT_MUTE)
 				m_pConsole->output(KVI_OUT_SYSTEMWARNING,__tr2qs("Can't resolve local host address, using user supplied one (%Q)"),
 					&(m_pUserInfo->localHostIp()));
@@ -664,7 +667,7 @@ void KviIrcConnection::resolveLocalHost()
 					&(m_pUserInfo->localHostIp()));
 		}
 	} else {
-		m_pUserInfo->setLocalHostIp(szIp.ptr());
+		m_pUserInfo->setLocalHostIp(szIp);
 		if(!_OUTPUT_QUIET)
 			m_pConsole->output(KVI_OUT_SYSTEMMESSAGE,__tr2qs("Local host address is %Q"),
 				&(m_pUserInfo->localHostIp()));
@@ -689,7 +692,7 @@ void KviIrcConnection::changeAwayState(bool bAway)
 void KviIrcConnection::userInfoReceived(const QString &szUserName,const QString &szHostName)
 {
 	userInfo()->setUserName(szUserName);
-
+	QString szUnmaskedHost = m_pUserInfo->unmaskedHostName();
 	// Update the user entry
 	KviIrcUserEntry * e = userDataBase()->find(userInfo()->nickName());
 	if(e) // should be there! (we have the permanent entry in the notify list view)
@@ -758,13 +761,24 @@ void KviIrcConnection::userInfoReceived(const QString &szUserName,const QString
 			if(!_OUTPUT_MUTE)
 				m_pConsole->output(KVI_OUT_SYSTEMMESSAGE,__tr2qs("The local IP address as seen by the IRC server is %Q"),&szHostName);
 			m_pUserInfo->setHostIp(szHostName);
+		} else 
+#ifdef COMPILE_IPV6_SUPPORT
+		if(KviNetUtils::isValidStringIp(szUnmaskedHost) || KviNetUtils::isValidStringIp_V6(szUnmaskedHost))
+#else
+		if(KviNetUtils::isValidStringIp(szUnmaskedHost))
+#endif		 
+		{
+			if(!_OUTPUT_MUTE)
+				m_pConsole->output(KVI_OUT_SYSTEMMESSAGE,__tr2qs("The local IP address as seen by the IRC server is %Q"),&szUnmaskedHost);
+			m_pUserInfo->setHostIp(szUnmaskedHost);
+		
 		} else {
 			// look it up too
 			if(m_pLocalhostDns)delete m_pLocalhostDns; // it could be only another local host lookup
 			m_pLocalhostDns = new KviDns();
 			connect(m_pLocalhostDns,SIGNAL(lookupDone(KviDns *)),this,SLOT(hostNameLookupTerminated(KviDns *)));
 
-			if(!m_pLocalhostDns->lookup(szHostName.latin1(),KviDns::Any))
+			if(!m_pLocalhostDns->lookup(szHostName,KviDns::Any))
 			{
 				if(!_OUTPUT_MUTE)
 				{
@@ -796,7 +810,11 @@ void KviIrcConnection::hostNameLookupTerminated(KviDns *pDns)
 	if(m_pLocalhostDns->state() != KviDns::Success)
 	{
 		QString szErr = KviError::getDescription(m_pLocalhostDns->error());
-		m_pConsole->output(KVI_OUT_SYSTEMMESSAGE,__tr2qs("Unable to resolve the local hostname as seen by the IRC server: %Q"),
+		if(!m_pUserInfo->hostIp().isEmpty())
+			m_pConsole->output(KVI_OUT_SYSTEMMESSAGE,__tr2qs("Unable to resolve the local hostname as seen by the IRC server: %Q, using previously resolved %Q"),
+			&szErr,&(m_pUserInfo->hostIp()));
+		else
+			m_pConsole->output(KVI_OUT_SYSTEMMESSAGE,__tr2qs("Unable to resolve the local hostname as seen by the IRC server: %Q"),
 			&szErr);
 	} else {
 		m_pConsole->output(KVI_OUT_SYSTEMMESSAGE,__tr2qs("Local hostname as seen by the IRC server resolved to %s"),m_pLocalhostDns->firstIpAddress());
@@ -881,10 +899,10 @@ void KviIrcConnection::loginToIrcServer()
 	}
 	
 	// FIXME: The server's encoding!
-
-	QCString szNick = console()->encodeText(m_pUserInfo->nickName()); // never empty
-	QCString szUser = console()->encodeText(m_pUserInfo->userName()); // never empty
-	QCString szReal = console()->encodeText(m_pUserInfo->realName()); // may be empty
+	setupTextCodec();
+	QCString szNick = encodeText(m_pUserInfo->nickName()); // never empty
+	QCString szUser = encodeText(m_pUserInfo->userName()); // never empty
+	QCString szReal = encodeText(m_pUserInfo->realName()); // may be empty
 
 	if(!szReal.data())szReal = "";
 
@@ -892,12 +910,9 @@ void KviIrcConnection::loginToIrcServer()
 		m_pConsole->output(KVI_OUT_SYSTEMMESSAGE,__tr2qs("Logging in as %Q!%Q :%Q"),
 			&(m_pUserInfo->nickName()),&(m_pUserInfo->userName()),&(m_pUserInfo->realName()));
 
-	if(!sendFmtData("USER %s 0 %s :%s",szUser.data(),
-			pServer->m_szHostname.latin1(),szReal.data()))
-	{
-		// disconnected in the meantime!
-		return;
-	}
+
+	// spity, 27.03.2005: follow the RFC2812 suggested order for connection registration
+	// first the PASS, then NICK and then USER
 
 	// The pass ?
 	pServer->m_szPass.stripWhiteSpace();
@@ -910,18 +925,29 @@ void KviIrcConnection::loginToIrcServer()
 		if(!_OUTPUT_MUTE)
 			m_pConsole->output(KVI_OUT_VERBOSE,__tr2qs("Sending %s as password"),szHidden.ptr());
 
-		if(!sendFmtData("PASS %s",pServer->m_szPass.latin1()))
+		// The colon should allow user to use passwords with whitespaces.
+		// Non-whitespace passwords are unaffected.
+		if(!sendFmtData("PASS :%s",encodeText(pServer->m_szPass).data()))
 		{
 			// disconnected in the meantime
 			return;
 		}
 	}
+	
 	if(!sendFmtData("NICK %s",szNick.data()))
 	{
 		// disconnected :(
 		return;
 	}
 
+	if(!sendFmtData("USER %s 0 %s :%s",szUser.data(),
+			pServer->m_szHostname.utf8().data(),szReal.data()))
+	{
+		// disconnected in the meantime!
+		return;
+	}
+
+
 	// permanent info in the user database
 	m_pConsole->notifyListView()->join(m_pUserInfo->nickName(),"*","*");
 
@@ -948,7 +974,7 @@ void KviIrcConnection::loginToIrcServer()
 	{
 		if(_OUTPUT_VERBOSE)
 			m_pConsole->output(KVI_OUT_VERBOSE,__tr2qs("Executing scheduled network specific \"on connect\" commands"));
-		g_pUserParser->parseCommandBuffer(tmp.ptr(),m_pConsole,0);
+		KviKvsScript::run(tmp.ptr(),m_pConsole);
 	}
 
 	tmp = pServer->onConnectCommand();
@@ -957,7 +983,7 @@ void KviIrcConnection::loginToIrcServer()
 	{
 		if(_OUTPUT_VERBOSE)
 			m_pConsole->output(KVI_OUT_VERBOSE,__tr2qs("Executing scheduled server specific \"on connect\" commands"));
-		g_pUserParser->parseCommandBuffer(tmp.ptr(),m_pConsole,0);
+		KviKvsScript::run(tmp.ptr(),m_pConsole);
 	}
 
 	// and wait for the server to agree...
@@ -972,7 +998,7 @@ void KviIrcConnection::nickChange(const QString &szNewNick)
 	m_pConsole->updateCaption();
 	m_pFrame->childConnectionNickNameChange(this);
 	emit nickNameChanged();
-	g_pApp->addRecentNickname(szNewNick.latin1());
+	g_pApp->addRecentNickname(szNewNick);
 }
 
 bool KviIrcConnection::changeUserMode(char mode,bool bSet)
@@ -1001,15 +1027,15 @@ void KviIrcConnection::loginComplete(const QString &szNickName)
 	if(szNickName != m_pUserInfo->nickName())
 	{
 		m_pConsole->output(KVI_OUT_SYSTEMMESSAGE,__tr2qs("The server refused the suggested nickname (%s) and named you %s instead"),
-			m_pUserInfo->nickName().latin1(),szNickName.latin1());
+			m_pUserInfo->nickName().utf8().data(),szNickName.utf8().data());
 		m_pConsole->notifyListView()->nickChange(m_pUserInfo->nickName(),szNickName);
 		m_pUserInfo->setNickName(szNickName);
 	}
 
-	g_pApp->addRecentNickname(szNickName.latin1());
+	g_pApp->addRecentNickname(szNickName);
 	
 	bool bHaltOutput = false;
-	bHaltOutput = TRIGGER_EVENT_RETVALUE(KviEvent_OnIrc,m_pConsole);
+	bHaltOutput = KVS_TRIGGER_EVENT_0_HALTED(KviEvent_OnIrc,m_pConsole);
 	
 	if(!bHaltOutput)
 		m_pConsole->outputNoFmt(KVI_OUT_IRC,__tr2qs("Login operations complete, happy ircing!"));
@@ -1023,7 +1049,7 @@ void KviIrcConnection::loginComplete(const QString &szNickName)
 	{
 		if(_OUTPUT_VERBOSE)
 			m_pConsole->output(KVI_OUT_VERBOSE,__tr2qs("Executing scheduled network specific \"on login\" commands"));
-		g_pUserParser->parseCommandBuffer(tmp.ptr(),m_pConsole,0);
+		KviKvsScript::run(tmp.ptr(),m_pConsole);
 	}
 
 	tmp = target()->server()->onLoginCommand();
@@ -1032,7 +1058,7 @@ void KviIrcConnection::loginComplete(const QString &szNickName)
 	{
 		if(_OUTPUT_VERBOSE)
 			m_pConsole->output(KVI_OUT_VERBOSE,__tr2qs("Executing scheduled server specific \"on login\" commands"));
-		g_pUserParser->parseCommandBuffer(tmp.ptr(),m_pConsole,0);
+		KviKvsScript::run(tmp.ptr(),m_pConsole);
 	}
 
 	// Set the configured umode
@@ -1044,7 +1070,7 @@ void KviIrcConnection::loginComplete(const QString &szNickName)
 	{
 		if(_OUTPUT_VERBOSE)
 			m_pConsole->output(KVI_OUT_VERBOSE,__tr2qs("Setting configured user mode"));
-		sendFmtData("MODE %s +%s",m_pUserInfo->nickName().latin1(),modeStr.ptr());
+		sendFmtData("MODE %s +%s",encodeText(QString(m_pUserInfo->nickName())).data(),modeStr.ptr());
 	}
 
 	delayedStartNotifyList();
@@ -1055,18 +1081,40 @@ void KviIrcConnection::loginComplete(const QString &szNickName)
 
 
 	// join saved channels
+	QString szChannels,szProtectedChannels,szPasswords,szCurPass;
+	
+	if(!(m_pStateData->commandToExecAfterConnect().isEmpty()))
+	{
+		KviStr tmp = m_pStateData->commandToExecAfterConnect();
+		KviKvsScript::run(tmp.ptr(),m_pConsole);
+	}
+	
 	if(target()->network()->autoJoinChannelList())
 	{
 		if(_OUTPUT_VERBOSE)
 			m_pConsole->output(KVI_OUT_VERBOSE,__tr2qs("Auto-joining network specific channels"));
 
 		QStringList * l = target()->network()->autoJoinChannelList();
-		for(QStringList::Iterator chanIt = l->begin();chanIt != l->end();++chanIt)
+		if(l->count()!=0)
 		{
-			KviStr tmp = "join ";
-			tmp += *chanIt;
-			g_pUserParser->parseCommandBuffer(tmp.ptr(),m_pConsole,0);
-		}
+			for ( QStringList::Iterator it = l->begin(); it != l->end(); ++it ) {
+				
+				szCurPass=(*it).section(':',1);
+				if(szCurPass.isEmpty())
+				{
+					if(!szChannels.isEmpty())
+						szChannels.append(",");
+					szChannels.append((*it).section(':',0,0));
+				} else {
+					if(!szProtectedChannels.isEmpty())
+						szProtectedChannels.append(",");
+					szProtectedChannels.append((*it).section(':',0,0));
+					if(!szPasswords.isEmpty())
+						szPasswords.append(",");
+					szPasswords.append(szCurPass);
+				}
+			}
+		}	
 	}
 
 	if(server()->autoJoinChannelList())
@@ -1075,21 +1123,39 @@ void KviIrcConnection::loginComplete(const QString &szNickName)
 			m_pConsole->output(KVI_OUT_VERBOSE,__tr2qs("Auto-joining server specific channels"));
 
 		QStringList * l = server()->autoJoinChannelList();
-		for(QStringList::Iterator chanIt = l->begin();chanIt != l->end();++chanIt)
+		if(l->count()!=0)
 		{
-			KviStr tmp = "join ";
-			tmp += *chanIt;
-			g_pUserParser->parseCommandBuffer(tmp.ptr(),m_pConsole,0);
+			for ( QStringList::Iterator it = l->begin(); it != l->end(); ++it ) {
+				szCurPass=(*it).section(':',1);
+				if(szCurPass.isEmpty())
+				{
+					if(!szChannels.isEmpty())
+						szChannels.append(",");
+					szChannels.append((*it).section(':',0,0));
+				} else {
+					if(!szProtectedChannels.isEmpty())
+						szProtectedChannels.append(",");
+					szProtectedChannels.append((*it).section(':',0,0));
+					if(!szPasswords.isEmpty())
+						szPasswords.append(",");
+					szPasswords.append(szCurPass);
+				}
+			}
 		}
 	}
-
-
-	if(!(m_pStateData->commandToExecAfterConnect().isEmpty()))
+	
+	QString szCommand;
+	if( (!szChannels.isEmpty()) || (!szProtectedChannels.isEmpty()) )
 	{
-		KviStr tmp = m_pStateData->commandToExecAfterConnect();
-		g_pUserParser->parseCommandBuffer(tmp.ptr(),m_pConsole,0);
+		szCommand.append(szProtectedChannels);
+		if(!szProtectedChannels.isEmpty() && !szChannels.isEmpty())
+			szCommand.append(',');
+		szCommand.append(szChannels);
+		szCommand.append(" ");
+		szCommand.append(szPasswords);
+		
+		sendFmtData("JOIN %s",encodeText(szCommand).data());
 	}
-
 	// minimize after connect
 	if(KVI_OPTION_BOOL(KviOption_boolMinimizeConsoleAfterConnect))
 		m_pConsole->minimize();
@@ -1111,3 +1177,50 @@ void KviIrcConnection::incomingMessage(const char * message)
 	// and pass it to the server parser for processing
 	g_pServerParser->parseMessage(message,this);
 }
+
+void KviIrcConnection::heartbeat(kvi_time_t tNow)
+{
+	if(m_eState == Connected)
+	{
+		if(!KVI_OPTION_BOOL(KviOption_boolDisableAwayListUpdates))
+		{
+			// update the channel WHO lists (fixes users away state)
+			// first of all, we send our request not more often than every 50 secs
+			if((tNow - stateData()->lastSentChannelWhoRequest()) > 50)
+			{
+				// we also make sure that the last sent request is older than
+				// the last received reply
+				if(stateData()->lastSentChannelWhoRequest() <= stateData()->lastReceivedChannelWhoReply())
+				{
+					// find the channel that has the older list now
+					kvi_time_t tOldest = tNow;
+					KviChannel * pOldest = 0;
+					for(KviChannel * pChan = m_pChannelList->first();pChan;pChan = m_pChannelList->next())
+					{
+						if(pChan->lastReceivedWhoReply() < tOldest)
+						{
+							pOldest = pChan;
+							tOldest = pChan->lastReceivedWhoReply();
+						}
+					}
+					// if the oldest chan who list is older than 150 secs, update it
+					if((tNow - tOldest) > 150)
+					{
+						// ok, sent the request for this channel
+						stateData()->setLastSentChannelWhoRequest(tNow);
+						QString szChanName = encodeText(pOldest->name());
+						if(_OUTPUT_PARANOIC)
+							console()->output(KVI_OUT_VERBOSE,__tr2qs("Updating away state for channel %Q"),&szChanName);
+						if(lagMeter())
+						{
+							KviStr tmp(KviStr::Format,"WHO %s",pOldest->name());
+							lagMeter()->lagCheckRegister(tmp.ptr(),70);
+						}
+						pOldest->setSentSyncWhoRequest();
+						if(!sendFmtData("WHO %s",encodeText(QString(pOldest->name())).data()))return;
+					}
+				}
+			}
+		}
+	}
+}
