@@ -30,6 +30,9 @@ KviIrcConnectionStateData::KviIrcConnectionStateData()
 {
 	m_bSentQuit = false;
 	m_uLoginNickIndex = 0;
+	m_bSimulateUnexpectedDisconnect = false;
+	m_tLastReceivedChannelWhoReply = kvi_unixTime();
+	m_tLastSentChannelWhoRequest = m_tLastReceivedChannelWhoReply;
 }
 
 KviIrcConnectionStateData::~KviIrcConnectionStateData()
