@@ -30,7 +30,6 @@
 #include "kvi_query.h"
 #include "kvi_frame.h"
 #include "kvi_debug.h"
-#include "kvi_uparser.h"
 #include "kvi_sparser.h"
 #include "kvi_ircconnection.h"
 #include "kvi_ircconnectiontarget.h"
@@ -45,9 +44,11 @@
 #include "kvi_locale.h"
 #include "kvi_ircdatastreammonitor.h"
 #include "kvi_error.h"
-#include "kvi_event.h"
 #include "kvi_thread.h" // for KviThread::msleep()
 #include "kvi_app.h"
+#include "kvi_kvs_eventtriggers.h"
+#include "kvi_kvs_script.h"
+#include "kvi_netutils.h"
 
 #define __KVI_DEBUG__
 #include "kvi_debug.h"
@@ -83,15 +84,20 @@ KviIrcContext::KviIrcContext(KviConsole * pConsole)
 	m_eState = Idle;
 
 	m_pAsynchronousConnectionData = 0;
+	m_pSavedAsynchronousConnectionData = 0;
 	m_uConnectAttemptCount = 0;
 	m_pMonitorList = 0;
 	m_pReconnectTimer = 0;
 
 	m_uConnectAttemptCount = 1;
+	
+	m_iHeartbeatTimerId = startTimer(5000);
 }
 
 KviIrcContext::~KviIrcContext()
 {
+	killTimer(m_iHeartbeatTimerId);
+
 	while(m_pMonitorList)
 	{
 		KviIrcDataStreamMonitor * m = m_pMonitorList->first();
@@ -113,6 +119,7 @@ KviIrcContext::~KviIrcContext()
 	
 	destroyConnection();
 	if(m_pAsynchronousConnectionData)delete m_pAsynchronousConnectionData;
+	if(m_pSavedAsynchronousConnectionData)delete m_pSavedAsynchronousConnectionData;
 }
 
 void KviIrcContext::unhighlightAllWindows()
@@ -291,13 +298,13 @@ bool KviIrcContext::unregisterDeadQuery(KviQuery * q)
 void KviIrcContext::createLinksWindow()
 {
 	if(m_pLinksWindow)return;
-	g_pUserParser->parseCommandBuffer("links.open",m_pConsole);
+	KviKvsScript::run("links.open",m_pConsole);
 }
 
 void KviIrcContext::createListWindow()
 {
 	if(m_pListWindow)return;
-	g_pUserParser->parseCommandBuffer("list.open",m_pConsole);
+	KviKvsScript::run("list.open",m_pConsole);
 }
 
 void KviIrcContext::destroyConnection()
@@ -386,12 +393,57 @@ void KviIrcContext::connectToCurrentServer()
 
 	m_pConsole->outputNoFmt(KVI_OUT_SYSTEMMESSAGE," "); // spacer
 
-	bool bGotServer = false;
+	if(!m_pAsynchronousConnectionData)
+	{
+		// No connection target specified.
+		// If we have a saved target, reuse it
+
+		if(m_pSavedAsynchronousConnectionData)
+		{
+			m_pAsynchronousConnectionData = m_pSavedAsynchronousConnectionData;
+			m_pSavedAsynchronousConnectionData = 0;
+		}
+	}
 
 	if(m_pAsynchronousConnectionData)
 	{
+		// we have a specified connection target (either from outside or saved)
+		if(m_pAsynchronousConnectionData->szServer.isEmpty())
+		{
+			// an empty server might mean "reuse the last server in context"
+			if(m_pAsynchronousConnectionData->bUseLastServerInContext)
+			{
+				if(m_pSavedAsynchronousConnectionData)
+				{
+					// reuse the saved connection data
+					// the server for sure
+					m_pAsynchronousConnectionData->szServer = m_pSavedAsynchronousConnectionData->szServer;
+					m_pAsynchronousConnectionData->uPort = m_pSavedAsynchronousConnectionData->uPort;
+					m_pAsynchronousConnectionData->bPortIsOk = true;
+					m_pAsynchronousConnectionData->bUseIpV6 = m_pSavedAsynchronousConnectionData->bUseIpV6;
+					m_pAsynchronousConnectionData->bUseSSL = m_pSavedAsynchronousConnectionData->bUseSSL;
+					// and the other info, only if not overridden by the user
+					if(m_pAsynchronousConnectionData->szBindAddress.isEmpty())
+						m_pAsynchronousConnectionData->szBindAddress = m_pSavedAsynchronousConnectionData->szBindAddress;
+					if(m_pAsynchronousConnectionData->szCommandToExecAfterConnect.isEmpty())
+						m_pAsynchronousConnectionData->szCommandToExecAfterConnect = m_pSavedAsynchronousConnectionData->szCommandToExecAfterConnect;
+					if(m_pAsynchronousConnectionData->szLinkFilter.isEmpty())
+						m_pAsynchronousConnectionData->szLinkFilter = m_pSavedAsynchronousConnectionData->szLinkFilter;
+					if(m_pAsynchronousConnectionData->szPass.isEmpty())
+						m_pAsynchronousConnectionData->szPass = m_pSavedAsynchronousConnectionData->szPass;
+					if(m_pAsynchronousConnectionData->szNick.isEmpty())
+						m_pAsynchronousConnectionData->szNick = m_pSavedAsynchronousConnectionData->szNick;
+					if(m_pAsynchronousConnectionData->szInitUMode.isEmpty())
+						m_pAsynchronousConnectionData->szInitUMode = m_pSavedAsynchronousConnectionData->szInitUMode;
+				} else
+					m_pConsole->outputNoFmt(KVI_OUT_SYSTEMWARNING,__tr2qs("This is the first connection in this IRC context: using the global server setting"));
+			} // else it just means "do connect" to the globally selected irc server in the options dialog
+		}
+	
 		if(!(m_pAsynchronousConnectionData->szServer.isEmpty()))
 		{
+			// ok , have a server to look for in the db
+			// FIXME: this is a bit ugly... could it be managed in some completly different and nicer way ?
 			KviIrcServerDefinition d;
 			d.szServer = m_pAsynchronousConnectionData->szServer;
 			d.bPortIsValid = m_pAsynchronousConnectionData->bPortIsOk;
@@ -402,24 +454,44 @@ void KviIrcContext::connectToCurrentServer()
 			d.szPass = m_pAsynchronousConnectionData->szPass;
 			d.szNick = m_pAsynchronousConnectionData->szNick;
 			d.szInitUMode = m_pAsynchronousConnectionData->szInitUMode;
-			g_pIrcServerDataBase->makeCurrentServer(&d);
-			bGotServer = true;
-		}
+			QString szError;
+			if(!g_pIrcServerDataBase->makeCurrentServer(&d,szError))
+			{
+				m_pConsole->outputNoFmt(KVI_OUT_SYSTEMERROR,szError);
+				destroyAsynchronousConnectionData();
+				return;
+			}
+		} // else we just connect to the globally selected irc server in the options dialog
 	}
 
 	KviIrcServerDataBaseRecord * rec = g_pIrcServerDataBase->currentRecord();
-	KviIrcNetwork * net = rec ? rec->network() : 0;
-	KviIrcServer  * srv = net ? rec->currentServer() : 0;
-	KviProxy      * prx = 0;
+	
+	KviIrcNetwork * net;
+	KviIrcServer  * srv;
+	
+	net = rec ? rec->network() : 0;
+	srv = net ? rec->currentServer() : 0;
 
+	KviProxy      * prx = 0;
+	
 	if(!srv)
 	{
 		m_pConsole->outputNoFmt(KVI_OUT_SYSTEMERROR,__tr2qs("No servers available. Check the options dialog or use the /SERVER command"));
 		destroyAsynchronousConnectionData();
 		return;
 	}
-
-	if(KVI_OPTION_BOOL(KviOption_boolUseProxyHost))
+	
+	if(!net)
+	{
+		// BUG
+		m_pConsole->outputNoFmt(KVI_OUT_SYSTEMERROR,__tr2qs("Ooops.. you've hit a bug in the servers database... I have found a server but not a network..."));
+		destroyAsynchronousConnectionData();
+		return;
+	}
+	
+	prx = srv->proxyServer(g_pProxyDataBase);
+	
+	if(!prx && (srv->proxy()!=-1) && KVI_OPTION_BOOL(KviOption_boolUseProxyHost))
 	{
 		prx = g_pProxyDataBase->currentProxy();
 		if(!prx)
@@ -458,6 +530,19 @@ void KviIrcContext::connectToCurrentServer()
 
 	m_pConsole->connectionAttached();
 
+	// save stuff for later
+
+	// FIXME: this management of "next" connection should be reviewed a bit anyway
+	if(m_pSavedAsynchronousConnectionData)delete m_pSavedAsynchronousConnectionData;
+	m_pSavedAsynchronousConnectionData = new KviAsynchronousConnectionData();
+	m_pSavedAsynchronousConnectionData->szServer = srv->m_szHostname;
+	m_pSavedAsynchronousConnectionData->uPort = srv->port();
+	m_pSavedAsynchronousConnectionData->bPortIsOk = true;
+	m_pSavedAsynchronousConnectionData->bUseIpV6 = srv->isIpV6();
+	m_pSavedAsynchronousConnectionData->bUseSSL = srv->useSSL();
+	m_pSavedAsynchronousConnectionData->szPass = srv->password();
+	m_pSavedAsynchronousConnectionData->szInitUMode = srv->m_szInitUMode;
+
 	// this never fails!
 	m_pConnection->start();
 }
@@ -468,7 +553,7 @@ void KviIrcContext::connectionFailed(int iError)
 
 	m_pConsole->output(KVI_OUT_SYSTEMERROR,
 		__tr2qs("Connection attempt failed [%s]"),
-		m_pConnection->target()->server()->m_szHostname.latin1());
+		m_pConnection->target()->server()->m_szHostname.utf8().data());
 
 	// if the connection has been aborted by the user then just go idle
 	if(iError == KviError_operationAborted)
@@ -487,14 +572,18 @@ void KviIrcContext::connectionFailed(int iError)
 				QString tmp;
 				KviQString::sprintf(tmp,__tr2qs("Will attempt to reconnect in %d seconds"),KVI_OPTION_UINT(KviOption_uintAutoReconnectDelay));
 				QString num;
-				KviQString::sprintf(num,__tr2qs("%d of %d"),
+				
+				if(!KVI_OPTION_UINT(KviOption_uintMaxAutoReconnectAttempts))
+				    KviQString::sprintf(num,__tr2qs("%d"),m_uConnectAttemptCount);
+				else
+				    KviQString::sprintf(num,__tr2qs("%d of %d"),
 						m_uConnectAttemptCount,KVI_OPTION_UINT(KviOption_uintMaxAutoReconnectAttempts));
 				tmp += " [" + num + "]";
 				m_pConsole->outputNoFmt(KVI_OUT_SYSTEMMESSAGE,tmp);
 			}
 	
 			KviIrcServer oldServer(*(connection()->server()));
-			KviStr oldNickname = connection()->userInfo()->nickName();
+			QString oldNickname = connection()->userInfo()->isAway() ? connection()->userInfo()->nickNameBeforeAway() : connection()->userInfo()->nickName();
 	
 			KviAsynchronousConnectionData * d = new KviAsynchronousConnectionData();
 			d->szServer = oldServer.m_szHostname;
@@ -548,29 +637,29 @@ void KviIrcContext::connectionEstabilished()
 
 	bool bStopOutput = false;
 
-	bStopOutput = TRIGGER_EVENT_RETVALUE(KviEvent_OnIrcConnectionEstabilished,m_pConsole);
+	setState(LoggingIn); // this must be set in order for $server and other functions to return the correct values
+
+	bStopOutput = KVS_TRIGGER_EVENT_0_HALTED(KviEvent_OnIrcConnectionEstabilished,m_pConsole);
 	
 	if(!bStopOutput)
 	{
 		m_pConsole->output(KVI_OUT_CONNECTION,__tr2qs("%Q established [%s (%s:%u)]"),
 			connection()->socket()->usingSSL() ? &(__tr2qs("Secure connection")) : &(__tr2qs("Connection")),
-			connection()->server()->m_szHostname.latin1(),
-			connection()->server()->m_szIp.latin1(),
+			connection()->server()->m_szHostname.utf8().data(),
+			connection()->server()->m_szIp.utf8().data(),
 			connection()->server()->m_uPort);
 	}
 
 	// Add to recent server list (build the url of type irc[6]://<server>:<port>
 	KviStr tmp(KviStr::Format,"irc%s//%s:%u",
 			connection()->server()->isIpV6() ? "6:" : ":",
-			connection()->server()->m_szHostname.latin1(),connection()->server()->m_uPort);
+			connection()->server()->m_szHostname.utf8().data(),connection()->server()->m_uPort);
 
 	g_pApp->addRecentServer(tmp.ptr());
 
 	// save the last server this console used
 	//if(m_pLastIrcServer)delete m_pLastIrcServer;
 	//m_pLastIrcServer = new KviIrcServer(*(connection()->server()));
-	
-	setState(LoggingIn);
 }
 
 void KviIrcContext::connectionTerminated()
@@ -578,50 +667,93 @@ void KviIrcContext::connectionTerminated()
 	if(!m_pConnection)return; // this may happen in the destructor!
 
 	KviIrcServer oldServer(*(connection()->server()));
-	KviStr oldNickname = connection()->userInfo()->nickName();
+	QString oldNickname = connection()->userInfo()->isAway() ? connection()->userInfo()->nickNameBeforeAway() : connection()->userInfo()->nickName();
 
-	bool bUnexpectedDisconnect = (!(connection()->stateData()->sentQuit())) && (m_eState == KviIrcContext::Connected);
+	// we consider it unexpected when we haven't sent a QUIT message and we're connected
+	// or alternatively when a simulation of such a termination is requested (this is used to keep the queries open etc..)
+	bool bUnexpectedDisconnect = (!(connection()->stateData()->sentQuit())) && (m_eState == KviIrcContext::Connected) ||
+									connection()->stateData()->simulateUnexpectedDisconnect();
 
 	KviStr szReconnectCmd;
 
-	if(bUnexpectedDisconnect && KVI_OPTION_BOOL(KviOption_boolAutoReconnectOnUnexpectedDisconnect))
+	if(bUnexpectedDisconnect)
 	{
-		if(KVI_OPTION_BOOL(KviOption_boolRejoinChannelsAfterReconnect))
+		if(KVI_OPTION_BOOL(KviOption_boolAutoReconnectOnUnexpectedDisconnect))
 		{
-			// FIXME: THIS SHOULD BE A KviIrcConnection FUNCTION
-			for(KviChannel * c = connection()->channelList()->first();c;c = connection()->channelList()->next())
+			if(KVI_OPTION_BOOL(KviOption_boolRejoinChannelsAfterReconnect))
 			{
-				// FIXME: password protected channels should have the pass appended
-				// FIXME: should join groups of 5 channels per message
-				if(szReconnectCmd.isEmpty())szReconnectCmd.append("join ");
-				else szReconnectCmd.append(",");
-				szReconnectCmd.append(c->windowName());
+				// FIXME: THIS SHOULD BE A KviIrcConnection FUNCTION
+				int idx = 0;
+				KviChannel * c;
+				// first only chans without key, in groups of 4
+				for(c = connection()->channelList()->first();c;c = connection()->channelList()->next())
+				{
+					if(c->channelKey().isEmpty())
+					{
+						idx++;
+						if(idx == 1)
+							szReconnectCmd.append("join ");
+						else
+							szReconnectCmd.append(",");
+						QString szChanName = c->windowName();
+						szChanName.replace("(","\\(");
+						szChanName.replace("\\","\\\\");
+						szReconnectCmd.append(szChanName);
+						if(idx == 4)
+						{
+							szReconnectCmd.append(";");
+							idx = 0; // 4 channels per join (this should be safe)
+						}
+					}
+				}
+				if((!szReconnectCmd.isEmpty()) && (idx != 0))szReconnectCmd.append(";");
+				// now channels with key
+				for(c = connection()->channelList()->first();c;c = connection()->channelList()->next())
+				{
+					if(!c->channelKey().isEmpty())
+					{
+						szReconnectCmd.append("join ");
+						szReconnectCmd.append(c->windowName());
+						szReconnectCmd.append(" ");
+						szReconnectCmd.append(c->channelKey());
+						szReconnectCmd.append(";");
+					}
+				}
+			}
+	
+			KviStr szQuery;
+	
+			if(KVI_OPTION_BOOL(KviOption_boolReopenQueriesAfterReconnect))
+			{
+				// FIXME: THIS SHOULD BE A KviIrcConnection FUNCTION
+				for(KviQuery * q = connection()->queryList()->first();q;q = connection()->queryList()->next())
+				{
+					if(szQuery.isEmpty())szQuery.append("query ");
+					else szQuery.append(",");
+					QString szQueryName = q->target();
+					szQueryName.replace("(","\\(");
+					szQueryName.replace("\\","\\\\");
+					szQuery.append(szQueryName);
+				}
+			}
+	
+			if(szQuery.hasData())
+			{
+				if(szReconnectCmd.hasData())szReconnectCmd.append(';');
+				szReconnectCmd.append(szQuery);
 			}
 		}
 
-		KviStr szQuery;
-
-		if(KVI_OPTION_BOOL(KviOption_boolReopenQueriesAfterReconnect))
-		{
-			// FIXME: THIS SHOULD BE A KviIrcConnection FUNCTION
-			for(KviQuery * q = connection()->queryList()->first();q;q = connection()->queryList()->next())
-			{
-				if(szQuery.isEmpty())szQuery.append("query ");
-				else szQuery.append(",");
-				szQuery.append(q->target());
-			}
-		}
-
-		if(szQuery.hasData())
-		{
-			if(szReconnectCmd.hasData())szReconnectCmd.append(';');
-			szReconnectCmd.append(szQuery);
-		}
-
-		if(KVI_OPTION_BOOL(KviOption_boolKeepChannelsOpenOnUnexpectedDisconnect))
+		if(KVI_OPTION_BOOL(KviOption_boolKeepChannelsOpenOnUnexpectedDisconnect) || KVI_OPTION_BOOL(KviOption_boolKeepChannelsOpenOnDisconnect))
 			connection()->keepChannelsOpenAfterDisconnect();
 
-		if(KVI_OPTION_BOOL(KviOption_boolKeepQueriesOpenOnUnexpectedDisconnect))
+		if(KVI_OPTION_BOOL(KviOption_boolKeepQueriesOpenOnUnexpectedDisconnect) || KVI_OPTION_BOOL(KviOption_boolKeepQueriesOpenOnDisconnect))
+			connection()->keepQueriesOpenAfterDisconnect();
+	} else {
+		if(KVI_OPTION_BOOL(KviOption_boolKeepChannelsOpenOnDisconnect))
+			connection()->keepChannelsOpenAfterDisconnect();
+
+		if(KVI_OPTION_BOOL(KviOption_boolKeepQueriesOpenOnDisconnect))
 			connection()->keepQueriesOpenAfterDisconnect();
 	}
 	
@@ -629,13 +761,13 @@ void KviIrcContext::connectionTerminated()
 
 	bool bStopOutput = false;
 
-	bStopOutput = TRIGGER_EVENT_RETVALUE(KviEvent_OnIrcConnectionTerminated,m_pConsole);
+	bStopOutput = KVS_TRIGGER_EVENT_0_HALTED(KviEvent_OnIrcConnectionTerminated,m_pConsole);
 
 	if(!bStopOutput)
 	{
 		m_pConsole->output(KVI_OUT_CONNECTION,__tr2qs("Connection terminated [%s (%s:%u)]"),
-				oldServer.hostName().latin1(),
-				oldServer.ip().latin1(),
+				oldServer.hostName().utf8().data(),
+				oldServer.ip().utf8().data(),
 				oldServer.port());
 	}
 
@@ -686,10 +818,12 @@ void KviIrcContext::asynchronousConnect()
 }
 
 
-void KviIrcContext::terminateConnectionRequest(bool bForce,const QString &szQuitMsg)
+void KviIrcContext::terminateConnectionRequest(bool bForce,const QString &szQuitMsg,bool bSimulateUnexpectedDisconnect)
 {
 	if(!connection())return; // hm ?
 
+	if(bSimulateUnexpectedDisconnect)connection()->stateData()->setSimulateUnexpectedDisconnect();
+
 	switch(m_eState)
 	{
 		case Connected:
@@ -698,14 +832,18 @@ void KviIrcContext::terminateConnectionRequest(bool bForce,const QString &szQuit
 			bool bWasSentQuit = true;
 			if(!connection()->stateData()->sentQuit())
 			{
-				TRIGGER_EVENT(KviEvent_OnDisconnectRequest,m_pConsole);
-				KviStr szQuit = szQuitMsg;
+				KVS_TRIGGER_EVENT_0(KviEvent_OnDisconnectRequest,m_pConsole);
+				QString szQuit = szQuitMsg;
 				if(szQuit.isEmpty())szQuit = KVI_OPTION_STRING(KviOption_stringQuitMessage);
-				szQuit.replaceAll(";","\\;");
-				szQuit.replaceAll("\n"," ");
-				KviStr buffer;
-				if(!g_pUserParser->substituteIdentifiers(szQuit.ptr(),console(),buffer))buffer = szQuit;
-				QCString dat = console()->encodeText(QString::fromUtf8(buffer.ptr()));
+				szQuit.replace(";","\\;");
+				szQuit.replace("\n"," ");
+				QString buffer;
+				KviKvsVariant ret;
+				if(KviKvsScript::evaluate(szQuit,console(),0,&ret))
+					ret.asString(buffer);
+				else
+					buffer = szQuit;
+				QCString dat = console()->encodeText(buffer);
 				bWasSentQuit = false;
 				connection()->stateData()->setSentQuit();
 				connection()->sendFmtData("QUIT :%s",dat.data() ? dat.data() : ""); // here theoretically we COULD get disconnected
@@ -740,3 +878,19 @@ void KviIrcContext::terminateConnectionRequest(bool bForce,const QString &szQuit
 	}
 }
 
+void KviIrcContext::timerEvent(QTimerEvent *e)
+{
+	if(e->timerId() != m_iHeartbeatTimerId)
+	{
+		QObject::timerEvent(e);
+		return;
+	}
+
+	// our heartbeat
+	
+	kvi_time_t tNow = kvi_unixTime();
+
+	if(m_pConnection)
+		m_pConnection->heartbeat(tNow);
+}
+
