@@ -183,12 +183,19 @@ void KviIrcSocket::outputSSLError(const QString &szMsg)
 
 void KviIrcSocket::outputProxyMessage(const QString &szMsg)
 {
-	m_pConsole->output(KVI_OUT_SOCKETMESSAGE,__tr2qs("[PROXY]: %Q"),&szMsg);
+	QStringList list=QStringList::split("\n",szMsg);
+	for ( QStringList::Iterator it = list.begin(); it != list.end(); ++it ) {
+		m_pConsole->output(KVI_OUT_SOCKETMESSAGE,__tr2qs("[PROXY]: %Q"),&((*it).stripWhiteSpace()));
+	}
 }
 
 void KviIrcSocket::outputProxyError(const QString &szMsg)
 {
-	m_pConsole->output(KVI_OUT_SOCKETERROR,__tr2qs("[PROXY ERROR]: %Q"),&szMsg);
+	QStringList list=QStringList::split("\n",szMsg);
+	for ( QStringList::Iterator it = list.begin(); it != list.end(); ++it ) {
+		m_pConsole->output(KVI_OUT_SOCKETERROR,__tr2qs("[PROXY ERROR]: %Q"),&((*it).stripWhiteSpace()));
+	}
+	
 }
 
 void KviIrcSocket::outputSocketMessage(const QString &szMsg)
@@ -220,7 +227,10 @@ void KviIrcSocket::raiseError(int iError)
 {
 	m_iLastError = iError;
 	//m_pConsole->socketError(iError);
-	outputSocketError(KviError::getDescription(iError));
+	if( (m_iLastError==KviError_remoteEndClosedConnection) && ( m_state == ProxyHttpError) )
+		outputSocketMessage(KviError::getDescription(iError));
+	else
+		outputSocketError(KviError::getDescription(iError));
 }
 
 int KviIrcSocket::startConnection(KviIrcServer *srv,KviProxy * prx,const char * bindAddress)
@@ -244,7 +254,9 @@ int KviIrcSocket::startConnection(KviIrcServer *srv,KviProxy * prx,const char *
 	m_pIrcServer = new KviIrcServer(*srv);
 
 	bool bTargetIpV6 = false;
-
+	bool bNeedServerIp = !prx;
+	if(prx) bNeedServerIp = (prx->protocol() != KviProxy::Http );
+	
 	// We're going to check the addresses now
 
 	// check the proxy stuff...
@@ -269,23 +281,26 @@ int KviIrcSocket::startConnection(KviIrcServer *srv,KviProxy * prx,const char *
 			if(!kvi_isValidStringIp(m_pProxy->ip()))return KviError_invalidProxyAddress;
 		}
 	}
-
-	// check the irc host ip
-#ifdef COMPILE_IPV6_SUPPORT
-	if(m_pIrcServer->isIpV6())
+	
+	if(bNeedServerIp)
 	{
-		// We have an IpV6 server host (Interesting if proxy is IpV4)
-		if(!kvi_isValidStringIp_V6(m_pIrcServer->ip().latin1()))return KviError_invalidIpAddress;
-		if(!m_pProxy)bTargetIpV6 = true; // otherwise the proxy rules
-	} else {
-#endif
-		// We have an IpV4 server host
-		if(!kvi_isValidStringIp(m_pIrcServer->ip().latin1()))return KviError_invalidIpAddress;
+		// check the irc host ip
 #ifdef COMPILE_IPV6_SUPPORT
-	}
+		if(m_pIrcServer->isIpV6())
+		{
+			// We have an IpV6 server host (Interesting if proxy is IpV4)
+			if( !KviNetUtils::isValidStringIp_V6(m_pIrcServer->ip()) )return KviError_invalidIpAddress;
+			if(!m_pProxy)bTargetIpV6 = true; // otherwise the proxy rules
+		} else {
 #endif
+			// We have an IpV4 server host
+			if(!KviNetUtils::isValidStringIp(m_pIrcServer->ip())) return KviError_invalidIpAddress;
+#ifdef COMPILE_IPV6_SUPPORT
+		}
+#endif
+	}
 
-	KviSockaddr sa(prx ? m_pProxy->ip() : m_pIrcServer->ip().latin1(),prx ? m_pProxy->port() : m_pIrcServer->port(),bTargetIpV6);
+	KviSockaddr sa(prx ? m_pProxy->ip() : m_pIrcServer->ip().utf8().data(),prx ? m_pProxy->port() : m_pIrcServer->port(),bTargetIpV6);
 
 	if(!sa.socketAddress())return KviError_invalidIpAddress;
 
@@ -453,6 +468,21 @@ void KviIrcSocket::connectedToProxy()
 	}
 }
 
+void KviIrcSocket::readHttpProxyErrorData(int)
+{
+	char buffer[256];
+	int readLength;
+	
+	readLength = kvi_socket_recv(m_sock,buffer,255);
+	if(readLength <= 0)
+	{
+		handleInvalidSocketRead(readLength);
+		return;
+	}
+	
+	outputProxyMessage(m_pConsole->decodeText(buffer));
+}
+
 void KviIrcSocket::connectedToIrcServer()
 {
 #ifdef COMPILE_SSL_SUPPORT
@@ -592,16 +622,26 @@ void KviIrcSocket::proxyLoginHttp()
 {
 	// Well..this is just plain and easy: connect to the proxy machine
 	// and say "CONNECT <irc.server>:<port> HTTP/<version>\n\n"
+	// if it requires auth than say Proxy-Authorization: Basic user:passwd
 	// Then expect a server reply header (2 newlines)
 	// HTTP 200 = Success
 	// HTTP Anything else = Failure
-
+	
 	if(_OUTPUT_VERBOSE)
 		outputProxyMessage(__tr2qs("Using HTTP protocol."));
 
 	setState(ProxyFinalHttp);
-	KviStr tmp(KviStr::Format,"CONNECT %s:%u HTTP/1.0\n\n",m_pIrcServer->hostName().latin1(),(unsigned int)(m_pIrcServer->port()));
-
+	KviStr tmp(KviStr::Format,"CONNECT %s:%u HTTP/1.0\r\n",m_pIrcServer->hostName().utf8().data(),(unsigned int)(m_pIrcServer->port()));
+	if(m_pProxy->hasUser())
+	{
+		KviStr auth(KviStr::Format,"%s:%s",m_pProxy->user(),m_pProxy->pass());
+		KviStr encoded;
+		encoded.bufferToBase64(auth.ptr(),auth.len());
+		tmp.append(KviStr::Format,"Proxy-Authorization: Basic %s\r\n\r\n",encoded.ptr());
+	} else {
+		tmp.append("\r\n");
+	}
+//	debug(tmp.ptr());
 	sendRawData(tmp.ptr(),tmp.len());
 }
 
@@ -1089,13 +1129,31 @@ void KviIrcSocket::proxyHandleHttpFinalReply(const char * buffer,int bufLen)
 		
 	}
 
-	QString msg = __tr2qs("Proxy said something about \"");
-	msg += tmp.ptr();
-	msg += __tr2qs("\" ...that looks like an error to me...");
-	outputProxyError(msg);
+	outputProxyError(__tr2qs("Proxy said something about: \n"));
+	outputProxyMessage(m_pConsole->decodeText(buffer));
+	
+	//Read HTTP error page and show it
+	
+	if(m_pWsn)
+	{
+		delete m_pWsn;
+		m_pWsn = 0;
+	}
 
-	raiseError(KviError_proxyHttpFailure);
-	reset();
+	if(m_pRsn)
+	{
+		delete m_pRsn;
+		m_pRsn = 0;
+	}
+
+	m_pRsn = new QSocketNotifier((int)m_sock,QSocketNotifier::Read);
+	QObject::connect(m_pRsn,SIGNAL(activated(int)),this,SLOT(readHttpProxyErrorData(int)));
+	m_pRsn->setEnabled(true);
+	
+	setState(ProxyHttpError);
+	
+//	raiseError(KviError_proxyHttpFailure);
+//	reset();
 }
 
 //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
@@ -1710,7 +1768,7 @@ handle_system_error:
 	//flushed completely ...
 }
 
-bool KviIrcSocket::getLocalHostIp(KviStr &szIp,bool bIpV6)
+bool KviIrcSocket::getLocalHostIp(QString &szIp,bool bIpV6)
 {
 	if(m_state != Connected)return false;
 
