@@ -24,7 +24,6 @@
 
 #define __KVIRC__
 
-#define _KVI_DEBUG_CHECK_RANGE_
 
 #include "kvi_debug.h"
 #include "kvi_notifylist.h"
@@ -40,12 +39,12 @@
 #include "kvi_sparser.h"
 #include "kvi_ircmask.h"
 #include "kvi_numeric.h"
-#include "kvi_event.h"
 #include "kvi_parameterlist.h"
 #include "kvi_ircconnection.h"
 #include "kvi_app.h"
 #include "kvi_qstring.h"
 #include "kvi_lagmeter.h"
+#include "kvi_kvs_eventtriggers.h"
 
 #include <qstringlist.h>
 
@@ -150,13 +149,16 @@ bool KviNotifyListManager::handleWatchReply(KviIrcMessage *)
 	return false;
 }
 
+// kvi_app.cpp
+extern KviRegisteredUserDataBase * g_pRegisteredUserDataBase;
+
 void KviNotifyListManager::notifyOnLine(const QString &nick,const QString &user,const QString &host,const QString &szReason,bool bJoin)
 {
 	if(bJoin)
 		m_pConsole->notifyListView()->join(nick,user,host);
 
 	KviWindow * out = KVI_OPTION_BOOL(KviOption_boolNotifyListChangesToActiveWindow) ? m_pConsole->activeWindow() : m_pConsole;
-	if(TRIGGER_EVENT_1PARAM_RETVALUE(KviEvent_OnNotifyOnLine,out,nick))return;
+	if(KVS_TRIGGER_EVENT_1_HALTED(KviEvent_OnNotifyOnLine,out,nick))return;
 
 	QString szWho;
 	QString szMsg;
@@ -165,11 +167,29 @@ void KviNotifyListManager::notifyOnLine(const QString &nick,const QString &user,
 		KviQString::sprintf(szWho,"\r!n\r%Q\r [%Q@\r!h\r%Q\r]",&nick,&user,&host);
 	else
 		KviQString::sprintf(szWho,"\r!n\r%Q\r",&nick);
-
+	
+	QDict<KviRegisteredUser> * d = g_pRegisteredUserDataBase->userDict();
+	QDictIterator<KviRegisteredUser> it(*d);
+	QString szNotify;
+	
+	while(KviRegisteredUser * u = it.current())
+	{
+		if(QStringList::split(",",u->getProperty("notify")).findIndex(nick)!=-1)
+		{
+			QString szComment=u->getProperty("comment");
+			if(!szComment.isEmpty())
+				KviQString::sprintf(szMsg,"%Q (%Q), Group \"%Q\" is on IRC as (%Q)",&(u->name()),&szComment,&(u->group()),&szWho);
+			else
+				KviQString::sprintf(szMsg,"%Q, Group \"%Q\" is on IRC as (%Q)",&(u->name()),&(u->group()),&szWho);
+			break;
+		}
+		++it;
+	}
 	QString szFmt = __tr2qs("%Q is on IRC");
-
-	KviQString::sprintf(szMsg,szFmt,&szWho);
-
+	
+	if(szMsg.isEmpty())
+		KviQString::sprintf(szMsg,szFmt,&szWho);
+	
 	if((!szReason.isEmpty()) && (_OUTPUT_VERBOSE))
 	{
 		szMsg += "(";
@@ -197,7 +217,7 @@ void KviNotifyListManager::notifyOnLine(const QString &nick,const QString &user,
 void KviNotifyListManager::notifyOffLine(const QString &nick,const QString &user,const QString &host,const QString &szReason)
 {
 	KviWindow * out = KVI_OPTION_BOOL(KviOption_boolNotifyListChangesToActiveWindow) ? m_pConsole->activeWindow() : m_pConsole;
-	if(!TRIGGER_EVENT_1PARAM_RETVALUE(KviEvent_OnNotifyOffLine,out,nick))
+	if(!KVS_TRIGGER_EVENT_1_HALTED(KviEvent_OnNotifyOffLine,out,nick))
 	{
 		QString szWho;
 			
@@ -207,7 +227,27 @@ void KviNotifyListManager::notifyOffLine(const QString &nick,const QString &user
 			KviQString::sprintf(szWho,"\r!n\r%Q\r",&nick);
 	
 		QString szMsg;
-		KviQString::sprintf(szMsg,__tr2qs("%Q has left IRC"),&szWho);
+		
+		QDict<KviRegisteredUser> * d = g_pRegisteredUserDataBase->userDict();
+		QDictIterator<KviRegisteredUser> it(*d);
+		QString szNotify;
+		
+		while(KviRegisteredUser * u = it.current())
+		{
+			if(QStringList::split(",",u->getProperty("notify")).findIndex(nick)!=-1)
+			{
+				QString szComment=u->getProperty("comment");
+				if(!szComment.isEmpty())
+					KviQString::sprintf(szMsg,"%Q (%Q), Group \"%Q\" has left IRC as (%Q)",&(u->name()),&szComment,&(u->group()),&szWho);
+				else
+					KviQString::sprintf(szMsg,"%Q, Group \"%Q\" has left IRC as (%Q)",&(u->name()),&(u->group()),&szWho);
+				break;
+			}
+			++it;
+		}
+		
+		if(szMsg.isEmpty())
+			KviQString::sprintf(szMsg,__tr2qs("%Q has left IRC"),&szWho);
 	
 		if((!szReason.isEmpty()) && (_OUTPUT_VERBOSE))
 		{
@@ -223,9 +263,6 @@ void KviNotifyListManager::notifyOffLine(const QString &nick,const QString &user
 }
 
 
-// kvi_app.cpp
-extern KviRegisteredUserDataBase * g_pRegisteredUserDataBase;
-
 
 //
 //  INTELLIGENT NOTIFY LIST MANAGER: NOTIFY PROCESS:
@@ -349,15 +386,17 @@ void KviIsOnNotifyListManager::buildRegUserDict()
 		if(u->getProperty("notify",notify))
 		{
 			notify.stripWhiteSpace();
-			if(!notify.isEmpty())
+			while(!notify.isEmpty())
 			{
 				int idx = notify.find(' ');
 				if(idx > 0)
 				{
 					QString single = notify.left(idx);
 					m_pRegUserDict->replace(single,new QString(u->name()));
+					notify.remove(0,idx+1);
 				} else {
 					m_pRegUserDict->replace(notify,new QString(u->name()));
+					notify = "";
 				}
 			}
 		}
@@ -1089,7 +1128,7 @@ void KviWatchNotifyListManager::start()
 					m_pConsole->output(KVI_OUT_SYSTEMMESSAGE,__tr2qs("Notify list: Adding watch entries for%Q"),&watchStr);
 				watchStr = "";
 			}
-			KviQString::sprintf(watchStr," +%Q",&nk);
+			KviQString::appendFormatted(watchStr," +%Q",&nk);
 		}
 		++it;
 	}
