@@ -63,6 +63,16 @@ DECLARE_OPTION_STRUCT(KviUIntOption,unsigned int)
 DECLARE_OPTION_STRUCT(KviMsgTypeOption,KviMsgType)
 DECLARE_OPTION_STRUCT(KviStringListOption,QStringList)
 
+#define KVI_COLOR_EXT_USER_OP 		50
+#define KVI_COLOR_EXT_USER_HALFOP	51
+#define KVI_COLOR_EXT_USER_ADMIN	52
+#define KVI_COLOR_EXT_USER_OWNER	53
+#define KVI_COLOR_EXT_USER_VOICE	54
+#define KVI_COLOR_EXT_USER_USEROP	55
+#define KVI_COLOR_EXT_USER_NORMAL	56
+#define KVI_COLOR_CUSTOM		255
+#define KVI_COLOR_OWN			254
+
 // option entry indexes
 
 #define KVI_RECT_OPTIONS_PREFIX "rect"
@@ -166,7 +176,7 @@ DECLARE_OPTION_STRUCT(KviStringListOption,QStringList)
 #define KviOption_boolIgnoreCtcpAvatar 82                            /* irc::ctcp::floodprotection */
 #define KviOption_boolCtcpRepliesToActiveWindow 83                   /* ctcp */
 #define KviOption_boolAutoAcceptDccCanvas 84
-//#define KviOption_boolEnableAutomaticNickServIdentification 85       /* ircengine::nickserv */
+#define KviOption_boolNotifyDccSendSuccessInNotifier 85              /* dcc::send */
 #define KviOption_boolKillBrokenTimers 86                            /* ircengine::uparser */
 #define KviOption_boolDccVoiceVolumeSliderControlsPCM 87             /* dcc::voice */
 #define KviOption_boolDccVoiceForceHalfDuplex 88                     /* dcc::voice */
@@ -261,9 +271,37 @@ DECLARE_OPTION_STRUCT(KviStringListOption,QStringList)
 #define KviOption_boolLimitDccSendSpeed 177                          /* dcc::file transfers */
 #define KviOption_boolLimitDccRecvSpeed 178                          /* dcc::file transfers */
 #define KviOption_boolIgnoreModuleVersions 179
+#define KviOption_boolDisableInputHistory 180                        /* interface::input */
+#define KviOption_boolUseSpecialColorForTimestamp 181                /* interface::output */
+#define KviOption_boolDisableAwayListUpdates 182                     /* irc::output */
+#define KviOption_boolDisableAvatars 183				 /* interface::userlist */ 
+#define KviOption_boolDisableUserListLabel 184		        	 /* interface::userlist */ 
+#define KviOption_boolDisablePopupIcons 185			     /* themes::general */ 
+#define KviOption_boolScriptErrorsToDebugWindow 186                  /* parser */
+#define KviOption_boolShowMinimizedDebugWindow 187                   /* parser */
+#define KviOption_boolShowExtendedInfoInQueryLabel 188               /* irc::query */
+#define KviOption_boolUseUserListColorsAsNickColors 189              /* irc::usermessages */
+#define KviOption_boolGzipLogs 190                                   /* ircengine::logging */
+#define KviOption_boolMinimizeInTray 191                                   /* interface */
+#define KviOption_boolDisableNotifierFlashing 192                                   /* interface::notifier */
+#define KviOption_boolCommandlineInUserFriendlyModeByDefault 193     /* interface::input */
+#define KviOption_boolEnableVisualEffects 194     /* interface */
+#define KviOption_boolDCCFileTransferReplaceOutgoingSpacesWithUnderscores 195
+#define KviOption_boolDoNotStretchAvatars 196
+#define KviOption_boolUseDifferentColorForOwnNick 197 /* interface */
+#define KviOption_boolCloseInTray 198                                   /* interface */
+#define KviOption_boolStartupMinimized 199                                   /* interface */
+#define KviOption_boolSortReceivedByDccFilesByNicks 200                                   /* dcc */
+#define KviOption_boolLogChannelHistory 201                                   /* irc */
+#define KviOption_boolClassicTaskBarSetMaximumButtonWidth 202
+#define KviOption_boolKeepChannelsOpenOnDisconnect 203 				/* connection */
+#define KviOption_boolKeepQueriesOpenOnDisconnect 204				/* connection */
+#define KviOption_boolHideWindowToolButtons 205			/* interface */
+#define KviOption_boolHideInputToolButtons 206			/* interface */
+#define KviOption_boolStripControlCodesInLogs 207			/* log */
+#define KviOption_boolDisableTrayIconFlashing 208
 
-#define KVI_NUM_BOOL_OPTIONS 180
-
+#define KVI_NUM_BOOL_OPTIONS 209
 
 
 
@@ -284,12 +322,12 @@ DECLARE_OPTION_STRUCT(KviStringListOption,QStringList)
 #define KviOption_stringExtendedPrivmsgPrefix 11                     /* ircengine::outputcontrol::privmsg */
 #define KviOption_stringExtendedPrivmsgPostfix 12                    /* ircengine::outputcontrol::privmsg */
 #define KviOption_stringCtcpVersionPostfix 13                        /* ircengine::ctcp::replies */
-#define KviOption_stringCtcpUserinfoReply 14                         /* ircengine::ctcp::replies */
+#define KviOption_stringCtcpUserInfoOther 14                         /* identity */
 #define KviOption_stringCtcpSourcePostfix 15                         /* ircengine::ctcp::replies */
 #define KviOption_stringUrlFtpCommand 16                             /* urlhandlers */
 #define KviOption_stringUrlHttpsCommand 17                           /* urlhandlers */
 #define KviOption_stringUrlMailtoCommand 18                          /* urlhandlers */
-//#define KviOption_stringUrlIrcCommand 19
+#define KviOption_stringPreferredMediaPlayer 19          /* still missing in options ! */
 #define KviOption_stringUrlFileCommand 20                            /* urlhandlers */
 #define KviOption_stringNickCompletionPostfix 21
 #define KviOption_stringUrlUnknownCommand 22                         /* urlhandlers */
@@ -316,7 +354,21 @@ DECLARE_OPTION_STRUCT(KviStringListOption,QStringList)
 // empty string means guess from locale
 #define KviOption_stringDefaultTextEncoding 43                       /* text encoding */
 
-#define KVI_NUM_STRING_OPTIONS 44
+#define KviOption_stringWinampTextEncoding 44
+#define KviOption_stringMp3TagsEncoding 45
+
+#define KviOption_stringCtcpUserInfoAge 46                                /* identity */
+#define KviOption_stringCtcpUserInfoGender 47                             /* identity */
+#define KviOption_stringCtcpUserInfoLocation 48                           /* identity */
+#define KviOption_stringCtcpUserInfoLanguages 49                          /* identity */
+#define KviOption_stringWordSplitters 50				/* highlight */
+#define KviOption_stringOnNewQueryOpenedSound 51				/* sounds */
+#define KviOption_stringOnHighlightedMessageSound 52				/* sounds */
+#define KviOption_stringOnMeKickedSound 53				/* sounds */
+#define KviOption_stringOnQueryMessageSound 54				/* sounds */
+#define KviOption_stringIrcViewTimestampFormat 55			
+
+#define KVI_NUM_STRING_OPTIONS 56
 
 
 
@@ -392,8 +444,10 @@ DECLARE_OPTION_STRUCT(KviStringListOption,QStringList)
 #define KviOption_colorUserListViewGrid 51                                        /* look & feel::interface look::userlist */
 #define KviOption_colorUserListViewChanOwnerForeground 52                         /* look & feel::interface look::userlist */
 #define KviOption_colorIrcViewMarkLine 53                                         /* look & feel::interface look::ircvew */
+#define KviOption_colorUserListViewAwayForeground 54                              /* look & feel::interface look::userlist */
+#define KviOption_colorUserListViewOwnForeground 55                              /* look & feel::interface look::userlist */
 
-#define KVI_NUM_COLOR_OPTIONS 54
+#define KVI_NUM_COLOR_OPTIONS 56
 
 
 
@@ -423,7 +477,7 @@ DECLARE_OPTION_STRUCT(KviStringListOption,QStringList)
 #define KviOption_uintModuleCleanupTimeout 2
 #define KviOption_uintMaxCtcpRequests 3                              /* ircengine::ctcp */
 #define KviOption_uintCtcpFloodCheckInterval 4                       /* ircengine::ctcp */
-#define KviOption_uintModeChangesPerMessage 5
+#define KviOption_uintTotalConnectionTime 5
 #define KviOption_uintDccSocketTimeout 6                             /* dcc::general */
 #define KviOption_uintDccSendIdleStepInMSec 7                        /* dcc::send */
 #define KviOption_uintDccSendPacketSize 8                            /* dcc::send */
@@ -471,8 +525,19 @@ DECLARE_OPTION_STRUCT(KviStringListOption,QStringList)
 #define KviOption_uintAutoReconnectDelay 44                          /* connection */
 #define KviOption_uintMaxDccSendSpeed 45                             /* dcc::file transfers */
 #define KviOption_uintMaxDccRecvSpeed 46                             /* dcc::file transfers */
+#define KviOption_uintTimeStampForeground 47
+#define KviOption_uintTimeStampBackground 48
+#define KviOption_uintUserExperienceLevel 49
+#define KviOption_uintClassicTaskBarMaximumButtonWidth 50
+#define KviOption_uintDefaultBanType 51
+#define KviOption_uintIrcViewPixmapAlign 52
+#define KviOption_uintUserListPixmapAlign 53
+#define KviOption_uintToolBarAppletPixmapAlign 54
+#define KviOption_uintTreeTaskBarPixmapAlign 55
+#define KviOption_uintInputPixmapAlign 56
+#define KviOption_uintAutoFlushLogs 57
 
-#define KVI_NUM_UINT_OPTIONS 47
+#define KVI_NUM_UINT_OPTIONS 58
 
 
 
@@ -505,7 +570,7 @@ DECLARE_OPTION_STRUCT(KviStringListOption,QStringList)
 #define KVI_MSGTYPE_OPTIONS_PREFIX "msgtype"
 #define KVI_MSGTYPE_OPTIONS_PREFIX_LEN 7
 
-#define KVI_NUM_MSGTYPE_OPTIONS 134
+#define KVI_NUM_MSGTYPE_OPTIONS 136
 
 
 
