@@ -470,26 +470,32 @@ bool KviKvsEventManager::triggerHandlers(KviPtrList<KviKvsEventHandler> * pHandl
 		{
 			case KviKvsEventHandler::Script:
 			{
-				KviKvsScript * s = ((KviKvsScriptEventHandler *)h)->script();
-				KviKvsScript copy(*s);
-				KviKvsVariant retVal;
-				if(!copy.run(pWnd,pParams,&retVal))
+				if(((KviKvsScriptEventHandler *)h)->isEnabled())
 				{
-					// error! disable the handler if it's broken
-					if(KVI_OPTION_BOOL(KviOption_boolDisableBrokenEventHandlers))
+					KviKvsScript * s = ((KviKvsScriptEventHandler *)h)->script();
+					KviKvsScript copy(*s);
+					KviKvsVariant retVal;
+					int iRet = copy.run(pWnd,pParams,&retVal,KviKvsScript::PreserveParams);
+					if(!iRet)
 					{
-						((KviKvsScriptEventHandler *)h)->setEnabled(false);
-						pWnd->output(KVI_OUT_PARSERERROR,__tr2qs("Event handler %Q is broken: disabling"),&(s->name()));
+						// error! disable the handler if it's broken
+						if(KVI_OPTION_BOOL(KviOption_boolDisableBrokenEventHandlers))
+						{
+							((KviKvsScriptEventHandler *)h)->setEnabled(false);
+							pWnd->output(KVI_OUT_PARSERERROR,__tr2qs("Event handler %Q is broken: disabling"),&(s->name()));
+						}
 					}
+					if(!bGotHalt)bGotHalt = (iRet & KviKvsScript::HaltEncountered);
 				}
-				if(!bGotHalt)bGotHalt = !retVal.isNothing();
 			}
 			break;
 			case KviKvsEventHandler::Module:
 			{
 				KviModule * m = (KviModule *)((KviKvsModuleEventHandler *)h)->moduleInterface();
 				KviKvsModuleEventHandlerRoutine * proc = ((KviKvsModuleEventHandler *)h)->handlerRoutine();
-				KviKvsModuleEventCall call(m,pParams);
+				KviKvsVariant retVal;
+				KviKvsRunTimeContext ctx(0,pWnd,pParams,&retVal);
+				KviKvsModuleEventCall call(m,&ctx,pParams);
 				if(!(*proc)(&call))bGotHalt = true;
 			}
 			break;
@@ -525,10 +531,9 @@ void KviKvsEventManager::loadRawEvents(const QString &szFileName)
 					KviQString::sprintf(tmp,"Name%u",index);
 					QString szName = cfg.readQStringEntry(tmp,"unnamed");
 					KviQString::sprintf(tmp,"Buffer%u",index);
-					QString * pszCode = new QString();
-					*pszCode = cfg.readQStringEntry(tmp,"");
+					QString szCode = cfg.readQStringEntry(tmp,"");
 					KviQString::sprintf(tmp,"RawEvent%u::%Q",index,&szName);
-					KviKvsScriptEventHandler * s = new KviKvsScriptEventHandler(szName,tmp,pszCode);
+					KviKvsScriptEventHandler * s = new KviKvsScriptEventHandler(szName,tmp,szCode);
 					KviQString::sprintf(tmp,"Enabled%u",index);
 					s->setEnabled(cfg.readBoolEntry(tmp,false));
 					m_rawEventTable[i]->append(s);
@@ -592,13 +597,12 @@ void KviKvsEventManager::loadAppEvents(const QString &szFileName)
 					KviQString::sprintf(tmp,"Name%u",index);
 					QString szName = cfg.readQStringEntry(tmp,"unnamed");
 					KviQString::sprintf(tmp,"Buffer%u",index);
-					QString * pszCode = new QString();
-					*pszCode = cfg.readQStringEntry(tmp,"");
+					QString szCode = cfg.readQStringEntry(tmp,"");
 					KviQString::sprintf(tmp,"Enabled%u",index);
 					bool bEnabled = cfg.readBoolEntry(tmp,false);
 					QString szCntx;
 					KviQString::sprintf(szCntx,"%Q::%Q",&(m_appEventTable[i].name()),&szName);
-					KviKvsScriptEventHandler *s = new KviKvsScriptEventHandler(szName,szCntx,pszCode,bEnabled);
+					KviKvsScriptEventHandler *s = new KviKvsScriptEventHandler(szName,szCntx,szCode,bEnabled);
 					m_appEventTable[i].addHandler(s);
 				}
 			}
