@@ -399,39 +399,38 @@ KviKvsEvent KviKvsEventManager::m_appEventTable[KVI_NUM_SCRIPT_EVENTS]=
 		@title:
 			OnWindowPopupRequest
 		@short:
-			Used for window popup customization
+			Triggered when the user middle-clicks in a text view (or right-clicks with the CTRL key pressed).
 		@parameters:
 			none
 		@window:
-			The window that the user has clicked in
+			The window that the user has clicked in.
 		@description:
 			This event is here to allow easy customization of the window popup (it was builtin in KVIrc version prior to 3.0.0).[br]
-			Actually it is triggered when the user middle-clicks in a text view (or right-clicks with the CTRL key pressed).[br]
-			You should use it to popup your own version of the window popup that should interface the
-			logging and window-behaviour functions.[br]
+ 			Actually it is triggered when the user middle-clicks in a text view (or right-clicks with the CTRL key pressed).[br]
+ 			You should use it to popup your own version of the window popup that should interface the logging and window-behaviour functions.
 	*/
 	EVENT("OnWindowPopupRequest",""),
-	/*
+		/*
 		@doc: onhostlinkpopuprequest
 		@type:
 			event
 		@title:
 			OnHostLinkPopupRequest
 		@short:
-			Host link popup customization
+			Triggered when an user right clicks on a hostname link in the text view.
 		@parameters:
-			$0 = host name
+			$0 = actual name
+			$1 - visible name
+			$2 - command name
 		@window:
-			The window that the user has clicked in
+			The window that the user has clicked in.
 		@description:
-			This event is triggered when an user has right clicked on a hostname link in
-			the text view. The double click link action is builtin and it causes KVIrc
-			to perform a DNS query on the hostname.
-			This is a good place to show a [cmd]popup[/cmd] menu with some operations
-			that can be done on the hostname like "telnet" , "traceroute" , some special
-			kind of DNS lookup (maybe an xterm with nslookup or sth).[br]
+			This event is triggered when an user has right clicked on a hostname link in the text view.
+			The double click link action is builtin and it causes KVIrc to perform a DNS query on the hostname.
+			This is a good place to show a popup menu with some operations that can be done on the hostname like
+			"telnet", "traceroute", some special kind of DNS lookup (maybe an xterm with nslookup or sth).
 	*/
-	EVENT("OnHostLinkPopupRequest","$0 = hostname"),
+	EVENT("OnHostLinkPopupRequest","$0 = actual name\n$1 - visible name\n$2 - command name"),
 	/*
 		@doc: onurllinkpopuprequest
 		@type:
@@ -441,10 +440,15 @@ KviKvsEvent KviKvsEventManager::m_appEventTable[KVI_NUM_SCRIPT_EVENTS]=
 		@short:
 			URL link popup customization
 		@parameters:
-			$0 = url
+			$0 = actual name
+			$1 - visible name
+			$2 - command name
 		@window:
 			The window that the user has clicked in
 		@description:
+			Actual name is equal to the command name if command name is not null.
+			Visible name is the link text (as it shown by user)
+			Command name is set if the visible name differs with the actual link target(probably contains mIRC colors,etc). It is equal to the actual target name.
 			This event is triggered when an user has right clicked on an URL link in
 			the text view. The double click link action is builtin and it causes KVIrc
 			to open the URL with a browser (konqueror in KDE compilation mode or netscape).[br]
@@ -452,7 +456,7 @@ KviKvsEvent KviKvsEventManager::m_appEventTable[KVI_NUM_SCRIPT_EVENTS]=
 			that can be performed with the URL , like bookmarking in some way or opening
 			with a specific browser.[br]
 	*/
-	EVENT("OnURLLinkPopupRequest","$0 = url"),
+	EVENT("OnURLLinkPopupRequest","$0 = actual name\n$1 - visible name\n$2 - command name"),
 	/*
 		@doc: onserverlinkpopuprequest
 		@type:
@@ -462,10 +466,15 @@ KviKvsEvent KviKvsEventManager::m_appEventTable[KVI_NUM_SCRIPT_EVENTS]=
 		@short:
 			Server link popup customization
 		@parameters:
-			$0 = server name
+			$0 = actual name
+			$1 - visible name
+			$2 - command name
 		@window:
 			The window that the user has clicked in
 		@description:
+			Actual name is equal to the command name if command name is not null.
+			Visible name is the link text (as it shown by user)
+			Command name is set if the visible name differs with the actual link target(probably contains mIRC colors,etc). It is equal to the actual target name.
 			This event is triggered when an user has right clicked on an server name link in
 			the text view. The double click link action is builtin and it causes KVIrc
 			to query the MOTD of the server.[br]
@@ -475,7 +484,7 @@ KviKvsEvent KviKvsEventManager::m_appEventTable[KVI_NUM_SCRIPT_EVENTS]=
 			it are often hidden. The name of the server is generally valid only within
 			the IRC network that you are connected to.[br]
 	*/
-	EVENT("OnServerLinkPopupRequest","$0 = server name"),
+	EVENT("OnServerLinkPopupRequest","$0 = actual name\n$1 - visible name\n$2 - command name"),
 	/*
 		@doc: onchannellinkpopuprequest
 		@type:
@@ -485,10 +494,15 @@ KviKvsEvent KviKvsEventManager::m_appEventTable[KVI_NUM_SCRIPT_EVENTS]=
 		@short:
 			Channel link popup customization
 		@parameters:
-			$0 = channel name
+			$0 = actual name
+			$1 - visible name
+			$2 - command name
 		@window:
 			The window that the user has clicked in
 		@description:
+			Actual name is equal to the command name if command name is not null.
+			Visible name is the link text (as it shown by user)
+			Command name is set if the visible name differs with the actual link target(probably contains mIRC colors,etc). It is equal to the actual target name.
 			This event is triggered when an user has right clicked on an channel link in
 			the text view. The double click link action is builtin and it causes KVIrc
 			to join the channel (unless the user is already on).[br]
@@ -496,7 +510,7 @@ KviKvsEvent KviKvsEventManager::m_appEventTable[KVI_NUM_SCRIPT_EVENTS]=
 			that can be performed with the channel name like bookmarking, joining with a password,
 			performing a [cmd]who[/cmd] etc...
 	*/
-	EVENT("OnChannelLinkPopupRequest","$0 = channel name"),
+	EVENT("OnChannelLinkPopupRequest","$0 = actual name\n$1 - visible name\n$2 - command name"),
 	/*
 		@doc: onnicklinkpopuprequest
 		@type:
@@ -506,10 +520,15 @@ KviKvsEvent KviKvsEventManager::m_appEventTable[KVI_NUM_SCRIPT_EVENTS]=
 		@short:
 			Nick link popup customization
 		@parameters:
-			$0 = nickname
+			$0 = actual name
+			$1 - visible name
+			$2 - command name
 		@window:
 			The window that the user has clicked in
 		@description:
+			Actual name is equal to the command name if command name is not null.
+			Visible name is the link text (as it shown by user)
+			Command name is set if the visible name differs with the actual link target(probably contains mIRC colors,etc). It is equal to the actual target name.
 			This event is triggered when an user has right clicked on an "unknown" nickname link in
 			the text view.[br]
 			This is a good place to show a [cmd]popup[/cmd] menu with some operations
@@ -520,7 +539,7 @@ KviKvsEvent KviKvsEventManager::m_appEventTable[KVI_NUM_SCRIPT_EVENTS]=
 			The nickname links that appear in the text view of the query in that they are a remote end
 			are handled by the [event:onquerynickpopuprequest]OnQueryNickPopupRequest[/event] event.[br]
 	*/
-	EVENT("OnNickLinkPopupRequest","$0 = nick name"),
+	EVENT("OnNickLinkPopupRequest","$0 = actual name\n$1 - visible name\n$2 - command name"),
 	/*
 		@doc: onquerynickpopuprequest
 		@type:
@@ -530,17 +549,22 @@ KviKvsEvent KviKvsEventManager::m_appEventTable[KVI_NUM_SCRIPT_EVENTS]=
 		@short:
 			Query nicklist popup customization
 		@parameters:
-			$0 = nickname list
+			$0 = actual name
+			$1 - visible name
+			$2 - command name
 		@window:
 			The window that the user has clicked in
 		@description:
+			Actual name is equal to the command name if command name is not null.
+			Visible name is the link text (as it shown by user)
+			Command name is set if the visible name differs with the actual link target(probably contains mIRC colors,etc). It is equal to the actual target name.
 			This event is triggered when an user has right clicked on a set of nicknames in
 			the query target list.[br]
 			This is a good place to show a [cmd]popup[/cmd] menu with some operations
 			that can be performed with the nickname like sending a file by dcc, opening a dcc.chat or
 			performing a whois.[br]
 	*/
-	EVENT("OnQueryNickPopupRequest","$0 = nickname list"),
+	EVENT("OnQueryNickPopupRequest","$0 = actual name\n$1 - visible name\n$2 - command name"),
 	/*
 		@doc: onquerynickdefaultactionrequest
 		@type:
@@ -550,16 +574,21 @@ KviKvsEvent KviKvsEventManager::m_appEventTable[KVI_NUM_SCRIPT_EVENTS]=
 		@short:
 			Query double click customization
 		@parameters:
-			$0 = nickname
+			$0 = actual name
+			$1 - visible name
+			$2 - command name
 		@window:
 			The window that the user has clicked in
 		@description:
+			Actual name is equal to the command name if command name is not null.
+			Visible name is the link text (as it shown by user)
+			Command name is set if the visible name differs with the actual link target(probably contains mIRC colors,etc). It is equal to the actual target name.
 			This event is triggered when an user has double clicked on a nickname in
 			the query target list.[br]
 			You should use this event to do some significant action associated with the double click.[br]
 			A good example might be a [cmd]whois[/cmd] query or a [cmd]dcc.chat[/cmd]
 	*/
-	EVENT("OnQueryNickDefaultActionRequest","$0 = nickname"),
+	EVENT("OnQueryNickDefaultActionRequest","$0 = actual name\n$1 - visible name\n$2 - command name"),
 	/*
 		@doc: onnicklinkdefaultactionrequest
 		@type:
@@ -569,10 +598,15 @@ KviKvsEvent KviKvsEventManager::m_appEventTable[KVI_NUM_SCRIPT_EVENTS]=
 		@short:
 			Nick link double click customization
 		@parameters:
-			$0 = nickname
+			$0 = actual name
+			$1 - visible name
+			$2 - command name
 		@window:
 			The window that the user has clicked in
 		@description:
+			Actual name is equal to the command name if command name is not null.
+			Visible name is the link text (as it shown by user)
+			Command name is set if the visible name differs with the actual link target(probably contains mIRC colors,etc). It is equal to the actual target name.
 			This event is triggered when an user has double clicked on a nickname link.[br]
 			You should use this event to perform some significant action associated with the double click.[br]
 			A good example might be a [cmd]whois[/cmd] or a [cmq]query[/cmd] call.[br]
@@ -581,9 +615,9 @@ KviKvsEvent KviKvsEventManager::m_appEventTable[KVI_NUM_SCRIPT_EVENTS]=
 			[event:onquerynickdefaultactionrequest]OnQueryNickDefaultActionRequest[/event] and
 			[event:onchannelnickdefatulactionrequest]OnChannelNickDefaultActionRequest[/event] events.[br]
 	*/
-	EVENT("OnNickLinkDefaultActionRequest","$0 = nickname"),
+	EVENT("OnNickLinkDefaultActionRequest","$0 = actual name\n$1 - visible name\n$2 - command name"),
 	/*
-		@doc: ontextviewdoubleclicked
+		@doc: ontextviewdoublecliked
 		@type:
 			event
 		@title:
@@ -591,14 +625,17 @@ KviKvsEvent KviKvsEventManager::m_appEventTable[KVI_NUM_SCRIPT_EVENTS]=
 		@short:
 			Text view link double click customization
 		@parameters:
-			None
+			$0 = actual name
+			$1 - visible name
+			$2 - command name
 		@window:
 			The window that the user has clicked in
 		@description:
-			This event is triggered when an user has double clicked on a text view and has not
-			hit any link.[br]
-			I have no idea what you can do in this event....maybe some sort of channel-central dialog ?[br]
-			:)[br]
+			Actual name is equal to the command name if command name is not null.
+			Visible name is the link text (as it shown by user).
+			Command name is set if the visible name differs with the actual link target (probably contains mIRC colors,etc). It is equal to the actual target name.
+			This event is triggered when an user has double clicked on a text view and has not hit any link.[br]
+			I have no idea what you can do in this event....maybe some sort of channel-central dialog? :)
 	*/
 	EVENT("OnTextViewDoubleClicked",""),
 	/*
@@ -987,7 +1024,7 @@ KviKvsEvent KviKvsEventManager::m_appEventTable[KVI_NUM_SCRIPT_EVENTS]=
 			[event:onquerytargetadded]OnQueryTargetAdded[/event]
 	*/
 	EVENT("OnQueryWindowCreated",""),
-	/* 
+	/*
 	 	@doc: onban
 		@type:
 			event
@@ -999,7 +1036,7 @@ KviKvsEvent KviKvsEventManager::m_appEventTable[KVI_NUM_SCRIPT_EVENTS]=
 			$0 = source nick
 			$1 = source username
 			$2 = source hostname
-			$3 = ban mask			
+			$3 = ban mask
 		@window:
 			Channel window
 		@description:
@@ -1027,7 +1064,7 @@ KviKvsEvent KviKvsEventManager::m_appEventTable[KVI_NUM_SCRIPT_EVENTS]=
 			Triggered when someone sets a -b flag on the channel
 		@seealso:
 			[event:onban]OnBan[/event]
-	*/		
+	*/
 	EVENT("OnUnBan","$0 = source nick\n$1 = source username\n$2 = source hostname\n$3 = ban mask"),
 	/*
 		@doc: onmeban
@@ -1353,15 +1390,19 @@ channel
             $1 = source username
             $2 = source host
             $3 = message
+		    $4 = [target mode prefixes]
         @window:
             Channels window
         @description:
             Triggered when a channel message is received
+            $0!$1@$2 is the source of the message, $3 is the message text
+            and $4 are the eventual mode prefixes added to the target channel (i.e:
+            if the message is only for channel operators then you will get the string @ in $4).
         @seealso:
             [event:onquerymessage]OnQueryMessage[/event]
 			[event:ondccchatmessage]OnDCCChatMessage[/event]
     */
-	EVENT("OnChannelMessage","$0 = source nick\n$1 = source username\n$2 = source hostname\n$3 = message"),
+	EVENT("OnChannelMessage","$0 = source nick\n$1 = source username\n$2 = source hostname\n$3 = message\n$4 = target mode prefixes"),
 	/*
 		@doc: onquerymessage
 		@type:
@@ -1793,7 +1834,7 @@ channel
 				case the event is triggered in the console window.[br]
 			Calling [cmd]halt[/cmd] in this event stops the message output.[br]
 	*/
-			
+
 	EVENT("OnAction", \
 			"$0 = source nick\n" \
 			"$1 = source username\n" \
@@ -1822,7 +1863,7 @@ channel
 		@seealso:
 			[event:onctcprequest]OnCTCPRequest[/event]
 			[event:onctcpreply]OnCTCPReply[/event]
-	*/	
+	*/
 	EVENT("OnCTCPFlood", "$0 = source nick\n$1 = source username\n$2 = source hostname\n$3 = target\n$4 = ctcp type\n$5- = ctcp parameters"),
 	/*
 	 	@doc: onchannelnotice
@@ -1841,7 +1882,9 @@ channel
 		@description:
 			Someone has issued a NOTICE with a channel as his target
 			For normal channel notices $2 parameter will only a channel name.
-			For channel op or channel voice notices it will contain also the leading '@' or '+' flag.			
+			For channel op or channel voice notices it will contain also the leading '@' or '+' flag.
+			Please note that this convention is different from the one used in [event:onchannelprivmsg]OnChannelPrivmsg[/event]:
+			the incompatibility is here for historical reasons: it is not a big deal so we're not fighting it :)
 		@seealso:
 			[event:onservernotice]OnServerNotice[/event]
 	*/
@@ -2030,11 +2073,11 @@ channel
 			Active window
 		@description:
 			An accelerator key has been pressed by the user.[br]
-			The key sequence that the user has pressed is passed in $0 as a 
+			The key sequence that the user has pressed is passed in $0 as a
 			human readable string.[br]
 			The keys that actually work with KVIrc are:[br]
-			F1 to F12, F1 to F12 with SHIFT key pressed, 0 to 9 with CTRL Key pressed.[br]
-			This means that you can set up 34 "quick" aliases...that's really more than
+			F2 to F12, F1 to F12 with SHIFT key pressed, 0 to 9 with CTRL Key pressed.[br]
+			This means that you can set up 33 "quick" aliases...that's really more than
 			an user can remember.[br]
 			If you need more keys , mail me , I'll see what I can do :)[br]
     */
@@ -2112,7 +2155,7 @@ channel
 			but this is rather a bad idea since KVIrc gives a lot of additional informations to the
 			user with some notify-methods).[br]
 			This is a good place to play a sound or attract the user attention in some other way.[br]
-			
+
 	*/
 	EVENT("OnNotifyOnLine","$0 = nickname"),
 	/*
@@ -2547,7 +2590,7 @@ channel
 	*/
 	EVENT("OnDCCSessionCreated","$0 = DCC session id"),
 	/*
-		@doc: ondccsessioncreated
+		@doc: ondccsessiondestroyed
 		@type:
 			event
 		@title:
@@ -2874,8 +2917,8 @@ channel
 		@short:
 			Drop in query
 		@parameters:
-			$target = source nick
-			$0 = filetosend
+			$0 = source nick
+			$1 = filedropped
 		@window:
 			Query
 		@description:
@@ -2883,6 +2926,170 @@ channel
 			$target is the nick.[br]
 			$0 is the dropped file.[br]
 	*/
-	EVENT("OnQueryFileDropped","$0 = nick\n$1 = filedropped") //-|Grifisx & Noldor|-
+	EVENT("OnQueryFileDropped","$0 = nick\n$1 = filedropped"), //-|Grifisx & Noldor|-
+	/*
+		@doc: OnHTTPGetTerminated
+		@type:
+			event
+		@title:
+			OnHTTPGetTerminated
+		@short:
+			A http.get command has terminated execution
+		@parameters:
+			$0 = status
+			$1 = remote url
+			$2 = local file
+			$3 = magic identifier
+		@window:
+			Console
+		@description:
+			Triggered when a [cmd]http.get[/cmd] file download has terminated execution.
+			If the transfer has been succesfull then $0 will contain the value 1 (true),
+			otherwise will contain the value 0. $1 contains the remote url and $2 the local
+			path on disk of the saved file. $3 contains the magic identifier passed to
+			the http.get command by the means of the -i switch.
+	*/
+	EVENT("OnHTTPGetTerminated","$0 = status\n$1 = remote url\n$2 = local file\n$3 = magic identifier"),
+	/*
+		@doc: OnChannelModeChange
+		@type:
+			event
+		@title:
+			OnChannelModeChange
+		@short:
+			The channel mode has been changed
+		@parameters:
+			$0 = source nick
+			$1 = source user
+			$2 = source host
+			$3 = modeflags
+			$4 = mode parameter string
+		@window:
+			channel
+		@description:
+			Triggered when a channel mode has been changed, AFTER all the mode specific events
+			were triggered. $3 contains the unparsed mode flags (-o+b...) and $4 contains
+			the unparsed mode parameter string (you need to split it!).
+	*/
+	EVENT("OnChannelModeChange","$0 = sourcenick\n$1 = sourceusername\n$2 = sourcehost\n$3 = modeflags\n$4 = mode params"),
+		/*
+		@doc: OnChannelNickLinkClick
+		@type:
+			event
+		@title:
+			OnChannelNickLinkClick
+		@short:
+			Clicked on nick in channel
+		@parameters:
+			$0 = nick list
+		@window:
+			channel
+		@description:
+			Triggered when user clicks on nickname in channel window
+			It will be triggered only at the left mouse button click
+	*/
+	EVENT("OnChannelNickLinkClick",""),
+		/*
+		@doc: OnQueryNickLinkClick
+		@type:
+			event
+		@title:
+			OnQueryNickLinkClick
+		@short:
+			Triggered when user clicks on nickname in query window
+		@parameters:
+			$0 = nickname list
+		@window:
+			query
+		@description:
+			Triggered when user clicks on nickname in query window
+			It will be triggered only at the left mouse button click
+	*/
+	EVENT("OnQueryNickLinkClick","$0 = nickname list"),
+		/*
+		@doc: OnConsoleNickLinkClick
+		@type:
+			event
+		@title:
+			OnConsoleNickLinkClick
+		@short:
+			Triggered when user clicks on nickname in console window
+		@parameters:
+			$0 = nickname
+		@window:
+			console
+		@description:
+			Triggered when user clicks on nickname in console window
+			It will be triggered only at the left mouse button click
+	*/
+	EVENT("OnConsoleNickLinkClick","$0 = nickname"),
+		/*
+		@doc: OnHostLinkClick
+		@type:
+			event
+		@title:
+			OnHostLinkClick
+		@short:
+			Triggered when user clicks on any hostname in any window
+		@parameters:
+			$0 = host name
+		@window:
+			any
+		@description:
+			Triggered when user clicks on any hostname in any window
+			It will be triggered only at the left mouse button click
+	*/
+	EVENT("OnHostLinkClick","$0 = host name")
+	,	/*
+		@doc: OnUrlLinkClick
+		@type:
+			event
+		@title:
+			OnUrlLinkClick
+		@short:
+			Triggered when user clicks on any URL link in any window
+		@parameters:
+			$0 = url
+		@window:
+			any
+		@description:
+			Triggered when user clicks on any URL link in any window
+			It will be triggered only at the left mouse button click
+	*/
+	EVENT("OnUrlLinkClick","$0 = url"),
+		/*
+		@doc: OnChannelLinkClick
+		@type:
+			event
+		@title:
+			OnChannelLinkClick
+		@short:
+			Triggered when user clicks on any channel name in any window
+		@parameters:
+			$0 = channel name
+		@window:
+			any
+		@description:
+			Triggered when user clicks on any channel name in any window
+			It will be triggered only at the left mouse button click
+	*/
+	EVENT("OnChannelLinkClick","$0 = channel name"),
+		/*
+		@doc: OnServerLinkClick
+		@type:
+			event
+		@title:
+			OnServerLinkClick
+		@short:
+			Triggered when user clicks on server name in any window
+		@parameters:
+			$0 = server name
+		@window:
+			any
+		@description:
+			Triggered when user clicks on server name in any window
+			It will be triggered only at the left mouse button click
+	*/
+	EVENT("OnServerLinkClick","$0 = server name")
 };
 
