@@ -33,13 +33,96 @@
 #include "kvi_kvs_kernel.h"
 
 #include "kvi_kvs_parser_macros.h"
+#include "kvi_kvs_object_functionhandler.h"
 
 #include "kvi_locale.h"
 
 #include "kvi_cmdformatter.h"
 
 
-KviKvsTreeNodeSpecialCommand * KviKvsParser::parseSpecialCommandBreak()
+KviKvsTreeNodeCommand * KviKvsParser::parseSpecialCommandPerlBegin()
+{
+	// in fact this is not a fully special command
+	// it is special only in the sense of parsing.
+	// Once parsed, the command is routed to the perl module
+	// with the perl code as FIRST parameter and the other parameters
+	// of the command following.
+	// the help page for perl.begin is in the perl module
+
+	// perl.begin(context) <perl code> perl.end
+	// 
+	
+	const QChar * pBegin = KVSP_curCharPointer;
+
+	skipSpaces();
+	KviKvsTreeNodeDataList * dl;
+	if(KVSP_curCharUnicode == '(')
+	{	
+		dl = parseCommaSeparatedParameterList();
+		if(!dl)return 0;
+	} else {
+		dl = new KviKvsTreeNodeDataList(pBegin);
+	}
+	
+	//while(!KVSP_curCharIsEndOfCommand)KVSP_skipChar;
+	if(!KVSP_curCharIsEndOfBuffer)KVSP_skipChar;
+	
+	if(!skipSpacesAndNewlines())
+	{
+		delete dl;
+		return 0;
+	}
+	
+	// allow a ';' after perl.begin
+	if(KVSP_curCharIsEndOfCommand && !KVSP_curCharIsEndOfBuffer)
+	{
+		KVSP_skipChar;
+		if(!skipSpacesAndNewlines())
+		{
+			delete dl;
+			return 0;
+		}
+	}
+	
+	const QChar * pPerlBegin = KVSP_curCharPointer;
+
+	// now look for perl.end[terminator]
+	static QString szPerlEnd("perl.end");
+	const QChar * pPerlEnd;
+	for(;;)
+	{
+		while(KVSP_curCharUnicode && (KVSP_curCharUnicode != 'p') && (KVSP_curCharUnicode != 'P'))
+			KVSP_skipChar;
+		if(KVSP_curCharIsEndOfBuffer)
+		{
+			delete dl;
+			error(KVSP_curCharPointer,__tr2qs("Unexpected end of command buffer while looking for the \"perl.end\" statement"));
+			return 0;
+		}
+		pPerlEnd = KVSP_curCharPointer;
+		if(KviQString::equalCIN(szPerlEnd,KVSP_curCharPointer,8))
+		{
+			KVSP_skipNChars(8);
+			if(KVSP_curCharIsEndOfCommand || (KVSP_curCharUnicode == ' ') || (KVSP_curCharUnicode == '\t'))
+			{
+				// yeah!
+				QString szPerl(pPerlBegin,pPerlEnd - pPerlBegin);
+				dl->prependItem(new KviKvsTreeNodeConstantData(pPerlBegin,new KviKvsVariant(szPerl)));
+				while(!KVSP_curCharIsEndOfCommand)KVSP_skipChar;
+				if(!KVSP_curCharIsEndOfBuffer)KVSP_skipChar;
+				break;
+			} else {
+				KVSP_backNChars(7);
+			}
+		} else {
+			KVSP_skipChar;
+		}
+	}
+
+	return new KviKvsTreeNodeModuleSimpleCommand(pBegin,"perl","begin",dl);
+}
+
+KviKvsTreeNodeCommand * KviKvsParser::parseSpecialCommandBreak()
 {
 	/*
 		@doc: break
@@ -69,7 +152,77 @@ KviKvsTreeNodeSpecialCommand * KviKvsParser::parseSpecialCommandBreak()
 	return new KviKvsTreeNodeSpecialCommandBreak(pBegin);
 }
 
-KviKvsTreeNodeSpecialCommand * KviKvsParser::parseSpecialCommandGlobal()
+KviKvsTreeNodeCommand * KviKvsParser::parseSpecialCommandUnset()
+{
+	/*
+		@doc: unset
+		@type:
+			command
+		@title:
+			unset
+		@syntax:
+			unset <variable_list>
+		@keyterms:
+			unsetting variables
+		@short:
+			Unsets a set of variables
+		@description:
+			Unsets the specified list of comma separated variables.
+			It is equivalent to assigning the default empty value
+			to each variable on its own: just does it all at aonce.
+			Note that KVIrc automatically frees the local variable memory
+			when they go out of scope and the global variable memory
+			when KVIrc terminates.
+		@examples:
+			[example]
+				%a = pippo
+				%b = 1
+				[cmd]echo[/cmd] %a %b
+				unset %a %b
+				[cmd]echo[/cmd] %a %b
+			[/example]
+	*/
+
+	const QChar * pCmdBegin = KVSP_curCharPointer;
+
+	KviPtrList<KviKvsTreeNodeVariable> * pVarList = new KviPtrList<KviKvsTreeNodeVariable>;
+	pVarList->setAutoDelete(true);
+	
+	while(KVSP_curCharUnicode == '%')
+	{
+		KviKvsTreeNodeVariable * d = parsePercent();
+		if(!d)return 0;
+
+		pVarList->append(d);
+
+		skipSpaces();
+		
+		if(KVSP_curCharUnicode == ',')
+		{
+			KVSP_skipChar;
+			skipSpaces();
+		}
+	}
+
+	if(!KVSP_curCharIsEndOfCommand)
+	{
+		warning(KVSP_curCharPointer,__tr2qs("The 'unset' command needs a variable list"));
+		error(KVSP_curCharPointer,__tr2qs("Found character %q (unicode %x) where a variable was expected"),KVSP_curCharPointer,KVSP_curCharUnicode);
+		return 0;
+	}
+
+	if(!KVSP_curCharIsEndOfBuffer)KVSP_skipChar;
+
+	if(pVarList->count() < 1)
+	{
+		delete pVarList;
+		warning(KVSP_curCharPointer,__tr2qs("'unset' command used without a variable list"));
+		return 0; // null unset ?
+	}
+	return new KviKvsTreeNodeSpecialCommandUnset(pCmdBegin,pVarList);
+}
+
+KviKvsTreeNodeCommand * KviKvsParser::parseSpecialCommandGlobal()
 {
 	/*
 		@doc: global
@@ -99,7 +252,6 @@ KviKvsTreeNodeSpecialCommand * KviKvsParser::parseSpecialCommandGlobal()
 		@examples:
 			global %a,%b,%c;
 	*/
-	
 	while(KVSP_curCharUnicode == '%')
 	{
 		KVSP_skipChar;
@@ -134,12 +286,307 @@ KviKvsTreeNodeSpecialCommand * KviKvsParser::parseSpecialCommandGlobal()
 	}
 
 	if(!KVSP_curCharIsEndOfBuffer)KVSP_skipChar;
-
 	return 0;
 }
 
+KviKvsTreeNodeCommand * KviKvsParser::parseSpecialCommandClass()
+{
+	/*
+		@doc: class
+		@title:
+			class
+		@short:
+			Defines a new object class
+		@keyterms:
+			defining an object class
+		@type:
+			command
+		@syntax:
+			class(<classname:string>[,<base_class_name:string>])
+			{
+				[internal] [function] <function_name>[([<parameter reminder>])]
+				{
+					<function body>
+				}
 
-KviKvsTreeNodeSpecialCommand * KviKvsParser::parseSpecialCommandWhile()
+				...
+			}
+		@description:
+			Defines a new implementation of the class <classname>.
+			If an implementation of that class was already existing
+			it is removed with all the derived classes (and all the instances of this class
+			and the derived ones are destroyed).
+			<base_class_name> is the name of the class that the 
+			new class has to inherit from.[br]
+			If <base_class_name> is omitted, the new class inherits automatically
+			from [class:object]object[/class].[br]
+			Note:[br]
+			The keywords "function" and "event" that were used in KVIrc versions
+			previous to 3.0.0 have been removed since "useless".[br]
+			The function keyword, however, is still permitted.
+			The keyword "internal" is useful when you want to hide
+			certain function from the outside world. An internal function
+			cannot be called by anyone else but the object instance itself. Note that
+			this is different from the C++ "protected" or "private" keywords
+			that refer to the object's class instead of the object instance.
+			The <parameter reminder> part is an optional string
+			that can be used to sign the parameters that the function expects;
+			it acts as a programmer reminder or comment and it has no other
+			meaning in KVIrc scripting. The <parameter reminder> respects the syntax
+			of an expression, so it is terminated by a closed parenthesis.
+			It's rather dangerous to use this command inside an object
+			function handler: if the class definition <class> was already 
+			existing and it is a parent of the object's class, you might
+			end up executing "inexisting" code.[br]
+			As a thumb rule, use this command only outside object function handlers.[br]
+			[br][br]
+			Only for the curious: implementing protected and private access
+			list on members would have a considerable runtime overhead because
+			of the strange nature of the KVS language. Object member calls
+			are resolved completly at runtime (and that permits a lot of funny tricks
+			like [cmd]privateimpl[/cmd]) but unfortunately this also forces us
+			to check access lists at runtime. Ok, this would be a relatively small footprint for the "private"
+			keyword where we need to run UP the called object inheritance hierarchy
+			but would have a significant performance footprint for the "protected"
+			keyword where we would need to traverse the WHOLE inheritance tree of the called and calling
+			objects... "internal" still allows hiding members in a lot of situations
+			and is really fast to verify at runtime: no inheritance tree traversal
+			is needed and only object pointers are compared.
+		@examples:
+			[example]
+				class(myclass,[class]object[/class])
+				{
+					constructor
+					{
+						[cmd]echo[/cmd] Hey this is my constructor
+						[cmd]echo[/cmd] I have been just created
+					}
+
+					destructor
+					{
+						[cmd]echo[/cmd] Ops...being destroyed
+					}
+
+					sayHello(this function expects no parameters)
+					{
+						[cmd]echo[/cmd] Hello world!
+					}
+				}
+			[/example]
+		@seealso:
+			[cmd]privateimpl[/cmd], [cmd]killclass[/cmd], [cmd]clearobjects[/cmd], [fnc]$classDefined[/fnc](),
+			[doc:objects]Objects documentation[/doc]
+	*/
+
+	if(KVSP_curCharUnicode != '(')
+	{
+		error(KVSP_curCharPointer,__tr2qs("Found character %q (unicode %x) where an open parenthesis was expected"),KVSP_curCharPointer,KVSP_curCharUnicode);
+		return 0;
+	}
+
+	const QChar * pBegin = KVSP_curCharPointer;
+
+	KviKvsTreeNodeDataList * l = parseCommaSeparatedParameterList();
+	if(!l)return 0;
+
+	KviKvsTreeNodeSpecialCommandClass * pClass = new KviKvsTreeNodeSpecialCommandClass(pBegin,l);
+
+	if(!skipSpacesAndNewlines())
+	{
+		delete pClass;
+		return 0;
+	}
+
+	if(KVSP_curCharUnicode != '{')
+	{
+		errorBadChar(KVSP_curCharPointer,'{',"class");
+		delete pClass;
+		return 0;
+	}
+
+	KVSP_skipChar;
+
+	if(!skipSpacesAndNewlines())
+	{
+		delete pClass;
+		return 0;
+	}
+
+	while(KVSP_curCharUnicode != '}')
+	{
+		if((KVSP_curCharUnicode == '#') || (KVSP_curCharUnicode == '/'))
+		{
+			parseComment();
+			if(error())
+			{
+				delete pClass;
+				return 0;
+			}
+			if(!skipSpacesAndNewlines())
+			{
+				delete pClass;
+				return 0;
+			}
+			continue;
+		}
+
+		const QChar * pLabelBegin = KVSP_curCharPointer;
+
+		if(KVSP_curCharIsLetter)
+		{
+			KVSP_skipChar;
+			while(KVSP_curCharIsLetterOrNumber)KVSP_skipChar;
+		}
+
+		if(KVSP_curCharIsEndOfBuffer)
+		{
+			error(KVSP_curCharPointer,__tr2qs("Unexpected end of buffer in class definition"));
+			delete pClass;
+			return 0;
+		}
+
+		if(KVSP_curCharPointer == pLabelBegin)
+		{
+			error(KVSP_curCharPointer,__tr2qs("Found character %q (unicode %x) where a function name was expected"),KVSP_curCharPointer,KVSP_curCharUnicode);
+			delete pClass;
+			return 0;
+		}
+
+		QString szLabel(pLabelBegin,KVSP_curCharPointer - pLabelBegin);
+		
+		unsigned int uHandlerFlags = 0;
+
+		if(szLabel.lower() == "internal")
+		{
+			uHandlerFlags |= KviKvsObjectFunctionHandler::Internal;
+			skipSpaces();
+			if(KVSP_curCharUnicode != '(')
+			{
+				pLabelBegin = KVSP_curCharPointer;
+		
+				while(KVSP_curCharIsLetterOrNumber)KVSP_skipChar;
+		
+				if(KVSP_curCharIsEndOfBuffer)
+				{
+					error(KVSP_curCharPointer,__tr2qs("Unexpected end of buffer in class definition"));
+					delete pClass;
+					return 0;
+				}
+		
+				if(KVSP_curCharPointer == pLabelBegin)
+				{
+					error(KVSP_curCharPointer,__tr2qs("Found character %q (unicode %x) where a function name was expected"),KVSP_curCharPointer,KVSP_curCharUnicode);
+					delete pClass;
+					return 0;
+				}
+				szLabel = QString(pLabelBegin,KVSP_curCharPointer - pLabelBegin);
+			}
+		}
+
+
+		if(szLabel.lower() == "function")
+		{
+			skipSpaces();
+			if(KVSP_curCharUnicode != '(')
+			{
+				pLabelBegin = KVSP_curCharPointer;
+		
+				while(KVSP_curCharIsLetterOrNumber)KVSP_skipChar;
+		
+				if(KVSP_curCharIsEndOfBuffer)
+				{
+					error(KVSP_curCharPointer,__tr2qs("Unexpected end of buffer in class definition"));
+					delete pClass;
+					return 0;
+				}
+		
+				if(KVSP_curCharPointer == pLabelBegin)
+				{
+					error(KVSP_curCharPointer,__tr2qs("Found character %q (unicode %x) where a function name was expected"),KVSP_curCharPointer,KVSP_curCharUnicode);
+					delete pClass;
+					return 0;
+				}
+				szLabel = QString(pLabelBegin,KVSP_curCharPointer - pLabelBegin);
+			}
+		}
+		
+		if(!skipSpacesAndNewlines())
+		{
+			delete pClass;
+			return 0;
+		}
+
+		if(KVSP_curCharUnicode == '(')
+		{
+			while((!(KVSP_curCharIsEndOfBuffer)) && (KVSP_curCharUnicode != ')'))
+				KVSP_skipChar;
+				
+			if(KVSP_curCharIsEndOfBuffer)
+			{
+				error(KVSP_curCharPointer,__tr2qs("Unexpected end of buffer in function parameter list reminder"));
+				delete pClass;
+				return 0;
+			}
+			
+			KVSP_skipChar;
+
+			if(!skipSpacesAndNewlines())
+			{
+				delete pClass;
+				return 0;
+			}
+		}
+
+		if(KVSP_curCharIsEndOfBuffer)
+		{
+			error(KVSP_curCharPointer,__tr2qs("Unexpected end of buffer in class definition"));
+			delete pClass;
+			return 0;
+		}
+
+		if(KVSP_curCharUnicode != '{')
+		{
+			errorBadChar(KVSP_curCharPointer,'{',"class");
+			delete pClass;
+			return 0;
+		}
+
+		pBegin = KVSP_curCharPointer;
+		KviKvsTreeNodeInstruction * pInstruction = parseInstruction();
+		if(!pInstruction)
+		{
+			// may be an empty instruction
+			if(error())
+			{
+				delete pClass;
+				return 0;
+			}
+		}
+		delete pInstruction;
+		int iLen = KVSP_curCharPointer - pBegin;
+		QString szInstruction;
+		if(iLen > 0)
+		{
+			szInstruction = QString(pBegin,KVSP_curCharPointer - pBegin);
+			KviCommandFormatter::bufferFromBlock(szInstruction);
+		}
+
+		pClass->addFunctionDefinition(new KviKvsTreeNodeSpecialCommandClassFunctionDefinition(pLabelBegin,szLabel,szInstruction,uHandlerFlags));
+
+		if(!skipSpacesAndNewlines())
+		{
+			delete pClass;
+			return 0;
+		}
+	}
+
+	KVSP_skipChar;
+
+	return pClass;
+}
+
+
+KviKvsTreeNodeCommand * KviKvsParser::parseSpecialCommandWhile()
 {
 	/*
 		@doc: while
@@ -184,7 +631,7 @@ KviKvsTreeNodeSpecialCommand * KviKvsParser::parseSpecialCommandWhile()
 
 	if(KVSP_curCharUnicode != '(')
 	{
-		warning(KVSP_curCharPointer,__tr2qs("The while command needs an expression enclosed in parentheses"));
+		warning(KVSP_curCharPointer,__tr2qs("The while command needs an expression enclosed in parenthesis"));
 		error(KVSP_curCharPointer,__tr2qs("Found character %q (unicode %x) where an open parenthesis was expected"),KVSP_curCharPointer,KVSP_curCharUnicode);
 		return 0;
 	}
@@ -200,7 +647,11 @@ KviKvsTreeNodeSpecialCommand * KviKvsParser::parseSpecialCommandWhile()
 		return 0;
 	}
 
-	skipSpacesAndNewlines();
+	if(!skipSpacesAndNewlines())
+	{
+		delete e;
+		return 0;
+	}
 
 	if(KVSP_curCharUnicode == 0)
 	{
@@ -221,7 +672,7 @@ KviKvsTreeNodeSpecialCommand * KviKvsParser::parseSpecialCommandWhile()
 	return new KviKvsTreeNodeSpecialCommandWhile(pBegin,e,i);
 }
 
-KviKvsTreeNodeSpecialCommand * KviKvsParser::parseSpecialCommandDo()
+KviKvsTreeNodeCommand * KviKvsParser::parseSpecialCommandDo()
 {
 	/*
 		@doc: do
@@ -275,7 +726,11 @@ KviKvsTreeNodeSpecialCommand * KviKvsParser::parseSpecialCommandDo()
 		if(error())return 0;
 	}
 
-	skipSpacesAndNewlines();
+	if(!skipSpacesAndNewlines())
+	{
+		if(i)delete i;
+		return 0;
+	}
 
 	static const unsigned short while_chars[10] = { 'W','w','H','h','I','i','L','l','E','e' };
 
@@ -287,7 +742,7 @@ KviKvsTreeNodeSpecialCommand * KviKvsParser::parseSpecialCommandDo()
 			if(KVSP_curCharUnicode != while_chars[j])
 			{
 				if(KVSP_curCharIsEndOfBuffer)
-					error(KVSP_curCharPointer,__tr2qs("Unexpected end of command after the 'do' command block: expected 'while' keywoard"));
+					error(KVSP_curCharPointer,__tr2qs("Unexpected end of command after the 'do' command block: expected 'while' keyword"));
 				else
 					error(KVSP_curCharPointer,__tr2qs("Found character %q (unicode %x) where a 'while' keyword was expected"),KVSP_curCharPointer,KVSP_curCharUnicode);
 				if(i)delete i;
@@ -297,11 +752,15 @@ KviKvsTreeNodeSpecialCommand * KviKvsParser::parseSpecialCommandDo()
 		KVSP_skipChar;
 	}
 	
-	skipSpacesAndNewlines();
+	if(!skipSpacesAndNewlines())
+	{
+		if(i)delete i;
+		return 0;
+	}
 
 	if(KVSP_curCharUnicode != '(')
 	{
-		warning(KVSP_curCharPointer,__tr2qs("The 'while' block of the 'do' command needs an expression enclosed in parentheses"));
+		warning(KVSP_curCharPointer,__tr2qs("The 'while' block of the 'do' command needs an expression enclosed in parenthesis"));
 		errorBadChar(KVSP_curCharPointer,'(',"do");
 		if(i)delete i;
 		return 0;
@@ -333,7 +792,7 @@ KviKvsTreeNodeSpecialCommand * KviKvsParser::parseSpecialCommandDo()
 
 
 
-KviKvsTreeNodeSpecialCommand * KviKvsParser::parseSpecialCommandIf()
+KviKvsTreeNodeCommand * KviKvsParser::parseSpecialCommandIf()
 {
 	/*
 		@doc: if
@@ -367,7 +826,7 @@ KviKvsTreeNodeSpecialCommand * KviKvsParser::parseSpecialCommandIf()
 
 	if(KVSP_curCharUnicode != '(')
 	{
-		warning(KVSP_curCharPointer,__tr2qs("The 'if' command needs an expression enclosed in parentheses"));
+		warning(KVSP_curCharPointer,__tr2qs("The 'if' command needs an expression enclosed in parenthesis"));
 		errorBadChar(KVSP_curCharPointer,'(',"if");
 		return 0;
 	}
@@ -384,7 +843,11 @@ KviKvsTreeNodeSpecialCommand * KviKvsParser::parseSpecialCommandIf()
 		return 0;
 	}
 
-	skipSpacesAndNewlines();
+	if(!skipSpacesAndNewlines())
+	{
+		delete e;
+		return 0;
+	}
 
 	if(KVSP_curCharUnicode == 0)
 	{
@@ -402,7 +865,11 @@ KviKvsTreeNodeSpecialCommand * KviKvsParser::parseSpecialCommandIf()
 		}
 	} // else , just an empty instruction
 
-	skipSpacesAndNewlines();
+	if(!skipSpacesAndNewlines())
+	{
+		if(i)delete i;
+		return 0;
+	}
 
 	const QChar * pElse = KVSP_curCharPointer;
 
@@ -429,11 +896,36 @@ KviKvsTreeNodeSpecialCommand * KviKvsParser::parseSpecialCommandIf()
 	KVSP_skipChar;
 	if(KVSP_curCharIsLetterOrNumber)
 	{
+		if((KVSP_curCharUnicode == 'i') || (KVSP_curCharUnicode == 'I'))
+		{
+			KVSP_skipChar;
+			if((KVSP_curCharUnicode == 'f') || (KVSP_curCharUnicode == 'F'))
+			{
+				KVSP_skipChar;
+				if(!KVSP_curCharIsLetterOrNumber)
+				{
+					// this is an "elseif"
+					KVSP_backChar;
+					KVSP_backChar;
+					// point to if
+					goto handle_else_instruction;
+				}
+				KVSP_backChar;
+			}
+			KVSP_backChar;
+		}
+
 		KVSP_setCurCharPointer(pElse);
 		return new KviKvsTreeNodeSpecialCommandIf(pBegin,e,i,0);
 	}
 
-	skipSpacesAndNewlines();
+handle_else_instruction:
+	if(!skipSpacesAndNewlines())
+	{
+		delete e;
+		if(i)delete i;
+		return 0;
+	}
 
 	KviKvsTreeNodeInstruction * i2 = parseInstruction();
 	if(!i2)
@@ -492,7 +984,7 @@ bool KviKvsParser::skipToEndOfForControlBlock()
 	return false;
 }
 
-KviKvsTreeNodeSpecialCommand * KviKvsParser::parseSpecialCommandFor()
+KviKvsTreeNodeCommand * KviKvsParser::parseSpecialCommandFor()
 {
 	/*
 		@doc: for
@@ -524,7 +1016,7 @@ KviKvsTreeNodeSpecialCommand * KviKvsParser::parseSpecialCommandFor()
 
 	if(KVSP_curCharUnicode != '(')
 	{
-		warning(KVSP_curCharPointer,__tr2qs("The 'for' command needs an expression enclosed in parentheses"));
+		warning(KVSP_curCharPointer,__tr2qs("The 'for' command needs an expression enclosed in parenthesis"));
 		errorBadChar(KVSP_curCharPointer,'(',"for");
 		return 0;
 	}
@@ -610,7 +1102,13 @@ KviKvsTreeNodeSpecialCommand * KviKvsParser::parseSpecialCommandFor()
 
 	KVSP_skipChar;
 
-	skipSpacesAndNewlines();
+	if(!skipSpacesAndNewlines())
+	{
+		if(i1)delete i1;
+		if(e)delete e;
+		if(i2)delete i2;
+		return 0;
+	}
 
 	KviKvsTreeNodeInstruction * loop = parseInstruction();
 	if(!loop)
@@ -638,7 +1136,7 @@ KviKvsTreeNodeSpecialCommand * KviKvsParser::parseSpecialCommandFor()
 
 
 
-KviKvsTreeNodeSpecialCommand * KviKvsParser::parseSpecialCommandForeach()
+KviKvsTreeNodeCommand * KviKvsParser::parseSpecialCommandForeach()
 {
 	/*
 		@doc: foreach
@@ -647,9 +1145,12 @@ KviKvsTreeNodeSpecialCommand * KviKvsParser::parseSpecialCommandForeach()
 		@title:
 			foreach
 		@syntax:
-			foreach(<variable>,[<item>[,<item>[,<item>[...]]]) <command>
+			foreach [-a] (<variable>,[<item>[,<item>[,<item>[...]]]) <command>
 		@keyterms:
 			iteration commands, flow control commands
+		@switches:
+			!sw: -a | --all
+			Include empty variables in the iteration loop.
 		@short:
 			Iteration command
 		@description:
@@ -660,6 +1161,9 @@ KviKvsTreeNodeSpecialCommand * KviKvsParser::parseSpecialCommandForeach()
 			the iteration is done through all the dictionary/array items.[br]
 			Please note that the iteration order of dictionary items is undefined.[br]
 			You can always break from the loop by using the [cmd]break[/cmd] command.[br]
+			foreach doesn't iterate over empty scalar variables (i.e. the ones set to [fnc]$nothing[/fnc])
+			unless you use the -a switch. (Note that an array with *some* empty entries is NOT empty so
+			the iteration is in fact done).
 		@examples:
 			[example]
 				foreach(%i,1,2,3,4,5,6,7,8,9)[cmd]echo[/cmd] %i
@@ -678,7 +1182,7 @@ KviKvsTreeNodeSpecialCommand * KviKvsParser::parseSpecialCommandForeach()
 
 	if(KVSP_curCharUnicode != '(')
 	{
-		warning(KVSP_curCharPointer,__tr2qs("The 'foreach' command needs an expression enclosed in parentheses"));
+		warning(KVSP_curCharPointer,__tr2qs("The 'foreach' command needs an expression enclosed in parenthesis"));
 		errorBadChar(KVSP_curCharPointer,'(',"foreach");
 		return 0;
 	}
@@ -690,7 +1194,7 @@ KviKvsTreeNodeSpecialCommand * KviKvsParser::parseSpecialCommandForeach()
 	skipSpaces();
 
 
-	if((KVSP_curCharUnicode != '%') && (KVSP_curCharUnicode != '$'))
+	if((KVSP_curCharUnicode != '%') && (KVSP_curCharUnicode != '$') && (KVSP_curCharUnicode != '@'))
 	{
 		warning(KVSP_curCharPointer,__tr2qs("The 'foreach' command expects a writeable iteration variable as first parameter"));
 		error(KVSP_curCharPointer,__tr2qs("Found character '%q' (unicode %x) where '%' or '$' was expected: see /help foreach for the command syntax"),KVSP_curCharPointer,KVSP_curCharUnicode);
@@ -728,7 +1232,12 @@ KviKvsTreeNodeSpecialCommand * KviKvsParser::parseSpecialCommandForeach()
 	KviKvsTreeNodeDataList * l = parseCommaSeparatedParameterList();
 	if(!l)return 0;
 
-	skipSpacesAndNewlines();
+	if(!skipSpacesAndNewlines())
+	{
+		delete d;
+		delete l;
+		return 0;
+	}
 
 	const QChar * pLoopBegin = KVSP_curCharPointer;
 
@@ -736,17 +1245,15 @@ KviKvsTreeNodeSpecialCommand * KviKvsParser::parseSpecialCommandForeach()
 	if(!loop)
 	{
 		if(error())return 0;
-		error(pLoopBegin,__tr2qs("Empty 'foreach' execution block: fix the script"));
-		delete d;
-		delete l;
-		return 0;
+		warning(pLoopBegin,__tr2qs("Found empty 'foreach' execution block: maybe you need to fix your script ?"));
+		loop = new KviKvsTreeNodeInstructionBlock(pLoopBegin);
 	}
 
 	return new KviKvsTreeNodeSpecialCommandForeach(pForeachBegin,d,l,loop);
 }
 
 
-KviKvsTreeNodeSpecialCommand * KviKvsParser::parseSpecialCommandSwitch()
+KviKvsTreeNodeCommand * KviKvsParser::parseSpecialCommandSwitch()
 {
 	/*
 		@doc: switch
@@ -859,7 +1366,7 @@ KviKvsTreeNodeSpecialCommand * KviKvsParser::parseSpecialCommandSwitch()
 
 	if(KVSP_curCharUnicode != '(')
 	{
-		warning(KVSP_curCharPointer,__tr2qs("The 'switch' command needs an expression enclosed in parentheses"));
+		warning(KVSP_curCharPointer,__tr2qs("The 'switch' command needs an expression enclosed in parenthesis"));
 		errorBadChar(KVSP_curCharPointer,'(',"switch");
 		return 0;
 	}
@@ -875,7 +1382,11 @@ KviKvsTreeNodeSpecialCommand * KviKvsParser::parseSpecialCommandSwitch()
 		return 0;
 	}
 
-	skipSpacesAndNewlines();
+	if(!skipSpacesAndNewlines())
+	{
+		delete e;
+		return 0;
+	}
 
 	if(KVSP_curCharUnicode != '{')
 	{
@@ -886,9 +1397,13 @@ KviKvsTreeNodeSpecialCommand * KviKvsParser::parseSpecialCommandSwitch()
 
 	KVSP_skipChar;
 
-	KviKvsTreeNodeSpecialCommandSwitch * pSwitch = new KviKvsTreeNodeSpecialCommandSwitch(pBegin,e);
+	if(!skipSpacesAndNewlines())
+	{
+		delete e;
+		return 0;
+	}
 
-	skipSpacesAndNewlines();
+	KviKvsTreeNodeSpecialCommandSwitch * pSwitch = new KviKvsTreeNodeSpecialCommandSwitch(pBegin,e);
 
 	KviKvsTreeNodeSpecialCommandSwitchLabel * pLabel = 0;
 
@@ -938,7 +1453,12 @@ KviKvsTreeNodeSpecialCommand * KviKvsParser::parseSpecialCommandSwitch()
 				pLabel->setTerminatingBreak(true);
 				skipSpaces();
 				if(KVSP_curCharUnicode == ';')KVSP_skipChar;
-				skipSpacesAndNewlines();
+				if(!skipSpacesAndNewlines())
+				{
+					delete pSwitch;
+					delete pLabel;
+					return 0;
+				}
 				continue;
 			} else {
 				error(pLabelBegin,__tr2qs("Found 'break' label where a 'case','match','regexp' or 'default' label was expected"));
@@ -976,7 +1496,12 @@ KviKvsTreeNodeSpecialCommand * KviKvsParser::parseSpecialCommandSwitch()
 
 		skipSpaces();
 		if(KVSP_curCharUnicode == ':')KVSP_skipChar;
-		skipSpacesAndNewlines();
+		if(!skipSpacesAndNewlines())
+		{
+			delete pSwitch;
+			delete pLabel;
+			return 0;
+		}
 
 		KviKvsTreeNodeInstruction * pInstruction = parseInstruction();
 		if(!pInstruction)
@@ -993,7 +1518,11 @@ KviKvsTreeNodeSpecialCommand * KviKvsParser::parseSpecialCommandSwitch()
 		pLabel->setInstruction(pInstruction);
 		pSwitch->addLabel(pLabel);
 
-		skipSpacesAndNewlines();
+		if(!skipSpacesAndNewlines())
+		{
+			delete pSwitch;
+			return 0;
+		}
 	}
 
 	KVSP_skipChar;
@@ -1020,7 +1549,11 @@ KviKvsTreeNodeSpecialCommandDefpopupLabelPopup * KviKvsParser::parseSpecialComma
 
 	KVSP_skipChar;
 	
-	skipSpacesAndNewlines();
+	if(!skipSpacesAndNewlines())
+	{
+		delete pPopup;
+		return 0;
+	}
 
 	while(KVSP_curCharUnicode != '}')
 	{
@@ -1049,46 +1582,29 @@ KviKvsTreeNodeSpecialCommandDefpopupLabelPopup * KviKvsParser::parseSpecialComma
 		
 		QString szCondition;
 
-		if((szLabelLow == "prologue") || (szLabelLow == "epilogue"))
-		{
-			/////////////////////////////////////////////////////////////////////////////////////////////////
-			bool bPrologue = (szLabelLow == "prologue");
-			skipSpacesAndNewlines();
-			const QChar * pBegin = KVSP_curCharPointer;
-			KviKvsTreeNodeInstruction * pInstruction = parseInstruction();
-			if(!pInstruction)
-			{
-				// may be an empty instruction
-				if(!error())
-				{
-					// senseless empty instruction
-					if(bPrologue)
-						error(pLabelBegin,__tr2qs("Senseless empty prologue block: fix the script"));
-					else
-						error(pLabelBegin,__tr2qs("Senseless empty epilogue block: fix the script"));
-				} // else the error has been already triggered
-				delete pPopup;
-				return 0;
-			}
-			delete pInstruction;
-			int iLen = KVSP_curCharPointer - pBegin;
-			if(iLen > 0)
-			{
-				QString szInstruction(pBegin,KVSP_curCharPointer - pBegin);
-				KviCommandFormatter::bufferFromBlock(szInstruction);
-				if(bPrologue)
-					pPopup->addLabel(new KviKvsTreeNodeSpecialCommandDefpopupLabelPrologue(pLabelBegin,szInstruction));
-				else
-					pPopup->addLabel(new KviKvsTreeNodeSpecialCommandDefpopupLabelEpilogue(pLabelBegin,szInstruction));
-			} else {
-				error(KVSP_curCharPointer,__tr2qs("defpopup: internal error"));
-				return 0;
-			}
-		} else if(szLabelLow == "separator")
-		{
-			/////////////////////////////////////////////////////////////////////////////////////////////////
+
+#define EXTRACT_POPUP_LABEL_PARAMETERS \
+			if(!skipSpacesAndNewlines()) \
+			{ \
+				delete pPopup; \
+				return 0; \
+			} \
+			if(KVSP_curCharUnicode != '(') \
+			{ \
+				errorBadChar(KVSP_curCharPointer,'(',"defpopup"); \
+				delete pPopup; \
+				return 0; \
+			} \
+			pParameters = parseCommaSeparatedParameterListNoTree(); \
+			if(!pParameters)return 0;
+
+
 #define EXTRACT_POPUP_LABEL_CONDITION \
-			skipSpacesAndNewlines(); \
+			if(!skipSpacesAndNewlines()) \
+			{ \
+				delete pPopup; \
+				return 0; \
+			} \
 			if(KVSP_curCharUnicode == '(') \
 			{ \
 				const QChar * pBegin = KVSP_curCharPointer; \
@@ -1106,27 +1622,82 @@ KviKvsTreeNodeSpecialCommandDefpopupLabelPopup * KviKvsParser::parseSpecialComma
 					szCondition.setUnicode(pBegin + 1,cLen); \
 				} \
 				delete pExpression; \
-				skipSpacesAndNewlines(); \
+				if(!skipSpacesAndNewlines()) \
+				{ \
+					if(pParameters)delete pParameters; \
+					delete pPopup; \
+					return 0; \
+				} \
 			}
 
+
+
+		if((szLabelLow == "prologue") || (szLabelLow == "epilogue"))
+		{
+			/////////////////////////////////////////////////////////////////////////////////////////////////
+			bool bPrologue = (szLabelLow == "prologue");
+			if(!skipSpacesAndNewlines())
+			{
+				delete pPopup;
+				return 0;
+			}
+
+			if(KVSP_curCharUnicode == '(')
+			{
+				EXTRACT_POPUP_LABEL_PARAMETERS
+				if(!skipSpacesAndNewlines())
+				{
+					if(pParameters)delete pParameters;
+					delete pPopup;
+					return 0;
+				}
+			}
+			const QChar * pBegin = KVSP_curCharPointer;
+			KviKvsTreeNodeInstruction * pInstruction = parseInstruction();
+			if(pInstruction)
+			{
+				// in fact we don't need it at all, we just need the code buffer...
+				delete pInstruction;
+			} else {
+				// may be an empty instruction
+				if(error())
+				{
+					// error
+					if(pParameters)delete pParameters;
+					delete pPopup;
+					return 0;
+				}
+				// empty instruction
+				if(bPrologue)
+					warning(pBegin,__tr2qs("Found empty prologue block: maybe you need to fix the script?"));
+				else
+					warning(pBegin,__tr2qs("Found empty epilogue block: maybe you need to fix the script?"));
+			}
+			int iLen = KVSP_curCharPointer - pBegin;
+			if(iLen > 0)
+			{
+				QString szInstruction(pBegin,KVSP_curCharPointer - pBegin);
+				KviCommandFormatter::bufferFromBlock(szInstruction);
+				QString * pItemName = pParameters ? pParameters->first() : 0;
+				QString szItemName = pItemName ? *pItemName : QString::null;
+				if(bPrologue)
+					pPopup->addLabel(new KviKvsTreeNodeSpecialCommandDefpopupLabelPrologue(pLabelBegin,szInstruction,szItemName));
+				else
+					pPopup->addLabel(new KviKvsTreeNodeSpecialCommandDefpopupLabelEpilogue(pLabelBegin,szInstruction,szItemName));
+			} // else the instruction was empty anyway: we don't need it in fact
+			if(pParameters)delete pParameters;
+		} else if(szLabelLow == "separator")
+		{
+			// FIXME: Separators can't have labels here :(((((
+			/////////////////////////////////////////////////////////////////////////////////////////////////
 			EXTRACT_POPUP_LABEL_CONDITION
 			if(KVSP_curCharUnicode == ';')KVSP_skipChar;
-			pPopup->addLabel(new KviKvsTreeNodeSpecialCommandDefpopupLabelSeparator(pLabelBegin,szCondition));
+			QString szItemName = "dummySeparator"; // <------- FIXME!
+			pPopup->addLabel(new KviKvsTreeNodeSpecialCommandDefpopupLabelSeparator(pLabelBegin,szCondition,szItemName));
 
 		} else if(szLabelLow == "label")
 		{
 			/////////////////////////////////////////////////////////////////////////////////////////////////
-#define EXTRACT_POPUP_LABEL_PARAMETERS \
-			skipSpacesAndNewlines(); \
-			if(KVSP_curCharUnicode != '(') \
-			{ \
-				errorBadChar(KVSP_curCharPointer,'(',"defpopup"); \
-				delete pPopup; \
-				return 0; \
-			} \
-			pParameters = parseCommaSeparatedParameterListNoTree(); \
-			if(!pParameters)return 0;
-
 			EXTRACT_POPUP_LABEL_PARAMETERS
 			EXTRACT_POPUP_LABEL_CONDITION
 
@@ -1140,7 +1711,8 @@ KviKvsTreeNodeSpecialCommandDefpopupLabelPopup * KviKvsParser::parseSpecialComma
 			}
 			QString * pIcon = pParameters->next();
 			if(KVSP_curCharUnicode == ';')KVSP_skipChar;
-			pPopup->addLabel(new KviKvsTreeNodeSpecialCommandDefpopupLabelLabel(pLabelBegin,szCondition,*pText,pIcon ? *pIcon : QString::null));
+			QString * pItemName = pParameters->next();
+			pPopup->addLabel(new KviKvsTreeNodeSpecialCommandDefpopupLabelLabel(pLabelBegin,szCondition,*pText,pIcon ? *pIcon : QString::null,pItemName ? *pItemName : QString::null));
 			delete pParameters;
 		} else if(szLabelLow == "popup")
 		{
@@ -1157,6 +1729,7 @@ KviKvsTreeNodeSpecialCommandDefpopupLabelPopup * KviKvsParser::parseSpecialComma
 				return 0;
 			}
 			QString * pIcon = pParameters->next();
+			QString * pItemName = pParameters->next();
 
 			KviKvsTreeNodeSpecialCommandDefpopupLabelPopup * pSubPopup = parseSpecialCommandDefpopupLabelPopup();
 			if(!pSubPopup)
@@ -1168,6 +1741,7 @@ KviKvsTreeNodeSpecialCommandDefpopupLabelPopup * KviKvsParser::parseSpecialComma
 			
 			pSubPopup->setCondition(szCondition);
 			pSubPopup->setText(*pText);
+			pSubPopup->setItemName(pItemName ? *pItemName : QString::null);
 			if(pIcon)pSubPopup->setIcon(*pIcon);
 			pPopup->addLabel(pSubPopup);
 			delete pParameters;
@@ -1186,28 +1760,37 @@ KviKvsTreeNodeSpecialCommandDefpopupLabelPopup * KviKvsParser::parseSpecialComma
 				return 0;
 			}
 			QString * pIcon = pParameters->next();
+			QString * pItemName = pParameters->next();
 
 			const QChar * pBegin = KVSP_curCharPointer;
 			KviKvsTreeNodeInstruction * pInstruction = parseInstruction();
-			if(!pInstruction)
+			if(pInstruction)
 			{
-				// senseless empty instruction ?
-				error(pLabelBegin,__tr2qs("Senseless empty instruction for popup item: fix the script"));
-				delete pParameters;
-				delete pPopup;
-				return 0;
+				// in fact we don't need it: we just need the code block
+				delete pInstruction;
+			} else {
+				// empty instruction or error ?
+				if(error())
+				{
+					// error
+					delete pParameters;
+					delete pPopup;
+					return 0;
+				}
+				// empty instruction
+				warning(pBegin,__tr2qs("Found empty instruction for popup item: maybe you need to fix the script?"));
 			}
-			delete pInstruction;
 			int iLen = KVSP_curCharPointer - pBegin;
 			if(iLen > 0)
 			{
 				QString szInstruction(pBegin,KVSP_curCharPointer - pBegin);
 				KviCommandFormatter::bufferFromBlock(szInstruction);
-				pPopup->addLabel(new KviKvsTreeNodeSpecialCommandDefpopupLabelItem(pLabelBegin,szCondition,*pText,pIcon ? *pIcon : QString::null,szInstruction));
+				pPopup->addLabel(new KviKvsTreeNodeSpecialCommandDefpopupLabelItem(pLabelBegin,szCondition,*pText,pIcon ? *pIcon : QString::null,szInstruction,pItemName ? *pItemName : QString::null));
 			} else {
-				error(KVSP_curCharPointer,__tr2qs("defpopup: internal error"));
-				delete pParameters;
-				return 0;
+				// zero length instruction, but still add the item
+				QString szInstruction = "";
+				KviCommandFormatter::bufferFromBlock(szInstruction);
+				pPopup->addLabel(new KviKvsTreeNodeSpecialCommandDefpopupLabelItem(pLabelBegin,szCondition,*pText,pIcon ? *pIcon : QString::null,szInstruction,pItemName ? *pItemName : QString::null));
 			}
 			delete pParameters;
 		} else if(szLabelLow == "extpopup")
@@ -1233,8 +1816,9 @@ KviKvsTreeNodeSpecialCommandDefpopupLabelPopup * KviKvsParser::parseSpecialComma
 				return 0;
 			}
 			QString * pIcon = pParameters->next();
+			QString * pItemName = pParameters->next();
 			if(KVSP_curCharUnicode == ';')KVSP_skipChar;
-			pPopup->addLabel(new KviKvsTreeNodeSpecialCommandDefpopupLabelExtpopup(pLabelBegin,szCondition,*pText,pIcon ? *pIcon : QString::null,*pName));
+			pPopup->addLabel(new KviKvsTreeNodeSpecialCommandDefpopupLabelExtpopup(pLabelBegin,szCondition,*pText,pIcon ? *pIcon : QString::null,*pName,pItemName ? *pItemName : QString::null));
 			delete pParameters;
 		} else {
 			/////////////////////////////////////////////////////////////////////////////////////////////////
@@ -1243,7 +1827,11 @@ KviKvsTreeNodeSpecialCommandDefpopupLabelPopup * KviKvsParser::parseSpecialComma
 			return 0;
 		}
 
-		skipSpacesAndNewlines();
+		if(!skipSpacesAndNewlines())
+		{
+			delete pPopup;
+			return 0;
+		}
 	}
 
 	KVSP_skipChar;
@@ -1251,7 +1839,7 @@ KviKvsTreeNodeSpecialCommandDefpopupLabelPopup * KviKvsParser::parseSpecialComma
 }
 
 
-KviKvsTreeNodeSpecialCommand * KviKvsParser::parseSpecialCommandDefpopup()
+KviKvsTreeNodeCommand * KviKvsParser::parseSpecialCommandDefpopup()
 {
 	// FIXME: This SHOULD be renamed to popup.create (NOT popup.define!)
 	//        and internally aliased to defpopup as backward compat
@@ -1266,20 +1854,20 @@ KviKvsTreeNodeSpecialCommand * KviKvsParser::parseSpecialCommandDefpopup()
 		@syntax:
 			defpopup [-m] (<popup_name>)
 			{
-				prologue <prologue_command>
+				prologue[(<id>)] <prologue_command>
 
-				epilogue <epilogue_command>
+				epilogue[(<id>)] <epilogue_command>
 
-				label(<text>)[(<expression>)][;]
+				label(<text>[,<id>])[(<expression>)][;]
 
-				item(<text>[,<icon>])[(<expression>)]<command>
+				item(<text>[,<icon>[,<id>]])[(<expression>)]<command>
 
-				popup(<text>[,<icon>])[(<expression>)]
+				popup(<text>[,<icon>[,<id>]])[(<expression>)]
 				{
 					<popup body>
 				}
 
-				extpopup(<text>,<name>[,<icon>])[(<expression>)][;]
+				extpopup(<text>,<name>[,<icon>[,<id>]])[(<expression>)][;]
 
 				separator[(<expression>)][;]
 				...
@@ -1287,7 +1875,7 @@ KviKvsTreeNodeSpecialCommand * KviKvsParser::parseSpecialCommandDefpopup()
 		@short:
 			Defines a popup menu
 		@switches:
-			!sw: -m
+			!sw: -m | --merge
 			Merges the new popup contents with an older popup version
 		@description:
 			Defines the popup menu <popup_name>. If the -m switch is NOT used
@@ -1317,6 +1905,9 @@ KviKvsTreeNodeSpecialCommand * KviKvsParser::parseSpecialCommandDefpopup()
 			There can be multiple prologue and epilogue commands: their execution order
 			is undefined.[br]
 			<icon> is always an [doc:image_id]image identifier[/doc].[br]
+			<id> is an unique identifier that can be used to remove single items
+			by the means of [cmd]delpopupitem[/cmd]. If <id> is omitted
+			then it is automatically generated.
 			Please note that using this command inside the prologue , epilogue
 			or item code of the modified popup menu is forbidden.
 			In other words: self modification of popup menus is NOT allowed.[br]
@@ -1341,7 +1932,7 @@ KviKvsTreeNodeSpecialCommand * KviKvsParser::parseSpecialCommandDefpopup()
 
 	if(KVSP_curCharUnicode != '(')
 	{
-		warning(KVSP_curCharPointer,__tr2qs("The 'defpopup' command needs an expression enclosed in parentheses"));
+		warning(KVSP_curCharPointer,__tr2qs("The 'defpopup' command needs an expression enclosed in parenthesis"));
 		errorBadChar(KVSP_curCharPointer,'(',"defpopup");
 		return 0;
 	}
@@ -1353,7 +1944,11 @@ KviKvsTreeNodeSpecialCommand * KviKvsParser::parseSpecialCommandDefpopup()
 	KviKvsTreeNodeData * pPopupName = parseSingleParameterInParenthesis();
 	if(!pPopupName)return 0;
 
-	skipSpacesAndNewlines();
+	if(!skipSpacesAndNewlines())
+	{
+		delete pPopupName;
+		return 0;
+	}
 
 	KviKvsTreeNodeSpecialCommandDefpopupLabelPopup * pMainPopup = parseSpecialCommandDefpopupLabelPopup();
 	if(!pMainPopup)
@@ -1365,3 +1960,31 @@ KviKvsTreeNodeSpecialCommand * KviKvsParser::parseSpecialCommandDefpopup()
 	return new KviKvsTreeNodeSpecialCommandDefpopup(pBegin,pPopupName,pMainPopup);
 }
 
+
+KviKvsTreeNodeCommand * KviKvsParser::parseSpecialCommandHelp()
+{
+	// again not a fully special command: this routine just returns
+	// a CoreSimpleCommand but parses the parameters as constants
+	
+	// we handle a single big parameter, with whitespace stripped
+
+	skipSpaces();
+
+	const QChar * pBegin = KVSP_curCharPointer;
+	while(!KVSP_curCharIsEndOfCommand)KVSP_skipChar;
+	
+	if(!KVSP_curCharIsEndOfBuffer)KVSP_skipChar;
+	
+	QString tmp(pBegin,KVSP_curCharPointer - pBegin);
+	tmp.stripWhiteSpace();
+
+	const QString szHelpName("help");
+
+	KviKvsCoreSimpleCommandExecRoutine * r = KviKvsKernel::instance()->findCoreSimpleCommandExecRoutine(szHelpName);
+	if(!r)return 0; // <--- internal error!
+
+	KviKvsTreeNodeDataList * p = new KviKvsTreeNodeDataList(pBegin);
+	p->addItem(new KviKvsTreeNodeConstantData(pBegin,new KviKvsVariant(tmp)));
+	
+	return new KviKvsTreeNodeCoreSimpleCommand(pBegin,szHelpName,p,r);
+}
