@@ -33,6 +33,7 @@
 #include "kvi_console.h"
 #include "kvi_config.h"
 #include "kvi_cmdformatter.h"
+#include "kvi_options.h"
 
 // popup names
 
@@ -55,10 +56,24 @@ KviKvsPopupMenuItem::KviKvsPopupMenuItem(Type t,const QString &szItemName,const
 	} else {
 		QString szName = "condition callback for ";
 		szName += szItemName;
-		m_pKvsCondition = new KviKvsScript(szName,new QString(szCondition),KviKvsScript::Expression);
+		m_pKvsCondition = new KviKvsScript(szName,szCondition,KviKvsScript::Expression);
 	}
 }
 
+KviKvsPopupMenuItem::KviKvsPopupMenuItem(Type t,const QString &szItemName,const KviKvsScript * pCondition)
+{
+	m_szItemName = szItemName;
+	m_eType = t;
+	if(!pCondition)
+	{
+		// true by default
+		m_pKvsCondition = 0;
+	} else {
+		m_pKvsCondition = new KviKvsScript(*pCondition);
+	}
+}
+
+
 KviKvsPopupMenuItem::~KviKvsPopupMenuItem()
 {
 	if(m_pKvsCondition)delete m_pKvsCondition;
@@ -109,6 +124,11 @@ KviKvsPopupMenuItemSeparator::KviKvsPopupMenuItemSeparator(const QString &szItem
 {
 }
 
+KviKvsPopupMenuItemSeparator::KviKvsPopupMenuItemSeparator(const QString &szItemName,const KviKvsScript * pCondition)
+: KviKvsPopupMenuItem(KviKvsPopupMenuItem::Separator,szItemName,pCondition)
+{
+}
+
 KviKvsPopupMenuItemSeparator::~KviKvsPopupMenuItemSeparator()
 {
 }
@@ -121,7 +141,7 @@ void KviKvsPopupMenuItemSeparator::fill(KviKvsPopupMenu * pMenu,KviKvsPopupMenuT
 
 KviKvsPopupMenuItem * KviKvsPopupMenuItemSeparator::clone() const
 {
-	return new KviKvsPopupMenuItemSeparator(m_szItemName,m_pKvsCondition->code());
+	return new KviKvsPopupMenuItemSeparator(m_szItemName,m_pKvsCondition);
 }
 
 
@@ -133,7 +153,7 @@ KviKvsPopupMenuItemWithTextAndIcon::KviKvsPopupMenuItemWithTextAndIcon(KviKvsPop
 {
 	QString szName = "text callback for ";
 	szName += szItemName;
-	m_pKvsText = new KviKvsScript(szName,new QString(szText),KviKvsScript::Parameter);
+	m_pKvsText = new KviKvsScript(szName,szText,KviKvsScript::Parameter);
 
 	if(szIcon.isEmpty())
 	{
@@ -141,10 +161,32 @@ KviKvsPopupMenuItemWithTextAndIcon::KviKvsPopupMenuItemWithTextAndIcon(KviKvsPop
 	} else {
 		szName = "icon callback for ";
 		szName += szItemName;
-		m_pKvsIcon = new KviKvsScript(szName,new QString(szIcon),KviKvsScript::Parameter);
+		m_pKvsIcon = new KviKvsScript(szName,szIcon,KviKvsScript::Parameter);
 	}
 }
 
+KviKvsPopupMenuItemWithTextAndIcon::KviKvsPopupMenuItemWithTextAndIcon(KviKvsPopupMenuItem::Type t,const QString &szItemName,const KviKvsScript * pText,const KviKvsScript * pIcon,const KviKvsScript * pCondition)
+: KviKvsPopupMenuItem(t,szItemName,pCondition)
+{
+	if(pText)
+	{
+		m_pKvsText = new KviKvsScript(*pText);
+	} else {
+		// hum.. this should never happen anyway
+		QString szName = "text callback for ";
+		szName += szItemName;
+		m_pKvsText = new KviKvsScript(szName,"",KviKvsScript::Parameter);
+	}
+
+	if(!pIcon)
+	{
+		m_pKvsIcon = 0;
+	} else {
+		m_pKvsIcon = new KviKvsScript(*pIcon);
+	}
+}
+
+
 KviKvsPopupMenuItemWithTextAndIcon::~KviKvsPopupMenuItemWithTextAndIcon()
 {
 	delete m_pKvsText;
@@ -164,6 +206,7 @@ KviKvsScript * KviKvsPopupMenuItemWithTextAndIcon::kvsText()
 
 QPixmap * KviKvsPopupMenuItemWithTextAndIcon::evaluateIcon(KviKvsPopupMenuTopLevelData * pData)
 {
+	if(KVI_OPTION_BOOL(KviOption_boolDisablePopupIcons))return 0;
 	if(!m_pKvsIcon)return 0;
 
 	KviKvsVariant vRet;
@@ -207,20 +250,66 @@ QString KviKvsPopupMenuItemWithTextAndIcon::evaluateText(KviKvsPopupMenuTopLevel
 
 //////////////////////////////////////////////////////////////////////////////////////////////////////////////////
 
+
+KviKvsPopupMenuItemLabelHelper::KviKvsPopupMenuItemLabelHelper(KviKvsPopupMenuItemLabel * pItem)
+: QObject()
+{
+	m_pItem = pItem;
+}
+
+KviKvsPopupMenuItemLabelHelper::~KviKvsPopupMenuItemLabelHelper()
+{
+}
+
+void KviKvsPopupMenuItemLabelHelper::labelDestroyed()
+{
+	m_pItem->labelDestroyed();
+}
+
 KviKvsPopupMenuItemLabel::KviKvsPopupMenuItemLabel(const QString &szItemName,const QString &szText,const QString &szIcon,const QString &szCondition)
 : KviKvsPopupMenuItemWithTextAndIcon(KviKvsPopupMenuItem::Label,szItemName,szText,szIcon,szCondition)
 {
 	m_pLabel = 0;
+	m_pSignalRelay = new KviKvsPopupMenuItemLabelHelper(this);
 }
 
+KviKvsPopupMenuItemLabel::KviKvsPopupMenuItemLabel(const QString &szItemName,const KviKvsScript * pText,const KviKvsScript * pIcon,const KviKvsScript * pCondition)
+: KviKvsPopupMenuItemWithTextAndIcon(KviKvsPopupMenuItem::Label,szItemName,pText,pIcon,pCondition)
+{
+	m_pLabel = 0;
+	m_pSignalRelay = new KviKvsPopupMenuItemLabelHelper(this);
+}
+
+
 KviKvsPopupMenuItemLabel::~KviKvsPopupMenuItemLabel()
 {
-	if(m_pLabel)delete m_pLabel;
+	if(m_pLabel)
+	{
+		QObject::disconnect(m_pLabel,SIGNAL(destroyed()),m_pSignalRelay,SLOT(labelDestroyed()));
+		delete m_pLabel;
+	}
+	delete m_pSignalRelay;
 }
 
+void KviKvsPopupMenuItemLabel::labelDestroyed()
+{
+	m_pLabel = 0; // someone (maybe qt, maybe us) has destroyed the label
+}
+
+
 KviKvsPopupMenuItem * KviKvsPopupMenuItemLabel::clone() const
 {
-	return new KviKvsPopupMenuItemLabel(m_szItemName,m_pKvsText->code(),m_pKvsIcon ? m_pKvsIcon->code() : QString::null,m_pKvsCondition->code());
+	return new KviKvsPopupMenuItemLabel(m_szItemName,m_pKvsText,m_pKvsIcon,m_pKvsCondition);
+}
+
+void KviKvsPopupMenuItemLabel::clear()
+{
+	if(m_pLabel)
+	{
+		QObject::disconnect(m_pLabel,SIGNAL(destroyed()),m_pSignalRelay,SLOT(labelDestroyed()));
+		delete m_pLabel;
+		m_pLabel = 0;
+	}
 }
 
 void KviKvsPopupMenuItemLabel::fill(KviKvsPopupMenu * pMenu,KviKvsPopupMenuTopLevelData * pData,int iIdx)
@@ -228,8 +317,13 @@ void KviKvsPopupMenuItemLabel::fill(KviKvsPopupMenu * pMenu,KviKvsPopupMenuTopLe
 	if(!evaluateCondition(pData))return;
 	QString szText = evaluateText(pData);
 	QPixmap * pPix = evaluateIcon(pData);
-	if(m_pLabel)delete m_pLabel;
+	if(m_pLabel)
+	{
+		QObject::disconnect(m_pLabel,SIGNAL(destroyed()),m_pSignalRelay,SLOT(labelDestroyed()));
+		delete m_pLabel;
+	}
 	m_pLabel = new QLabel(szText,pMenu);
+	QObject::connect(m_pLabel,SIGNAL(destroyed()),m_pSignalRelay,SLOT(labelDestroyed()));
 	pMenu->insertItem(m_pLabel);
 	m_pLabel->setFrameStyle(QFrame::StyledPanel | QFrame::Raised);
 	if(pPix)m_pLabel->setPixmap(*pPix);
@@ -243,9 +337,16 @@ KviKvsPopupMenuItemItem::KviKvsPopupMenuItemItem(const QString &szItemName,const
 {
 	QString szName = "click callback for ";
 	szName += szItemName;
-	m_pKvsCode = new KviKvsScript(szName,new QString(szCode));
+	m_pKvsCode = new KviKvsScript(szName,szCode);
 }
 
+KviKvsPopupMenuItemItem::KviKvsPopupMenuItemItem(const QString &szItemName,const KviKvsScript * pCode,const KviKvsScript * pText,const KviKvsScript * pIcon,const KviKvsScript * pCondition)
+: KviKvsPopupMenuItemWithTextAndIcon(KviKvsPopupMenuItem::Item,szItemName,pText,pIcon,pCondition)
+{
+	m_pKvsCode = new KviKvsScript(*pCode);
+}
+
+
 KviKvsPopupMenuItemItem::~KviKvsPopupMenuItemItem()
 {
 	delete m_pKvsCode;
@@ -264,7 +365,7 @@ void KviKvsPopupMenuItemItem::fill(KviKvsPopupMenu * pMenu,KviKvsPopupMenuTopLev
 
 KviKvsPopupMenuItem * KviKvsPopupMenuItemItem::clone() const
 {
-	return new KviKvsPopupMenuItemItem(m_szItemName,m_pKvsCode->code(),m_pKvsText->code(),m_pKvsIcon ? m_pKvsIcon->code() : QString::null,m_pKvsCondition->code());
+	return new KviKvsPopupMenuItemItem(m_szItemName,m_pKvsCode,m_pKvsText,m_pKvsIcon,m_pKvsCondition);
 }
 
 KviKvsScript * KviKvsPopupMenuItemItem::kvsCode()
@@ -281,6 +382,13 @@ KviKvsPopupMenuItemMenu::KviKvsPopupMenuItemMenu(const QString &szItemName,KviKv
 	m_pMenu = pMenu;
 }
 
+KviKvsPopupMenuItemMenu::KviKvsPopupMenuItemMenu(const QString &szItemName,KviKvsPopupMenu * pMenu,const KviKvsScript * pText,const KviKvsScript * pIcon,const KviKvsScript * pCondition)
+: KviKvsPopupMenuItemWithTextAndIcon(KviKvsPopupMenuItem::Menu,szItemName,pText,pIcon,pCondition)
+{
+	m_pMenu = pMenu;
+}
+
+
 KviKvsPopupMenuItemMenu::~KviKvsPopupMenuItemMenu()
 {
 	delete m_pMenu;
@@ -290,7 +398,7 @@ KviKvsPopupMenuItem * KviKvsPopupMenuItemMenu::clone() const
 {
 	KviKvsPopupMenu * copy = new KviKvsPopupMenu(m_pMenu->name());
 	copy->copyFrom(m_pMenu);
-	return new KviKvsPopupMenuItemMenu(m_szItemName,copy,m_pKvsText->code(),m_pKvsIcon ? m_pKvsIcon->code() : QString::null,m_pKvsCondition->code());
+	return new KviKvsPopupMenuItemMenu(m_szItemName,copy,m_pKvsText,m_pKvsIcon,m_pKvsCondition);
 }
 
 
@@ -300,6 +408,7 @@ void KviKvsPopupMenuItemMenu::fill(KviKvsPopupMenu * pMenu,KviKvsPopupMenuTopLev
 	QString szText = evaluateText(pData);
 	QPixmap * pPix = evaluateIcon(pData);
 	int id;
+	m_pMenu->setParentPopup(pMenu);
 	if(pPix)id = pMenu->insertItem(*pPix,szText,m_pMenu);
 	else id = pMenu->insertItem(szText,m_pMenu);
 	pMenu->setItemParameter(id,iIdx);
@@ -319,9 +428,27 @@ KviKvsPopupMenuItemExtMenu::KviKvsPopupMenuItemExtMenu(const QString &szItemName
 : KviKvsPopupMenuItemWithTextAndIcon(KviKvsPopupMenuItem::ExtMenu,szItemName,szText,szIcon,szCondition)
 {
 	m_szMenuName = szMenuName;
+	if(m_szMenuName[0]=='"' && m_szMenuName[(int)(m_szMenuName.length()-1)]=='"')
+	{
+		m_szMenuName.remove(0,1);
+		m_szMenuName.remove(m_szMenuName.length()-1,1);
+	}
 	m_pMenu = 0;
 }
 
+KviKvsPopupMenuItemExtMenu::KviKvsPopupMenuItemExtMenu(const QString &szItemName,const QString &szMenuName,const KviKvsScript * pText,const KviKvsScript * pIcon,const KviKvsScript * pCondition)
+: KviKvsPopupMenuItemWithTextAndIcon(KviKvsPopupMenuItem::ExtMenu,szItemName,pText,pIcon,pCondition)
+{
+	m_szMenuName = szMenuName;
+	if(m_szMenuName[0]=='"' && m_szMenuName[(int)(m_szMenuName.length()-1)]=='"')
+	{
+		m_szMenuName.remove(0,1);
+		m_szMenuName.remove(m_szMenuName.length()-1,1);
+	}
+	m_pMenu = 0;
+}
+
+
 KviKvsPopupMenuItemExtMenu::~KviKvsPopupMenuItemExtMenu()
 {
 	if(m_pMenu)delete m_pMenu;
@@ -338,7 +465,7 @@ void KviKvsPopupMenuItemExtMenu::clear()
 
 KviKvsPopupMenuItem * KviKvsPopupMenuItemExtMenu::clone() const
 {
-	return new KviKvsPopupMenuItemExtMenu(m_szItemName,m_szMenuName,m_pKvsText->code(),m_pKvsIcon ? m_pKvsIcon->code() : QString::null,m_pKvsCondition->code());
+	return new KviKvsPopupMenuItemExtMenu(m_szItemName,m_szMenuName,m_pKvsText,m_pKvsIcon,m_pKvsCondition);
 }
 
 
@@ -378,9 +505,11 @@ void KviKvsPopupMenuItemExtMenu::fill(KviKvsPopupMenu * pMenu,KviKvsPopupMenuTop
 
 KviKvsPopupMenuTopLevelData::KviKvsPopupMenuTopLevelData(KviKvsVariantList * pParameters,KviWindow * pWindow)
 {
-	m_pExtendedRunTimeData = new KviKvsExtendedRunTimeData(new KviKvsHash());
+	m_pExtendedRunTimeData = new KviKvsExtendedRunTimeData(new KviKvsHash(),TRUE);
 	m_pParameters = pParameters;
 	m_pWindow = pWindow;
+	m_bTestMode = false;
+	m_bLocked = false;
 }
 
 KviKvsPopupMenuTopLevelData::~KviKvsPopupMenuTopLevelData()
@@ -416,6 +545,7 @@ KviKvsPopupMenu::KviKvsPopupMenu(const QString &szName)
 
 KviKvsPopupMenu::~KviKvsPopupMenu()
 {
+	clearMenuContents();
 	delete m_pItemList;
 	delete m_pPrologues;
 	delete m_pEpilogues;
@@ -445,20 +575,22 @@ void KviKvsPopupMenu::copyFrom(const KviKvsPopupMenu * src)
 	}
 }
 
-void KviKvsPopupMenu::addPrologue(const QString &szCode)
+void KviKvsPopupMenu::addPrologue(const QString &szItemName,const QString &szCode)
 {
 	// FIXME: translate this or not ?
-	QString szName = "prologue for ";
-	szName += m_szName;
-	m_pPrologues->append(new KviKvsScript(szName,new QString(szCode)));
+	QString szName = szItemName;
+	if(szName.isEmpty())
+		KviQString::sprintf(szName,"prologue_%u_for_%Q",m_pPrologues->count(),&m_szName);
+	m_pPrologues->append(new KviKvsScript(szName,szCode));
 }
 
-void KviKvsPopupMenu::addEpilogue(const QString &szCode)
+void KviKvsPopupMenu::addEpilogue(const QString &szItemName,const QString &szCode)
 {
 	// FIXME: translate this or not ?
-	QString szName = "epilogue for ";
-	szName += m_szName;
-	m_pEpilogues->append(new KviKvsScript(szName,new QString(szCode)));
+	QString szName = szItemName;
+	if(szName.isEmpty())
+		KviQString::sprintf(szName,"epilogue_%u_for_%Q",m_pPrologues->count(),&m_szName);
+	m_pEpilogues->append(new KviKvsScript(szName,szCode));
 }
 
 KviKvsPopupMenuTopLevelData * KviKvsPopupMenu::topLevelData()
@@ -467,6 +599,54 @@ KviKvsPopupMenuTopLevelData * KviKvsPopupMenu::topLevelData()
 	return m_pTopLevelData;
 }
 
+bool KviKvsPopupMenu::removeItemByName(const QString &szItemName,bool bRecursive)
+{
+	KviKvsScript * se;
+	
+	for(se = m_pEpilogues->first();se;se = m_pEpilogues->next())
+	{
+		if(KviQString::equalCI(szItemName,se->name()))
+		{
+			m_pEpilogues->removeRef(se);
+			return true;
+		}
+	}
+
+	for(se = m_pPrologues->first();se;se = m_pPrologues->next())
+	{
+		if(KviQString::equalCI(szItemName,se->name()))
+		{
+			m_pPrologues->removeRef(se);
+			return true;
+		}
+	}
+
+	for(KviKvsPopupMenuItem * it = m_pItemList->first();it;it = m_pItemList->next())
+	{
+		if(KviQString::equalCI(szItemName,it->name()))
+		{
+			m_pItemList->removeRef(it); // bye :)
+			return true;
+		}
+	}
+
+	if(bRecursive)
+	{
+		for(KviKvsPopupMenuItem * ii = m_pItemList->first();ii;ii = m_pItemList->next())
+		{
+			if(ii->isMenu())
+			{
+				if(((KviKvsPopupMenuItemMenu *)ii)->menu())
+				{
+					bool bRet = ((KviKvsPopupMenuItemMenu *)ii)->menu()->removeItemByName(szItemName,true);
+					if(bRet)return true;
+				}
+			}
+		}
+	}
+	
+	return false;
+}
 
 bool KviKvsPopupMenu::isLocked()
 {
@@ -483,43 +663,48 @@ KviKvsPopupMenu * KviKvsPopupMenu::topLevelPopup()
 }
 
 
-KviKvsPopupMenu * KviKvsPopupMenu::addPopup(const QString &szText,const QString &szIcon,const QString &szCondition)
+KviKvsPopupMenu * KviKvsPopupMenu::addPopup(const QString &szItemName,const QString &szText,const QString &szIcon,const QString &szCondition)
 {
-	QString szName;
-	KviQString::sprintf(szName,"%Q.subpopup%d",&m_szName,m_pItemList->count());
+	QString szName = szItemName;
+	if(szName.isEmpty())
+		KviQString::sprintf(szName,"%Q.subpopup%d",&m_szName,m_pItemList->count());
 	KviKvsPopupMenu * pNew = new KviKvsPopupMenu(szName);
 	pNew->setParentPopup(this);
 	addItemInternal(new KviKvsPopupMenuItemMenu(szName,pNew,szText,szIcon,szCondition));
 	return pNew;
 }
 
-void KviKvsPopupMenu::addSeparator(const QString &szCondition)
+void KviKvsPopupMenu::addSeparator(const QString &szItemName,const QString &szCondition)
 {
-	QString szName;
-	KviQString::sprintf(szName,"%Q.separator%d",&m_szName,m_pItemList->count());
+	QString szName = szItemName;
+	if(szName.isEmpty())
+		KviQString::sprintf(szName,"%Q.separator%d",&m_szName,m_pItemList->count());
 	addItemInternal(new KviKvsPopupMenuItemSeparator(szName,szCondition));
 }
 
-void KviKvsPopupMenu::addLabel(const QString &szText,const QString &szIcon,const QString &szCondition)
+void KviKvsPopupMenu::addLabel(const QString &szItemName,const QString &szText,const QString &szIcon,const QString &szCondition)
 {
-	QString szName;
-	KviQString::sprintf(szName,"%Q.label%d",&m_szName,m_pItemList->count());
+	QString szName = szItemName;
+	if(szName.isEmpty())
+		KviQString::sprintf(szName,"%Q.label%d",&m_szName,m_pItemList->count());
 	addItemInternal(new KviKvsPopupMenuItemLabel(szName,szText,szIcon,szCondition));
 }
 
-void KviKvsPopupMenu::addItem(const QString &szCode,const QString &szText,const QString &szIcon,const QString &szCondition)
+void KviKvsPopupMenu::addItem(const QString &szItemName,const QString &szCode,const QString &szText,const QString &szIcon,const QString &szCondition)
 {
-	QString szName;
-	KviQString::sprintf(szName,"%Q.item%d",&m_szName,m_pItemList->count());
+	QString szName = szItemName;
+	if(szName.isEmpty())
+		KviQString::sprintf(szName,"%Q.item%d",&m_szName,m_pItemList->count());
 	addItemInternal(new KviKvsPopupMenuItemItem(szName,szCode,szText,szIcon,szCondition));
 
 }
 
-void KviKvsPopupMenu::addExtPopup(const QString &szName,const QString szText,const QString &szIcon,const QString &szCondition)
+void KviKvsPopupMenu::addExtPopup(const QString &szItemName,const QString &szPopupName,const QString szText,const QString &szIcon,const QString &szCondition)
 {
-	QString szItemName;
-	KviQString::sprintf(szItemName,"%Q.%Q",&m_szName,&szName);
-	addItemInternal(new KviKvsPopupMenuItemExtMenu(szItemName,szName,szText,szIcon,szCondition));
+	QString szName = szItemName;
+	if(szName.isEmpty())
+		KviQString::sprintf(szName,"%Q.%Q",&m_szName,&szName);
+	addItemInternal(new KviKvsPopupMenuItemExtMenu(szName,szPopupName,szText,szIcon,szCondition));
 
 }
 
@@ -529,12 +714,13 @@ void KviKvsPopupMenu::addItemInternal(KviKvsPopupMenuItem * it)
 	m_pItemList->append(it);
 }
 
-void KviKvsPopupMenu::doPopup(const QPoint & pnt,KviWindow * wnd,KviKvsVariantList * pParams)
+void KviKvsPopupMenu::doPopup(const QPoint & pnt,KviWindow * wnd,KviKvsVariantList * pParams,bool bTestMode)
 {
 	// This might be a compat problem later :(((((
 	// it is also an ugly trick
 	clearMenuContents();
 	m_pTempTopLevelData = new KviKvsPopupMenuTopLevelData(pParams,wnd);
+	m_pTempTopLevelData->setTestMode(bTestMode);
 	QPopupMenu::popup(pnt);
 }
 
@@ -593,7 +779,6 @@ void KviKvsPopupMenu::lock(bool bLock)
 void KviKvsPopupMenu::setupMenuContents()
 {
 	// This might be a compat problem later :((((
-
 	if(parentPopup() == 0)
 	{
 		if(m_pTempTopLevelData == 0)
@@ -621,9 +806,7 @@ void KviKvsPopupMenu::setupMenuContents()
 	// HACK...this is to remove the separator inserted by Qt when popup() is called and the popup is empty
 	clear();
 
-
 	KviKvsPopupMenuTopLevelData * d = topLevelData();
-
 	if(!d)
 	{
 		debug("Ops...menu contents changed behind my back!");
@@ -636,7 +819,8 @@ void KviKvsPopupMenu::setupMenuContents()
 
 	if(!g_pApp->windowExists(d->window()))d->setWindow(g_pApp->activeConsole());
 
-	executePrologues(d);
+	if(!d->testMode())
+		executePrologues(d);
 
 	// Fill this menu contents
 	int idx = 0;
@@ -646,7 +830,8 @@ void KviKvsPopupMenu::setupMenuContents()
 		++idx;
 	}
 
-	executeEpilogues(d);
+	if(!d->testMode())
+		executeEpilogues(d);
 
 	lock(false);
 
@@ -692,21 +877,24 @@ void KviKvsPopupMenu::itemClicked(int itemId)
 	{
 		if(it->isItem())
 		{
-			// rebind if window is lost
-			if(!g_pApp->windowExists(d->window()))d->setWindow(g_pApp->activeConsole());
-
-			// FIXME: we could avoid locking since scripts can be shared now!
-			// see KviKvsTimerManager implementation
-			lock(true);
-			((KviKvsPopupMenuItemItem *)it)->kvsCode()->run(
-					d->window(),
-					d->parameters(),
-					0,
-					KviKvsScript::PreserveParams,
-					d->extendedRunTimeData());
-			// FIXME: should we print somethng if run() returns false ?
-			lock(false);
-			
+			if(d->testMode())
+			{
+				emit testModeItemClicked(it);
+			} else {
+				// rebind if window is lost
+				if(!g_pApp->windowExists(d->window()))d->setWindow(g_pApp->activeConsole());
+				// FIXME: we could avoid locking since scripts can be shared now!
+				// see KviKvsTimerManager implementation
+				lock(true);
+				((KviKvsPopupMenuItemItem *)it)->kvsCode()->run(
+						d->window(),
+						d->parameters(),
+						0,
+						KviKvsScript::PreserveParams,
+						d->extendedRunTimeData());
+				// FIXME: should we print somethng if run() returns false ?
+				lock(false);
+			}
 		} else debug("oops....clicked something that is not an item at position %d",param);
 		// FIXME: #warning "Maybe tell that the window has changed"
 	} else debug("oops....no menu item at position %d",param);
@@ -735,13 +923,15 @@ void KviKvsPopupMenu::load(const QString &prefix,KviConfig * cfg)
 		{
 			KviQString::sprintf(tmp,"%Q_Prologue%d",&(prefix),idx);
 			QString pr = cfg->readQStringEntry(tmp,"");
-			if(!pr.isEmpty())addPrologue(pr);
+			KviQString::sprintf(tmp,"%Q_PrologueId%d",&(prefix),idx);
+			QString itemName = cfg->readQStringEntry(tmp,"");
+			if(!pr.isEmpty())addPrologue(itemName,pr);
 		}
 	} else {
 		// Might be old version!
 		KviQString::sprintf(tmp,"%Q_Prologue",&(prefix));
 		QString pr = cfg->readQStringEntry(tmp,"");
-		if(!pr.isEmpty())addPrologue(pr);
+		if(!pr.isEmpty())addPrologue(QString::null,pr);
 	}
 
 	KviQString::sprintf(tmp,"%Q_EpilogueCount",&prefix);
@@ -753,13 +943,15 @@ void KviKvsPopupMenu::load(const QString &prefix,KviConfig * cfg)
 		{
 			KviQString::sprintf(tmp,"%Q_Epilogue%d",&prefix,idx);
 			QString ep = cfg->readQStringEntry(tmp,"");
-			if(!ep.isEmpty())addEpilogue(ep);
+			KviQString::sprintf(tmp,"%Q_PrologueId%d",&(prefix),idx);
+			QString itemName = cfg->readQStringEntry(tmp,"");
+			if(!ep.isEmpty())addEpilogue(itemName,ep);
 		}
 	} else {
 		// Might be old version!
 		KviQString::sprintf(tmp,"%Q_Epilogue",&prefix);
 		QString ep = cfg->readQStringEntry(tmp,"");
-		if(!ep.isEmpty())addEpilogue(ep);
+		if(!ep.isEmpty())addEpilogue(QString::null,ep);
 	}
 
 
@@ -771,6 +963,9 @@ void KviKvsPopupMenu::load(const QString &prefix,KviConfig * cfg)
 	{
 		QString pre;
 		KviQString::sprintf(pre,"%Q_%d",&prefix,idx);
+		
+		KviQString::sprintf(tmp,"%Q_Id",&pre);
+		QString itemName = cfg->readQStringEntry(tmp,QString::null);
 		KviQString::sprintf(tmp,"%Q_Type",&pre);
 		
 		int type = cfg->readIntEntry(tmp,3);
@@ -781,7 +976,7 @@ void KviKvsPopupMenu::load(const QString &prefix,KviConfig * cfg)
 				QString expr;
 				KviQString::sprintf(tmp,"%Q_Expr",&pre);
 				expr = cfg->readQStringEntry(tmp,"");
-				addSeparator(expr);
+				addSeparator(itemName,expr);
 			}
 			break;
 			case 1: // item
@@ -795,7 +990,7 @@ void KviKvsPopupMenu::load(const QString &prefix,KviConfig * cfg)
 				code = cfg->readQStringEntry(tmp,"");
 				KviQString::sprintf(tmp,"%Q_Expr",&pre);
 				expr = cfg->readQStringEntry(tmp,"");
-				addItem(code,text,icon,expr);
+				addItem(itemName,code,text,icon,expr);
 			}
 			break;
 			case 2: // menu
@@ -807,7 +1002,7 @@ void KviKvsPopupMenu::load(const QString &prefix,KviConfig * cfg)
 				icon = cfg->readQStringEntry(tmp,"");
 				KviQString::sprintf(tmp,"%Q_Expr",&pre);
 				expr = cfg->readQStringEntry(tmp,"");
-				KviKvsPopupMenu * pop = addPopup(text,icon,expr);
+				KviKvsPopupMenu * pop = addPopup(itemName,text,icon,expr);
 				pop->load(pre,cfg);
 			}
 			break;
@@ -820,7 +1015,7 @@ void KviKvsPopupMenu::load(const QString &prefix,KviConfig * cfg)
 				icon = cfg->readQStringEntry(tmp,"");
 				KviQString::sprintf(tmp,"%Q_Expr",&pre);
 				expr = cfg->readQStringEntry(tmp,"");
-				addLabel(text,icon,expr);
+				addLabel(itemName,text,icon,expr);
 			}
 			break;
 			case 4: // extmenu
@@ -834,7 +1029,7 @@ void KviKvsPopupMenu::load(const QString &prefix,KviConfig * cfg)
 				code = cfg->readQStringEntry(tmp,"");
 				KviQString::sprintf(tmp,"%Q_Expr",&pre);
 				expr = cfg->readQStringEntry(tmp,"");
-				addExtPopup(code,text,icon,expr);
+				addExtPopup(itemName,code,text,icon,expr);
 			}
 			break;
 			default: // ignore
@@ -861,6 +1056,8 @@ void KviKvsPopupMenu::save(const QString & prefix,KviConfig * cfg)
 	{
 		KviQString::sprintf(tmp,"%Q_Prologue%d",&prefix,idx);
 		cfg->writeEntry(tmp,s->code());
+		KviQString::sprintf(tmp,"%Q_PrologueId%d",&prefix,idx);
+		cfg->writeEntry(tmp,s->name());
 		idx++;
 	}
 
@@ -872,6 +1069,8 @@ void KviKvsPopupMenu::save(const QString & prefix,KviConfig * cfg)
 	{
 		KviQString::sprintf(tmp,"%Q_Epilogue%d",&prefix,idx);
 		cfg->writeEntry(tmp,s->code());
+		KviQString::sprintf(tmp,"%Q_EpilogueId%d",&prefix,idx);
+		cfg->writeEntry(tmp,s->name());
 		idx++;
 	}
 
@@ -897,6 +1096,9 @@ void KviKvsPopupMenu::save(const QString & prefix,KviConfig * cfg)
 
 		cfg->writeEntry(tmp,typeCode);
 
+		KviQString::sprintf(tmp,"%Q_Id",&pre);
+		cfg->writeEntry(tmp,it->name());
+
 		s = it->kvsCondition();
 		if(s)
 		{
