@@ -39,6 +39,12 @@ KviKvsTreeNodeExpression::~KviKvsTreeNodeExpression()
 {
 }
 
+
+void KviKvsTreeNodeExpression::contextDescription(QString &szBuffer)
+{
+	szBuffer = "Expression Evaluation";
+}
+
 void KviKvsTreeNodeExpression::dump(const char * prefix)
 {
 	debug("%s Expression",prefix);
@@ -99,12 +105,17 @@ KviKvsTreeNodeExpressionVariableOperand::~KviKvsTreeNodeExpressionVariableOperan
 	delete m_pData;
 }
 
+void KviKvsTreeNodeExpressionVariableOperand::contextDescription(QString &szBuffer)
+{
+	szBuffer = "Expression Variable Operand Evaluation";
+}
+
 void KviKvsTreeNodeExpressionVariableOperand::dump(const char * prefix)
 {
 	debug("%s ExpressionVariableOperand",prefix);
 	QString tmp = prefix;
 	tmp.append("  ");
-	m_pData->dump(tmp.latin1());
+	m_pData->dump(tmp.utf8().data());
 }
 
 bool KviKvsTreeNodeExpressionVariableOperand::evaluateReadOnly(KviKvsRunTimeContext * c,KviKvsVariant * pBuffer)
@@ -118,25 +129,41 @@ bool KviKvsTreeNodeExpressionVariableOperand::evaluateReadOnly(KviKvsRunTimeCont
 KviKvsTreeNodeExpressionConstantOperand::KviKvsTreeNodeExpressionConstantOperand(const QChar * pLocation,KviKvsVariant * pConstant)
 : KviKvsTreeNodeExpression(pLocation)
 {
+#ifdef COMPILE_NEW_KVS
 	m_pConstant = pConstant;
+#endif
 }
 
 KviKvsTreeNodeExpressionConstantOperand::~KviKvsTreeNodeExpressionConstantOperand()
 {
+#ifdef COMPILE_NEW_KVS
 	delete m_pConstant;
+#endif
 }
 
+void KviKvsTreeNodeExpressionConstantOperand::contextDescription(QString &szBuffer)
+{
+#ifdef COMPILE_NEW_KVS
+	szBuffer = "Expression Constant Operand Evaluation";
+#endif
+}
+
+
 void KviKvsTreeNodeExpressionConstantOperand::dump(const char * prefix)
 {
+#ifdef COMPILE_NEW_KVS
 	debug("%s ExpressionConstantOperand",prefix);
 	QString tmp = prefix;
 	tmp.append("  ");
-	m_pConstant->dump(tmp.latin1());
+	m_pConstant->dump(tmp.utf8().data());
+#endif
 }
 
 bool KviKvsTreeNodeExpressionConstantOperand::evaluateReadOnly(KviKvsRunTimeContext * c,KviKvsVariant * pBuffer)
 {
+#ifdef COMPILE_NEW_KVS
 	pBuffer->copyFrom(m_pConstant);
+#endif
 	return true;
 }
 
@@ -152,9 +179,19 @@ KviKvsTreeNodeExpressionOperator::~KviKvsTreeNodeExpressionOperator()
 {
 }
 
+void KviKvsTreeNodeExpressionOperator::contextDescription(QString &szBuffer)
+{
+#ifdef COMPILE_NEW_KVS
+	szBuffer = "Expression Operator Evaluation";
+#endif
+}
+
+
 void KviKvsTreeNodeExpressionOperator::dump(const char * prefix)
 {
+#ifdef COMPILE_NEW_KVS
 	debug("%s ExpressionOperator",prefix);
+#endif
 }
 
 
@@ -165,35 +202,52 @@ void KviKvsTreeNodeExpressionOperator::dump(const char * prefix)
 KviKvsTreeNodeExpressionUnaryOperator::KviKvsTreeNodeExpressionUnaryOperator(const QChar * pLocation,KviKvsTreeNodeExpression * pData)
 : KviKvsTreeNodeExpressionOperator(pLocation)
 {
+#ifdef COMPILE_NEW_KVS
 	m_pData = pData;
 	m_pData->setParent(this);
 	m_pData->setParentExpression(this);
+#endif
 }
 
 KviKvsTreeNodeExpressionUnaryOperator::~KviKvsTreeNodeExpressionUnaryOperator()
 {
+#ifdef COMPILE_NEW_KVS
 	delete m_pData;
+#endif
 }
 
+void KviKvsTreeNodeExpressionUnaryOperator::contextDescription(QString &szBuffer)
+{
+#ifdef COMPILE_NEW_KVS
+	szBuffer = "Expression Unary Operator Evaluation";
+#endif
+}
+
+
+
+
 void KviKvsTreeNodeExpressionUnaryOperator::dump(const char * prefix)
 {
+#ifdef COMPILE_NEW_KVS
 	debug("%s ExpressionUnaryOperator",prefix);
 	QString tmp = prefix;
 	tmp.append("  ");
-	m_pData->dump(tmp.latin1());
+	m_pData->dump(tmp.utf8().data());
+#endif
 }
 
 bool KviKvsTreeNodeExpressionUnaryOperator::evaluateOperand(KviKvsRunTimeContext * c)
 {
+#ifdef COMPILE_NEW_KVS
 	KviKvsVariant v;
 	if(!m_pData->evaluateReadOnly(c,&v))return false;
 
 	if(!v.asNumber(m_nData))
 	{
-		c->error(location(),__tr2qs("Operand of unary operator didn't evaluate to a number"));
+		c->error(this,__tr2qs("Operand of unary operator didn't evaluate to a number"));
 		return false;
 	}
-
+#endif
 	return true;
 }
 
@@ -209,12 +263,23 @@ KviKvsTreeNodeExpressionUnaryOperatorNegate::~KviKvsTreeNodeExpressionUnaryOpera
 {
 }
 
+
+void KviKvsTreeNodeExpressionUnaryOperatorNegate::contextDescription(QString &szBuffer)
+{
+#ifdef COMPILE_NEW_KVS
+	szBuffer = "Expression Unary Operator Negate";
+#endif
+}
+
+
 void KviKvsTreeNodeExpressionUnaryOperatorNegate::dump(const char * prefix)
 {
+#ifdef COMPILE_NEW_KVS
 	debug("%s ExpressionUnaryOperatorNegate",prefix);
 	QString tmp = prefix;
 	tmp.append("  ");
-	m_pData->dump(tmp.latin1());
+	m_pData->dump(tmp.utf8().data());
+#endif
 }
 
 int KviKvsTreeNodeExpressionUnaryOperatorNegate::precedence()
@@ -224,9 +289,11 @@ int KviKvsTreeNodeExpressionUnaryOperatorNegate::precedence()
 
 bool KviKvsTreeNodeExpressionUnaryOperatorNegate::evaluateReadOnly(KviKvsRunTimeContext * c,KviKvsVariant * pBuffer)
 {
+#ifdef COMPILE_NEW_KVS
 	if(!evaluateOperand(c))return false;
 	if(m_nData.isReal())pBuffer->setReal(-m_nData.real());
 	else pBuffer->setInteger(-m_nData.integer());
+#endif
 	return true;
 }
 
@@ -243,12 +310,21 @@ KviKvsTreeNodeExpressionUnaryOperatorBitwiseNot::~KviKvsTreeNodeExpressionUnaryO
 {
 }
 
+void KviKvsTreeNodeExpressionUnaryOperatorBitwiseNot::contextDescription(QString &szBuffer)
+{
+#ifdef COMPILE_NEW_KVS
+	szBuffer = "Expression Unary Operator Bitwise Not";
+#endif
+}
+
 void KviKvsTreeNodeExpressionUnaryOperatorBitwiseNot::dump(const char * prefix)
 {
+#ifdef COMPILE_NEW_KVS
 	debug("%s ExpressionUnaryOperatorBitwiseNot",prefix);
 	QString tmp = prefix;
 	tmp.append("  ");
-	m_pData->dump(tmp.latin1());
+	m_pData->dump(tmp.utf8().data());
+#endif
 }
 
 int KviKvsTreeNodeExpressionUnaryOperatorBitwiseNot::precedence()
@@ -258,9 +334,11 @@ int KviKvsTreeNodeExpressionUnaryOperatorBitwiseNot::precedence()
 
 bool KviKvsTreeNodeExpressionUnaryOperatorBitwiseNot::evaluateReadOnly(KviKvsRunTimeContext * c,KviKvsVariant * pBuffer)
 {
+#ifdef COMPILE_NEW_KVS
 	if(!evaluateOperand(c))return false;
 	if(m_nData.isReal())pBuffer->setInteger(~(int)(m_nData.real()));
 	else pBuffer->setInteger(~m_nData.integer());
+#endif
 	return true;
 }
 
@@ -277,12 +355,21 @@ KviKvsTreeNodeExpressionUnaryOperatorLogicalNot::~KviKvsTreeNodeExpressionUnaryO
 {
 }
 
+void KviKvsTreeNodeExpressionUnaryOperatorLogicalNot::contextDescription(QString &szBuffer)
+{
+#ifdef COMPILE_NEW_KVS
+	szBuffer = "Expression Unary Operator Logical Not";
+#endif
+}
+
 void KviKvsTreeNodeExpressionUnaryOperatorLogicalNot::dump(const char * prefix)
 {
+#ifdef COMPILE_NEW_KVS
 	debug("%s ExpressionUnaryOperatorLogicalNot",prefix);
 	QString tmp = prefix;
 	tmp.append("  ");
-	m_pData->dump(tmp.latin1());
+	m_pData->dump(tmp.utf8().data());
+#endif
 }
 
 int KviKvsTreeNodeExpressionUnaryOperatorLogicalNot::precedence()
@@ -292,10 +379,12 @@ int KviKvsTreeNodeExpressionUnaryOperatorLogicalNot::precedence()
 
 bool KviKvsTreeNodeExpressionUnaryOperatorLogicalNot::evaluateReadOnly(KviKvsRunTimeContext * c,KviKvsVariant * pBuffer)
 {
+#ifdef COMPILE_NEW_KVS
 	KviKvsVariant v;
 	if(!m_pData->evaluateReadOnly(c,&v))return false;
 //#warning "FIXME: We could use setNothing() for false and setInteger(1) for true: this would save memory allocations for false conditions"
-	pBuffer->setInteger(v.asBoolean() ? 0 : 1);
+	pBuffer->setBoolean(!v.asBoolean());
+#endif
 	return true;
 }
 
@@ -323,14 +412,14 @@ bool KviKvsTreeNodeExpressionBinaryOperator::evaluateOperands(KviKvsRunTimeConte
 	if(!m_pLeft->evaluateReadOnly(c,&v1))return false;
 	if(!v1.asNumber(m_nLeft))
 	{
-		c->error(location(),__tr2qs("Left operand didn't evaluate to a number"));
+		c->error(this,__tr2qs("Left operand didn't evaluate to a number"));
 		return false;
 	}
 	KviKvsVariant v2;
 	if(!m_pRight->evaluateReadOnly(c,&v2))return false;
 	if(!v2.asNumber(m_nRight))
 	{
-		c->error(location(),__tr2qs("Right operand didn't evaluate to a number"));
+		c->error(this,__tr2qs("Right operand didn't evaluate to a number"));
 		return false;
 	}
 	return true;
@@ -370,8 +459,13 @@ void KviKvsTreeNodeExpressionBinaryOperator::dumpOperands(const char * prefix)
 {
 	QString tmp = prefix;
 	tmp.append("  ");
-	if(m_pLeft)m_pLeft->dump(tmp.latin1());
-	if(m_pRight)m_pRight->dump(tmp.latin1());
+	if(m_pLeft)m_pLeft->dump(tmp.utf8().data());
+	if(m_pRight)m_pRight->dump(tmp.utf8().data());
+}
+
+void KviKvsTreeNodeExpressionBinaryOperator::contextDescription(QString &szBuffer)
+{
+	szBuffer = "Expression Binary Operator";
 }
 
 void KviKvsTreeNodeExpressionBinaryOperator::dump(const char * prefix)
@@ -383,14 +477,15 @@ void KviKvsTreeNodeExpressionBinaryOperator::dump(const char * prefix)
 ////////////////////////////////////////////////////////////////////////////////
 
 
-#define PREIMPLEMENT_BINARY_OPERATOR(__name,__stringname,__precedence) \
+#define PREIMPLEMENT_BINARY_OPERATOR(__name,__stringname,__contextdescription,__precedence) \
 	__name::__name(const QChar * pLocation) \
 	: KviKvsTreeNodeExpressionBinaryOperator(pLocation){} \
 	__name::~__name(){} \
 	void __name::dump(const char * prefix){ debug("%s " __stringname,prefix); dumpOperands(prefix); } \
+	void __name::contextDescription(QString &szBuffer){ szBuffer = __contextdescription; } \
 	int __name::precedence(){ return __precedence; };
 
-PREIMPLEMENT_BINARY_OPERATOR(KviKvsTreeNodeExpressionBinaryOperatorSum,"ExpressionBinaryOperatorSum",PREC_OP_SUM)
+PREIMPLEMENT_BINARY_OPERATOR(KviKvsTreeNodeExpressionBinaryOperatorSum,"ExpressionBinaryOperatorSum","Expression Binary Operator \"+\"",PREC_OP_SUM)
 
 bool KviKvsTreeNodeExpressionBinaryOperatorSum::evaluateReadOnly(KviKvsRunTimeContext * c,KviKvsVariant * pBuffer)
 {
@@ -406,7 +501,7 @@ bool KviKvsTreeNodeExpressionBinaryOperatorSum::evaluateReadOnly(KviKvsRunTimeCo
 	return true;
 }
 
-PREIMPLEMENT_BINARY_OPERATOR(KviKvsTreeNodeExpressionBinaryOperatorSubtraction,"ExpressionBinaryOperatorSubtraction",PREC_OP_SUBTRACTION)
+PREIMPLEMENT_BINARY_OPERATOR(KviKvsTreeNodeExpressionBinaryOperatorSubtraction,"ExpressionBinaryOperatorSubtraction","Expression Binary Operator \"-\"",PREC_OP_SUBTRACTION)
 
 bool KviKvsTreeNodeExpressionBinaryOperatorSubtraction::evaluateReadOnly(KviKvsRunTimeContext * c,KviKvsVariant * pBuffer)
 {
@@ -414,15 +509,15 @@ bool KviKvsTreeNodeExpressionBinaryOperatorSubtraction::evaluateReadOnly(KviKvsR
 	if(m_nLeft.isInteger())
 	{
 		if(m_nRight.isInteger())pBuffer->setInteger(m_nLeft.integer() - m_nRight.integer());
-		else pBuffer->setReal(((double)(m_nLeft.integer())) - m_nRight.real());
+		else pBuffer->setReal(((kvs_real_t)(m_nLeft.integer())) - m_nRight.real());
 	} else {
-		if(m_nRight.isInteger())pBuffer->setReal(m_nLeft.real() - ((double)(m_nRight.integer())));
+		if(m_nRight.isInteger())pBuffer->setReal(m_nLeft.real() - ((kvs_real_t)(m_nRight.integer())));
 		else pBuffer->setReal(m_nLeft.real() - m_nRight.real());
 	}
 	return true;
 }
 
-PREIMPLEMENT_BINARY_OPERATOR(KviKvsTreeNodeExpressionBinaryOperatorMultiplication,"ExpressionBinaryOperatorMultiplication",PREC_OP_MULTIPLICATION)
+PREIMPLEMENT_BINARY_OPERATOR(KviKvsTreeNodeExpressionBinaryOperatorMultiplication,"ExpressionBinaryOperatorMultiplication","Expression Binary Operator \"*\"",PREC_OP_MULTIPLICATION)
 
 bool KviKvsTreeNodeExpressionBinaryOperatorMultiplication::evaluateReadOnly(KviKvsRunTimeContext * c,KviKvsVariant * pBuffer)
 {
@@ -430,44 +525,44 @@ bool KviKvsTreeNodeExpressionBinaryOperatorMultiplication::evaluateReadOnly(KviK
 	if(m_nLeft.isInteger())
 	{
 		if(m_nRight.isInteger())pBuffer->setInteger(m_nLeft.integer() * m_nRight.integer());
-		else pBuffer->setReal(m_nLeft.integer() * m_nRight.real());
+		else pBuffer->setReal(((kvs_real_t)(m_nLeft.integer())) * m_nRight.real());
 	} else {
-		if(m_nRight.isInteger())pBuffer->setReal(m_nLeft.real() * m_nRight.integer());
+		if(m_nRight.isInteger())pBuffer->setReal(m_nLeft.real() * ((kvs_real_t)(m_nRight.integer())));
 		else pBuffer->setReal(m_nLeft.real() * m_nRight.real());
 	}
 	return true;
 }
 
 
-PREIMPLEMENT_BINARY_OPERATOR(KviKvsTreeNodeExpressionBinaryOperatorDivision,"ExpressionBinaryOperatorDivision",PREC_OP_DIVISION)
+PREIMPLEMENT_BINARY_OPERATOR(KviKvsTreeNodeExpressionBinaryOperatorDivision,"ExpressionBinaryOperatorDivision","Expression Binary Operator \"/\"",PREC_OP_DIVISION)
 
 bool KviKvsTreeNodeExpressionBinaryOperatorDivision::evaluateReadOnly(KviKvsRunTimeContext * c,KviKvsVariant * pBuffer)
 {
-	if(!evaluateOperands(c))return false;	
+	if(!evaluateOperands(c))return false;
 
 	if(m_nRight.isInteger())
 	{
 		if(m_nRight.integer() == 0)
 		{
-			c->error(location(),__tr2qs("Division by zero"));
+			c->error(this,__tr2qs("Division by zero"));
 			return false;
 		}
 		if(m_nLeft.isInteger())pBuffer->setInteger(m_nLeft.integer() / m_nRight.integer());
-		else pBuffer->setReal(m_nLeft.real() / ((double)(m_nRight.integer())));
+		else pBuffer->setReal(m_nLeft.real() / ((kvs_real_t)(m_nRight.integer())));
 	} else {
 		if(m_nRight.real() == 0.0)
 		{
-			c->error(location(),__tr2qs("Division by zero"));
+			c->error(this,__tr2qs("Division by zero"));
 			return false;
 		}
-		if(m_nLeft.isInteger())pBuffer->setReal(((double)(m_nLeft.integer())) / m_nRight.real());
+		if(m_nLeft.isInteger())pBuffer->setReal(((kvs_real_t)(m_nLeft.integer())) / m_nRight.real());
 		else pBuffer->setReal(m_nLeft.real() / m_nRight.real());
 	}
 	return true;
 }
 
 
-PREIMPLEMENT_BINARY_OPERATOR(KviKvsTreeNodeExpressionBinaryOperatorModulus,"ExpressionBinaryOperatorModulus",PREC_OP_MODULUS)
+PREIMPLEMENT_BINARY_OPERATOR(KviKvsTreeNodeExpressionBinaryOperatorModulus,"ExpressionBinaryOperatorModulus","Expression Binary Operator \"modulus\"",PREC_OP_MODULUS)
 
 bool KviKvsTreeNodeExpressionBinaryOperatorModulus::evaluateReadOnly(KviKvsRunTimeContext * c,KviKvsVariant * pBuffer)
 {
@@ -477,79 +572,79 @@ bool KviKvsTreeNodeExpressionBinaryOperatorModulus::evaluateReadOnly(KviKvsRunTi
 	{
 		if(m_nRight.integer() == 0)
 		{
-			c->error(location(),__tr2qs("Division by zero"));
+			c->error(this,__tr2qs("Division by zero"));
 			return false;
 		}
 		if(m_nLeft.isInteger())pBuffer->setInteger(m_nLeft.integer() % m_nRight.integer());
-		else pBuffer->setReal(fmod(m_nLeft.real(),((double)(m_nRight.integer()))));
+		else pBuffer->setReal(fmod(m_nLeft.real(),((kvs_real_t)(m_nRight.integer()))));
 	} else {
 		if(m_nRight.real() == 0.0)
 		{
-			c->error(location(),__tr2qs("Division by zero"));
+			c->error(this,__tr2qs("Division by zero"));
 			return false;
 		}
-		if(m_nLeft.isInteger())pBuffer->setReal(fmod(((double)(m_nLeft.integer())),m_nRight.real()));
+		if(m_nLeft.isInteger())pBuffer->setReal(fmod(((kvs_real_t)(m_nLeft.integer())),m_nRight.real()));
 		else pBuffer->setReal(fmod(m_nLeft.real(),m_nRight.real()));
 	}
 	return true;
 }
 
-PREIMPLEMENT_BINARY_OPERATOR(KviKvsTreeNodeExpressionBinaryOperatorBitwiseAnd,"ExpressionBinaryOperatorBitwiseAnd",PREC_OP_BITWISEAND)
+PREIMPLEMENT_BINARY_OPERATOR(KviKvsTreeNodeExpressionBinaryOperatorBitwiseAnd,"ExpressionBinaryOperatorBitwiseAnd","Expression Binary Operator \"&\"",PREC_OP_BITWISEAND)
 
 bool KviKvsTreeNodeExpressionBinaryOperatorBitwiseAnd::evaluateReadOnly(KviKvsRunTimeContext * c,KviKvsVariant * pBuffer)
 {
 	if(!evaluateOperands(c))return false;
-	int iLeft = m_nLeft.isInteger() ? m_nLeft.integer() : (int)m_nLeft.real();
-	int iRight = m_nRight.isInteger() ? m_nRight.integer() : (int)m_nRight.real();
+	int iLeft = m_nLeft.isInteger() ? m_nLeft.integer() : (kvs_int_t)m_nLeft.real();
+	int iRight = m_nRight.isInteger() ? m_nRight.integer() : (kvs_int_t)m_nRight.real();
 	pBuffer->setInteger(iLeft & iRight);
 	return true;
 }
 
-PREIMPLEMENT_BINARY_OPERATOR(KviKvsTreeNodeExpressionBinaryOperatorBitwiseOr,"ExpressionBinaryOperatorBitwiseOr",PREC_OP_BITWISEOR)
+PREIMPLEMENT_BINARY_OPERATOR(KviKvsTreeNodeExpressionBinaryOperatorBitwiseOr,"ExpressionBinaryOperatorBitwiseOr","Expression Binary Operator \"|\"",PREC_OP_BITWISEOR)
 
 bool KviKvsTreeNodeExpressionBinaryOperatorBitwiseOr::evaluateReadOnly(KviKvsRunTimeContext * c,KviKvsVariant * pBuffer)
 {
 	if(!evaluateOperands(c))return false;
-	int iLeft = m_nLeft.isInteger() ? m_nLeft.integer() : (int)m_nLeft.real();
-	int iRight = m_nRight.isInteger() ? m_nRight.integer() : (int)m_nRight.real();
+	int iLeft = m_nLeft.isInteger() ? m_nLeft.integer() : (kvs_int_t)m_nLeft.real();
+	int iRight = m_nRight.isInteger() ? m_nRight.integer() : (kvs_int_t)m_nRight.real();
 	pBuffer->setInteger(iLeft | iRight);
 	return true;
 }
 
-PREIMPLEMENT_BINARY_OPERATOR(KviKvsTreeNodeExpressionBinaryOperatorBitwiseXor,"ExpressionBinaryOperatorBitwiseXor",PREC_OP_BITWISEXOR)
+PREIMPLEMENT_BINARY_OPERATOR(KviKvsTreeNodeExpressionBinaryOperatorBitwiseXor,"ExpressionBinaryOperatorBitwiseXor","Expression Binary Operator \"^\"",PREC_OP_BITWISEXOR)
 
 bool KviKvsTreeNodeExpressionBinaryOperatorBitwiseXor::evaluateReadOnly(KviKvsRunTimeContext * c,KviKvsVariant * pBuffer)
 {
 	if(!evaluateOperands(c))return false;
-	int iLeft = m_nLeft.isInteger() ? m_nLeft.integer() : (int)m_nLeft.real();
-	int iRight = m_nRight.isInteger() ? m_nRight.integer() : (int)m_nRight.real();
+	int iLeft = m_nLeft.isInteger() ? m_nLeft.integer() : (kvs_int_t)m_nLeft.real();
+	int iRight = m_nRight.isInteger() ? m_nRight.integer() : (kvs_int_t)m_nRight.real();
 	pBuffer->setInteger(iLeft ^ iRight);
 	return true;
 }
 
-PREIMPLEMENT_BINARY_OPERATOR(KviKvsTreeNodeExpressionBinaryOperatorShiftLeft,"ExpressionBinaryOperatorShiftLeft",PREC_OP_SHIFTLEFT)
+PREIMPLEMENT_BINARY_OPERATOR(KviKvsTreeNodeExpressionBinaryOperatorShiftLeft,"ExpressionBinaryOperatorShiftLeft","Expression Binary Operator \"<<\"",PREC_OP_SHIFTLEFT)
 
 bool KviKvsTreeNodeExpressionBinaryOperatorShiftLeft::evaluateReadOnly(KviKvsRunTimeContext * c,KviKvsVariant * pBuffer)
 {
 	if(!evaluateOperands(c))return false;
-	int iLeft = m_nLeft.isInteger() ? m_nLeft.integer() : (int)(m_nLeft.real());
-	int iRight = m_nRight.isInteger() ? m_nRight.integer() : (int)(m_nRight.real());
+	int iLeft = m_nLeft.isInteger() ? m_nLeft.integer() : (kvs_int_t)(m_nLeft.real());
+	int iRight = m_nRight.isInteger() ? m_nRight.integer() : (kvs_int_t)(m_nRight.real());
 	pBuffer->setInteger(iLeft << iRight);
 	return true;
 }
 
-PREIMPLEMENT_BINARY_OPERATOR(KviKvsTreeNodeExpressionBinaryOperatorShiftRight,"ExpressionBinaryOperatorShiftRight",PREC_OP_SHIFTRIGHT)
+PREIMPLEMENT_BINARY_OPERATOR(KviKvsTreeNodeExpressionBinaryOperatorShiftRight,"ExpressionBinaryOperatorShiftRight","Expression Binary Operator \">>\"",PREC_OP_SHIFTRIGHT)
 
 bool KviKvsTreeNodeExpressionBinaryOperatorShiftRight::evaluateReadOnly(KviKvsRunTimeContext * c,KviKvsVariant * pBuffer)
 {
 	if(!evaluateOperands(c))return false;
-	int iLeft = m_nLeft.isInteger() ? m_nLeft.integer() : (int)(m_nLeft.real());
-	int iRight = m_nRight.isInteger() ? m_nRight.integer() : (int)(m_nRight.real());
+	int iLeft = m_nLeft.isInteger() ? m_nLeft.integer() : (kvs_int_t)(m_nLeft.real());
+	int iRight = m_nRight.isInteger() ? m_nRight.integer() : (kvs_int_t)(m_nRight.real());
 	pBuffer->setInteger(iLeft >> iRight);
 	return true;
 }
 
-PREIMPLEMENT_BINARY_OPERATOR(KviKvsTreeNodeExpressionBinaryOperatorAnd,"ExpressionBinaryOperatorAnd",PREC_OP_AND)
+PREIMPLEMENT_BINARY_OPERATOR(KviKvsTreeNodeExpressionBinaryOperatorAnd,"ExpressionBinaryOperatorAnd","Expression Binary Operator \"&&\"",PREC_OP_AND)
 
 bool KviKvsTreeNodeExpressionBinaryOperatorAnd::evaluateReadOnly(KviKvsRunTimeContext * c,KviKvsVariant * pBuffer)
 {
@@ -559,16 +654,16 @@ bool KviKvsTreeNodeExpressionBinaryOperatorAnd::evaluateReadOnly(KviKvsRunTimeCo
 	//#warning "FIXME: We could use setNothing() as false: this would save memory allocations (and thus time)"
 	if(!v1.asBoolean())
 	{
-		pBuffer->setInteger(0);
+		pBuffer->setBoolean(false);
 		return true;
 	}
 	if(!m_pRight->evaluateReadOnly(c,&v2))return false;
-	pBuffer->setInteger(v2.asBoolean() ? 1 : 0);
+	pBuffer->setBoolean(v2.asBoolean());
 	return true;
 }
 
 
-PREIMPLEMENT_BINARY_OPERATOR(KviKvsTreeNodeExpressionBinaryOperatorOr,"ExpressionBinaryOperatorOr",PREC_OP_OR)
+PREIMPLEMENT_BINARY_OPERATOR(KviKvsTreeNodeExpressionBinaryOperatorOr,"ExpressionBinaryOperatorOr","Expression Binary Operator \"||\"",PREC_OP_OR)
 
 bool KviKvsTreeNodeExpressionBinaryOperatorOr::evaluateReadOnly(KviKvsRunTimeContext * c,KviKvsVariant * pBuffer)
 {
@@ -578,16 +673,16 @@ bool KviKvsTreeNodeExpressionBinaryOperatorOr::evaluateReadOnly(KviKvsRunTimeCon
 	//#warning "FIXME: We could use setNothing() as false: this would save memory allocations (and thus time)"
 	if(v1.asBoolean())
 	{
-		pBuffer->setInteger(1);
+		pBuffer->setBoolean(true);
 		return true;
 	}
 	if(!m_pRight->evaluateReadOnly(c,&v2))return false;
-	pBuffer->setInteger(v2.asBoolean() ? 1 : 0);
+	pBuffer->setBoolean(v2.asBoolean());
 	return true;
 }
 
 
-PREIMPLEMENT_BINARY_OPERATOR(KviKvsTreeNodeExpressionBinaryOperatorXor,"ExpressionBinaryOperatorXor",PREC_OP_XOR)
+PREIMPLEMENT_BINARY_OPERATOR(KviKvsTreeNodeExpressionBinaryOperatorXor,"ExpressionBinaryOperatorXor","Expression Binary Operator \"^^\"",PREC_OP_XOR)
 
 bool KviKvsTreeNodeExpressionBinaryOperatorXor::evaluateReadOnly(KviKvsRunTimeContext * c,KviKvsVariant * pBuffer)
 {
@@ -597,17 +692,17 @@ bool KviKvsTreeNodeExpressionBinaryOperatorXor::evaluateReadOnly(KviKvsRunTimeCo
 	if(!m_pRight->evaluateReadOnly(c,&v2))return false;
 	//#warning "FIXME: We could use setNothing() as false: this would save memory allocations (and thus time)"
 	if(v1.asBoolean())
-		pBuffer->setInteger(v2.asBoolean() ? 0 : 1);
+		pBuffer->setBoolean(!v2.asBoolean());
 	else {
 		if(v2.asBoolean())
-			pBuffer->setInteger(v1.asBoolean() ? 0 : 1);
+			pBuffer->setBoolean(!v1.asBoolean());
 		else
-			pBuffer->setInteger(0);
+			pBuffer->setBoolean(false);
 	}
 	return true;
 }
 
-PREIMPLEMENT_BINARY_OPERATOR(KviKvsTreeNodeExpressionBinaryOperatorLowerThan,"ExpressionBinaryOperatorLowerThan",PREC_OP_LOWERTHAN)
+PREIMPLEMENT_BINARY_OPERATOR(KviKvsTreeNodeExpressionBinaryOperatorLowerThan,"ExpressionBinaryOperatorLowerThan","Expression Binary Operator \"<\"",PREC_OP_LOWERTHAN)
 
 bool KviKvsTreeNodeExpressionBinaryOperatorLowerThan::evaluateReadOnly(KviKvsRunTimeContext * c,KviKvsVariant * pBuffer)
 {
@@ -615,34 +710,11 @@ bool KviKvsTreeNodeExpressionBinaryOperatorLowerThan::evaluateReadOnly(KviKvsRun
 	KviKvsVariant v2;
 	if(!m_pLeft->evaluateReadOnly(c,&v1))return false;
 	if(!m_pRight->evaluateReadOnly(c,&v2))return false;
-
-	bool bBothNumbers = true;
-	
-	if(!v1.asNumber(m_nLeft))bBothNumbers = false;
-	else if(!v2.asNumber(m_nRight))bBothNumbers = false;
-
-	if(bBothNumbers)
-	{
-		if(m_nLeft.isInteger())
-		{
-			if(m_nRight.isInteger())pBuffer->setInteger(m_nLeft.integer() < m_nRight.integer());
-			else pBuffer->setInteger(((double)(m_nLeft.integer())) < m_nRight.real());
-		} else {
-			if(m_nRight.isInteger())pBuffer->setInteger(m_nLeft.real() < ((double)(m_nRight.integer())));
-			else pBuffer->setInteger(m_nLeft.real() < m_nRight.real());
-		}
-	} else {
-		// at least one is not a number
-		QString sz1;
-		QString sz2;
-		v1.asString(sz1);
-		v2.asString(sz2);
-		pBuffer->setInteger(sz1 < sz2);
-	}
+	pBuffer->setBoolean(v1.compare(&v2,true) > 0);
 	return true;
 }
 
-PREIMPLEMENT_BINARY_OPERATOR(KviKvsTreeNodeExpressionBinaryOperatorGreaterThan,"ExpressionBinaryOperatorGreaterThan",PREC_OP_GREATERTHAN)
+PREIMPLEMENT_BINARY_OPERATOR(KviKvsTreeNodeExpressionBinaryOperatorGreaterThan,"ExpressionBinaryOperatorGreaterThan","Expression Binary Operator \">\"",PREC_OP_GREATERTHAN)
 
 bool KviKvsTreeNodeExpressionBinaryOperatorGreaterThan::evaluateReadOnly(KviKvsRunTimeContext * c,KviKvsVariant * pBuffer)
 {
@@ -650,34 +722,11 @@ bool KviKvsTreeNodeExpressionBinaryOperatorGreaterThan::evaluateReadOnly(KviKvsR
 	KviKvsVariant v2;
 	if(!m_pLeft->evaluateReadOnly(c,&v1))return false;
 	if(!m_pRight->evaluateReadOnly(c,&v2))return false;
-
-	bool bBothNumbers = true;
-	
-	if(!v1.asNumber(m_nLeft))bBothNumbers = false;
-	else if(!v2.asNumber(m_nRight))bBothNumbers = false;
-
-	if(bBothNumbers)
-	{
-		if(m_nLeft.isInteger())
-		{
-			if(m_nRight.isInteger())pBuffer->setInteger(m_nLeft.integer() > m_nRight.integer());
-			else pBuffer->setInteger(((double)(m_nLeft.integer())) > m_nRight.real());
-		} else {
-			if(m_nRight.isInteger())pBuffer->setInteger(m_nLeft.real() > ((double)(m_nRight.integer())));
-			else pBuffer->setInteger(m_nLeft.real() > m_nRight.real());
-		}
-	} else {
-		// at least one is not a number
-		QString sz1;
-		QString sz2;
-		v1.asString(sz1);
-		v2.asString(sz2);
-		pBuffer->setInteger(sz1 > sz2);
-	}
+	pBuffer->setBoolean(v1.compare(&v2,true) < 0);
 	return true;
 }
 
-PREIMPLEMENT_BINARY_OPERATOR(KviKvsTreeNodeExpressionBinaryOperatorLowerOrEqualTo,"ExpressionBinaryOperatorLowerOrEqualTo",PREC_OP_LOWEROREQUALTO)
+PREIMPLEMENT_BINARY_OPERATOR(KviKvsTreeNodeExpressionBinaryOperatorLowerOrEqualTo,"ExpressionBinaryOperatorLowerOrEqualTo","Expression Binary Operator \"<=\"",PREC_OP_LOWEROREQUALTO)
 
 bool KviKvsTreeNodeExpressionBinaryOperatorLowerOrEqualTo::evaluateReadOnly(KviKvsRunTimeContext * c,KviKvsVariant * pBuffer)
 {
@@ -685,34 +734,11 @@ bool KviKvsTreeNodeExpressionBinaryOperatorLowerOrEqualTo::evaluateReadOnly(KviK
 	KviKvsVariant v2;
 	if(!m_pLeft->evaluateReadOnly(c,&v1))return false;
 	if(!m_pRight->evaluateReadOnly(c,&v2))return false;
-
-	bool bBothNumbers = true;
-	
-	if(!v1.asNumber(m_nLeft))bBothNumbers = false;
-	else if(!v2.asNumber(m_nRight))bBothNumbers = false;
-
-	if(bBothNumbers)
-	{
-		if(m_nLeft.isInteger())
-		{
-			if(m_nRight.isInteger())pBuffer->setInteger(m_nLeft.integer() <= m_nRight.integer());
-			else pBuffer->setInteger(((double)(m_nLeft.integer())) <= m_nRight.real());
-		} else {
-			if(m_nRight.isInteger())pBuffer->setInteger(m_nLeft.real() <= ((double)(m_nRight.integer())));
-			else pBuffer->setInteger(m_nLeft.real() <= m_nRight.real());
-		}
-	} else {
-		// at least one is not a number
-		QString sz1;
-		QString sz2;
-		v1.asString(sz1);
-		v2.asString(sz2);
-		pBuffer->setInteger(sz1 <= sz2);
-	}
+	pBuffer->setBoolean(v1.compare(&v2,true) >= 0);
 	return true;
 }
 
-PREIMPLEMENT_BINARY_OPERATOR(KviKvsTreeNodeExpressionBinaryOperatorGreaterOrEqualTo,"ExpressionBinaryOperatorGreaterOrEqualTo",PREC_OP_GREATEROREQUALTO)
+PREIMPLEMENT_BINARY_OPERATOR(KviKvsTreeNodeExpressionBinaryOperatorGreaterOrEqualTo,"ExpressionBinaryOperatorGreaterOrEqualTo","Expression Binary Operator \">=\"",PREC_OP_GREATEROREQUALTO)
 
 bool KviKvsTreeNodeExpressionBinaryOperatorGreaterOrEqualTo::evaluateReadOnly(KviKvsRunTimeContext * c,KviKvsVariant * pBuffer)
 {
@@ -720,34 +746,11 @@ bool KviKvsTreeNodeExpressionBinaryOperatorGreaterOrEqualTo::evaluateReadOnly(Kv
 	KviKvsVariant v2;
 	if(!m_pLeft->evaluateReadOnly(c,&v1))return false;
 	if(!m_pRight->evaluateReadOnly(c,&v2))return false;
-
-	bool bBothNumbers = true;
-	
-	if(!v1.asNumber(m_nLeft))bBothNumbers = false;
-	else if(!v2.asNumber(m_nRight))bBothNumbers = false;
-
-	if(bBothNumbers)
-	{
-		if(m_nLeft.isInteger())
-		{
-			if(m_nRight.isInteger())pBuffer->setInteger(m_nLeft.integer() >= m_nRight.integer());
-			else pBuffer->setInteger(((double)(m_nLeft.integer())) >= m_nRight.real());
-		} else {
-			if(m_nRight.isInteger())pBuffer->setInteger(m_nLeft.real() >= ((double)(m_nRight.integer())));
-			else pBuffer->setInteger(m_nLeft.real() >= m_nRight.real());
-		}
-	} else {
-		// at least one is not a number
-		QString sz1;
-		QString sz2;
-		v1.asString(sz1);
-		v2.asString(sz2);
-		pBuffer->setInteger(sz1 >= sz2);
-	}
+	pBuffer->setBoolean(v1.compare(&v2,true) <= 0);
 	return true;
 }
 
-PREIMPLEMENT_BINARY_OPERATOR(KviKvsTreeNodeExpressionBinaryOperatorEqualTo,"ExpressionBinaryOperatorEqualTo",PREC_OP_EQUALTO)
+PREIMPLEMENT_BINARY_OPERATOR(KviKvsTreeNodeExpressionBinaryOperatorEqualTo,"ExpressionBinaryOperatorEqualTo","Expression Binary Operator \"==\"",PREC_OP_EQUALTO)
 
 bool KviKvsTreeNodeExpressionBinaryOperatorEqualTo::evaluateReadOnly(KviKvsRunTimeContext * c,KviKvsVariant * pBuffer)
 {
@@ -755,35 +758,12 @@ bool KviKvsTreeNodeExpressionBinaryOperatorEqualTo::evaluateReadOnly(KviKvsRunTi
 	KviKvsVariant v2;
 	if(!m_pLeft->evaluateReadOnly(c,&v1))return false;
 	if(!m_pRight->evaluateReadOnly(c,&v2))return false;
-
-	bool bBothNumbers = true;
-	
-	if(!v1.asNumber(m_nLeft))bBothNumbers = false;
-	else if(!v2.asNumber(m_nRight))bBothNumbers = false;
-
-	if(bBothNumbers)
-	{
-		if(m_nLeft.isInteger())
-		{
-			if(m_nRight.isInteger())pBuffer->setInteger(m_nLeft.integer() == m_nRight.integer());
-			else pBuffer->setInteger(((double)(m_nLeft.integer())) == m_nRight.real());
-		} else {
-			if(m_nRight.isInteger())pBuffer->setInteger(m_nLeft.real() == ((double)(m_nRight.integer())));
-			else pBuffer->setInteger(m_nLeft.real() == m_nRight.real());
-		}
-	} else {
-		// at least one is not a number
-		QString sz1;
-		QString sz2;
-		v1.asString(sz1);
-		v2.asString(sz2);
-		pBuffer->setInteger(sz1 == sz2);
-	}
+	pBuffer->setBoolean(v1.compare(&v2,true) == 0);
 	return true;
 }
 
 
-PREIMPLEMENT_BINARY_OPERATOR(KviKvsTreeNodeExpressionBinaryOperatorNotEqualTo,"ExpressionBinaryOperatorNotEqualTo",PREC_OP_NOTEQUALTO)
+PREIMPLEMENT_BINARY_OPERATOR(KviKvsTreeNodeExpressionBinaryOperatorNotEqualTo,"ExpressionBinaryOperatorNotEqualTo","Expression Binary Operator \"!=\"",PREC_OP_NOTEQUALTO)
 
 bool KviKvsTreeNodeExpressionBinaryOperatorNotEqualTo::evaluateReadOnly(KviKvsRunTimeContext * c,KviKvsVariant * pBuffer)
 {
@@ -791,29 +771,6 @@ bool KviKvsTreeNodeExpressionBinaryOperatorNotEqualTo::evaluateReadOnly(KviKvsRu
 	KviKvsVariant v2;
 	if(!m_pLeft->evaluateReadOnly(c,&v1))return false;
 	if(!m_pRight->evaluateReadOnly(c,&v2))return false;
-
-	bool bBothNumbers = true;
-	
-	if(!v1.asNumber(m_nLeft))bBothNumbers = false;
-	else if(!v2.asNumber(m_nRight))bBothNumbers = false;
-
-	if(bBothNumbers)
-	{
-		if(m_nLeft.isInteger())
-		{
-			if(m_nRight.isInteger())pBuffer->setInteger(m_nLeft.integer() != m_nRight.integer());
-			else pBuffer->setInteger(((double)(m_nLeft.integer())) != m_nRight.real());
-		} else {
-			if(m_nRight.isInteger())pBuffer->setInteger(m_nLeft.real() != ((double)(m_nRight.integer())));
-			else pBuffer->setInteger(m_nLeft.real() != m_nRight.real());
-		}
-	} else {
-		// at least one is not a number
-		QString sz1;
-		QString sz2;
-		v1.asString(sz1);
-		v2.asString(sz2);
-		pBuffer->setInteger(sz1 != sz2);
-	}
+	pBuffer->setBoolean(v1.compare(&v2,true) != 0);
 	return true;
 }
