@@ -29,6 +29,7 @@
 #include "kvi_kvs_runtimecontext.h"
 #include "kvi_locale.h"
 
+#include <qregexp.h>
 
 #include <math.h>
 
@@ -50,12 +51,17 @@ void KviKvsTreeNodeOperation::setTargetVariableReference(KviKvsTreeNodeData * r)
 	m_pTargetData->setParent(this);
 }
 
+void KviKvsTreeNodeOperation::contextDescription(QString &szBuffer)
+{
+	szBuffer = "Operation";
+}
+
 void KviKvsTreeNodeOperation::dump(const char * prefix)
 {
 	debug("%s Operation",prefix);
 	QString tmp = prefix;
 	tmp.append("  ");
-	m_pTargetData->dump(tmp.latin1());
+	m_pTargetData->dump(tmp.utf8().data());
 }
 
 
@@ -74,13 +80,19 @@ KviKvsTreeNodeOperationAssignment::~KviKvsTreeNodeOperationAssignment()
 	delete m_pRightSide;
 }
 
+void KviKvsTreeNodeOperationAssignment::contextDescription(QString &szBuffer)
+{
+	szBuffer = "Assignment";
+}
+
+
 void KviKvsTreeNodeOperationAssignment::dump(const char * prefix)
 {
 	debug("%s OperationAssignment",prefix);
 	QString tmp = prefix;
 	tmp.append("  ");
-	m_pTargetData->dump(tmp.latin1());
-	m_pRightSide->dump(tmp.latin1());
+	m_pTargetData->dump(tmp.utf8().data());
+	m_pRightSide->dump(tmp.utf8().data());
 }
 
 bool KviKvsTreeNodeOperationAssignment::execute(KviKvsRunTimeContext * c)
@@ -110,20 +122,31 @@ KviKvsTreeNodeOperationDecrement::~KviKvsTreeNodeOperationDecrement()
 {
 }
 
+void KviKvsTreeNodeOperationDecrement::contextDescription(QString &szBuffer)
+{
+#ifdef COMPILE_NEW_KVS
+	szBuffer = "Operator \"--\"";
+#endif
+}
+
+
 void KviKvsTreeNodeOperationDecrement::dump(const char * prefix)
 {
+#ifdef COMPILE_NEW_KVS
 	debug("%s OperationDecrement",prefix);
 	QString tmp = prefix;
 	tmp.append("  ");
-	m_pTargetData->dump(tmp.latin1());
+	m_pTargetData->dump(tmp.utf8().data());
+#endif
 }
 
 bool KviKvsTreeNodeOperationDecrement::execute(KviKvsRunTimeContext * c)
 {
+#ifdef COMPILE_NEW_KVS
 	KviKvsRWEvaluationResult * v = m_pTargetData->evaluateReadWrite(c);
 	if(!v)return false;
 
-	int iVal;
+	kvs_int_t iVal;
 	if(v->result()->asInteger(iVal))
 	{
 		v->result()->setInteger(iVal - 1);
@@ -132,7 +155,7 @@ bool KviKvsTreeNodeOperationDecrement::execute(KviKvsRunTimeContext * c)
 		return true;
 	}
 
-	double dVal;
+	kvs_real_t dVal;
 	if(v->result()->asReal(dVal))
 	{
 		v->result()->setReal(dVal - 1.0);
@@ -141,8 +164,9 @@ bool KviKvsTreeNodeOperationDecrement::execute(KviKvsRunTimeContext * c)
 		return true;
 	}
 
-	c->error(location(),__tr2qs("The target variable didn't evaluate to an integer or real value"));
+	c->error(this,__tr2qs("The target variable didn't evaluate to an integer or real value"));
 	delete v;
+#endif
 	return false;
 }
 
@@ -161,20 +185,31 @@ KviKvsTreeNodeOperationIncrement::~KviKvsTreeNodeOperationIncrement()
 {
 }
 
+void KviKvsTreeNodeOperationIncrement::contextDescription(QString &szBuffer)
+{
+#ifdef COMPILE_NEW_KVS
+	szBuffer = "Operator \"++\"";
+#endif
+}
+
+
 void KviKvsTreeNodeOperationIncrement::dump(const char * prefix)
 {
+#ifdef COMPILE_NEW_KVS
 	debug("%s OperationIncrement",prefix);
 	QString tmp = prefix;
 	tmp.append("  ");
-	m_pTargetData->dump(tmp.latin1());
+	m_pTargetData->dump(tmp.utf8().data());
+#endif
 }
 
 bool KviKvsTreeNodeOperationIncrement::execute(KviKvsRunTimeContext * c)
 {
+#ifdef COMPILE_NEW_KVS
 	KviKvsRWEvaluationResult * v = m_pTargetData->evaluateReadWrite(c);
 	if(!v)return false;
 
-	int iVal;
+	kvs_int_t iVal;
 	if(v->result()->asInteger(iVal))
 	{
 		v->result()->setInteger(iVal + 1);
@@ -183,7 +218,7 @@ bool KviKvsTreeNodeOperationIncrement::execute(KviKvsRunTimeContext * c)
 		return true;
 	}
 
-	double dVal;
+	kvs_real_t dVal;
 	if(v->result()->asReal(dVal))
 	{
 		v->result()->setReal(dVal + 1.0);
@@ -191,8 +226,9 @@ bool KviKvsTreeNodeOperationIncrement::execute(KviKvsRunTimeContext * c)
         v = 0;
 		return true;
 	}
-	c->error(location(),__tr2qs("The target variable didn't evaluate to an integer or real value"));
+	c->error(this,__tr2qs("The target variable didn't evaluate to an integer or real value"));
 	delete v;
+#endif
 	return false;
 }
 
@@ -202,45 +238,60 @@ bool KviKvsTreeNodeOperationIncrement::execute(KviKvsRunTimeContext * c)
 KviKvsTreeNodeOperationSelfAnd::KviKvsTreeNodeOperationSelfAnd(const QChar * pLocation,KviKvsTreeNodeData * pRightSide)
 : KviKvsTreeNodeOperation(pLocation)
 {
+#ifdef COMPILE_NEW_KVS
 	m_pRightSide = pRightSide;
 	m_pRightSide->setParent(this);
+#endif
 }
 
 KviKvsTreeNodeOperationSelfAnd::~KviKvsTreeNodeOperationSelfAnd()
 {
+#ifdef COMPILE_NEW_KVS
 	delete m_pRightSide;
+#endif
 }
 
+void KviKvsTreeNodeOperationSelfAnd::contextDescription(QString &szBuffer)
+{
+#ifdef COMPILE_NEW_KVS
+	szBuffer = "Operator \"&=\"";
+#endif
+}
+
+
 void KviKvsTreeNodeOperationSelfAnd::dump(const char * prefix)
 {
+#ifdef COMPILE_NEW_KVS
 	debug("%s OperationSelfAnd",prefix);
 	QString tmp = prefix;
 	tmp.append("  ");
-	m_pTargetData->dump(tmp.latin1());
-	m_pRightSide->dump(tmp.latin1());
+	m_pTargetData->dump(tmp.utf8().data());
+	m_pRightSide->dump(tmp.utf8().data());
+#endif
 }
 
 bool KviKvsTreeNodeOperationSelfAnd::execute(KviKvsRunTimeContext * c)
 {
+#ifdef COMPILE_NEW_KVS
 	KviKvsVariant v;
 	if(!m_pRightSide->evaluateReadOnly(c,&v))return false;
 
-	int iRVal;
+	kvs_int_t iRVal;
 
 	if(!v.asInteger(iRVal))
 	{
-		c->error(location(),__tr2qs("The right side of operator '&=' didn't evaluate to an integer"));
+		c->error(this,__tr2qs("The right side of operator '&=' didn't evaluate to an integer"));
 		return false;
 	}
 
 	KviKvsRWEvaluationResult * target = m_pTargetData->evaluateReadWrite(c);
 	if(!target)return false;
 	
-	int iLVal;
+	kvs_int_t iLVal;
 	
 	if(!target->result()->asInteger(iLVal))
 	{
-		c->error(location(),__tr2qs("The left side of operator '&=' didn't evaluate to an integer"));
+		c->error(this,__tr2qs("The left side of operator '&=' didn't evaluate to an integer"));
 		delete target;
         target = 0;
 		return false;
@@ -248,6 +299,7 @@ bool KviKvsTreeNodeOperationSelfAnd::execute(KviKvsRunTimeContext * c)
 	
 	target->result()->setInteger(iLVal & iRVal);
 	delete target;
+#endif
 	return true;
 }
 
@@ -256,26 +308,41 @@ bool KviKvsTreeNodeOperationSelfAnd::execute(KviKvsRunTimeContext * c)
 KviKvsTreeNodeOperationSelfDivision::KviKvsTreeNodeOperationSelfDivision(const QChar * pLocation,KviKvsTreeNodeData * pRightSide)
 : KviKvsTreeNodeOperation(pLocation)
 {
+#ifdef COMPILE_NEW_KVS
 	m_pRightSide = pRightSide;
 	m_pRightSide->setParent(this);
+#endif
 }
 
 KviKvsTreeNodeOperationSelfDivision::~KviKvsTreeNodeOperationSelfDivision()
 {
+#ifdef COMPILE_NEW_KVS
 	delete m_pRightSide;
+#endif
 }
 
+void KviKvsTreeNodeOperationSelfDivision::contextDescription(QString &szBuffer)
+{
+#ifdef COMPILE_NEW_KVS
+	szBuffer = "Operator \"/=\"";
+#endif
+}
+
+
 void KviKvsTreeNodeOperationSelfDivision::dump(const char * prefix)
 {
+#ifdef COMPILE_NEW_KVS
 	debug("%s OperationSelfDivision",prefix);
 	QString tmp = prefix;
 	tmp.append("  ");
-	m_pTargetData->dump(tmp.latin1());
-	m_pRightSide->dump(tmp.latin1());
+	m_pTargetData->dump(tmp.utf8().data());
+	m_pRightSide->dump(tmp.utf8().data());
+#endif
 }
 
 bool KviKvsTreeNodeOperationSelfDivision::execute(KviKvsRunTimeContext * c)
 {
+#ifdef COMPILE_NEW_KVS
 	KviKvsVariant v;
 	if(!m_pRightSide->evaluateReadOnly(c,&v))return false;
 
@@ -283,7 +350,7 @@ bool KviKvsTreeNodeOperationSelfDivision::execute(KviKvsRunTimeContext * c)
 
 	if(!v.asNumber(rnum))
 	{
-		c->error(location(),__tr2qs("The right side of operator '/=' didn't evaluate to a number"));
+		c->error(this,__tr2qs("The right side of operator '/=' didn't evaluate to a number"));
 		return false;
 	}
 	
@@ -294,7 +361,7 @@ bool KviKvsTreeNodeOperationSelfDivision::execute(KviKvsRunTimeContext * c)
 	
 	if(!target->result()->asNumber(lnum))
 	{
-		c->error(location(),__tr2qs("The left side of operator '/=' didn't evaluate to a number"));
+		c->error(this,__tr2qs("The left side of operator '/=' didn't evaluate to a number"));
 		delete target;
         target = 0;
 		return false;
@@ -304,7 +371,7 @@ bool KviKvsTreeNodeOperationSelfDivision::execute(KviKvsRunTimeContext * c)
 	{
 		if(rnum.integer() == 0)
 		{
-			c->error(location(),__tr2qs("Division by zero"));
+			c->error(this,__tr2qs("Division by zero"));
 			delete target;
             target = 0;
 			return false;
@@ -312,55 +379,71 @@ bool KviKvsTreeNodeOperationSelfDivision::execute(KviKvsRunTimeContext * c)
 		if(lnum.isInteger())
 			target->result()->setInteger(lnum.integer() / rnum.integer());
 		else
-			target->result()->setReal(lnum.real() / (double)(rnum.integer()));
+			target->result()->setReal(lnum.real() / (kvs_real_t)(rnum.integer()));
 	} else {
 		if(rnum.real() == 0.0)
 		{
-			c->error(location(),__tr2qs("Division by zero"));
+			c->error(this,__tr2qs("Division by zero"));
 			delete target;
             target = 0;
 			return false;
 		}
 		if(lnum.isInteger())
-			target->result()->setReal(((double)(lnum.integer())) / rnum.real());
+			target->result()->setReal(((kvs_real_t)(lnum.integer())) / rnum.real());
 		else
 			target->result()->setReal(lnum.real() / rnum.real());
 	}
 
 	delete target;
+#endif
 	return true;
 }
 
 KviKvsTreeNodeOperationSelfModulus::KviKvsTreeNodeOperationSelfModulus(const QChar * pLocation,KviKvsTreeNodeData * pRightSide)
 : KviKvsTreeNodeOperation(pLocation)
 {
+#ifdef COMPILE_NEW_KVS
 	m_pRightSide = pRightSide;
 	m_pRightSide->setParent(this);
+#endif
 }
 
 KviKvsTreeNodeOperationSelfModulus::~KviKvsTreeNodeOperationSelfModulus()
 {
+#ifdef COMPILE_NEW_KVS
 	delete m_pRightSide;
+#endif
 }
 
+void KviKvsTreeNodeOperationSelfModulus::contextDescription(QString &szBuffer)
+{
+#ifdef COMPILE_NEW_KVS
+	szBuffer = "Operator \"Self Modulo\"";
+#endif
+}
+
+
 void KviKvsTreeNodeOperationSelfModulus::dump(const char * prefix)
 {
+#ifdef COMPILE_NEW_KVS
 	debug("%s OperationSelfModulus",prefix);
 	QString tmp = prefix;
 	tmp.append("  ");
-	m_pTargetData->dump(tmp.latin1());
-	m_pRightSide->dump(tmp.latin1());
+	m_pTargetData->dump(tmp.utf8().data());
+	m_pRightSide->dump(tmp.utf8().data());
+#endif
 }
 
 bool KviKvsTreeNodeOperationSelfModulus::execute(KviKvsRunTimeContext * c)
 {
+#ifdef COMPILE_NEW_KVS
 	KviKvsVariant v;
 	if(!m_pRightSide->evaluateReadOnly(c,&v))return false;
 	KviKvsNumber rnum;
 
 	if(!v.asNumber(rnum))
 	{
-		c->error(location(),__tr2qs("The right side of operator '%=' didn't evaluate to a number"));
+		c->error(this,__tr2qs("The right side of operator '%=' didn't evaluate to a number"));
 		return false;
 	}
 	
@@ -371,7 +454,7 @@ bool KviKvsTreeNodeOperationSelfModulus::execute(KviKvsRunTimeContext * c)
 	
 	if(!target->result()->asNumber(lnum))
 	{
-		c->error(location(),__tr2qs("The left side of operator '%=' didn't evaluate to a number"));
+		c->error(this,__tr2qs("The left side of operator '%=' didn't evaluate to a number"));
 		delete target;
         target = 0;
 		return false;
@@ -381,7 +464,7 @@ bool KviKvsTreeNodeOperationSelfModulus::execute(KviKvsRunTimeContext * c)
 	{
 		if(rnum.integer() == 0)
 		{
-			c->error(location(),__tr2qs("Division by zero"));
+			c->error(this,__tr2qs("Division by zero"));
 			delete target;
             target = 0;
 			return false;
@@ -389,21 +472,21 @@ bool KviKvsTreeNodeOperationSelfModulus::execute(KviKvsRunTimeContext * c)
 		if(lnum.isInteger())
 			target->result()->setInteger(lnum.integer() % rnum.integer());
 		else
-			target->result()->setReal(fmod(lnum.real(),(double)(rnum.integer())));
+			target->result()->setReal(fmod(lnum.real(),(kvs_real_t)(rnum.integer())));
 	} else {
 		if(rnum.real() == 0.0)
 		{
-			c->error(location(),__tr2qs("Division by zero"));
+			c->error(this,__tr2qs("Division by zero"));
 			delete target;
             target = 0;
 			return false;
 		}
 		if(lnum.isInteger())
-			target->result()->setReal(fmod(((double)(lnum.integer())),rnum.real()));
+			target->result()->setReal(fmod(((kvs_real_t)(lnum.integer())),rnum.real()));
 		else
 			target->result()->setReal(fmod(lnum.real(),rnum.real()));
 	}
-
+#endif
 	return true;
 }
 
@@ -414,33 +497,48 @@ bool KviKvsTreeNodeOperationSelfModulus::execute(KviKvsRunTimeContext * c)
 KviKvsTreeNodeOperationSelfMultiplication::KviKvsTreeNodeOperationSelfMultiplication(const QChar * pLocation,KviKvsTreeNodeData * pRightSide)
 : KviKvsTreeNodeOperation(pLocation)
 {
+#ifdef COMPILE_NEW_KVS
 	m_pRightSide = pRightSide;
 	m_pRightSide->setParent(this);
+#endif
 }
 
 KviKvsTreeNodeOperationSelfMultiplication::~KviKvsTreeNodeOperationSelfMultiplication()
 {
+#ifdef COMPILE_NEW_KVS
 	delete m_pRightSide;
+#endif
 }
 
+void KviKvsTreeNodeOperationSelfMultiplication::contextDescription(QString &szBuffer)
+{
+#ifdef COMPILE_NEW_KVS
+	szBuffer = "Operator \"*=\"";
+#endif
+}
+
+
 void KviKvsTreeNodeOperationSelfMultiplication::dump(const char * prefix)
 {
+#ifdef COMPILE_NEW_KVS
 	debug("%s OperationSelfMultiplication",prefix);
 	QString tmp = prefix;
 	tmp.append("  ");
-	m_pTargetData->dump(tmp.latin1());
-	m_pRightSide->dump(tmp.latin1());
+	m_pTargetData->dump(tmp.utf8().data());
+	m_pRightSide->dump(tmp.utf8().data());
+#endif
 }
 
 bool KviKvsTreeNodeOperationSelfMultiplication::execute(KviKvsRunTimeContext * c)
 {
+#ifdef COMPILE_NEW_KVS
 	KviKvsVariant v;
 	if(!m_pRightSide->evaluateReadOnly(c,&v))return false;
 	KviKvsNumber rnum;
 
 	if(!v.asNumber(rnum))
 	{
-		c->error(location(),__tr2qs("The right side of operator '*=' didn't evaluate to a number"));
+		c->error(this,__tr2qs("The right side of operator '*=' didn't evaluate to a number"));
 		return false;
 	}
 	
@@ -451,7 +549,7 @@ bool KviKvsTreeNodeOperationSelfMultiplication::execute(KviKvsRunTimeContext * c
 	
 	if(!target->result()->asNumber(lnum))
 	{
-		c->error(location(),__tr2qs("The left side of operator '*=' didn't evaluate to a number"));
+		c->error(this,__tr2qs("The left side of operator '*=' didn't evaluate to a number"));
 		delete target;
 		return false;
 	}
@@ -461,13 +559,14 @@ bool KviKvsTreeNodeOperationSelfMultiplication::execute(KviKvsRunTimeContext * c
 		if(lnum.isInteger())
 			target->result()->setInteger(lnum.integer() * rnum.integer());
 		else
-			target->result()->setReal(lnum.real() * (double)(rnum.integer()));
+			target->result()->setReal(lnum.real() * (kvs_real_t)(rnum.integer()));
 	} else {
 		if(lnum.isInteger())
-			target->result()->setReal(((double)(lnum.integer())) * rnum.real());
+			target->result()->setReal(((kvs_real_t)(lnum.integer())) * rnum.real());
 		else
 			target->result()->setReal(lnum.real() * rnum.real());
 	}
+#endif
 	return true;
 }
 
@@ -477,44 +576,59 @@ bool KviKvsTreeNodeOperationSelfMultiplication::execute(KviKvsRunTimeContext * c
 KviKvsTreeNodeOperationSelfOr::KviKvsTreeNodeOperationSelfOr(const QChar * pLocation,KviKvsTreeNodeData * pRightSide)
 : KviKvsTreeNodeOperation(pLocation)
 {
+#ifdef COMPILE_NEW_KVS
 	m_pRightSide = pRightSide;
 	m_pRightSide->setParent(this);
+#endif
 }
 
 KviKvsTreeNodeOperationSelfOr::~KviKvsTreeNodeOperationSelfOr()
 {
+#ifdef COMPILE_NEW_KVS
 	delete m_pRightSide;
+#endif
 }
 
+void KviKvsTreeNodeOperationSelfOr::contextDescription(QString &szBuffer)
+{
+#ifdef COMPILE_NEW_KVS
+	szBuffer = "Operator \"|=\"";
+#endif
+}
+
+
 void KviKvsTreeNodeOperationSelfOr::dump(const char * prefix)
 {
+#ifdef COMPILE_NEW_KVS
 	debug("%s OperationSelfOr",prefix);
 	QString tmp = prefix;
 	tmp.append("  ");
-	m_pTargetData->dump(tmp.latin1());
-	m_pRightSide->dump(tmp.latin1());
+	m_pTargetData->dump(tmp.utf8().data());
+	m_pRightSide->dump(tmp.utf8().data());
+#endif
 }
 
 bool KviKvsTreeNodeOperationSelfOr::execute(KviKvsRunTimeContext * c)
 {
+#ifdef COMPILE_NEW_KVS
 	KviKvsVariant v;
 	if(!m_pRightSide->evaluateReadOnly(c,&v))return false;
 
-	int iRVal;
+	kvs_int_t iRVal;
 	if(!v.asInteger(iRVal))
 	{
-		c->error(location(),__tr2qs("The right side of operator '|=' didn't evaluate to an integer"));
+		c->error(this,__tr2qs("The right side of operator '|=' didn't evaluate to an integer"));
 		return false;
 	}
 	
-	int iLVal;
+	kvs_int_t iLVal;
 
 	KviKvsRWEvaluationResult * target = m_pTargetData->evaluateReadWrite(c);
 	if(!target)return false;
 	
 	if(!target->result()->asInteger(iLVal))
 	{
-		c->error(location(),__tr2qs("The left side of operator '|=' didn't evaluate to an integer"));
+		c->error(this,__tr2qs("The left side of operator '|=' didn't evaluate to an integer"));
 		delete target;
         target = 0;
 		return false;
@@ -522,6 +636,7 @@ bool KviKvsTreeNodeOperationSelfOr::execute(KviKvsRunTimeContext * c)
 	
 	target->result()->setInteger(iLVal | iRVal);
 	delete target;
+#endif
 	return true;
 }
 
@@ -531,45 +646,60 @@ bool KviKvsTreeNodeOperationSelfOr::execute(KviKvsRunTimeContext * c)
 KviKvsTreeNodeOperationSelfShl::KviKvsTreeNodeOperationSelfShl(const QChar * pLocation,KviKvsTreeNodeData * pRightSide)
 : KviKvsTreeNodeOperation(pLocation)
 {
+#ifdef COMPILE_NEW_KVS
 	m_pRightSide = pRightSide;
 	m_pRightSide->setParent(this);
+#endif
 }
 
 KviKvsTreeNodeOperationSelfShl::~KviKvsTreeNodeOperationSelfShl()
 {
+#ifdef COMPILE_NEW_KVS
 	delete m_pRightSide;
+#endif
 }
 
+void KviKvsTreeNodeOperationSelfShl::contextDescription(QString &szBuffer)
+{
+#ifdef COMPILE_NEW_KVS
+	szBuffer = "Operator \"<<=\"";
+#endif
+}
+
+
 void KviKvsTreeNodeOperationSelfShl::dump(const char * prefix)
 {
+#ifdef COMPILE_NEW_KVS
 	debug("%s OperationSelfShl",prefix);
 	QString tmp = prefix;
 	tmp.append("  ");
-	m_pTargetData->dump(tmp.latin1());
-	m_pRightSide->dump(tmp.latin1());
+	m_pTargetData->dump(tmp.utf8().data());
+	m_pRightSide->dump(tmp.utf8().data());
+#endif
 }
 
 bool KviKvsTreeNodeOperationSelfShl::execute(KviKvsRunTimeContext * c)
 {
+#ifdef COMPILE_NEW_KVS
 	KviKvsVariant v;
 	if(!m_pRightSide->evaluateReadOnly(c,&v))return false;
 
-	int iRVal;
+	kvs_int_t iRVal;
 
 	if(!v.asInteger(iRVal))
 	{
-		c->error(location(),__tr2qs("The right side of operator '<<=' didn't evaluate to an integer"));
+		c->error(this,__tr2qs("The right side of operator '<<=' didn't evaluate to an integer"));
 		return false;
 	}
 	
-	int iLVal;
+	kvs_int_t iLVal;
 
 	KviKvsRWEvaluationResult * target = m_pTargetData->evaluateReadWrite(c);
 	if(!target)return false;
 	
 	if(!target->result()->asInteger(iLVal))
 	{
-		c->error(location(),__tr2qs("The left side of operator '<<=' didn't evaluate to an integer"));
+		c->error(this,__tr2qs("The left side of operator '<<=' didn't evaluate to an integer"));
 		delete target;
         target = 0;
 		return false;
@@ -577,6 +707,7 @@ bool KviKvsTreeNodeOperationSelfShl::execute(KviKvsRunTimeContext * c)
 	
 	target->result()->setInteger(iLVal << iRVal);
 	delete target;
+#endif
 	return true;
 }
 
@@ -587,45 +718,60 @@ bool KviKvsTreeNodeOperationSelfShl::execute(KviKvsRunTimeContext * c)
 KviKvsTreeNodeOperationSelfShr::KviKvsTreeNodeOperationSelfShr(const QChar * pLocation,KviKvsTreeNodeData * pRightSide)
 : KviKvsTreeNodeOperation(pLocation)
 {
+#ifdef COMPILE_NEW_KVS
 	m_pRightSide = pRightSide;
 	m_pRightSide->setParent(this);
+#endif
 }
 
 KviKvsTreeNodeOperationSelfShr::~KviKvsTreeNodeOperationSelfShr()
 {
+#ifdef COMPILE_NEW_KVS
 	delete m_pRightSide;
+#endif
 }
 
+void KviKvsTreeNodeOperationSelfShr::contextDescription(QString &szBuffer)
+{
+#ifdef COMPILE_NEW_KVS
+	szBuffer = "Operator \">>=\"";
+#endif
+}
+
+
 void KviKvsTreeNodeOperationSelfShr::dump(const char * prefix)
 {
+#ifdef COMPILE_NEW_KVS
 	debug("%s OperationSelfShr",prefix);
 	QString tmp = prefix;
 	tmp.append("  ");
-	m_pTargetData->dump(tmp.latin1());
-	m_pRightSide->dump(tmp.latin1());
+	m_pTargetData->dump(tmp.utf8().data());
+	m_pRightSide->dump(tmp.utf8().data());
+#endif
 }
 
 bool KviKvsTreeNodeOperationSelfShr::execute(KviKvsRunTimeContext * c)
 {
+#ifdef COMPILE_NEW_KVS
 	KviKvsVariant v;
 	if(!m_pRightSide->evaluateReadOnly(c,&v))return false;
 
-	int iRVal;
+	kvs_int_t iRVal;
 
 	if(!v.asInteger(iRVal))
 	{
-		c->error(location(),__tr2qs("The right side of operator '>>=' didn't evaluate to an integer"));
+		c->error(this,__tr2qs("The right side of operator '>>=' didn't evaluate to an integer"));
 		return false;
 	}
 	
-	int iLVal;
+	kvs_int_t iLVal;
 
 	KviKvsRWEvaluationResult * target = m_pTargetData->evaluateReadWrite(c);
 	if(!target)return false;
 	
 	if(!target->result()->asInteger(iLVal))
 	{
-		c->error(location(),__tr2qs("The left side of operator '>>=' didn't evaluate to an integer"));
+		c->error(this,__tr2qs("The left side of operator '>>=' didn't evaluate to an integer"));
 		delete target;
         target = 0;
 		return false;
@@ -633,6 +779,7 @@ bool KviKvsTreeNodeOperationSelfShr::execute(KviKvsRunTimeContext * c)
 	
 	target->result()->setInteger(iLVal >> iRVal);
 	delete target;
+#endif
 	return true;
 }
 
@@ -649,33 +796,48 @@ bool KviKvsTreeNodeOperationSelfShr::execute(KviKvsRunTimeContext * c)
 KviKvsTreeNodeOperationSelfSubtraction::KviKvsTreeNodeOperationSelfSubtraction(const QChar * pLocation,KviKvsTreeNodeData * pRightSide)
 : KviKvsTreeNodeOperation(pLocation)
 {
+#ifdef COMPILE_NEW_KVS
 	m_pRightSide = pRightSide;
 	m_pRightSide->setParent(this);
+#endif
 }
 
 KviKvsTreeNodeOperationSelfSubtraction::~KviKvsTreeNodeOperationSelfSubtraction()
 {
+#ifdef COMPILE_NEW_KVS
 	delete m_pRightSide;
+#endif
 }
 
+void KviKvsTreeNodeOperationSelfSubtraction::contextDescription(QString &szBuffer)
+{
+#ifdef COMPILE_NEW_KVS
+	szBuffer = "Operator \"-=\"";
+#endif
+}
+
+
 void KviKvsTreeNodeOperationSelfSubtraction::dump(const char * prefix)
 {
+#ifdef COMPILE_NEW_KVS
 	debug("%s OperationSelfSubtraction",prefix);
 	QString tmp = prefix;
 	tmp.append("  ");
-	m_pTargetData->dump(tmp.latin1());
-	m_pRightSide->dump(tmp.latin1());
+	m_pTargetData->dump(tmp.utf8().data());
+	m_pRightSide->dump(tmp.utf8().data());
+#endif
 }
 
 bool KviKvsTreeNodeOperationSelfSubtraction::execute(KviKvsRunTimeContext * c)
 {
+#ifdef COMPILE_NEW_KVS
 	KviKvsVariant v;
 	if(!m_pRightSide->evaluateReadOnly(c,&v))return false;
 	KviKvsNumber rnum;
 
 	if(!v.asNumber(rnum))
 	{
-		c->error(location(),__tr2qs("The right side of operator '-=' didn't evaluate to a number"));
+		c->error(this,__tr2qs("The right side of operator '-=' didn't evaluate to a number"));
 		return false;
 	}
 	
@@ -686,7 +848,7 @@ bool KviKvsTreeNodeOperationSelfSubtraction::execute(KviKvsRunTimeContext * c)
 	
 	if(!target->result()->asNumber(lnum))
 	{
-		c->error(location(),__tr2qs("The left side of operator '-=' didn't evaluate to a number"));
+		c->error(this,__tr2qs("The left side of operator '-=' didn't evaluate to a number"));
 		delete target;
         target = 0;
 		return false;
@@ -697,15 +859,16 @@ bool KviKvsTreeNodeOperationSelfSubtraction::execute(KviKvsRunTimeContext * c)
 		if(lnum.isInteger())
 			target->result()->setInteger(lnum.integer() - rnum.integer());
 		else
-			target->result()->setReal(lnum.real() - (double)(rnum.integer()));
+			target->result()->setReal(lnum.real() - (kvs_real_t)(rnum.integer()));
 	} else {
 		if(lnum.isInteger())
-			target->result()->setReal(((double)(lnum.integer())) - rnum.real());
+			target->result()->setReal(((kvs_real_t)(lnum.integer())) - rnum.real());
 		else
 			target->result()->setReal(lnum.real() - rnum.real());
 	}
 
 	delete target;
+#endif
 	return true;
 }
 
@@ -716,33 +879,48 @@ bool KviKvsTreeNodeOperationSelfSubtraction::execute(KviKvsRunTimeContext * c)
 KviKvsTreeNodeOperationSelfSum::KviKvsTreeNodeOperationSelfSum(const QChar * pLocation,KviKvsTreeNodeData * pRightSide)
 : KviKvsTreeNodeOperation(pLocation)
 {
+#ifdef COMPILE_NEW_KVS
 	m_pRightSide = pRightSide;
 	m_pRightSide->setParent(this);
+#endif
 }
 
 KviKvsTreeNodeOperationSelfSum::~KviKvsTreeNodeOperationSelfSum()
 {
+#ifdef COMPILE_NEW_KVS
 	delete m_pRightSide;
+#endif
 }
 
+void KviKvsTreeNodeOperationSelfSum::contextDescription(QString &szBuffer)
+{
+#ifdef COMPILE_NEW_KVS
+	szBuffer = "Operator \"+=\"";
+#endif
+}
+
+
 void KviKvsTreeNodeOperationSelfSum::dump(const char * prefix)
 {
+#ifdef COMPILE_NEW_KVS
 	debug("%s OperationSelfSum",prefix);
 	QString tmp = prefix;
 	tmp.append("  ");
-	m_pTargetData->dump(tmp.latin1());
-	m_pRightSide->dump(tmp.latin1());
+	m_pTargetData->dump(tmp.utf8().data());
+	m_pRightSide->dump(tmp.utf8().data());
+#endif
 }
 
 bool KviKvsTreeNodeOperationSelfSum::execute(KviKvsRunTimeContext * c)
 {
+#ifdef COMPILE_NEW_KVS
 	KviKvsVariant v;
 	if(!m_pRightSide->evaluateReadOnly(c,&v))return false;
 	KviKvsNumber rnum;
 
 	if(!v.asNumber(rnum))
 	{
-		c->error(location(),__tr2qs("The right side of operator '+=' didn't evaluate to a number"));
+		c->error(this,__tr2qs("The right side of operator '+=' didn't evaluate to a number"));
 		return false;
 	}
 	
@@ -753,7 +931,7 @@ bool KviKvsTreeNodeOperationSelfSum::execute(KviKvsRunTimeContext * c)
 	
 	if(!target->result()->asNumber(lnum))
 	{
-		c->error(location(),__tr2qs("The left side of operator '+=' didn't evaluate to a number"));
+		c->error(this,__tr2qs("The left side of operator '+=' didn't evaluate to a number"));
 		delete target;
         target = 0;
 		return false;
@@ -764,15 +942,16 @@ bool KviKvsTreeNodeOperationSelfSum::execute(KviKvsRunTimeContext * c)
 		if(lnum.isInteger())
 			target->result()->setInteger(lnum.integer() + rnum.integer());
 		else
-			target->result()->setReal(lnum.real() + (double)(rnum.integer()));
+			target->result()->setReal(lnum.real() + (kvs_real_t)(rnum.integer()));
 	} else {
 		if(lnum.isInteger())
-			target->result()->setReal(((double)(lnum.integer())) + rnum.real());
+			target->result()->setReal(((kvs_real_t)(lnum.integer())) + rnum.real());
 		else
 			target->result()->setReal(lnum.real() + rnum.real());
 	}
 
 	delete target;
+#endif
 	return true;
 }
 
@@ -785,45 +964,60 @@ bool KviKvsTreeNodeOperationSelfSum::execute(KviKvsRunTimeContext * c)
 KviKvsTreeNodeOperationSelfXor::KviKvsTreeNodeOperationSelfXor(const QChar * pLocation,KviKvsTreeNodeData * pRightSide)
 : KviKvsTreeNodeOperation(pLocation)
 {
+#ifdef COMPILE_NEW_KVS
 	m_pRightSide = pRightSide;
 	m_pRightSide->setParent(this);
+#endif
 }
 
 KviKvsTreeNodeOperationSelfXor::~KviKvsTreeNodeOperationSelfXor()
 {
+#ifdef COMPILE_NEW_KVS
 	delete m_pRightSide;
+#endif
 }
 
+void KviKvsTreeNodeOperationSelfXor::contextDescription(QString &szBuffer)
+{
+#ifdef COMPILE_NEW_KVS
+	szBuffer = "Operator \"^=\"";
+#endif
+}
+
+
 void KviKvsTreeNodeOperationSelfXor::dump(const char * prefix)
 {
+#ifdef COMPILE_NEW_KVS
 	debug("%s OperationSelfXor",prefix);
 	QString tmp = prefix;
 	tmp.append("  ");
-	m_pTargetData->dump(tmp.latin1());
-	m_pRightSide->dump(tmp.latin1());
+	m_pTargetData->dump(tmp.utf8().data());
+	m_pRightSide->dump(tmp.utf8().data());
+#endif
 }
 
 bool KviKvsTreeNodeOperationSelfXor::execute(KviKvsRunTimeContext * c)
 {
+#ifdef COMPILE_NEW_KVS
 	KviKvsVariant v;
 	if(!m_pRightSide->evaluateReadOnly(c,&v))return false;
 
-	int iRVal;
+	kvs_int_t iRVal;
 
 	if(!v.asInteger(iRVal))
 	{
-		c->error(location(),__tr2qs("The right side of operator '^=' didn't evaluate to an integer"));
+		c->error(this,__tr2qs("The right side of operator '^=' didn't evaluate to an integer"));
 		return false;
 	}
 	
-	int iLVal;
+	kvs_int_t iLVal;
 
 	KviKvsRWEvaluationResult * target = m_pTargetData->evaluateReadWrite(c);
 	if(!target)return false;
 	
 	if(!target->result()->asInteger(iLVal))
 	{
-		c->error(location(),__tr2qs("The left side of operator '^=' didn't evaluate to an integer"));
+		c->error(this,__tr2qs("The left side of operator '^=' didn't evaluate to an integer"));
 		delete target;
         target = 0;
 		return false;
@@ -831,6 +1025,7 @@ bool KviKvsTreeNodeOperationSelfXor::execute(KviKvsRunTimeContext * c)
 	
 	target->result()->setInteger(iLVal ^ iRVal);
 	delete target;
+#endif
 	return true;
 }
 
@@ -852,13 +1047,19 @@ KviKvsTreeNodeOperationStringAppend::~KviKvsTreeNodeOperationStringAppend()
 	delete m_pRightSide;
 }
 
+void KviKvsTreeNodeOperationStringAppend::contextDescription(QString &szBuffer)
+{
+	szBuffer = "Operator \"<<\"";
+}
+
+
 void KviKvsTreeNodeOperationStringAppend::dump(const char * prefix)
 {
 	debug("%s OperationStringAppend",prefix);
 	QString tmp = prefix;
 	tmp.append("  ");
-	m_pTargetData->dump(tmp.latin1());
-	m_pRightSide->dump(tmp.latin1());
+	m_pTargetData->dump(tmp.utf8().data());
+	m_pRightSide->dump(tmp.utf8().data());
 }
 
 bool KviKvsTreeNodeOperationStringAppend::execute(KviKvsRunTimeContext * c)
@@ -881,30 +1082,127 @@ bool KviKvsTreeNodeOperationStringAppend::execute(KviKvsRunTimeContext * c)
 
 
 
-
-KviKvsTreeNodeOperationStringAppendWithComma::KviKvsTreeNodeOperationStringAppendWithComma(const QChar * pLocation,KviKvsTreeNodeData * pRightSide)
+KviKvsTreeNodeOperationArrayAppend::KviKvsTreeNodeOperationArrayAppend(const QChar * pLocation,KviKvsTreeNodeData * pRightSide)
 : KviKvsTreeNodeOperation(pLocation)
 {
 	m_pRightSide = pRightSide;
 	m_pRightSide->setParent(this);
 }
 
-KviKvsTreeNodeOperationStringAppendWithComma::~KviKvsTreeNodeOperationStringAppendWithComma()
+KviKvsTreeNodeOperationArrayAppend::~KviKvsTreeNodeOperationArrayAppend()
 {
 	delete m_pRightSide;
 }
 
+void KviKvsTreeNodeOperationArrayAppend::contextDescription(QString &szBuffer)
+{
+	szBuffer = "Operator \"<+\"";
+}
+
+
+void KviKvsTreeNodeOperationArrayAppend::dump(const char * prefix)
+{
+	debug("%s OperationArrayAppend",prefix);
+	QString tmp = prefix;
+	tmp.append("  ");
+	m_pTargetData->dump(tmp.utf8().data());
+	m_pRightSide->dump(tmp.utf8().data());
+}
+
+bool KviKvsTreeNodeOperationArrayAppend::execute(KviKvsRunTimeContext * c)
+{
+	KviKvsVariant v;
+	if(!m_pRightSide->evaluateReadOnly(c,&v))return false;
+	KviKvsRWEvaluationResult * target = m_pTargetData->evaluateReadWrite(c);
+	if(!target)return false;
+
+	target->result()->convertToArray();
+	KviKvsArray * a = target->result()->array();
+
+	switch(v.type())
+	{
+		case KviKvsVariantData::Nothing:
+			// do nothing
+		break;
+		case KviKvsVariantData::Array:
+		{
+			KviKvsArray * s = v.array();
+			unsigned int uIdx = 0;
+			unsigned int uSize = s->size();
+			while(uIdx < uSize)
+			{
+				KviKvsVariant * pInternal = s->at(uIdx);
+				if(pInternal)
+					a->set(a->size(),new KviKvsVariant(*pInternal));
+				// else
+				//	leave an empty entry
+				uIdx++;
+			}
+		}
+		break;
+		case KviKvsVariantData::Hash:
+		{
+			KviKvsHashIterator it(*(v.hash()->dict()));
+			while(KviKvsVariant * pInternal = it.current())
+			{
+				a->set(a->size(),new KviKvsVariant(*pInternal));
+				++it;
+			}
+		}
+		break;
+		default:
+			a->set(a->size(),new KviKvsVariant(v));
+		break;
+	}
+	delete target;
+
+	return true;
+}
+
+
+
+
+
+
+
+KviKvsTreeNodeOperationStringAppendWithComma::KviKvsTreeNodeOperationStringAppendWithComma(const QChar * pLocation,KviKvsTreeNodeData * pRightSide)
+: KviKvsTreeNodeOperation(pLocation)
+{
+#ifdef COMPILE_NEW_KVS
+	m_pRightSide = pRightSide;
+	m_pRightSide->setParent(this);
+#endif
+}
+
+KviKvsTreeNodeOperationStringAppendWithComma::~KviKvsTreeNodeOperationStringAppendWithComma()
+{
+#ifdef COMPILE_NEW_KVS
+	delete m_pRightSide;
+#endif
+}
+
+void KviKvsTreeNodeOperationStringAppendWithComma::contextDescription(QString &szBuffer)
+{
+#ifdef COMPILE_NEW_KVS
+	szBuffer = "Operator \"<,\"";
+#endif
+}
+
+
 void KviKvsTreeNodeOperationStringAppendWithComma::dump(const char * prefix)
 {
+#ifdef COMPILE_NEW_KVS
 	debug("%s OperationStringAppendWithComma",prefix);
 	QString tmp = prefix;
 	tmp.append("  ");
-	m_pTargetData->dump(tmp.latin1());
-	m_pRightSide->dump(tmp.latin1());
+	m_pTargetData->dump(tmp.utf8().data());
+	m_pRightSide->dump(tmp.utf8().data());
+#endif
 }
 
 bool KviKvsTreeNodeOperationStringAppendWithComma::execute(KviKvsRunTimeContext * c)
 {
+#ifdef COMPILE_NEW_KVS
 	KviKvsVariant v;
 	if(!m_pRightSide->evaluateReadOnly(c,&v))return false;
 	KviKvsRWEvaluationResult * target = m_pTargetData->evaluateReadWrite(c);
@@ -923,6 +1221,7 @@ bool KviKvsTreeNodeOperationStringAppendWithComma::execute(KviKvsRunTimeContext
 	}
 	target->result()->setString(sz1);
 	delete target;
+#endif
 	return true;
 }
 
@@ -934,26 +1233,41 @@ bool KviKvsTreeNodeOperationStringAppendWithComma::execute(KviKvsRunTimeContext
 KviKvsTreeNodeOperationStringAppendWithSpace::KviKvsTreeNodeOperationStringAppendWithSpace(const QChar * pLocation,KviKvsTreeNodeData * pRightSide)
 : KviKvsTreeNodeOperation(pLocation)
 {
+#ifdef COMPILE_NEW_KVS
 	m_pRightSide = pRightSide;
 	m_pRightSide->setParent(this);
+#endif
 }
 
 KviKvsTreeNodeOperationStringAppendWithSpace::~KviKvsTreeNodeOperationStringAppendWithSpace()
 {
+#ifdef COMPILE_NEW_KVS
 	delete m_pRightSide;
+#endif
 }
 
+void KviKvsTreeNodeOperationStringAppendWithSpace::contextDescription(QString &szBuffer)
+{
+#ifdef COMPILE_NEW_KVS
+	szBuffer = "Operator \"<+\"";
+#endif
+}
+
+
 void KviKvsTreeNodeOperationStringAppendWithSpace::dump(const char * prefix)
 {
+#ifdef COMPILE_NEW_KVS
 	debug("%s OperationStringAppendWithSpace",prefix);
 	QString tmp = prefix;
 	tmp.append("  ");
-	m_pTargetData->dump(tmp.latin1());
-	m_pRightSide->dump(tmp.latin1());
+	m_pTargetData->dump(tmp.utf8().data());
+	m_pRightSide->dump(tmp.utf8().data());
+#endif
 }
 
 bool KviKvsTreeNodeOperationStringAppendWithSpace::execute(KviKvsRunTimeContext * c)
 {
+#ifdef COMPILE_NEW_KVS
 	KviKvsVariant v;
 	if(!m_pRightSide->evaluateReadOnly(c,&v))return false;
 	KviKvsRWEvaluationResult * target = m_pTargetData->evaluateReadWrite(c);
@@ -972,5 +1286,212 @@ bool KviKvsTreeNodeOperationStringAppendWithSpace::execute(KviKvsRunTimeContext
 	}
 	target->result()->setString(sz1);
 	delete target;
+#endif
 	return true;
 }
+
+
+
+
+
+
+
+
+
+KviKvsTreeNodeOperationStringTransliteration::KviKvsTreeNodeOperationStringTransliteration(const QChar * pLocation,KviKvsTreeNodeData * pLeft,KviKvsTreeNodeData * pRight,KviKvsTreeNodeData * pFlags)
+: KviKvsTreeNodeOperation(pLocation)
+{
+#ifdef COMPILE_NEW_KVS
+	m_pLeft = pLeft;
+	m_pLeft->setParent(this);
+	m_pRight = pRight;
+	m_pRight->setParent(this);
+	m_pFlags = pFlags;
+	m_pFlags->setParent(this);
+#endif
+}
+
+KviKvsTreeNodeOperationStringTransliteration::~KviKvsTreeNodeOperationStringTransliteration()
+{
+#ifdef COMPILE_NEW_KVS
+	delete m_pLeft;
+	delete m_pRight;
+	delete m_pFlags;
+#endif
+}
+
+
+void KviKvsTreeNodeOperationStringTransliteration::contextDescription(QString &szBuffer)
+{
+#ifdef COMPILE_NEW_KVS
+	szBuffer = "Binding Operator tr///";
+#endif
+}
+
+
+void KviKvsTreeNodeOperationStringTransliteration::dump(const char * prefix)
+{
+#ifdef COMPILE_NEW_KVS
+	debug("%s OperationStringTransliteration",prefix);
+	QString tmp = prefix;
+	tmp.append("  ");
+	m_pLeft->dump(tmp.utf8().data());
+	m_pRight->dump(tmp.utf8().data());
+	m_pFlags->dump(tmp.utf8().data());
+#endif
+}
+
+bool KviKvsTreeNodeOperationStringTransliteration::execute(KviKvsRunTimeContext * c)
+{
+#ifdef COMPILE_NEW_KVS
+	KviKvsVariant vl;
+	if(!m_pLeft->evaluateReadOnly(c,&vl))return false;
+	KviKvsVariant vr;
+	if(!m_pRight->evaluateReadOnly(c,&vr))return false;
+
+	// for now we don't use the flags at all here...
+	//KviKvsVariant vf;
+	//if(!m_pFlags->evaluateReadOnly(c,&vf))return false;
+	KviKvsRWEvaluationResult * target = m_pTargetData->evaluateReadWrite(c);
+	if(!target)return false;
+
+	QString szL,szR,szF;
+
+	vl.asString(szL);
+	vr.asString(szR);
+	//vf.asString(szF);
+
+	QString sz1;
+	target->result()->asString(sz1);
+	KviQString::transliterate(sz1,szL,szR);
+	target->result()->setString(sz1);
+	delete target;
+#endif
+	return true;
+}
+
+
+
+
+
+
+
+
+
+
+KviKvsTreeNodeOperationStringSubstitution::KviKvsTreeNodeOperationStringSubstitution(const QChar * pLocation,KviKvsTreeNodeData * pLeft,KviKvsTreeNodeData * pRight,KviKvsTreeNodeData * pFlags)
+: KviKvsTreeNodeOperation(pLocation)
+{
+#ifdef COMPILE_NEW_KVS
+	m_pLeft = pLeft;
+	m_pLeft->setParent(this);
+	m_pRight = pRight;
+	m_pRight->setParent(this);
+	m_pFlags = pFlags;
+	m_pFlags->setParent(this);
+#endif
+}
+
+KviKvsTreeNodeOperationStringSubstitution::~KviKvsTreeNodeOperationStringSubstitution()
+{
+#ifdef COMPILE_NEW_KVS
+	delete m_pLeft;
+	delete m_pRight;
+	delete m_pFlags;
+#endif
+}
+
+
+void KviKvsTreeNodeOperationStringSubstitution::contextDescription(QString &szBuffer)
+{
+#ifdef COMPILE_NEW_KVS
+	szBuffer = "Binding Operator s///";
+#endif
+}
+
+
+void KviKvsTreeNodeOperationStringSubstitution::dump(const char * prefix)
+{
+#ifdef COMPILE_NEW_KVS
+	debug("%s OperationStringSubstitution",prefix);
+	QString tmp = prefix;
+	tmp.append("  ");
+	m_pLeft->dump(tmp.utf8().data());
+	m_pRight->dump(tmp.utf8().data());
+	m_pFlags->dump(tmp.utf8().data());
+#endif
+}
+
+bool KviKvsTreeNodeOperationStringSubstitution::execute(KviKvsRunTimeContext * c)
+{
+#ifdef COMPILE_NEW_KVS
+	KviKvsVariant vl;
+	if(!m_pLeft->evaluateReadOnly(c,&vl))return false;
+	KviKvsVariant vr;
+	if(!m_pRight->evaluateReadOnly(c,&vr))return false;
+	KviKvsVariant vf;
+	if(!m_pFlags->evaluateReadOnly(c,&vf))return false;
+	KviKvsRWEvaluationResult * target = m_pTargetData->evaluateReadWrite(c);
+	if(!target)return false;
+
+	QString szL,szR,szF;
+
+	vl.asString(szL);
+	vr.asString(szR);
+	vf.asString(szF);
+
+	QString str;
+	target->result()->asString(str);
+
+	bool bGlobal = szF.find('g',false) != -1;
+
+	QRegExp re(szL,szF.find('i',false) == -1,szF.find('w',false) != -1);
+	re.setMinimal(szF.find('m',false) != -1); // greedy or minimal ?
+
+	int idx = 0;
+
+	while((!str.isEmpty()) && (idx != -1))
+	{
+		int len;
+		idx = re.search(str,idx);
+		if(idx != -1)
+		{
+			len = re.matchedLength();
+			if(len == 0)
+			{
+				c->warning(this,__tr2qs("The regular expression matched a zero-length substring: this is not valid in the s/// operator, skipping"));
+				goto end_of_this_stuff;
+			}
+			str.remove(idx,len);
+
+			QString szReplaced;
+			const QChar * p2 = KviQString::nullTerminatedArray(szR);
+			const QChar * begin = p2;
+			while(p2->unicode())
+			{
+				if(p2->unicode() == '\\')
+				{
+					p2++;
+					if(p2->unicode() >= '0' && p2->unicode() <= '9')
+					{
+						szReplaced.append(QString(begin,(p2 - begin) - 1));
+						szReplaced.append(re.cap(p2->unicode() - '0'));
+						p2++;
+						begin = p2;
+					} else if(p2->unicode())p2++;
+				} else p2++;
+			}
+			if(begin != p2)szReplaced.append(QString(begin,p2 - begin));
+			str.insert(idx,szReplaced);
+			idx += szReplaced.length();
+
+			if(!bGlobal)goto end_of_this_stuff;
+		}
+	}
+end_of_this_stuff:
+	target->result()->setString(str);
+	delete target;
+#endif
+	return true;
+}
+
