@@ -72,6 +72,7 @@
 //      RPL_????????   11,12,13
 #define RPL_YOURCOOKIE       14 /* <cookie> :is your reconnection cookie                 [E,i?]    # */
 
+#define RPL_CONNECTING 	     20 //:irc.dotsrc.org 020 * :Please wait while we process your connection.
 
 
 
@@ -100,7 +101,7 @@
 #define RPL_STATSQLINE       217 /* [I,E,U,D] */
 #define RPL_STATSYLINE       218 /* [I,E,U,D] */
 #define RPL_ENDOFSTATS	     219 //	[I,E,U,D] */
-//      RPL_??????????       220
+// #define RPL_????       220 
 #define RPL_UMODEIS          221 /* <nick> <mode>                                            [I,E,U,D] # */
 #define RPL_STATSBLINE       222 /* [E] */
 #define RPL_SQLINE_NICK      222 /* [D] */
@@ -201,7 +202,7 @@
 #define RPL_CHANURL          328 /* [D] [services.dal.net][328] Pragma #linux :http://dalnet.linuxguru.net */
 //[04:25:18] [services.dal.net][328] rettter #cebu :none.
 #define RPL_CREATIONTIME     329 /* "<channel> %lu"                                        [E,U,D] # */
-#define RPL_WHOISAUTH        330 /* %s :is authed as ....                               [Quakenet] # */
+#define RPL_WHOISAUTH        330 /* %s %s :is authed as ....                            [Quakenet] # */
 #define RPL_NOTOPIC          331 /* "<channel> :No topic is set."                        [I,E,U,D] # */
 #define RPL_TOPIC            332 /* "<channel> :%s"                                      [I,E,U,D] # */
 #define RPL_TOPICWHOTIME     333 /* "<channel> <whoset> %lu"                               [e,U,D] # */
@@ -366,7 +367,7 @@
 #define RPL_ENDOFWATCH       607 /* :End of WATCH <char l=list s=stats>                         [D] # */
 // WeNet codepage extension
 #define RPL_CODEPAGESET      700 /* :irc.wenet.ru 700 Pragma CP1251 :is now your translation scheme # */
-
+#define RPL_WHOISSCHEME      703 /* :interdacom.vgd.wenet.ru 703 Alexey Begemot KOI8-R :translation scheme */
 
 ///* 303 */	RPL_ISON, ":",
 ///* 304 */	RPL_TEXT, (char *)NULL,
