@@ -26,6 +26,8 @@
 // FIXME: #warning "CTCP BEEP == WAKEUP == AWAKE"
 // FIXME: #warning "CTCP AVATARREQ or QUERYAVATAR"
 
+#include "kvi_mirccntrl.h"
+#include "kvi_osinfo.h"
 #include "kvi_app.h"
 #include "kvi_sparser.h"
 #include "kvi_window.h"
@@ -34,7 +36,6 @@
 #include "kvi_ircsocket.h"
 #include "kvi_channel.h"
 #include "kvi_defaults.h"
-//#include "kvi_ircmask.h"
 #include "kvi_channel.h"
 #include "kvi_query.h"
 #include "kvi_ircuserdb.h"
@@ -43,20 +44,24 @@
 #include "kvi_sharedfiles.h"
 #include "kvi_time.h"
 #include "kvi_fileutils.h"
-#include "kvi_event.h"
 #include "kvi_ctcppagedialog.h"
 #include "kvi_useraction.h"
-#include "kvi_uparser.h"
 #include "kvi_options.h"
 #include "kvi_ircconnection.h"
 #include "kvi_ircconnectionuserinfo.h"
 #include "kvi_ircconnectionantictcpflooddata.h"
 #include "kvi_lagmeter.h"
+#include "kvi_kvs_eventtriggers.h"
+#include "kvi_kvs_script.h"
+#include "kvi_sourcesdate.h"
 
 #include <stdlib.h>
 
 #include <qdatetime.h>
 
+
+
+
 extern KVIRC_API KviSharedFilesManager * g_pSharedFilesManager;
 extern KVIRC_API KviCtcpPageDialog * g_pCtcpPageDialog;
 
@@ -378,11 +383,165 @@ extern KVIRC_API KviCtcpPageDialog * g_pCtcpPageDialog;
 */
 
 
+
+void KviServerParser::encodeCtcpParameter(const char * param,KviStr &buffer,bool bSpaceBreaks)
+{
+	//
+	// This one encodes a single ctcp parameter with the simplest
+	// subset of rules and places it in the supplied buffer
+	//
+	if(!(*param))
+	{
+		// empty parameter: the only reason we REALLY need the double quotes
+		if(bSpaceBreaks)
+			buffer.append("\"\"");
+		return;
+	}
+
+	const char * begin = param;
+	
+	bool bMustQuote = false;
+
+	while(*param)
+	{
+		switch(*param)
+		{
+			case ' ':
+				if(bSpaceBreaks)
+				{
+					if(param != begin)buffer.append(begin,param - begin);
+					buffer.append("\\040");
+					param++;
+					begin = param;
+				} else {
+					// space is non breaking (last parameter)
+					param++;
+				}
+			break;
+			case '\r':
+				if(param != begin)buffer.append(begin,param - begin);
+				buffer.append("\\015");
+				param++;
+				begin = param;
+			break;
+			case '\n':
+				if(param != begin)buffer.append(begin,param - begin);
+				buffer.append("\\012");
+				param++;
+				begin = param;
+			break;
+			case '"':
+				if(param != begin)buffer.append(begin,param - begin);
+				buffer.append("\\042");
+				param++;
+				begin = param;
+			break;
+			case '\\':
+				if(param != begin)buffer.append(begin,param - begin);
+				buffer.append("\\143");
+				param++;
+				begin = param;
+			break;
+			case 0x01:
+				if(param != begin)buffer.append(begin,param - begin);
+				buffer.append("\\001");
+				param++;
+				begin = param;
+			break;
+			default:
+				param++;
+			break;
+		}
+	}
+	
+	if(param != begin)buffer.append(begin,param - begin);
+}
+
+void KviServerParser::encodeCtcpParameter(const char * parametr,QString &resultBuffer,bool bSpaceBreaks)
+{
+	//
+	// This one encodes a single ctcp parameter with the simplest
+	// subset of rules and places it in the supplied buffer
+	//
+	QCString buffer;
+	register const char* param=parametr;
+	if(!param)
+	{
+		if(bSpaceBreaks)
+			buffer.append("\"\"");
+		return;
+	}
+	if(!(*param))
+	{
+		// empty parameter: the only reason we REALLY need the double quotes
+		if(bSpaceBreaks)
+			buffer.append("\"\"");
+		return;
+	}
+
+	bool bMustQuote = false;
+
+	while(*param)
+	{
+		switch(*param)
+		{
+			case ' ':
+				if(bSpaceBreaks)
+				{
+					buffer.append("\\040");
+					param++;
+				} else {
+					buffer+=*param;
+					param++;
+				}
+			break;
+			case '\r':
+				buffer.append("\\015");
+				param++;
+			break;
+			case '\n':
+				buffer.append("\\012");
+				param++;
+			break;
+			case '"':
+				buffer.append("\\042");
+				param++;
+			break;
+			case '\\':
+				buffer.append("\\143");
+				param++;
+			break;
+			case 0x01:
+				buffer.append("\\001");
+				param++;
+			break;
+			default:
+				buffer+=*param;
+				param++;
+			break;
+		}
+	}
+	resultBuffer=buffer;
+}
+
+
+
 const char * KviServerParser::decodeCtcpEscape(const char * msg_ptr,KviStr &buffer)
 {
+	//
+	// This one decodes an octal sequence
+	// and returns the pointer "just after it".
+	// It should be called when *msg_ptr points
+	// just after a backslash.
+	// The decoded escape is appended to the buffer
+	//
+	// We're also assuming that *msg_ptr is not null here
+	//
+	char c;
 	if((*msg_ptr >= '0') && (*msg_ptr < '8'))
 	{
-		char c = *msg_ptr - '0';
+		// a digit follows the backslash */
+		c = *msg_ptr - '0';
 		msg_ptr++;
 		if((*msg_ptr >= '0') && (*msg_ptr < '8'))
 		{
@@ -392,56 +551,259 @@ const char * KviServerParser::decodeCtcpEscape(const char * msg_ptr,KviStr &buff
 			{
 				c = ((c << 3) + (*msg_ptr - '0'));
 				msg_ptr++;
-			}
-		}
-		// FIXME: #warning "bAllowNull, bAllowCrLf"
+			} // else broken message, but let's be flexible
+		} // else it is broken, but let's be flexible
 		buffer.append(c);
 		return msg_ptr;
-	} else {
-		buffer.append(*msg_ptr);
+	}      
+
+	if(*msg_ptr == 'r')
+	{
+		buffer.append('\r');
 		return ++msg_ptr;
 	}
+        
+	if(*msg_ptr == 'n')
+	{
+		buffer.append('\n');
+		return ++msg_ptr;
+	}
+        
+	//
+	// null escape: just append the following
+	// character (thus discarding its semantics)
+	//
+	
+	buffer.append(msg_ptr);
+	return ++msg_ptr;
 }
 
+
+const char * KviServerParser::decodeCtcpEscape(const char * msg_ptr,QCString &buffer)
+{
+	//
+	// This one decodes an octal sequence
+	// and returns the pointer "just after it".
+	// It should be called when *msg_ptr points
+	// just after a backslash.
+	// The decoded escape is appended to the buffer
+	//
+	// We're also assuming that *msg_ptr is not null here
+	//
+	char c;
+	if((*msg_ptr >= '0') && (*msg_ptr < '8'))
+	{
+		// a digit follows the backslash */
+		c = *msg_ptr - '0';
+		msg_ptr++;
+		if((*msg_ptr >= '0') && (*msg_ptr < '8'))
+		{
+			c = ((c << 3) + (*msg_ptr - '0'));
+			msg_ptr++;
+			if((*msg_ptr >= '0') && (*msg_ptr < '8'))
+			{
+				c = ((c << 3) + (*msg_ptr - '0'));
+				msg_ptr++;
+			} // else broken message, but let's be flexible
+		} // else it is broken, but let's be flexible
+		buffer+= (c ? c : ' ') ;
+		return msg_ptr;
+	}      
+
+	if(*msg_ptr == 'r')
+	{
+		buffer+='\r';
+		return ++msg_ptr;
+	}
+	if(*msg_ptr == 'n')
+	{
+		buffer+='\n';
+		return ++msg_ptr;
+	}
+	//
+	// null escape: just append the following
+	// character (thus discarding its semantics)
+	//
+	
+	buffer+=*msg_ptr;
+	return ++msg_ptr;
+}
+
+
 const char * KviServerParser::extractCtcpParameter(const char * msg_ptr,KviStr &buffer,bool bSpaceBreaks)
 {
-	while(*msg_ptr == ' ')msg_ptr++;
-	bool bInString = false;
-	const char * aux = msg_ptr;
+	//
+	// This one extracts the "next" ctcp parameter in msg_ptr
+	// and puts it in the supplied buffer.
+	// It is assumed that the leading and trailing CTCP 
+	// tags have been already removed.
+	// Skips the leading and trailing spaces.
+	// bSpaceBreaks should be set to false if (and only if) the
+	// extracted parameter is the last in a positional parameter
+	// based CTCP message.
+	//
+        
+	int bInString = 0;
+	if(!msg_ptr) return 0;
+	while(*msg_ptr == ' ')msg_ptr++; // skip leading spaces
+	
+	if(*msg_ptr == '"')
+	{
+		// a quoted parameter
+		bInString = 1;
+		msg_ptr++;
+	}
+
+	const char * begin = msg_ptr;
+	
 	while(*msg_ptr)
 	{
 		switch(*msg_ptr)
 		{
 			case '\\':
-				buffer.append(aux,msg_ptr - aux);
+				// backslash : escape sequence
+				if(msg_ptr != begin)buffer.append(begin,msg_ptr - begin);
 				msg_ptr++;
-				if(*msg_ptr)msg_ptr = decodeCtcpEscape(msg_ptr,buffer);
-				else return msg_ptr;
-				
-				aux = msg_ptr;
+				if(*msg_ptr)
+				{
+					// decode the escape
+					msg_ptr = decodeCtcpEscape(msg_ptr,buffer);
+					begin = msg_ptr;
+				}
+				// else it is a senseless trailing backslash.
+				// Just ignore and let the function
+				// return spontaneously.
 			break;
 			case ' ':
+				// space : separate tokens if not in string
 				if(bInString || (!bSpaceBreaks))msg_ptr++;
-				else
-				{
-					buffer.append(aux,msg_ptr - aux);
+				else {
+					// Not in string and space breaks: end of token
+					// skip trailing white space (this could be avoided)
+					// and return
+					if(msg_ptr != begin)buffer.append(begin,msg_ptr - begin);
 					while(*msg_ptr == ' ')msg_ptr++;
 					return msg_ptr;
 				}
 			break;
 			case '"':
-				bInString = !bInString;
-				msg_ptr++;
+				if(bInString)
+				{
+					// A string terminator. We don't return
+					// immediately since if !bSpaceBreaks
+					// we must handle tokens until the end
+					// and otherwise we just run up to the
+					// next breaking space (but that's a bug anyway, heh).
+					if(msg_ptr != begin)buffer.append(begin,msg_ptr - begin);
+					bInString = 0;
+					msg_ptr++;
+					begin = msg_ptr;
+				} else {
+					// we don't begin a string here
+					// since we're in the middle of the token
+					// it is assumed to be simply a non encoded "
+					msg_ptr++;
+				}
 			break;
 			default:
+				// any other char
 				msg_ptr++;
 			break;
 		}
 	}
-	buffer.append(aux,msg_ptr - aux);
+	if(msg_ptr != begin)buffer.append(begin,msg_ptr - begin);
 	return msg_ptr;
 }
 
+const char * KviServerParser::extractCtcpParameter(const char * p_msg_ptr,QString &resultBuffer,bool bSpaceBreaks)
+{
+	//
+	// This one extracts the "next" ctcp parameter in p_msg_ptr
+	// and puts it in the supplied buffer.
+	// It is assumed that the leading and trailing CTCP 
+	// tags have been already removed.
+	// Skips the leading and trailing spaces.
+	// bSpaceBreaks should be set to false if (and only if) the
+	// extracted parameter is the last in a positional parameter
+	// based CTCP message.
+	//
+    
+	QCString buffer;
+	register const char* msg_ptr=p_msg_ptr;
+	int bInString = 0;
+	if(!msg_ptr) return 0;
+	while(*msg_ptr == ' ')msg_ptr++; // skip leading spaces
+	
+	if(*msg_ptr == '"')
+	{
+		// a quoted parameter
+		bInString = 1;
+		msg_ptr++;
+	}
+	
+	while(*msg_ptr)
+	{
+		switch(*msg_ptr)
+		{
+			case '\\':
+				// backslash : escape sequence
+				msg_ptr++;
+				if(*msg_ptr)
+				{
+					// decode the escape
+					msg_ptr = decodeCtcpEscape(msg_ptr,buffer);
+				}
+				// else it is a senseless trailing backslash.
+				// Just ignore and let the function
+				// return spontaneously.
+			break;
+			case ' ':
+				// space : separate tokens if not in string
+				if(bInString || (!bSpaceBreaks)) {
+					buffer+=*msg_ptr;
+					msg_ptr++;
+				}
+				else {
+					// Not in string and space breaks: end of token
+					// skip trailing white space (this could be avoided)
+					// and return
+					while(*msg_ptr == ' ')msg_ptr++;
+					resultBuffer=buffer;
+					return msg_ptr;
+				}
+			break;
+			case '"':
+				if(bInString)
+				{
+					// A string terminator. We don't return
+					// immediately since if !bSpaceBreaks
+					// we must handle tokens until the end
+					// and otherwise we just run up to the
+					// next breaking space (but that's a bug anyway, heh).
+					buffer+=*msg_ptr;
+					bInString = 0;
+					msg_ptr++;
+				} else {
+					// we don't begin a string here
+					// since we're in the middle of the token
+					// it is assumed to be simply a non encoded "
+					buffer+=*msg_ptr;
+					msg_ptr++;
+				}
+			break;
+			default:
+				// any other char
+				buffer+=*msg_ptr;
+				msg_ptr++;
+			break;
+		}
+	}
+	resultBuffer=buffer;
+	return msg_ptr;
+}
+
+                
+
 
 
 void KviServerParser::parseCtcpRequest(KviCtcpMessage *msg)
@@ -450,23 +812,36 @@ void KviServerParser::parseCtcpRequest(KviCtcpMessage *msg)
 
 	for(int i=0;m_ctcpRequestParseProcTable[i].msgName;i++)
 	{
-		if(kvi_strEqualCI(msg->szTag.ptr(),m_ctcpRequestParseProcTable[i].msgName))
+		if(KviQString::equalCI(msg->szTag,m_ctcpRequestParseProcTable[i].msgName))
 		{
 			if(!(m_ctcpReplyParseProcTable[i].iFlags & KVI_CTCP_MESSAGE_PARSE_TRIGGERNOEVENT))
 			{
-				if(TRIGGER_EVENT_6PARAM_RETVALUE(KviEvent_OnCtcpRequest, \
-					msg->msg->console(),msg->pSource->nick(),msg->pSource->user(),msg->pSource->host(), \
-					msg->szTarget,msg->szTag.ptr(),msg->pData))return;
+				QString szData = msg->msg->connection()->decodeText(msg->pData);
+				if(KVS_TRIGGER_EVENT_6_HALTED(KviEvent_OnCtcpRequest, \
+					msg->msg->console(), \
+					msg->pSource->nick(), \
+					msg->pSource->user(), \
+					msg->pSource->host(), \
+					msg->szTarget, \
+					msg->szTag, \
+					szData))return;
 			}
 			(this->*(m_ctcpRequestParseProcTable[i].proc))(msg);
 			return;
 		}
 	}
 
+	QString szData = msg->msg->connection()->decodeText(msg->pData);
 	// trigger the event on unrecognized requests too
-	if(TRIGGER_EVENT_6PARAM_RETVALUE(KviEvent_OnCtcpRequest, \
-		msg->msg->console(),msg->pSource->nick(),msg->pSource->user(),msg->pSource->host(), \
-		msg->szTarget,msg->szTag.ptr(),msg->pData))return;
+	if(KVS_TRIGGER_EVENT_6_HALTED(KviEvent_OnCtcpRequest, \
+		msg->msg->console(), \
+		msg->pSource->nick(), \
+		msg->pSource->user(), \
+		msg->pSource->host(), \
+		msg->szTarget, \
+		msg->szTag,
+		szData))
+			return;
 
 	// unknown
 	msg->bUnknown = true;
@@ -476,26 +851,28 @@ void KviServerParser::parseCtcpRequest(KviCtcpMessage *msg)
 void KviServerParser::parseCtcpReply(KviCtcpMessage *msg)
 {
 	msg->pData = extractCtcpParameter(msg->pData,msg->szTag);
-
+	
 	for(int i=0;m_ctcpReplyParseProcTable[i].msgName;i++)
 	{
-		if(kvi_strEqualCI(msg->szTag.ptr(),m_ctcpReplyParseProcTable[i].msgName))
+		if(KviQString::equalCI(msg->szTag,m_ctcpReplyParseProcTable[i].msgName))
 		{
 			if(!(m_ctcpReplyParseProcTable[i].iFlags & KVI_CTCP_MESSAGE_PARSE_TRIGGERNOEVENT))
 			{
-				if(TRIGGER_EVENT_6PARAM_RETVALUE(KviEvent_OnCtcpReply, \
+				QString szData = msg->msg->connection()->decodeText(msg->pData);
+				if(KVS_TRIGGER_EVENT_6_HALTED(KviEvent_OnCtcpReply, \
 					msg->msg->console(),msg->pSource->nick(),msg->pSource->user(), \
-					msg->pSource->host(),msg->szTarget,msg->szTag.ptr(),msg->pData))return;
+					msg->pSource->host(),msg->szTarget,msg->szTag,szData))return;
 			}
 			(this->*(m_ctcpReplyParseProcTable[i].proc))(msg);
 			return;
 		}
 	}
 
+	QString szData = msg->msg->connection()->decodeText(msg->pData);
 	// trigger the event on unrecognized replies too
-	if(TRIGGER_EVENT_6PARAM_RETVALUE(KviEvent_OnCtcpReply, \
+	if(KVS_TRIGGER_EVENT_6_HALTED(KviEvent_OnCtcpReply, \
 		msg->msg->console(),msg->pSource->nick(),msg->pSource->user(), \
-		msg->pSource->host(),msg->szTarget,msg->szTag.ptr(),msg->pData))return;
+		msg->pSource->host(),msg->szTarget,msg->szTag,szData))return;
 
 	// unknown
 	msg->bUnknown = true;
@@ -534,8 +911,9 @@ bool KviServerParser::checkCtcpFlood(KviCtcpMessage *msg)
 
 void KviServerParser::replyCtcp(KviCtcpMessage *msg,const char * data)
 {
-	msg->msg->connection()->sendFmtData("NOTICE %Q :%c%s %s%c",
-		&(msg->pSource->nick()),0x01,msg->szTag.ptr(),data,0x01);
+	QCString szNick = msg->msg->connection()->encodeText(msg->pSource->nick());
+	msg->msg->connection()->sendFmtData("NOTICE %s :%c%s %s%c",
+		szNick.data(),0x01,msg->msg->connection()->encodeText(msg->szTag).data(),msg->msg->connection()->encodeText(data).data(),0x01);
 }
 
 void KviServerParser::echoCtcpReply(KviCtcpMessage * msg)
@@ -564,8 +942,8 @@ void KviServerParser::echoCtcpReply(KviCtcpMessage * msg)
 
 		pOut->output(
 			msg->bUnknown ? KVI_OUT_CTCPREPLYUNKNOWN : KVI_OUT_CTCPREPLY,
-			__tr2qs("%Q %S reply from \r!n\r%Q\r [%Q@\r!h\r%Q\r]: %Q"),
-			&szWhat,&msg->szTag,&(msg->pSource->nick()),&(msg->pSource->user()),
+			__tr2qs("%Q %Q reply from \r!n\r%Q\r [%Q@\r!h\r%Q\r]: %Q"),
+			&szWhat,&(msg->szTag),&(msg->pSource->nick()),&(msg->pSource->user()),
 			&(msg->pSource->host()),&szData);
 	}
 }
@@ -576,8 +954,8 @@ void KviServerParser::echoCtcpRequest(KviCtcpMessage *msg)
 	// FIXME: #warning "DEDICATED CTCP WINDOW...MINIMIZED ?"
 	if(!msg->msg->haltOutput())
 	{
-		KviStr req = msg->szTag.ptr();
-		if(*(msg->pData))req.append(KviStr::Format," %s",msg->pData);
+		QString req = msg->szTag;
+		if(*(msg->pData))KviQString::appendFormatted(req," %s",msg->pData);
 
 		KviWindow * pOut = KVI_OPTION_BOOL(KviOption_boolCtcpRequestsToActiveWindow) ? msg->msg->console()->activeWindow() : msg->msg->console();
 
@@ -597,15 +975,17 @@ void KviServerParser::echoCtcpRequest(KviCtcpMessage *msg)
 		}
 
 
-		QString szRequest = pOut->decodeText(req.ptr());
+		QString szRequest = pOut->decodeText(req);
+		QString szTag = pOut->decodeText(msg->szTag);
 		QString szWhat = bIsChannel ? __tr2qs("Channel CTCP") : QString("CTCP");
 
 		if(msg->bIsFlood)
 		{
-			if(!TRIGGER_EVENT_6PARAM_RETVALUE(KviEvent_OnCtcpFlood,pOut,msg->pSource->nick(),msg->pSource->username(),msg->pSource->host(),msg->szTarget,msg->szTag.ptr(),msg->pData))
+			QString szData = msg->msg->connection()->decodeText(msg->pData);
+			if(!KVS_TRIGGER_EVENT_6_HALTED(KviEvent_OnCtcpFlood,pOut,msg->pSource->nick(),msg->pSource->username(),msg->pSource->host(),msg->szTarget,msg->szTag,szData))
 				pOut->output(KVI_OUT_CTCPREQUESTFLOOD,
-					__tr2qs("%Q %S request from \r!n\r%Q\r [%Q@\r!h\r%Q\r] (%Q), ignored (flood limit exceeded)"),
-					&szWhat,&msg->szTag,&(msg->pSource->nick()),
+					__tr2qs("%Q %Q%c request from \r!n\r%Q\r [%Q@\r!h\r%Q\r] (%Q), ignored (flood limit exceeded)"),
+					&szWhat,&szTag,KVI_TEXT_RESET,&(msg->pSource->nick()),
 					&(msg->pSource->user()),&(msg->pSource->host()),&szRequest);
 		} else {
 			QString szAction = msg->bUnknown ? __tr2qs("ignored (unrecognized)") :
@@ -613,8 +993,8 @@ void KviServerParser::echoCtcpRequest(KviCtcpMessage *msg)
 			pOut->output(
 				msg->bUnknown ? KVI_OUT_CTCPREQUESTUNKNOWN : 
 					(msg->bIgnored ? KVI_OUT_CTCPREQUESTIGNORED : KVI_OUT_CTCPREQUESTREPLIED),
-				__tr2qs("%Q %S request from \r!n\r%Q\r [%Q@\r!h\r%Q\r] (%Q), %Q"),
-				&szWhat,&msg->szTag,&(msg->pSource->nick()),
+				__tr2qs("%Q %Q%c request from \r!n\r%Q\r [%Q@\r!h\r%Q\r] (%Q), %Q"),
+				&szWhat,&szTag,KVI_TEXT_RESET,&(msg->pSource->nick()),
 				&(msg->pSource->user()),&(msg->pSource->host()),&szRequest,&szAction);
 		}
 	}
@@ -717,14 +1097,25 @@ void KviServerParser::parseCtcpRequestVersion(KviCtcpMessage *msg)
 	{
 		if(!KVI_OPTION_BOOL(KviOption_boolIgnoreCtcpVersion))
 		{
-			KviStr version = KVI_CTCP_VERSION_REPLY;
+			QString szVersion;
+#ifdef COMPILE_ON_WINDOWS
+			szVersion = "KVIrc " KVI_VERSION " '" KVI_RELEASE_NAME "' " KVI_SOURCES_DATE " - build " KVI_BUILD_DATE;
+			szVersion.append(QString(" - %1").arg(KviOsInfo::version()));
+#else
+			szVersion = "KVIrc " KVI_VERSION " '" KVI_RELEASE_NAME "' " KVI_SOURCES_DATE " - build " KVI_BUILD_DATE " - " KVI_BUILD_FLAGS;
+			szVersion.append(QString(" - %1 (%2)").arg(KviOsInfo::name()).arg(KviOsInfo::release()));
+#endif
+			szVersion.append(QString(" - QT Version: %1 - %2").arg(qVersion()).arg(__tr2qs("http://www.kvirc.net/")));
 			if(!KVI_OPTION_STRING(KviOption_stringCtcpVersionPostfix).isEmpty())
 			{
-				QCString sz = msg->msg->connection()->encodeText(KVI_OPTION_STRING(KviOption_stringCtcpVersionPostfix));
-				if(sz.data())
-				version.append(KviStr::Format," :%s",sz.data());
+				QString sz = KVI_OPTION_STRING(KviOption_stringCtcpVersionPostfix);
+				if(!sz.isEmpty())
+				{
+					szVersion.append(" :");
+					szVersion.append(sz);
+				}
 			}
-			replyCtcp(msg,version.ptr());
+			replyCtcp(msg,szVersion.utf8().data());
 		} else msg->bIgnored = true;
 	}
 
@@ -737,9 +1128,37 @@ void KviServerParser::parseCtcpRequestUserinfo(KviCtcpMessage *msg)
 	{
 		if(!KVI_OPTION_BOOL(KviOption_boolIgnoreCtcpUserinfo))
 		{
-			QCString sz = msg->msg->connection()->encodeText(KVI_OPTION_STRING(KviOption_stringCtcpUserinfoReply));
-			if(!sz.data())sz = "";
-			replyCtcp(msg,sz.data());
+			QString szReply;
+			if(!KVI_OPTION_STRING(KviOption_stringCtcpUserInfoAge).isEmpty())
+			{
+				szReply = "Age=";
+				szReply += KVI_OPTION_STRING(KviOption_stringCtcpUserInfoAge);
+			}
+			if(!KVI_OPTION_STRING(KviOption_stringCtcpUserInfoGender).isEmpty())
+			{
+				if(!szReply.isEmpty())szReply += "; ";
+				szReply += "Gender=";
+				szReply += KVI_OPTION_STRING(KviOption_stringCtcpUserInfoGender);
+			}
+			if(!KVI_OPTION_STRING(KviOption_stringCtcpUserInfoLocation).isEmpty())
+			{
+				if(!szReply.isEmpty())szReply += "; ";
+				szReply += "Location=";
+				szReply += KVI_OPTION_STRING(KviOption_stringCtcpUserInfoLocation);
+			}
+			if(!KVI_OPTION_STRING(KviOption_stringCtcpUserInfoLanguages).isEmpty())
+			{
+				if(!szReply.isEmpty())szReply += "; ";
+				szReply += "Languages=";
+				szReply += KVI_OPTION_STRING(KviOption_stringCtcpUserInfoLanguages);
+			}
+			if(!KVI_OPTION_STRING(KviOption_stringCtcpUserInfoOther).isEmpty())
+			{
+				if(!szReply.isEmpty())szReply += "; ";
+				szReply += KVI_OPTION_STRING(KviOption_stringCtcpUserInfoOther);
+			}
+			if(szReply.isEmpty())szReply = KVI_DEFAULT_CTCP_USERINFO_REPLY;
+			replyCtcp(msg,szReply.utf8().data());
 		} else msg->bIgnored = true;
 	}
 
@@ -754,14 +1173,14 @@ static const char * ctcpTagTable[][2]=
 	{ "PING"        , "Returns given parameters without parsing them"               },
 	{ "VERSION"     , "Returns the version of this client"                          },
 	{ "CLIENTINFO"  , "With no parameters, lists supported CTCP tags," \
-                          " 'CLIENTINFO <tag>' describes <tag>"                         },
+                          " 'CLIENTINFO <tag>' describes <tag>"                     },
 	{ "USERINFO"    , "Returns personal information about the current user"         },
 	{ "FINGER"      , "Returns information about the current user"                  },
 	{ "SOURCE"      , "Returns the client homepage URL"                             },
 	{ "TIME"        , "Returns the current local time"                              },
 	{ "ACTION"      , "Used to describe actions, generates no reply"                },
 	{ "AVATAR"      , "Returns the current avatar (may trigger a DCC GET) or" \
-                          " sets your own on this side if sent through a NOTICE"        },
+                          " sets your own on this side if sent through a NOTICE"    },
 	{ "DCC"         , "Initiates a DCC connection (XDCC,TDCC)"                      },
 	{ "PAGE"        , "Leaves a message for this user"                              },
 	{ 0             , 0                                                             }
@@ -780,7 +1199,7 @@ void KviServerParser::parseCtcpRequestClientinfo(KviCtcpMessage *msg)
 			szTag.toUpper();
 			if(szTag.isEmpty())
 			{
-				KviStr reply(KviStr::Format,"%s - Supported tags: ",KVI_CTCP_CLIENTINFO_VERSION);
+				KviStr reply("KVIrc " KVI_VERSION " '" KVI_RELEASE_NAME "' " KVI_SOURCES_DATE " - http://www.kvirc.net - Supported tags: ");
 				for(int i=0;ctcpTagTable[i][0];i++)
 				{
 					reply.append(ctcpTagTable[i][0]);
@@ -837,14 +1256,13 @@ void KviServerParser::parseCtcpRequestSource(KviCtcpMessage *msg)
 	{
 		if(!KVI_OPTION_BOOL(KviOption_boolIgnoreCtcpSource))
 		{
-			KviStr version = KVI_CTCP_SOURCE_REPLY;
+			QString version = "KVIrc " KVI_VERSION " '" KVI_RELEASE_NAME "' - http://www.kvirc.net/";
 			if(!KVI_OPTION_STRING(KviOption_stringCtcpSourcePostfix).isEmpty())
 			{
-				QCString sz = msg->msg->connection()->encodeText(KVI_OPTION_STRING(KviOption_stringCtcpSourcePostfix));
-				if(!sz.data())sz = "";
-				version.append(KviStr::Format," :%s",sz.data());
+				version+=" :";
+				version+= KVI_OPTION_STRING(KviOption_stringCtcpSourcePostfix);
 			}
-			replyCtcp(msg,version.ptr());
+			replyCtcp(msg,version.utf8().data());
 		} else msg->bIgnored = true;
 	}
 
@@ -857,9 +1275,7 @@ void KviServerParser::parseCtcpRequestTime(KviCtcpMessage *msg)
 	{
 		if(!KVI_OPTION_BOOL(KviOption_boolIgnoreCtcpTime))
 		{
-			QCString tmp = msg->msg->connection()->encodeText(QDateTime::currentDateTime().toString());
-			if(!tmp.data())tmp = "";
-			replyCtcp(msg,tmp.data());
+			replyCtcp(msg,QDateTime::currentDateTime().toString());
 		} else msg->bIgnored = true;
 	}
 
@@ -877,13 +1293,15 @@ void KviServerParser::parseCtcpRequestPage(KviCtcpMessage *msg)
 			{
 				KVI_OPTION_STRING(KviOption_stringCtcpPageReply) = KVI_DEFAULT_CTCP_PAGE_REPLY;
 			}
-			QCString sz = msg->msg->connection()->encodeText(KVI_OPTION_STRING(KviOption_stringCtcpPageReply));
-			if(!sz.data())sz = "";
-			replyCtcp(msg,sz.data());
+			replyCtcp(msg,KVI_OPTION_STRING(KviOption_stringCtcpPageReply));
 			if(KVI_OPTION_BOOL(KviOption_boolShowDialogOnCtcpPage))
 			{
 				if(!g_pCtcpPageDialog)g_pCtcpPageDialog = new KviCtcpPageDialog();
-				g_pCtcpPageDialog->addPage(msg->pSource->nick(),msg->pSource->user(),msg->pSource->host(),msg->pData);
+				KviStr szData8;
+				szData8 = msg->pData;
+				QString szData;
+				szData = msg->msg->connection()->decodeText(szData8.ptr());
+				g_pCtcpPageDialog->addPage(msg->pSource->nick(),msg->pSource->user(),msg->pSource->host(),szData);
 				g_pCtcpPageDialog->popup();
 			}
 		} else msg->bIgnored = true;
@@ -895,7 +1313,9 @@ void KviServerParser::parseCtcpRequestPage(KviCtcpMessage *msg)
 void KviServerParser::parseCtcpRequestAction(KviCtcpMessage *msg)
 {
 	KviStr szData8;
-	msg->pData = extractCtcpParameter(msg->pData,szData8,false);
+	// CTCP ACTION is a special exception... most clients do not encode/decode it.
+	//msg->pData = extractCtcpParameter(msg->pData,szData8,false);
+	szData8 = msg->pData;
 
 	KviWindow * pOut = 0;
 	bool bIsChannel = !IS_ME(msg->msg,msg->szTarget);
@@ -918,7 +1338,7 @@ void KviServerParser::parseCtcpRequestAction(KviCtcpMessage *msg)
 			{
 				// We still want to create it
 				// Give the scripter a chance to filter it out again
-				if(TRIGGER_EVENT_4PARAM_RETVALUE(KviEvent_OnQueryWindowRequest,msg->msg->console(),
+				if(KVS_TRIGGER_EVENT_4_HALTED(KviEvent_OnQueryWindowRequest,msg->msg->console(),
 							msg->pSource->nick(),
 							msg->pSource->username(),
 							msg->pSource->host(),
@@ -945,7 +1365,7 @@ void KviServerParser::parseCtcpRequestAction(KviCtcpMessage *msg)
 		pOut = KVI_OPTION_BOOL(KviOption_boolExternalMessagesToActiveWindow) ? msg->msg->console()->activeWindow() : msg->msg->console();
 	}
 
-	if(TRIGGER_EVENT_5PARAM_RETVALUE(KviEvent_OnAction,pOut,
+	if(KVS_TRIGGER_EVENT_5_HALTED(KviEvent_OnAction,pOut,
 				msg->pSource->nick(),
 				msg->pSource->username(),
 				msg->pSource->host(),
@@ -968,10 +1388,9 @@ void KviServerParser::parseCtcpRequestAction(KviCtcpMessage *msg)
 				pOut->demandAttention();
 			if(KVI_OPTION_BOOL(KviOption_boolPopupNotifierOnHighlightedMessages))
 			{
-				QString szMsg = "";
-				//QString szMsg = "<b>";
-				//szMsg += msg->pSource->nick();
-				//szMsg += "</b> ";
+				QString szMsg = "<b>";
+				szMsg += msg->pSource->nick();
+				szMsg += "</b> ";
 				szMsg += szData;
 				//debug("kvi_sp_ctcp.cpp:975 debug: %s",szMsg.data());
 				g_pApp->notifierMessage(pOut,KVI_OPTION_MSGTYPE(KVI_OUT_ACTION).pixId(),szMsg,90);
@@ -981,9 +1400,7 @@ void KviServerParser::parseCtcpRequestAction(KviCtcpMessage *msg)
 
 	if(bTargetFound)
 	{
-		QString szMsg = "\r!n\r";
-		szMsg += msg->pSource->nick();
-		szMsg += "\r ";
+		QString szMsg = QString("\r!n\r%1\r ").arg(msg->pSource->nick());
 		szMsg += szData;
 		if(bIsChannel)
 		{
@@ -1022,14 +1439,19 @@ void KviServerParser::parseCtcpRequestAvatar(KviCtcpMessage *msg)
 				QString szUserMask;
 				msg->pSource->mask(szUserMask);
 
-				QString szReply = a->name();
-				// escape the spaces with the right octal code
-				szReply.replace(" ","\\040");
+				QString szReply,szFileName;
+				szFileName=a->name();
+				if(KVI_OPTION_BOOL(KviOption_boolDCCFileTransferReplaceOutgoingSpacesWithUnderscores))
+					szFileName.replace(" ","_");
 
+				// escape the spaces with the right octal code
+				encodeCtcpParameter(szFileName,szReply);
+				
+				
 				if(!a->isRemote())
 				{
 					KviSharedFile * o;
-					if(!(o = g_pSharedFilesManager->addSharedFile(a->name().utf8().data(),a->localPath().utf8().data(),szUserMask,KVI_OPTION_UINT(KviOption_uintAvatarOfferTimeoutInSecs))))
+					if(!(o = g_pSharedFilesManager->addSharedFile(szFileName,a->localPath(),szUserMask,KVI_OPTION_UINT(KviOption_uintAvatarOfferTimeoutInSecs))))
 					{
 						// Don't delete o...it has been already deleted by g_pSharedFilesManager
 						msg->msg->console()->output(KVI_OUT_SYSTEMWARNING,__tr2qs("Unable to add file offer for file %Q (File not readable?)"),&(a->localPath()));
@@ -1040,12 +1462,10 @@ void KviServerParser::parseCtcpRequestAvatar(KviCtcpMessage *msg)
 								KVI_OPTION_UINT(KviOption_uintAvatarOfferTimeoutInSecs),&(a->name()),&(a->localPath()),&szUserMask);
 						}
 					}
-					if(o)KviQString::appendFormatted(szReply," %u",o->fileSize());
+					if(o)szReply.append(QString(" %1").arg(o->fileSize()));
 				}
-
-				QString szR = msg->msg->connection()->encodeText(szReply);
-				if(!szR.data())szR = "";
-				replyCtcp(msg,szR.data());
+				
+				replyCtcp(msg,szReply.utf8().data());
 			}
 		} else {
 			// no avatar set.. ignore channel requests if the user wishes
@@ -1064,10 +1484,11 @@ void KviServerParser::parseCtcpRequestAvatar(KviCtcpMessage *msg)
 
 void KviServerParser::parseCtcpReplyAvatar(KviCtcpMessage *msg)
 {
-	KviStr szRemoteFile;
-	KviStr szSize;
-	msg->pData = extractCtcpParameter(msg->pData,szRemoteFile,true);
+	QString szRemoteFile;
+	QString szSize;
+	QString decoded=msg->msg->console()->decodeText(msg->pData);
 
+	decoded = extractCtcpParameter(decoded.utf8().data(),szRemoteFile,true);
 	szRemoteFile.stripWhiteSpace();
 
 	bool bPrivate = IS_ME(msg->msg,msg->szTarget);
@@ -1078,10 +1499,8 @@ void KviServerParser::parseCtcpReplyAvatar(KviCtcpMessage *msg)
 	bool bResetAvatar = true;
 
 	QString nickLink;
-	QString hostLink;
 
 	KviQString::sprintf(nickLink,"\r!n\r%Q\r",&(msg->pSource->nick()));
-	KviQString::sprintf(hostLink,"\r!h\r%Q\r",&(msg->pSource->host()));
 
 	KviIrcUserEntry * e = msg->msg->connection()->userDataBase()->find(msg->pSource->nick());
 
@@ -1091,22 +1510,27 @@ void KviServerParser::parseCtcpReplyAvatar(KviCtcpMessage *msg)
 	if(szRemoteFile.isEmpty())
 	{
 		// avatar unset
-		KviQString::sprintf(textLine,__tr2qs("%Q [%Q@%Q] unsets avatar (%Q %Q)"),
-			&nickLink,&(msg->pSource->user()),&hostLink,&szWhere,&szWhat);
+		KviQString::sprintf(textLine,__tr2qs("%Q unsets avatar"),
+			&nickLink);
+		if(_OUTPUT_VERBOSE)
+			KviQString::appendFormatted(textLine," (%Q %Q)",&szWhere,&szWhat);
 	} else {
 		
 		// FIXME: #warning "The avatar should be the one with the requested size!!"
-		KviQString::sprintf(textLine,__tr2qs("%Q [%Q@%Q] changes avatar to %s (%Q %Q)"),
-			&nickLink,&(msg->pSource->user()),&hostLink,szRemoteFile.ptr(),&szWhere,&szWhat);
-		bool bIsUrl = kvi_strEqualCIN("http://",szRemoteFile.ptr(),7) && (szRemoteFile.len() > 7);
+		KviQString::sprintf(textLine,__tr2qs("%Q changes avatar to %s"),
+			&nickLink,szRemoteFile.utf8().data(),&szWhere,&szWhat);
+		if(_OUTPUT_VERBOSE)
+			KviQString::appendFormatted(textLine," (%Q %Q)",&szWhere,&szWhat);
+
+		bool bIsUrl = KviQString::equalCIN("http://",szRemoteFile,7) && (szRemoteFile.length() > 7);
 		if(!bIsUrl)
 		{
 			// no hacks
-			szRemoteFile.cutToLast('/');
-			szRemoteFile.cutToLast('\\');
+			KviQString::cutToLast(szRemoteFile,'/');
+			KviQString::cutToLast(szRemoteFile,'\\');
 		}
 
-		avatar = g_pIconManager->getAvatar(QString::null,szRemoteFile.ptr());
+		avatar = g_pIconManager->getAvatar(QString::null,szRemoteFile);
 
 		if((avatar == 0) && e)
 		{
@@ -1124,51 +1548,60 @@ void KviServerParser::parseCtcpReplyAvatar(KviCtcpMessage *msg)
 					g_pIconManager->urlToCachedFileName(szLocalFile);
 					g_pApp->getLocalKvircDirectory(szLocalFilePath,KviApp::Avatars,szLocalFile.ptr());
 					szLocalFilePath.replaceAll('\\',"\\\\"); // <-- this is especially for windows
-					KviStr szCommand = "http.get -w=nm ";
+					QString szCommand = "http.get -w=nm ";
 					unsigned int uMaxSize = KVI_OPTION_UINT(KviOption_uintMaximumRequestedAvatarSize);
-					if(uMaxSize > 0)szCommand.append(KviStr::Format,"-m=%u ",uMaxSize);
+					if(uMaxSize > 0)KviQString::appendFormatted(szCommand,"-m=%u ",uMaxSize);
 					szCommand += szRemoteFile;
 					szCommand += " \"";
-					szCommand += szLocalFilePath;
+					szCommand += szLocalFilePath.ptr();
 					szCommand += "\"";
 
-					if(g_pUserParser->parseCommandBuffer(szCommand.ptr(),msg->msg->console()))
+					if(KviKvsScript::run(szCommand,msg->msg->console()))
 					{
-						KviQString::appendFormatted(textLine,
-							__tr2qs(": No valid local copy of avatar available, requesting one (HTTP GET %s)"),
-							szRemoteFile.ptr());
+						if(_OUTPUT_VERBOSE)
+						{
+							KviQString::appendFormatted(textLine,
+								__tr2qs(": No valid local copy of avatar available, requesting one (HTTP GET %s)"),
+								szRemoteFile.utf8().data());
+						}
 						g_pApp->setAvatarOnFileReceived(msg->msg->console(),
-							szRemoteFile.ptr(),msg->pSource->nick(),msg->pSource->username(),msg->pSource->host());
+							szRemoteFile,msg->pSource->nick(),msg->pSource->username(),msg->pSource->host());
 					} else {
-						KviQString::appendFormatted(textLine,__tr2qs(": No valid local copy of avatar available; failed to start an HTTP transfer, ignoring"));
+						if(_OUTPUT_VERBOSE)
+							KviQString::appendFormatted(textLine,__tr2qs(": No valid local copy of avatar available; failed to start an HTTP transfer, ignoring"));
 					}
 				} else {
 					if(!checkCtcpFlood(msg))
 					{
-						KviQString::appendFormatted(textLine,
-							__tr2qs(": No valid local copy of avatar available, requesting one (DCC GET %s)"),
-							szRemoteFile.ptr());
+						if(_OUTPUT_VERBOSE)
+						{
+							KviQString::appendFormatted(textLine,
+								__tr2qs(": No valid local copy of avatar available, requesting one (DCC GET %s)"),
+								szRemoteFile.utf8().data());
+						}
 
-						KviStr szFName = szRemoteFile;
-						szFName.replaceAll(' ',"\\040");
-
-						msg->msg->connection()->sendFmtData("PRIVMSG %Q :%cDCC GET %s%c",
-								&(msg->pSource->nick()),0x01,szFName.ptr(),0x01);
+						QString szFName;
+						encodeCtcpParameter(szRemoteFile.utf8().data(),szFName);
+						msg->msg->connection()->sendFmtData("PRIVMSG %s :%cDCC GET %s%c",
+								msg->msg->connection()->encodeText(msg->pSource->nick()).data(),0x01,msg->msg->connection()->encodeText(szFName.utf8().data()).data(),0x01);
 						g_pApp->setAvatarOnFileReceived(msg->msg->console(),
-							szRemoteFile.ptr(),msg->pSource->nick(),msg->pSource->username(),msg->pSource->host());
+							szRemoteFile,msg->pSource->nick(),msg->pSource->username(),msg->pSource->host());
 					} else {
-						KviQString::appendFormatted(textLine,__tr2qs(": No valid local copy of avatar available; flood limit exceeded, ignoring"));
+						if(_OUTPUT_VERBOSE)
+							KviQString::appendFormatted(textLine,__tr2qs(": No valid local copy of avatar available; flood limit exceeded, ignoring"));
 					}
 				}
 			} else {
-				KviQString::appendFormatted(textLine,__tr2qs(": No valid local copy of avatar available, ignoring"));
+				if(_OUTPUT_VERBOSE)
+					KviQString::appendFormatted(textLine,__tr2qs(": No valid local copy of avatar available, ignoring"));
 			}
 		}
 	}
 
 	if(!e)
 	{
-		KviQString::appendFormatted(textLine,__tr2qs(": No such nickname in the user database, ignoring the change"));
+		if(_OUTPUT_VERBOSE)
+			KviQString::appendFormatted(textLine,__tr2qs(": No such nickname in the user database, ignoring the change"));
 		msg->msg->console()->outputNoFmt(KVI_OUT_AVATAR,textLine);
 		return;
 	}
@@ -1206,7 +1639,7 @@ void KviServerParser::parseCtcpRequestDcc(KviCtcpMessage *msg)
 		if(!msg->msg->haltOutput())
 		{
 			msg->msg->console()->output(KVI_OUT_DCCREQUEST,
-				__tr2qs("Ignoring DCC %S request from \r!n\r%Q\r [%Q@\r!h\r%Q\r] (%S %S)"),
+				__tr2qs("Ignoring DCC %S request from \r!n\r%Q\r [%Q@\r!h\r%Q\r] (%Q %S)"),
 				&p.szType,&(msg->pSource->nick()),
 				&(msg->pSource->user()),&(msg->pSource->host()),
 				&msg->szTag,&aux);
@@ -1232,11 +1665,13 @@ void KviServerParser::parseCtcpRequestDcc(KviCtcpMessage *msg)
 	{
 		if(!msg->msg->haltOutput())
 		{
+			QString decoded = msg->msg->console()->decodeText(p.szType.ptr());
 			msg->msg->console()->output(KVI_OUT_DCCREQUEST,
-				__tr2qs("Processing DCC %S request from \r!n\r%Q\r [%Q@\r!h\r%Q\r] (%S %S)"),
-				&p.szType,&(msg->pSource->nick()),
+				__tr2qs("Processing DCC %Q request from \r!n\r%Q\r [%Q@\r!h\r%Q\r] (%s %s)"),
+				&decoded,&(msg->pSource->nick()),
 				&(msg->pSource->user()),&(msg->pSource->host()),
-				&msg->szTag,&aux);
+				msg->msg->console()->decodeText(msg->szTag).data(),
+				msg->msg->console()->decodeText(aux.ptr()).data());
 		}
 
 		KviModule * m = g_pModuleManager->getModule("dcc");
