@@ -40,8 +40,6 @@
 #include "kvi_userlistview.h"
 #include "kvi_antispam.h"
 #include "kvi_nickserv.h"
-#include "kvi_uparser.h"
-#include "kvi_event.h"
 #include "kvi_parameterlist.h"
 #include "kvi_ircuserdb.h"
 #include "kvi_app.h"
@@ -53,10 +51,12 @@
 #include "kvi_ircconnectionuserinfo.h"
 #include "kvi_ircconnectiontarget.h"
 #include "kvi_ircconnectionserverinfo.h"
+#include "kvi_ircconnectionstatedata.h"
 #include "kvi_ircconnectionnetsplitdetectordata.h"
 #include "kvi_iconmanager.h"
 #include "kvi_lagmeter.h"
 #include "kvi_ircserver.h"
+#include "kvi_kvs_eventtriggers.h"
 
 #include "kvi_settings.h"
 
@@ -65,6 +65,8 @@
 	#include "kvi_cryptcontroller.h"
 #endif
 
+#include "kvi_kvs_script.h"
+
 //#include "kvi_regusersdb.h"
 //#include "kvi_iconmanager.h"
 #include <qdatetime.h>
@@ -84,16 +86,19 @@ void KviServerParser::parseLiteralPing(KviIrcMessage *msg)
 {
 	// PING
 	// <optional_prefix> PING :<argument>
-	msg->connection()->sendFmtData("PONG %s",msg->allParams());
+	msg->connection()->sendFmtData("PONG %s",msg->console()->connection()->encodeText(msg->allParams()).data());
 
-	if(TRIGGER_EVENT_2PARAM_RETVALUE(KviEvent_OnPing,msg->console(),msg->safePrefix(),msg->allParams()))
+	QString szPrefix = msg->connection()->decodeText(msg->safePrefix());
+	QString szAllParams = msg->connection()->decodeText(msg->allParams());
+
+	if(KVS_TRIGGER_EVENT_2_HALTED(KviEvent_OnPing,msg->console(),szPrefix,szAllParams))
 		msg->setHaltOutput();
 
 	if((!msg->haltOutput()) && KVI_OPTION_BOOL(KviOption_boolShowPingPong))
 	{
 		msg->console()->output(KVI_OUT_SERVERPING,
-			__tr2qs("Received ping from \r!s\r%s\r (PING %s), replied pong"),
-			msg->safePrefix(),msg->allParams());
+			__tr2qs("Received ping from \r!s\r%Q\r (PING %Q), replied pong"),
+			&szPrefix,&szAllParams);
 	}
 }
 
@@ -104,7 +109,10 @@ void KviServerParser::parseLiteralPing(KviIrcMessage *msg)
 
 void KviServerParser::parseLiteralPong(KviIrcMessage *msg)
 {
-	if(TRIGGER_EVENT_2PARAM_RETVALUE(KviEvent_OnPong,msg->console(),msg->safePrefix(),msg->allParams()))
+	QString szPrefix = msg->connection()->decodeText(msg->safePrefix());
+	QString szAllParams = msg->connection()->decodeText(msg->allParams());
+
+	if(KVS_TRIGGER_EVENT_2_HALTED(KviEvent_OnPong,msg->console(),szPrefix,szAllParams))
 		msg->setHaltOutput();
 
 	if(msg->console()->connection()->lagMeter())
@@ -133,20 +141,16 @@ void KviServerParser::parseLiteralError(KviIrcMessage *msg)
 	// <optional_prefix> ERROR :<argument>
 	// ERROR :Closing Link: phoenix.pragmaware.net (Ping timeout)
 
-	
-	if(TRIGGER_EVENT_2PARAM_RETVALUE(KviEvent_OnError,msg->console(),msg->safePrefix(),msg->allParams()))
+	QString szPrefix = msg->connection()->decodeText(msg->safePrefix());
+	QString szParams = msg->connection()->decodeText(msg->allParams());
+
+	if(KVS_TRIGGER_EVENT_2_HALTED(KviEvent_OnError,msg->console(),szPrefix,szParams))
 		msg->setHaltOutput();
 
 	if(!msg->haltOutput())
 	{
-		QString szParms = msg->connection()->decodeText(msg->allParams());
 		msg->console()->output(KVI_OUT_SERVERERROR,
-			__tr2qs("Server ERROR: %Q"),&szParms);
-		/*
-		msg->console()->output(KVI_OUT_SERVERPING,
-			__tr2qs("Received ping from \r!s\r%s\r (PING %s), replied pong"),
-			msg->safePrefix(),msg->allParams());
-		*/
+			__tr2qs("Server ERROR: %Q"),&szParams);
 	}
 }
 
@@ -201,14 +205,13 @@ void KviServerParser::parseLiteralJoin(KviIrcMessage *msg)
 		if(bIsMe)
 		{
 			msg->connection()->userInfoReceived(szUser,szHost);
-			chan = msg->connection()->createChannel(channel); // New channel (will resurrect dead an eventual dead one too!)
+			chan = msg->connection()->createChannel(channel); // New channel (will resurrect an eventual dead one too!)
 		} else {
 			// Someone is joining an inexsisting channel!!!
 			UNRECOGNIZED_MESSAGE(msg,__tr("Received a join message for an unknown channel, possible desync"));
 			return;
 		}
 
-
 		int iFlags = 0;
 		switch(chExtMode)
 		{
@@ -234,7 +237,8 @@ void KviServerParser::parseLiteralJoin(KviIrcMessage *msg)
 			}
 		}
 
-		if(TRIGGER_EVENT_RETVALUE(KviEvent_OnMeJoin,chan))msg->setHaltOutput();
+		if(KVS_TRIGGER_EVENT_0_HALTED(KviEvent_OnMeJoin,chan))
+			msg->setHaltOutput();
 
 		// the channel requests must be sent AFTER we have created and accessed the chan
 		// since it MAY happen that a sendFmtData() call fails by detecting a disconnect
@@ -261,12 +265,19 @@ void KviServerParser::parseLiteralJoin(KviIrcMessage *msg)
 				chan->setSentInviteListRequest();
 			}
 		}
+
 		// MODE %s b MUST BE THE LAST AUTOMATIC CHANNEL QUERY
 		// so we get RPL_ENDOFBANLIST as the last reply
 		// and we know that the channel is in sync
 
 		if(!KVI_OPTION_BOOL(KviOption_boolDisableWhoRequestOnJoin))
 		{
+			msg->connection()->stateData()->setLastSentChannelWhoRequest(kvi_unixTime());
+			if(msg->connection()->lagMeter())
+			{
+				KviStr tmp(KviStr::Format,"WHO %s",encodedChan);
+				msg->connection()->lagMeter()->lagCheckRegister(tmp.ptr(),60);
+			}
 			if(!msg->connection()->sendFmtData("WHO %s",encodedChan))return; // disconnected
 			chan->setSentWhoRequest();
 		}
@@ -296,9 +307,8 @@ void KviServerParser::parseLiteralJoin(KviIrcMessage *msg)
 		//       if later it is needed, make it return a pointer
 		if(!(it->globalData()->avatar()))console->checkDefaultAvatar(it->globalData(),szNick,szUser,szHost);
 
-		if(TRIGGER_EVENT_3PARAM_RETVALUE(KviEvent_OnJoin,chan,szNick,szUser,szHost))
+		if(KVS_TRIGGER_EVENT_3_HALTED(KviEvent_OnJoin,chan,szNick,szUser,szHost))
 			msg->setHaltOutput();
-
 		// FIXME: #warning "WE COULD OPTIONALLY REQUEST A /WHO FOR THE USERS JOINING THAT WE DON'T KNOW THE HOST OF"
 	}
 
@@ -320,17 +330,20 @@ void KviServerParser::parseLiteralJoin(KviIrcMessage *msg)
 		}
 	}
 
-	if(!bIsMe && KVI_OPTION_BOOL(KviOption_boolEnableQueryTracing))
+	//if(!bisMe) deleted because we can open query with our nick
+	QString szChans;
+	int iChans = msg->connection()->getCommonChannels(szNick,szChans);
+	KviQuery * q = console->connection()->findQuery(szNick);
+	if(q)
 	{
-		QString szChans;
-		int iChans = msg->connection()->getCommonChannels(szNick,szChans);
-		KviQuery * q = console->connection()->findQuery(szNick);
-		if(q)
+		if(KVI_OPTION_BOOL(KviOption_boolEnableQueryTracing))
 		{
 			q->output(KVI_OUT_QUERYTRACE,
 				__tr2qs("\r!n\r%Q\r [%Q@\r!h\r%Q\r] has just joined \r!c\r%Q\r"),&szNick,&szUser,
 				&szHost,&channel);
 			q->notifyCommonChannels(szNick,szUser,szHost,iChans,szChans);
+		} else {
+			q->updateLabelText();
 		}
 	}
 }
@@ -366,8 +379,7 @@ void KviServerParser::parseLiteralPart(KviIrcMessage *msg)
 
 	if(IS_ME(msg,szNick))
 	{
-		// FIXME: the events should be triggered as wide string text ?
-		if(TRIGGER_EVENT_1PARAM_RETVALUE(KviEvent_OnMePart,chan,partMsg))
+		if(KVS_TRIGGER_EVENT_1_HALTED(KviEvent_OnMePart,chan,partMsg))
 			msg->setHaltOutput();
 
 		KviWindow * pOut = console;
@@ -396,7 +408,7 @@ void KviServerParser::parseLiteralPart(KviIrcMessage *msg)
 	} else {
 		// Someone else
 
-		if(TRIGGER_EVENT_4PARAM_RETVALUE(KviEvent_OnPart,chan,szNick,szUser,szHost,partMsg))
+		if(KVS_TRIGGER_EVENT_4_HALTED(KviEvent_OnPart,chan,szNick,szUser,szHost,partMsg))
 			msg->setHaltOutput();
 
 		chan->part(szNick);
@@ -423,11 +435,11 @@ void KviServerParser::parseLiteralPart(KviIrcMessage *msg)
 			{
 				if(!partMsg.isEmpty())
 					q->output(KVI_OUT_QUERYTRACE,
-						__tr2qs("\r!n\r%Q\r [%Q@\r!h\r%Q\r] has just left \r!c\r%Q\r: %Q"),
+						__tr2qs("\r!nc\r%Q\r [%Q@\r!h\r%Q\r] has just left \r!c\r%Q\r: %Q"),
 						&szNick,&szUser,&szHost,&szChan,&partMsg);
 				else
 					q->output(KVI_OUT_QUERYTRACE,
-						__tr2qs("\r!n\r%Q\r [%Q@\r!h\r%Q\r] has just left \r!c\r%Q\r"),
+						__tr2qs("\r!nc\r%Q\r [%Q@\r!h\r%Q\r] has just left \r!c\r%Q\r"),
 						&szNick,&szUser,&szHost,&szChan);
 				q->notifyCommonChannels(szNick,szUser,szHost,iChans,szChans);
 			}
@@ -489,8 +501,12 @@ void KviServerParser::parseLiteralQuit(KviIrcMessage *msg)
 
 			if(!bDuplicate)
 			{
-				if(!TRIGGER_EVENT_2PARAM_RETVALUE(KviEvent_OnNetsplit,console,
-					KviStr(aux,daSpace - aux),KviStr(daSpace + 1)))
+				KviStr sz1(aux,daSpace - aux);
+				KviStr sz2(daSpace + 1);
+				
+				QString szD1 = msg->connection()->decodeText(sz1.ptr());
+				QString szD2 = msg->connection()->decodeText(sz2.ptr());
+				if(!KVS_TRIGGER_EVENT_2_HALTED(KviEvent_OnNetsplit,console,szD1,szD2))
 				{
 					if(!msg->haltOutput())
 						console->output(KVI_OUT_SPLIT,__tr2qs("Netsplit detected: %s"),aux);
@@ -518,11 +534,16 @@ void KviServerParser::parseLiteralQuit(KviIrcMessage *msg)
 				}
 			}
 		}
+		
+		KviKvsVariantList vList;
+		vList.append(szNick);
+		vList.append(szUser);
+		vList.append(szHost);
+		vList.append(szReason);
+		vList.append(chanlist);
 
-		if(g_pUserParser->triggerEvent(KviEvent_OnQuit,console,
-					new KviParameterList(new KviStr(szNick),new KviStr(szNick),
-					new KviStr(szNick),new KviStr(szReason),new KviStr(chanlist))))
-				msg->setHaltOutput();
+		if(KviKvsEventManager::instance()->trigger(KviEvent_OnQuit,console,&vList))
+			msg->setHaltOutput();
 	}
 
 
@@ -593,9 +614,10 @@ void KviServerParser::parseLiteralKick(KviIrcMessage *msg)
 	{
 		// ops...I have been kicked
 
-		if(TRIGGER_EVENT_4PARAM_RETVALUE(KviEvent_OnMeKick,chan,
+		if(KVS_TRIGGER_EVENT_4_HALTED(KviEvent_OnMeKick,chan,
 				szNick,szUser,szHost,szKickMsg))
 			msg->setHaltOutput();
+		if(!KVI_OPTION_STRING(KviOption_stringOnMeKickedSound).isEmpty()) KviKvsScript::run("snd.play $0",0,new KviKvsVariantList(new KviKvsVariant(KVI_OPTION_STRING(KviOption_stringOnMeKickedSound))));
 
 		QString szPass = chan->channelKey().ptr();
 
@@ -637,7 +659,7 @@ void KviServerParser::parseLiteralKick(KviIrcMessage *msg)
 	} else {
 		// ok...someone else has been kicked
 
-		if(TRIGGER_EVENT_5PARAM_RETVALUE(KviEvent_OnKick,chan,
+		if(KVS_TRIGGER_EVENT_5_HALTED(KviEvent_OnKick,chan,
 				szNick,szUser,szHost,victim,szKickMsg))
 			msg->setHaltOutput();
 
@@ -736,7 +758,7 @@ void KviServerParser::parseLiteralPrivmsg(KviIrcMessage *msg)
 				if(KVI_OPTION_BOOL(KviOption_boolVerboseIgnore))
 				{
 					QString szMsg = msg->connection()->decodeText(msg->safeTrailing());
-					console->output(KVI_OUT_IGNORE,__tr2qs("Ignoring PRIVMSG from \r!n\r%Q\r [%Q@\r!h\r%Q\r]: %Q"),
+					console->output(KVI_OUT_IGNORE,__tr2qs("Ignoring PRIVMSG from \r!nc\r%Q\r [%Q@\r!h\r%Q\r]: %Q"),
 							&szNick,&szUser,&szHost,&szMsg);
 				}
 				return;
@@ -750,7 +772,8 @@ void KviServerParser::parseLiteralPrivmsg(KviIrcMessage *msg)
 	KviStr * pTrailing = msg->trailingString();
 	if(pTrailing)
 	{
-		if(*(pTrailing->ptr()) == 0x01){
+		if(*(pTrailing->ptr()) == 0x01)
+		{
 			if(pTrailing->len() > 1)
 			{
 				if(pTrailing->lastCharIs(0x01))pTrailing->cutRight(1);
@@ -853,7 +876,7 @@ void KviServerParser::parseLiteralPrivmsg(KviIrcMessage *msg)
 				QString szMsg = msg->connection()->decodeText(msg->safeTrailing());
 				// We still want to create it
 				// Give the scripter a chance to filter it out again
-				if(TRIGGER_EVENT_4PARAM_RETVALUE(KviEvent_OnQueryWindowRequest,
+				if(KVS_TRIGGER_EVENT_4_HALTED(KviEvent_OnQueryWindowRequest,
 						console,szNick,szUser,szHost,szMsg))
 				{
 					// check if the scripter hasn't created it
@@ -878,8 +901,12 @@ void KviServerParser::parseLiteralPrivmsg(KviIrcMessage *msg)
 			DECRYPT_IF_NEEDED(query,msg->safeTrailing(),KVI_OUT_QUERYPRIVMSG,KVI_OUT_QUERYPRIVMSGCRYPTED,szBuffer,txtptr,msgtype)
 			// trigger the script event and eventually kill the output
 			QString szMsgText = query->decodeText(txtptr);
-			if(TRIGGER_EVENT_4PARAM_RETVALUE(KviEvent_OnQueryMessage,query,szNick,szUser,szHost,szMsgText)) 
+			if(KVS_TRIGGER_EVENT_4_HALTED(KviEvent_OnQueryMessage,query,szNick,szUser,szHost,szMsgText)) 
 				msg->setHaltOutput();
+			
+			if(!KVI_OPTION_STRING(KviOption_stringOnQueryMessageSound).isEmpty() && query!=g_pActiveWindow)
+			 	KviKvsScript::run("snd.play $0",0,new KviKvsVariantList(new KviKvsVariant(KVI_OPTION_STRING(KviOption_stringOnQueryMessageSound))));
+			
 			// spit out the message text
 			if(!msg->haltOutput())
 			{
@@ -914,8 +941,11 @@ void KviServerParser::parseLiteralPrivmsg(KviIrcMessage *msg)
 			// no query creation: no decryption possible
 			// trigger the query message event in the console
 			QString szMsgText = msg->connection()->decodeText(msg->safeTrailing());
-			if(TRIGGER_EVENT_4PARAM_RETVALUE(KviEvent_OnQueryMessage,console,szNick,szUser,szHost,szMsgText))
+			if(KVS_TRIGGER_EVENT_4_HALTED(KviEvent_OnQueryMessage,console,szNick,szUser,szHost,szMsgText))
 					msg->setHaltOutput();
+			
+			if(!KVI_OPTION_STRING(KviOption_stringOnQueryMessageSound).isEmpty() && query!=g_pActiveWindow)
+			 	KviKvsScript::run("snd.play $0",0,new KviKvsVariantList(new KviKvsVariant(KVI_OPTION_STRING(KviOption_stringOnQueryMessageSound))));
 			// spit the message text out
 			if(!msg->haltOutput())
 			{
@@ -937,13 +967,27 @@ void KviServerParser::parseLiteralPrivmsg(KviIrcMessage *msg)
 					}
 				}
 
-				pOut->output(KVI_OUT_QUERYPRIVMSG,"[PRIVMSG \r!n\r%Q\r]: %Q",&szNick,&szMsgText);
+				pOut->output(KVI_OUT_QUERYPRIVMSG,"[PRIVMSG \r!nc\r%Q\r]: %Q",&szNick,&szMsgText);
 			}
 		}
 	} else {
 		// Channel PRIVMSG
 		KviChannel * chan = msg->connection()->findChannel(szTarget);
 
+		QString szOriginalTarget = szTarget;
+		QString szPrefixes;
+
+		if(!chan)
+		{
+			// check if the channel has some leading mode prefixes 
+			while((szTarget.length() > 0) && console->connection()->serverInfo()->isSupportedModePrefix(szTarget[0].unicode()))
+			{
+				szPrefixes += szTarget[0];
+				szTarget.remove(0,1);
+			}
+			chan = msg->connection()->findChannel(szTarget);
+		}
+
 		if(!chan)
 		{
 			if(!msg->haltOutput())
@@ -952,7 +996,7 @@ void KviServerParser::parseLiteralPrivmsg(KviIrcMessage *msg)
 				KviWindow * pOut = KVI_OPTION_BOOL(KviOption_boolOperatorMessagesToActiveWindow) ? 
 					console->activeWindow() : (KviWindow *)(console);
 				QString broad;
-				KviQString::sprintf(broad,"[>> %Q] %Q",&szTarget,&szMsgText);
+				KviQString::sprintf(broad,"[>> %Q] %Q",&szOriginalTarget,&szMsgText);
 				console->outputPrivmsg(pOut,KVI_OUT_BROADCASTPRIVMSG,szNick,szUser,szHost,broad,0);
 			}
 		} else {
@@ -972,12 +1016,23 @@ void KviServerParser::parseLiteralPrivmsg(KviIrcMessage *msg)
 			}
 #endif
 */
+
 			QString szMsgText = chan->decodeText(txtptr);
-			if(TRIGGER_EVENT_4PARAM_RETVALUE(KviEvent_OnChannelMessage,chan,szNick,szUser,szHost,szMsgText))
+
+			if(KVS_TRIGGER_EVENT_5_HALTED(KviEvent_OnChannelMessage,chan,szNick,szUser,szHost,szMsgText,szPrefixes))
 				msg->setHaltOutput();
 
 			if(!msg->haltOutput())
-				console->outputPrivmsg(chan,msgtype,szNick,szUser,szHost,szMsgText,0);
+			{
+				if(szPrefixes.length() > 0)
+				{
+					QString szBroad;
+					KviQString::sprintf(szBroad,"[>> %Q\r!c\r%Q\r] %Q",&szPrefixes,&szTarget,&szMsgText);
+					console->outputPrivmsg(chan,msgtype,szNick,szUser,szHost,szBroad,0);
+				} else {
+					console->outputPrivmsg(chan,msgtype,szNick,szUser,szHost,szMsgText,0);
+				}
+			}
 		}
 	}
 }
@@ -1001,7 +1056,7 @@ void KviServerParser::parseLiteralNotice(KviIrcMessage *msg)
 				// server notice
 				// FIXME: "Dedicated window for server notices ?"
 				QString szMsgText = msg->connection()->decodeText(msg->safeTrailing());
-				if(TRIGGER_EVENT_2PARAM_RETVALUE(KviEvent_OnServerNotice,console,szNick,szMsgText))
+				if(KVS_TRIGGER_EVENT_2_HALTED(KviEvent_OnServerNotice,console,szNick,szMsgText))
 					msg->setHaltOutput();
 				if(!msg->haltOutput())
 				{
@@ -1065,7 +1120,7 @@ void KviServerParser::parseLiteralNotice(KviIrcMessage *msg)
 		if(KviQString::equalCI(szNick,"NickServ"))
 		{
 			QString szMsgText = msg->connection()->decodeText(msg->safeTrailing());
-			if(TRIGGER_EVENT_4PARAM_RETVALUE(KviEvent_OnNickServNotice,console,szNick,szUser,szHost,szMsgText))
+			if(KVS_TRIGGER_EVENT_4_HALTED(KviEvent_OnNickServNotice,console,szNick,szUser,szHost,szMsgText))
 				msg->setHaltOutput();
 
 			// nickname service... does it ask for identification ?
@@ -1087,9 +1142,8 @@ void KviServerParser::parseLiteralNotice(KviIrcMessage *msg)
 					if(rule)
 					{
 						bAuthDone = true;
-						KviStr szIdentifyCommand = rule->identifyCommand();
 						console->outputNoFmt(KVI_OUT_SYSTEMMESSAGE,__tr2qs("NickServ requests authentication, executing scheduled command"));
-						if(!g_pUserParser->parseCommandBuffer(szIdentifyCommand.ptr(),console))
+						if(!KviKvsScript::run(rule->identifyCommand(),console))
 						{
 							console->outputNoFmt(KVI_OUT_SYSTEMERROR,__tr2qs("The scheduled NickServ identification command appears to be broken, please change the setting"));
 						}
@@ -1105,9 +1159,8 @@ void KviServerParser::parseLiteralNotice(KviIrcMessage *msg)
 					KviNickServRule * rule = g_pNickServRuleSet->matchRule(msg->connection()->currentNickName(),&talker,szMsgText,msg->connection()->currentServerName());
 					if(rule)
 					{
-						KviStr szIdentifyCommand = rule->identifyCommand();
 						console->outputNoFmt(KVI_OUT_SYSTEMMESSAGE,__tr2qs("NickServ requests authentication, executing scheduled command"));
-						if(!g_pUserParser->parseCommandBuffer(szIdentifyCommand.ptr(),console))
+						if(!KviKvsScript::run(rule->identifyCommand(),console))
 						{
 							console->outputNoFmt(KVI_OUT_SYSTEMERROR,__tr2qs("The scheduled NickServ identification command appears to be broken, please change the setting"));
 						}
@@ -1121,7 +1174,7 @@ void KviServerParser::parseLiteralNotice(KviIrcMessage *msg)
 		if(KviQString::equalCI(szNick,"ChanServ"))
 		{
 			QString szMsgText = msg->connection()->decodeText(msg->safeTrailing());
-			if(TRIGGER_EVENT_4PARAM_RETVALUE(KviEvent_OnChanServNotice,console,szNick,szUser,szHost,szMsgText))
+			if(KVS_TRIGGER_EVENT_4_HALTED(KviEvent_OnChanServNotice,console,szNick,szUser,szHost,szMsgText))
 				msg->setHaltOutput();
 			if(!msg->haltOutput())
 			{
@@ -1178,7 +1231,7 @@ void KviServerParser::parseLiteralNotice(KviIrcMessage *msg)
 				QString szMsgText = msg->connection()->decodeText(msg->safeTrailing());
 				// We still want to create it
 				// Give the scripter a chance to filter it out again
-				if(TRIGGER_EVENT_4PARAM_RETVALUE(KviEvent_OnQueryWindowRequest,console,szNick,szUser,szHost,szMsgText))
+				if(KVS_TRIGGER_EVENT_4_HALTED(KviEvent_OnQueryWindowRequest,console,szNick,szUser,szHost,szMsgText))
 				{
 					// check if the scripter hasn't created it
 					query = msg->connection()->findQuery(szNick);
@@ -1202,7 +1255,7 @@ void KviServerParser::parseLiteralNotice(KviIrcMessage *msg)
 			DECRYPT_IF_NEEDED(query,msg->safeTrailing(),KVI_OUT_QUERYNOTICE,KVI_OUT_QUERYNOTICECRYPTED,szBuffer,txtptr,msgtype)
 			QString szMsgText = query->decodeText(txtptr);
 			// trigger the script event and eventually kill the output
-			if(TRIGGER_EVENT_4PARAM_RETVALUE(KviEvent_OnQueryNotice,query,szNick,szUser,szHost,szMsgText)) 
+			if(KVS_TRIGGER_EVENT_4_HALTED(KviEvent_OnQueryNotice,query,szNick,szUser,szHost,szMsgText)) 
 				msg->setHaltOutput();
 			// spit out the message text
 			if(!msg->haltOutput())
@@ -1240,7 +1293,7 @@ void KviServerParser::parseLiteralNotice(KviIrcMessage *msg)
 			QString szMsgText = msg->connection()->decodeText(msg->safeTrailing());
 			// no query creation: no decryption possible
 			// trigger the query message event in the console
-			if(TRIGGER_EVENT_4PARAM_RETVALUE(KviEvent_OnQueryNotice,console,szNick,szUser,szHost,szMsgText))
+			if(KVS_TRIGGER_EVENT_4_HALTED(KviEvent_OnQueryNotice,console,szNick,szUser,szHost,szMsgText))
 				msg->setHaltOutput();
 			// spit the message text out
 			if(!msg->haltOutput())
@@ -1270,14 +1323,20 @@ void KviServerParser::parseLiteralNotice(KviIrcMessage *msg)
 	}
 
 	// Channel NOTICE
-	QChar flaggedNotice(0);
 	KviChannel * chan = msg->connection()->findChannel(szTarget);
 	
-	if(!chan && (szTarget.length() > 0))
+	QString szOriginalTarget = szTarget;
+	QString szPrefixes;
+
+	if(!chan)
 	{
-		flaggedNotice = szTarget[0];
-		if(msg->connection()->serverInfo()->supportedModePrefixes().find(flaggedNotice) != -1)
-			chan = msg->connection()->findChannel(szTarget.right(szTarget.length() - 1));
+		// check if the channel has some leading mode prefixes 
+		while((szTarget.length() > 0) && console->connection()->serverInfo()->isSupportedModePrefix(szTarget[0].unicode()))
+		{
+			szPrefixes += szTarget[0];
+			szTarget.remove(0,1);
+		}
+		chan = msg->connection()->findChannel(szTarget);
 	}
 
 	if(!chan)
@@ -1288,7 +1347,7 @@ void KviServerParser::parseLiteralNotice(KviIrcMessage *msg)
 				console->activeWindow() : (KviWindow *)(console);
 			QString szBroad;
 			QString szMsgText = msg->connection()->decodeText(msg->safeTrailing());
-			KviQString::sprintf(szBroad,"[>> %Q] %Q",&szTarget,&szMsgText);
+			KviQString::sprintf(szBroad,"[>> %Q] %Q",&szOriginalTarget,&szMsgText);
 			console->outputPrivmsg(pOut,KVI_OUT_BROADCASTNOTICE,szNick,szUser,szHost,szBroad,0);
 			return;
 		}
@@ -1299,16 +1358,14 @@ void KviServerParser::parseLiteralNotice(KviIrcMessage *msg)
 		DECRYPT_IF_NEEDED(chan,msg->safeTrailing(),KVI_OUT_CHANNELNOTICE,KVI_OUT_CHANNELNOTICECRYPTED,szBuffer,txtptr,msgtype)
 		QString szMsgText = chan->decodeText(txtptr);
 
-		if(TRIGGER_EVENT_3PARAM_RETVALUE(KviEvent_OnChannelNotice,chan,szNick,szMsgText,szTarget))msg->setHaltOutput();
+		if(KVS_TRIGGER_EVENT_3_HALTED(KviEvent_OnChannelNotice,chan,szNick,szMsgText,szOriginalTarget))msg->setHaltOutput();
 
 		if(!msg->haltOutput())
 		{
-			if(flaggedNotice.unicode())
+			if(szPrefixes.length() > 0)
 			{
 				QString szBroad;
-				QString szFlag = flaggedNotice;
-				QString szNk = szTarget.right(szTarget.length() - 1);
-				KviQString::sprintf(szBroad,"[>> %Q\r!c\r%Q\r] %Q",&szFlag,&szNk,&szMsgText);
+				KviQString::sprintf(szBroad,"[>> %Q\r!c\r%Q\r] %Q",&szPrefixes,&szTarget,&szMsgText);
 				console->outputPrivmsg(chan,msgtype,szNick,szUser,szHost,szBroad,0);
 			} else {
 				console->outputPrivmsg(chan,msgtype,szNick,szUser,szHost,szMsgText,0);
@@ -1343,8 +1400,8 @@ void KviServerParser::parseLiteralTopic(KviIrcMessage *msg)
 
 	QString szTopic = chan->decodeText(msg->safeTrailing());
 
-	if(TRIGGER_EVENT_4PARAM_RETVALUE(KviEvent_OnTopic,chan,szNick,szUser,szHost,szTopic))
-			msg->setHaltOutput();
+	if(KVS_TRIGGER_EVENT_4_HALTED(KviEvent_OnTopic,chan,szNick,szUser,szHost,szTopic))
+		msg->setHaltOutput();
 
 	chan->topicWidget()->setTopic(szTopic);
 	chan->topicWidget()->setTopicSetBy(szNick);
@@ -1377,10 +1434,10 @@ void KviServerParser::parseLiteralNick(KviIrcMessage *msg)
 		// We have changed our nick
 		msg->connection()->nickChange(szNewNick);
 
-		if(TRIGGER_EVENT_2PARAM_RETVALUE(KviEvent_OnMeNickChange,console,szNick,szNewNick))
+		if(KVS_TRIGGER_EVENT_2_HALTED(KviEvent_OnMeNickChange,console,szNick,szNewNick))
 				msg->setHaltOutput();
 	} else {
-		if(TRIGGER_EVENT_4PARAM_RETVALUE(KviEvent_OnNickChange,console,szNick,szUser,szHost,szNewNick))
+		if(KVS_TRIGGER_EVENT_4_HALTED(KviEvent_OnNickChange,console,szNick,szUser,szHost,szNewNick))
 				msg->setHaltOutput();
 	}
 
@@ -1408,66 +1465,58 @@ void KviServerParser::parseLiteralNick(KviIrcMessage *msg)
 				q->output(KVI_OUT_NICK,__tr2qs("You have changed your nickname to %Q"),&szNewNick);
 			q->updateCaption();
 		}
-	} else {
-		KviQuery * q = console->connection()->findQuery(szNick);
-		// It CAN happen that szNewNick first queries us without being
-		// on any channel then he QUITS , he reconnects , he joins
-		// a channel with szNick , queries us and changes nick to szNewNick : gotcha!
-		// should merge the queries!
-		KviQuery * old = console->connection()->findQuery(szNewNick);
-		if(old && (old != q))
-		{
-			if(KVI_OPTION_BOOL(KviOption_boolEnableQueryTracing) && (!_OUTPUT_QUIET))
-			{
-				old->output(KVI_OUT_QUERYTRACE,
-					__tr2qs("The target of this query was lost and has been found when \r!n\r%Q\r [%Q@\r!h\r%Q\r] changed his nickname to \r!n\r%Q\r"),
-					&szNick,&szUser,&szHost,&szNewNick);
-			}
-
-			if(q)
-			{
-				bool bQWasActive = (q == g_pActiveWindow);
-				if(!_OUTPUT_MUTE)
-				{
-					old->output(KVI_OUT_SYSTEMWARNING,
-							__tr2qs("The recent nickname change from \r!n\r%Q\r to \r!n\r%Q\r caused a query collision: merging output"),
-							&szNick,&szNewNick);
-				}
-				old->mergeQuery(q);
-				q->frame()->closeWindow(q); // deleted path
-
-				if(!msg->haltOutput())
-					old->output(KVI_OUT_NICK,__tr2qs("\r!n\r%Q\r [%Q@\r!h\r%Q\r] is now known as \r!n\r%Q\r"),
-						&szNick,&szUser,&szHost,&szNewNick);
-
-				if(!_OUTPUT_MUTE)
-					old->output(KVI_OUT_SYSTEMWARNING,__tr2qs("End of merged output"));
-
-				old->userAction(szNewNick,szUser,szHost,KVI_USERACTION_NICK);
-				if(bQWasActive)old->delayedAutoRaise();
-			}
-
-			if(KVI_OPTION_BOOL(KviOption_boolEnableQueryTracing))
-			{
-				QString szChans;
-				int iChans = console->connection()->getCommonChannels(szNewNick,szChans);
-				old->notifyCommonChannels(szNewNick,szUser,szHost,iChans,szChans);
-			}
-		} else {
-			if(q)
-			{
-				// the target SHOULD have changed his nick here
-				if(!q->nickChange(szNick,szNewNick))
-					debug("Internal error: query %s failed to change nick from %s to %s",szNick.latin1(),szNick.latin1(),szNewNick.latin1());
-				if(!msg->haltOutput())
-					q->output(KVI_OUT_NICK,__tr2qs("\r!n\r%Q\r [%Q@\r!h\r%Q\r] is now known as \r!n\r%Q\r"),
-						&szNick,&szUser,&szHost,&szNewNick);
-				q->userAction(szNewNick,szUser,szHost,KVI_USERACTION_NICK);
-			}
-		}
-
 	}
-
+	KviQuery * q = console->connection()->findQuery(szNick);
+	// It CAN happen that szNewNick first queries us without being
+	// on any channel then he QUITS , he reconnects , he joins
+	// a channel with szNick , queries us and changes nick to szNewNick : gotcha!
+	// should merge the queries!
+	KviQuery * old = console->connection()->findQuery(szNewNick);
+	if(old && (old != q))
+	{
+		if(KVI_OPTION_BOOL(KviOption_boolEnableQueryTracing) && (!_OUTPUT_QUIET))
+		{
+		old->output(KVI_OUT_QUERYTRACE,
+			__tr2qs("The target of this query was lost and has been found when \r!n\r%Q\r [%Q@\r!h\r%Q\r] changed his nickname to \r!n\r%Q\r"),
+			&szNick,&szUser,&szHost,&szNewNick);
+		}
+		if(q)
+		{
+			bool bQWasActive = (q == g_pActiveWindow);
+			if(!_OUTPUT_MUTE)
+				{
+				old->output(KVI_OUT_SYSTEMWARNING,
+						__tr2qs("The recent nickname change from \r!n\r%Q\r to \r!n\r%Q\r caused a query collision: merging output"),
+						&szNick,&szNewNick);
+			}
+				old->mergeQuery(q);
+			q->frame()->closeWindow(q); // deleted path
+			if(!msg->haltOutput())
+					old->output(KVI_OUT_NICK,__tr2qs("\r!n\r%Q\r [%Q@\r!h\r%Q\r] is now known as \r!n\r%Q\r"),
+					&szNick,&szUser,&szHost,&szNewNick);
+			if(!_OUTPUT_MUTE)
+				old->output(KVI_OUT_SYSTEMWARNING,__tr2qs("End of merged output"));
+				old->userAction(szNewNick,szUser,szHost,KVI_USERACTION_NICK);
+			if(bQWasActive)old->delayedAutoRaise();
+		}
+		if(KVI_OPTION_BOOL(KviOption_boolEnableQueryTracing))
+		{
+				QString szChans;
+			int iChans = console->connection()->getCommonChannels(szNewNick,szChans);
+			old->notifyCommonChannels(szNewNick,szUser,szHost,iChans,szChans);
+		}
+	} else {
+		if(q)
+		{
+			// the target SHOULD have changed his nick here
+			if(!q->nickChange(szNick,szNewNick))
+					debug("Internal error: query %s failed to change nick from %s to s",szNick.utf8().data(),szNick.utf8().data(),szNewNick.utf8().data());
+			if(!msg->haltOutput())
+				q->output(KVI_OUT_NICK,__tr2qs("\r!n\r%Q\r [%Q@\r!h\r%Q\r] is now known as \r!n\r%Q\r"),
+					&szNick,&szUser,&szHost,&szNewNick);
+			q->userAction(szNewNick,szUser,szHost,KVI_USERACTION_NICK);
+		}
+	}
 
 // FIXME: #warning "UPDATE ALL THE OTHER CONNECTION RELATED WINDOW CAPTIONS WHEN bIsMe!!"
 }
@@ -1484,7 +1533,7 @@ void KviServerParser::parseLiteralInvite(KviIrcMessage *msg)
 
 	if(IS_ME(msg,szTarget))
 	{
-		if(TRIGGER_EVENT_4PARAM_RETVALUE(KviEvent_OnInvite,msg->console(),szNick,szUser,szHost,szChannel))
+		if(KVS_TRIGGER_EVENT_4_HALTED(KviEvent_OnInvite,msg->console(),szNick,szUser,szHost,szChannel))
 			msg->setHaltOutput();
 		
 		if(!msg->haltOutput())
@@ -1513,7 +1562,7 @@ void KviServerParser::parseLiteralWallops(KviIrcMessage *msg)
 
 	QString szMsg = msg->connection()->decodeText(msg->safeTrailing());
 
-	if(TRIGGER_EVENT_4PARAM_RETVALUE(KviEvent_OnWallops,msg->console(),szNick,szUser,szHost,szMsg))
+	if(KVS_TRIGGER_EVENT_4_HALTED(KviEvent_OnWallops,msg->console(),szNick,szUser,szHost,szMsg))
 		msg->setHaltOutput();
 
 	if(!msg->haltOutput())
@@ -1541,8 +1590,10 @@ void KviServerParser::parseUserMode(KviIrcMessage *msg,const char * modeflptr)
 					// There was a mode change
 					if(KviKvsEventManager::instance()->hasAppHandlers(KviEvent_OnUserMode))
 					{
-						KviStr * modefl = new KviStr(KviStr::Format,"%c%c",bSet ? '+' : '-',*modeflptr);
-						if(g_pUserParser->triggerEvent(KviEvent_OnUserMode,msg->console(),new KviParameterList(modefl)))
+						QString szModeFlag(bSet ? QChar('+') : QChar('-'));
+						szModeFlag += QChar(*modeflptr);
+						KviKvsVariantList vList(new KviKvsVariant(szModeFlag));
+						if(KviKvsEventManager::instance()->trigger(KviEvent_OnUserMode,msg->console(),&vList))
 							msg->setHaltOutput();
 					}
 				}
@@ -1650,10 +1701,10 @@ void KviServerParser::parseChannelMode(const QString &szNick,const QString &szUs
 				chan->setChannelKey(aParam);
 
 				if(bSet) {
-					if(TRIGGER_EVENT_4PARAM_RETVALUE(KviEvent_OnKeySet,chan,szNick,szUser,szHost,aParam))
+					if(KVS_TRIGGER_EVENT_4_HALTED(KviEvent_OnKeySet,chan,szNick,szUser,szHost,aParam))
 						msg->setHaltOutput();
 				} else {
-					if(TRIGGER_EVENT_3PARAM_RETVALUE(KviEvent_OnKeyUnset,chan,szNick,szUser,szHost))
+					if(KVS_TRIGGER_EVENT_3_HALTED(KviEvent_OnKeyUnset,chan,szNick,szUser,szHost))
 						msg->setHaltOutput();
 				}
 				
@@ -1673,10 +1724,10 @@ void KviServerParser::parseChannelMode(const QString &szNick,const QString &szUs
 				chan->setChannelLimit(aParam);
 				
 				if(bSet) {
-					if(TRIGGER_EVENT_4PARAM_RETVALUE(KviEvent_OnLimitSet,chan,szNick,szUser,szHost,aParam))
+					if(KVS_TRIGGER_EVENT_4_HALTED(KviEvent_OnLimitSet,chan,szNick,szUser,szHost,aParam))
 						msg->setHaltOutput();
 				} else {
-					if(TRIGGER_EVENT_3PARAM_RETVALUE(KviEvent_OnLimitUnset,chan,szNick,szUser,szHost))
+					if(KVS_TRIGGER_EVENT_3_HALTED(KviEvent_OnLimitUnset,chan,szNick,szUser,szHost))
 						msg->setHaltOutput();
 				}
 				
@@ -1698,10 +1749,10 @@ void KviServerParser::parseChannelMode(const QString &szNick,const QString &szUs
 				bIsMe = IS_ME(msg,aParam); \
 				if(bIsMe) \
 				{ \
-					if(TRIGGER_EVENT_3PARAM_RETVALUE(bSet ? __evmeset : __evmeunset,chan,szNick,szUser,szHost))msg->setHaltOutput(); \
+					if(KVS_TRIGGER_EVENT_3_HALTED(bSet ? __evmeset : __evmeunset,chan,szNick,szUser,szHost))msg->setHaltOutput(); \
 					chan->updateCaption(); \
 				} else { \
-					if(TRIGGER_EVENT_4PARAM_RETVALUE(bSet ? __evset : __evunset,chan,szNick,szUser,szHost,aParam))msg->setHaltOutput(); \
+					if(KVS_TRIGGER_EVENT_4_HALTED(bSet ? __evset : __evunset,chan,szNick,szUser,szHost,aParam))msg->setHaltOutput(); \
 				} \
 				if(!(msg->haltOutput() || KVI_OPTION_BOOL(KviOption_boolShowCompactModeChanges))) \
 				{ \
@@ -1720,7 +1771,7 @@ void KviServerParser::parseChannelMode(const QString &szNick,const QString &szUs
 #define CHANNEL_MODE(__modefl,__evmeset,__evmeunset,__evset,__evunset,__icomeset,__icomeunset,__icoset,__icounset) \
 			case __modefl: \
 				aParam = msg->connection()->decodeText(msg->safeParam(curParam++)); \
-				chan->setMask(*aux,aParam,bSet,msg->connection()->decodeText(msg->safePrefix()),QDateTime::currentDateTime().toString()); \
+				chan->setMask(*aux,aParam,bSet,msg->connection()->decodeText(msg->safePrefix()),QDateTime::currentDateTime().toTime_t()); \
 				auxMask = new KviIrcMask(aParam); \
 				bIsMe = auxMask->matchesFixed( \
 							msg->connection()->userInfo()->nickName(), \
@@ -1729,9 +1780,9 @@ void KviServerParser::parseChannelMode(const QString &szNick,const QString &szUs
 				delete auxMask; \
 				if(bIsMe) \
 				{ \
-					if(TRIGGER_EVENT_4PARAM_RETVALUE(bSet ? __evmeset : __evmeunset,chan,szNick,szUser,szHost,aParam))msg->setHaltOutput(); \
+					if(KVS_TRIGGER_EVENT_4_HALTED(bSet ? __evmeset : __evmeunset,chan,szNick,szUser,szHost,aParam))msg->setHaltOutput(); \
 				} else { \
-					if(TRIGGER_EVENT_4PARAM_RETVALUE(bSet ? __evset : __evunset,chan,szNick,szUser,szHost,aParam))msg->setHaltOutput(); \
+					if(KVS_TRIGGER_EVENT_4_HALTED(bSet ? __evset : __evunset,chan,szNick,szUser,szHost,aParam))msg->setHaltOutput(); \
 				} \
 				if(!(msg->haltOutput() || KVI_OPTION_BOOL(KviOption_boolShowCompactModeChanges))) \
 				{ \
@@ -1760,18 +1811,21 @@ void KviServerParser::parseChannelMode(const QString &szNick,const QString &szUs
 		++aux;
 	}
 
+	QString param;
+	QString params;
+	param = msg->connection()->decodeText(msg->safeParam(curParamSave++));
+	while(!param.isEmpty())
+	{
+		if(!params.isEmpty())params.append(' ');
+		params.append(param);
+		param = msg->connection()->decodeText(msg->safeParam(curParamSave++));
+	}
+
+	if(KVS_TRIGGER_EVENT_5_HALTED(KviEvent_OnChannelModeChange,chan,szNick,szUser,szHost,modefl.ptr(),params))
+		msg->setHaltOutput();
+
 	if(KVI_OPTION_BOOL(KviOption_boolShowCompactModeChanges) && (!msg->haltOutput()) && (!kvi_strEqualCS(modefl.ptr(),"+")))
 	{
-		QString param;
-		QString params;
-		param = msg->connection()->decodeText(msg->safeParam(curParamSave++));
-		while(!param.isEmpty())
-		{
-			if(!params.isEmpty())params.append(' ');
-			params.append(param);
-			param = msg->connection()->decodeText(msg->safeParam(curParamSave++));
-		}
-
 		if(!params.isEmpty())
 		{
 			chan->output(KVI_OUT_CHANMODE,__tr2qs("%Q [%Q@%Q] has set mode %s %Q"),
