@@ -26,6 +26,7 @@
 
 #include "kvi_sparser.h"
 #include "kvi_window.h"
+#include "kvi_query.h"
 #include "kvi_out.h"
 #include "kvi_locale.h"
 #include "kvi_ircsocket.h"
@@ -36,12 +37,10 @@
 #include "kvi_defaults.h"
 #include "kvi_mirccntrl.h"
 #include "kvi_frame.h"
-#include "kvi_uparser.h"
 #include "kvi_parameterlist.h"
 #include "kvi_app.h"
 #include "kvi_notifylist.h"
 #include "kvi_numeric.h"
-#include "kvi_event.h"
 #include "kvi_ircconnection.h"
 #include "kvi_ircconnectionstatedata.h"
 #include "kvi_ircconnectionuserinfo.h"
@@ -49,10 +48,16 @@
 #include "kvi_ircconnectionasyncwhoisdata.h"
 #include "kvi_ircconnectiontarget.h"
 #include "kvi_time.h"
+#include "kvi_lagmeter.h"
 
 #include <qpixmap.h>
 #include <qdatetime.h>
 #include <qtextcodec.h>
+#include <qregexp.h> 
+
+#include "kvi_kvs_eventtriggers.h"
+#include "kvi_kvs_script.h"
+#include "kvi_kvs_variantlist.h"
 
 // #define IS_CHANNEL_TYPE_FLAG(_str) ((*(_str) == '#') || (*(_str) == '&') || (*(_str) == '!'))
 #define IS_CHANNEL_TYPE_FLAG(_qchar) (msg->connection()->serverInfo()->supportedChannelTypes().find(_qchar) != -1)
@@ -68,11 +73,18 @@ void KviServerParser::parseNumeric001(KviIrcMessage *msg)
 	// 001: RPL_WELCOME
 	// :prefix 001 target :Welcome to the Internet Relay Network <usermask>
 	// FIXME: #warning "SET THE USERMASK FROM SERVER"
-	
+	QString szText = msg->connection()->decodeText(msg->safeTrailing());
+	QRegExp rx( " ([^ ]+)!([^ ]+)@([^ ]+)$" );
+	if( rx.search(szText) != -1)
+	{
+		msg->connection()->userInfo()->setUnmaskedHostName(rx.cap(3));
+		msg->connection()->userInfo()->setNickName(rx.cap(1));
+		msg->connection()->userInfoReceived(rx.cap(2),rx.cap(3));
+	}
 	if(msg->connection()->context()->state() != KviIrcContext::Connected)
 		msg->connection()->loginComplete(msg->connection()->decodeText(msg->param(0)));
 	if(!msg->haltOutput())
-		msg->console()->output(KVI_OUT_SERVERINFO,msg->connection()->decodeText(msg->safeTrailing()));
+		msg->console()->output(KVI_OUT_SERVERINFO,szText);
 }
 
 void KviServerParser::parseNumeric002(KviIrcMessage *msg)
@@ -116,6 +128,8 @@ const QString & getChannelModeDescription(char c)
 		case 'm': return __tr2qs("Moderated"); break;
 		case 'r': return __tr2qs("Registered"); break;
 		case 'G': return __tr2qs("Censor swear words"); break;
+		case 'M': return __tr2qs("Impossible to speak and change nick if not authenticated with NickServ"); break;
+		case 'N': return __tr2qs("Impossible to change nick if not authenticated with NickServ"); break;
 		case 'R': return __tr2qs("Only registered nicks can join"); break;
 		case 'Q': return __tr2qs("No kicks able (unless U-Line)"); break;
 		case 'O': return __tr2qs("IRC-Op only channel"); break;
@@ -125,6 +139,8 @@ const QString & getChannelModeDescription(char c)
 		case 'L': return __tr2qs("Redirect on channel full"); break;
 		case 'C': return __tr2qs("Forbid channel CTCPs"); break;
 		case 'u': return __tr2qs("User-operators OR Auditorium: /NAMES and /WHO show only ops"); break;
+		case '7': return __tr2qs("Only 7-bit letters in nicknames allowed"); break;
+		case 'x': return __tr2qs("Impossible to use both 7-bit and 8-bit letters in nicknames"); break;
 		default: return __tr2qs("Unknown channel mode"); break;
 	}
 }
@@ -241,6 +257,7 @@ void KviServerParser::parseNumeric005(KviIrcMessage *msg)
 	// :prefix 005 target <proto> <proto> .... :are available/supported on this server
 	// 005: RPL_BOUNCE [?]
 	// :prefix 005 target :Try server <server>, port <port>
+	// 005: RPL_ISUPPORT
 	if(msg->connection()->context()->state() != KviIrcContext::Connected)
 		msg->connection()->loginComplete(msg->connection()->decodeText(msg->param(0)));
 
@@ -269,14 +286,22 @@ void KviServerParser::parseNumeric005(KviIrcMessage *msg)
 				p+=10;
 				KviStr tmp = p;
 				if(tmp.hasData())msg->connection()->serverInfo()->setSupportedChannelTypes(tmp.ptr());
-				
-			} else if(kvi_strEqualCI("WATCH",p))
+			} else if(kvi_strEqualCI("WATCH",p) || kvi_strEqualCIN("WATCH=",p,6))
 			{
 				msg->connection()->serverInfo()->setSupportsWatchList(true);
 				if((!_OUTPUT_MUTE) && (!msg->haltOutput()) && KVI_OPTION_BOOL(KviOption_boolShowExtendedServerInfo))
 				{
 					msg->console()->outputNoFmt(KVI_OUT_SERVERINFO,__tr2qs("This server supports the WATCH notify list method, it will be used"));
 				}
+			} else if(kvi_strEqualCIN("TOPICLEN=",p,9))
+			{
+				p += 9;
+				QString tmp = p;
+				if(!tmp.isEmpty()) {
+					bool ok;
+        				int len = tmp.toInt( &ok );
+        				if(ok) msg->connection()->serverInfo()->setMaxTopicLen(len);
+				}
 			} else if(kvi_strEqualCIN("NETWORK=",p,8))
 			{
 				p += 8;
@@ -295,7 +320,22 @@ void KviServerParser::parseNumeric005(KviIrcMessage *msg)
 				{
 					msg->console()->outputNoFmt(KVI_OUT_SERVERINFO,__tr2qs("This server supports the CODEPAGE command, it will be used"));
 				}
-			}
+			} else if(kvi_strEqualCIN("CHANMODES=",p,10))
+			{
+				p+=10;
+				QString tmp = p;
+				msg->connection()->serverInfo()->setSupportedChannelModes(tmp);
+			}else if(kvi_strEqualCIN("MODES=",p,6))
+			{
+				p+=6;
+				QString tmp = p;
+				bool bok;
+				int num=tmp.toUInt(&bok);
+				if(bok)
+					msg->connection()->serverInfo()->setMaxModeChanges(num);
+			}/* else if(kvi_strEqualCIN("CAPAB",p,5))
+			{
+			}*/
 		}
 		if((!_OUTPUT_MUTE) && (!msg->haltOutput()))
 		{
@@ -304,7 +344,7 @@ void KviServerParser::parseNumeric005(KviIrcMessage *msg)
 			while(*aux && (*aux != ' '))aux++;
 			while(*aux == ' ')aux++;
 			if(*aux == ':')aux++;
-			if(!msg->haltOutput())msg->console()->output(KVI_OUT_SERVERINFO,__tr2qs("This server supports: %s"),aux);
+			if(!msg->haltOutput())msg->console()->output(KVI_OUT_SERVERINFO,__tr2qs("This server supports: %s"),msg->connection()->decodeText(aux).utf8().data());
 		}
 	} else {
 		QString inf = msg->connection()->decodeText(msg->safeTrailing());
@@ -361,6 +401,20 @@ void KviServerParser::parseNumericEndOfNames(KviIrcMessage *msg)
 	}
 }
 
+void KviServerParser::parseNumeric020(KviIrcMessage *msg)
+{
+	// 020: RPL_CONNECTING
+	//:irc.dotsrc.org 020 * :Please wait while we process your connection.
+	QString szServer;
+	if(!msg->haltOutput())
+	{
+		QString szWText = msg->console()->decodeText(msg->safeTrailing());
+		msg->console()->output(
+			KVI_OUT_CONNECTION,__tr2qs("%c\r!s\r%s\r%c: %Q"),KVI_TEXT_BOLD,
+			msg->safePrefix(),KVI_TEXT_BOLD,&szWText);
+	}
+}
+
 void KviServerParser::parseNumericNames(KviIrcMessage *msg)
 {
 	// 353: RPL_NAMREPLY [I,E,U,D]
@@ -369,11 +423,9 @@ void KviServerParser::parseNumericNames(KviIrcMessage *msg)
 	// [=|*|@] is the type of the channel:
 	// = --> public  * --> private   @ --> secret
 	// ...but we ignore it
-
+	//QString szChan = msg->connection()->decodeText(msg->cSafeParam(2)->data()); // <-- QCString::data() is implicitly unsafe: it CAN return 0
 	QString szChan = msg->connection()->decodeText(msg->safeParam(2));
-
 	KviChannel * chan = msg->connection()->findChannel(szChan);
-
 	// and run to the first nickname
 	char * aux = msg->safeTrailingString().ptr();
 	while((*aux) && (*aux == ' '))aux++;
@@ -530,6 +582,13 @@ void KviServerParser::parseNumericTopicWhoTime(KviIrcMessage *msg)
 	QString szDate = dt.toString();
 	QString szWho = msg->connection()->decodeText(msg->safeParam(2));
 	KviIrcMask who(szWho);
+	QString szDisplayableWho;
+	if( !(who.hasUser() && who.hasHost()) )
+	{
+		szDisplayableWho="\r!n\r"+szWho+"\r";
+	} else {
+		KviQString::sprintf(szDisplayableWho,"\r!n\r%Q\r!%Q@\r!h\r%Q\r",&(who.nick()),&(who.user()),&(who.host()));
+	}
 	if(chan)
 	{
 		chan->topicWidget()->setTopicSetBy(szWho);
@@ -538,8 +597,8 @@ void KviServerParser::parseNumericTopicWhoTime(KviIrcMessage *msg)
 		{
 			if(!msg->haltOutput())
 			{
-				if(bOk)chan->output(KVI_OUT_TOPIC,__tr2qs("Topic was set by \r!n\r%Q\r!%Q@\r!h\r%Q\r on %Q"),&(who.nick()),&(who.user()),&(who.host()),&szDate);
-				else chan->output(KVI_OUT_TOPIC,__tr2qs("Topic was set by \r!n\r%Q\r!%Q@\r!h\r%Q\r"),&(who.nick()),&(who.user()),&(who.host()));
+				if(bOk)chan->output(KVI_OUT_TOPIC,__tr2qs("Topic was set by %Q on %Q"),&szDisplayableWho,&szDate);
+				else chan->output(KVI_OUT_TOPIC,__tr2qs("Topic was set by %Q"),&szDisplayableWho);
 			}
 		}
 	} else {
@@ -547,11 +606,11 @@ void KviServerParser::parseNumericTopicWhoTime(KviIrcMessage *msg)
 			msg->console()->activeWindow() : (KviWindow *)(msg->console());
 		if(bOk)
 		{
-			pOut->output(KVI_OUT_TOPIC,__tr2qs("Topic for \r!c\r%Q\r was set by \r!n\r%Q\r!%Q@\r!h\r%Q\r on %Q"),
-					&szChan,&(who.nick()),&(who.user()),&(who.host()),&szDate);
+			pOut->output(KVI_OUT_TOPIC,__tr2qs("Topic for \r!c\r%Q\r was set by %Q on %Q"),
+					&szChan,&szDisplayableWho,&szDate);
 		} else {
-			pOut->output(KVI_OUT_TOPIC,__tr2qs("Topic for \r!c\r%Q\r was set by \r!n\r%Q\r!%Q@\r!h\r%Q\r"),
-					&szChan,&(who.nick()),&(who.user()),&(who.host()));
+			pOut->output(KVI_OUT_TOPIC,__tr2qs("Topic for \r!c\r%Q\r was set by %Q"),
+					&szChan,&szDisplayableWho);
 		}
 	}
 }
@@ -629,7 +688,7 @@ PARSE_NUMERIC_ENDOFLIST(parseNumericEndOfExceptList,setHasBanExceptionList,sentB
 		KviChannel * chan = msg->connection()->findChannel(szChan); \
 		if(chan) \
 		{ \
-			chan->setMask(__modechar,banmask,true,bansetby,bansetat); \
+			chan->setMask(__modechar,banmask,true,bansetby,QString(msg->safeParam(4)).toUInt()); \
 			if(chan->__sentRequest())return; \
 		} \
 		if(!msg->haltOutput()) \
@@ -662,6 +721,8 @@ void KviServerParser::parseNumericWhoReply(KviIrcMessage *msg)
 	QString szHost = msg->connection()->decodeText(msg->safeParam(3));
 	QString szServ = msg->connection()->decodeText(msg->safeParam(4));
 	QString szNick = msg->connection()->decodeText(msg->safeParam(5));
+	QString szFlag = msg->connection()->decodeText(msg->safeParam(6));
+	bool bAway = szFlag.find('G') != -1;
 
 	KviStr trailing = msg->safeTrailing();
 	KviStr hops = trailing.getToken(' ');
@@ -680,6 +741,9 @@ void KviServerParser::parseNumericWhoReply(KviIrcMessage *msg)
 		e->setHost(szHost);
 		e->setServer(szServ);
 		e->setRealName(szReal);
+		e->setAway(bAway);
+		KviQuery * q = msg->connection()->findQuery(szNick);
+		if(q) q->updateLabelText();
 		if(!e->avatar())
 		{
 			// FIXME: #warning "THE AVATAR SHOULD BE RESIZED TO MATCH THE MAX WIDTH/HEIGHT"
@@ -696,6 +760,11 @@ void KviServerParser::parseNumericWhoReply(KviIrcMessage *msg)
 			// FIXME: #warning "IF VERBOSE && SHOW INTERNAL WHO REPLIES...."
 			return;
 		}
+		if(chan->sentSyncWhoRequest())
+		{
+			// FIXME: #warning "IF VERBOSE && SHOW INTERNAL WHO REPLIES...."
+			return;
+		}
 	}
 
 	// FIXME: #warning "SYNC OP/VOICE on channel!!!"
@@ -705,8 +774,7 @@ void KviServerParser::parseNumericWhoReply(KviIrcMessage *msg)
 		KviWindow * pOut = chan ? chan : KVI_OPTION_BOOL(KviOption_boolServerRepliesToActiveWindow) ?
 			msg->console()->activeWindow() : (KviWindow *)(msg->console());
 
-		QString szAway = (*(msg->safeParam(6)) == 'G') ? __tr2qs("Yes") : __tr2qs("No");
-		QString szFlags = msg->connection()->decodeText(msg->safeParam(6));
+		QString szAway = bAway ? __tr2qs("Yes") : __tr2qs("No");
 
 		pOut->output(KVI_OUT_WHO,
 			__tr2qs("WHO entry for %c\r!n\r%Q\r%c [%Q@\r!h\r%Q\r]: %cChannel%c: \r!c\r%Q\r, %cServer%c: \r!s\r%Q\r, %cHops%c: %d, %cFlags%c: %Q, %cAway%c: %Q, %cReal name%c: %Q"),
@@ -715,7 +783,7 @@ void KviServerParser::parseNumericWhoReply(KviIrcMessage *msg)
 			KVI_TEXT_UNDERLINE,&szChan,KVI_TEXT_UNDERLINE,
 			KVI_TEXT_UNDERLINE,&szServ,KVI_TEXT_UNDERLINE,
 			KVI_TEXT_UNDERLINE,iHops, KVI_TEXT_UNDERLINE, KVI_TEXT_UNDERLINE,
-			&szFlags, KVI_TEXT_UNDERLINE, KVI_TEXT_UNDERLINE,
+			&szFlag, KVI_TEXT_UNDERLINE, KVI_TEXT_UNDERLINE,
 			&szAway, KVI_TEXT_UNDERLINE,
 			KVI_TEXT_UNDERLINE, &szReal);
 	}
@@ -731,13 +799,30 @@ void KviServerParser::parseNumericEndOfWho(KviIrcMessage *msg)
 	KviChannel * chan = msg->connection()->findChannel(szChan);
 	if(chan)
 	{
+		kvi_time_t tNow = kvi_unixTime();
+		msg->connection()->stateData()->setLastReceivedChannelWhoReply(tNow);
+		chan->setLastReceivedWhoReply(tNow);
+		if(msg->connection()->lagMeter())
+		{
+			KviStr tmp(KviStr::Format,"WHO %s",msg->safeParam(1));
+			msg->connection()->lagMeter()->lagCheckComplete(tmp.ptr());
+		}
+
 		if(!chan->hasWhoList())
 		{
 			// FIXME: #warning "IF VERBOSE && SHOW INTERNAL WHO REPLIES...."
 			chan->setHasWhoList();
 			return;
 		}
+		
+		if(chan->sentSyncWhoRequest())
+		{
+			// FIXME: #warning "IF VERBOSE && SHOW INTERNAL WHO REPLIES...."
+			chan->clearSentSyncWhoRequest();
+			return;
+		}
 	}
+
 	if(!msg->haltOutput())
 	{
 		KviWindow * pOut = chan ? chan : KVI_OPTION_BOOL(KviOption_boolServerRepliesToActiveWindow) ?
@@ -867,6 +952,65 @@ void KviServerParser::parseNumericCantJoinChannel(KviIrcMessage *msg)
 	}
 }
 
+void KviServerParser::otherChannelError(KviIrcMessage *msg)
+{
+	// 482: ERR_CHANOPRIVSNEEDED
+	// 467: ERR_KEYSET
+	// 472: ERR_UNKNOWNMODE
+	// :prefix 4?? <target> <channel> :error text
+	if(!msg->haltOutput())
+	{
+		KviWindow * pOut = (KviWindow *)(msg->connection()->findChannel(msg->safeParam(1)));
+		if(!pOut)pOut = (KviWindow *)(msg->console());
+		QString szChannel = msg->connection()->decodeText(msg->safeParam(1));
+		QString szWText = msg->connection()->decodeText(msg->safeTrailing());
+		pOut->output(KVI_OUT_GENERICERROR,
+			"\r!c\r%Q\r: %Q",&szChannel,&szWText);
+	}
+}
+
+void KviServerParser::commandSyntaxHelp(KviIrcMessage *msg)
+{
+	// 704 RPL_COMMANDSYNTAX
+	// :prefix 704 <target> <command> :text
+	if(!msg->haltOutput())
+	{
+		KviWindow * pOut = (KviWindow *)(msg->console());
+		QString szCommand = msg->connection()->decodeText(msg->safeParam(1));
+		QString szWText = msg->connection()->decodeText(msg->safeTrailing());
+		pOut->output(KVI_OUT_HELP,
+			"Синатксис команды %Q: %Q",&szCommand,&szWText);
+	}
+}
+
+void KviServerParser::commandHelp(KviIrcMessage *msg)
+{
+	// 705 RPL_COMMANDHELP
+	// :prefix 705 <target> <command> :text
+	if(!msg->haltOutput())
+	{
+		KviWindow * pOut = (KviWindow *)(msg->console());
+		QString szCommand = msg->connection()->decodeText(msg->safeParam(1));
+		QString szWText = msg->connection()->decodeText(msg->safeTrailing());
+		pOut->output(KVI_OUT_HELP,
+			"%Q",&szWText);
+	}
+}
+
+void KviServerParser::commandEndOfHelp(KviIrcMessage *msg)
+{
+	// 704 RPL_COMMANDSYNTAX
+	// 705 RPL_COMMANDHELP
+	// :prefix 706 <target> <command> :End of /HELP.
+	if(!msg->haltOutput())
+	{
+		KviWindow * pOut = (KviWindow *)(msg->console());
+		QString szCommand = msg->connection()->decodeText(msg->safeParam(1));
+		pOut->output(KVI_OUT_HELP,
+			"Конец справки по команде %Q",&szCommand);
+	}
+}
+
 void KviServerParser::parseNumericNicknameProblem(KviIrcMessage *msg)
 {
 	// 433: ERR_NICKNAMEINUSE [I,E,U,D]
@@ -892,9 +1036,14 @@ void KviServerParser::parseNumericNicknameProblem(KviIrcMessage *msg)
 void KviServerParser::parseNumericWhoisAway(KviIrcMessage * msg)
 {
 // FIXME: #warning "Need an icon here too: sth like KVI_OUT_WHOISSERVER, but with 'A' letter"
+	QString szNk = msg->connection()->decodeText(msg->safeParam(1));
+	KviIrcUserDataBase * db = msg->connection()->userDataBase();
+	KviIrcUserEntry * e = db->find(szNk);
+	if(e)e->setAway(true);
+	KviQuery * q = msg->connection()->findQuery(szNk);
+	if(q) q->updateLabelText();
 	if(!msg->haltOutput())
 	{
-		QString szNk = msg->connection()->decodeText(msg->safeParam(1));
 		KviWindow * pOut = (KviWindow *)(msg->connection()->findQuery(szNk));
 		QString szWText = pOut ? pOut->decodeText(msg->safeTrailing()) : msg->connection()->decodeText(msg->safeTrailing());
 
@@ -920,6 +1069,8 @@ void KviServerParser::parseNumericWhoisUser(KviIrcMessage *msg)
 		e->setUser(szUser);
 		e->setHost(szHost);
 		e->setRealName(szReal);
+		KviQuery * q = msg->connection()->findQuery(szNick);
+		if(q) q->updateLabelText();
 		if(!e->avatar())
 		{
 			// FIXME: #warning "THE AVATAR SHOULD BE RESIZED TO MATCH THE MAX WIDTH/HEIGHT"
@@ -1106,7 +1257,8 @@ void KviServerParser::parseNumericWhoisServer(KviIrcMessage *msg)
 	KviIrcUserDataBase * db = msg->connection()->userDataBase();
 	KviIrcUserEntry * e = db->find(szNick);
 	if(e)e->setServer(szServ);
-
+	KviQuery * q = msg->connection()->findQuery(szNick);
+	if(q) q->updateLabelText();
 	KviAsyncWhoisInfo * i = msg->connection()->asyncWhoisData()->lookup(msg->safeParam(1));
 	if(i)
 	{
@@ -1126,6 +1278,24 @@ void KviServerParser::parseNumericWhoisServer(KviIrcMessage *msg)
 	}
 }
 
+void KviServerParser::parseNumericWhoisAuth(KviIrcMessage *msg)
+{
+	// :prefix RPL_WHOISAUTH <target> <nick> :is authed as
+	// actually seen only on Quakenet
+
+	QString szNick = msg->connection()->decodeText(msg->safeParam(1));
+	QString szAuth = msg->connection()->decodeText(msg->safeParam(2));
+
+	if(!msg->haltOutput())
+	{
+		KviWindow * pOut = KVI_OPTION_BOOL(KviOption_boolWhoisRepliesToActiveWindow) ?
+			msg->console()->activeWindow() : (KviWindow *)(msg->console());
+		pOut->output(
+			KVI_OUT_WHOISOTHER,__tr2qs("%c\r!n\r%Q\r%c is authenticated as %Q"),KVI_TEXT_BOLD,
+			&szNick,KVI_TEXT_BOLD,&szAuth);
+	}
+}
+
 void KviServerParser::parseNumericWhoisOther(KviIrcMessage *msg)
 {
 	// *: RPL_WHOIS* [?]
@@ -1143,7 +1313,7 @@ void KviServerParser::parseNumericWhoisOther(KviIrcMessage *msg)
 		return;
 	}
 
-// FIXME: #warning "NICK LINKS"
+	// FIXME: #warning "NICK LINKS"
 	if(!msg->haltOutput())
 	{
 		KviWindow * pOut = KVI_OPTION_BOOL(KviOption_boolWhoisRepliesToActiveWindow) ?
@@ -1166,20 +1336,22 @@ void KviServerParser::parseNumericEndOfWhois(KviIrcMessage *msg)
 	if(i)
 	{
 		if(!g_pApp->windowExists(i->pWindow))i->pWindow = msg->console();
-		KviParameterList *params = new KviParameterList();
-		params->append(new KviStr(i->szNick));
-		params->append(new KviStr(i->szUser));
-		params->append(new KviStr(i->szHost));
-		params->append(new KviStr(i->szReal));
-		params->append(new KviStr(i->szServer));
-		params->append(new KviStr(i->szIdle));
-		params->append(new KviStr(i->szSignon));
-		params->append(new KviStr(i->szChannels));
-		params->append(new KviStr(msg->safePrefix()));
-		params->append(new KviStr(i->szSpecial));
-		params->append(new KviStr(i->szMagic));
-		KviStr tmp = i->szCode;
-		g_pUserParser->parseCommandBuffer(tmp.ptr(),i->pWindow,params);
+		
+		// that's the new KVS engine!
+		KviKvsVariantList vl;
+		vl.setAutoDelete(true);
+		vl.append(new KviKvsVariant(i->szNick));
+		vl.append(new KviKvsVariant(i->szUser));
+		vl.append(new KviKvsVariant(i->szHost));
+		vl.append(new KviKvsVariant(i->szReal));
+		vl.append(new KviKvsVariant(i->szServer));
+		vl.append(new KviKvsVariant(i->szIdle));
+		vl.append(new KviKvsVariant(i->szSignon));
+		vl.append(new KviKvsVariant(i->szChannels));
+		vl.append(new KviKvsVariant(QString(msg->safePrefix())));
+		vl.append(new KviKvsVariant(i->szSpecial));
+		vl.append(new KviKvsVariant(*(i->pMagic)));
+		i->pCallback->run(i->pWindow,&vl,0,KviKvsScript::PreserveParams);
 		msg->connection()->asyncWhoisData()->remove(i);
 		return;
 	}
@@ -1227,10 +1399,21 @@ void KviServerParser::parseNumericNoSuchNick(KviIrcMessage *msg)
 		if(i)
 		{
 			if(!g_pApp->windowExists(i->pWindow))i->pWindow = msg->console();
-			KviParameterList *params = new KviParameterList();
-			params->append(new KviStr(i->szNick));
-			KviStr tmp = i->szCode;
-			g_pUserParser->parseCommandBuffer(tmp.ptr(),i->pWindow,params);
+			// that's the new KVS engine!
+			KviKvsVariantList vl;
+			vl.setAutoDelete(true);
+			vl.append(new KviKvsVariant(i->szNick));
+			vl.append(new KviKvsVariant());
+			vl.append(new KviKvsVariant());
+			vl.append(new KviKvsVariant());
+			vl.append(new KviKvsVariant());
+			vl.append(new KviKvsVariant());
+			vl.append(new KviKvsVariant());
+			vl.append(new KviKvsVariant());
+			vl.append(new KviKvsVariant());
+			vl.append(new KviKvsVariant());
+			vl.append(new KviKvsVariant(*(i->pMagic)));
+			i->pCallback->run(i->pWindow,&vl,0,KviKvsScript::PreserveParams);
 			msg->connection()->asyncWhoisData()->remove(i);
 			return;
 		}
@@ -1401,7 +1584,7 @@ void KviServerParser::parseNumericLinks(KviIrcMessage *msg)
 		// module loaded
 		msg->console()->ircContext()->linksWindow()->processData(msg);
 	} else {
-		// ops...can't load the module...
+		// ops...can't load the module... or the event halted the window creation
 		if(!msg->haltOutput())
 		{
 			QString szList = msg->connection()->decodeText(msg->allParams());
@@ -1437,9 +1620,10 @@ void KviServerParser::parseNumericBackFromAway(KviIrcMessage * msg)
 	msg->connection()->changeAwayState(false);
 
 	// trigger the event
-	KviStr tmp(KviStr::Format,"%u",bWasAway ? (unsigned int)(msg->connection()->userInfo()->awayTime()) : 0);
-	if(TRIGGER_EVENT_2PARAM_RETVALUE(KviEvent_OnMeBack,msg->console(),tmp.ptr(),szWText))msg->setHaltOutput();
-
+	QString tmp;
+	KviQString::sprintf(tmp,"%u",bWasAway ? (unsigned int)(msg->connection()->userInfo()->awayTime()) : 0);
+	if(KVS_TRIGGER_EVENT_2_HALTED(KviEvent_OnMeBack,msg->console(),tmp,szWText))
+		msg->setHaltOutput();
 	if(!msg->haltOutput())
 	{
 		KviWindow * pOut = KVI_OPTION_BOOL(KviOption_boolServerRepliesToActiveWindow) ?
@@ -1473,7 +1657,7 @@ void KviServerParser::parseNumericAway(KviIrcMessage * msg)
 	msg->connection()->changeAwayState(true);
 	QString szWText = msg->connection()->decodeText(msg->safeTrailing());
 	
-	if(TRIGGER_EVENT_1PARAM_RETVALUE(KviEvent_OnMeAway,msg->console(),szWText))msg->setHaltOutput();
+	if(KVS_TRIGGER_EVENT_1_HALTED(KviEvent_OnMeAway,msg->console(),szWText))msg->setHaltOutput();
 
 	if(!msg->haltOutput())
 	{
@@ -1491,7 +1675,10 @@ void KviServerParser::parseNumericAway(KviIrcMessage * msg)
 			if(nick.length() > 5)szNewNick = nick.left(5);
 			else szNewNick = nick;
 			szNewNick.append("AWAY");
-		} else szNewNick = KVI_OPTION_STRING(KviOption_stringCustomAwayNick);
+		} else {
+			szNewNick = KVI_OPTION_STRING(KviOption_stringCustomAwayNick);
+			szNewNick.replace("%nick%",nick);
+		}
 
 		if(_OUTPUT_PARANOIC)
 			msg->console()->output(KVI_OUT_AWAY,__tr2qs("Setting away nickname (%Q)"),&szNewNick);
@@ -1543,24 +1730,231 @@ void KviServerParser::parseNumericStats(KviIrcMessage * msg)
 				if(szParms.hasData())szParms.append(' ');
 				szParms.append(*p);
 			}
-			pOut->outputNoFmt(KVI_OUT_STATS,szParms.ptr());
+			pOut->outputNoFmt(KVI_OUT_STATS,msg->connection()->decodeText(szParms).utf8().data());
 		} else {
-			pOut->outputNoFmt(KVI_OUT_STATS,msg->safeTrailing());
+			pOut->outputNoFmt(KVI_OUT_STATS,msg->connection()->decodeText(msg->safeTrailing()).utf8().data());
 		}
 	}
 }
 
+void KviServerParser::parseNumericServerAdminInfoTitle(KviIrcMessage * msg)
+{
+//RPL_ADMINME          256
+	if(!msg->haltOutput())
+	{
+		KviWindow * pOut = (KviWindow *)(msg->console());
+			pOut->output(KVI_OUT_SERVERINFO,msg->connection()->decodeText(msg->safeTrailing()).utf8().data());
+	}
+}
+void KviServerParser::parseNumericServerAdminInfoServerName(KviIrcMessage * msg)
+{
+//RPL_ADMINLOC1        257
+	if(!msg->haltOutput())
+	{
+		KviWindow * pOut = (KviWindow *)(msg->console());
+			QString szInfo = msg->connection()->decodeText(msg->safeTrailing());
+			pOut->output(KVI_OUT_SERVERINFO,__tr2qs("%c\r!s\r%s\r%c's server info: %s"),KVI_TEXT_BOLD,msg->prefix(),KVI_TEXT_BOLD,szInfo.utf8().data());
+	}
+}
+
+void KviServerParser::parseNumericServerAdminInfoAdminName(KviIrcMessage * msg)
+{
+//RPL_ADMINLOC2        258
+	if(!msg->haltOutput())
+	{
+		KviWindow * pOut = (KviWindow *)(msg->console());
+		QString szInfo = msg->connection()->decodeText(msg->safeTrailing());
+		pOut->output(KVI_OUT_SERVERINFO,__tr2qs("%c\r!s\r%s\r%c's administrator is %s"),KVI_TEXT_BOLD,msg->prefix(),KVI_TEXT_BOLD,szInfo.utf8().data());
+	}
+}
+
+void KviServerParser::parseNumericServerAdminInfoAdminContact(KviIrcMessage * msg)
+{
+//RPL_ADMINEMAIL       259
+	if(!msg->haltOutput())
+	{
+		KviWindow * pOut = (KviWindow *)(msg->console());
+			QString szInfo = msg->connection()->decodeText(msg->safeTrailing());
+			pOut->output(KVI_OUT_SERVERINFO,__tr2qs("%c\r!s\r%s\r%c's contact adress is %s"),KVI_TEXT_BOLD,msg->prefix(),KVI_TEXT_BOLD,szInfo.utf8().data());
+	}
+}
+
+void KviServerParser::parseNumericCommandSyntax(KviIrcMessage * msg)
+{
+//RPL_COMMANDSYNTAX    334
+	if(!msg->haltOutput())
+	{
+		KviWindow * pOut = (KviWindow *)(msg->console());
+		pOut->outputNoFmt(KVI_OUT_STATS,msg->connection()->decodeText(msg->safeTrailing()));
+	}
+}
+
+void KviServerParser::parseNumericInviting(KviIrcMessage * msg)
+{
+//RPL_INVITING         341
+	if(!msg->haltOutput())
+	{
+		QString szWho = msg->connection()->decodeText(msg->safeParam(0));
+		QString szTarget = msg->connection()->decodeText(msg->safeParam(1));
+		QString szChan = msg->connection()->decodeText(msg->safeParam(2));
+		KviChannel * chan = msg->connection()->findChannel(szChan);
+		if(chan)
+		{
+			chan->output(KVI_OUT_INVITE,__tr2qs("\r!n\r%Q\r invited %Q into channel %Q"),&szWho,&szTarget,&szChan);
+		} else {
+			KviWindow * pOut = (KviWindow *)(msg->console());
+			pOut->output(KVI_OUT_INVITE,__tr2qs("\r!n\r%Q\r invited %Q into channel %Q"),&szWho,&szTarget,&szChan);
+		}
+	}
+}
+
+void KviServerParser::parseNumericInfo(KviIrcMessage * msg)
+{
+//RPL_INFO             371
+	if(!msg->haltOutput())
+	{
+		KviWindow * pOut = (KviWindow *)(msg->console());
+		QString szInfo = msg->connection()->decodeText(msg->safeTrailing());
+		pOut->outputNoFmt(KVI_OUT_SERVERINFO,szInfo);
+	}
+}
+
+void KviServerParser::parseNumericInfoStart(KviIrcMessage * msg)
+{
+//RPL_INFOSTART        373
+	if(!msg->haltOutput())
+	{
+		KviWindow * pOut = (KviWindow *)(msg->console());
+		pOut->output(KVI_OUT_SERVERINFO,__tr2qs("%c\r!s\r%s\r%c's information:"),KVI_TEXT_BOLD,msg->prefix(),KVI_TEXT_BOLD);
+	}
+}
+
+void KviServerParser::parseNumericInfoEnd(KviIrcMessage * msg)
+{
+//RPL_ENDOFINFO        374
+	if(!msg->haltOutput())
+	{
+		KviWindow * pOut = (KviWindow *)(msg->console());
+		pOut->output(KVI_OUT_SERVERINFO,__tr2qs("End of %c\r!s\r%s\r%c's information"),KVI_TEXT_BOLD,msg->prefix(),KVI_TEXT_BOLD);
+	}
+}
+
+void KviServerParser::parseNumericTime(KviIrcMessage * msg)
+{
+//RPL_TIME             391
+	if(!msg->haltOutput())
+	{
+		KviWindow * pOut = (KviWindow *)(msg->console());
+		QString szInfo = msg->connection()->decodeText(msg->safeTrailing());
+		pOut->output(KVI_OUT_SERVERINFO,__tr2qs("%c\r!s\r%s\r%c's time is %Q"),KVI_TEXT_BOLD,msg->prefix(),KVI_TEXT_BOLD,&szInfo);
+	}
+}
+
+void KviServerParser::parseNumericNoSuchServer(KviIrcMessage * msg)
+{
+//ERR_NOSUCHSERVER     402
+	if(!msg->haltOutput())
+	{
+		KviWindow * pOut = (KviWindow *)(msg->console());
+		QString szWhat = msg->connection()->decodeText(msg->safeParam(1));
+		pOut->output(KVI_OUT_GENERICERROR,__tr2qs("%Q: no such server"),&szWhat);
+	}
+}
+
+void KviServerParser::parseNumericNoSuchChannel(KviIrcMessage * msg)
+{
+//ERR_NOSUCHCHANNEL    403
+	if(!msg->haltOutput())
+	{
+		KviWindow * pOut = (KviWindow *)(msg->console());
+		QString szWhat = msg->connection()->decodeText(msg->safeParam(1));
+		pOut->output(KVI_OUT_GENERICERROR,__tr2qs("%Q: no such channel"),&szWhat);
+	}
+}
+
+void KviServerParser::parseNumericCannotSendColor(KviIrcMessage * msg)
+{
+//ERR_NOCOLORSONCHAN   408
+	if(!msg->haltOutput())
+	{
+		QString szChan = msg->connection()->decodeText(msg->safeParam(1));
+		QString szInfo = msg->connection()->decodeText(msg->safeTrailing());
+		KviChannel * chan = msg->connection()->findChannel(szChan);
+		if(chan)
+		{
+			chan->output(KVI_OUT_GENERICERROR,__tr2qs("Cannot sent to channel: %Q"),&szInfo);
+		} else {
+			KviWindow * pOut = (KviWindow *)(msg->console());
+			pOut->output(KVI_OUT_GENERICERROR,__tr2qs("Cannot sent text to channel %Q: %Q"),&szChan,&szInfo);
+		}
+	}
+}
+
+void KviServerParser::parseNumericCannotSend(KviIrcMessage * msg)
+{
+//ERR_CANNOTSENDTOCHAN 404
+	if(!msg->haltOutput())
+	{
+		QString szChan = msg->connection()->decodeText(msg->safeParam(1));
+		QString szInfo = msg->connection()->decodeText(msg->safeTrailing());
+		KviChannel * chan = msg->connection()->findChannel(szChan);
+		if(chan)
+		{
+			chan->output(KVI_OUT_GENERICERROR,__tr2qs("Cannot sent to channel"));
+		} else {
+			KviWindow * pOut = (KviWindow *)(msg->console());
+			pOut->output(KVI_OUT_GENERICERROR,__tr2qs("Cannot sent text to channel %Q"),&szChan);
+		}
+	}
+}
+
+
 void KviServerParser::parseNumericCodePageSet(KviIrcMessage *msg)
 {
 	// a nice extension for irc.wenet.ru
 	// 700: RPL_CODEPAGESET
 	// :prefix 700 target <encoding> :is now your translation scheme
+	
+	QString encoding = msg->connection()->decodeText(msg->safeParam(1));
 	if(msg->connection()->serverInfo()->supportsCodePages())
 	{
-		QString encoding = msg->connection()->decodeText(msg->safeParam(1));
-		// KVI_SMALLICON_TEXTENCODING ?
-		msg->console()->output(KVI_OUT_SYSTEMMESSAGE,__tr2qs("Your encoding is now %Q"),&encoding);
+		msg->console()->output(KVI_OUT_TEXTENCODING,__tr2qs("Your encoding is now %Q"),&encoding);
 		msg->connection()->setEncoding(encoding);
+	} else {
+		QString szMe = msg->connection()->decodeText(msg->safeParam(0));
+		if( (szMe==msg->connection()->currentNickName() || szMe == "*" ) //fix for pre-login codepage message
+			&& KviLocale::codecForName(encoding.utf8().data()))
+		{
+			msg->console()->output(KVI_OUT_TEXTENCODING,__tr2qs("Your encoding is now %Q"),&encoding);
+			msg->connection()->setEncoding(encoding);
+		} else if(!msg->haltOutput()) // simply unhandled
+		{
+			QString szWText = msg->connection()->decodeText(msg->allParams());
+			msg->connection()->console()->output(KVI_OUT_UNHANDLED,
+				"[%s][%s] %Q",msg->prefix(),msg->command(),&szWText);
+		}
+	}
+}
+
+void KviServerParser::parseNumericCodePageScheme(KviIrcMessage *msg)
+{
+	// a nice extension for irc.wenet.ru
+	// 703: RPL_WHOISSCHEME
+	// :prefix 703 <mynick> <nick> <encoding> :translation scheme
+	if(msg->connection()->serverInfo()->supportsCodePages())
+	{
+		QString szNick = msg->connection()->decodeText(msg->safeParam(1));
+		QString szCodepage = msg->connection()->decodeText(msg->safeParam(2));
+	
+		if(!msg->haltOutput())
+		{
+			KviWindow * pOut = KVI_OPTION_BOOL(KviOption_boolWhoisRepliesToActiveWindow) ?
+			msg->console()->activeWindow() : (KviWindow *)(msg->console());
+			QString szWText = pOut->decodeText(msg->safeTrailing());
+			pOut->output(
+				KVI_OUT_WHOISOTHER,__tr2qs("%c\r!n\r%Q\r%c's codepage is %Q: %Q"),KVI_TEXT_BOLD,
+				&szNick,KVI_TEXT_BOLD,&szCodepage,&szWText);
+		}
 	} else {
 		// simply unhandled
 		if(!msg->haltOutput())
@@ -1585,3 +1979,13 @@ void KviServerParser::parseNumericUserMode(KviIrcMessage *msg)
 		pOut->output(KVI_OUT_MODE,__tr2qs("Your user mode is %s"),msg->safeParam(1));
 	}
 }
+
+void KviServerParser::parseNumericEndOfStats(KviIrcMessage *msg)
+{
+	if(!msg->haltOutput())
+	{
+		KviWindow * pOut = (KviWindow *)(msg->console());
+		QString szText = msg->connection()->decodeText(msg->safeTrailing());
+		pOut->outputNoFmt(KVI_OUT_STATS, szText);
+	}
+}
\ No newline at end of file
