@@ -106,7 +106,7 @@ messageParseProc KviServerParser::m_numericParseProcTable[1000]=
 	0,                                               // 017
 	0,                                               // 018
 	0,                                               // 019
-	0,                                               // 020
+	PTM(parseNumeric020)                           , // 020
 	0,                                               // 021
 	0,                                               // 022
 	0,                                               // 023
@@ -305,7 +305,7 @@ messageParseProc KviServerParser::m_numericParseProcTable[1000]=
 	0,                                               // 216
 	0,                                               // 217
 	0,                                               // 218
-	0,                                               // 219
+	PTM(parseNumericEndOfStats),                     // 219 RPL_ENDOFSTATS
 	0,                                               // 220
 	PTM(parseNumericUserMode)                      , // 221 RPL_UMODEIS
 	0,                                               // 222
@@ -342,10 +342,10 @@ messageParseProc KviServerParser::m_numericParseProcTable[1000]=
 	PTM(parseNumericStats)                         , // 253 RPL_LUSERUNKNOWN
 	PTM(parseNumericStats)                         , // 254 RPL_LUSERCHANNELS
 	PTM(parseNumericStats)                         , // 255 RPL_LUSERME  
-	0,                                               // 256
-	0,                                               // 257
-	0,                                               // 258
-	0,                                               // 259
+	PTM(parseNumericServerAdminInfoTitle)          , // 256 RPL_ADMINME
+	PTM(parseNumericServerAdminInfoServerName)     , // 257 RPL_ADMINLOC1
+	PTM(parseNumericServerAdminInfoAdminName)      , // 258 RPL_ADMINLOC2
+	PTM(parseNumericServerAdminInfoAdminContact)   , // 259 RPL_ADMINEMAIL
 	0,                                               // 260
 	0,                                               // 261
 	0,                                               // 262
@@ -416,18 +416,18 @@ messageParseProc KviServerParser::m_numericParseProcTable[1000]=
 	0,                                               // 327
 	0,                                               // 328
 	PTM(parseNumericCreationTime)                  , // 329 RPL_CREATIONTIME
-	PTM(parseNumericWhoisOther)                    , // 330 RPL_WHOISAUTH
+	PTM(parseNumericWhoisAuth)                     , // 330 RPL_WHOISAUTH
 	PTM(parseNumericNoTopic)                       , // 331 RPL_NOTOPIC
 	PTM(parseNumericTopic)                         , // 332 RPL_TOPIC
 	PTM(parseNumericTopicWhoTime)                  , // 333 RPL_TOPICWHOTIME
-	0,                                               // 334
+	PTM(parseNumericCommandSyntax)                 , // 334 RPL_COMMANDSYNTAX
 	0,                                               // 335
 	0,                                               // 336
 	0,                                               // 337
-	0,                                               // 338
+	PTM(parseNumericWhoisOther),                                               // 338 RPL_WHOISACTUALLY
 	0,                                               // 339
 	0,                                               // 340
-	0,                                               // 341
+	PTM(parseNumericInviting)                      , // 351 RPL_INVITING
 	0,                                               // 342
 	0,                                               // 343
 	0,                                               // 344
@@ -457,10 +457,10 @@ messageParseProc KviServerParser::m_numericParseProcTable[1000]=
 	PTM(parseNumericEndOfBanList)                  , // 368 RPL_ENDOFBANLIST
 	PTM(parseNumericEndOfWhowas)                   , // 369 RPL_ENDOFWHOWAS
 	0,                                               // 370
-	0,                                               // 371
+	PTM(parseNumericInfo)                          , // 371 RPL_INFO
 	PTM(parseNumericMotd)                          , // 372 RPL_MOTD
-	0,                                               // 373
-	0,                                               // 374
+	PTM(parseNumericInfoStart)                     , // 373 RPL_INFOSTART
+	PTM(parseNumericInfoEnd)                       , // 374 RPL_ENDOFINFO
 	PTM(parseNumericMotd)                          , // 375 RPL_MOTDSTART
 	PTM(parseNumericMotd)                          , // 376 RPL_ENDOFMOTD
 	PTM(parseNumericMotd)                          , // 377 RPL_MOTD2
@@ -477,7 +477,7 @@ messageParseProc KviServerParser::m_numericParseProcTable[1000]=
 	0,                                               // 388
 	0,                                               // 389
 	0,                                               // 390
-	0,                                               // 391
+	PTM(parseNumericTime)                          , // 391 RPL_TIME
 	0,                                               // 392
 	0,                                               // 393
 	0,                                               // 394
@@ -488,13 +488,13 @@ messageParseProc KviServerParser::m_numericParseProcTable[1000]=
 	0,                                               // 399
 	0,                                               // 400
 	PTM(parseNumericNoSuchNick)                    , // 401 ERR_NOSUCHNICK
-	0,                                               // 402
-	0,                                               // 403
-	0,                                               // 404
+	PTM(parseNumericNoSuchServer)                  , // 402 ERR_NOSUCHSERVER
+	PTM(parseNumericNoSuchChannel)                 , // 403 ERR_NOSUCHCHANNEL
+	PTM(parseNumericCannotSend)                    , // 404 ERR_CANNOTSENDTOCHAN
 	0,                                               // 405
 	PTM(parseNumericNoSuchNick)                    , // 406 ERR_WASNOSUCHNICK
 	0,                                               // 407
-	0,                                               // 408
+	PTM(parseNumericCannotSendColor)               , // 408 ERR_NOCOLORSONCHAN
 	0,                                               // 409
 	0,                                               // 410
 	0,                                               // 411
@@ -553,12 +553,12 @@ messageParseProc KviServerParser::m_numericParseProcTable[1000]=
 	0,                                               // 464
 	0,                                               // 465
 	0,                                               // 466
-	0,                                               // 467
+	PTM(otherChannelError),                          // 467 ERR_KEYSET
 	0,                                               // 468
 	0,                                               // 469
 	0,                                               // 470
 	PTM(parseNumericCantJoinChannel)               , // 471 ERR_CHANNELISFULL
-	0,                                               // 472
+	PTM(otherChannelError),                          // 472 ERR_UNKNOWNMODE
 	PTM(parseNumericCantJoinChannel)               , // 473 ERR_INVITEONLYCHAN
 	PTM(parseNumericCantJoinChannel)               , // 474 ERR_BANNEDFROMCHAN
 	PTM(parseNumericCantJoinChannel)               , // 475 ERR_BADCHANNELKEY
@@ -568,7 +568,7 @@ messageParseProc KviServerParser::m_numericParseProcTable[1000]=
 	0,                                               // 479
 	0,                                               // 480
 	0,                                               // 481
-	0,                                               // 482
+	PTM(otherChannelError),                          // 482 ERR_CHANOPRIVSNEEDED
 	0,                                               // 483
 	0,                                               // 484
 	0,                                               // 485
@@ -789,10 +789,10 @@ messageParseProc KviServerParser::m_numericParseProcTable[1000]=
 	PTM(parseNumericCodePageSet)                   , // 700 RPL_CODEPAGESET
 	0,                                               // 701
 	0,                                               // 702
-	0,                                               // 703
-	0,                                               // 704
-	0,                                               // 705
-	0,                                               // 706
+	PTM(parseNumericCodePageScheme)                   , // 703 RPL_WHOISSCHEME
+	PTM(commandSyntaxHelp),                          // 704 RPL_COMMANDSYNTAX
+	PTM(commandHelp),                                // 705 RPL_COMMANDHELP
+	PTM(commandEndOfHelp),                           // 706 RPL_ENDOFCOMMANDHELP
 	0,                                               // 707
 	0,                                               // 708
 	0,                                               // 709
