@@ -28,6 +28,8 @@
 
 #define __KVIRC__
 
+#include "kvi_toolwindows_container.h"
+#include "kvi_styled_controls.h"
 #include "kvi_channel.h"
 #include "kvi_console.h"
 #include "kvi_iconmanager.h"
@@ -44,8 +46,6 @@
 #include "kvi_config.h"
 #include "kvi_themedlabel.h"
 #include "kvi_maskeditor.h"
-#include "kvi_event.h"
-#include "kvi_uparser.h"
 #include "kvi_mirccntrl.h"
 #include "kvi_settings.h"
 #include "kvi_parameterlist.h"
@@ -55,13 +55,19 @@
 #include "kvi_ircconnection.h"
 #include "kvi_ircconnectionserverinfo.h"
 #include "kvi_defaults.h"
-#include "kvi_uparser.h"
+//#include "kvi_uparser.h"
+#include "kvi_sparser.h"
+#include "kvi_modew.h"
+#include "kvi_mirccntrl.h"
 
 #ifdef COMPILE_CRYPT_SUPPORT
 	#include "kvi_crypt.h"
 	#include "kvi_cryptcontroller.h"
 #endif
 
+#include "kvi_kvs_script.h"
+#include "kvi_kvs_eventtriggers.h"
+
 #include <time.h>
 
 #include <qsplitter.h>
@@ -73,6 +79,7 @@
 #include <qpopupmenu.h>
 #include <qdict.h>
 #include <qmessagebox.h>
+#include <qwidgetstack.h> 
 
 #ifndef AVERAGE_CHANNEL_USERS
 	#define AVERAGE_CHANNEL_USERS 101
@@ -109,48 +116,23 @@ KviChannel::KviChannel(KviFrame * lpFrm,KviConsole * lpConsole,const char * name
 	// And create the widgets layout
 	// Button box
 	m_pButtonBox = new QHBox(this,"button_box");
-
+	
 	m_pTopSplitter = new QSplitter(QSplitter::Horizontal,m_pButtonBox,"top_splitter");
 
 	m_pButtonBox->setStretchFactor(m_pTopSplitter,1);
-
+	
+	m_pButtonContainer = new QHBox(m_pButtonBox);
+	
 	// Topic widget on the left
 	m_pTopicWidget = new KviTopicWidget(m_pTopSplitter,"topic_widget");
 
 	connect(m_pTopicWidget,SIGNAL(topicSelected(const QString &)),
 		this,SLOT(topicSelected(const QString &)));
 	// mode label follows the topic widget
-	m_pModeLabel = new KviThemedLabel(m_pTopSplitter,"mode_label");
-	QToolTip::add(m_pModeLabel,__tr2qs("Channel mode"));
+	m_pModeWidget = new KviModeWidget(m_pTopSplitter,this,"mode_");
+	QToolTip::add(m_pModeWidget,__tr2qs("Channel mode"));
 
-	// and the related buttons
-	m_pDoubleViewButton = createToolButton(m_pButtonBox,"double_view_button",KVI_SMALLICON_HIDEDOUBLEVIEW,KVI_SMALLICON_SHOWDOUBLEVIEW,__tr2qs("Split View"),false);
-	connect(m_pDoubleViewButton,SIGNAL(clicked()),this,SLOT(toggleDoubleView()));
-	m_pListViewButton = createToolButton(m_pButtonBox,"list_view_button",KVI_SMALLICON_HIDELISTVIEW,KVI_SMALLICON_SHOWLISTVIEW,__tr2qs("Show User List"),true);
-	connect(m_pListViewButton,SIGNAL(clicked()),this,SLOT(toggleListView()));
-	m_pBanEditorButton = createToolButton(m_pButtonBox,"ban_editor_button",KVI_SMALLICON_UNBAN,KVI_SMALLICON_BAN,__tr2qs("Show Ban Editor"),false);
-	connect(m_pBanEditorButton,SIGNAL(clicked()),this,SLOT(toggleBanEditor()));
-
-	if(m_pConsole->connection()->serverInfo()->supportsModesIe())
-	{
-		m_pBanExceptionEditorButton = createToolButton(m_pButtonBox,"ban_exception_editor_button",KVI_SMALLICON_BANUNEXCEPT,KVI_SMALLICON_BANEXCEPT,__tr2qs("Show Ban Exception Editor"),false);
-		connect(m_pBanExceptionEditorButton,SIGNAL(clicked()),this,SLOT(toggleBanExceptionEditor()));
-		m_pInviteEditorButton = createToolButton(m_pButtonBox,"invite_exception_editor_button",KVI_SMALLICON_INVITEUNEXCEPT,KVI_SMALLICON_INVITEEXCEPT,__tr2qs("Show Invite Exception Editor"),false);
-		connect(m_pInviteEditorButton,SIGNAL(clicked()),this,SLOT(toggleInviteEditor()));
-	} else {
-		m_pBanExceptionEditorButton = 0;
-		m_pInviteEditorButton = 0;
-	}
-
-	m_pModeEditorButton = createToolButton(m_pButtonBox,"mode_editor_button",KVI_SMALLICON_CHANMODEHIDE,KVI_SMALLICON_CHANMODE,__tr2qs("Show Mode Editor"),false);
-	connect(m_pModeEditorButton,SIGNAL(clicked()),this,SLOT(toggleModeEditor()));
-	m_pModeEditor = 0;
-
-#ifdef COMPILE_CRYPT_SUPPORT
-	createCryptControllerButton(m_pButtonBox);
-#endif
-
-	createTextEncodingButton(m_pButtonBox);
+	createTextEncodingButton(m_pButtonContainer);
 
 	// Central splitter
 	m_pSplitter = new QSplitter(QSplitter::Horizontal,this,"main_splitter");
@@ -163,8 +145,54 @@ KviChannel::KviChannel(KviFrame * lpFrm,KviConsole * lpConsole,const char * name
 	// And the double view (that may be unused)
 	m_pMessageView = 0;
 	// The userlist on the right
-	m_pUserListView = new KviUserListView(m_pSplitter,connection()->userDataBase(),this,
+	//m_pEditorsContainer= new KviToolWindowsContainer(m_pSplitter);
+	
+	
+	// and the related buttons
+	m_pDoubleViewButton = createToolButton(m_pButtonContainer,"double_view_button",KVI_SMALLICON_HIDEDOUBLEVIEW,KVI_SMALLICON_SHOWDOUBLEVIEW,__tr2qs("Split View"),false);
+	connect(m_pDoubleViewButton,SIGNAL(clicked()),this,SLOT(toggleDoubleView()));
+	
+	m_pListViewButton = new KviWindowToolPageButton(KVI_SMALLICON_HIDELISTVIEW,KVI_SMALLICON_SHOWLISTVIEW,__tr2qs("User List"),buttonContainer(),true,"list_view_button");
+	connect(m_pListViewButton,SIGNAL(clicked()),this,SLOT(toggleListView()));
+	m_pBanEditorButton = new KviWindowToolPageButton(KVI_SMALLICON_UNBAN,KVI_SMALLICON_BAN,__tr2qs("Ban Editor"),buttonContainer(),false,"ban_editor_button");
+	connect(m_pBanEditorButton,SIGNAL(clicked()),this,SLOT(toggleBanEditor()));
+	
+	if(m_pConsole->connection()->serverInfo()->supportedListModes().contains('e'))
+	{
+		m_pBanExceptionEditorButton =new KviWindowToolPageButton(KVI_SMALLICON_BANUNEXCEPT,KVI_SMALLICON_BANEXCEPT,__tr2qs("Ban Exception Editor"),buttonContainer(),false,"ban_exception_editor_button");
+		connect(m_pBanExceptionEditorButton,SIGNAL(clicked()),this,SLOT(toggleBanExceptionEditor()));
+	} else {
+		m_pBanExceptionEditorButton=0;
+	}
+	if(m_pConsole->connection()->serverInfo()->supportedListModes().contains('I'))
+	{
+		m_pInviteEditorButton =new KviWindowToolPageButton(KVI_SMALLICON_INVITEUNEXCEPT,KVI_SMALLICON_INVITEEXCEPT,__tr2qs("Invite Exception Editor"),buttonContainer(),false,"invite_exception_editor_button");
+		connect(m_pInviteEditorButton,SIGNAL(clicked()),this,SLOT(toggleInviteEditor()));
+	} else {
+		m_pInviteEditorButton = 0;
+	}
+	m_pModeEditorButton = new KviWindowToolPageButton(KVI_SMALLICON_CHANMODEHIDE,KVI_SMALLICON_CHANMODE,__tr2qs("Mode Editor"),buttonContainer(),false,"mode_editor_button");
+	connect(m_pModeEditorButton,SIGNAL(clicked()),this,SLOT(toggleModeEditor()));
+	m_pModeEditor = 0;
+	
+#ifdef COMPILE_CRYPT_SUPPORT
+	createCryptControllerButton(m_pButtonContainer);
+#endif
+
+	m_pHideToolsButton = new KviStyledToolButton(m_pButtonBox,"hide_container_button");
+	
+	m_pHideToolsButton->setUsesBigPixmap(false);
+	m_pHideToolsButton->setFixedWidth(10);
+
+	if(g_pIconManager->getBigIcon("kvi_horizontal_left.png"))
+		m_pHideToolsButton->setPixmap(*(g_pIconManager->getBigIcon("kvi_horizontal_left.png")));
+	
+	connect(m_pHideToolsButton,SIGNAL(clicked()),this,SLOT(toggleToolButtons()));
+	
+	m_pUserListView = new KviUserListView(m_pSplitter,m_pListViewButton,connection()->userDataBase(),this,
 								AVERAGE_CHANNEL_USERS,__tr2qs("User List"),"user_list_view");
+//	m_pEditorsContainer->addWidget(m_pUserListView);
+//	m_pEditorsContainer->raiseWidget(m_pUserListView);
 	// And finally the input line on the bottom
 	m_pInput   = new KviInput(this,m_pUserListView);
 	// no mask editors yet
@@ -179,6 +207,8 @@ KviChannel::KviChannel(KviFrame * lpFrm,KviConsole * lpConsole,const char * name
 
 	applyOptions();
 	m_joinTime = QDateTime::currentDateTime();
+	m_tLastReceivedWhoReply = (kvi_time_t)m_joinTime.toTime_t();
+	
 }
 
 KviChannel::~KviChannel()
@@ -193,26 +223,51 @@ KviChannel::~KviChannel()
 	delete m_pBanList;
 	delete m_pBanExceptionList;
 	delete m_pInviteList;
-    delete m_pTmpHighLighted;
+	delete m_pTmpHighLighted;
 }
 
+void KviChannel::toggleToolButtons()
+{
+	if(!buttonContainer()) return;
+	toggleButtonContainer();
+	QPixmap* pix= buttonContainer()->isVisible() ? 
+		g_pIconManager->getBigIcon("kvi_horizontal_left.png") : 
+		g_pIconManager->getBigIcon("kvi_horizontal_right.png");
+	if(pix)
+		m_pHideToolsButton->setPixmap(*pix);
+}
 
 void KviChannel::triggerCreationEvents()
 {
-	TRIGGER_EVENT(KviEvent_OnChannelWindowCreated,this);
+	KVS_TRIGGER_EVENT_0(KviEvent_OnChannelWindowCreated,this);
 }
 
 void KviChannel::textViewRightClicked()
 {
-	TRIGGER_EVENT(KviEvent_OnChannelPopupRequest,this);
+	KVS_TRIGGER_EVENT_0(KviEvent_OnChannelPopupRequest,this);
 }
 
 void KviChannel::getBaseLogFileName(KviStr &buffer)
 {
 	if (console()->connection())
-		buffer.sprintf("%s.%s",windowName().latin1(),console()->currentNetworkName());
+		buffer.sprintf("%s.%s",windowName().utf8().data(),console()->currentNetworkName().utf8().data());
 	else
-		buffer.sprintf("%s.%u",windowName().latin1(),console()->ircContextId());
+		buffer.sprintf("%s.%u",windowName().utf8().data(),console()->ircContextId());
+}
+void KviChannel::getBaseLogFileName(QString &buffer)
+{
+	QString szChan=windowName();
+	szChan.replace(".","%2e");
+	if (console()->connection())
+	{
+		buffer=windowName();
+		buffer+=".";
+		buffer+=console()->currentNetworkName();
+	} else {
+		buffer=szChan;
+		buffer+=".";
+		buffer+=console()->ircContextId();
+	}
 }
 
 void KviChannel::applyOptions()
@@ -222,13 +277,13 @@ void KviChannel::applyOptions()
 
 	if(m_pMessageView)m_pMessageView->applyOptions();
 
-	m_pModeLabel->applyOptions();
+	m_pModeWidget->applyOptions();
 
 	// this applies options for IrcView and Input and forces the window to relayout
 	KviWindow::applyOptions();
 }
 
-void KviChannel::getConfigGroupName(KviStr &buf)
+void KviChannel::getConfigGroupName(QString &buf)
 {
 	buf = windowName();
 }
@@ -238,9 +293,12 @@ void KviChannel::saveProperties(KviConfig *cfg)
 	KviWindow::saveProperties(cfg);
 	cfg->writeEntry("TopSplitter",m_pTopSplitter->sizes());
 	cfg->writeEntry("Splitter",m_pSplitter->sizes());
-	cfg->writeEntry("VertSplitter",m_pVertSplitter->sizes());
+	cfg->writeEntry("VertSplitter",m_pMessageView ? m_pVertSplitter->sizes() : m_VertSplitterSizesList);
 	cfg->writeEntry("PrivateBackground",m_privateBackground);
 	cfg->writeEntry("DoubleView",m_pMessageView ? true : false);
+	if(m_pUserListView)
+		cfg->writeEntry("UserListHidden",m_pUserListView->isHidden());
+	cfg->writeEntry("ToolButtonsHidden",buttonContainer()->isHidden());
 }
 
 void KviChannel::loadProperties(KviConfig *cfg)
@@ -256,11 +314,15 @@ void KviChannel::loadProperties(KviConfig *cfg)
 	def.append((w * 18) / 100);
 	m_pSplitter->setSizes(cfg->readIntListEntry("Splitter",def));
 	//debug("SETTING DEFAULT SIZES");
-	showDoubleView(cfg->readBoolEntry("DoubleView",false));
 	def.clear();
+	
+	def.append((w * 60) / 100);
+	def.append((w * 40) / 100);
+	m_VertSplitterSizesList=cfg->readIntListEntry("VertSplitter",def);
+	showDoubleView(cfg->readBoolEntry("DoubleView",false));
 	//def.append((w * 50) / 100);
 	//def.append((w * 50) / 100);
-	m_pVertSplitter->setSizes(cfg->readIntListEntry("VertSplitter",def));
+
 	m_privateBackground = cfg->readPixmapEntry("PrivateBackground",KviPixmap());
 	if(m_privateBackground.pixmap())
 	{
@@ -269,6 +331,14 @@ void KviChannel::loadProperties(KviConfig *cfg)
 	}
 
 	KviWindow::loadProperties(cfg);
+	if(m_pUserListView)
+	{
+		bool bHidden=cfg->readBoolEntry("UserListHidden",0);
+		m_pUserListView->setHidden(bHidden);
+		resizeEvent(0);
+	}
+	if(cfg->readBoolEntry("ToolButtonsHidden",KVI_OPTION_BOOL(KviOption_boolHideWindowToolButtons))!=buttonContainer()->isHidden())
+		toggleToolButtons();
 }
 
 
@@ -278,12 +348,14 @@ void KviChannel::showDoubleView(bool bShow)
 	{
 		if(bShow)return;
 		m_pIrcView->joinMessagesFrom(m_pMessageView);
+		m_VertSplitterSizesList=m_pVertSplitter->sizes();
 		delete m_pMessageView;
 		m_pMessageView = 0;
 		if(m_pDoubleViewButton->isOn())m_pDoubleViewButton->setOn(false);
 	} else {
 		if(!bShow)return;
 		m_pMessageView = new KviIrcView(m_pVertSplitter,m_pFrm,this);
+		m_pVertSplitter->setSizes(m_VertSplitterSizesList);
 		//setFocusHandler(m_pInput,m_pMessageView); //socket it!
 		if(!(m_pDoubleViewButton->isOn()))m_pDoubleViewButton->setOn(true);
 		if(m_privateBackground.pixmap())
@@ -321,11 +393,11 @@ void KviChannel::toggleModeEditor()
 	{
 		delete m_pModeEditor;
 		m_pModeEditor = 0;
-		if(m_pModeEditorButton->isOn())m_pModeEditorButton->setOn(false);
 		m_pSplitter->setMinimumHeight(20); //gfgf
+		if(m_pModeEditorButton->isOn()) m_pModeEditorButton->setOn(false);
 		resizeEvent(0);
 	} else {
-		m_pModeEditor = new KviModeEditor(m_pSplitter,"mode_editor",m_szChannelMode.ptr(),m_szChannelKey.ptr(),m_szChannelLimit.ptr());
+		m_pModeEditor = new KviModeEditor(m_pSplitter,m_pModeEditorButton,"mode_editor",console(),m_szChannelMode,m_szChannelKey.ptr(),m_szChannelLimit.ptr());
 		connect(m_pModeEditor,SIGNAL(setMode(const char *)),this,SLOT(setMode(const char *)));
 		connect(m_pModeEditor,SIGNAL(done()),this,SLOT(modeSelectorDone()));
 		m_pModeEditor->show();
@@ -364,14 +436,14 @@ void KviChannel::toggleInviteEditor()
 			m_pInviteList,'I',"invite_exception_editor");
 }
 
-void KviChannel::toggleEditor(KviMaskEditor ** ppEd,QToolButton ** ppBtn,KviPtrList<KviMaskEntry> *l,char flag,const char *edName)
+void KviChannel::toggleEditor(KviMaskEditor ** ppEd,KviWindowToolPageButton ** ppBtn,KviPtrList<KviMaskEntry> *l,char flag,const char *edName)
 {
 	if(*ppEd)
 	{
 		delete *ppEd;
 		*ppEd = 0;
 		if(!(*ppBtn))return;
-		if((*ppBtn)->isOn())(*ppBtn)->setOn(false);
+		if((*ppBtn)->isOn()) (*ppBtn)->setOn(false);
 	} else {
 		bool bHasList = true;
 		switch(flag)
@@ -407,7 +479,7 @@ void KviChannel::toggleEditor(KviMaskEditor ** ppEd,QToolButton ** ppBtn,KviPtrL
 			}
 		}
 
-		*ppEd = new KviMaskEditor(m_pSplitter,l,flag,edName);
+		*ppEd = new KviMaskEditor(m_pSplitter,*ppBtn,l,flag,edName);
 		connect(*ppEd,SIGNAL(removeMasks(KviMaskEditor *,KviPtrList<KviMaskEntry> *)),
 			this,SLOT(removeMasks(KviMaskEditor *,KviPtrList<KviMaskEntry> *)));
 		//setFocusHandler(m_pInput,*ppEd); //socket it!
@@ -428,12 +500,12 @@ void KviChannel::removeMasks(KviMaskEditor *ed,KviPtrList<KviMaskEntry> *l)
 		masks.append(e->szMask);
 		flags.append(ed->flag());
 		count++;
-		if(count == KVI_OPTION_UINT(KviOption_uintModeChangesPerMessage))
+		if(count == connection()->serverInfo()->maxModeChanges())
 		{
 			if(connection())
 			{
 				QCString szName = connection()->encodeText(m_szName);
-				connection()->sendFmtData("MODE %s -%s %s",szName.data(),flags.ptr(),masks.ptr());
+				connection()->sendFmtData("MODE %s -%s %s",szName.data(),flags.ptr(),connection()->encodeText(QString(masks)).data());
 			}
 			flags = "";
 			masks = "";
@@ -445,7 +517,7 @@ void KviChannel::removeMasks(KviMaskEditor *ed,KviPtrList<KviMaskEntry> *l)
 		if(connection())
 		{
 			QCString szName = connection()->encodeText(m_szName);
-			connection()->sendFmtData("MODE %s -%s %s",szName.data(),flags.ptr(),masks.ptr());
+			connection()->sendFmtData("MODE %s -%s %s",szName.data(),flags.ptr(),connection()->encodeText(QString(masks)).data());
 		}
 	}
 }
@@ -473,18 +545,19 @@ QSize KviChannel::sizeHint() const
 
 void KviChannel::setChannelMode(char  mode,bool bAdd)
 {
-	if(bAdd)
-	{
-		if(!(m_szChannelMode.contains(mode)))m_szChannelMode.append(mode);
-	} else {
-		if(m_szChannelMode.contains(mode))
+	if(!m_pConsole->connection()->serverInfo()->supportedListModes().contains(mode)){
+		if(bAdd)
 		{
-			KviStr tmp = mode;
-			m_szChannelMode.replaceAll(tmp.ptr(),"");
+			if(!(m_szChannelMode.contains(mode)))m_szChannelMode.append(mode);
+		} else {
+			if(m_szChannelMode.contains(mode))
+			{
+				m_szChannelMode.replace(mode,"");
+			}
 		}
+		updateModeLabel();
+		updateCaption();
 	}
-	updateModeLabel();
-	updateCaption();
 }
 
 void KviChannel::setChannelKey(const char * key)
@@ -519,7 +592,7 @@ void KviChannel::removeHighlightedUser(const char * nick)
 
 void KviChannel::getChannelModeString(QString &buffer)
 {
-	buffer = m_szChannelMode.ptr();
+	buffer = m_szChannelMode;
 	if(m_szChannelKey.hasData())buffer.append('k');
 	if(m_szChannelLimit.hasData())buffer.append('l');
 }
@@ -527,7 +600,7 @@ void KviChannel::getChannelModeString(QString &buffer)
 void KviChannel::setDeadChan()
 {
 	m_iStateFlags |= KVI_CHANNEL_STATE_DEADCHAN;
-	m_iStateFlags &= ~KVI_CHANNEL_STATE_NOCLOSEONPART;
+	m_iStateFlags &= ~(KVI_CHANNEL_STATE_NOCLOSEONPART | KVI_CHANNEL_STATE_SENTSYNCWHOREQUEST);
 
 	m_pUserListView->enableUpdates(false);
 	m_pUserListView->partAll();
@@ -976,13 +1049,13 @@ void KviChannel::ownMessage(const QString &buffer)
 				switch(cryptSessionInfo()->pEngine->encrypt(d,encrypted))
 				{
 					case KviCryptEngine::Encrypted:
-						connection()->sendFmtData("PRIVMSG %s :%s",szName.data(),encrypted.ptr());
+						if(!connection()->sendFmtData("PRIVMSG %s :%s",szName.data(),encrypted.ptr()))return;
 						m_pConsole->outputPrivmsg(this,KVI_OUT_OWNPRIVMSGCRYPTED,
-							QString::null,QString::null,QString::null,buffer,KviConsole::NoNotifications);
+								QString::null,QString::null,QString::null,buffer,KviConsole::NoNotifications);
 					break;
 					case KviCryptEngine::Encoded:
 					{
-						connection()->sendFmtData("PRIVMSG %s :%s",szName.data(),encrypted.ptr());
+						if(!connection()->sendFmtData("PRIVMSG %s :%s",szName.data(),encrypted.ptr()))return;
 						// ugly ,but we must redecode here
 						QString szRedecoded = decodeText(encrypted.ptr());
 						m_pConsole->outputPrivmsg(this,KVI_OUT_OWNPRIVMSG,
@@ -1000,7 +1073,7 @@ void KviChannel::ownMessage(const QString &buffer)
 			} else {
 				d++; //eat the escape code
 				QString tmp = buffer.right(buffer.length() - 1);
-				connection()->sendFmtData("PRIVMSG %s :%s",szName.data(),d);
+				if(!connection()->sendFmtData("PRIVMSG %s :%s",szName.data(),d))return;
 				m_pConsole->outputPrivmsg(this,KVI_OUT_OWNPRIVMSG,QString::null,QString::null,QString::null,tmp,KviConsole::NoNotifications);
 				userAction(connection()->currentNickName(),KVI_USERACTION_PRIVMSG);
 				return;
@@ -1009,9 +1082,11 @@ void KviChannel::ownMessage(const QString &buffer)
 	}
 #endif
 
-	connection()->sendFmtData("PRIVMSG %s :%s",szName.data(),d);
-	m_pConsole->outputPrivmsg(this,KVI_OUT_OWNPRIVMSG,QString::null,QString::null,QString::null,buffer,KviConsole::NoNotifications);
-	userAction(connection()->currentNickName(),KVI_USERACTION_PRIVMSG);
+	if(connection()->sendFmtData("PRIVMSG %s :%s",szName.data(),d))
+	{
+		m_pConsole->outputPrivmsg(this,KVI_OUT_OWNPRIVMSG,QString::null,QString::null,QString::null,buffer,KviConsole::NoNotifications);
+		userAction(connection()->currentNickName(),KVI_USERACTION_PRIVMSG);
+	}
 }
 
 void KviChannel::ownAction(const QString &buffer)
@@ -1021,8 +1096,8 @@ void KviChannel::ownAction(const QString &buffer)
 	QCString szData = encodeText(buffer);
 	const char * d = szData.data();
 	if(!d)return;
-	connection()->sendFmtData("PRIVMSG %s :%cACTION %s%c",szName.data(),0x01,d,0x01);
-	QString szBuffer = "\r!n\r";
+	if(!connection()->sendFmtData("PRIVMSG %s :%cACTION %s%c",szName.data(),0x01,d,0x01))return;
+	QString szBuffer = "\r!nc\r";
 	szBuffer += connection()->currentNickName();
 	szBuffer += "\r ";
 	szBuffer += buffer;
@@ -1148,7 +1223,7 @@ void KviChannel::lostUserFocus()
 {
 	KviWindow::lostUserFocus();
 	if(!m_pMessageView)return;
-	if(m_pMessageView->hasLineMark())m_pMessageView->clearLineMark();
+	if(m_pMessageView->hasLineMark())m_pMessageView->clearLineMark(true);
 }
 
 
@@ -1282,12 +1357,14 @@ void KviChannel::closeEvent(QCloseEvent *e)
 		// FIXME: #warning "THIS PART SHOULD BECOME A COMMAND /PART $option()..so the identifiers are parsed"
 		if(connection())
 		{
-			KviStr tmp = KVI_OPTION_STRING(KviOption_stringPartMessage);
-			tmp.replaceAll(";","\\;");
-			tmp.replaceAll("\n"," ");
-			KviStr buffer;
-			if(!g_pUserParser->substituteIdentifiers(tmp.ptr(),this,buffer))buffer = tmp;
-			QCString dat = encodeText(QString::fromUtf8(buffer.ptr()));
+			QString tmp = KVI_OPTION_STRING(KviOption_stringPartMessage);
+			tmp.replace(";","\\;");
+			tmp.replace("\n"," ");
+			KviKvsVariant vRet;
+			
+			if(KviKvsScript::evaluate(tmp,this,0,&vRet))vRet.asString(tmp);
+
+			QCString dat = encodeText(tmp);
 			partMessageSent();
 			QCString szName = connection()->encodeText(m_szName);
 			connection()->sendFmtData("PART %s :%s",szName.data(),dat.data() ? dat.data() : "");
@@ -1326,7 +1403,7 @@ int KviChannel::myFlags()
 }
 
 
-void KviChannel::setMask(char flag, const QString &mask,bool bAdd,const QString &setBy,const QString &setAt)
+void KviChannel::setMask(char flag, const QString &mask,bool bAdd,const QString &setBy,unsigned int setAt)
 {
 	if(!connection())return;
 	KviPtrList<KviMaskEntry> * list = m_pBanList;
@@ -1352,19 +1429,19 @@ void KviChannel::setMask(char flag, const QString &mask,bool bAdd,const QString
 	m_pUserListView->setMaskEntries(flag,(int)list->count());
 }
 
-void KviChannel::internalMask(const QString &mask,bool bAdd,const QString &setBy,const QString &setAt,KviPtrList<KviMaskEntry> *l,KviMaskEditor **ppEd)
+void KviChannel::internalMask(const QString &mask,bool bAdd,const QString &setBy,unsigned int setAt,KviPtrList<KviMaskEntry> *l,KviMaskEditor **ppEd)
 {
 	KviMaskEntry * e = 0;
 	if(bAdd)
 	{
 		for(e = l->first();e;e = l->next())
 		{
-			if(kvi_strEqualCI(e->szMask,mask))return; //already there
+			if(KviQString::equalCI(e->szMask,mask))return; //already there
 		}
 		e = new KviMaskEntry;
 		e->szMask = mask;
 		e->szSetBy = setBy ? setBy : __tr2qs("(Unknown)");
-		e->szSetAt = setAt ? setAt : __tr2qs("(Unknown)");
+		e->uSetAt = setAt;
 		l->append(e);
 		if(*ppEd)(*ppEd)->addMask(e);
 	} else {
@@ -1382,9 +1459,11 @@ void KviChannel::internalMask(const QString &mask,bool bAdd,const QString &setBy
 
 void KviChannel::updateModeLabel()
 {
-	QString tmp = m_szChannelMode.ptr();
+	QString tmp = m_szChannelMode;
 	QString tip = __tr2qs("<b>Channel mode:</b>");
-	const char * aux = m_szChannelMode.ptr();
+	//const char * aux = m_szChannelMode.utf8().data(); leaks memory and will not work with getChannelModeDescription() (can channel modes be multibyte ?)
+	KviStr mod = m_szChannelMode;
+	const char * aux = mod.ptr();
 	while(*aux)
 	{
 		KviQString::appendFormatted(tip,"<br>%c: %Q",*aux,&(getChannelModeDescription(*aux)));
@@ -1405,9 +1484,9 @@ void KviChannel::updateModeLabel()
 		KviQString::appendFormatted(tip,__tr2qs("<br><b>Limit:</b> %s"),m_szChannelLimit.ptr());
 	}
 
-	m_pModeLabel->setText(tmp);
-	QToolTip::remove(m_pModeLabel);
-	QToolTip::add(m_pModeLabel,tip);
+	m_pModeWidget->refreshModes();
+	QToolTip::remove(m_pModeWidget);
+	QToolTip::add(m_pModeWidget,tip);
 }
 
 /*
@@ -1443,7 +1522,9 @@ void KviChannel::outputMessage(int msg_type,const char *format,...)
 
 void KviChannel::outputMessage(int msg_type,const QString &msg)
 {
-	const QChar * pC = KviQString::nullTerminatedArray(msg);
+	QString szBuf(msg);
+	preprocessMessage(szBuf);
+	const QChar * pC = KviQString::nullTerminatedArray(szBuf);
 	if(!pC)return;
 	internalOutput(m_pMessageView ? m_pMessageView : m_pIrcView,msg_type,(const kvi_wchar_t *)pC);
 }
@@ -1478,15 +1559,11 @@ void KviChannel::checkChannelSync()
 	// we already have all the spontaneous server replies
 	// (so probably mode, topic (or no topic is set),names)
 	// we have already received the I and e lists (if requested)
-	bool bStop = false;
-	int iSyncTime = m_joinTime.time().msecsTo(QTime::currentTime());
+	kvs_int_t iSyncTime = m_joinTime.time().msecsTo(QTime::currentTime());
 	if(iSyncTime < 0)iSyncTime += 86400000;
 
-	if(KviKvsEventManager::instance()->hasAppHandlers(KviEvent_OnChannelSync))
-	{
-		bStop = g_pUserParser->triggerEvent(KviEvent_OnChannelSync,this,
-					new KviParameterList(new KviStr(KviStr::Format,"%d",iSyncTime)));
-	}
+	bool bStop = KVS_TRIGGER_EVENT_1_HALTED(KviEvent_OnChannelSync,this,iSyncTime);
+
 	if(!bStop && KVI_OPTION_BOOL(KviOption_boolShowChannelSyncTime))
 	{
 		output(KVI_OUT_SYSTEMMESSAGE,__tr2qs("Channel synchronized in %d.%d seconds"),iSyncTime / 1000,iSyncTime % 1000);
@@ -1502,4 +1579,29 @@ bool KviChannel::eventFilter(QObject * o, QEvent * e)
 	return KviWindow::eventFilter(o, e);
 }
 
+
+void KviChannel::preprocessMessage(QString & szMessage)
+{
+	QStringList strings = QStringList::split(" ",szMessage, TRUE);
+	for ( QStringList::Iterator it = strings.begin(); it != strings.end(); ++it ) {
+		if((*it).contains('\r')) continue;
+		QString tmp = KviMircCntrl::stripControlBytes(*it);
+		if( findEntry(*it) ) 	*it=QString("\r!n\r%1\r").arg(*it);
+		if(m_pConsole)
+			if(m_pConsole->connection())
+				if(m_pConsole->connection()->serverInfo()->supportedChannelTypes().contains(tmp[0]))
+					if((*it)==tmp)
+						*it=QString("\r!c\r%1\r").arg(*it);
+					else
+						*it=QString("\r!c%1\r%2\r").arg(tmp).arg(*it);
+	}
+	szMessage=strings.join(" ");
+}
+
+void KviChannel::unhighlight()
+{
+	if(!m_pTaskBarItem)return;
+	m_pTaskBarItem->unhighlight();
+}
+
 #include "kvi_channel.moc"
