@@ -40,14 +40,13 @@
 #include "kvi_proxydb.h"
 #include "kvi_netutils.h"
 #include "kvi_ircserver.h"
+#include "kvi_ircserverdb.h"
 #include "kvi_dns.h"
 #include "kvi_defaults.h"
 #include "kvi_ircuserdb.h"
 #include "kvi_channel.h"
 #include "kvi_query.h"
-#include "kvi_event.h"
 #include "kvi_parameterlist.h"
-#include "kvi_uparser.h"
 #include "kvi_regusersdb.h"
 #include "kvi_userlistview.h"
 #include "kvi_out.h"
@@ -71,11 +70,15 @@
 #include "kvi_ircconnectionstatistics.h"
 #include "kvi_asynchronousconnectiondata.h"
 #include "kvi_ircdatastreammonitor.h"
+#include "kvi_toolwindows_container.h"
 
 #ifdef COMPILE_SSL_SUPPORT
 	#include "kvi_sslmaster.h"
 #endif
 
+#include "kvi_kvs_script.h"
+#include "kvi_kvs_eventtriggers.h"
+
 #include <qsplitter.h>
 #include <qtoolbar.h>
 #include <qtoolbutton.h>
@@ -117,8 +120,6 @@ KviConsole::KviConsole(KviFrame * lpFrm,int iFlags)
 	m_pButtonBox->setStretchFactor(l,1);
 	QToolTip::add(l,__tr2qs("What's this?"));
 
-	m_pNotifyViewButton = createToolButton(m_pButtonBox,"list_view_button",KVI_SMALLICON_HIDELISTVIEW,KVI_SMALLICON_SHOWLISTVIEW,__tr2qs("Toggle Notify List"),true);
-	connect(m_pNotifyViewButton,SIGNAL(clicked()),this,SLOT(toggleNotifyView()));
 
 	m_pSplitter = new QSplitter(QSplitter::Horizontal,this,"splitter");
 	m_pIrcView = new KviIrcView(m_pSplitter,lpFrm,this);
@@ -127,7 +128,11 @@ KviConsole::KviConsole(KviFrame * lpFrm,int iFlags)
 	// FIXME: #warning "If notify list is disabled avoid to show this"
 	// FIXME: #warning "Button to show/hide the notifyListView (NOT DELETE RE_CREATE!)"
 	// The userlist on the right
-	m_pNotifyListView = new KviUserListView(m_pSplitter,0,this,19,__tr2qs("Notify List"),"notify_list_view");
+	//m_pEditorsContainer= new KviToolWindowsContainer(m_pSplitter);
+	m_pNotifyViewButton = new KviWindowToolPageButton(KVI_SMALLICON_HIDELISTVIEW,KVI_SMALLICON_SHOWLISTVIEW,__tr2qs("Notify List"),buttonContainer(),true,"list_view_button");
+	connect(m_pNotifyViewButton,SIGNAL(clicked()),this,SLOT(toggleNotifyView()));
+	
+	m_pNotifyListView = new KviUserListView(m_pSplitter,m_pNotifyViewButton,0,this,19,__tr2qs("Notify List"),"notify_list_view");
 
 	m_pInput   = new KviInput(this,m_pNotifyListView);
 
@@ -160,12 +165,12 @@ KviConsole::~KviConsole()
 	// Force connection close: it will just return if no connection is present
 	context()->terminateConnectionRequest(true);
 
-	TRIGGER_EVENT(KviEvent_OnIrcContextDestroyed,this);
+	KVS_TRIGGER_EVENT_0(KviEvent_OnIrcContextDestroyed,this);
 
 	if(g_pFrame->consoleCount() <= 1)
 	{
-		TRIGGER_EVENT(KviEvent_OnFrameWindowDestroyed,this);
-		TRIGGER_EVENT(KviEvent_OnKVIrcShutdown,this);
+		KVS_TRIGGER_EVENT_0(KviEvent_OnFrameWindowDestroyed,this);
+		KVS_TRIGGER_EVENT_0(KviEvent_OnKVIrcShutdown,this);
 	}
 
 	//if(m_pLastIrcServer)delete m_pLastIrcServer;
@@ -184,9 +189,9 @@ unsigned int KviConsole::ircContextId()
 	return m_pContext->id();
 }
 
-const char * KviConsole::currentNetworkName()
+QString KviConsole::currentNetworkName()
 {
-	return (connection() ? connection()->networkName().latin1() : KviStr::emptyString().ptr());
+	return (connection() ? connection()->networkName() : QString::null);
 }
 
 
@@ -194,7 +199,7 @@ void KviConsole::triggerCreationEvents()
 {
 	if(m_iFlags & KVI_CONSOLE_FLAG_FIRSTINAPP) // this is the first context in the application
 	{
-		TRIGGER_EVENT(KviEvent_OnKVIrcStartup,this);
+		KVS_TRIGGER_EVENT_0(KviEvent_OnKVIrcStartup,this);
 
 		if(KVI_OPTION_BOOL(KviOption_boolShowTipAtStartup))
 			g_pFrame->executeInternalCommand(KVI_INTERNALCOMMAND_TIP_OPEN);
@@ -205,10 +210,10 @@ void KviConsole::triggerCreationEvents()
 
 	if(m_iFlags & KVI_CONSOLE_FLAG_FIRSTINFRAME)
 	{
-		TRIGGER_EVENT(KviEvent_OnFrameWindowCreated,this);
+		KVS_TRIGGER_EVENT_0(KviEvent_OnFrameWindowCreated,this);
 	}
 
-	TRIGGER_EVENT(KviEvent_OnIrcContextCreated,this);
+	KVS_TRIGGER_EVENT_0(KviEvent_OnIrcContextCreated,this);
 }
 
 
@@ -250,9 +255,13 @@ void KviConsole::completeChannel(const QString &word,KviPtrList<QString> * match
 		if(kvi_strEqualCIN(c->windowName(),word.ptr(),word.len()))matches->append(new KviStr((*it)
 	}
 	*/
-	for(QStringList::Iterator it = KVI_OPTION_STRINGLIST(KviOption_stringlistRecentChannels).begin(); it != KVI_OPTION_STRINGLIST(KviOption_stringlistRecentChannels).end(); ++it)
+	QStringList *pList = g_pApp->getRecentChannels(currentNetworkName());
+	if(pList)
 	{
-		if(KviQString::equalCIN((*it),word,word.length()))matches->append(new QString(*it));
+		for(QStringList::Iterator it = pList->begin(); it != pList->end(); ++it)
+		{
+			if(KviQString::equalCIN((*it),word,word.length()))matches->append(new QString(*it));
+		}
 	}
 }
 
@@ -276,6 +285,8 @@ void KviConsole::completeServer(const QString &word, KviPtrList<QString> * match
 
 void KviConsole::getUserTipText(const QString &nick,KviIrcUserEntry *e,QString &buffer)
 {
+	KviRegisteredMask *u = g_pRegisteredUserDataBase->findMatchingMask(nick,e->user(),e->host());
+
 	buffer = "<table width=\"100%\">" \
 				"<tr><td bgcolor=\"#303030\">" \
 					"<center><font color=\"#FFFFFF\"><b>";
@@ -288,7 +299,16 @@ void KviConsole::getUserTipText(const QString &nick,KviIrcUserEntry *e,QString &
 
 
 	buffer += "</b></font></center></td></tr>";
-
+	if(u)
+	{
+		QString szComment=u->user()->getProperty("comment");
+		if(!szComment.isEmpty())
+		{
+			buffer += "<tr bgcolor=\"#F0F0F0\"><td><center><font size=\"-1\">(";
+			buffer += szComment;
+			buffer += ")</font></center></td></tr>";
+		}
+	}
 	if(e->avatar())
 	{
 		QMimeSourceFactory::defaultFactory()->setPixmap("ulv_avatar",*(e->avatar()->pixmap()));
@@ -302,23 +322,21 @@ void KviConsole::getUserTipText(const QString &nick,KviIrcUserEntry *e,QString &
 		buffer += "</b></center></td></tr>";
 	}
 
-	if(!e->user().isEmpty())
+	if(u)
 	{
-		KviRegisteredMask *u = g_pRegisteredUserDataBase->findMatchingMask(nick,e->user(),e->host());
-		if(u)
-		{
-			QString mask;
-			u->mask()->mask(mask);
-			buffer += "<tr bgcolor=\"#F0F0F0\"><td>";
-			buffer += __tr2qs("Registered as");
-			buffer += " <b>";
-			buffer += u->user()->name();
-			buffer += "</b> <font size=\"-1\">(";
-			buffer += __tr2qs("Matched by");
-			buffer += " ";
-			buffer += mask;
-			buffer += ")</td></tr>";
-		}
+		QString mask;
+		u->mask()->mask(mask);
+		buffer += "<tr bgcolor=\"#F0F0F0\"><td>";
+		buffer += __tr2qs("Registered as");
+		buffer += " <b>";
+		buffer += u->user()->name();
+		buffer += "</b>; Group ";
+		buffer += u->user()->group();
+		buffer += "</td></tr><tr bgcolor=\"#F0F0F0\"><td><font size=\"-1\">";
+		buffer += __tr2qs("(Matched by");
+		buffer += " ";
+		buffer += mask;
+		buffer += ")</td></tr>";
 	}
 
 	if(connection())
@@ -347,6 +365,13 @@ void KviConsole::getUserTipText(const QString &nick,KviIrcUserEntry *e,QString &
 			buffer += "</nobr></td></tr></table>";
 		}
 	}
+	
+	if(e->isAway())
+	{
+		buffer += "<tr><td bgcolor=\"#F0F0F0\">";
+		buffer += __tr2qs("Probably Away");
+		buffer += "</td></tr>";
+	}
 }
 
 
@@ -364,7 +389,7 @@ void KviConsole::toggleNotifyView()
 
 void KviConsole::executeInternalCommand(int index)
 {
-	g_pUserParser->parseCommandBuffer(kvi_getInternalCommandBuffer(index),this);
+	KviKvsScript::run(kvi_getInternalCommandBuffer(index),this);
 }
 
 void KviConsole::saveProperties(KviConfig *cfg)
@@ -378,6 +403,10 @@ void KviConsole::getBaseLogFileName(KviStr &buffer)
 {
 	buffer.sprintf("CONSOLE%u",ircContextId());
 }
+void KviConsole::getBaseLogFileName(QString &buffer)
+{
+	buffer=QString("CONSOLE%1").arg(ircContextId());
+}
 
 void KviConsole::loadProperties(KviConfig *cfg)
 {
@@ -399,7 +428,7 @@ void KviConsole::loadProperties(KviConfig *cfg)
 
 void KviConsole::textViewRightClicked()
 {
-	TRIGGER_EVENT(KviEvent_OnConsolePopupRequest,this);
+	KVS_TRIGGER_EVENT_0(KviEvent_OnConsolePopupRequest,this);
 }
 
 
@@ -439,8 +468,7 @@ void KviConsole::closeEvent(QCloseEvent *e)
 			{
 				switch(QMessageBox::warning(this,
 					__tr2qs("Confirmation - KVIrc"),
-					__tr2qs("You have just attempted to close a console window<br>" \
-						"with an active connection inside.<br>" \
+					__tr2qs("You have just attempted to close a console window with an active connection inside.\n" \
 						"Are you sure you wish to terminate the connection?"),
 					__tr2qs("&Always"),
 					__tr2qs("&Yes"),
@@ -474,7 +502,7 @@ void KviConsole::closeEvent(QCloseEvent *e)
 	{
 		switch(QMessageBox::warning(this,
 				__tr2qs("Confirmation - KVIrc"),
-				__tr2qs("You have just attempted to close the last console window.<br>Are you sure you wish to quit KVIrc?"),
+				__tr2qs("You have just attempted to close the last console window.\nAre you sure you wish to quit KVIrc?"),
 				__tr2qs("&Always"),
 				__tr2qs("&Yes"),
 				__tr2qs("&No"),
@@ -498,11 +526,13 @@ void KviConsole::closeEvent(QCloseEvent *e)
 // internal helper for applyHighlighting
 int KviConsole::triggerOnHighlight(KviWindow *wnd,int type,const QString &nick,const QString &user,const QString &host,const QString &szMsg,const QString &trigger)
 {
-	KviStr szMessageType(KviStr::Format,"%d",type);
-	if(TRIGGER_EVENT_7PARAM_RETVALUE(KviEvent_OnHighlight,
+	if(!KVI_OPTION_STRING(KviOption_stringOnHighlightedMessageSound).isEmpty() && wnd!=g_pActiveWindow) KviKvsScript::run("snd.play $0",0,new KviKvsVariantList(new KviKvsVariant(KVI_OPTION_STRING(KviOption_stringOnHighlightedMessageSound))));
+	QString szMessageType;
+	KviQString::sprintf(szMessageType,"%d",type);
+	if(KVS_TRIGGER_EVENT_7_HALTED(KviEvent_OnHighlight,
 					wnd,nick,user,host,
 					szMsg,trigger,
-					szMessageType.ptr(),(type == KVI_OUT_ACTION) ? '1' : '0'))
+					szMessageType,(type == KVI_OUT_ACTION)))
 		return -1;
 	return KVI_OUT_HIGHLIGHT;
 }
@@ -510,9 +540,28 @@ int KviConsole::triggerOnHighlight(KviWindow *wnd,int type,const QString &nick,c
 // if it returns -1 you should just return and not display the message
 int KviConsole::applyHighlighting(KviWindow *wnd,int type,const QString &nick,const QString &user,const QString &host,const QString &szMsg)
 {
+	QString szPattern=KVI_OPTION_STRING(KviOption_stringWordSplitters);
+	QString szSource;
+	QString szStripMsg=KviMircCntrl::stripControlBytes(szMsg);
+	QChar* aux=(QChar*)(szStripMsg.ucs2());
+	if(aux)
+	{
+		while(aux->unicode())
+		{
+			if( KVI_OPTION_STRING(KviOption_stringWordSplitters).find(*aux) > -1 )
+				szSource.append(' ');
+			else
+				szSource.append(*aux);
+			aux++;
+		}
+	} else {
+		szSource=szStripMsg;
+	}
+	szSource.append(' ');
+	szSource.prepend(' ');
 	if(KVI_OPTION_BOOL(KviOption_boolAlwaysHighlightNick) && connection())
 	{
-		if(szMsg.find(connection()->userInfo()->nickName(),0,false) > -1)
+		if(szSource.find(QString(" %1 ").arg(connection()->userInfo()->nickName()),0,false) > -1)
 			return triggerOnHighlight(wnd,type,nick,user,host,szMsg,connection()->userInfo()->nickName());
 	}
 
@@ -521,8 +570,8 @@ int KviConsole::applyHighlighting(KviWindow *wnd,int type,const QString &nick,co
 		for(QStringList::Iterator it = KVI_OPTION_STRINGLIST(KviOption_stringlistHighlightWords).begin();
 				it != KVI_OPTION_STRINGLIST(KviOption_stringlistHighlightWords).end() ; ++it)
 		{
-			// FIXME : This is SLOOOOOOOOW (QString -> ascii translation!!)
-			if(szMsg.find(*it,0,false) > -1)
+			// FIXME : This is SLOOOOOOOOW (QString -> ascii translation!!) !!!!
+			if(szSource.find(QString(" %1 ").arg(*it),0,false) > -1)
 				return triggerOnHighlight(wnd,type,nick,user,host,szMsg,*it);
 		}
 	}
@@ -636,7 +685,7 @@ void KviConsole::outputPrivmsg(KviWindow *wnd,
 	// <PREFIX>nick[!user@host]<POSTFIX>This is a test message
 
 	QString szNick;
-	KviQString::sprintf(szNick,"\r!n\r%Q\r",&nick);
+	KviQString::sprintf(szNick,"\r!nc\r%Q\r",&nick);
 
 	if(KVI_OPTION_BOOL(KviOption_boolShowUserAndHostInPrivmsgView))
 		KviQString::appendFormatted(szNick,"!%Q@\r!h\r%Q\r",&user,&host);
@@ -644,7 +693,7 @@ void KviConsole::outputPrivmsg(KviWindow *wnd,
 	if(bIsChan && KVI_OPTION_BOOL(KviOption_boolShowChannelUserFlagInPrivmsgView))
 		((KviChannel *)wnd)->prependUserFlag(nick,szNick);
 
-	if(KVI_OPTION_BOOL(KviOption_boolColorNicks))
+	if(KVI_OPTION_BOOL(KviOption_boolColorNicks) && !KVI_OPTION_BOOL(KviOption_boolUseUserListColorsAsNickColors))
 	{
 		int sum = 0;
 		int i = nick.length();
@@ -660,7 +709,41 @@ void KviConsole::outputPrivmsg(KviWindow *wnd,
 		szNick.prepend(KVI_TEXT_COLOR);
 		szNick.append(KVI_TEXT_COLOR);
 	}
-
+/*	if(KVI_OPTION_BOOL(KviOption_boolUseUserListColorsAsNickColors) && bIsChan)
+	{
+		if(((KviChannel*)wnd)->userListView())
+		{
+			KviUserListEntry *e = ((KviChannel*)wnd)->userListView()->findEntry(nick);
+			if(e)
+			{
+				int sum = 0;
+				int i = nick.length();
+				const QChar * aux = nick.unicode();
+				// FIXME: Shouldn't this be case insensitive ?
+				while(i > 0)
+				{
+					sum += aux->unicode();
+					aux++;
+					i--;
+				}
+				int color;
+				if(e->flags() == 0)
+				{
+					color = KVI_COLOR_EXT_USER_NORMAL;
+				} else {
+					color = (e->flags() & KVI_USERFLAG_CHANOWNER) ? \
+						KVI_COLOR_EXT_USER_OWNER : ((e->flags() & KVI_USERFLAG_CHANADMIN) ? \
+						KVI_COLOR_EXT_USER_ADMIN : ((e->flags() & KVI_USERFLAG_OP) ? \
+						KVI_COLOR_EXT_USER_OP : ((e->flags() & KVI_USERFLAG_HALFOP) ? \
+						KVI_COLOR_EXT_USER_HALFOP : ((e->flags() & KVI_USERFLAG_VOICE) ? \
+						KVI_COLOR_EXT_USER_VOICE : KVI_COLOR_EXT_USER_USEROP))));
+				}
+				szNick.prepend(QString("%1").arg(color));
+				szNick.prepend(KVI_TEXT_COLOR);
+				szNick.append(KVI_TEXT_COLOR);
+			}
+		}
+	}*/
 	if(KVI_OPTION_BOOL(KviOption_boolBoldedNicks))
 	{
 		szNick.prepend(KVI_TEXT_BOLD);
@@ -746,9 +829,9 @@ void KviConsole::avatarChanged(KviAvatar * avatar,const QString &nick,const QStr
 	{
 		// cache it
 		if(avatar)
-			KviAvatarCache::instance()->replace(avatar->identificationString(),KviIrcMask(nick,user,host),currentNetworkName());
+			KviAvatarCache::instance()->replace(avatar->identificationString(),KviIrcMask(nick,user,host),currentNetworkName().utf8().data());
 		else
-			KviAvatarCache::instance()->remove(KviIrcMask(nick,user,host),currentNetworkName());
+			KviAvatarCache::instance()->remove(KviIrcMask(nick,user,host),currentNetworkName().utf8().data());
 	}
 	
 	avatarChangedUpdateWindows(nick,textLine);
@@ -757,7 +840,7 @@ void KviConsole::avatarChanged(KviAvatar * avatar,const QString &nick,const QStr
 void KviConsole::checkDefaultAvatar(KviIrcUserEntry *e,const QString &nick,const QString &user,const QString &host)
 {
 	// look it up in the cache
-	QString szAvatar = KviAvatarCache::instance()->lookup(KviIrcMask(nick,user,host),currentNetworkName());
+	QString szAvatar = KviAvatarCache::instance()->lookup(KviIrcMask(nick,user,host),currentNetworkName().utf8().data());
 	if(!szAvatar.isEmpty())
 	{
 		// got a cache hit... is it on disk ?
@@ -771,7 +854,7 @@ void KviConsole::checkDefaultAvatar(KviIrcUserEntry *e,const QString &nick,const
 		} else {
 			// no cached image on disk.. will need to requery it anyway
 			// remove from cache
-			KviAvatarCache::instance()->remove(KviIrcMask(nick,user,host),currentNetworkName());
+			KviAvatarCache::instance()->remove(KviIrcMask(nick,user,host),currentNetworkName().utf8().data());
 		}
 	}
 
