@@ -23,7 +23,6 @@
 //=============================================================================
 
 #define __KVIRC__
-#define _KVI_DEBUG_CHECK_RANGE_
 #define _KVI_FRAME_CPP_
 
 #include "kvi_debug.h"
@@ -40,12 +39,10 @@
 #include "kvi_console.h"
 #include "kvi_config.h"
 #include "kvi_internalcmd.h"
-#include "kvi_uparser.h"
 #include "kvi_console.h"
 #include "kvi_debug.h"
 #include "kvi_irctoolbar.h"
 #include "kvi_confignames.h"
-#include "kvi_event.h"
 #include "kvi_parameterlist.h"
 #include "kvi_module.h"
 #include "kvi_mextoolbar.h"
@@ -58,13 +55,16 @@
 #include "kvi_actionmanager.h"
 #include "kvi_defaults.h"
 
+#include "kvi_kvs_script.h"
+#include "kvi_kvs_eventtriggers.h"
+
 #include <qsplitter.h>
 #include <qvariant.h>
 #include <qlineedit.h>
 #include <qmessagebox.h>
 #include <qcheckbox.h>
 #include <qaccel.h>
-
+#include <qtimer.h>
 #include <qlayout.h>
 
 #if QT_VERSION >= 300
@@ -103,8 +103,10 @@ KviFrame::KviFrame()
 	m_pActiveContext = 0;
 
 	m_pDockExtension = 0;
-
+	
 	m_pSplitter = new QSplitter(QSplitter::Horizontal,this,"main_splitter");
+//	m_pSplitter->setFrameShape(QFrame::NoFrame);
+	
 	setCentralWidget(m_pSplitter);
 
 	setUsesBigPixmaps(KVI_OPTION_BOOL(KviOption_boolUseBigIcons));
@@ -146,6 +148,8 @@ KviFrame::KviFrame()
 
 KviFrame::~KviFrame()
 {
+	g_pApp->frameDestructorCallback();
+
 	KVI_OPTION_RECT(KviOption_rectFrameGeometry) = QRect(pos().x(),pos().y(),
 			size().width(),size().height());
 
@@ -204,13 +208,13 @@ void KviFrame::restoreModuleExtensionToolBars()
 {
 	for(QStringList::Iterator it = KVI_OPTION_STRINGLIST(KviOption_stringlistModuleExtensionToolbars).begin();it != KVI_OPTION_STRINGLIST(KviOption_stringlistModuleExtensionToolbars).end();++it)
 	{
-		KviStr szEntry = *it;
-		int idx = szEntry.findFirstIdx(':');
+		QString szEntry = *it;
+		int idx = szEntry.find(':');
 		if(idx != -1)
 		{
-			KviStr szMod = szEntry.left(idx);
-			szEntry.cutLeft(idx + 1);
-			g_pModuleExtensionManager->allocateExtension("toolbar",szEntry.ptr(),firstConsole(),0,0,szMod.ptr());
+			QString szMod = szEntry.left(idx);
+			szEntry.remove(0,idx + 1);
+			g_pModuleExtensionManager->allocateExtension("toolbar",szEntry,firstConsole(),0,0,szMod);
 		}
 	}
 }
@@ -262,6 +266,7 @@ KviMexToolBar * KviFrame::moduleExtensionToolBar(int extensionId)
 		[b]Ctrl+UpArrow[/b]: Maximize current window[br]
 		[b]Ctrl+DownArrow[/b] or [b]ESC[/b]: Minimize current window[br]
 		[b]Ctrl+&lt;digit&gt;[/b], [b]F1-F12[/b], [b]Shift+(F1-F12)[/b]: Script accelerators (see [event:onaccelkeypressed]OnAccelKeyPressed[/event])[br]
+		[b]Shift+&lt;F1-F12&gt;[/b] window switch[br]
 		[b]Tab in the first word of input[/b]: Completes nicknames in the current channel or query[br]
 		[b]Tab after a leading /[/b]: Completes commands[br]
 		[b]Tab after a / in the middle of input[/b]: Completes directories[br]
@@ -290,7 +295,7 @@ KviMexToolBar * KviFrame::moduleExtensionToolBar(int extensionId)
 		[b]Shift+PageDown[/b]: Scrolls the output window down one line[b]
 		[b]Alt+lt;numeric_sequence&gt;[/b]: Inserts the character by ASCII/Unicode code[br]
 		[b]Ctrl+Backspace[/b]: Shows or hides the multiline editor[br]
-		[b]Ctrl+End[/b]: Closes the current window[br]
+		[b]Ctrl+F4[/b]: Closes the current window[br]
 		<example>
 			Alt+32: Inserts ASCII/Unicode character 32: ' ' (a space)
 			Alt+00032: Same as above :)
@@ -309,10 +314,10 @@ QAccel * KviFrame::installAccelerators(QWidget * wnd)
 		Key_Right + ALT ,   // next window
 		Key_Up + CTRL ,      // maximize window
 		Key_Down + CTRL ,    // minimize window
-		Key_Escape ,         // minimize window
+		Key_Escape +CTRL,         // minimize window
 		Key_Left + ALT + SHIFT ,  // prev window in context
 		Key_Right + ALT + SHIFT,  // next window in context
-		Key_End + CTRL ,     // close current window
+		Key_F4 + CTRL ,     // close current window
 		Key_1 + CTRL ,       // script accels...
 		Key_2 + CTRL ,
 		Key_3 + CTRL ,
@@ -323,7 +328,7 @@ QAccel * KviFrame::installAccelerators(QWidget * wnd)
 		Key_8 + CTRL ,
 		Key_9 + CTRL ,
 		Key_0 + CTRL ,
-		Key_F1 ,
+		Key_F1 , // reserved for context sensitive help
 		Key_F2 ,
 		Key_F3 ,
 		Key_F4 ,
@@ -335,7 +340,7 @@ QAccel * KviFrame::installAccelerators(QWidget * wnd)
 		Key_F10 ,
 		Key_F11 ,
 		Key_F12 ,
-		Key_F1 + SHIFT ,
+/*		Key_F1 + SHIFT ,       // window select...
 		Key_F2 + SHIFT ,
 		Key_F3 + SHIFT ,
 		Key_F4 + SHIFT ,
@@ -346,7 +351,7 @@ QAccel * KviFrame::installAccelerators(QWidget * wnd)
 		Key_F9 + SHIFT ,
 		Key_F10 + SHIFT ,
 		Key_F11 + SHIFT ,
-		Key_F12 + SHIFT ,
+		Key_F12 + SHIFT ,*/
 		0
 	};
 
@@ -367,36 +372,77 @@ void KviFrame::accelActivated(int id)
 	QAccel * acc = (QAccel *)sender();
 
 	int keys = (int)(acc->key(id));
-
+	KviTaskBarItem *item = 0;
 	switch(keys)
 	{
-		case (Key_Left+ALT):  switchToPrevWindow(); break;
+		case (Key_Left+ALT): switchToPrevWindow(); break;
 		case (Key_Right+ALT): switchToNextWindow(); break;
-		case (Key_Up+CTRL):    maximizeWindow(); break;
-		case (Key_Escape):
-		case (Key_Down+CTRL):  minimizeWindow(); break;
-		case (Key_Left+ALT+SHIFT):   switchToPrevWindowInContext(); break;
-		case (Key_Right+ALT+SHIFT):  switchToNextWindowInContext(); break;
-		case (Key_End+CTRL):   if(g_pActiveWindow)g_pActiveWindow->close(); break;
+		case (Key_Up+CTRL): maximizeWindow(); break;
+		case (Key_Escape+CTRL):
+		case (Key_Down+CTRL): minimizeWindow(); break;
+		case (Key_Left+ALT+SHIFT): switchToPrevWindowInContext(); break;
+		case (Key_Right+ALT+SHIFT): switchToNextWindowInContext(); break;
+		case (Key_F4+CTRL):	if(g_pActiveWindow)g_pActiveWindow->close(); break;
+		case (Key_F1): g_pApp->contextSensitiveHelp(); break;
+/*		case(Key_F1 + SHIFT):
+			item = m_pTaskBar->item(0);
+			if(item) setActiveWindow(item->window());
+			break;
+		case(Key_F2 + SHIFT):
+			item = m_pTaskBar->item(1);
+			if(item) setActiveWindow(item->window());
+			break;
+		case(Key_F3 + SHIFT):
+			item = m_pTaskBar->item(2);
+			if(item) setActiveWindow(item->window());
+			break;
+		case(Key_F4 + SHIFT):
+			item = m_pTaskBar->item(3);
+			if(item) setActiveWindow(item->window());
+			break;
+		case(Key_F5 + SHIFT):
+			item = m_pTaskBar->item(4);
+			if(item) setActiveWindow(item->window());
+			break;
+		case(Key_F6 + SHIFT):
+			item = m_pTaskBar->item(5);
+			if(item) setActiveWindow(item->window());
+			break;
+		case(Key_F7 + SHIFT):
+			item = m_pTaskBar->item(6);
+			if(item) setActiveWindow(item->window());
+			break;
+		case(Key_F8 + SHIFT):
+			item = m_pTaskBar->item(7);
+			if(item) setActiveWindow(item->window());
+			break;
+		case(Key_F9 + SHIFT):
+			item = m_pTaskBar->item(8);
+			if(item) setActiveWindow(item->window());
+			break;
+		case(Key_F10 + SHIFT):
+			item = m_pTaskBar->item(9);
+			if(item) setActiveWindow(item->window());
+			break;
+		case(Key_F11 + SHIFT):
+			item = m_pTaskBar->item(10);
+			if(item) setActiveWindow(item->window());
+			break;
+		case(Key_F12 + SHIFT):
+			item = m_pTaskBar->item(11);
+			if(item) setActiveWindow(item->window());
+			break;*/
 		default:
 		{
-//			debug("Unhandled accel keys %d",keys);
-#if QT_VERSION >= 300
-			KviStr * keysstr = new KviStr((QString)(acc->key(id)));
-#else
-			KviStr * keysstr = new KviStr(QAccel::keyToString(acc->key(id)));
-#endif
-			TRIGGER_EVENT_1PARAM_NONEW(KviEvent_OnAccelKeyPressed,g_pActiveWindow,keysstr);
+			KVS_TRIGGER_EVENT_1(KviEvent_OnAccelKeyPressed,g_pActiveWindow,(QString)(acc->key(id)));
 		}
 		break;
 	};
 }
 
-
-
 void KviFrame::executeInternalCommand(int index)
 {
-	g_pUserParser->parseCommandBuffer(kvi_getInternalCommandBuffer(index),firstConsole());
+	KviKvsScript::run(kvi_getInternalCommandBuffer(index),firstConsole());
 }
 
 
@@ -463,10 +509,10 @@ void KviFrame::closeWindow(KviWindow *wnd)
 	// save it's properties
 	if(KVI_OPTION_BOOL(KviOption_boolWindowsRememberProperties)) // && (wnd->type() == KVI_WINDOW_TYPE_CHANNEL))
 	{
-		KviStr group;
+		QString group;
 		wnd->getConfigGroupName(group);
 		// not uses default settings : store it always
-		saveWindowProperties(wnd,group.ptr());
+		saveWindowProperties(wnd,group);
 	}
 
 	// forget it...
@@ -520,18 +566,20 @@ void KviFrame::addWindow(KviWindow *wnd,bool bShow)
 	m_pWinList->append(wnd);
 	wnd->createTaskBarItem(); // create the window taskbar item AFTER it has been constructed
 
-	KviStr group;
+	QString group;
 	wnd->getConfigGroupName(group);
 
 	bool bGroupSettings = false;
 
-	if(g_pWinPropertiesConfig->hasGroup(group.ptr()))
-		g_pWinPropertiesConfig->setGroup(group.ptr());
-	else {
+	if(g_pWinPropertiesConfig->hasGroup(group))
+	{
+		g_pWinPropertiesConfig->setGroup(group);
+	} else {
 		bGroupSettings = true;
 		if(g_pWinPropertiesConfig->hasGroup(wnd->typeString()))
+		{
 			g_pWinPropertiesConfig->setGroup(wnd->typeString());
-		else {
+		} else {
 			g_pWinPropertiesConfig->setGroup("no_settings_group");
 			wnd->loadProperties(g_pWinPropertiesConfig); // load it anyway (will set defaults if windows don't remember properties)
 			goto default_docking; // no settings stored
@@ -678,7 +726,7 @@ KviConsole * KviFrame::firstConsole()
 		if(wnd->type() == KVI_WINDOW_TYPE_CONSOLE)return (KviConsole *)wnd;
 	}
 	__range_valid(false);
-	return 0; //newer here!
+	return 0; //should newer be here!.. but sometimes we are ?
 }
 
 KviConsole * KviFrame::firstNotConnectedConsole()
@@ -784,9 +832,9 @@ void KviFrame::childWindowActivated(KviWindow *wnd)
 	//wnd->gainedActiveWindowStatus(); // <-- atm unused
 
 	emit activeWindowChanged();
-	TRIGGER_EVENT(KviEvent_OnWindowActivated,wnd);
-		
 	if(bActiveContextChanged)emit activeContextChanged();
+
+	KVS_TRIGGER_EVENT_0(KviEvent_OnWindowActivated,wnd);
 }
 
 void KviFrame::windowActivationChange(bool bOldActive)
@@ -823,6 +871,8 @@ void KviFrame::leftSdiMode()
 	updateCaption();
 }
 
+#define KVI_DEFAULT_FRAME_CAPTION "KVIrc " KVI_VERSION " " KVI_RELEASE_NAME
+
 void KviFrame::updateCaption()
 {
 	if(g_pActiveWindow)
@@ -843,48 +893,65 @@ void KviFrame::updateCaption()
 void KviFrame::closeEvent(QCloseEvent *e)
 {
 	e->ignore();
-	
-	if(KVI_OPTION_BOOL(KviOption_boolConfirmCloseWhenThereAreConnections))
+
+	if(KVI_OPTION_BOOL(KviOption_boolCloseInTray))
 	{
-		// check for running connections
-		
-		bool bGotRunningConnection = false;
-		for(KviWindow * w = m_pWinList->first();w;w = m_pWinList->next())
+		if(!dockExtension())
 		{
-			if(w->type() == KVI_WINDOW_TYPE_CONSOLE)
+		    executeInternalCommand(KVI_INTERNALCOMMAND_DOCKWIDGET_SHOW);
+		}
+		if(dockExtension())
+		{
+
+			dockExtension()->setPrevWindowState(windowState());
+			QTimer::singleShot( 0, this, SLOT(hide()) );
+		}
+	}
+	else
+	{
+	
+		if(KVI_OPTION_BOOL(KviOption_boolConfirmCloseWhenThereAreConnections))
+		{
+			// check for running connections
+			
+			bool bGotRunningConnection = false;
+			for(KviWindow * w = m_pWinList->first();w;w = m_pWinList->next())
 			{
-				if(((KviConsole *)w)->connectionInProgress())
+				if(w->type() == KVI_WINDOW_TYPE_CONSOLE)
 				{
-					bGotRunningConnection = true;
+					if(((KviConsole *)w)->connectionInProgress())
+					{
+						bGotRunningConnection = true;
+						break;
+					}
+				}
+			}
+			
+			if(bGotRunningConnection)
+			{
+				QString txt = "<p>";
+				txt += __tr2qs("There are active connections, are you sure you wish to ");
+				txt += __tr2qs("quit KVIrc?");
+				txt += "</p>";
+		
+				switch(QMessageBox::warning(this,__tr2qs("Confirmation - KVIrc"),txt,__tr2qs("&Yes"),__tr2qs("&Always"),__tr2qs("&No"),2,2))
+				{
+					case 0:
+						// ok to close
+					break;
+					case 1:
+						// ok to close but don't ask again
+						KVI_OPTION_BOOL(KviOption_boolConfirmCloseWhenThereAreConnections) = false;
+					break;
+					case 2:
+						return;
 					break;
 				}
 			}
 		}
-		
-		if(bGotRunningConnection)
-		{
-			QString txt = "<p>";
-			txt += __tr2qs("There are active connections, are you sure you wish to ");
-			txt += __tr2qs("quit KVIrc?");
-			txt += "</p>";
-	
-			switch(QMessageBox::warning(this,__tr2qs("Confirmation - KVIrc"),txt,__tr2qs("&Yes"),__tr2qs("&Always"),__tr2qs("&No"),2,2))
-			{
-				case 0:
-					// ok to close
-				break;
-				case 1:
-					// ok to close but don't ask again
-					KVI_OPTION_BOOL(KviOption_boolConfirmCloseWhenThereAreConnections) = false;
-				break;
-				case 2:
-					return;
-				break;
-			}
-		}
-	}
 
 	g_pApp->destroyFrame();
+	}
 }
 
 void KviFrame::resizeEvent(QResizeEvent *e)
@@ -1005,7 +1072,7 @@ void KviFrame::fillToolBarsPopup(QPopupMenu * p)
 
 void KviFrame::customizeToolBars()
 {
-	g_pUserParser->parseCommandBuffer("toolbareditor.open",g_pActiveWindow);
+	KviKvsScript::run("toolbareditor.open",g_pActiveWindow);
 }
 
 void KviFrame::toolbarsPopupSelected(int id)
@@ -1064,10 +1131,10 @@ bool KviFrame::focusNextPrevChild(bool next)
 
 void KviFrame::saveToolBarPositions()
 {
-	KviStr szTemp;
+	QString szTemp;
 	g_pApp->getLocalKvircDirectory(szTemp,KviApp::Config,KVI_CONFIGFILE_TOOLBARS);
 
-	QFile f(szTemp.ptr());
+	QFile f(szTemp);
 	if(f.open(IO_WriteOnly | IO_Truncate))
 	{
 		QTextStream ts(&f);
@@ -1079,10 +1146,10 @@ void KviFrame::saveToolBarPositions()
 
 void KviFrame::restoreToolBarPositions()
 {
-	KviStr szTemp;
+	QString szTemp;
 	g_pApp->getLocalKvircDirectory(szTemp,KviApp::Config,KVI_CONFIGFILE_TOOLBARS);
 
-	QFile f(szTemp.ptr());
+	QFile f(szTemp);
 	
 	bool bNeedDefaults = false;
 	
@@ -1129,7 +1196,7 @@ void KviFrame::restoreToolBarPositions()
 		}
 		*/
 
-	} else if(m_pTaskBar->inherits("KviClassicTaskBar"))
+	} /*else if(m_pTaskBar->inherits("KviClassicTaskBar"))
 	{
 		QDockArea * a = m_pTaskBar->area();
 		if((a == leftDock()) || (a == rightDock()))
@@ -1139,7 +1206,7 @@ void KviFrame::restoreToolBarPositions()
 			bottomDock()->moveDockWindow(m_pTaskBar);
 			bottomDock()->lineUp(true);
 		}
-	}
+	}*/
 	
 	if(bNeedDefaults)
 		lineUpDockWindows(false);
@@ -1209,6 +1276,41 @@ void KviFrame::recreateTaskBar()
 	if(g_pActiveWindow)m_pTaskBar->setActiveItem(g_pActiveWindow->taskBarItem());
 }
 
+
+#if QT_VERSION == 0x030201
+unsigned int KviFrame::windowState()
+{
+	/*		enum GNWindowState {	WindowNoState = 0x00000000, WindowMinimized = 0x00000001,
+									WindowMaximized = 0x00000002, WindowFullScreen = 0x00000004, WindowActive = 0x00000008 };
+			GNWindowState GNWState;
+			if(isMinimized())		GNWState=WindowMinimized;
+			else if(isMaximized())		GNWState=WindowMaximized;
+			else if(isActiveWindow())	GNWState=WindowActive;
+			else if(isFullScreen())		GNWState=WindowFullScreen;
+			else GNWState=WindowNoState; */
+
+/*	WindowNoState	=	0x00000000	WindowMinimized		= 0x00000001	
+	WindowMaximized	=	0x00000002	WindowFullScreen	= 0x00000004 WindowActive = 0x00000008*/
+			if(isMinimized())		return 0x00000001;
+			else if(isMaximized())		return 0x00000002;
+			else if(isActiveWindow())	return 0x00000008;
+			else if(isFullScreen())		return 0x00000004;
+			else return 0x00000000;
+}
+void KviFrame::setWindowState(unsigned int GNWState)
+{
+	switch(GNWState)
+	{
+	case 0x00000001:
+		showMinimized();
+		break;
+	case 0x00000002:
+		showMaximized();
+		break;
+	} // switch
+}
+#endif
+
 ////////////////////////////////////////////////////////////////////////////////////////////////////
 // Some accelerators
 ////////////////////////////////////////////////////////////////////////////////////////////////////
@@ -1245,4 +1347,21 @@ void KviFrame::switchToNextWindowInContext(void)
 	m_pTaskBar->switchWindow(true,true);
 }
 
+void KviFrame::hideEvent ( QHideEvent * e)
+{
+	if(KVI_OPTION_BOOL(KviOption_boolMinimizeInTray))
+	{
+		if(e->spontaneous())
+		{
+		
+		if(!dockExtension())
+		{
+			executeInternalCommand(KVI_INTERNALCOMMAND_DOCKWIDGET_SHOW);
+		}
+			 QTimer::singleShot( 0, this, SLOT(hide()) );	
+		}
+		
+	}
+}
+
 #include "kvi_frame.moc"
