@@ -56,6 +56,7 @@
 KviMdiManager::KviMdiManager(QWidget * parent,KviFrame * pFrm,const char * name)
 : QScrollView(parent,name)
 {
+	setFrameShape(NoFrame);
 	m_pZ = new KviPtrList<KviMdiChild>;
 	m_pZ->setAutoDelete(true);
 
@@ -116,7 +117,6 @@ void KviMdiManager::drawContents(QPainter *p,int x,int y,int w,int h)
 	{
 		QPoint pnt = viewport()->mapToGlobal(contentsToViewport(r.topLeft()));
 		p->drawTiledPixmap(r,*(g_pShadedParentGlobalDesktopBackground),pnt);
-		return;
 	}
 #endif
 
@@ -490,7 +490,7 @@ void KviMdiManager::updateSDIMode()
 
 	if(!m_pSdiIconButton)
 	{
-		m_pSdiIconButton = new KviMenuBarToolButton(b,*pix);
+		m_pSdiIconButton = new KviMenuBarToolButton(b,*pix,"nonne");
 		connect(m_pSdiIconButton,SIGNAL(clicked()),this,SLOT(activeChildSystemPopup()));
 		m_iSdiIconItemId = b->insertItem(m_pSdiIconButton,-1,0);
 		connect(m_pSdiIconButton,SIGNAL(destroyed()),this,SLOT(sdiIconButtonDestroyed()));
@@ -526,17 +526,17 @@ void KviMdiManager::enterSDIMode(KviMdiChild *lpC)
 
 	if(!m_pSdiCloseButton)
 	{
-		m_pSdiMinimizeButton = new KviMenuBarToolButton(b,*(g_pIconManager->getSmallIcon(KVI_SMALLICON_MINIMIZE)));
+		m_pSdiMinimizeButton = new KviMenuBarToolButton(b,*(g_pIconManager->getSmallIcon(KVI_SMALLICON_MINIMIZE)),"btnminimize");
 		connect(m_pSdiMinimizeButton,SIGNAL(clicked()),this,SLOT(minimizeActiveChild()));
 		m_iSdiMinimizeItemId = b->insertItem(m_pSdiMinimizeButton,-1,b->count());
 		connect(m_pSdiMinimizeButton,SIGNAL(destroyed()),this,SLOT(sdiMinimizeButtonDestroyed()));
 
-		m_pSdiRestoreButton = new KviMenuBarToolButton(b,*(g_pIconManager->getSmallIcon(KVI_SMALLICON_RESTORE)));
+		m_pSdiRestoreButton = new KviMenuBarToolButton(b,*(g_pIconManager->getSmallIcon(KVI_SMALLICON_RESTORE)),"btnrestore");
 		connect(m_pSdiRestoreButton,SIGNAL(clicked()),this,SLOT(restoreActiveChild()));
 		m_iSdiRestoreItemId = b->insertItem(m_pSdiRestoreButton,-1,b->count());
 		connect(m_pSdiRestoreButton,SIGNAL(destroyed()),this,SLOT(sdiRestoreButtonDestroyed()));
 
-		m_pSdiCloseButton = new KviMenuBarToolButton(b,*(g_pIconManager->getSmallIcon(KVI_SMALLICON_CLOSE)));
+		m_pSdiCloseButton = new KviMenuBarToolButton(b,*(g_pIconManager->getSmallIcon(KVI_SMALLICON_CLOSE)),"btnclose");
 		connect(m_pSdiCloseButton,SIGNAL(clicked()),this,SLOT(closeActiveChild()));
 		m_iSdiCloseItemId = b->insertItem(m_pSdiCloseButton,-1,b->count());
 		connect(m_pSdiCloseButton,SIGNAL(destroyed()),this,SLOT(sdiCloseButtonDestroyed()));
