@@ -39,7 +39,7 @@ QString KviOptionsWidget::m_szBasicTipStart; // empty decl
 QString KviOptionsWidget::m_szBasicTipEnd;
 
 KviOptionsWidget::KviOptionsWidget(QWidget * parent,const char * name,bool bSunken)
-: QFrame(parent,name)
+: QFrame(parent,name), KviSelectorInterface()
 {
 	if(m_szBasicTipStart.isEmpty())
 	{
@@ -53,6 +53,7 @@ KviOptionsWidget::KviOptionsWidget(QWidget * parent,const char * name,bool bSunk
 	//if(bSunken)setFrameStyle(QFrame::StyledPanel | QFrame::Sunken);
 	//setFrameStyle(QFrame::StyledPanel | QFrame::Plain);
 	m_pLayout = 0;
+	m_pTabWidget = 0;
 	m_iResetFlags = 0;
 	m_iSelectors = 0;
 	m_pSelectorInterfaceList = new KviPtrList<KviSelectorInterface>;
@@ -83,9 +84,27 @@ void KviOptionsWidget::mergeTip(QWidget * w,const QString &tip)
 
 void KviOptionsWidget::createLayout(int rows,int cols)
 {
+	if(m_pLayout)delete m_pLayout;
 	m_pLayout = new QGridLayout(this,rows,cols,KVI_OPTIONSWIDGET_GRIDLAYOUT_BORDER,KVI_OPTIONSWIDGET_GRIDLAYOUT_SPACE);
 }
 
+void KviOptionsWidget::createTabbedPage()
+{
+	createLayout(1,1);
+	layout()->setMargin(0);
+	layout()->setSpacing(0);
+	m_pTabWidget = new QTabWidget(this);
+	addWidgetToLayout(m_pTabWidget,0,0,0,0);
+}
+
+void KviOptionsWidget::addOptionsWidget(const QString &szText,const QIconSet &iconSet,KviOptionsWidget * pWidget)
+{
+	if(pWidget->layout())
+		pWidget->layout()->setMargin(10);
+	m_pTabWidget->addTab(pWidget,iconSet,szText);
+	m_pSelectorInterfaceList->append(pWidget);
+}
+
 
 void KviOptionsWidget::addWidgetToLayout(QWidget * w,int x1,int y1,int x2,int y2)
 {
@@ -161,6 +180,50 @@ KviFileSelector * KviOptionsWidget::addFileSelector(QWidget * pParent,const QStr
 	return d;
 }
 
+///////////////////////////////////////////////////////////////////////////////////////////////////
+
+
+KviSoundSelector * KviOptionsWidget::addSoundSelector(int x1,int y1,int x2,int y2,const QString & txt,QString *pOption,bool bEnabled)
+{
+	KviSoundSelector * d = new KviSoundSelector(this,txt,pOption,bEnabled);
+	addWidgetToLayout(d,x1,y1,x2,y2);
+	m_pSelectorInterfaceList->append(d);
+	return d;
+}
+
+KviSoundSelector * KviOptionsWidget::addSoundSelector(QWidget * pParent,const QString & txt,QString *pOption,bool bEnabled)
+{
+	KviSoundSelector * d = new KviSoundSelector(pParent,txt,pOption,bEnabled);
+	m_pSelectorInterfaceList->append(d);
+	return d;
+}
+
+KviSoundSelector * KviOptionsWidget::addSoundSelector(int x1,int y1,int x2,int y2,const QString & txt,int optId,bool bEnabled)
+{
+	m_iResetFlags |= (g_stringOptionsTable[optId].flags & KviOption_resetMask);
+	KviSoundSelector * d = addSoundSelector(x1,y1,x2,y2,txt,&(KVI_OPTION_STRING(optId)),bEnabled);
+
+	QString tmp = m_szBasicTipStart;
+	tmp += g_stringOptionsTable[optId].name;
+	tmp += m_szBasicTipEnd;
+	QToolTip::add(d,tmp);
+
+	return d;
+}
+
+KviSoundSelector * KviOptionsWidget::addSoundSelector(QWidget * pParent,const QString & txt,int optId,bool bEnabled)
+{
+	m_iResetFlags |= (g_stringOptionsTable[optId].flags & KviOption_resetMask);
+	KviSoundSelector * d =  addSoundSelector(pParent,txt,&(KVI_OPTION_STRING(optId)),bEnabled);
+
+	QString tmp = m_szBasicTipStart;
+	tmp += g_stringOptionsTable[optId].name;
+	tmp += m_szBasicTipEnd;
+	QToolTip::add(d,tmp);
+
+	return d;
+}
+
 
 ///////////////////////////////////////////////////////////////////////////////////////////////////
 
@@ -425,6 +488,49 @@ KviColorSelector * KviOptionsWidget::addColorSelector(QWidget * pParent,const QS
 	return d;
 }
 
+/////////////////////////////////////////////////////////////////////////////////////////////////
+
+
+KviMircTextColorSelector * KviOptionsWidget::addMircTextColorSelector(int x1,int y1,int x2,int y2,const QString & txt,unsigned int *uFore,unsigned int *uBack,bool bEnabled)
+{
+	KviMircTextColorSelector * s = new KviMircTextColorSelector(this,txt,uFore,uBack,bEnabled);
+	addWidgetToLayout(s,x1,y1,x2,y2);
+	m_pSelectorInterfaceList->append(s);
+	return s;
+}
+
+KviMircTextColorSelector * KviOptionsWidget::addMircTextColorSelector(QWidget * pParent,const QString & txt,unsigned int *uFore,unsigned int *uBack,bool bEnabled)
+{
+	KviMircTextColorSelector * s = new KviMircTextColorSelector(pParent,txt,uFore,uBack,bEnabled);
+	m_pSelectorInterfaceList->append(s);
+	return s;
+}
+
+KviMircTextColorSelector * KviOptionsWidget::addMircTextColorSelector(int x1,int y1,int x2,int y2,const QString & txt,int optForeId,int optBackId,bool bEnabled)
+{
+	m_iResetFlags |= (g_uintOptionsTable[optForeId].flags & KviOption_resetMask);
+	m_iResetFlags |= (g_uintOptionsTable[optBackId].flags & KviOption_resetMask);
+	KviMircTextColorSelector * d = addMircTextColorSelector(x1,y1,x2,y2,txt,&(KVI_OPTION_UINT(optForeId)),&(KVI_OPTION_UINT(optBackId)),bEnabled);
+	QString tmp = m_szBasicTipStart;
+	tmp += g_uintOptionsTable[optForeId].name;
+	tmp += m_szBasicTipEnd;
+	QToolTip::add(d,tmp);
+	return d;
+}
+
+KviMircTextColorSelector * KviOptionsWidget::addMircTextColorSelector(QWidget * pParent,const QString & txt,int optForeId,int optBackId,bool bEnabled)
+{
+	m_iResetFlags |= (g_uintOptionsTable[optForeId].flags & KviOption_resetMask);
+	m_iResetFlags |= (g_uintOptionsTable[optBackId].flags & KviOption_resetMask);
+	KviMircTextColorSelector * d = addMircTextColorSelector(pParent,txt,&(KVI_OPTION_UINT(optForeId)),&(KVI_OPTION_UINT(optBackId)),bEnabled);
+	QString tmp = m_szBasicTipStart;
+	tmp += g_uintOptionsTable[optForeId].name;
+	tmp += m_szBasicTipEnd;
+	QToolTip::add(d,tmp);
+	return d;
+}
+
+
 ////////////////////////////////////////////////////////////////////////////////////////////////
 
 
@@ -523,6 +629,17 @@ QGroupBox * KviOptionsWidget::addGroupBox(int x1,int y1,int x2,int y2,int nStrip
 	return g;
 }
 
+void KviOptionsWidget::addAdvancedButton(int x1,int y1,int x2,int y2)
+{
+	QWidget * w = topLevelWidget();
+	if(!w)return;
+	if(!w->inherits("KviGeneralOptionsDialog"))return;
+
+	QPushButton * b = new QPushButton(__tr2qs("Advanced..."),this);
+	connect(b,SIGNAL(clicked()),this,SLOT(switchToAdvancedPage()));
+	addWidgetToLayout(b,x1,y1,x2,y2);
+}
+
 
 void KviOptionsWidget::commitSelectors()
 {
@@ -549,4 +666,12 @@ void KviOptionsWidget::commit()
 	commitOptionsReset();
 }
 
+void KviOptionsWidget::switchToAdvancedPage()
+{
+	// FIXME: what happens if we're toplevel ????
+	// (so nobody listens to this signal....)
+
+	emit wantToSwitchToAdvancedPage(this);
+}
+
 #include "kvi_optionswidget.moc"
