@@ -35,26 +35,31 @@
 #include "kvi_out.h"
 #include "kvi_taskbar.h"
 #include "kvi_config.h"
-#include "kvi_event.h"
-#include "kvi_uparser.h"
 #include "kvi_mirccntrl.h"
 #include "kvi_settings.h"
 #include "kvi_themedlabel.h"
 #include "kvi_useraction.h"
 #include "kvi_parameterlist.h"
 #include "kvi_ircconnection.h"
+#include "kvi_ircconnectionuserinfo.h"
+#include "kvi_sparser.h"
+#include "kvi_ircuserdb.h"
+#include "kvi_mirccntrl.h"
+#include "kvi_toolwindows_container.h"
 
 #ifdef COMPILE_CRYPT_SUPPORT
 	#include "kvi_crypt.h"
 	#include "kvi_cryptcontroller.h"
 #endif
 
+#include "kvi_kvs_eventtriggers.h"
+
 #include <qpixmap.h>
 #include <qsplitter.h>
 #include <qhbox.h>
 #include <qtoolbutton.h>
 #include <qdragobject.h>
-
+#include <qgrid.h> 
 
 KviQuery::KviQuery(KviFrame * lpFrm,KviConsole * lpConsole,const QString &nick)
 : KviWindow(KVI_WINDOW_TYPE_QUERY,lpFrm,nick,lpConsole)
@@ -65,30 +70,39 @@ KviQuery::KviQuery(KviFrame * lpFrm,KviConsole * lpConsole,const QString &nick)
 
 	//m_pTopSplitter = new QSplitter(QSplitter::Horizontal,this,"top_splitter");
 	m_pButtonBox = new QHBox(this,"button_box");
-	KviThemedLabel * l = new KviThemedLabel(m_pButtonBox,"dummy_label");
-	m_pButtonBox->setStretchFactor(l,1);
-	QToolTip::add(l,__tr2qs("What's this?"));
+	m_pLabel = new KviThemedLabel(m_pButtonBox,"query_label");
+	m_pLabel->setAutoHeight(1);
+	updateLabelText();
+	m_pButtonBox->setStretchFactor(m_pLabel,1);
 
 	// The button box on the right
 	//QHBox * box = new QHBox(m_pTopSplitter,"button_box");
+	if(KVI_OPTION_BOOL(KviOption_boolShowExtendedInfoInQueryLabel))
+		m_pButtonGrid= new QGrid(2,Qt::Vertical,m_pButtonBox);
+	else
+		m_pButtonGrid= new QGrid(1,Qt::Vertical,m_pButtonBox);
 
-	m_pListViewButton = createToolButton(m_pButtonBox,"list_view_button",KVI_SMALLICON_HIDELISTVIEW,KVI_SMALLICON_SHOWLISTVIEW,__tr2qs("Show User List"),true);
-	connect(m_pListViewButton,SIGNAL(clicked()),this,SLOT(toggleListView()));
 
-#ifdef COMPILE_CRYPT_SUPPORT
-	createCryptControllerButton(m_pButtonBox);
-#endif
-
-	createTextEncodingButton(m_pButtonBox);
+	createTextEncodingButton(m_pButtonGrid);
 
 	m_pSplitter = new QSplitter(QSplitter::Horizontal,this,"main_splitter");
 	m_pIrcView = new KviIrcView(m_pSplitter,lpFrm,this);
 	connect(m_pIrcView,SIGNAL(rightClicked()),this,SLOT(textViewRightClicked()));
-	m_pUserListView = new KviUserListView(m_pSplitter,connection()->userDataBase(),this,7,__tr2qs("Query Targets"),"user_list_view");
+	//m_pEditorsContainer= new KviToolWindowsContainer(m_pSplitter);
+		
+		
+	m_pListViewButton = new KviWindowToolPageButton(KVI_SMALLICON_HIDELISTVIEW,KVI_SMALLICON_SHOWLISTVIEW,__tr2qs("Show User List"),buttonContainer(),true,"list_view_button");
+	connect(m_pListViewButton,SIGNAL(clicked()),this,SLOT(toggleListView()));
+	
+#ifdef COMPILE_CRYPT_SUPPORT
+	createCryptControllerButton(m_pButtonGrid);
+#endif
+	
+	m_pUserListView = new KviUserListView(m_pSplitter,m_pListViewButton,connection()->userDataBase(),this,7,__tr2qs("Query Targets"),"user_list_view");
 
 	m_pInput   = new KviInput(this,m_pUserListView);
 
-	if(KVI_OPTION_BOOL(KviOption_boolAutoLogQueries))m_pIrcView->startLogging(0);
+	if(KVI_OPTION_BOOL(KviOption_boolAutoLogQueries)) m_pIrcView->startLogging(0);
 	// FIXME: #warning "Maybe tell the user all that we know about the remote end(s)....channels..."
 
 	m_pIrcView->enableDnd(TRUE);
@@ -106,32 +120,183 @@ KviQuery::~KviQuery()
 		connection()->unregisterQuery(this);
 }
 
+void KviQuery::updateLabelText()
+{
+	QString szText=getInfoLabelText();
+	if(szText!=m_pLabel->text())
+	{
+		m_pLabel->setText(szText);
+		QToolTip::add(m_pLabel,getInfoLabelTipText());
+	}
+}
+
+QString KviQuery::getInfoLabelTipText()
+{
+	QString txt;
+	KviIrcUserEntry * e = connection()->userDataBase()->find(m_szName);
+	if(e)
+	{
+		QString tmp;
+		QString szMask;
+		if(e->hasUser()) 
+			szMask+=e->user();
+		else 
+			szMask+="*";
+		szMask+="@";
+		if(e->hasHost()) 
+			szMask+=e->host();
+		else 
+			szMask+="*";
+		tmp+="\n";
+		QString szChans;
+		connection()->getCommonChannels(m_szName,szChans,0);
+		if(console()->connection())
+		{
+		
+			txt = "<html>" \
+					"<body>" \
+						"<table width=\"100%\">";
+		
+			txt +=          "<tr><td bgcolor=\"#303030\">" \
+								"<font color=\"#FFFFFF\"><b>";
+			txt += __tr2qs("Query target:");
+			txt +=              "</b></font>" \
+							"</td></tr>";
+			txt +=          "<tr><td>";
+		
+			if(e->hasRealName())
+				tmp=__tr2qs("%1 is %2 (%3)").arg(m_szName).arg(szMask).arg(KviMircCntrl::stripControlBytes(e->realName()));
+			else
+				tmp=__tr2qs("%1 is %2").arg(m_szName).arg(szMask);
+			
+			tmp.replace('&',"&amp;");
+			tmp.replace('<',"&lt;");
+			tmp.replace('>',"&gt;");
+			
+			txt += tmp;
+			
+			txt +=          "</td></tr>";
+			
+			if(e->hasServer())
+			{
+				txt+="<tr><td>";
+				if(e->hasHops())
+					txt+=__tr2qs("%1 is using irc server: %2 (%3 hops)").arg(m_szName).arg(e->server()).arg(e->hops());
+				else
+					txt+=__tr2qs("%1 is using irc server: %2").arg(m_szName).arg(e->server());
+				txt+="</td></tr>";
+			}
+			
+			if(e->isAway())
+			{
+				txt+="<tr><td>";
+				txt+=__tr2qs("%1 is probably away").arg(m_szName);
+				txt+="</td></tr>";
+			}
+			
+			txt+="<tr><td>";
+			tmp=__tr2qs("Common channels with %1: %2").arg(m_szName).arg(szChans);
+			
+			tmp.replace('&',"&amp;");
+			tmp.replace('<',"&lt;");
+			tmp.replace('>',"&gt;");
+			
+			txt+=tmp;
+			txt +="</td></tr>";	
+			
+			txt += "</table>" \
+				"</body>" \
+			"<html>";
+		} else {
+			txt=__tr2qs("[Dead Query]");
+		}
+	}
+	return txt;
+}
+
+QString KviQuery::getInfoLabelText()
+{
+	QString tmp;
+	if(KVI_OPTION_BOOL(KviOption_boolShowExtendedInfoInQueryLabel))
+	{
+		KviIrcUserEntry * e = connection()->userDataBase()->find(m_szName);
+		if(e)
+		{
+			QString szMask;
+			if(console()->connection())
+			{
+				if(e->hasUser()) 
+					szMask+=e->user();
+				else 
+					szMask+="*";
+				szMask+="@";
+				if(e->hasHost()) 
+					szMask+=e->host();
+				else 
+					szMask+="*";
+				if(e->hasRealName())
+					tmp=__tr2qs("Query with %1!%2 (%3)").arg(m_szName).arg(szMask).arg(KviMircCntrl::stripControlBytes(e->realName()));
+				else
+					tmp=__tr2qs("Query with %1!%2").arg(m_szName).arg(szMask);
+				if(e->hasServer())
+					tmp+=__tr2qs(", using server %1").arg(e->server());
+				if(e->hasHops())
+					tmp+=__tr2qs(" (%1 hops)").arg(e->hops());
+				if(e->isAway())
+					tmp+=__tr2qs(", probably away");
+				tmp+="\n";
+				QString szChans;
+				connection()->getCommonChannels(m_szName,szChans,0);
+				tmp+=__tr2qs("Common channels: %2").arg(szChans);
+			} else {
+				tmp=__tr2qs("[Dead Query]");
+			}
+		}
+	}
+	return tmp;
+}
 void KviQuery::slotDndEvents(const char *file)
 {	
-	TRIGGER_EVENT_1PARAM(KviEvent_OnQueryFileDropped,this,file);
+	KVS_TRIGGER_EVENT_1(KviEvent_OnQueryFileDropped,this,QString(file));
 }
 
 void KviQuery::triggerCreationEvents()
 {
-	TRIGGER_EVENT(KviEvent_OnQueryWindowCreated,this);
+	if(!KVI_OPTION_STRING(KviOption_stringOnNewQueryOpenedSound).isEmpty()) KviKvsScript::run("snd.play $0",0,new KviKvsVariantList(new KviKvsVariant(KVI_OPTION_STRING(KviOption_stringOnNewQueryOpenedSound))));
+	KVS_TRIGGER_EVENT_0(KviEvent_OnQueryWindowCreated,this);
 }
 
 void KviQuery::getBaseLogFileName(KviStr &buffer)
 {
-	if (console()->connection())
-		buffer.sprintf("%s.%s",windowName().latin1(),console()->currentNetworkName());
+	if(console()->connection())
+		buffer.sprintf("%s.%s",windowName().utf8().data(),console()->currentNetworkName().utf8().data());
 	else
-		buffer.sprintf("%s.%u",windowName().latin1(),console()->ircContextId());
+		buffer.sprintf("%s.%u",windowName().utf8().data(),console()->ircContextId());
+}
+
+void KviQuery::getBaseLogFileName(QString &buffer)
+{
+	if(console()->connection())
+	{
+		buffer=windowName();
+		buffer += ".";
+		buffer += console()->currentNetworkName();
+	} else {
+		buffer=windowName();
+		buffer+=".";
+		buffer+=console()->ircContextId();
+	}
 }
 
 void KviQuery::mergeQuery(KviQuery * q)
 {
 	m_pIrcView->appendMessagesFrom(q->m_pIrcView);
+	updateLabelText();
 }
 
 void KviQuery::textViewRightClicked()
 {
-	TRIGGER_EVENT(KviEvent_OnQueryPopupRequest,this);
+	KVS_TRIGGER_EVENT_0(KviEvent_OnQueryPopupRequest,this);
 }
 
 void KviQuery::saveProperties(KviConfig *cfg)
@@ -160,6 +325,7 @@ void KviQuery::notifyTargetChange(const QString &oldNick,const QString &oldUser,
 	output(KVI_OUT_QUERYTRACE,
 		__tr2qs("The target of this query has changed from \r!n\r%Q\r [%Q@\r!h\r%Q\r] to \r!n\r%Q\r [%Q@\r!h\r%Q\r]"),
 		&oldN,&oldU,&oldH,&nick,&user,&host);
+	updateLabelText();
 }
 
 
@@ -174,6 +340,7 @@ void KviQuery::userAction(const QString &nick,const QString &user,const QString
 	} else {
 		m_pUserListView->userAction(nick,user,host,iTemperature);
 	}
+	updateLabelText();
 }
 
 
@@ -181,6 +348,7 @@ void KviQuery::userAction(const QString &nick,unsigned int uActionType)
 {
 	int iTemperature = kvi_getUserActionTemperature(uActionType);
 	m_pUserListView->userAction(nick,iTemperature);
+	updateLabelText();
 }
 
 void KviQuery::userAction(KviIrcMask *user,unsigned int uActionType)
@@ -194,6 +362,7 @@ void KviQuery::userAction(KviIrcMask *user,unsigned int uActionType)
 	} else {
 		m_pUserListView->userAction(user,iTemperature);
 	}
+	updateLabelText();
 }
 
 
@@ -213,9 +382,9 @@ KviUserListEntry * KviQuery::setTarget(const QString &nick,const QString &user,c
 		notifyCommonChannels(nick,user,host,iChans,szChans);
 	}
 
-	TRIGGER_EVENT_3PARAM(KviEvent_OnQueryTargetAdded,this,nick,user,host);
-
-	return e;
+	KVS_TRIGGER_EVENT_3(KviEvent_OnQueryTargetAdded,this,nick,user,host);
+	updateLabelText();
+	return e;	
 }
 
 void KviQuery::notifyCommonChannels(const QString &nick,const QString &user,const QString &host,int iChans,const QString &szChans)
@@ -229,6 +398,7 @@ void KviQuery::notifyCommonChannels(const QString &nick,const QString &user,cons
 		output(KVI_OUT_QUERYTRACE,__tr2qs("No common channels for \r!n\r%Q\r [%Q@\r!h\r%Q\r]"),
 			&nick,user.isEmpty() ? &star : &user,host.isEmpty() ? &star : &host);
 	}
+	updateLabelText();
 }
 
 
@@ -327,6 +497,7 @@ bool KviQuery::nickChange(const QString &oldNick,const QString &newNick)
 	if(!bRet)return false; // ugh!! ?
 	setWindowName(newNick);
 	updateCaption();
+	updateLabelText();
 	return bRet;
 }
 
@@ -361,6 +532,7 @@ void KviQuery::setDeadQuery()
 
 	updateIcon();
 	updateCaption();
+	updateLabelText();
 };
 
 void KviQuery::setAliveQuery()
@@ -371,14 +543,16 @@ void KviQuery::setAliveQuery()
 	context()->unregisterDeadQuery(this);
 	connection()->registerQuery(this);
 	// Update log file name
-	if(m_pIrcView->isLogging())m_pIrcView->startLogging(0);
+	if(m_pIrcView->isLogging()) m_pIrcView->startLogging(0);
 	updateIcon();
 	updateCaption();
+	updateLabelText();
 }
 
 void KviQuery::applyOptions()
 {
 	m_pUserListView->applyOptions();
+	updateLabelText();
 	// this applies options for IrcView and Input and forces the window to relayout
 	KviWindow::applyOptions();
 }
@@ -429,13 +603,13 @@ void KviQuery::ownMessage(const QString &buffer)
 				switch(cryptSessionInfo()->pEngine->encrypt(d,encrypted))
 				{
 					case KviCryptEngine::Encrypted:
-						connection()->sendFmtData("PRIVMSG %s :%s",szName.data(),encrypted.ptr());
+						if(!connection()->sendFmtData("PRIVMSG %s :%s",szName.data(),encrypted.ptr()))return;
 						m_pConsole->outputPrivmsg(this,KVI_OUT_OWNPRIVMSGCRYPTED,
 							QString::null,QString::null,QString::null,buffer,KviConsole::NoNotifications);
 					break;
 					case KviCryptEngine::Encoded:
 					{
-						connection()->sendFmtData("PRIVMSG %s :%s",szName.data(),encrypted.ptr());
+						if(!connection()->sendFmtData("PRIVMSG %s :%s",szName.data(),encrypted.ptr()))return;
 						// ugly ,but we must redecode here
 						QString szRedecoded = decodeText(encrypted.ptr());
 						m_pConsole->outputPrivmsg(this,KVI_OUT_OWNPRIVMSG,
@@ -453,7 +627,7 @@ void KviQuery::ownMessage(const QString &buffer)
 			} else {
 				d++; //eat the escape code
 				QString tmp = buffer.right(buffer.length() - 1);
-				connection()->sendFmtData("PRIVMSG %s :%s",szName.data(),d);
+				if(!connection()->sendFmtData("PRIVMSG %s :%s",szName.data(),d))return;
 				m_pConsole->outputPrivmsg(this,KVI_OUT_OWNPRIVMSG,QString::null,QString::null,QString::null,tmp,KviConsole::NoNotifications);
 				userAction(connection()->currentNickName(),KVI_USERACTION_PRIVMSG);
 				return;
@@ -462,9 +636,7 @@ void KviQuery::ownMessage(const QString &buffer)
 	}
 #endif
 
-
-
-	connection()->sendFmtData("PRIVMSG %s :%s",szName.data(),d);
+	if(!connection()->sendFmtData("PRIVMSG %s :%s",szName.data(),d))return;
 	m_pConsole->outputPrivmsg(this,KVI_OUT_OWNPRIVMSG,QString::null,QString::null,QString::null,buffer,KviConsole::NoNotifications);
 	userAction(connection()->currentNickName(),KVI_USERACTION_PRIVMSG);
 }
@@ -478,10 +650,11 @@ void KviQuery::ownAction(const QString &buffer)
 		QCString szBuffer = encodeText(buffer);
 		if(!szBuffer.data())return;
 		QCString sz = connection()->encodeText(windowName());
-		connection()->sendFmtData("PRIVMSG %s :%cACTION %s%c",
-			sz.data(),0x01,szBuffer.data(),0x01);
-		output(KVI_OUT_ACTION,"\r!n\r%Q\r %Q",&(connection()->currentNickName()),&buffer);
-		m_pUserListView->userAction(connection()->currentNickName().latin1(),KVI_USERACTION_ACTION);
+		if(sz.isEmpty())return;
+		if(!connection()->sendFmtData("PRIVMSG %s :%cACTION %s%c",
+			sz.data(),0x01,szBuffer.data(),0x01))return;
+		output(KVI_OUT_ACTION,"\r!nc\r%Q\r %Q",&(connection()->currentNickName()),&buffer);
+		m_pUserListView->userAction(connection()->currentNickName(),KVI_USERACTION_ACTION);
 	}
 }
 
