@@ -27,6 +27,8 @@
 #define KVI_TASKBARBUTTON_CONTEXTINDICATORHEIGHT 6
 #define KVI_TASKBAR_MIN_WIDTH 120
 
+#define KVI_NUM_STEPS 20
+#define KVI_TIMER_DELAY 18
 
 
 #define _KVI_DEBUG_CHECK_RANGE_
@@ -42,6 +44,7 @@
 #include "kvi_settings.h"
 #include "kvi_channel.h"
 #include "kvi_ircconnection.h"
+#include "kvi_doublebuffer.h"
 
 // FIXME: #warning "The tree taskbar min width should be configurable"
 #include <qtimer.h>
@@ -49,6 +52,7 @@
 #include <qpainter.h>
 #include <qpixmap.h>
 #include <qheader.h>
+#include <qimage.h>
 
 #ifdef COMPILE_PSEUDO_TRANSPARENCY
 	extern QPixmap * g_pShadedChildGlobalDesktopBackground;
@@ -79,6 +83,19 @@ void KviTaskBarBase::updateActivityMeter()
 {
 }
 
+KviTaskBarItem *  KviTaskBarBase::item(int number)
+{
+	KviTaskBarItem * it = firstItem();
+	if(!setIterationPointer(it))return 0;
+
+	while(it && (number > 0))
+	{
+		it = nextItem();
+		number--;
+	}
+	return it;
+}
+
 void KviTaskBarBase::switchWindow(bool bNext,bool bInContextOnly)
 {
 	if(!g_pActiveWindow)return;
@@ -615,6 +632,8 @@ void KviClassicTaskBar::doLayout()
 		{
 			int inRow = ((totCount < btnsInRow ) ? totCount : btnsInRow);
 			theWidth = baseWidth / (inRow ? inRow : 1);
+			if( KVI_OPTION_BOOL(KviOption_boolClassicTaskBarSetMaximumButtonWidth) && (theWidth > KVI_OPTION_UINT(KviOption_uintClassicTaskBarMaximumButtonWidth)) && (orientation() == Horizontal))
+				theWidth = KVI_OPTION_UINT(KviOption_uintClassicTaskBarMaximumButtonWidth);
 			theX     = 0;
 			theY    += m_iButtonHeight;
 			btnInRow = 1;
@@ -679,18 +698,45 @@ inline bool KviClassicTaskBar::setIterationPointer(KviTaskBarItem * it)
 KviTreeTaskBarItem::KviTreeTaskBarItem(QListView * par,KviWindow * wnd)
 : QListViewItem(par) , KviTaskBarItem(wnd)
 {
-//	m_bHighlighted = false;
-//	m_bAltColor = false;
-
+	m_iStepNumber=0;
+	m_bIncreasing=0;
+	m_pInternal=new KviTreeTaskBarItemInternal(this);
+	m_pAnimTimer=new QTimer();
+	QObject::connect( m_pAnimTimer, SIGNAL(timeout()), m_pInternal, SLOT(timerShot()));
+	applyOptions();
 }
 
 KviTreeTaskBarItem::KviTreeTaskBarItem(KviTreeTaskBarItem * par,KviWindow * wnd)
 : QListViewItem(par) , KviTaskBarItem(wnd)
 {
+	m_iStepNumber=0;
+	m_bIncreasing=0;
+	m_pInternal=new KviTreeTaskBarItemInternal(this);
+	m_pAnimTimer=new QTimer();
+	QObject::connect( m_pAnimTimer, SIGNAL(timeout()), m_pInternal, SLOT(timerShot()));
+	applyOptions();
+}
+
+int KviTreeTaskBarItem::calculateColor(int col1,int col2)
+{
+	int result=col1+(col2-col1)/KVI_NUM_STEPS*m_iStepNumber;
+	return result<255 ? result :255;
 }
 
 KviTreeTaskBarItem::~KviTreeTaskBarItem()
 {
+	QListView* pView=listView();
+	if(pView)
+		if(((KviTreeTaskBarListView*)(pView))->m_pPrevItem==this) ((KviTreeTaskBarListView*)(listView()))->m_pPrevItem=0;
+	delete m_pAnimTimer;
+	delete m_pInternal;
+}
+
+void KviTreeTaskBarItem::applyOptions()
+{
+	m_iRedDiff=(KVI_OPTION_COLOR(KviOption_colorTreeTaskBarActiveBackground).red()-KVI_OPTION_COLOR(KviOption_colorTreeTaskBarBackground).red())/KVI_NUM_STEPS;
+	m_iGreenDiff=(KVI_OPTION_COLOR(KviOption_colorTreeTaskBarActiveBackground).green()-KVI_OPTION_COLOR(KviOption_colorTreeTaskBarBackground).green())/KVI_NUM_STEPS;
+	m_iBlueDiff=(KVI_OPTION_COLOR(KviOption_colorTreeTaskBarActiveBackground).blue()-KVI_OPTION_COLOR(KviOption_colorTreeTaskBarBackground).blue())/KVI_NUM_STEPS;
 }
 
 void KviTreeTaskBarItem::captionChanged()
@@ -749,30 +795,48 @@ void KviTreeTaskBarItem::paintBranches(QPainter *p,const QColorGroup &,int w,int
 	((KviTreeTaskBarListView *)listView())->paintEmptyArea(p,QRect(0,y,w,totalHeight() - height()));
 }
 
-void KviTreeTaskBarItem::paintCell(QPainter *p,const QColorGroup &cg,int column,int width,int)
+void KviTreeTaskBarItem::paintCell(QPainter *painter,const QColorGroup &cg,int column,int width,int)
 {
+	KviDoubleBuffer db(width,height());
+	QPixmap * pMemBuffer = db.pixmap();
+	QPainter p(pMemBuffer);
+	
 	if(isSelected())
 	{
-		p->fillRect(0,0,width,height(),KVI_OPTION_COLOR(KviOption_colorTreeTaskBarActiveBackground));
+		p.fillRect(0,0,width,height(),KVI_OPTION_COLOR(KviOption_colorTreeTaskBarActiveBackground));
 	} else {
-#ifdef COMPILE_PSEUDO_TRANSPARENCY
-		if(g_pShadedChildGlobalDesktopBackground)
+		if(!m_iStepNumber)
 		{
-			QPoint pnt = listView()->viewport()->mapToGlobal(QPoint(int(p->worldMatrix().dx()),int(p->worldMatrix().dy())));
-			p->drawTiledPixmap(0,0,width,height(),*g_pShadedChildGlobalDesktopBackground,pnt.x(),pnt.y());
-		} else {
+#ifdef COMPILE_PSEUDO_TRANSPARENCY
+			if(g_pShadedChildGlobalDesktopBackground)
+			{
+				QPoint pnt = listView()->viewport()->mapToGlobal(QPoint(int(painter->worldMatrix().dx()),int(painter->worldMatrix().dy())));
+				p.drawTiledPixmap(0,0,width,height(),*g_pShadedChildGlobalDesktopBackground,pnt.x(),pnt.y());
+			} else {
 #endif
+				p.fillRect(0,0,width,height(),KVI_OPTION_COLOR(KviOption_colorTreeTaskBarBackground));
+#ifdef COMPILE_PSEUDO_TRANSPARENCY
+			}
+#endif		
 			QPixmap * pix = KVI_OPTION_PIXMAP(KviOption_pixmapTreeTaskBarBackground).pixmap();
 			if(pix)
 			{
-				QPoint pnt = listView()->viewportToContents(QPoint(int(p->worldMatrix().dx()),int(p->worldMatrix().dy())));
-				p->drawTiledPixmap(0,0,width,height(),*pix,pnt.x(),pnt.y());
-			} else {
-				p->fillRect(0,0,width,height(),KVI_OPTION_COLOR(KviOption_colorTreeTaskBarBackground));
+				QPoint pnt = listView()->viewportToContents(QPoint(int(painter->worldMatrix().dx()),int(painter->worldMatrix().dy())));
+				//p.drawTiledPixmap(0,0,width,height(),*pix,pnt.x(),pnt.y());
+				debug("%i %i",pnt.x(),pnt.y());
+				p.translate(-pnt.x(),-pnt.y());
+				KviPixmapUtils::drawPixmapWithPainter(&p,pix,(Qt::AlignmentFlags)(KVI_OPTION_UINT(KviOption_uintTreeTaskBarPixmapAlign)),QRect(pnt.x(),pnt.y(),width,height()),listView()->width(),listView()->height());
+				p.translate(pnt.x(),pnt.y());
 			}
-#ifdef COMPILE_PSEUDO_TRANSPARENCY
+		} else {
+			p.fillRect(0,0,width,height(),
+				QColor(KVI_OPTION_COLOR(KviOption_colorTreeTaskBarBackground).red()+m_iRedDiff*m_iStepNumber,
+					KVI_OPTION_COLOR(KviOption_colorTreeTaskBarBackground).green()+m_iGreenDiff*m_iStepNumber,
+					KVI_OPTION_COLOR(KviOption_colorTreeTaskBarBackground).blue()+m_iBlueDiff*m_iStepNumber
+					)
+				);
+			
 		}
-#endif
 	}
 
 	int h = height();
@@ -793,11 +857,11 @@ void KviTreeTaskBarItem::paintCell(QPainter *p,const QColorGroup &cg,int column,
 				QColor cntx = KVI_OPTION_ICCOLOR(m_pWindow->console()->ircContextId() % KVI_NUM_ICCOLOR_OPTIONS);
 				base.setRgb((base.red() + cntx.red()) >> 1,(base.green() + cntx.green()) >> 1,
 					(base.blue() + cntx.blue()) >> 1);
-				p->fillRect(im + 2,yPixmap + 1,14,15,base);
-				draw_frame_helper(p,im + 1,yPixmap,im + 15,yPixmap + 15,base.light(180),base.dark());
+				p.fillRect(im + 2,yPixmap + 1,14,15,base);
+				draw_frame_helper(&p,im + 1,yPixmap,im + 15,yPixmap + 15,base.light(180),base.dark());
 				if(KVI_OPTION_BOOL(KviOption_boolUseTaskBarIcons))
 				{
-					p->drawPixmap(im + 20,yPixmap,*(m_pWindow->myIconPtr()));
+					p.drawPixmap(im + 20,yPixmap,*(m_pWindow->myIconPtr()));
 					cRect.setLeft(cRect.left() + 37);
 				} else {
 					cRect.setLeft(cRect.left() + 20);
@@ -805,13 +869,13 @@ void KviTreeTaskBarItem::paintCell(QPainter *p,const QColorGroup &cg,int column,
 			} else {
 				if(KVI_OPTION_BOOL(KviOption_boolUseTaskBarIcons))
 				{
-					p->drawPixmap(im,yPixmap,*(m_pWindow->myIconPtr()));
+					p.drawPixmap(im,yPixmap,*(m_pWindow->myIconPtr()));
 					cRect.setLeft(cRect.left() + 17);
 				}
 			}
 			QFont f = QFont();
 			f.setBold(true);
-			p->setFont(f);
+			p.setFont(f);
 			if (((KviConsole *)m_pWindow)->isConnected())
 				   szText = ((KviConsole *)m_pWindow)->connection()->currentServerName();
 			else szText = ((KviConsole *)m_pWindow)->statusString();
@@ -822,7 +886,7 @@ void KviTreeTaskBarItem::paintCell(QPainter *p,const QColorGroup &cg,int column,
 			szText = ((KviChannel *)m_pWindow)->nameWithUserFlag();
 			if(KVI_OPTION_BOOL(KviOption_boolUseTaskBarIcons))
 			{
-				p->drawPixmap(im,yPixmap,*(m_pWindow->myIconPtr()));
+				p.drawPixmap(im,yPixmap,*(m_pWindow->myIconPtr()));
 				cRect.setLeft(cRect.left() + 17);
 			}
 		break;
@@ -831,7 +895,7 @@ void KviTreeTaskBarItem::paintCell(QPainter *p,const QColorGroup &cg,int column,
 			szText = m_pWindow->windowName();
 			if(KVI_OPTION_BOOL(KviOption_boolUseTaskBarIcons))
 			{
-				p->drawPixmap(im,yPixmap,*(m_pWindow->myIconPtr()));
+				p.drawPixmap(im,yPixmap,*(m_pWindow->myIconPtr()));
 				cRect.setLeft(cRect.left() + 17);
 			}
 		break;
@@ -839,7 +903,7 @@ void KviTreeTaskBarItem::paintCell(QPainter *p,const QColorGroup &cg,int column,
 			szText = m_pWindow->plainTextCaption();
 			if(KVI_OPTION_BOOL(KviOption_boolUseTaskBarIcons))
 			{
-				p->drawPixmap(im,yPixmap,*(m_pWindow->myIconPtr()));
+				p.drawPixmap(im,yPixmap,*(m_pWindow->myIconPtr()));
 				cRect.setLeft(cRect.left() + 17);
 			}
 		break;
@@ -851,7 +915,7 @@ void KviTreeTaskBarItem::paintCell(QPainter *p,const QColorGroup &cg,int column,
 		unsigned int uActivityTemperature;
 		if(m_pWindow->activityMeter(&uActivityValue,&uActivityTemperature))
 		{
-			p->drawPixmap(cRect.left(),yPixmap,*g_pActivityMeterPixmap,uActivityValue * 5,uActivityTemperature * 16,5,16);
+			p.drawPixmap(cRect.left(),yPixmap,*g_pActivityMeterPixmap,uActivityValue * 5,uActivityTemperature * 16,5,16);
 			cRect.setLeft(cRect.left() + 7);
 		}
 	}
@@ -860,12 +924,12 @@ void KviTreeTaskBarItem::paintCell(QPainter *p,const QColorGroup &cg,int column,
 	{
 		// paint the progress bar
 		int wdth = (m_iProgress * cRect.width()) / 100;
-		p->fillRect(cRect.x(),cRect.y(),wdth,cRect.height(),KVI_OPTION_COLOR(KviOption_colorTreeTaskBarProgress));
+		p.fillRect(cRect.x(),cRect.y(),wdth,cRect.height(),KVI_OPTION_COLOR(KviOption_colorTreeTaskBarProgress));
 	}
 
 	if(isSelected())
 	{
-		p->setPen(KVI_OPTION_COLOR(KviOption_colorTreeTaskBarActiveForeground));
+		p.setPen(KVI_OPTION_COLOR(KviOption_colorTreeTaskBarActiveForeground));
 	} else {
 		int iLevel;
 		switch(m_iHighlightLevel)
@@ -877,7 +941,13 @@ void KviTreeTaskBarItem::paintCell(QPainter *p,const QColorGroup &cg,int column,
 			case 4: iLevel = KviOption_colorTreeTaskBarHighlight4Foreground; break;
 			default: iLevel = KviOption_colorTreeTaskBarHighlight5Foreground; break;
 		}
-		p->setPen(KVI_OPTION_COLOR(iLevel));
+		p.setPen(
+			QColor(
+				calculateColor(KVI_OPTION_COLOR(iLevel).red(),KVI_OPTION_COLOR(KviOption_colorTreeTaskBarActiveForeground).red()),
+				calculateColor(KVI_OPTION_COLOR(iLevel).green(),KVI_OPTION_COLOR(KviOption_colorTreeTaskBarActiveForeground).green()),
+				calculateColor(KVI_OPTION_COLOR(iLevel).blue(),KVI_OPTION_COLOR(KviOption_colorTreeTaskBarActiveForeground).blue())
+				)
+			);
 	}
 
 	if(m_pWindow->isMinimized())
@@ -885,10 +955,13 @@ void KviTreeTaskBarItem::paintCell(QPainter *p,const QColorGroup &cg,int column,
 		QString tmp = QChar('(');
 		tmp += szText;
 		tmp += QChar(')');
-		p->drawText(cRect,AlignLeft | AlignVCenter,tmp,-1,0);
+		p.drawText(cRect,AlignLeft | AlignVCenter,tmp,-1,0);
 	} else {
-		p->drawText(cRect,AlignLeft | AlignVCenter,szText,-1,0);
+		p.drawText(cRect,AlignLeft | AlignVCenter,szText,-1,0);
 	}
+	
+	painter->drawPixmap(0,0,*pMemBuffer,0,0,width,height());
+	//bitBlt(painter->pixmap(),0,0,pMemBuffer,0,0,width,height(),Qt::CopyROP,false);
 }
 
 QString KviTreeTaskBarItem::key(int,bool) const
@@ -898,6 +971,42 @@ QString KviTreeTaskBarItem::key(int,bool) const
 	return ret;
 }
 
+void KviTreeTaskBarItem::timerShot() 
+{
+	if(m_bIncreasing)
+		m_iStepNumber++;
+	else
+		m_iStepNumber--;
+	
+	if((m_iStepNumber>=KVI_NUM_STEPS) && m_bIncreasing)
+	{
+		m_pAnimTimer->stop();
+		m_iStepNumber=KVI_NUM_STEPS; //make shure, that we cannot get out of range
+	} else if((m_iStepNumber<=0) && !m_bIncreasing) {
+		m_pAnimTimer->stop();
+		m_iStepNumber=0; //make shure, that we cannot get out of range
+	}
+	repaint();
+}
+
+void KviTreeTaskBarItem::mouseEnter()
+{
+	if(KVI_OPTION_BOOL(KviOption_boolEnableVisualEffects))
+	{
+		m_bIncreasing=true;
+		if(!m_pAnimTimer->isActive()) m_pAnimTimer->start(KVI_TIMER_DELAY);
+	}
+}
+
+void KviTreeTaskBarItem::mouseLeave()
+{
+	if(KVI_OPTION_BOOL(KviOption_boolEnableVisualEffects))
+	{
+		m_bIncreasing=false;
+		if(!m_pAnimTimer->isActive()) m_pAnimTimer->start(KVI_TIMER_DELAY);
+	}
+}
+
 
 //////////////////////////////////////////////////////////////////////////////////////////////////////////////////
 // KviTreeTaskBarListView
@@ -905,17 +1014,39 @@ QString KviTreeTaskBarItem::key(int,bool) const
 
 
 KviTreeTaskBarListView::KviTreeTaskBarListView(QWidget * par)
-: QListView(par)
+: QListView(par,"ttblistview")
 {
 	//setSorting(0);
 	setShowSortIndicator(true);
 	setItemMargin(2);
+	setFrameShape(NoFrame);
+	viewport()->setMouseTracking(TRUE);
+	m_pPrevItem=0;
+	setHScrollBarMode(QScrollView::AlwaysOff);
 }
 
 KviTreeTaskBarListView::~KviTreeTaskBarListView()
 {
 }
 
+void KviTreeTaskBarListView::contentsMouseMoveEvent ( QMouseEvent * e ) 
+{
+	if(!e) return;
+	KviTreeTaskBarItem* pCur=(KviTreeTaskBarItem*)(itemAt(contentsToViewport(e->pos())));
+	if(pCur!=m_pPrevItem)
+	{
+		if(m_pPrevItem) m_pPrevItem->mouseLeave();
+		if(pCur) pCur->mouseEnter();
+		m_pPrevItem=pCur;
+	}
+}
+void KviTreeTaskBarListView::leaveEvent(QEvent *)
+{
+	if(m_pPrevItem) m_pPrevItem->mouseLeave();
+	m_pPrevItem=0;
+}
+
+
 void KviTreeTaskBarListView::contentsMousePressEvent(QMouseEvent *e)
 {
 	QListViewItem * it = itemAt(contentsToViewport(e->pos()));
@@ -947,17 +1078,18 @@ void KviTreeTaskBarListView::paintEmptyArea(QPainter * p,const QRect &rct)
 		p->drawTiledPixmap(rct.x(),rct.y(),rct.width(),rct.height(),*g_pShadedChildGlobalDesktopBackground,pnt.x(),pnt.y());
 	} else {
 #endif
-		QPixmap * pix = KVI_OPTION_PIXMAP(KviOption_pixmapTreeTaskBarBackground).pixmap();
-		if(pix)
-		{
-			QPoint pnt = viewportToContents(QPoint(rct.x() + int(p->worldMatrix().dx()),rct.y() + int(p->worldMatrix().dy())));
-			p->drawTiledPixmap(rct.x(),rct.y(),rct.width(),rct.height(),*pix,pnt.x(),pnt.y());
-		} else {
-			p->fillRect(rct.x(),rct.y(),rct.width(),rct.height(),KVI_OPTION_COLOR(KviOption_colorTreeTaskBarBackground));
-		}
+		p->fillRect(rct.x(),rct.y(),rct.width(),rct.height(),KVI_OPTION_COLOR(KviOption_colorTreeTaskBarBackground));
 #ifdef COMPILE_PSEUDO_TRANSPARENCY
 	}
 #endif
+	
+	QPixmap * pix = KVI_OPTION_PIXMAP(KviOption_pixmapTreeTaskBarBackground).pixmap();
+	if(pix)
+	{
+		QPoint pnt = viewportToContents(QPoint(rct.x() + int(p->worldMatrix().dx()),rct.y() + int(p->worldMatrix().dy())));
+		//p->drawTiledPixmap(rct.x(),rct.y(),rct.width(),rct.height(),*pix,pnt.x(),pnt.y());
+		KviPixmapUtils::drawPixmapWithPainter(p,pix,(Qt::AlignmentFlags)(KVI_OPTION_UINT(KviOption_uintTreeTaskBarPixmapAlign)),rct,viewport()->width(),viewport()->height(),pnt.x(),pnt.y());
+	}
 }
 
 /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
