@@ -33,7 +33,6 @@
 #include "kvi_defaults.h"
 #include "kvi_iconmanager.h"
 #include "kvi_regusersdb.h"
-#include "kvi_event.h"
 #include "kvi_parameterlist.h"
 #include "kvi_window.h"
 #include "kvi_console.h"
@@ -42,6 +41,11 @@
 #include "kvi_qstring.h"
 #include "kvi_frame.h"
 #include "kvi_mdimanager.h"
+#include "kvi_kvs_eventtriggers.h"
+#include "kvi_toolwindows_container.h"
+#include "kvi_doublebuffer.h"
+#include "kvi_stringconversion.h"
+#include "kvi_ircconnection.h"
 
 #include <qlabel.h>
 #include <qscrollbar.h>
@@ -98,6 +102,7 @@ KviUserListEntry::KviUserListEntry(KviUserListView * parent,const QString &nick,
 	m_iTemperature   = bJoinTimeUnknown ? 0 : KVI_USERACTION_JOIN;
 
 	m_bSelected      = false;
+	
 	recalcSize();
 }
 
@@ -105,32 +110,91 @@ KviUserListEntry::~KviUserListEntry()
 {
 }
 
-
+bool KviUserListEntry::color(QColor& color)
+{
+	//
+	// FIXME: Unused ?
+	//
+	KviRegisteredUser* pRegisteredUser=g_pRegisteredUserDataBase->findMatchingUser(m_szNick,m_pGlobalData->user(),m_pGlobalData->host());
+	if(pRegisteredUser)
+	{
+		if(pRegisteredUser->getBoolProperty("useCustomColor"))
+		{
+			QString szTmp=pRegisteredUser->getProperty("customColor");
+			KviStringConversion::fromString(szTmp,color);
+			return true;
+		}
+	}
+	if(KVI_OPTION_BOOL(KviOption_boolUseDifferentColorForOwnNick) && m_pListView->m_pKviWindow->connection())
+	{
+		if(m_szNick==m_pListView->m_pKviWindow->connection()->currentNickName())
+		{
+			color = KVI_OPTION_COLOR(KviOption_colorUserListViewOwnForeground);
+			return true;
+		}
+	} 
+	if(m_iFlags == 0)
+	{
+		color=KVI_OPTION_COLOR(KviOption_colorUserListViewNormalForeground);
+		return true;
+	} else {
+		color = KVI_OPTION_COLOR((m_iFlags & KVI_USERFLAG_CHANOWNER) ? \
+			KviOption_colorUserListViewChanOwnerForeground : ((m_iFlags & KVI_USERFLAG_CHANADMIN) ? \
+			KviOption_colorUserListViewChanAdminForeground : ((m_iFlags & KVI_USERFLAG_OP) ? \
+			KviOption_colorUserListViewOpForeground : ((m_iFlags & KVI_USERFLAG_HALFOP) ? \
+			KviOption_colorUserListViewHalfOpForeground : ((m_iFlags & KVI_USERFLAG_VOICE) ? \
+			KviOption_colorUserListViewVoiceForeground : KviOption_colorUserListViewUserOpForeground)))));
+			return true;
+	}
+	return true;
+}
 
 void KviUserListEntry::recalcSize()
 {
 	KviAvatar * av = m_pGlobalData->avatar();
-	m_iHeight = m_pListView->m_iFontHeight + 4;
-    if(m_iHeight < 22)m_iHeight = 22;
-	if(av)
+	m_iHeight = m_pListView->m_iFontHeight;
+	if(KVI_OPTION_BOOL(KviOption_boolShowUserChannelIcons) && (m_iHeight < 22))m_iHeight = 22;
+	
+	if(!KVI_OPTION_BOOL(KviOption_boolDisableAvatars))//G&N  2005
 	{
-		m_iHeight += KVI_OPTION_BOOL(KviOption_boolScaleAvatars) ? (int)KVI_OPTION_UINT(KviOption_uintAvatarScaleHeight) : av->pixmap()->height();
-		m_iHeight += 3;
+		if(av)
+		{
+			if( KVI_OPTION_BOOL(KviOption_boolScaleAvatars) &&
+					(
+						!KVI_OPTION_BOOL(KviOption_boolDoNotStretchAvatars) ||
+						(av->pixmap()->width() > KVI_OPTION_UINT(KviOption_uintAvatarScaleWidth)) ||
+						(av->pixmap()->height() > KVI_OPTION_UINT(KviOption_uintAvatarScaleHeight))
+					)
+				)
+			{
+				QPixmap * pix = av->scaledPixmap(KVI_OPTION_UINT(KviOption_uintAvatarScaleWidth), KVI_OPTION_UINT(KviOption_uintAvatarScaleHeight));
+			
+				m_iHeight += pix->height();
+			}
+			else
+			{
+				m_iHeight += av->pixmap()->height();
+			}
+		}
 	}
+	m_iHeight += 3;
 }
 
 ///////////////////////////////////////////////
 
 
 
-KviUserListView::KviUserListView(QWidget * par,KviIrcUserDataBase * db,KviWindow * pWnd,int dictSize,const QString & label_text,const char * name)
-: QWidget(par,name)
+KviUserListView::KviUserListView(QWidget * parent,KviWindowToolPageButton* button,KviIrcUserDataBase * db,KviWindow * pWnd,int dictSize,const QString & label_text,const char * name)
+: KviWindowToolWidget(parent,button,name)
 {
+	setAutoDelete(0);
 	m_pKviWindow  = pWnd;
 	m_pEntryDict  = new QDict<KviUserListEntry>(dictSize,false);
 	m_pEntryDict->setAutoDelete(true);
-	m_pUsersLabel = new QLabel(this);
+
+	m_pUsersLabel = new QLabel(this,"userslabel");
 	QToolTip::add(m_pUsersLabel,label_text);
+
 	m_pViewArea   = new KviUserListViewArea(this);
 	m_pToolTip    = new KviUserListToolTip(this,m_pViewArea);
 	m_pTopItem    = 0;
@@ -150,7 +214,7 @@ KviUserListView::KviUserListView(QWidget * par,KviIrcUserDataBase * db,KviWindow
 	m_iSelectedCount = 0;
 //	setBackgroundMode(QWidget::NoBackground);
 	applyOptions();
-
+	registerSelf();
 }
 
 KviUserListView::~KviUserListView()
@@ -176,7 +240,9 @@ void KviUserListView::emitRightClick()
 		{
 			QString nicks;
 			appendSelectedNicknames(nicks);
-			g_pUserParser->triggerEvent(ev,m_pKviWindow,new KviParameterList(new KviStr(nicks)));
+			KviKvsVariantList vList;
+			vList.append(nicks);
+			KviKvsEventManager::instance()->trigger(ev,m_pKviWindow,&vList);
 		} else {
 			g_pApp->checkSuggestRestoreDefaultScript();
 		}
@@ -199,7 +265,9 @@ void KviUserListView::emitDoubleClick()
 		{
 			QString nicks;
 			appendSelectedNicknames(nicks);
-			g_pUserParser->triggerEvent(ev,m_pKviWindow,new KviParameterList(new KviStr(nicks)));
+			KviKvsVariantList vList;
+			vList.append(nicks);
+			KviKvsEventManager::instance()->trigger(ev,m_pKviWindow,&vList);
 		} else {
 			g_pApp->checkSuggestRestoreDefaultScript();
 		}
@@ -214,6 +282,14 @@ void KviUserListView::updateScrollBarRange()
 
 void KviUserListView::applyOptions()
 {
+/*	if(!KVI_OPTION_BOOL(KviOption_boolDisableUserListLabel)){ 
+		m_pUsersLabel->hide();
+		m_pViewArea->setGeometry(0,0,width(),height());
+	}
+	if(KVI_OPTION_BOOL(KviOption_boolDisableUserListLabel)){ 
+		m_pUsersLabel->show();
+		updateUsersLabel();
+	}*/
 	setFont(KVI_OPTION_FONT(KviOption_fontUserListView));
 	QFontMetrics fm(KVI_OPTION_FONT(KviOption_fontUserListView));
 	m_iFontHeight = fm.lineSpacing();
@@ -229,6 +305,7 @@ void KviUserListView::applyOptions()
 	m_pUsersLabel->setFont(KVI_OPTION_FONT(KviOption_fontUserListView));
 	setMinimumWidth(100);
 	resizeEvent(0); // this will call update() too
+	repaint();
 }
 
 void KviUserListView::enableUpdates(bool bEnable)
@@ -804,6 +881,7 @@ int KviUserListView::flags(const QString &nick)
 		} else { \
 			if(it->m_iFlags & __flag)it->m_iFlags &= ~__flag; \
 		} \
+		updateScrollBarRange(); \
 		insertUserEntry(nick,it); \
 		m_pViewArea->update(); \
 		return true; \
@@ -878,9 +956,6 @@ void KviUserListView::appendSelectedNicknames(QString &buffer)
 	}
 }
 
-
-
-
 bool KviUserListView::partInternal(const QString &nick,bool bRemove)
 {
 	KviUserListEntry * it = m_pEntryDict->find(nick);
@@ -967,11 +1042,8 @@ bool KviUserListView::nickChange(const QString &oldNick,const QString &newNick)
 		it->m_iTemperature += KVI_USERACTION_NICK;
 		if(av)
 		{
-//			if(!it->m_pGlobalData->avatar())
-//			{
-				it->m_pGlobalData->setAvatar(av);
-				avatarChanged(newNick);
-//			} else delete av; //it should never happen anyway
+			it->m_pGlobalData->setAvatar(av);
+			avatarChanged(newNick);
 		}
 		return true;
 	}
@@ -981,19 +1053,22 @@ bool KviUserListView::nickChange(const QString &oldNick,const QString &newNick)
 
 void KviUserListView::updateUsersLabel()
 {
-	KviStr tmp;
-	tmp.sprintf("<nobr><b>[%u]</b>",m_pEntryDict->count());
-	if(m_iChanOwnerCount)tmp.append(KviStr::Format," q:%d",m_iChanOwnerCount);
-	if(m_iChanAdminCount)tmp.append(KviStr::Format," a:%d",m_iChanAdminCount);
-	if(m_iOpCount)tmp.append(KviStr::Format," o:%d",m_iOpCount);
-	if(m_iHalfOpCount)tmp.append(KviStr::Format," h:%d",m_iHalfOpCount);
-	if(m_iVoiceCount)tmp.append(KviStr::Format," v:%d",m_iVoiceCount);
-	if(m_iUserOpCount)tmp.append(KviStr::Format," u:%d",m_iUserOpCount);
-	if(m_ibEntries)tmp.append(KviStr::Format," b:%d",m_ibEntries);
-	if(m_ieEntries)tmp.append(KviStr::Format," e:%d",m_ieEntries);
-	if(m_iIEntries)tmp.append(KviStr::Format," I:%d",m_iIEntries);
-	tmp.append("</nobr>");
-	m_pUsersLabel->setText(tmp.ptr());
+	if(!KVI_OPTION_BOOL(KviOption_boolDisableUserListLabel))//G&N  2005
+	{
+		KviStr tmp;
+		tmp.sprintf("<nobr><b>[%u]</b>",m_pEntryDict->count());
+		if(m_iChanOwnerCount)tmp.append(KviStr::Format," q:%d",m_iChanOwnerCount);
+		if(m_iChanAdminCount)tmp.append(KviStr::Format," a:%d",m_iChanAdminCount);
+		if(m_iOpCount)tmp.append(KviStr::Format," o:%d",m_iOpCount);
+		if(m_iHalfOpCount)tmp.append(KviStr::Format," h:%d",m_iHalfOpCount);
+		if(m_iVoiceCount)tmp.append(KviStr::Format," v:%d",m_iVoiceCount);
+		if(m_iUserOpCount)tmp.append(KviStr::Format," u:%d",m_iUserOpCount);
+		if(m_ibEntries)tmp.append(KviStr::Format," b:%d",m_ibEntries);
+		if(m_ieEntries)tmp.append(KviStr::Format," e:%d",m_ieEntries);
+		if(m_iIEntries)tmp.append(KviStr::Format," I:%d",m_iIEntries);
+		tmp.append("</nobr>");
+		m_pUsersLabel->setText(tmp.ptr());
+	}
 }
 
 // FIXME: this could be done really better
@@ -1053,10 +1128,20 @@ void KviUserListView::partAll()
 
 void KviUserListView::resizeEvent(QResizeEvent *)
 {
-	int hght = m_pUsersLabel->sizeHint().height();
-	if(hght < 16)hght = 16; // at least
-	m_pUsersLabel->setGeometry(0,0,width(),hght);
+	int hght;
+	if(!KVI_OPTION_BOOL(KviOption_boolDisableUserListLabel))//G&N  2005
+	{
+		hght = m_pUsersLabel->sizeHint().height();
+		if(hght < 16)hght = 16; // at least
+		m_pUsersLabel->setGeometry(0,0,width(),hght);
+	}
+	else
+	{ 
+		hght =0;
+	}
+
 	m_pViewArea->setGeometry(0,hght,width(),height() - hght);
+
 	updateScrollBarRange();
 }
 
@@ -1215,13 +1300,12 @@ KviUserListViewArea::KviUserListViewArea(KviUserListView * par)
 {
 	m_pListView = par;
 	setBackgroundMode(QWidget::NoBackground);
-	m_pScrollBar = new QScrollBar(QScrollBar::Vertical,this);
+	m_pScrollBar = new QScrollBar(QScrollBar::Vertical,this,"scrollbar");
 	m_pScrollBar->setRange(0,0);
 	m_pScrollBar->setValue(0);
 	connect(m_pScrollBar,SIGNAL(valueChanged(int)),this,SLOT(scrollBarMoved(int)));
 	m_pScrollBar->setPageStep(height());
 	m_pScrollBar->setLineStep(m_pListView->m_iFontHeight);
-	m_pMemBuffer = new QPixmap();
 	m_iLastScrollBarVal = 0;
 	m_iTopItemOffset = 0;
 	m_bIgnoreScrollBar = false;
@@ -1230,7 +1314,6 @@ KviUserListViewArea::KviUserListViewArea(KviUserListView * par)
 
 KviUserListViewArea::~KviUserListViewArea()
 {
-	delete m_pMemBuffer;
 }
 
 void KviUserListViewArea::scrollBarMoved(int newVal)
@@ -1313,7 +1396,10 @@ void KviUserListViewArea::paintEvent(QPaintEvent *ev)
 
 	//debug("PAINT EVENT %d,%d,%d,%d",r.left(),r.top(),r.width(),r.height());
 
-	QPainter p(m_pMemBuffer);
+	KviDoubleBuffer db(width(),height());
+	QPixmap * pMemBuffer = db.pixmap();
+
+	QPainter p(pMemBuffer);
 	p.setFont(KVI_OPTION_FONT(KviOption_fontUserListView));
 
 #ifdef COMPILE_PSEUDO_TRANSPARENCY
@@ -1323,19 +1409,17 @@ void KviUserListViewArea::paintEvent(QPaintEvent *ev)
 		p.drawTiledPixmap(r.left(),r.top(),r.width(),r.height(),*g_pShadedChildGlobalDesktopBackground,pnt.x(),pnt.y());
 	} else {
 #endif
-
-		QPixmap * pix = KVI_OPTION_PIXMAP(KviOption_pixmapUserListViewBackground).pixmap();
-		if(pix)
-		{
-			p.drawTiledPixmap(r.left(),r.top(),r.width(),r.height(),*pix,r.left(),r.top());
-		} else {
-			p.fillRect(r.left(),r.top(),r.width(),r.height(),KVI_OPTION_COLOR(KviOption_colorUserListViewBackground));
-		}
-
+		p.fillRect(r.left(),r.top(),r.width(),r.height(),KVI_OPTION_COLOR(KviOption_colorUserListViewBackground));
 #ifdef COMPILE_PSEUDO_TRANSPARENCY
 	}
 #endif
 
+	QPixmap *pix=KVI_OPTION_PIXMAP(KviOption_pixmapUserListViewBackground).pixmap();
+	if(pix)
+	{
+		KviPixmapUtils::drawPixmapWithPainter(&p,pix,(Qt::AlignmentFlags)(KVI_OPTION_UINT(KviOption_uintUserListPixmapAlign)),r,width(),height());
+	}
+
 	KviUserListEntry * e = m_pListView->m_pTopItem;
 
 	int theY = KVI_USERLIST_BORDER_WIDTH - m_iTopItemOffset;
@@ -1344,7 +1428,7 @@ void KviUserListViewArea::paintEvent(QPaintEvent *ev)
 
 	bool bShowIcons = KVI_OPTION_BOOL(KviOption_boolShowUserChannelIcons);
 	bool bShowState = KVI_OPTION_BOOL(KviOption_boolShowUserChannelState);
-
+	
 	while(e && theY <= r.bottom())
 	{
 		int bottom = theY + e->m_iHeight;
@@ -1355,25 +1439,53 @@ void KviUserListViewArea::paintEvent(QPaintEvent *ev)
 
 		if(bottom >= r.top())
 		{
-			QColor * pClrFore;
+			QColor * pClrFore = 0;
+			bool bColorAllocated=0; // FIXME: bool is true or false
 			if(e->m_bSelected)
 			{
 				p.fillRect(0,theY,wdth,e->m_iHeight,KVI_OPTION_COLOR(KviOption_colorUserListViewSelectionBackground));
 				pClrFore = &(KVI_OPTION_COLOR(KviOption_colorUserListViewSelectionForeground));
-			} else {
-				if(e->m_iFlags == 0)
+			} else if(KVI_OPTION_BOOL(KviOption_boolUseDifferentColorForOwnNick) && m_pListView->m_pKviWindow->connection())
+			{
+				if(e->m_szNick==m_pListView->m_pKviWindow->connection()->currentNickName())
+					pClrFore = &(KVI_OPTION_COLOR(KviOption_colorUserListViewOwnForeground));
+			} 
+			 
+			if(!pClrFore){
+				// FIXME:
+				//
+				// this is slow... VERY slow when one has a lot of registered users.
+				//   (this is NOT a simple lookup in the user db... it is a mask match)
+				// if we REALLY need to use custom colors for regged users then
+				// they should be updated ONCE and stored (cached) in the KviUserListEntry structure
+				//
+				KviRegisteredUser* pRegisteredUser=g_pRegisteredUserDataBase->findMatchingUser(e->m_szNick,e->m_pGlobalData->user(),e->m_pGlobalData->host());
+				if(pRegisteredUser)
 				{
-					pClrFore = &(KVI_OPTION_COLOR(KviOption_colorUserListViewNormalForeground));
-				} else {
-					pClrFore = &(KVI_OPTION_COLOR((e->m_iFlags & KVI_USERFLAG_CHANOWNER) ? \
-						KviOption_colorUserListViewChanOwnerForeground : ((e->m_iFlags & KVI_USERFLAG_CHANADMIN) ? \
-						KviOption_colorUserListViewChanAdminForeground : ((e->m_iFlags & KVI_USERFLAG_OP) ? \
-						KviOption_colorUserListViewOpForeground : ((e->m_iFlags & KVI_USERFLAG_HALFOP) ? \
-						KviOption_colorUserListViewHalfOpForeground : ((e->m_iFlags & KVI_USERFLAG_VOICE) ? \
-						KviOption_colorUserListViewVoiceForeground : KviOption_colorUserListViewUserOpForeground))))));
+					if(pRegisteredUser->getBoolProperty("useCustomColor"))
+					{
+						pClrFore = new QColor();
+						QString szTmp=pRegisteredUser->getProperty("customColor");
+						KviStringConversion::fromString(szTmp,(*pClrFore));
+						bColorAllocated=1; // bool is true or false
+					}
+				}
+				if(!pClrFore)
+				{
+					if(e->m_iFlags == 0)
+					{
+						pClrFore = &(KVI_OPTION_COLOR(KviOption_colorUserListViewNormalForeground));
+					} else {
+						pClrFore = &(KVI_OPTION_COLOR((e->m_iFlags & KVI_USERFLAG_CHANOWNER) ? \
+							KviOption_colorUserListViewChanOwnerForeground : ((e->m_iFlags & KVI_USERFLAG_CHANADMIN) ? \
+							KviOption_colorUserListViewChanAdminForeground : ((e->m_iFlags & KVI_USERFLAG_OP) ? \
+							KviOption_colorUserListViewOpForeground : ((e->m_iFlags & KVI_USERFLAG_HALFOP) ? \
+							KviOption_colorUserListViewHalfOpForeground : ((e->m_iFlags & KVI_USERFLAG_VOICE) ? \
+							KviOption_colorUserListViewVoiceForeground : KviOption_colorUserListViewUserOpForeground))))));
+					}
 				}
 			}
-
+			
 			int theX = KVI_USERLIST_BORDER_WIDTH + 1;
 	
 			int iAvatarAndTextX = theX;
@@ -1414,20 +1526,36 @@ void KviUserListViewArea::paintEvent(QPaintEvent *ev)
 				iAvatarAndTextX += 1;
 			}
 
-			p.setPen(*pClrFore);
-
-			theY+=2;
-
-			KviAvatar * av = e->m_pGlobalData->avatar();
-			if(av && KVI_OPTION_UINT(KviOption_uintAvatarScaleWidth) && KVI_OPTION_UINT(KviOption_uintAvatarScaleHeight))
+			if(e->globalData()->isAway())
 			{
-				QPixmap * pix = KVI_OPTION_BOOL(KviOption_boolScaleAvatars) ? \
-						av->scaledPixmap(KVI_OPTION_UINT(KviOption_uintAvatarScaleWidth),KVI_OPTION_UINT(KviOption_uintAvatarScaleHeight)) : \
-						av->pixmap();
-				p.drawPixmap(iAvatarAndTextX,theY,*pix);
-				theY += pix->height() + 1;
+				QRgb rgb2 = pClrFore->rgb();
+				QRgb rgb1 = KVI_OPTION_COLOR(KviOption_colorUserListViewAwayForeground).rgb();
+				p.setPen(QColor(((qRed(rgb1)*2) + qRed(rgb2)) / 3,((qGreen(rgb1)*2) + qGreen(rgb2)) / 3,((qBlue(rgb1)*2) + qBlue(rgb2)) / 3));
+			} else {
+				p.setPen(*pClrFore);
+			}
+			theY+=2;
+		
+			if(!KVI_OPTION_BOOL(KviOption_boolDisableAvatars))//G&N  2005
+			{
+				KviAvatar * av = e->m_pGlobalData->avatar();
+				if(av && KVI_OPTION_UINT(KviOption_uintAvatarScaleWidth) && KVI_OPTION_UINT(KviOption_uintAvatarScaleHeight))
+				{
+					QPixmap * pix;
+					if( KVI_OPTION_BOOL(KviOption_boolScaleAvatars) &&
+						(
+							!KVI_OPTION_BOOL(KviOption_boolDoNotStretchAvatars) ||
+							(av->pixmap()->width() > KVI_OPTION_UINT(KviOption_uintAvatarScaleWidth)) ||
+							(av->pixmap()->height() > KVI_OPTION_UINT(KviOption_uintAvatarScaleHeight))
+						)
+					)
+					pix=av->scaledPixmap(KVI_OPTION_UINT(KviOption_uintAvatarScaleWidth),KVI_OPTION_UINT(KviOption_uintAvatarScaleHeight));
+					else 
+						pix=av->pixmap();
+					p.drawPixmap(iAvatarAndTextX,theY,*pix);
+					theY += pix->height() + 1;
+			}
 			}
-
 			if(bShowState)
 			{
 				if(e->m_lastActionTime)
@@ -1501,12 +1629,25 @@ void KviUserListViewArea::paintEvent(QPaintEvent *ev)
 				if(e->m_iFlags != 0)
 				{
 					QPixmap * ico = g_pIconManager->getSmallIcon( \
+											e->globalData()->isAway() ? \
+												( \
+													(e->m_iFlags & KVI_USERFLAG_CHANOWNER) ? \
+													KVI_SMALLICON_CHANOWNERAWAY : ((e->m_iFlags & KVI_USERFLAG_CHANADMIN) ? \
+													KVI_SMALLICON_CHANADMINAWAY : ((e->m_iFlags & KVI_USERFLAG_OP) ? \
+													KVI_SMALLICON_OPAWAY : ((e->m_iFlags & KVI_USERFLAG_HALFOP) ? \
+													KVI_SMALLICON_HALFOPAWAY : ((e->m_iFlags & KVI_USERFLAG_VOICE) ? \
+													KVI_SMALLICON_VOICEAWAY : KVI_SMALLICON_USEROPAWAY))))
+												) \
+											: \
+												( \
 													(e->m_iFlags & KVI_USERFLAG_CHANOWNER) ? \
 													KVI_SMALLICON_CHANOWNER : ((e->m_iFlags & KVI_USERFLAG_CHANADMIN) ? \
 													KVI_SMALLICON_CHANADMIN : ((e->m_iFlags & KVI_USERFLAG_OP) ? \
 													KVI_SMALLICON_OP : ((e->m_iFlags & KVI_USERFLAG_HALFOP) ? \
 													KVI_SMALLICON_HALFOP : ((e->m_iFlags & KVI_USERFLAG_VOICE) ? \
-													KVI_SMALLICON_VOICE : KVI_SMALLICON_USEROP)))));
+													KVI_SMALLICON_VOICE : KVI_SMALLICON_USEROP)))) \
+												) \
+										);
 					p.drawPixmap(theX,theY,*ico);
 				}
 				p.drawText(iAvatarAndTextX,theY,wdth - theX,m_pListView->m_iFontHeight,AlignLeft|AlignVCenter,e->m_szNick);
@@ -1521,6 +1662,7 @@ void KviUserListViewArea::paintEvent(QPaintEvent *ev)
 					p.drawText(iAvatarAndTextX,theY,wdth - theX,m_pListView->m_iFontHeight,AlignLeft|AlignVCenter,e->m_szNick);
 				}
 			}
+			if(bColorAllocated) delete pClrFore;
 		}
 
 		theY = bottom;
@@ -1534,12 +1676,11 @@ void KviUserListViewArea::paintEvent(QPaintEvent *ev)
 	p.drawLine(1,height()-1,wdth,height()-1);
 	p.drawLine(wdth - 1,1,wdth - 1,height());
 
-	bitBlt(this,r.left(),r.top(),m_pMemBuffer,r.left(),r.top(),r.width(),r.height(),Qt::CopyROP,false);
+	bitBlt(this,r.left(),r.top(),pMemBuffer,r.left(),r.top(),r.width(),r.height(),Qt::CopyROP,false);
 }
 
 void KviUserListViewArea::resizeEvent(QResizeEvent *)
 {
-	m_pMemBuffer->resize(width(),height());
 	int iScr = m_pScrollBar->sizeHint().width();
 	m_pScrollBar->setGeometry(width() - iScr,0,iScr,height());
 	m_pScrollBar->setPageStep(height());
@@ -1548,6 +1689,7 @@ void KviUserListViewArea::resizeEvent(QResizeEvent *)
 
 void KviUserListViewArea::mousePressEvent(QMouseEvent *e)
 {
+	setFocus();
 	if(e->button() & LeftButton)
 	{
 		KviUserListEntry * entry = m_pListView->itemAt(e->pos());
@@ -1602,6 +1744,43 @@ void KviUserListViewArea::mousePressEvent(QMouseEvent *e)
 		m_pListView->emitRightClick();
 	}
 }
+void KviUserListViewArea::keyPressEvent( QKeyEvent * e )
+{
+	QString szKey=e->text();
+	if(!szKey.isEmpty())
+	{
+		KviUserListEntry * nick=0;
+		KviUserListEntry * aux = m_pListView->m_pHeadItem;
+		while(aux)
+		{
+			//debug("%s %s %i %s %i",__FILE__,__FUNCTION__,__LINE__,aux->nick().utf8().data(),aux->nick().find(szKey,0,0));
+			if(aux->nick().find(szKey,0,0)==0) 
+			{
+				nick=aux;
+				break;
+			}
+			aux = aux->m_pNext;
+		}
+		if(nick)
+		{
+			bool bThereWasSelection = false;
+			if(m_pListView->m_iSelectedCount > 0)
+			{
+				aux = m_pListView->m_pHeadItem;
+				while(aux)
+				{
+					aux->m_bSelected = false;
+					aux = aux->m_pNext;
+				}
+				bThereWasSelection = true;
+			}
+			nick->m_bSelected = true;
+			m_pListView->m_iSelectedCount = 1;
+			if(!bThereWasSelection)g_pFrame->childWindowSelectionStateChange(m_pListView->m_pKviWindow,true);
+			update();
+		}
+	}
+}
 
 void KviUserListViewArea::mouseDoubleClickEvent(QMouseEvent *e)
 {
