@@ -47,6 +47,11 @@
 #include "kvi_config.h"
 #include "kvi_irccontext.h"
 #include "kvi_console.h"
+#include "kvi_ircconnectionserverinfo.h"
+#include "kvi_mirccntrl.h"
+#include "kvi_toolwindows_container.h"
+#include "kvi_styled_controls.h"
+#include "kvi_kvs_script.h"
 
 #include <qpixmap.h>
 #include <qpopupmenu.h>
@@ -90,18 +95,18 @@ static QPopupMenu * g_pMdiWindowSystemTextEncodingPopup = 0;
 static QPopupMenu * g_pMdiWindowSystemTextEncodingPopupStandard = 0;
 static QPopupMenu * g_pMdiWindowSystemTextEncodingPopupSmart = 0;
 
-unsigned int g_uUniqueWindowId = 1;
+unsigned long int g_uUniqueWindowId = 1;
 
 // FIXME: #warning "Mouse wheel support"
 KviWindow::KviWindow(int type,KviFrame * lpFrm,const QString &name,KviConsole * lpConsole)
 		: QWidget(0)
 {
-	m_szId.setNum(g_uUniqueWindowId);
+	m_uId = g_uUniqueWindowId;
 	g_uUniqueWindowId++;
 
 
 	// FIXME: REMOVE THIS
-	setName(name.latin1());
+	setName(name);
 	// END FIXME
 	m_szName = name;
 
@@ -120,6 +125,8 @@ KviWindow::KviWindow(int type,KviFrame * lpFrm,const QString &name,KviConsole *
 	m_pLastFocusedChild     = 0;
 	m_pTextCodec            = 0; // will be set by loadProperties
 	m_pTextEncodingButton   = 0;
+	m_pHideToolsButton	= 0;
+//	m_pEditorsContainer     = 0;
 
 #ifdef COMPILE_CRYPT_SUPPORT
 
@@ -168,6 +175,15 @@ void KviWindow::setWindowName(const QString &szName)
 	emit windowNameChanged();
 }
 
+void KviWindow::toggleButtonContainer()
+{
+	QFrame *pContainer=buttonContainer();
+	if(pContainer)
+	{
+		pContainer->setHidden(!pContainer->isHidden());
+	}
+}
+
 void KviWindow::setName(const char * name)
 {
 	m_szName = name;
@@ -319,7 +335,7 @@ QTextCodec * KviWindow::defaultTextCodec()
 
 QCString KviWindow::encodeText(const QString &szText)
 {
-	if(!m_pTextCodec)return QCString(szText.latin1());
+	if(!m_pTextCodec)return QCString(szText.utf8().data());
 	return m_pTextCodec->fromUnicode(szText);
 }
 
@@ -360,6 +376,7 @@ const char * KviWindow::m_typeTable[KVI_WINDOW_NUM_TYPES + 1]=
         "scripteditor",
         "scriptobject",
         "userwindow",
+		"debug",
         // <------ NEW TYPES GO HERE!
         "unknown"
 };
@@ -394,7 +411,7 @@ void KviWindow::destroyTaskBarItem()
 
 QToolButton * KviWindow::createToolButton(QWidget * par,const char * nam,int pixon,int pixoff,const QString & tooltip,bool bOn)
 {
-	QToolButton * b = new QToolButton(par,nam);
+	QToolButton * b = new KviStyledToolButton(par,nam);
 	b->setToggleButton(true);
 	b->setUsesBigPixmap(false);
 #if QT_VERSION >= 300
@@ -419,7 +436,7 @@ QToolButton * KviWindow::createToolButton(QWidget * par,const char * nam,int pix
 void KviWindow::createCryptControllerButton(QWidget * par)
 {
 #ifdef COMPILE_CRYPT_SUPPORT
-	m_pCryptControllerButton = createToolButton(par,"crypt_controller_button",KVI_SMALLICON_UNLOCKEDOFF,KVI_SMALLICON_UNLOCKED,__tr2qs("Show Crypto Controller"),false);
+	m_pCryptControllerButton = new KviWindowToolPageButton(KVI_SMALLICON_UNLOCKEDOFF,KVI_SMALLICON_UNLOCKED,__tr2qs("Crypting"),buttonContainer(),false,"crypt_controller_button");
 	connect(m_pCryptControllerButton,SIGNAL(clicked()),this,SLOT(toggleCryptController()));
 #endif // COMPILE_CRYPT_SUPPORT
 }
@@ -445,24 +462,34 @@ const QString & KviWindow::lastLineOfText()
 	return QString::null;
 }
 
+const QString & KviWindow::lastMessageText()
+{
+	if(m_pIrcView)
+		return m_pIrcView->lastMessageText();
+	return QString::null;
+}
+
 // The following three have to be here even if the crypt support is disabled...moc does not support conditional compilations
 void KviWindow::toggleCryptController()
 {
 #ifdef COMPILE_CRYPT_SUPPORT
-	if(m_pCryptController)
+	if(!m_pCryptControllerButton->isOn())
 	{
-		delete m_pCryptController;
-		m_pCryptController = 0;
-		if(!m_pCryptControllerButton)
-			return;
-		if(m_pCryptControllerButton->isOn())
-			m_pCryptControllerButton->setOn(false);
+		if(m_pCryptController)
+		{
+			delete m_pCryptController;
+			m_pCryptController = 0;
+			if(!m_pCryptControllerButton)
+				return;
+			if(m_pCryptControllerButton->isOn())
+				m_pCryptControllerButton->setOn(false);
+		}
 	}
 	else
 	{
 		if(m_pSplitter && m_pInput)
 		{
-			m_pCryptController = new KviCryptController(m_pSplitter,"crypt_controller",this,m_pCryptSessionInfo);
+			m_pCryptController = new KviCryptController(m_pSplitter,m_pCryptControllerButton,"crypt_controller",this,m_pCryptSessionInfo);
 			connect(m_pCryptController,SIGNAL(done()),this,SLOT(cryptControllerFinished()));
 			//setFocusHandlerNoClass(m_pInput,m_pCryptController,"QLineEdit"); //link it!
 			m_pCryptController->show();
@@ -475,12 +502,12 @@ void KviWindow::toggleCryptController()
 #endif // COMPILE_CRYPT_SUPPORT
 }
 
-void KviWindow::cryptControllerFinished()
-{
 #ifdef COMPILE_CRYPT_SUPPORT
+void KviWindow::setCryptSessionInfo(KviCryptSessionInfo * inf)
+{
 	if(m_pCryptSessionInfo)
 		KviCryptController::destroyCryptSessionInfo(&m_pCryptSessionInfo);
-	m_pCryptSessionInfo = m_pCryptController->getNewSessionInfo();
+	m_pCryptSessionInfo = inf;
 	if(m_pCryptSessionInfo)
 	{
 		connect(m_pCryptSessionInfo->pEngine,SIGNAL(destroyed()),this,SLOT(cryptSessionInfoDestroyed()));
@@ -503,6 +530,14 @@ void KviWindow::cryptControllerFinished()
 		if(m_pCryptControllerButton->isOn())
 			m_pCryptControllerButton->setOn(false);
 	}
+}
+#endif // COMPILE_CRYPT_SUPPORT
+
+void KviWindow::cryptControllerFinished()
+{
+#ifdef COMPILE_CRYPT_SUPPORT
+	KviCryptSessionInfo * inf = m_pCryptController->getNewSessionInfo();
+	setCryptSessionInfo(inf);
 	delete m_pCryptController;
 	m_pCryptController = 0;
 #endif
@@ -532,34 +567,41 @@ void KviWindow::listWindowTypes()
 		outputNoFmt(KVI_OUT_SYSTEMMESSAGE,m_typeTable[i]);
 }
 
-void KviWindow::getConfigGroupName(KviStr &buf)
+void KviWindow::getConfigGroupName(QString &buf)
 {
 	buf = typeString();
 }
 
-void KviWindow::getDefaultLogFileName(KviStr &buffer)
+
+void KviWindow::getDefaultLogFileName(QString &buffer)
 {
 	// FIXME: #warning "Make it configurable ?"
+	QString date;
 	QDate dt(QDate::currentDate());
-	KviStr date(KviStr::Format,"%d.",dt.year());
-	if(dt.month() < 10)date.append('0');
-	date.append(KviStr::Format,"%d.",dt.month());
-	if(dt.day() < 10)date.append('0');
-	date.append(KviStr::Format,"%d",dt.day());
-	KviStr base;
+	date=dt.toString("yyyy.MM.dd");
+	QString base;
 	getBaseLogFileName(base);
 	kvi_encodeFileName(base);
-	base.transliterate(" \\/:*?\"<>|\"","____________"); // just to be sure :D
-	base.toLower();
-	KviStr tmp(KviStr::Format,"%s_%s_%s.log",typeString(),base.ptr(),date.ptr());
-	g_pApp->getLocalKvircDirectory(buffer,KviApp::Log,tmp.ptr());
+	base=base.lower();
+	QString tmp;
+	if(KVI_OPTION_BOOL(KviOption_boolGzipLogs))
+		KviQString::sprintf(tmp,"%s_%s_%s.log.gz",typeString(),base.utf8().data(),date.utf8().data());
+	else
+		KviQString::sprintf(tmp,"%s_%s_%s.log",typeString(),base.utf8().data(),date.utf8().data());
+	g_pApp->getLocalKvircDirectory(buffer,KviApp::Log,tmp);
 }
 
-void KviWindow::getBaseLogFileName(KviStr &buffer)
+/*void KviWindow::getBaseLogFileName(KviStr &buffer)
+{
+	buffer = m_szName;
+}*/
+
+void KviWindow::getBaseLogFileName(QString &buffer)
 {
 	buffer = m_szName;
 }
 
+
 void KviWindow::saveProperties(KviConfig *cfg)
 {
 	// store only the non-default text encoding.
@@ -570,13 +612,28 @@ void KviWindow::saveProperties(KviConfig *cfg)
 	QString szKey = "TextEncoding_";
 	szKey += m_szName;
 	cfg->writeEntry(szKey,szCodec);
+	if(m_pInput) cfg->writeEntry("inputToolButtonsHidden",m_pInput->isButtonsHidden());
+	/*if(m_pIrcView && m_iType==KVI_WINDOW_TYPE_CHANNEL)
+		if(m_pIrcView->isLogging())
+			cfg->writeEntry("LoggingEnabled",m_pIrcView->isLogging());*/
 }
 
 void KviWindow::loadProperties(KviConfig *cfg)
 {
 	QString szKey = "TextEncoding_";
 	szKey += m_szName;
-	setTextEncoding(cfg->readQStringEntry(szKey,QString::null).latin1());
+	setTextEncoding(cfg->readQStringEntry(szKey,QString::null).utf8().data());
+	if(m_pInput) m_pInput->setButtonsHidden(cfg->readBoolEntry("inputToolButtonsHidden",KVI_OPTION_BOOL(KviOption_boolHideInputToolButtons)));
+/*	if(m_pIrcView && m_iType==KVI_WINDOW_TYPE_CHANNEL)
+	{
+		bool bEnableLogs=cfg->readBoolEntry("LoggingEnabled",0);
+		if(!m_pIrcView->isLogging() && bEnableLogs)
+		{
+			QString szTmp;
+			getBaseLogFileName(szTmp);
+			m_pIrcView->startLogging(0);
+		}
+	}*/
 }
 
 QPixmap * KviWindow::myIconPtr()
@@ -650,7 +707,7 @@ void KviWindow::createSystemTextEncodingPopup()
 	else
 	{
 		g_pMdiWindowSystemTextEncodingPopupStandard->clear();
-		disconnect(g_pMdiWindowSystemTextEncodingPopupStandard,SIGNAL(activated(int)),this,SLOT(systemTextEncodingPopupStandardActivated(int)));
+		disconnect(g_pMdiWindowSystemTextEncodingPopupStandard,SIGNAL(activated(int)),0,0);
 	}
 
 	if(!g_pMdiWindowSystemTextEncodingPopupSmart)
@@ -658,7 +715,7 @@ void KviWindow::createSystemTextEncodingPopup()
 	else
 	{
 		g_pMdiWindowSystemTextEncodingPopupSmart->clear();
-		disconnect(g_pMdiWindowSystemTextEncodingPopupSmart,SIGNAL(activated(int)),this,SLOT(systemTextEncodingPopupSmartActivated(int)));
+		disconnect(g_pMdiWindowSystemTextEncodingPopupSmart,SIGNAL(activated(int)),0,0);
 	}
 
 
@@ -773,13 +830,13 @@ void KviWindow::systemTextEncodingPopupStandardActivated(int id)
 
 void KviWindow::savePropertiesAsDefault()
 {
-	KviStr group;
+	QString group;
 	getConfigGroupName(group);
 
-	if(!kvi_strEqualCI(group.ptr(),typeString()))
+	if(!kvi_strEqualCI(group,typeString()))
 	{
 		// save also the settings for THIS specialized window
-		g_pFrame->saveWindowProperties(this,group.ptr());
+		g_pFrame->saveWindowProperties(this,group);
 	}
 
 	g_pFrame->saveWindowProperties(this,typeString());
@@ -933,6 +990,7 @@ void KviWindow::focusInEvent(QFocusEvent *)
 	// we should be already the active window at this point.
 	// If we're not, then run activateSelf() to fix this.
 	if(g_pActiveWindow != this)activateSelf();
+	updateCaption();
 }
 
 bool KviWindow::eventFilter(QObject *o,QEvent *e)
@@ -1182,7 +1240,7 @@ KviWindow * KviWindow::outputProxy()
 void KviWindow::lostUserFocus()
 {
 	if(!m_pIrcView)return;
-	if(m_pIrcView->hasLineMark())m_pIrcView->clearLineMark();
+	if(m_pIrcView->hasLineMark())m_pIrcView->clearLineMark(true);
 }
 
 
@@ -1207,7 +1265,9 @@ void KviWindow::internalOutput(KviIrcView * pView,int msg_type,const kvi_wchar_t
 		if(wnd)wnd->outputNoFmt(msg_type,pText,iFlags);
 	}
 
-	if(!m_pTaskBarItem)return;
+	if(!m_pTaskBarItem) {
+		return;
+	}
 
 	// if this option is checked we dont highlight other than channel msg
 	if(KVI_OPTION_BOOL(KviOption_boolHighlightOnlyNormalMsg))
@@ -1227,7 +1287,8 @@ void KviWindow::internalOutput(KviIrcView * pView,int msg_type,const kvi_wchar_t
 						KVI_OPTION_BOOL(KviOption_boolHighlightOnlyNormalMsgHighlightInChanToo) && (msg_type == KVI_OUT_HIGHLIGHT)
 					)
 				)
-			)return;
+			)
+			return;
 		}
 	}
 
@@ -1242,33 +1303,16 @@ void KviWindow::internalOutput(KviIrcView * pView,int msg_type,const kvi_wchar_t
 
 void KviWindow::output(int msg_type,const char *format,...)
 {
-	kvi_wchar_t txt_ptr[512]; //It should be enough for all outputs...
-	kvi_va_list list;
-	kvi_va_start(list,format);
-	if(kvi_wvsnprintcf(txt_ptr,512,format,list) < 0)
-	{
-		//Just in case...
-		kvi_va_end(list);
-		int len = 512;
-		kvi_wchar_t *long_txt_ptr = 0;
-		int result;
-		do
-		{
-			len += 512;
-			//first time long_txt_ptr == 0 so it is equivalent to malloc
-			//At least the man page says that...
-			long_txt_ptr = (kvi_wchar_t *)kvi_realloc((void *)long_txt_ptr,len * sizeof(kvi_wchar_t));
-			kvi_va_start(list,format);
-			result = kvi_wvsnprintcf(long_txt_ptr,len,format,list);
-			kvi_va_end(list);
-		}while(result < 0);
-		internalOutput(m_pIrcView,msg_type,long_txt_ptr);
-		kvi_free((void *)long_txt_ptr);
-	} else {
-		//Succesful vsnprintf
-		kvi_va_end(list);
-		internalOutput(m_pIrcView,msg_type,txt_ptr);
-	}
+	QString szFmt(format);
+	kvi_va_list l;
+	kvi_va_start(l,format);
+	QString szBuf;
+	KviQString::vsprintf(szBuf,szFmt,l);
+	kvi_va_end(l);
+	preprocessMessage(szBuf);
+	const QChar * pC = KviQString::nullTerminatedArray(szBuf);
+	if(!pC)return;
+	internalOutput(m_pIrcView,msg_type,(kvi_wchar_t *)pC);
 }
 
 void KviWindow::output(int msg_type,const QString &szFmt,...)
@@ -1278,6 +1322,7 @@ void KviWindow::output(int msg_type,const QString &szFmt,...)
 	QString szBuf;
 	KviQString::vsprintf(szBuf,szFmt,l);
 	kvi_va_end(l);
+	preprocessMessage(szBuf);
 	const QChar * pC = KviQString::nullTerminatedArray(szBuf);
 	if(!pC)return;
 	internalOutput(m_pIrcView,msg_type,(kvi_wchar_t *)pC);
@@ -1285,54 +1330,32 @@ void KviWindow::output(int msg_type,const QString &szFmt,...)
 
 void KviWindow::output(int msg_type,const kvi_wchar_t *format,...)
 {
-	kvi_wchar_t txt_ptr[512]; //It should be enough for all outputs...
-	kvi_va_list list;
-	kvi_va_start(list,format);
-	if(kvi_wvsnprintf(txt_ptr,512,format,list) < 0)
-	{
-		//Just in case...
-		kvi_va_end(list);
-		int len = 512;
-		kvi_wchar_t *long_txt_ptr = 0;
-		int result;
-		do
-		{
-			len += 512;
-			//first time long_txt_ptr == 0 so it is equivalent to malloc
-			//At least the man page says that...
-			long_txt_ptr = (kvi_wchar_t *)kvi_realloc((void *)long_txt_ptr,len * sizeof(kvi_wchar_t));
-			kvi_va_start(list,format);
-			result = kvi_wvsnprintf(long_txt_ptr,len,format,list);
-			kvi_va_end(list);
-		}
-		while(result < 0);
-		internalOutput(m_pIrcView,msg_type,long_txt_ptr);
-		kvi_free((void *)long_txt_ptr);
-	}
-	else
-	{
-		//Succesful vsnprintf
-		kvi_va_end(list);
-		internalOutput(m_pIrcView,msg_type,txt_ptr);
-	}
+	QString szFmt=QString::fromUtf8(KviStr(format).ptr());
+	kvi_va_list l;
+	kvi_va_start(l,format);
+	QString szBuf;
+	KviQString::vsprintf(szBuf,szFmt,l);
+	kvi_va_end(l);
+	preprocessMessage(szBuf);
+	const QChar * pC = KviQString::nullTerminatedArray(szBuf);
+	if(!pC)return;
+	internalOutput(m_pIrcView,msg_type,(kvi_wchar_t *)pC);
 }
 
 void KviWindow::outputNoFmt(int msg_type,const char * text,int iFlags)
 {
-	__range_valid(text);
-
-	// convert from Latin1
-	kvi_wchar_t daBuffer[4096];
-	int idx = 0;
-	while(*text && (idx < 4095))daBuffer[idx++] = (unsigned char)(*text++);
-	daBuffer[idx] = 0x0;
-	
-	internalOutput(m_pIrcView,msg_type,daBuffer,iFlags);
+	QString szText(text);
+	preprocessMessage(szText);
+	const QChar * pC = KviQString::nullTerminatedArray(szText);
+	if(!pC)return;
+	internalOutput(m_pIrcView,msg_type,(kvi_wchar_t *)pC,iFlags);
 }
 
 void KviWindow::outputNoFmt(int msg_type,const QString &szText,int iFlags)
 {
-	const QChar * pC = KviQString::nullTerminatedArray(szText);
+	QString szBuf(szText);
+	preprocessMessage(szBuf);
+	const QChar * pC = KviQString::nullTerminatedArray(szBuf);
 	if(!pC)return;
 	internalOutput(m_pIrcView,msg_type,(kvi_wchar_t *)pC,iFlags);
 }
@@ -1343,5 +1366,34 @@ void KviWindow::unhighlight()
 	m_pTaskBarItem->unhighlight();
 }
 
+/* This messes up a bit: for example it breaks the WHOIS output where
+   escapes are already present (checking for them here would be an overkill).
+   This should be eventually done ONLY for remote user message texts
+   in the server parser.
+
+	Fixed
+*/
+
+void KviWindow::preprocessMessage(QString & szMessage)
+{
+	QStringList strings = QStringList::split(" ",szMessage, TRUE);
+	for ( QStringList::Iterator it = strings.begin(); it != strings.end(); ++it ) {
+		QString tmp(*it);
+		if(tmp.contains('\r')) continue;
+		tmp = KviMircCntrl::stripControlBytes(tmp);
+		tmp.stripWhiteSpace();
+		if(m_pConsole)
+			if(m_pConsole->connection())
+				if(m_pConsole->connection()->serverInfo()->supportedChannelTypes().contains(tmp[0]))
+					if((*it)==tmp)
+						*it=QString("\r!c\r%1\r").arg(*it);
+					else
+					{
+						*it=QString("\r!c%1\r%2\r").arg(tmp).arg(*it);
+					}	
+	}
+	szMessage=strings.join(" ");
+}
+
 
 #include "kvi_window.moc"
