@@ -34,239 +34,14 @@
 
 #include <qregexp.h>
 
-/*
-	@doc: operators
-	@title:
-		Operators
-	@keyterms:
-		operator,operators,assignment,assign
-	@type:
-		language
-	@short:
-		Variable operators , assignments & co.
-	@body:
-		Operator constructs are commands just like the other ones.
-		All the operators work on local or global variables and dictionaries.[br]
-		The generic operator syntax is:[br]
-		[br]
-		[b]<left_operand> <operator> [right_operand][/b][br]
-		[br]
-		where <left_operand> and [right_operand] depend on the <operator>.[br]
-		Some operators have no [right_operand] and these are called [b]unary operators[/b]:
-		they operate directly on <left_operand>.[br]
 
-		[br]
-		[big]= (Assignment)[/big][br]
-		[br]
-
-		The assignment is the "plainest" of the binary operators: it works just like in any other programming language.[br]
-		[br]
-		If the <left_operand> is a variable, then the <right_operand> is assigned to it;
-		if the variable doesn't exists yet , it is created.[br]
-		If <right_operand> evaluates to an empty value then the variable is unset.[br]
-		[example]
-		[comment]# Assigning a constant to the global variable %Tmp[/comment]
-		%Tmp = 1
-		[comment]# Assigning a string constant to the global variable %Tmp[/comment]
-		%Tmp = some string
-		[comment]# Assigning a string constant to the local variable %tmp[/comment]
-		%tmp = "some string with whitespace &nbsp; &nbsp; &nbsp; &nbsp; preserved"
-		[comment]# Assigning a variable to another variable copies its contents[/comment]
-		%tmp = %somevariable
-		[comment]# Assigning a variable string to the global variable %Z[/comment]
-		%Z = my eyes are %color
-		[comment]# Assigning a variable string (with a function call inside) to the local variable %x[/comment]
-		%x = the system os is [fnc]$system.osname[/fnc]
-		[comment]# Assigning an empty string to the local variable %y unsets %y[/comment]
-		%y =
-		[comment]# This is equivalent to the above[/comment]
-		%y = ""
-		[comment]# This is equivalent too, if $function evalutates to an empty string[/comment]
-		%y = $function()
-		[/example]
-		[br]
-		If the <left_operand> is a dictionary/array entry, then the <right_operand> is assigned to it;
-		if the dictionary/array entry doesn't exist (or the whole dictionary/array doesn't exists) it is created.[br]
-		If <right_operand> evaluates to an empty value then the dictionary/array entry (and eventually the whole
-		dictionary/array, if there are no other entries) is unset.[br]
-		[example]
-		[comment]# Assigning a variable string to a global dictionary entry[/comment]
-		%Dict[key] = [fnc]$system.osname[/fnc]\ian
-		[comment]# Unsetting a local dictionary entry[/comment]
-		%mydict[23] = ""
-		[/example]
-		[br]
-		If the <left_operand> is an array reference then the semantics depend on the <right_operand> type.
-		If <right_operand> is an array reference then its contents are copied to the <left_operand>.[br]
-		If <right_operand> is a dictionary (keys) reference then its values are copied to to <left_operand>.[br]
-		If <right_operand> is a scalar then the value is assigned to every entry of <left_operand>.[br]
-		This is an easy way of unsetting a whole array: just assign an empty string.[br]
-		If the <left_operand> is a dictionary reference then the semantics depend on the <right_operand> type.
-		If <right_operand> is a dictionary reference then its contents are copied to the <left_operand>.[br]
-		If <right_operand> is an array reference then its contents are copied to to <left_operand> using
-		the array indexes as keys.[br]
-		If <right_operand> is a dictionary key reference then the keys are copied to the <left_operand>
-		using numeric indexes starting from 0 as keys.[br]
-		If <right_operand> is a scalar then the value is assigned to every key of <left_operand>.[br]
-		This is an easy way of unsetting a whole dictionary: just assign an empty string to all its keys.[br]
-		(If you play with huge dictionaries/arrays it might be a good idea to unset them when no longer needed)
-		[example]
-		[comment]# Assigning a dictionary to another: %mydict[] becomes a copy of %anotherdict[][/comment]
-		%mydict[] = %anotherdict[]
-		[comment]# %mydict[] gets the values of the dict returned by $features[/comment]
-		%mydict[] = [fnc]$features[/fnc]
-		[comment]# Assigning a string to ALL the keys of %mydict[/comment]
-		%mydict[] = "some default value"
-		[comment]# Unsetting a whole dictionary[/comment]
-		%mydict[] =
-		%AnotherGlobalDict[] = ""
-		[/example]
-
-
-		[br]
-		[big]=~ (Binding operator)[/big][br]
-		[br]
-
-		This operator is a really ugly, poor and clueless attempt to reach at least 1% of the
-		power of the perl =~ operator :D[br]
-		It allows some complex string operations to be performed efficently by operating directly
-		on the left operand (in fact this is a lot faster in KVIrc since at least one step of parsing is skipped).[br]
-		Its basic syntax is:[br]
-		[b]<left_operand> =~ <operation>[parameters][/b][br]
-		Where <operation> may be one of 't','s' and parameters depend on it.[br]
-		<left_operand> is the target of the <operation>.[br]
-		If <left_operand> is an array or dictionary, the <operation> is executed on each item they contain.[br]
-		Operation 't' is the transliteration.[br]
-		The complete syntax with parameters is:[br]
-		[b]<left_operand> =~ t/<search characters>/<replacement characters>/[/b][br]
-		where <search characters> is a string of characters that are replaced with the corresponding
-		characters in <replacement characters>.[br]
-		This operation can be also named 'y' or 'tr' (to preserve some compatibility with other languages).[br]
-		[example]
-		%A=This is a test string
-		echo %A
-		%A=~ tr/abcdefghi/ABCDEFGHI/
-		echo %A
-		[/example]
-		Operation 's' is the substitution.[br]
-		The complete syntax with parameters is:[br]
-		[b]<left_operand> =~ s/<search pattern>/<replacement pattern>/[flags][/b][br]
-		where <search pattern> is an extended regular expression to be matched in the <left_operand>
-		and <replacement string> is a special pattern that will replace any occurence found.[br]
-		<search pattern> may contain parentheses to capture parts of the matched text.
-		<replacement string> can contain the escape sequences \\N where N is a number between 1 and 9
-		to be replaced by the captured text.[br]
-		(We use \\N because KVIrc will first unquote the string when parsing...)[br]
-		\\0 is a special escape that will be replaced by the entire match (is always valid!).[br]
-		WARNING: the "capture-text" feature is not available if KVIrc has been compiled
-		with qt older than 3.0.0. You can find out if the feature is available by
-		looking for the string "Qt3" in the array returned by [fnc]$features[/fnc].[br]
-		[flags] may be a combination of the letters 'g','i' and 'w'.[br]
-		'g' causes the search to be global and not stop after the first occurence of <search pattern>.[br]
-		'i' causes the search to be case insensitive.[br]
-		'w' causes the search pattern to be interpreted as a simple wildcard regular expression.[br]
-		[example]
-		%A=This is a test string
-		echo %A
-		%A=~ s/([a-z])i([a-z])/\\1I\\2/
-		echo %A
-		%A=~ s/([a-z])i([a-z])/\\1@\\2/gi
-		echo %A
-		[/example]
-
-		[br]
-		[big]X= (Arithmetic Self-operators)[/big][br]
-		[br]
-
-		The general syntax is:[br]
-		[b]<left_operand> <operation> <right_operand>[/b][br]
-		Where <left_operand> and <right_operand> must evaluate to numbers.[br]
-		All these operators perform the operation on <left_operand> and <right_operand> and then
-		store the result in <left_operand> (which therefore must be a variable, an array entry or a dictionary entry).[br]
-		<operation> may be one of:[br]
-		+= : sums the <right_operand> to <left_operand>[br]
-		-= : subtracts <right_operand> from <left_operand>[br]
-		*= : multiplies <left_operand> by <right_operand>[br]
-		%= : calculates <left_operand> modulus <right_operand>[br]
-		|= : calculates <left_operand> bitwise-or <right_operand>[br]
-		&= : calculates <left_operand> bitwise-and <right_operand>[br]
-		/= : divides <left_operand> by <right_operand>[br]
-
-		[br]
-		[big]++ and -- (Increment and Decrement)[/big][br]
-		[br]
-
-		These two operators work only on numeric operands.[br]
-		The general syntax is:[br]
-		[b]<left_operand> <operator>[/b][br]
-		There is no <right_operand>.[br]
-		++ increments <left_operand> by one, -- decrements <left_operand> by one.[br]
-		These are equivalent to += 1 and -= 1.[br]
-
-		[br]
-		[big].= , << , <+ , <, (String concatenation operators)[/big][br]
-		[br]
-
-		All these operators work also on whole arrays and dictionaries.[br]
-		Operator [b].=[/b] : APPENDS the <right_operand> to the <left_operand>[br]
-		Operator [b]<+[/b] is a synonim for .= (backward compatibility)[br]
-		Operator [b]<<[/b] : appends <right_operand> to <left_operand>
-		separating the two strings by a single space if and only if <left_operand> and <right_operand>
-		are non-empty.[br]
-		Operator [b]<,[/b] : is similar to '<<' ; appends , separating with a single ',' with the same condition.[br]
-
-	@examples:
-		First set the variable %var
-		[example]
-		%var = Ciao ciao
-		[/example]
-		Then append a nickname...
-		[example]
-		%var << Pragma
-		[/example]
-		%var now contains "Ciao ciao Pragma"[br]
-		Append a '!' character
-		[example]
-		%var <+ !
-		[/example]
-		%var now contains "Ciao ciao Pragma!"
-		Now reset it.
-		[example]
-		%var =
-		[/example]
-		Now %var is unset.[br]
-		Reset it with a comma separated list of items
-		[example]
-		%var = Pragma,Diabl0,Arter|o	
-		%var <, MalboroLi
-		[/example]
-		%var now contains "Pragma,Diabl0,Arter|o,MalboroLi"[br]
-		[br]
-		Now a longer example.
-		[example]
-		%var = l
-		[cmd]echo[/cmd] It's name starts with the letter %var!
-		%var &lt;+ inux
-		[cmd]echo[/cmd] Yes , it is %var!
-		%var &lt;&lt; OS
-		[cmd]echo[/cmd] Use %var!
-		%var &lt;, Mac OS
-		[cmd]echo[/cmd] There are two items in this list : %var
-		%var = [fnc]$strlen[/fnc](%var)
-		[cmd]echo[/cmd] And it is %var characters long (including the comma)
-		%var--
-		[cmd]echo[/cmd] Excluding the comma : %var
-		%var+=%var
-		[cmd]echo[/cmd] Now it is doubled : %var
-		%var =
-		[cmd]echo[/cmd] Now the var is unset (empty): (%var) !
-		[/example]
-*/
 
 
 
 bool KviUserParser::parseOperatorAssignment(KviCommand *c,KviDataType * d)
 {
+	return false;
+	/*
 	ENTER_STACK_FRAME(c,"parseOperatorAssignment");
 
 	c->skipSpace();
@@ -553,10 +328,13 @@ parse_simple_rvalue:
 	}
 
 	return c->leaveStackFrame();
+	*/
 }
 
 bool KviUserParser::executeOperatorSelfStringManipulate(KviCommand *c,KviStr * pVariable,char operation,const KviStr &szPar1,const KviStr &szPar2,const KviStr &szFlags)
 {
+	return false;
+	/*
 	ENTER_STACK_FRAME(c,"executeOperatorSelfStringManipulate (=~)");
 
 	if(pVariable == 0)goto end_of_this_stuff;
@@ -572,6 +350,7 @@ bool KviUserParser::executeOperatorSelfStringManipulate(KviCommand *c,KviStr * p
 			bool bGlobal = szFlags.contains('g');
 
 			QRegExp re(szPar1.ptr(),!szFlags.contains('i'),szFlags.contains('w'));
+			re.setMinimal(false); // we want greedy matching!
 			int idx = 0;
 			QString str(pVariable->ptr());
 			while((!str.isEmpty()) && (idx != -1))
@@ -581,6 +360,12 @@ bool KviUserParser::executeOperatorSelfStringManipulate(KviCommand *c,KviStr * p
 				if(idx != -1)
 				{
 					len = re.matchedLength();
+					if(len == 0)
+					{
+						c->warning(__tr2qs("The regular expression matched a zero-length substring: this is not valid in the s/// operator"));
+						c->warning(__tr2qs("Skipping the substitution"));
+						goto end_of_this_stuff;
+					}
 					str.remove(idx,len);
 
 					KviStr szReplaced;
@@ -618,10 +403,13 @@ bool KviUserParser::executeOperatorSelfStringManipulate(KviCommand *c,KviStr * p
 end_of_this_stuff:
 
 	return c->leaveStackFrame();
+	*/
 }
 
 bool KviUserParser::parseOperatorSelfStringManipulate(KviCommand *c,KviDataType * d)
 {
+	return false;
+	/*
 	ENTER_STACK_FRAME(c,"parseOperatorSelfStringManipulate (=~)");
 
 	c->skipSpace();
@@ -649,12 +437,6 @@ bool KviUserParser::parseOperatorSelfStringManipulate(KviCommand *c,KviDataType
 			if(!parseCmdFinalPart(c,szFlags))return false;
 		}
 		break;
-/*
-		case 'u':
-		break;
-		case 'l':
-		break;
-*/
 		default:
 			return c->error(KviError_unknownStringManipulationOperator);
 		break;
@@ -692,10 +474,13 @@ bool KviUserParser::parseOperatorSelfStringManipulate(KviCommand *c,KviDataType
 	}
 
 	return c->leaveStackFrame();
+	*/
 }
 
 bool KviUserParser::parseOperatorAppendString(KviCommand *c,KviOperatorParseStruct *s)
 {
+	return false;
+	/*
 	// strings
 	ENTER_STACK_FRAME(c,"parseOperatorAppendString (.=)");
 
@@ -739,10 +524,13 @@ bool KviUserParser::parseOperatorAppendString(KviCommand *c,KviOperatorParseStru
 	}
 
 	return c->leaveStackFrame();
+	*/
 }
 
 bool KviUserParser::parseOperatorAppendToken(KviCommand *c,KviOperatorParseStruct *s)
 {
+	return false;
+	/*
 	// strings
 	ENTER_STACK_FRAME(c,"parseOperatorAppendToken (<<)");
 
@@ -789,11 +577,14 @@ bool KviUserParser::parseOperatorAppendToken(KviCommand *c,KviOperatorParseStruc
 	}
 
 	return c->leaveStackFrame();
+	*/
 }
 
 
 bool KviUserParser::parseOperatorAppendList(KviCommand *c,KviOperatorParseStruct *s)
 {
+	return false;
+	/*
 	// strings
 	ENTER_STACK_FRAME(c,"parseOperatorAppendList (<,)");
 
@@ -840,10 +631,13 @@ bool KviUserParser::parseOperatorAppendList(KviCommand *c,KviOperatorParseStruct
 	}
 
 	return c->leaveStackFrame();
+	*/
 }
 
 bool KviUserParser::evaluateNumericLeftOperand(KviCommand *c,KviOperatorParseStruct *s,const char * oper)
 {
+	return false;
+	/*
 	ENTER_STACK_FRAME(c,"evaluateNumericLeftOperand");
 
 	switch(s->pDataType->iDataType)
@@ -893,10 +687,13 @@ bool KviUserParser::evaluateNumericLeftOperand(KviCommand *c,KviOperatorParseStr
 		}
 	}
 	return c->leaveStackFrame();
+	*/
 }
 
 bool KviUserParser::evaluateNumericRightOperand(KviCommand *c,KviOperatorParseStruct *s,const char * oper)
 {
+	return false;
+	/*
 	ENTER_STACK_FRAME(c,"evaluateNumericRightOperand");
 	bool bOk;
 	s->iRightOp = s->buffer.toInt(&bOk);
@@ -906,79 +703,104 @@ bool KviUserParser::evaluateNumericRightOperand(KviCommand *c,KviOperatorParseSt
 			oper,s->buffer.ptr());
 	}
 	return c->leaveStackFrame();
+	*/
 }
 
 bool KviUserParser::parseOperatorIncrement(KviCommand *c,KviOperatorParseStruct *s)
 {
+	return false;
+	/*
 	ENTER_STACK_FRAME(c,"parseOperatorIncrement");
 	if(!evaluateNumericLeftOperand(c,s,"++"))return false;
 	s->iLeftOp++;
 	s->pDataType->pVariable->setNum(s->iLeftOp);
 	return c->leaveStackFrame();
+	*/
 }
 
 bool KviUserParser::parseOperatorSelfAddition(KviCommand *c,KviOperatorParseStruct *s)
 {
+	return false;
+	/*
 	ENTER_STACK_FRAME(c,"parseOperatorSelfAddition");
 	if(!evaluateNumericLeftOperand(c,s,"+="))return false;
 	if(!evaluateNumericRightOperand(c,s,"+="))return false;
 	s->pDataType->pVariable->setNum(s->iLeftOp + s->iRightOp);
 	return c->leaveStackFrame();
+	*/
 }
 
 bool KviUserParser::parseOperatorDecrement(KviCommand *c,KviOperatorParseStruct *s)
 {
+	return false;
+	/*
 	ENTER_STACK_FRAME(c,"parseOperatorDecrement");
 	if(!evaluateNumericLeftOperand(c,s,"--"))return false;
 	s->iLeftOp--;
 	s->pDataType->pVariable->setNum(s->iLeftOp);
 	return c->leaveStackFrame();
+	*/
 }
 
 bool KviUserParser::parseOperatorSelfSubtraction(KviCommand *c,KviOperatorParseStruct *s)
 {
+	return false;
+	/*
 	ENTER_STACK_FRAME(c,"parseOperatorSelfSubtraction");
 	if(!evaluateNumericLeftOperand(c,s,"-="))return false;
 	if(!evaluateNumericRightOperand(c,s,"-="))return false;
 	s->pDataType->pVariable->setNum(s->iLeftOp - s->iRightOp);
 	return c->leaveStackFrame();
+	*/
 }
 
 bool KviUserParser::parseOperatorSelfMultiply(KviCommand *c,KviOperatorParseStruct *s)
 {
+	return false;
+	/*
 	ENTER_STACK_FRAME(c,"parseOperatorSelfMultiply");
 	if(!evaluateNumericLeftOperand(c,s,"*="))return false;
 	if(!evaluateNumericRightOperand(c,s,"*="))return false;
 	s->pDataType->pVariable->setNum(s->iLeftOp * s->iRightOp);
 	return c->leaveStackFrame();
+	*/
 }
 
 bool KviUserParser::parseOperatorSelfModulo(KviCommand *c,KviOperatorParseStruct *s)
 {
+	return false;
+	/*
 	ENTER_STACK_FRAME(c,"parseOperatorSelfModulo");
 	if(!evaluateNumericLeftOperand(c,s,"%="))return false;
 	if(!evaluateNumericRightOperand(c,s,"%="))return false;
 	if(s->iRightOp == 0)return c->error(KviError_moduloByZero);
 	else s->pDataType->pVariable->setNum(s->iLeftOp % s->iRightOp);
 	return c->leaveStackFrame();
+	*/
 }
 
 bool KviUserParser::parseOperatorSelfOr(KviCommand *c,KviOperatorParseStruct *s)
 {
+	return false;
+	/*
 	ENTER_STACK_FRAME(c,"parseOperatorSelfOr");
 	if(!evaluateNumericLeftOperand(c,s,"|="))return false;
 	if(!evaluateNumericRightOperand(c,s,"|="))return false;
 	s->pDataType->pVariable->setNum(s->iLeftOp * s->iRightOp);
 	return c->leaveStackFrame();
+	*/
 }
 
 bool KviUserParser::parseOperatorSelfAnd(KviCommand *c,KviOperatorParseStruct *s)
 {
+	return false;
+	/*
 	ENTER_STACK_FRAME(c,"parseOperatorSelfAnd");
 	if(!evaluateNumericLeftOperand(c,s,"&="))return false;
 	if(!evaluateNumericRightOperand(c,s,"&="))return false;
 	s->pDataType->pVariable->setNum(s->iLeftOp & s->iRightOp);
 	return c->leaveStackFrame();
+	*/
 }
 
 //bool KviUserParser::parseOperatorSelfNot(KviCommand *c,KviOperatorParseStruct *s)
@@ -992,12 +814,15 @@ bool KviUserParser::parseOperatorSelfAnd(KviCommand *c,KviOperatorParseStruct *s
 
 bool KviUserParser::parseOperatorSelfDivide(KviCommand *c,KviOperatorParseStruct *s)
 {
+	return false;
+	/*
 	ENTER_STACK_FRAME(c,"parseOperatorSelfDivide");
 	if(!evaluateNumericLeftOperand(c,s,"/="))return false;
 	if(!evaluateNumericRightOperand(c,s,"/="))return false;
 	if(s->iRightOp == 0)return c->error(KviError_divisionByZero);
 	else s->pDataType->pVariable->setNum(s->iLeftOp / s->iRightOp);
 	return c->leaveStackFrame();
+	*/
 }
 /*
 
@@ -1027,6 +852,8 @@ typedef bool (KviUserParser::*parse_operator_func)(KviCommand *,KviOperatorParse
 
 bool KviUserParser::parseOperator(KviCommand *c,KviDataType * d)
 {
+	return false;
+	/*
 	ENTER_STACK_FRAME(c,"parseOperator");
 
 	parse_operator_func func = ((parse_operator_func)0);
@@ -1121,6 +948,6 @@ bool KviUserParser::parseOperator(KviCommand *c,KviDataType * d)
 		if(!parseCmdFinalPart(c,ops.buffer))return false;
 		return (this->*func)(c,&ops) ? c->leaveStackFrame() : false;
 	}
-
+	*/
 	// newer here!
 }
