@@ -30,15 +30,18 @@
 
 void KviUserParser::skipComment(KviCommand *c)
 {
+/*
 	__range_valid(*(c->m_ptr) == '#');
 
 	++(c->m_ptr);
 	while(*(c->m_ptr) && (*(c->m_ptr) != '\n'))++(c->m_ptr);
 	if(*(c->m_ptr) == '\n')++(c->m_ptr);
+*/
 }
 
 bool KviUserParser::skipNormalCommand(KviCommand *c)
 {
+/*
 	ENTER_STACK_FRAME(c,"skipNormalCommand");
 	for(;;){
 		// Now skip all the non interesting chars
@@ -62,10 +65,13 @@ bool KviUserParser::skipNormalCommand(KviCommand *c)
 		}
 	}
 	return c->leaveStackFrame();
+*/
+	return true;
 }
 
 bool KviUserParser::skipCommand(KviCommand *c)
 {
+	/*
 	ENTER_STACK_FRAME(c,"skipCommand");
 	if(*(c->m_ptr) == '{'){
 		if(!skipCommandBlock(c))return false;
@@ -75,10 +81,13 @@ bool KviUserParser::skipCommand(KviCommand *c)
 		// Here c->m_ptr points to the char immediately after the separator...
 	}
 	return c->leaveStackFrame();
+	*/
+	return true;
 }
 
 bool KviUserParser::skipCommandBlock(KviCommand *c)
 {
+	/*
 	ENTER_STACK_FRAME(c,"skipCommandBlock");
 
 	__range_valid(*(c->m_ptr)=='{');
@@ -94,10 +103,13 @@ bool KviUserParser::skipCommandBlock(KviCommand *c)
 	}
 	// We should newer reach this point
 	return c->error(KviError_missingClosingBrace);
+	*/
+	return false;
 }
 
 bool KviUserParser::skipSingleCommand(KviCommand *c)
 {
+	/*
 	ENTER_STACK_FRAME(c,"skipSingleCommand");
 
 	if(*(c->m_ptr) == '#')
@@ -203,10 +215,13 @@ bool KviUserParser::skipSingleCommand(KviCommand *c)
 
 //	return c->error(KviError_unknownCommand);
 	return c->leaveStackFrame();
+	*/
+	return false;
 }
 
 bool KviUserParser::skipLValueCommand(KviCommand *c,bool bStopAtParenthesis)
 {
+	/*
 	ENTER_STACK_FRAME(c,"skipLValueCommand");
 	__range_valid((*(c->m_ptr)=='%') || (*(c->m_ptr) == '$'));
 
@@ -250,10 +265,13 @@ bool KviUserParser::skipLValueCommand(KviCommand *c,bool bStopAtParenthesis)
 	// never here
 
 	return c->error(KviError_internalError);
+	*/
+	return false;
 }
 
 bool KviUserParser::skipSwitches(KviCommand *c)
 {
+	/*
 	ENTER_STACK_FRAME(c,"skipSwitches");
 	// This one 
 	c->skipSpace();
@@ -262,10 +280,13 @@ bool KviUserParser::skipSwitches(KviCommand *c)
 		c->skipSpace();
 	}
 	return c->leaveStackFrame();
+	*/
+	return false;
 }
 
 bool KviUserParser::skipSingleSwitch(KviCommand *c)
 {
+	/*
 	ENTER_STACK_FRAME(c,"skipSingleSwitch");
 
 	__range_valid(*(c->m_ptr) == '-');
@@ -342,11 +363,14 @@ bool KviUserParser::skipSingleSwitch(KviCommand *c)
 	}
 
 	return c->leaveStackFrame();
+	*/
+	return false;
 }
 
 
 bool KviUserParser::skipVariable(KviCommand *c)
 {
+	/*
 	ENTER_STACK_FRAME(c,"skipVariable");
 
 	__range_valid(*c->m_ptr == '%');
@@ -387,6 +411,8 @@ bool KviUserParser::skipVariable(KviCommand *c)
 	}
 
 	return c->leaveStackFrame();
+	*/
+	return false;
 }
 
 
@@ -394,6 +420,7 @@ bool KviUserParser::skipVariable(KviCommand *c)
 
 bool KviUserParser::skipVariableKey(KviCommand *c)
 {
+	/*
 	ENTER_STACK_FRAME(c,"skipVariableKey");
 
 	c->skipSpace();
@@ -457,11 +484,14 @@ bool KviUserParser::skipVariableKey(KviCommand *c)
 	}
 	// Newer arrive here...
 	return c->error(KviError_internalError);
+	*/
+	return false;
 }
 
 
 bool KviUserParser::skipVariableIndex(KviCommand *c)
 {
+	/*
 	ENTER_STACK_FRAME(c,"skipVariableIndex");
 
 	c->skipSpace();
@@ -511,10 +541,13 @@ bool KviUserParser::skipVariableIndex(KviCommand *c)
 	}
 	// Newer arrive here...
 	return c->error(KviError_internalError);
+	*/
+	return false;
 }
 
 bool KviUserParser::skipForCondition(KviCommand *c)
 {
+	/*
 	ENTER_STACK_FRAME(c,"skipForCondition");
 	// strips an expression body that is in the for(*;<condition>;*)form
 	bool inString =false;
@@ -552,10 +585,13 @@ bool KviUserParser::skipForCondition(KviCommand *c)
 	}
 	// newer here...
 	return c->error(KviError_internalError);
+	*/
+	return false;
 }
 
 bool KviUserParser::skipExpressionBody(KviCommand *c)
 {
+	/*
 	ENTER_STACK_FRAME(c,"skipExpressionBody");
 	// skips a thing like '( <expression like for calc> )'
 	// we never skip unterminated expressions (like in popup conditions)
@@ -608,10 +644,13 @@ bool KviUserParser::skipExpressionBody(KviCommand *c)
 	}
 	// newer here...
 	return c->error(KviError_internalError);
+	*/
+	return false;
 }
 
 bool KviUserParser::skipIdentifier(KviCommand *c)
 {
+	/*
 	ENTER_STACK_FRAME(c,"skipIdentifier");
 
 	__range_valid(*c->m_ptr == '$');
@@ -667,10 +706,13 @@ bool KviUserParser::skipIdentifier(KviCommand *c)
 	}
 
 	return c->leaveStackFrame();
+	*/
+	return false;
 }
 
 bool KviUserParser::skipClass(KviCommand *c)
 {
+	/*
 	ENTER_STACK_FRAME(c,"skipClass");
 	c->skipWhiteSpace();
 	if(*(c->m_ptr) != '(')return c->error(KviError_openParenthesisExpected);
@@ -703,10 +745,13 @@ bool KviUserParser::skipClass(KviCommand *c)
 	if(!(*(c->m_ptr)))return c->error(KviError_missingClosingBrace);
 	++(c->m_ptr);
 	return c->leaveStackFrame();
+	*/
+	return false;
 }
 
 bool KviUserParser::skipCallbackCommand(KviCommand *c)
 {
+	/*
 	ENTER_STACK_FRAME(c,"skipCallbackCommand");
 //	if(!skipSwitches(c))return false;
 	c->skipWhiteSpace();
@@ -716,10 +761,13 @@ bool KviUserParser::skipCallbackCommand(KviCommand *c)
 	//if(*(c->m_ptr) != '{')return c->error(KviError_openBraceExpected);
 	if(!skipCommand(c))return false;
 	return c->leaveStackFrame();
+	*/
+	return false;
 }
 
 bool KviUserParser::skipIterationCommand(KviCommand *c)
 {
+	/*
 	ENTER_STACK_FRAME(c,"skipIterationCommand");
 	c->skipWhiteSpace();
 //	if(!skipSwitches(c))return false;
@@ -728,11 +776,14 @@ bool KviUserParser::skipIterationCommand(KviCommand *c)
 	c->skipWhiteSpace();
 	if(!skipCommand(c))return false;
 	return c->leaveStackFrame();
+	*/
+	return false;
 }
 
 
 bool KviUserParser::skipIf(KviCommand *c)
 {
+	/*
 	ENTER_STACK_FRAME(c,"skipIf");
 	c->skipWhiteSpace();
 	if(*(c->m_ptr) != '(')return c->error(KviError_openParenthesisExpected);
@@ -748,10 +799,13 @@ bool KviUserParser::skipIf(KviCommand *c)
 		if(!skipCommand(c))return false;
 	} else c->m_ptr = aux;
 	return c->leaveStackFrame();
+	*/
+	return false;
 }
 
 bool KviUserParser::skipDefpopup(KviCommand *c)
 {
+	/*
 	ENTER_STACK_FRAME(c,"skipDefpopup");
 	c->skipWhiteSpace();
 	if(*(c->m_ptr) != '(')return c->error(KviError_openParenthesisExpected);
@@ -760,11 +814,14 @@ bool KviUserParser::skipDefpopup(KviCommand *c)
 	if(*(c->m_ptr) != '{')return c->error(KviError_openBraceExpected);
 	if(!skipPopupBody(c))return false;
 	return c->leaveStackFrame();
+	*/
+	return false;
 }
 
 
 bool KviUserParser::skipPopupBody(KviCommand *c)
 {
+	/*
 	ENTER_STACK_FRAME(c,"skipPopupBody");
 	__range_valid(*(c->m_ptr) == '{');
 
@@ -851,10 +908,13 @@ bool KviUserParser::skipPopupBody(KviCommand *c)
 	if(!(*(c->m_ptr)))return c->error(KviError_missingClosingBrace);
 	++(c->m_ptr);
 	return c->leaveStackFrame();
+	*/
+	return false;
 }
 
 bool KviUserParser::skipCaseLabel(KviCommand *c)
 {
+	/*
 	ENTER_STACK_FRAME(c,"skipCaseLabel");
 	c->skipWhiteSpace();
 	if(*(c->m_ptr) != '(')return c->error(KviError_openParenthesisExpected);
@@ -867,10 +927,13 @@ bool KviUserParser::skipCaseLabel(KviCommand *c)
 	}
 	if(!skipCommand(c))return false;
 	return c->leaveStackFrame();
+	*/
+	return false;
 }
 
 bool KviUserParser::skipDefaultLabel(KviCommand *c)
 {
+	/*
 	ENTER_STACK_FRAME(c,"skipDefaultLabel");
 	c->skipWhiteSpace();
 	if(*(c->m_ptr) == ':')
@@ -880,10 +943,13 @@ bool KviUserParser::skipDefaultLabel(KviCommand *c)
 	}
 	if(!skipCommand(c))return false;
 	return c->leaveStackFrame();
+	*/
+	return false;
 }
 
 bool KviUserParser::skipSwitch(KviCommand *c)
 {
+	/*
 	ENTER_STACK_FRAME(c,"skipSwitch");
 
 	c->skipWhiteSpace();
@@ -929,11 +995,14 @@ bool KviUserParser::skipSwitch(KviCommand *c)
 	++(c->m_ptr);
 
 	return c->leaveStackFrame();
+	*/
+	return false;
 }
 
 
 bool KviUserParser::skipFor(KviCommand *c)
 {
+	/*
 	ENTER_STACK_FRAME(c,"skipFor");
 	c->skipWhiteSpace();
 	if(*(c->m_ptr) != '(')return c->error(KviError_openParenthesisExpected);
@@ -952,4 +1021,6 @@ bool KviUserParser::skipFor(KviCommand *c)
 	c->skipWhiteSpace();
 	if(!skipCommand(c))return false;
 	return c->leaveStackFrame();
+	*/
+	return false;
 }
