@@ -23,6 +23,7 @@
 //=============================================================================
 #include "actioneditor.h"
 
+#include "kvi_styled_controls.h"
 #include "kvi_iconmanager.h"
 #include "kvi_options.h"
 #include "kvi_locale.h"
@@ -59,6 +60,7 @@
 #include <qtabwidget.h>
 #include <qgroupbox.h>
 #include <qvaluelist.h>
+#include <qlabel.h>
 
 extern KviActionEditorWindow * g_pActionEditorWindow;
 
@@ -233,7 +235,7 @@ KviSingleActionEditor::KviSingleActionEditor(QWidget * par,KviActionEditor * ed)
 	gl = new QGridLayout(tab,12,4,8,4);
 
 
-	m_pNeedsContextCheck = new QCheckBox(__tr2qs("Needs IRC Context"),tab);
+	m_pNeedsContextCheck = new KviStyledCheckBox(__tr2qs("Needs IRC Context"),tab);
 	connect(m_pNeedsContextCheck,SIGNAL(toggled(bool)),this,SLOT(needsContextCheckToggled(bool)));
 	QToolTip::add(m_pNeedsContextCheck,__tr2qs("Check this option if this action should be enabled only when " \
 						"the active window belongs to an irc context"));
@@ -243,7 +245,7 @@ KviSingleActionEditor::KviSingleActionEditor(QWidget * par,KviActionEditor * ed)
 	l->setMinimumWidth(40);
 	gl->addWidget(l,1,0);
 
-	m_pNeedsConnectionCheck = new QCheckBox(__tr2qs("Needs IRC Connection"),tab);
+	m_pNeedsConnectionCheck = new KviStyledCheckBox(__tr2qs("Needs IRC Connection"),tab);
 	connect(m_pNeedsConnectionCheck,SIGNAL(toggled(bool)),this,SLOT(needsConnectionCheckToggled(bool)));
 	QToolTip::add(m_pNeedsConnectionCheck,__tr2qs("Check this option if this action should be enabled only when " \
 						"the active window has an active IRC connection"));
@@ -253,51 +255,51 @@ KviSingleActionEditor::KviSingleActionEditor(QWidget * par,KviActionEditor * ed)
 	l->setMinimumWidth(40);
 	gl->addWidget(l,2,1);
 
-	m_pEnableAtLoginCheck = new QCheckBox(__tr2qs("Enable at Login"),tab);
+	m_pEnableAtLoginCheck = new KviStyledCheckBox(__tr2qs("Enable at Login"),tab);
 	QToolTip::add(m_pEnableAtLoginCheck,__tr2qs("Check this option if this action should be enabled also during " \
 						"the login operations (so when the logical IRC connection hasn't been estabilished yet)"));
 	gl->addMultiCellWidget(m_pEnableAtLoginCheck,2,2,2,3);
 
-	m_pSpecificWindowsCheck = new QCheckBox(__tr2qs("Enable Only in Specified Windows"),tab);
+	m_pSpecificWindowsCheck = new KviStyledCheckBox(__tr2qs("Enable Only in Specified Windows"),tab);
 	connect(m_pSpecificWindowsCheck,SIGNAL(toggled(bool)),this,SLOT(specificWindowsCheckToggled(bool)));
 	QToolTip::add(m_pSpecificWindowsCheck,__tr2qs("Check this option if this action should be enabled only when " \
 						"the active window is of a specified type"));
 	gl->addMultiCellWidget(m_pSpecificWindowsCheck,3,3,0,3);
 
-	m_pWindowConsoleCheck = new QCheckBox(__tr2qs("Enable in Console Windows"),tab);
+	m_pWindowConsoleCheck = new KviStyledCheckBox(__tr2qs("Enable in Console Windows"),tab);
 	QToolTip::add(m_pWindowConsoleCheck,__tr2qs("Check this option if this action should be enabled only when " \
 						"the active window is a console"));
 	connect(m_pWindowConsoleCheck,SIGNAL(toggled(bool)),this,SLOT(channelQueryOrConsoleWindowCheckToggled(bool)));
 	gl->addMultiCellWidget(m_pWindowConsoleCheck,4,4,1,3);
 
-	m_pConsoleOnlyIfUsersSelectedCheck = new QCheckBox(__tr2qs("Only If There Are Selected Users"),tab);
+	m_pConsoleOnlyIfUsersSelectedCheck = new KviStyledCheckBox(__tr2qs("Only If There Are Selected Users"),tab);
 	QToolTip::add(m_pConsoleOnlyIfUsersSelectedCheck,__tr2qs("This will enable the action only if there are " \
 						"selected users in the active window"));
 	gl->addMultiCellWidget(m_pConsoleOnlyIfUsersSelectedCheck,5,5,2,3);
 
-	m_pWindowChannelCheck = new QCheckBox(__tr2qs("Enable in Channel Windows"),tab);
+	m_pWindowChannelCheck = new KviStyledCheckBox(__tr2qs("Enable in Channel Windows"),tab);
 	QToolTip::add(m_pWindowChannelCheck,__tr2qs("Check this option if this action should be enabled only when " \
 						"the active window is a channel"));
 	connect(m_pWindowChannelCheck,SIGNAL(toggled(bool)),this,SLOT(channelQueryOrConsoleWindowCheckToggled(bool)));
 	gl->addMultiCellWidget(m_pWindowChannelCheck,6,6,1,3);
 
-	m_pChannelOnlyIfUsersSelectedCheck = new QCheckBox(__tr2qs("Only If There Are Selected Users"),tab);
+	m_pChannelOnlyIfUsersSelectedCheck = new KviStyledCheckBox(__tr2qs("Only If There Are Selected Users"),tab);
 	QToolTip::add(m_pChannelOnlyIfUsersSelectedCheck,__tr2qs("This will enable the action only if there are " \
 						"selected users in the active window"));
 	gl->addMultiCellWidget(m_pChannelOnlyIfUsersSelectedCheck,7,7,2,3);
 
-	m_pWindowQueryCheck = new QCheckBox(__tr2qs("Enable in Query Windows"),tab);
+	m_pWindowQueryCheck = new KviStyledCheckBox(__tr2qs("Enable in Query Windows"),tab);
 	QToolTip::add(m_pWindowQueryCheck,__tr2qs("Check this option if this action should be enabled only when " \
 						"the active window is a query"));
 	connect(m_pWindowQueryCheck,SIGNAL(toggled(bool)),this,SLOT(channelQueryOrConsoleWindowCheckToggled(bool)));
 	gl->addMultiCellWidget(m_pWindowQueryCheck,8,8,1,3);
 
-	m_pQueryOnlyIfUsersSelectedCheck = new QCheckBox(__tr2qs("Only If There Are Selected Users"),tab);
+	m_pQueryOnlyIfUsersSelectedCheck = new KviStyledCheckBox(__tr2qs("Only If There Are Selected Users"),tab);
 	QToolTip::add(m_pQueryOnlyIfUsersSelectedCheck,__tr2qs("This will enable the action only if there are " \
 						"selected users in the active window"));
 	gl->addMultiCellWidget(m_pQueryOnlyIfUsersSelectedCheck,9,9,2,3);
 
-	m_pWindowDccChatCheck = new QCheckBox(__tr2qs("Enable in DCC Chat Windows"),tab);
+	m_pWindowDccChatCheck = new KviStyledCheckBox(__tr2qs("Enable in DCC Chat Windows"),tab);
 	QToolTip::add(m_pWindowDccChatCheck,__tr2qs("Check this option if this action should be enabled only when " \
 						"the active window is a dcc chat"));
 	gl->addMultiCellWidget(m_pWindowDccChatCheck,10,10,1,3);
@@ -383,7 +385,7 @@ void KviSingleActionEditor::chooseSmallIcon()
 	QString s = d->selectedImage();
 	delete d;
 	if(ret != QDialog::Accepted)return;
-	QPixmap * p = g_pIconManager->getImage(s.latin1());
+	QPixmap * p = g_pIconManager->getImage(s.utf8().data());
 	if(!p)return;
 	m_pSmallIconEdit->setText(s);
 	m_pSmallIconButton->setPixmap(*p);
@@ -397,7 +399,7 @@ void KviSingleActionEditor::chooseBigIcon()
 	QString s = d->selectedImage();
 	delete d;
 	if(ret != QDialog::Accepted)return;
-	QPixmap * p = g_pIconManager->getBigIcon(s.latin1());
+	QPixmap * p = g_pIconManager->getBigIcon(s.utf8().data());
 	if(!p)return;
 	m_pBigIconEdit->setText(s);
 	m_pBigIconButton->setPixmap(*p);
@@ -728,7 +730,7 @@ void KviActionEditor::exportActions()
 	
 	QString szFile;
 	
-	if(!KviFileDialog::askForSaveFileName(szFile,__tr2qs("Choose a Filename - KVIrc"),szName,QString::null,true,true))return;
+	if(!KviFileDialog::askForSaveFileName(szFile,__tr2qs("Choose a Filename - KVIrc"),szName,QString::null,true,true,true))return;
 
 	QString szCode;
 
