@@ -1,9 +1,10 @@
+//=============================================================================
 //
 //   File : libkviconfig.cpp
 //   Creation date : Thu Jan 31 2002 22:50:12 GMT by Szymon Stefanek
 //
 //   This config is part of the KVirc irc client distribution
-//   Copyright (C) 2001 Szymon Stefanek (pragma@kvirc.net)
+//   Copyright (C) 2002-2005 Szymon Stefanek (pragma@kvirc.net)
 //
 //   This program is FREE software. You can redistribute it and/or
 //   modify it under the terms of the GNU General Public License
@@ -19,19 +20,20 @@
 //   along with this program. If not, write to the Free Software Foundation,
 //   Inc. ,59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 //
+//=============================================================================
 
 #include "kvi_module.h"
 #include "kvi_string.h"
 #include "kvi_config.h"
 #include "kvi_fileutils.h"
-#include "kvi_uparser.h"
+//#include "kvi_uparser.h"
 #include "kvi_command.h"
 #include "kvi_app.h"
 #include "kvi_locale.h"
 
-#include <qasciidict.h>
+#include <qdict.h>
 
-static QAsciiDict<KviConfig> * g_pConfigDict = 0;
+static QDict<KviConfig> * g_pConfigDict = 0;
 static int g_iNextConfigId = 0;
 
 /*
@@ -43,7 +45,7 @@ static int g_iNextConfigId = 0;
 	@short:
 		Opens a config file
 	@syntax:
-		$config.open(<filename>[,<flags>])
+		$config.open(<filename:string>[,<flags:string>])
 	@description:
 		Opens a config file.
 		If the file already exists , its contents
@@ -79,52 +81,55 @@ static int g_iNextConfigId = 0;
 		[module:config]Config module documentation[/module]
 */
 
-static bool config_module_fnc_open(KviModule *m,KviCommand *c,KviParameterList * parms,KviStr &buffer)
-{
-	ENTER_STACK_FRAME(c,"config_module_fnc_open");
-	KviStr szFile = parms->safeFirstParam();
-	KviStr szR    = parms->safeNextParam();
 
-	KviConfig::FileMode fm;
-
-	if(szR.contains('r'))
+static bool config_kvs_fnc_open(KviKvsModuleFunctionCall * c)
+{ 
+	QString szFile; 
+	QString szMode;
+	
+	KVSM_PARAMETERS_BEGIN(c)
+		KVSM_PARAMETER("filename",KVS_PT_STRING,0,szFile)
+		KVSM_PARAMETER("mode",KVS_PT_STRING,KVS_PF_OPTIONAL,szMode)
+	KVSM_PARAMETERS_END(c)
+	
+	KviConfig::FileMode fileMode;
+	
+	if(szMode.contains('r'))
 	{
-		if(szR.contains('w'))fm = KviConfig::ReadWrite;
-		else fm = KviConfig::Read;
+		if(szMode.contains('w'))fileMode = KviConfig::ReadWrite;
+		else fileMode = KviConfig::Read;
 	} else {
-		if(szR.contains('w'))fm = KviConfig::Write;
-		else fm = KviConfig::ReadWrite;
+		if(szMode.contains('w'))fileMode = KviConfig::Write;
+		else fileMode = KviConfig::ReadWrite;
 	}
+	
+	KviFileUtils::adjustFilePath(szFile);
+	QString szAbsFile;
+	
+	if(KviFileUtils::isAbsolutePath(szFile))szAbsFile = szFile;
+	else g_pApp->getLocalKvircDirectory(szAbsFile,KviApp::ConfigScripts,szFile,true);
 
-	kvi_adjustFilePath(szFile);
-
-	KviStr szAbsFile;
-
-	if(kvi_isAbsolutePath(szFile))szAbsFile = szFile;
-	else g_pApp->getLocalKvircDirectory(szAbsFile,KviApp::ConfigScripts,szFile.ptr(),true);
-
-
-	QAsciiDictIterator<KviConfig> it(*g_pConfigDict);
+	QDictIterator<KviConfig> it(*g_pConfigDict);
 	while(it.current())
 	{
-		if(kvi_strEqualCI(it.current()->fileName(),szAbsFile.ptr()))
+		if(KviQString::equalCI(it.current()->fileName(),szAbsFile))
 		{
-			buffer.append(KviStr::Format,"%s",it.currentKey());
-			if(it.current()->readOnly() && (fm & KviConfig::Write))
+			c->returnValue()->setString(it.currentKey());
+			if(it.current()->readOnly() && (fileMode & KviConfig::Write))
 			{
 				it.current()->setReadOnly(false);
 			}
-			return c->leaveStackFrame();
+			return true;
 		}
 		++it;
 	}
 
-	KviConfig * cfg = new KviConfig(szAbsFile.ptr(),fm);
+	KviConfig * cfg = new KviConfig(szAbsFile,fileMode);
 	g_iNextConfigId++;
-	KviStr tmp(KviStr::Format,"%d",g_iNextConfigId);
-	g_pConfigDict->insert(tmp.ptr(),cfg);
-	buffer.append(tmp);
-	return c->leaveStackFrame();
+	QString tmp = QString("%1").arg(g_iNextConfigId);
+	g_pConfigDict->insert(tmp,cfg);
+	c->returnValue()->setString(tmp);
+	return true;
 }
 
 /*
@@ -145,30 +150,35 @@ static bool config_module_fnc_open(KviModule *m,KviCommand *c,KviParameterList *
 		[module:config]Config module documentation[/module]
 */
 
-static bool config_module_fnc_id(KviModule *m,KviCommand *c,KviParameterList * parms,KviStr &buffer)
-{
-	ENTER_STACK_FRAME(c,"config_module_fnc_id");
-	KviStr szFile = parms->safeFirstParam();
+static bool config_kvs_fnc_id(KviKvsModuleFunctionCall * c)
+{ 
+	QString szFile; 
+	
+	KVSM_PARAMETERS_BEGIN(c)
+		KVSM_PARAMETER("file",KVS_PT_STRING,0,szFile)
+	KVSM_PARAMETERS_END(c)
+	
+	KviFileUtils::adjustFilePath(szFile);
 
-	kvi_adjustFilePath(szFile);
+	QString szAbsFile;
 
-	KviStr szAbsFile;
+	if(KviFileUtils::isAbsolutePath(szFile))szAbsFile = szFile;
+	else g_pApp->getLocalKvircDirectory(szAbsFile,KviApp::ConfigScripts,szFile,true);
 
-	if(kvi_isAbsolutePath(szFile))szAbsFile = szFile;
-	else g_pApp->getLocalKvircDirectory(szAbsFile,KviApp::ConfigScripts,szFile.ptr(),true);
-
-	QAsciiDictIterator<KviConfig> it(*g_pConfigDict);
+	QDictIterator<KviConfig> it(*g_pConfigDict);
 	while(it.current())
 	{
-		if(kvi_strEqualCI(it.current()->fileName(),szAbsFile.ptr()))
+		if(KviQString::equalCI(it.current()->fileName(),szAbsFile))
 		{
-			buffer.append(KviStr::Format,"%s",it.currentKey());
-			return c->leaveStackFrame();
+			c->returnValue()->setString(it.currentKey());
+			return true;
 		}
+		++it;
 	}
 
-	buffer.append('0');
-	return c->leaveStackFrame();
+	c->returnValue()->setString("0");
+	
+	return true;
 }
 
 /*
@@ -180,7 +190,7 @@ static bool config_module_fnc_id(KviModule *m,KviCommand *c,KviParameterList * p
 	@short:
 		Reads an entry from a config file
 	@syntax:
-		$config.read(<id>,<key>[,<default>])
+		$config.read(<id:string>,<key:string>[,<default:string>])
 	@description:
 		Reads and returns the value associated to <key> in the current section of the config
 		file identified by <id>.[br]
@@ -191,23 +201,27 @@ static bool config_module_fnc_id(KviModule *m,KviCommand *c,KviParameterList * p
 		[module:config]Config module documentation[/module]
 */
 
-static bool config_module_fnc_read(KviModule *m,KviCommand *c,KviParameterList * parms,KviStr &buffer)
-{
-	ENTER_STACK_FRAME(c,"config_module_fnc_read");
-	KviStr szId   = parms->safeFirstParam();
-	KviStr szKey  = parms->safeNextParam();
-	KviStr szDefault = parms->safeNextParam();
-
-	KviConfig * cfg = g_pConfigDict->find(szId.ptr());
+static bool config_kvs_fnc_read(KviKvsModuleFunctionCall * c)
+{ 
+	QString szId; 
+	QString szKey;
+	QString szDefault;
+	
+	KVSM_PARAMETERS_BEGIN(c)
+		KVSM_PARAMETER("id",KVS_PT_STRING,0,szId)
+		KVSM_PARAMETER("key",KVS_PT_STRING,0,szKey)
+		KVSM_PARAMETER("default",KVS_PT_STRING,KVS_PF_OPTIONAL,szDefault)
+	KVSM_PARAMETERS_END(c)
+	
+	KviConfig * cfg = g_pConfigDict->find(szId);
 	
 	if(cfg)
 	{
-		buffer.append(cfg->readEntry(QString::fromUtf8(szKey.ptr()),QString::fromUtf8(szDefault.ptr())));
+		c->returnValue()->setString(cfg->readEntry(szKey,szDefault));
 	} else {
-		c->warning(__tr("The config file with id '%s' is not open"),szId.ptr());
+		c->warning(__tr2qs("The config file with id '%Q' is not open"),&szId);
 	}
-
-	return c->leaveStackFrame();
+	return true;
 }
 
 /*
@@ -227,24 +241,25 @@ static bool config_module_fnc_read(KviModule *m,KviCommand *c,KviParameterList *
 		[module:config]Config module documentation[/module]
 */
 
-static bool config_module_fnc_section(KviModule *m,KviCommand *c,KviParameterList * parms,KviStr &buffer)
-{
-	ENTER_STACK_FRAME(c,"config_module_fnc_section");
-	KviStr szId   = parms->safeFirstParam();
-
-	KviConfig * cfg = g_pConfigDict->find(szId.ptr());
+static bool config_kvs_fnc_section(KviKvsModuleFunctionCall * c)
+{ 
+	QString szId; 
+	
+	KVSM_PARAMETERS_BEGIN(c)
+		KVSM_PARAMETER("id",KVS_PT_STRING,0,szId)
+	KVSM_PARAMETERS_END(c)
+	
+	KviConfig * cfg = g_pConfigDict->find(szId);
 	
 	if(cfg)
 	{
-		buffer.append(cfg->group());
+		c->returnValue()->setString(cfg->group());
 	} else {
-		c->warning(__tr("The config file with id '%s' is not open"),szId.ptr());
+		c->warning(__tr2qs("The config file with id '%Q' is not open"),&szId);
 	}
-
-	return c->leaveStackFrame();
+	return true;
 }
 
-
 /*
 	@doc: config.readonly
 	@type:
@@ -264,22 +279,24 @@ static bool config_module_fnc_section(KviModule *m,KviCommand *c,KviParameterLis
 		[module:config]Config module documentation[/module]
 */
 
-static bool config_module_fnc_readonly(KviModule *m,KviCommand *c,KviParameterList * parms,KviStr &buffer)
-{
-	ENTER_STACK_FRAME(c,"config_module_fnc_readonly");
-	KviStr szId   = parms->safeFirstParam();
-
-	KviConfig * cfg = g_pConfigDict->find(szId.ptr());
+static bool config_kvs_fnc_readonly(KviKvsModuleFunctionCall * c)
+{ 
+	QString szId; 
+	
+	KVSM_PARAMETERS_BEGIN(c)
+		KVSM_PARAMETER("id",KVS_PT_STRING,0,szId)
+	KVSM_PARAMETERS_END(c)
+	
+	KviConfig * cfg = g_pConfigDict->find(szId);
 	
 	if(cfg)
 	{
-		buffer.append(cfg->readOnly() ? '1' : '0');
+		c->returnValue()->setBoolean(cfg->readOnly());
 	} else {
-		c->warning(__tr("The config file with id '%s' is not open"),szId.ptr());
-		buffer.append('0');
+		c->warning(__tr2qs("The config file with id '%Q' is not open"),&szId);
+		c->returnValue()->setInteger(false);
 	}
-
-	return c->leaveStackFrame();
+	return true;
 }
 
 /*
@@ -300,13 +317,21 @@ static bool config_module_fnc_readonly(KviModule *m,KviCommand *c,KviParameterLi
 		[module:config]Config module documentation[/module]
 */
 
-static bool config_module_fnc_filename(KviModule *m,KviCommand *c,KviParameterList * parms,KviStr &buffer)
-{
-	ENTER_STACK_FRAME(c,"config_module_fnc_filename");
-	KviStr szId   = parms->safeFirstParam();
-	KviConfig * cfg = g_pConfigDict->find(szId.ptr());
-	if(cfg)buffer.append(cfg->fileName());
-	return c->leaveStackFrame();
+static bool config_kvs_fnc_filename(KviKvsModuleFunctionCall * c)
+{ 
+	QString szId; 
+	
+	KVSM_PARAMETERS_BEGIN(c)
+		KVSM_PARAMETER("id",KVS_PT_STRING,0,szId)
+	KVSM_PARAMETERS_END(c)
+	
+	KviConfig * cfg = g_pConfigDict->find(szId);
+	
+	if(cfg)
+	{
+		c->returnValue()->setString(cfg->fileName());
+	}
+	return true;
 }
 
 /*
@@ -327,22 +352,25 @@ static bool config_module_fnc_filename(KviModule *m,KviCommand *c,KviParameterLi
 		[module:config]Config module documentation[/module]
 */
 
-static bool config_module_fnc_hassection(KviModule *m,KviCommand *c,KviParameterList * parms,KviStr &buffer)
-{
-	ENTER_STACK_FRAME(c,"config_module_fnc_hassection");
-	KviStr szId   = parms->safeFirstParam();
-	KviStr szSect = parms->safeNextParam();
-
-	KviConfig * cfg = g_pConfigDict->find(szId.ptr());
+static bool config_kvs_fnc_hassection(KviKvsModuleFunctionCall * c)
+{ 
+	QString szId;
+	QString szSect;
+	
+	KVSM_PARAMETERS_BEGIN(c)
+		KVSM_PARAMETER("id",KVS_PT_STRING,0,szId)
+		KVSM_PARAMETER("id",KVS_PT_STRING,0,szSect)
+	KVSM_PARAMETERS_END(c)
+	
+	KviConfig * cfg = g_pConfigDict->find(szId);
 	
 	if(cfg)
 	{
-		buffer.append(cfg->hasGroup(QString::fromUtf8(szSect.ptr())) ? '1' : '0');
+		c->returnValue()->setBoolean(cfg->hasGroup(szSect));
 	} else {
-		c->warning(__tr("The config file with id '%s' is not open"),szId.ptr());
+		c->warning(__tr2qs("The config file with id '%Q' is not open"),&szId);
 	}
-
-	return c->leaveStackFrame();
+	return true;
 }
 
 /*
@@ -362,32 +390,31 @@ static bool config_module_fnc_hassection(KviModule *m,KviCommand *c,KviParameter
 		[module:config]Config module documentation[/module]
 */
 
-static bool config_module_fnc_sectionlist(KviModule *m,KviCommand *c,KviParameterList * parms,KviStr &buffer)
-{
-	ENTER_STACK_FRAME(c,"config_module_fnc_sectionlist");
-	KviStr szId   = parms->safeFirstParam();
-
-	KviConfig * cfg = g_pConfigDict->find(szId.ptr());
+static bool config_kvs_fnc_sectionlist(KviKvsModuleFunctionCall * c)
+{ 
+	QString szId; 
+	
+	KVSM_PARAMETERS_BEGIN(c)
+		KVSM_PARAMETER("id",KVS_PT_STRING,0,szId)
+	KVSM_PARAMETERS_END(c)
+	
+	KviConfig * cfg = g_pConfigDict->find(szId);
 	
-	c->beginListArrayOrDictionaryReturnIdentifier();
-
 	if(cfg)
 	{
 		KviConfigIterator it(*(cfg->dict()));
+		KviKvsArray* pArray = new KviKvsArray();
 		int id=0;
 		while(it.current())
 		{
-//			if(!bFirst)buffer.append(',');
-//			else bFirst = false;
-//			buffer.append(it.currentKey());
-			c->addListArrayOrDictionaryReturnValue(id++,KviStr(it.currentKey()),buffer);
+			pArray->set(id++, new KviKvsVariant(it.currentKey()));
 			++it;
 		}
+		c->returnValue()->setArray(pArray);
 	} else {
-		c->warning(__tr("The config file with id '%s' is not open"),szId.ptr());
+		c->warning(__tr2qs("The config file with id '%Q' is not open"),&szId);
 	}
-
-	return c->leaveStackFrame();
+	return true;
 }
 
 /*
@@ -407,40 +434,38 @@ static bool config_module_fnc_sectionlist(KviModule *m,KviCommand *c,KviParamete
 		[module:config]Config module documentation[/module]
 */
 
-static bool config_module_fnc_keylist(KviModule *m,KviCommand *c,KviParameterList * parms,KviStr &buffer)
-{
-	ENTER_STACK_FRAME(c,"config_module_fnc_keylist");
-	KviStr szId   = parms->safeFirstParam();
-
-	KviConfig * cfg = g_pConfigDict->find(szId.ptr());
-
-	c->beginListArrayOrDictionaryReturnIdentifier();
-
+static bool config_kvs_fnc_keylist(KviKvsModuleFunctionCall * c)
+{ 
+	QString szId; 
+	
+	KVSM_PARAMETERS_BEGIN(c)
+		KVSM_PARAMETER("id",KVS_PT_STRING,0,szId)
+	KVSM_PARAMETERS_END(c)
+	
+	KviConfig * cfg = g_pConfigDict->find(szId);
+	
 	if(cfg)
 	{
 		KviConfigGroup * d = cfg->dict()->find(cfg->group());
-		if(!d)return c->leaveStackFrame();
+		if(!d)return true;
 
 		KviConfigGroupIterator it(*d);
 
-//		bool bFirst = true;
+		KviKvsArray* pArray = new KviKvsArray();
 		int id=0;
+		
 		while(it.current())
 		{
-//			if(!bFirst)buffer.append(',');
-//			else bFirst = false;
-//			buffer.append(it.currentKey());
-			c->addListArrayOrDictionaryReturnValue(id++,KviStr(it.currentKey()),buffer);
+			pArray->set(id++, new KviKvsVariant(it.currentKey()));
 			++it;
 		}
+		c->returnValue()->setArray(pArray);
 	} else {
-		c->warning(__tr("The config file with id '%s' is not open"),szId.ptr());
+		c->warning(__tr2qs("The config file with id '%Q' is not open"),&szId);
 	}
-
-	return c->leaveStackFrame();
+	return true;
 }
 
-
 /*
 	@doc: config.filelist
 	@type:
@@ -457,28 +482,25 @@ static bool config_module_fnc_keylist(KviModule *m,KviCommand *c,KviParameterLis
 		[module:config]Config module documentation[/module]
 */
 
-static bool config_module_fnc_filelist(KviModule *m,KviCommand *c,KviParameterList * parms,KviStr &buffer)
-{
-	ENTER_STACK_FRAME(c,"config_module_fnc_filelist");
-	QAsciiDictIterator<KviConfig> it(*g_pConfigDict);
-
-	c->beginListArrayOrDictionaryReturnIdentifier();
-
-//	bool bFirst = true;
+static bool config_kvs_fnc_filelist(KviKvsModuleFunctionCall * c)
+{ 
+	
+	KVSM_PARAMETERS_BEGIN(c)
+	KVSM_PARAMETERS_END(c)
+	
+	KviKvsArray* pArray = new KviKvsArray();
 	int id=0;
+	
+	QDictIterator<KviConfig> it(*g_pConfigDict);
 	while(it.current())
 	{
-//		if(!bFirst)buffer.append(',');
-//		else bFirst = false;
-//		buffer.append(it.currentKey());
-		c->addListArrayOrDictionaryReturnValue(id++,KviStr(it.currentKey()),buffer);
+		pArray->set(id++, new KviKvsVariant(it.currentKey()));
 		++it;
 	}
-
-	return c->leaveStackFrame();
+	c->returnValue()->setArray(pArray);
+	return true;
 }
 
-
 /*
 	@doc: config.close
 	@type:
@@ -495,34 +517,36 @@ static bool config_module_fnc_filelist(KviModule *m,KviCommand *c,KviParameterLi
 		Closes the config file identified by <id>.[br]
 		<id> must be a a valid config file identifier returned by [fnc]$config.open[/fnc].[br]
 		If the config file was opened as read-write (default), the changes will be stored
-		to disk. If the config was opened as read-only , non changes will be written.[br]
+		to disk. If the config was opened as read-only , changes will not be written.[br]
 		If the <id> does not match any open config file, a warning is printed unless
 		the -q switch is used.[br]
 	@seealso:
 		[module:config]Config module documentation[/module]
 */
 
-static bool config_module_cmd_close(KviModule *m,KviCommand *c)
+static bool config_kvs_cmd_close(KviKvsModuleCommandCall * c)
 {
-	ENTER_STACK_FRAME(c,"config_module_cmd_close");
-	KviStr szId;
-	if(!g_pUserParser->parseCmdFinalPart(c,szId))return false;
+	QString szId;
+	KVSM_PARAMETERS_BEGIN(c) 
+		KVSM_PARAMETER("id",KVS_PT_STRING,0,szId) 
+	KVSM_PARAMETERS_END(c) 
 
-	KviConfig * cfg = g_pConfigDict->find(szId.ptr());
+	KviConfig * cfg = g_pConfigDict->find(szId);
 	
 	if(cfg)
 	{
 		if(cfg->dirty() && cfg->readOnly())
 		{
-			if(!c->hasSwitch('q'))c->warning(__tr("The config file '%s' has been changed but is opened as read-only: changes will be lost"),szId.ptr());
+			if(!c->hasSwitch('q',"quiet"))
+				c->warning(__tr2qs("The config file '%s' has been changed but is opened as read-only: changes will be lost"),&szId);
 		}
-		
-		g_pConfigDict->remove(szId.ptr()); // byez!
+		g_pConfigDict->remove(szId); 
 	} else {
-		if(!c->hasSwitch('q'))c->warning(__tr("The config file with id '%s' is not open"),szId.ptr());
+		if(!c->hasSwitch('q',"quiet"))
+			c->warning(__tr2qs("The config file with id '%Q' is not open"),&szId);
 	}
-
-	return c->leaveStackFrame();
+	
+	return true;
 }
 
 /*
@@ -547,23 +571,26 @@ static bool config_module_cmd_close(KviModule *m,KviCommand *c)
 		[module:config]Config module documentation[/module]
 */
 
-static bool config_module_cmd_flush(KviModule *m,KviCommand *c)
+static bool config_kvs_cmd_flush(KviKvsModuleCommandCall * c)
 {
-	ENTER_STACK_FRAME(c,"config_module_cmd_flush");
-	KviStr szId;
-	if(!g_pUserParser->parseCmdFinalPart(c,szId))return false;
+	QString szId;
+	KVSM_PARAMETERS_BEGIN(c) 
+		KVSM_PARAMETER("id",KVS_PT_STRING,0,szId) 
+	KVSM_PARAMETERS_END(c) 
 
-	KviConfig * cfg = g_pConfigDict->find(szId.ptr());
+	KviConfig * cfg = g_pConfigDict->find(szId);
 	
 	if(cfg)
 	{
-		if(cfg->readOnly())c->warning(__tr("The config file with id '%s' is read only"),szId.ptr());
-		else cfg->sync();
+		if(cfg->readOnly())
+			c->warning(__tr2qs("The config file with id '%Q' is read only"),&szId);
+		else
+			cfg->sync();
 	} else {
-		c->warning(__tr("The config file with id '%s' is not open"),szId.ptr());
+		c->warning(__tr2qs("The config file with id '%Q' is not open"),&szId);
 	}
-
-	return c->leaveStackFrame();
+	
+	return true;
 }
 
 /*
@@ -588,25 +615,25 @@ static bool config_module_cmd_flush(KviModule *m,KviCommand *c)
 		[module:config]Config module documentation[/module]
 */
 
-static bool config_module_cmd_clear(KviModule *m,KviCommand *c)
+static bool config_kvs_cmd_clear(KviKvsModuleCommandCall * c)
 {
-	ENTER_STACK_FRAME(c,"config_module_cmd_clear");
-	KviStr szId;
-	if(!g_pUserParser->parseCmdFinalPart(c,szId))return false;
+	QString szId;
+	KVSM_PARAMETERS_BEGIN(c) 
+		KVSM_PARAMETER("id",KVS_PT_STRING,0,szId) 
+	KVSM_PARAMETERS_END(c) 
 
-	KviConfig * cfg = g_pConfigDict->find(szId.ptr());
+	KviConfig * cfg = g_pConfigDict->find(szId);
 	
 	if(cfg)
 	{
 		cfg->clear();
 	} else {
-		c->warning(__tr("The config file with id '%s' is not open"),szId.ptr());
+		c->warning(__tr2qs("The config file with id '%Q' is not open"),&szId);
 	}
-
-	return c->leaveStackFrame();
+	
+	return true;
 }
 
-
 /*
 	@doc: config.clearsection
 	@type:
@@ -630,26 +657,27 @@ static bool config_module_cmd_clear(KviModule *m,KviCommand *c)
 		[module:config]Config module documentation[/module]
 */
 
-static bool config_module_cmd_clearsection(KviModule *m,KviCommand *c)
+static bool config_kvs_cmd_clearsection(KviKvsModuleCommandCall * c)
 {
-	ENTER_STACK_FRAME(c,"config_module_cmd_clearsection");
-	KviStr szId,szSect;
-	if(!g_pUserParser->parseCmdSingleToken(c,szId))return false;
-	if(!g_pUserParser->parseCmdFinalPart(c,szSect))return false;
+	QString szId;
+	QString szSect;
+	KVSM_PARAMETERS_BEGIN(c) 
+		KVSM_PARAMETER("id",KVS_PT_STRING,0,szId) 
+		KVSM_PARAMETER("section",KVS_PT_STRING,0,szSect)
+	KVSM_PARAMETERS_END(c) 
 
-	KviConfig * cfg = g_pConfigDict->find(szId.ptr());
+	KviConfig * cfg = g_pConfigDict->find(szId);
 	
 	if(cfg)
 	{
-		cfg->clearGroup(QString::fromUtf8(szSect.ptr()));
+		cfg->clearGroup(szSect);
 	} else {
-		c->warning(__tr("The config file with id '%s' is not open"),szId.ptr());
+		c->warning(__tr2qs("The config file with id '%Q' is not open"),&szId);
 	}
-
-	return c->leaveStackFrame();
+	
+	return true;
 }
 
-
 /*
 	@doc: config.write
 	@type:
@@ -676,27 +704,30 @@ static bool config_module_cmd_clearsection(KviModule *m,KviCommand *c)
 		[module:config]Config module documentation[/module]
 */
 
-static bool config_module_cmd_write(KviModule *m,KviCommand *c)
+static bool config_kvs_cmd_write(KviKvsModuleCommandCall * c)
 {
-	ENTER_STACK_FRAME(c,"config_module_cmd_write");
-	KviStr szId,szKey,szVal;
-	if(!g_pUserParser->parseCmdSingleToken(c,szId))return false;
-	if(!g_pUserParser->parseCmdSingleToken(c,szKey))return false;
-	if(!g_pUserParser->parseCmdFinalPart(c,szVal))return false;
+	QString szId;
+	QString szKey;
+	QString szVal;
+	
+	KVSM_PARAMETERS_BEGIN(c) 
+		KVSM_PARAMETER("id",KVS_PT_STRING,0,szId) 
+		KVSM_PARAMETER("key",KVS_PT_STRING,0,szKey)
+		KVSM_PARAMETER("value",KVS_PT_STRING,0,szVal)
+	KVSM_PARAMETERS_END(c) 
 
-	KviConfig * cfg = g_pConfigDict->find(szId.ptr());
+	KviConfig * cfg = g_pConfigDict->find(szId);
 	
 	if(cfg)
 	{
-		cfg->writeEntry(QString::fromUtf8(szKey.ptr()),QString::fromUtf8(szVal.ptr()));
+		cfg->writeEntry(szKey,szVal);
 	} else {
-		c->warning(__tr("The config file with id '%s' is not open"),szId.ptr());
+		c->warning(__tr2qs("The config file with id '%Q' is not open"),&szId);
 	}
-
-	return c->leaveStackFrame();
+	
+	return true;
 }
 
-
 /*
 	@doc: config.setsection
 	@type:
@@ -718,26 +749,28 @@ static bool config_module_cmd_write(KviModule *m,KviCommand *c)
 		[module:config]Config module documentation[/module]
 */
 
-static bool config_module_cmd_setsection(KviModule *m,KviCommand *c)
+static bool config_kvs_cmd_setsection(KviKvsModuleCommandCall * c)
 {
-	ENTER_STACK_FRAME(c,"config_module_cmd_setsection");
-	KviStr szId,szSec;
-	if(!g_pUserParser->parseCmdSingleToken(c,szId))return false;
-	if(!g_pUserParser->parseCmdFinalPart(c,szSec))return false;
+	QString szId;
+	QString szSect;
+	
+	KVSM_PARAMETERS_BEGIN(c) 
+		KVSM_PARAMETER("id",KVS_PT_STRING,0,szId) 
+		KVSM_PARAMETER("section",KVS_PT_STRING,0,szSect)
+	KVSM_PARAMETERS_END(c) 
 
-	KviConfig * cfg = g_pConfigDict->find(szId.ptr());
+	KviConfig * cfg = g_pConfigDict->find(szId);
 	
 	if(cfg)
 	{
-		cfg->setGroup(QString::fromUtf8(szSec.ptr()));
+		cfg->setGroup(szSect);
 	} else {
-		c->warning(__tr("The config file with id '%s' is not open"),szId.ptr());
+		c->warning(__tr2qs("The config file with id '%Q' is not open"),&szId);
 	}
-
-	return c->leaveStackFrame();
+	
+	return true;
 }
 
-
 /*
 	@doc: config
 	@type:
@@ -799,29 +832,26 @@ static bool config_module_cmd_setsection(KviModule *m,KviCommand *c)
 
 static bool config_module_init(KviModule * m)
 {
-
-
-	g_pConfigDict = new QAsciiDict<KviConfig>;
+	g_pConfigDict = new QDict<KviConfig>;
 	g_pConfigDict->setAutoDelete(true);
 
-	m->registerFunction("open",config_module_fnc_open);
-	m->registerFunction("read",config_module_fnc_read);
-	m->registerFunction("section",config_module_fnc_section);
-	m->registerFunction("hassection",config_module_fnc_hassection);
-	m->registerFunction("sectionlist",config_module_fnc_sectionlist);
-	m->registerFunction("keylist",config_module_fnc_keylist);
-	m->registerFunction("filelist",config_module_fnc_filelist);
-	m->registerFunction("filename",config_module_fnc_filename);
-	m->registerFunction("id",config_module_fnc_id);
-	m->registerFunction("readonly",config_module_fnc_readonly);
-
-	m->registerCommand("close",config_module_cmd_close);
-	m->registerCommand("flush",config_module_cmd_flush);
-	m->registerCommand("clear",config_module_cmd_clear);
-	m->registerCommand("clearsection",config_module_cmd_clearsection);
-	m->registerCommand("setsection",config_module_cmd_setsection);
-	m->registerCommand("write",config_module_cmd_write);
-
+	KVSM_REGISTER_FUNCTION(m,"open",config_kvs_fnc_open);
+	KVSM_REGISTER_FUNCTION(m,"id",config_kvs_fnc_id);
+	KVSM_REGISTER_FUNCTION(m,"read",config_kvs_fnc_read);
+	KVSM_REGISTER_FUNCTION(m,"section",config_kvs_fnc_section);
+	KVSM_REGISTER_FUNCTION(m,"hassection",config_kvs_fnc_hassection);
+	KVSM_REGISTER_FUNCTION(m,"sectionlist",config_kvs_fnc_sectionlist);
+	KVSM_REGISTER_FUNCTION(m,"keylist",config_kvs_fnc_keylist);
+	KVSM_REGISTER_FUNCTION(m,"filelist",config_kvs_fnc_filelist);
+	KVSM_REGISTER_FUNCTION(m,"filename",config_kvs_fnc_filename);
+	KVSM_REGISTER_FUNCTION(m,"readonly",config_kvs_fnc_readonly);
+	
+	KVSM_REGISTER_SIMPLE_COMMAND(m,"close",config_kvs_cmd_close);
+	KVSM_REGISTER_SIMPLE_COMMAND(m,"flush",config_kvs_cmd_flush);
+	KVSM_REGISTER_SIMPLE_COMMAND(m,"clear",config_kvs_cmd_clear);
+	KVSM_REGISTER_SIMPLE_COMMAND(m,"clearsection",config_kvs_cmd_clearsection);
+	KVSM_REGISTER_SIMPLE_COMMAND(m,"setsection",config_kvs_cmd_setsection);
+	KVSM_REGISTER_SIMPLE_COMMAND(m,"write",config_kvs_cmd_write);
 
 	return true;
 }
