@@ -43,7 +43,7 @@ extern KVIRC_API KviMediaManager * g_pMediaManager;
 extern KVIRC_API KviSharedFilesManager * g_pSharedFilesManager;
 
 #include <qfileinfo.h>
-
+#include <qstring.h>
 
 //#warning "The broker might lookup the remote host name"
 
@@ -57,10 +57,14 @@ KviDccBroker::KviDccBroker()
 
 	m_pDccWindowList = new KviPtrList<KviWindow>;
 	m_pDccWindowList->setAutoDelete(false);
+
+	m_pZeroPortTags = new QDict<KviDccZeroPortTag>(17);
+	m_pZeroPortTags->setAutoDelete(true);
 }
 
 KviDccBroker::~KviDccBroker()
 {
+	delete m_pZeroPortTags;
 	while(m_pBoxList->first())delete m_pBoxList->first();
 	delete m_pBoxList;
 	m_pBoxList = 0;
@@ -69,6 +73,39 @@ KviDccBroker::~KviDccBroker()
 	KviDccFileTransfer::done();
 }
 
+
+KviDccZeroPortTag * KviDccBroker::addZeroPortTag()
+{
+	static unsigned int g_uNextZeroPortTag = 0;
+	g_uNextZeroPortTag++;
+	KviDccZeroPortTag * t = new KviDccZeroPortTag;
+	t->m_tTimestamp = QDateTime::currentDateTime();
+	t->m_szTag.setNum(g_uNextZeroPortTag);
+	//t->m_szTag.prepend("mIrc-zero-port-");
+	t->m_uResumePosition = 0;
+	// FIXME: we should clear this dict if it grows too high....
+	m_pZeroPortTags->insert(t->m_szTag,t);
+	return t;
+}
+
+KviDccZeroPortTag * KviDccBroker::findZeroPortTag(const QString &szTag)
+{
+	KviDccZeroPortTag * t = m_pZeroPortTags->find(szTag);
+	if(!t)return 0;
+	if(t->m_tTimestamp.secsTo(QDateTime::currentDateTime()) > 180)
+	{
+		// too late man...
+		m_pZeroPortTags->remove(szTag);
+		return 0;
+	}
+	return t;
+}
+
+void KviDccBroker::removeZeroPortTag(const QString &szTag)
+{
+	m_pZeroPortTags->remove(szTag);
+}
+
 unsigned int KviDccBroker::dccBoxCount()
 {
 	return m_pBoxList->count();
@@ -100,7 +137,7 @@ void KviDccBroker::cancelDcc(KviDccBox *box,KviDccDescriptor * dcc)
 void KviDccBroker::rsendManage(KviDccDescriptor * dcc)
 {
 	// We need the filename...
-	QFileInfo fi(dcc->szLocalFileName.ptr());
+	QFileInfo fi(dcc->szLocalFileName);
 	if(fi.exists())rsendExecute(0,dcc);
 	else rsendAskForFileName(dcc);
 }
@@ -124,37 +161,63 @@ void KviDccBroker::rsendExecute(KviDccBox * box,KviDccDescriptor * dcc)
 	{
 		// No way...we NEED the right IRC context...
 		g_pApp->activeConsole()->output(KVI_OUT_DCCERROR,
-			__tr2qs_ctx("Can't send DCC %s request to %s: IRC connection has been terminated","dcc"),
-			dcc->szType.ptr(),dcc->szNick.ptr());
+			__tr2qs_ctx("Can't send DCC %Q request to %Q: IRC connection has been terminated","dcc"),
+			&(dcc->szType),&(dcc->szNick));
 		delete dcc;
 		return;
 	}
 
 	// Ok...we need the file to exist
-	QFileInfo fi(dcc->szLocalFileName.ptr());
+	QFileInfo fi(dcc->szLocalFileName);
 	if(!(fi.exists() && fi.isReadable() && (fi.isFile()) && (fi.size() > 0)))
 	{
-		dcc->console()->output(KVI_OUT_DCCERROR,__tr2qs_ctx("Can't open file %s for reading","dcc"),
-			dcc->szLocalFileName.ptr());
+		dcc->console()->output(KVI_OUT_DCCERROR,__tr2qs_ctx("Can't open file %Q for reading","dcc"),
+			&(dcc->szLocalFileName));
 		delete dcc;
 		return;
 	}
 
 	dcc->szFileName = dcc->szLocalFileName;
-	dcc->szFileName.cutToLast('/');
-	dcc->szFileName.cutToLast('\\');
+	dcc->szFileName = QFileInfo(dcc->szFileName).fileName();
 
-	KviStr fName = dcc->szFileName;
-	fName.replaceAll(' ',"\\040"); // be cool :)
+	QString fName = dcc->szFileName;
+	fName.replace(' ',"\\040"); // be cool :)
 
-	dcc->console()->connection()->sendFmtData("PRIVMSG %s :%cDCC %s %s %u%c",
-		dcc->szNick.ptr(),0x01,dcc->szType.ptr(),fName.ptr(),fi.size(),0x01);
+	QString szTag;
+	if(dcc->isZeroPortRequest())
+	{
+		// actually we tagged it as "nonempty" in /dcc.rsend --zero-port
+		// retag it with something more reasonable
+		KviDccZeroPortTag * t = addZeroPortTag();
+		t->m_uFileSize = fi.size();
+		dcc->setZeroPortRequestTag(t->m_szTag.latin1()); // latin1() should be ok here
+		szTag = t->m_szTag;
+
+		// DCC [ST]SEND <filename> <fakeipaddress> <zero-port> <filesize> <sessionid>
+		dcc->console()->connection()->sendFmtData("PRIVMSG %s :%cDCC %s %s 0.0.0.0 0 %u %s%c",
+			dcc->console()->connection()->encodeText(dcc->szNick).data(),
+			0x01,
+			dcc->console()->connection()->encodeText(dcc->szType).data(),
+			dcc->console()->connection()->encodeText(fName).data(),
+			fi.size(),
+			dcc->console()->connection()->encodeText(szTag).data(),
+			0x01);
+	} else {
+		dcc->console()->connection()->sendFmtData("PRIVMSG %s :%cDCC %s %s %u%c",
+			dcc->console()->connection()->encodeText(dcc->szNick).data(),
+			0x01,
+			dcc->console()->connection()->encodeText(dcc->szType).data(),
+			dcc->console()->connection()->encodeText(fName).data(),
+			fi.size(),0x01);
+		szTag = dcc->szFileName;
+	}
 
 	// now add a file offer , so he we will accept it automatically
 	// 120 secs is a reasonable timeout
-	KviStr mask(KviStr::Format,"%s!*@*",dcc->szNick.ptr());
+	QString szMask = dcc->szNick;
+	szMask += "!*@*";
 
-	g_pSharedFilesManager->addSharedFile(dcc->szFileName.ptr(),dcc->szLocalFileName.ptr(),mask.ptr(),120);
+	g_pSharedFilesManager->addSharedFile(szTag,dcc->szLocalFileName,szMask,120);
 
 	delete dcc;
 }
@@ -174,18 +237,25 @@ void KviDccBroker::handleChatRequest(KviDccDescriptor * dcc)
 		QString tmp = __tr2qs_ctx( \
 				"<b>%1 [%2@%3]</b> requests a " \
 				"<b>Direct Client Connection</b> in <b>%4</b> mode.<br>", \
-				"dcc").arg(dcc->szNick.ptr()).arg(dcc->szUser.ptr()).arg(dcc->szHost.ptr()).arg(dcc->szType.ptr());
+				"dcc").arg(dcc->szNick).arg(dcc->szUser).arg(dcc->szHost).arg(dcc->szType);
 
 #ifdef COMPILE_SSL_SUPPORT
 		if(dcc->bIsSSL)tmp += __tr2qs_ctx("The connection will be secured using SSL.<br>","dcc");
 #endif
 
-		tmp += __tr2qs_ctx( \
+		if(dcc->isZeroPortRequest())
+		{
+			tmp += __tr2qs_ctx( \
+					"You will be the passive side of the connection.<br>" \
+					,"dcc");
+		} else {
+			tmp += __tr2qs_ctx( \
 					"The connection target will be host <b>%1</b> on port <b>%2</b><br>" \
-					,"dcc").arg(dcc->szIp.ptr()).arg(dcc->szPort.ptr());
+					,"dcc").arg(dcc->szIp).arg(dcc->szPort);
+		}
 
 
-		QString caption = __tr2qs_ctx("DCC %1 Request - KVIrc","dcc").arg(dcc->szType.ptr());
+		QString caption = __tr2qs_ctx("DCC %1 Request - KVIrc","dcc").arg(dcc->szType);
 
 		KviDccAcceptBox * box = new KviDccAcceptBox(this,dcc,tmp,caption);
 
@@ -214,8 +284,8 @@ void KviDccBroker::executeChat(KviDccBox *box,KviDccDescriptor * dcc)
 	KviStr szSubProto = dcc->szType;
 	szSubProto.toLower();
 
-	KviStr tmp(KviStr::Format,"dcc: %s %s@%s:%s",szSubProto.ptr(),dcc->szNick.ptr(),dcc->szIp.ptr(),dcc->szPort.ptr());
-	KviDccChat * chat = new KviDccChat(dcc->console()->frame(),dcc,tmp.ptr());
+	QString tmp = QString("dcc: %1 %2@%3:%4").arg(szSubProto.ptr()).arg(dcc->szNick).arg(dcc->szIp).arg(dcc->szPort);
+	KviDccChat * chat = new KviDccChat(dcc->console()->frame(),dcc,tmp.utf8().data());
 
 	bool bMinimized = dcc->bOverrideMinimize ? dcc->bShowMinimized : \
 			(KVI_OPTION_BOOL(KviOption_boolCreateMinimizedDccChat) || \
@@ -240,7 +310,7 @@ void KviDccBroker::activeVoiceManage(KviDccDescriptor * dcc)
 					"<b>Direct Client Connection</b> in <b>VOICE</b> mode.<br>" \
 					"The connection target will be host <b>%4</b> on port <b>%5</b><br>" \
 				,"dcc" \
-			).arg(dcc->szNick.ptr()).arg(dcc->szUser.ptr()).arg(dcc->szHost.ptr()).arg(dcc->szIp.ptr()).arg(dcc->szPort.ptr());
+			).arg(dcc->szNick).arg(dcc->szUser).arg(dcc->szHost).arg(dcc->szIp).arg(dcc->szPort);
 
 		KviDccAcceptBox * box = new KviDccAcceptBox(this,dcc,tmp,__tr2qs_ctx("DCC VOICE request","dcc"));
 		m_pBoxList->append(box);
@@ -265,7 +335,7 @@ void KviDccBroker::activeVoiceExecute(KviDccBox *box,KviDccDescriptor * dcc)
 		dcc->setConsole(g_pApp->activeConsole());
 	}
 
-	KviStr tmp(KviStr::Format,"dcc: voice %s@%s:%s",dcc->szNick.ptr(),dcc->szIp.ptr(),dcc->szPort.ptr());
+	KviStr tmp(KviStr::Format,"dcc: voice %s@%s:%s",dcc->szNick.utf8().data(),dcc->szIp.utf8().data(),dcc->szPort.utf8().data());
 	KviDccVoice * v = new KviDccVoice(dcc->console()->frame(),dcc,tmp.ptr());
 
 	bool bMinimized = dcc->bOverrideMinimize ? dcc->bShowMinimized : \
@@ -285,7 +355,7 @@ void KviDccBroker::activeVoiceExecute(KviDccBox *box,KviDccDescriptor * dcc)
 
 void KviDccBroker::passiveVoiceExecute(KviDccDescriptor * dcc)
 {
-	KviStr tmp(KviStr::Format,"dcc: voice %s@%s:%s",dcc->szNick.ptr(),dcc->szIp.ptr(),dcc->szPort.ptr());
+	KviStr tmp(KviStr::Format,"dcc: voice %s@%s:%s",dcc->szNick.utf8().data(),dcc->szIp.utf8().data(),dcc->szPort.utf8().data());
 	KviDccVoice * v = new KviDccVoice(dcc->console()->frame(),dcc,tmp.ptr());
 //#warning "Create minimized dcc voice ?... or maybe it's too much ? :)"
 	bool bMinimized = dcc->bOverrideMinimize ? dcc->bShowMinimized : KVI_OPTION_BOOL(KviOption_boolCreateMinimizedDccChat);
@@ -309,7 +379,7 @@ void KviDccBroker::activeCanvasManage(KviDccDescriptor * dcc)
 					"<b>Direct Client Connection</b> in <b>CANVAS</b> mode.<br>" \
 					"The connection target will be host <b>%4</b> on port <b>%5</b><br>" \
 				,"dcc" \
-			).arg(dcc->szNick.ptr()).arg(dcc->szUser.ptr()).arg(dcc->szHost.ptr()).arg(dcc->szIp.ptr()).arg(dcc->szPort.ptr());
+			).arg(dcc->szNick).arg(dcc->szUser).arg(dcc->szHost).arg(dcc->szIp).arg(dcc->szPort);
 
 		KviDccAcceptBox * box = new KviDccAcceptBox(this,dcc,tmp,__tr2qs_ctx("DCC CANVAS request","dcc"));
 		m_pBoxList->append(box);
@@ -334,7 +404,7 @@ void KviDccBroker::activeCanvasExecute(KviDccBox *box,KviDccDescriptor * dcc)
 		dcc->setConsole(g_pApp->activeConsole());
 	}
 
-	KviStr tmp(KviStr::Format,"dcc: canvas %s@%s:%s",dcc->szNick.ptr(),dcc->szIp.ptr(),dcc->szPort.ptr());
+	KviStr tmp(KviStr::Format,"dcc: canvas %s@%s:%s",dcc->szNick.utf8().data(),dcc->szIp.utf8().data(),dcc->szPort.utf8().data());
 	KviDccCanvas * cnv = new KviDccCanvas(dcc->console()->frame(),dcc,tmp.ptr());
 
 //#warning "This option should be dedicated to Dcc Canvas!....for now we are using the DccChat options"
@@ -354,7 +424,7 @@ void KviDccBroker::activeCanvasExecute(KviDccBox *box,KviDccDescriptor * dcc)
 
 void KviDccBroker::passiveCanvasExecute(KviDccDescriptor * dcc)
 {
-	KviStr tmp(KviStr::Format,"dcc: canvas %s@%s:%s",dcc->szNick.ptr(),dcc->szIp.ptr(),dcc->szPort.ptr());
+	KviStr tmp(KviStr::Format,"dcc: canvas %s@%s:%s",dcc->szNick.utf8().data(),dcc->szIp.utf8().data(),dcc->szPort.utf8().data());
 	KviDccCanvas * cnv = new KviDccCanvas(dcc->console()->frame(),dcc,tmp.ptr());
 //#warning "This option should be dedicated to Dcc Canvas!....for now we are using the DccChat options"
 	bool bMinimized = dcc->bOverrideMinimize ? dcc->bShowMinimized : KVI_OPTION_BOOL(KviOption_boolCreateMinimizedDccChat);
@@ -385,9 +455,9 @@ void KviDccBroker::recvFileManage(KviDccDescriptor * dcc)
 						"<b>%5</b> bytes large.<br>" \
 						"The connection target will be host <b>%6</b> on port <b>%7</b><br>" \
 					,"dcc" \
-				).arg(dcc->szNick.ptr()).arg(dcc->szUser.ptr()).arg(dcc->szHost.ptr()).arg(
-					dcc->szFileName.ptr()).arg(dcc->szFileSize.ptr()).arg(
-					dcc->szIp.ptr()).arg(dcc->szPort.ptr());
+				).arg(dcc->szNick).arg(dcc->szUser).arg(dcc->szHost).arg(
+					dcc->szFileName).arg(dcc->szFileSize).arg(
+					dcc->szIp).arg(dcc->szPort);
 
 		} else {
 			// passive: we will be listening!
@@ -398,8 +468,8 @@ void KviDccBroker::recvFileManage(KviDccDescriptor * dcc)
 						"<b>%5</b> bytes large.<br>" \
 						"You will be the passive side of the connection.<br>" \
 					,"dcc" \
-				).arg(dcc->szNick.ptr()).arg(dcc->szUser.ptr()).arg(dcc->szHost.ptr()).arg(
-					dcc->szFileName.ptr()).arg(dcc->szFileSize.ptr());
+				).arg(dcc->szNick).arg(dcc->szUser).arg(dcc->szHost).arg(
+					dcc->szFileName).arg(dcc->szFileSize);
 		}
 
 		if(dcc->bIsIncomingAvatar)
@@ -420,7 +490,7 @@ void KviDccBroker::recvFileManage(KviDccDescriptor * dcc)
 
 //#warning "Maybe remove the pending avatar if rejected ?"
 
-		QString title = __tr2qs_ctx("DCC %1 Request - KVIrc","dcc").arg(dcc->szType.ptr());
+		QString title = __tr2qs_ctx("DCC %1 Request - KVIrc","dcc").arg(dcc->szType);
 
 		KviDccAcceptBox * box = new KviDccAcceptBox(this,dcc,tmp,title);
 		m_pBoxList->append(box);
@@ -434,9 +504,9 @@ void KviDccBroker::recvFileManage(KviDccDescriptor * dcc)
 
 		if(_OUTPUT_VERBOSE)
 		{
-			dcc->console()->output(KVI_OUT_DCCMSG,__tr2qs_ctx("Auto-accepting DCC %s request from %s!%s@%s for file %s","dcc"),
-				dcc->szType.ptr(),dcc->szNick.ptr(),dcc->szUser.ptr(),
-				dcc->szHost.ptr(),dcc->szFileName.ptr());
+			dcc->console()->output(KVI_OUT_DCCMSG,__tr2qs_ctx("Auto-accepting DCC %Q request from %Q!%Q@%Q for file %Q","dcc"),
+				&(dcc->szType),&(dcc->szNick),&(dcc->szUser),
+				&(dcc->szHost),&(dcc->szFileName));
 		}
 		chooseSaveFileName(0,dcc);
 	}
@@ -456,23 +526,41 @@ void KviDccBroker::chooseSaveFileName(KviDccBox *box,KviDccDescriptor *dcc)
 		if(KVI_OPTION_BOOL(KviOption_boolUseIncomingDccMediaTypeSavePath))
 		{
 			g_pMediaManager->lock();
-			if(KviMediaType * mt = g_pMediaManager->findMediaType(dcc->szFileName.ptr(),false))
+			if(KviMediaType * mt = g_pMediaManager->findMediaType(dcc->szFileName.utf8().data(),false))
 			{
 				if(mt->szSavePath.hasData())
 				{
-					if(kvi_directoryExists(mt->szSavePath.ptr()))dcc->szLocalFileName = mt->szSavePath;
+					if(KviFileUtils::directoryExists(mt->szSavePath.ptr()))dcc->szLocalFileName = mt->szSavePath;
 					else {
-						if(kvi_makeDir(mt->szSavePath.ptr()))dcc->szLocalFileName = mt->szSavePath;
+						if(KviFileUtils::makeDir(mt->szSavePath.ptr()))dcc->szLocalFileName = mt->szSavePath;
 					}
+					if(KVI_OPTION_BOOL(KviOption_boolSortReceivedByDccFilesByNicks))
+					{
+						KviQString::ensureLastCharIs(dcc->szLocalFileName,KVI_PATH_SEPARATOR_CHAR);
+						dcc->szLocalFileName.append(dcc->szNick);
+						KviFileUtils::adjustFilePath(dcc->szLocalFileName);
+					}
+					KviFileUtils::makeDir(dcc->szLocalFileName);
 				}
 			}
 			g_pMediaManager->unlock();
 		}
 	
-		if(dcc->szLocalFileName.isEmpty())g_pApp->getLocalKvircDirectory(dcc->szLocalFileName,KviApp::Incoming);
+		if(dcc->szLocalFileName.isEmpty()) 
+		{
+			g_pApp->getLocalKvircDirectory(dcc->szLocalFileName,KviApp::Incoming);
+			if(KVI_OPTION_BOOL(KviOption_boolSortReceivedByDccFilesByNicks))
+			{
+				KviQString::ensureLastCharIs(dcc->szLocalFileName,KVI_PATH_SEPARATOR_CHAR);
+				dcc->szLocalFileName.append(dcc->szNick);
+				KviFileUtils::adjustFilePath(dcc->szLocalFileName);
+				KviFileUtils::makeDir(dcc->szLocalFileName);
+			}
+		}
 	}
-
-	dcc->szLocalFileName.ensureLastCharIs('/');
+	int pos = dcc->szLocalFileName.length()-1;
+	dcc->szLocalFileName[pos]!='/';
+	dcc->szLocalFileName+="/";
 
 	if(!(dcc->bAutoAccept))
 	{
@@ -490,8 +578,8 @@ void KviDccBroker::chooseSaveFileName(KviDccBox *box,KviDccDescriptor *dcc)
 
 		if(_OUTPUT_VERBOSE)
 		{
-			dcc->console()->output(KVI_OUT_DCCMSG,__tr2qs_ctx("Auto-saving DCC %s file %s as \r![!dbl]play $0\r%s\r","dcc"),
-				dcc->szType.ptr(),dcc->szFileName.ptr(),dcc->szLocalFileName.ptr());
+			dcc->console()->output(KVI_OUT_DCCMSG,__tr2qs_ctx("Auto-saving DCC %Q file %Q as \r![!dbl]play $0\r%Q\r","dcc"),
+				&(dcc->szType),&(dcc->szFileName),&(dcc->szLocalFileName));
 		}
 
 		renameOverwriteResume(0,dcc);
@@ -503,7 +591,7 @@ void KviDccBroker::renameOverwriteResume(KviDccBox *box,KviDccDescriptor * dcc)
 	if(box)box->forgetDescriptor();
 
 	// Check if file exists
-	QFileInfo fi(dcc->szLocalFileName.ptr());
+	QFileInfo fi(dcc->szLocalFileName);
 	if(fi.exists() && (fi.size() > 0)) // 0 byte files are senseless for us
 	{
 		dcc->szLocalFileSize.setNum(fi.size());
@@ -531,7 +619,8 @@ void KviDccBroker::renameOverwriteResume(KviDccBox *box,KviDccDescriptor * dcc)
 							"<b>auto-rename</b> the new file, or<br>" \
 							"<b>resume</b> an incomplete download?" \
 						,"dcc" \
-					).arg(dcc->szLocalFileName.ptr()).arg(fi.size());
+					);
+				tmp.arg(dcc->szLocalFileName).arg(fi.size());
 			} else {
 				bDisableResume = true;
 				// the file on disk is larger or equal to the remote one
@@ -542,7 +631,7 @@ void KviDccBroker::renameOverwriteResume(KviDccBox *box,KviDccDescriptor * dcc)
 							"<b>overwrite</b> the existing file, or<br> " \
 							"<b>auto-rename</b> the new file ?" \
 						,"dcc" \
-					).arg(dcc->szLocalFileName.ptr());
+					).arg(dcc->szLocalFileName);
 			}
 
 			KviDccRenameBox * box = new KviDccRenameBox(this,dcc,tmp,bDisableResume);
@@ -560,7 +649,7 @@ void KviDccBroker::renameOverwriteResume(KviDccBox *box,KviDccDescriptor * dcc)
 			if(KVI_OPTION_BOOL(KviOption_boolAutoResumeDccSendWhenAutoAccepted) &&
 				(iRemoteSize > -1) && // only if the remote size is really known
 				(iRemoteSize > ((int)(fi.size()))) && // only if the remote size is larger than the local size
-				(!KviDccFileTransfer::nonFailedTransferWithLocalFileName(dcc->szLocalFileName.ptr()))) // only if there is no transfer with this local file name yet
+				(!KviDccFileTransfer::nonFailedTransferWithLocalFileName(dcc->szLocalFileName.utf8().data()))) // only if there is no transfer with this local file name yet
 			{
 				// yep, auto resume...
 				dcc->bResume = true;
@@ -582,7 +671,7 @@ void KviDccBroker::renameDccSendFile(KviDccBox *box,KviDccDescriptor * dcc)
 	if(box)box->forgetDescriptor();
 
 
-	if(kvi_fileExists(dcc->szLocalFileName.ptr()))
+	if(QFileInfo(dcc->szLocalFileName).exists())
 	{
 		KviStr szOrig = dcc->szLocalFileName;
 		int i = 1;
@@ -602,12 +691,12 @@ void KviDccBroker::renameDccSendFile(KviDccBox *box,KviDccDescriptor * dcc)
 				dcc->szLocalFileName += szNum;
 			}
 			i++;
-		} while(kvi_fileExists(dcc->szLocalFileName.ptr()));
+		} while(QFileInfo(dcc->szLocalFileName).exists());
 
 		if(_OUTPUT_VERBOSE)
 		{
-			dcc->console()->output(KVI_OUT_DCCMSG,__tr2qs_ctx("File %s exists, auto-renaming to %s","dcc"),
-				szOrig.ptr(),dcc->szLocalFileName.ptr());
+			dcc->console()->output(KVI_OUT_DCCMSG,__tr2qs_ctx("File %s exists, auto-renaming to %Q","dcc"),
+				szOrig.ptr(),&(dcc->szLocalFileName));
 		}
 	}
 	
@@ -626,11 +715,6 @@ void KviDccBroker::recvFileExecute(KviDccBox *box,KviDccDescriptor * dcc)
 		dcc->setConsole(g_pApp->activeConsole());
 	}
 
-//	KviStr szSubProto = dcc->szType;
-//	szSubProto.toLower();
-
-//	KviStr tmp(KviStr::Format,"dcc: %s %s@%s:%s %s",szSubProto.ptr(),dcc->szNick.ptr(),dcc->szIp.ptr(),dcc->szPort.ptr(),dcc->szLocalFileName.ptr());
-
 	//KviDccSend * send = new KviDccSend(dcc->console()->frame(),dcc,tmp.ptr());
 	KviDccFileTransfer * send = new KviDccFileTransfer(dcc);
 
@@ -639,10 +723,6 @@ void KviDccBroker::recvFileExecute(KviDccBox *box,KviDccDescriptor * dcc)
 				(dcc->bAutoAccept && KVI_OPTION_BOOL(KviOption_boolCreateMinimizedDccSendWhenAutoAccepted)));
 
 	send->invokeTransferWindow(dcc->console(),bMinimized,bMinimized);
-
-	//dcc->console()->frame()->addWindow(send,!bMinimized);
-	//if(bMinimized)send->minimize();
-	//m_pDccWindowList->append(send);
 }
 
 
@@ -667,35 +747,25 @@ void KviDccBroker::sendFileExecute(KviDccBox * box,KviDccDescriptor *dcc)
 		dcc->setConsole(g_pApp->activeConsole());
 	}
 
-	QFileInfo fi(dcc->szLocalFileName.ptr());
+	QFileInfo fi(dcc->szLocalFileName);
 	if(!(fi.exists() && fi.isReadable() && (fi.isFile()) && (fi.size() > 0)))
 	{
-		dcc->console()->output(KVI_OUT_DCCERROR,__tr2qs_ctx("Can't open file %s for reading","dcc"),
-			dcc->szLocalFileName.ptr());
+		dcc->console()->output(KVI_OUT_DCCERROR,__tr2qs_ctx("Can't open file %Q for reading","dcc"),
+			&(dcc->szLocalFileName));
 		delete dcc;
 		return;
 	}
-
+	
 	dcc->szFileName = dcc->szLocalFileName;
-	dcc->szFileName.cutToLast('/');    // cut both!
-	dcc->szFileName.cutToLast('\\');   // cut both!
+	dcc->szFileName = QFileInfo(dcc->szFileName).fileName();
 
 	dcc->szLocalFileSize.setNum(fi.size());
 
-//	KviStr szSubProto = dcc->szType;
-//	szSubProto.toLower();
-
-	//KviStr tmp(KviStr::Format,"dcc: %s %s@%s:%s %s",szSubProto.ptr(),dcc->szNick.ptr(),dcc->szIp.ptr(),dcc->szPort.ptr(),dcc->szLocalFileName.ptr());
-//	KviDccSend * send = new KviDccSend(dcc->console()->frame(),dcc,tmp.ptr());
 	KviDccFileTransfer * send = new KviDccFileTransfer(dcc);
 
 	bool bMinimized = dcc->bOverrideMinimize ? dcc->bShowMinimized : KVI_OPTION_BOOL(KviOption_boolCreateMinimizedDccSend);
 
 	send->invokeTransferWindow(dcc->console(),bMinimized,bMinimized);
-
-//	dcc->console()->frame()->addWindow(send,!bMinimized);
-//	if(bMinimized)send->minimize();
-//	m_pDccWindowList->append(send);
 }
 
 bool KviDccBroker::canUnload()
@@ -709,13 +779,50 @@ bool KviDccBroker::canUnload()
 	return true;
 }
 
-bool KviDccBroker::handleResumeAccepted(const char * filename,const char * port)
+bool KviDccBroker::handleResumeAccepted(const char * filename,const char * port,const char * szZeroPortTag)
 {
-	return KviDccFileTransfer::handleResumeAccepted(filename,port);
+	return KviDccFileTransfer::handleResumeAccepted(filename,port,szZeroPortTag);
 }
 
-bool KviDccBroker::handleResumeRequest(const char * filename,const char * port,unsigned int filePos)
+bool KviDccBroker::handleResumeRequest(KviDccRequest * dcc,const char * filename,const char * port,unsigned int filePos,const char * szZeroPortTag)
 {
+	//debug("HANDLE %s %s %u %s",filename,port,filePos,szZeroPortTag);
+	// the zeroPOrtTag is nonempty here only if port == 0
+	if(kvi_strEqualCI("0",port) && szZeroPortTag)
+	{
+		// zero port resume request (we have sent out a DCC SEND <filename> <fakeip> 0 <tag>
+		KviDccZeroPortTag * t = findZeroPortTag(QString(szZeroPortTag));
+		if(t)
+		{
+			//debug("FOUND");
+			// valid zero port resume request
+			if(filePos < t->m_uFileSize)
+			{
+				//debug("VALID");
+				// ok!
+				t->m_uResumePosition = filePos;
+
+				KviStr szBuffy;
+				KviServerParser::encodeCtcpParameter(filename,szBuffy);
+
+				dcc->ctcpMsg->msg->console()->connection()->sendFmtData(
+					"PRIVMSG %s :%cDCC ACCEPT %s %s %u %s%c",
+					dcc->ctcpMsg->msg->console()->connection()->encodeText(dcc->ctcpMsg->pSource->nick()).data(),
+					0x01,
+					szBuffy.ptr(),
+					port,
+					filePos,
+					szZeroPortTag,
+					0x01);
+
+				return true;
+			} else {
+				return false; // invalid resume size
+			}
+		}
+	}
+	//debug("NOT A ZeRO PORT");
+
 	return KviDccFileTransfer::handleResumeRequest(filename,port,filePos);
 }
 
