@@ -1520,7 +1520,7 @@ void KviCanvasItemPropertiesWidget::editItem(QCanvasItem * it)
 
 	for(QMap<QString,QVariant>::ConstIterator iter = m->begin();iter != m->end();++iter)
 	{
-		item = new QTableItem(this,QTableItem::Never,iter.key().latin1());
+		item = new QTableItem(this,QTableItem::Never,iter.key().utf8().data());
 		setItem(idx,0,item);
 		item = new KviVariantTableItem(this,iter.data());
 		setItem(idx,1,item);
