@@ -51,10 +51,10 @@
 #include "kvi_themedlabel.h"
 #include "kvi_socket.h"
 #include "kvi_app.h"
-#include "kvi_event.h"
 #include "kvi_parameterlist.h"
 #include "kvi_ircconnection.h"
 #include "kvi_ircconnectionuserinfo.h"
+#include "kvi_kvs_eventtriggers.h"
 
 #ifdef COMPILE_CRYPT_SUPPORT
 	#include "kvi_crypt.h"
@@ -71,6 +71,8 @@
 	#include "kvi_sslmaster.h"
 #endif
 
+
+
 extern KviDccBroker * g_pDccBroker;
 
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
@@ -134,17 +136,17 @@ KviDccChat::~KviDccChat()
 
 void KviDccChat::textViewRightClicked()
 {
-	TRIGGER_EVENT_1PARAM(KviEvent_OnDCCChatPopupRequest,this,m_pDescriptor->idString().latin1());
+	KVS_TRIGGER_EVENT_1(KviEvent_OnDCCChatPopupRequest,this,m_pDescriptor->idString());
 }
 
 void KviDccChat::triggerCreationEvents()
 {
-	TRIGGER_EVENT_1PARAM(KviEvent_OnDCCChatWindowCreated,this,m_pDescriptor->idString().latin1());
+	KVS_TRIGGER_EVENT_1(KviEvent_OnDCCChatWindowCreated,this,m_pDescriptor->idString());
 }
 
 void KviDccChat::triggerDestructionEvents()
 {
-	TRIGGER_EVENT_1PARAM(KviEvent_OnDCCChatWindowClosing,this,m_pDescriptor->idString().latin1());
+	KVS_TRIGGER_EVENT_1(KviEvent_OnDCCChatWindowClosing,this,m_pDescriptor->idString());
 }
 
 void KviDccChat::startConnection()
@@ -152,21 +154,21 @@ void KviDccChat::startConnection()
 	if(!(m_pDescriptor->bActive))
 	{
 		// PASSIVE CONNECTION
-		output(KVI_OUT_DCCMSG,__tr2qs_ctx("Attempting a passive DCC %s connection","dcc"),m_pDescriptor->szType.ptr());
+		output(KVI_OUT_DCCMSG,__tr2qs_ctx("Attempting a passive DCC %s connection","dcc"),m_pDescriptor->szType.utf8().data());
 #ifdef COMPILE_SSL_SUPPORT
-		int ret = m_pMarshal->dccListen(m_pDescriptor->szListenIp.ptr(),m_pDescriptor->szListenPort.ptr(),m_pDescriptor->bDoTimeout,m_pDescriptor->bIsSSL);
+		int ret = m_pMarshal->dccListen(m_pDescriptor->szListenIp,m_pDescriptor->szListenPort,m_pDescriptor->bDoTimeout,m_pDescriptor->bIsSSL);
 #else
-		int ret = m_pMarshal->dccListen(m_pDescriptor->szListenIp.ptr(),m_pDescriptor->szListenPort.ptr(),m_pDescriptor->bDoTimeout);
+		int ret = m_pMarshal->dccListen(m_pDescriptor->szListenIp,m_pDescriptor->szListenPort,m_pDescriptor->bDoTimeout);
 #endif
 		if(ret != KviError_success)handleMarshalError(ret);
 
 	} else {
 		// ACTIVE CONNECTION
-		output(KVI_OUT_DCCMSG,__tr2qs_ctx("Attempting an active DCC %s connection","dcc"),m_pDescriptor->szType.ptr());
+		output(KVI_OUT_DCCMSG,__tr2qs_ctx("Attempting an active DCC %s connection","dcc"),m_pDescriptor->szType.utf8().data());
 #ifdef COMPILE_SSL_SUPPORT
-		int ret = m_pMarshal->dccConnect(m_pDescriptor->szIp.ptr(),m_pDescriptor->szPort.ptr(),m_pDescriptor->bDoTimeout,m_pDescriptor->bIsSSL);
+		int ret = m_pMarshal->dccConnect(m_pDescriptor->szIp.utf8().data(),m_pDescriptor->szPort.utf8().data(),m_pDescriptor->bDoTimeout,m_pDescriptor->bIsSSL);
 #else
-		int ret = m_pMarshal->dccConnect(m_pDescriptor->szIp.ptr(),m_pDescriptor->szPort.ptr(),m_pDescriptor->bDoTimeout);
+		int ret = m_pMarshal->dccConnect(m_pDescriptor->szIp.utf8().data(),m_pDescriptor->szPort.utf8().data(),m_pDescriptor->bDoTimeout);
 #endif
 		if(ret != KviError_success)handleMarshalError(ret);
 	}
@@ -176,15 +178,15 @@ void KviDccChat::connectionInProgress()
 {
 	if(m_pDescriptor->bActive)
 	{
-		output(KVI_OUT_DCCMSG,__tr2qs_ctx("Contacting host %s on port %s","dcc"),m_pDescriptor->szIp.ptr(),m_pDescriptor->szPort.ptr());
+		output(KVI_OUT_DCCMSG,__tr2qs_ctx("Contacting host %Q on port %Q","dcc"),&(m_pDescriptor->szIp),&(m_pDescriptor->szPort));
 	} else {
-		output(KVI_OUT_DCCMSG,__tr2qs_ctx("Listening on interface %s port %s","dcc"),m_pMarshal->localIp(),m_pMarshal->localPort());
+		output(KVI_OUT_DCCMSG,__tr2qs_ctx("Listening on interface %Q port %Q","dcc"),&(m_pMarshal->localIp()),&(m_pMarshal->localPort()));
 
 		if(m_pDescriptor->bSendRequest)
 		{
 		
 			KviStr ip;
-			if(m_pDescriptor->szFakeIp.hasData())
+			if(!m_pDescriptor->szFakeIp.isEmpty())
 			{
 				ip = m_pDescriptor->szFakeIp;
 			} else {
@@ -197,7 +199,7 @@ void KviDccChat::connectionInProgress()
 						// try to get the IP that the IRC server can see
 						if(m_pDescriptor->console())
 						{
-							KviStr tmp = m_pDescriptor->console()->connection() ? m_pDescriptor->console()->connection()->userInfo()->hostIp().latin1() : "";
+							KviStr tmp = m_pDescriptor->console()->connection() ? m_pDescriptor->console()->connection()->userInfo()->hostIp().utf8().data() : "";
 							if(tmp.hasData())
 							{
 								ip = tmp;
@@ -212,24 +214,28 @@ void KviDccChat::connectionInProgress()
 				}
 			}
 
-			KviStr port   = m_pDescriptor->szFakePort.hasData() ? m_pDescriptor->szFakePort.ptr() : m_pMarshal->localPort();
+			QString port   = !m_pDescriptor->szFakePort.isEmpty() ? m_pDescriptor->szFakePort : QString(m_pMarshal->localPort());
 
 			//FIXME: #warning "OPTION FOR SENDING 127.0.0.1 and so on (not an unsigned number)"
 			struct in_addr a;
 			if(kvi_stringIpToBinaryIp(ip.ptr(),&a))ip.setNum(htonl(a.s_addr));
 
-			KviStr szReq(KviStr::Format,"PRIVMSG %s :%cDCC %s chat %s %s",
-				m_pDescriptor->szNick.ptr(),0x01,m_pDescriptor->szType.ptr(),ip.ptr(),port.ptr());
+			QString szReq = QString("PRIVMSG %1 :%2DCC %3 chat %4 %5").arg(m_pDescriptor->szNick).arg((char)0x01).arg(m_pDescriptor->szType).arg(ip.ptr()).arg(port);
 
-			if(m_pDescriptor->isZeroPortRequest())szReq.append(KviStr::Format," %s",m_pDescriptor->zeroPortRequestTag());
+			if(m_pDescriptor->isZeroPortRequest())	
+			{
+				szReq.append(" ");
+				szReq+=m_pDescriptor->zeroPortRequestTag();
+			}
 			szReq.append((char)(0x01));
 
-			m_pDescriptor->console()->connection()->sendData(szReq.ptr());
-			output(KVI_OUT_DCCMSG,__tr2qs_ctx("Sent DCC %s request to %s, waiting for the remote client to connect...","dcc"),
-				m_pDescriptor->szType.ptr(),m_pDescriptor->szNick.ptr());
-		} else output(KVI_OUT_DCCMSG,__tr2qs_ctx("DCC %s request not sent, awaiting manual connection","dcc"),m_pDescriptor->szType.ptr());
+			m_pDescriptor->console()->connection()->sendData(m_pDescriptor->console()->connection()->encodeText(szReq).data());
+			output(KVI_OUT_DCCMSG,__tr2qs_ctx("Sent DCC %Q request to %Q, waiting for the remote client to connect...","dcc"),
+				&(m_pDescriptor->szType),&(m_pDescriptor->szNick));
+			//qDebug(m_pDescriptor->szNick);
+		} else output(KVI_OUT_DCCMSG,__tr2qs_ctx("DCC %Q request not sent, awaiting manual connection","dcc"),&(m_pDescriptor->szType));
 	}
-	TRIGGER_EVENT_1PARAM(KviEvent_OnDCCChatConnectionInProgress,this,m_pDescriptor->idString().latin1());
+	KVS_TRIGGER_EVENT_1(KviEvent_OnDCCChatConnectionInProgress,this,m_pDescriptor->idString());
 }
 
 void KviDccChat::startingSSLHandshake()
@@ -243,7 +249,7 @@ void KviDccChat::startingSSLHandshake()
 void KviDccChat::sslError(const char * msg)
 {
 #ifdef COMPILE_SSL_SUPPORT
-	if(!TRIGGER_EVENT_2PARAM_RETVALUE(KviEvent_OnDCCChatError,this,msg,m_pDescriptor->idString().latin1()))
+	if(!KVS_TRIGGER_EVENT_2_HALTED(KviEvent_OnDCCChatError,this,QString(msg),m_pDescriptor->idString()))
 		output(KVI_OUT_DCCERROR,__tr2qs_ctx("[SSL ERROR]: %s","dcc"),msg);
 #endif
 }
@@ -251,26 +257,30 @@ void KviDccChat::sslError(const char * msg)
 const QString & KviDccChat::target()
 {
 	// This may change on the fly...
-	m_szTarget.sprintf("%s@%s:%s",m_pDescriptor->szNick.ptr(),m_pDescriptor->szIp.ptr(),m_pDescriptor->szPort.ptr());
+	m_szTarget = m_pDescriptor->szNick;
+	m_szTarget += "@";
+	m_szTarget += m_pDescriptor->szIp;
+	m_szTarget += ":";
+	m_szTarget += m_pDescriptor->szPort;
 	return m_szTarget;
 }
 
 void KviDccChat::fillCaptionBuffers()
 {
-	KviStr tmp(KviStr::Format,"DCC %s %s@%s:%s",
+	QString tmp = QString("DCC %1 %2@%3:%4").arg(
 #ifdef COMPILE_SSL_SUPPORT
-		m_pDescriptor->bIsSSL ? "SChat" : "Chat",
+		m_pDescriptor->bIsSSL ? "SChat" : "Chat").arg(
 #else
-		"Chat",
+		"Chat").arg(
 #endif
-		m_pDescriptor->szNick.ptr(),m_pDescriptor->szIp.ptr(),m_pDescriptor->szPort.ptr());
+		m_pDescriptor->szNick).arg(m_pDescriptor->szIp).arg(m_pDescriptor->szPort);
 
 	m_szPlainTextCaption = tmp;
 
 	m_szHtmlActiveCaption.sprintf("<nobr><font color=\"%s\"><b>%s</b></font></nobr>",
-		KVI_OPTION_COLOR(KviOption_colorCaptionTextActive).name().ascii(),tmp.ptr());
+		KVI_OPTION_COLOR(KviOption_colorCaptionTextActive).name().ascii(),tmp.utf8().data());
 	m_szHtmlInactiveCaption.sprintf("<nobr><font color=\"%s\"><b>%s</b></font></nobr>",
-		KVI_OPTION_COLOR(KviOption_colorCaptionTextInactive).name().ascii(),tmp.ptr());
+		KVI_OPTION_COLOR(KviOption_colorCaptionTextInactive).name().ascii(),tmp.utf8().data());
 }
 
 QPixmap * KviDccChat::myIconPtr()
@@ -281,7 +291,7 @@ QPixmap * KviDccChat::myIconPtr()
 
 void KviDccChat::getBaseLogFileName(KviStr &buffer)
 {
-	buffer.sprintf("%s_%s_%s",m_pDescriptor->szNick.ptr(),m_pDescriptor->szIp.ptr(),m_pDescriptor->szPort.ptr());
+	buffer.sprintf("%s_%s_%s",m_pDescriptor->szNick.utf8().data(),m_pDescriptor->szIp.utf8().data(),m_pDescriptor->szPort.utf8().data());
 }
 
 void KviDccChat::ownMessage(const QString &text)
@@ -312,8 +322,8 @@ void KviDccChat::ownMessage(const QString &text)
 						KviStr buf(KviStr::Format,"%s\r\n",encrypted.ptr());
 						m_pSlaveThread->sendRawData(buf.ptr(),buf.len());
 						m_pFrm->firstConsole()->outputPrivmsg(this,KVI_OUT_OWNPRIVMSGCRYPTED,
-							m_pDescriptor->szLocalNick.ptr(),m_pDescriptor->szLocalUser.ptr(),
-							m_pDescriptor->szLocalHost.ptr(),text,KviConsole::NoNotifications);
+							m_pDescriptor->szLocalNick.utf8().data(),m_pDescriptor->szLocalUser.utf8().data(),
+							m_pDescriptor->szLocalHost.utf8().data(),text,KviConsole::NoNotifications);
 					}
 					break;
 					case KviCryptEngine::Encoded:
@@ -322,8 +332,8 @@ void KviDccChat::ownMessage(const QString &text)
 						m_pSlaveThread->sendRawData(buf.ptr(),buf.len());
 						QString encr = decodeText(encrypted.ptr());
 						m_pFrm->firstConsole()->outputPrivmsg(this,KVI_OUT_OWNPRIVMSG,
-							m_pDescriptor->szLocalNick.ptr(),m_pDescriptor->szLocalUser.ptr(),
-							m_pDescriptor->szLocalHost.ptr(),encr,KviConsole::NoNotifications);
+							m_pDescriptor->szLocalNick.utf8().data(),m_pDescriptor->szLocalUser.utf8().data(),
+							m_pDescriptor->szLocalHost.utf8().data(),encr,KviConsole::NoNotifications);
 					}
 					break;
 					default: // also case KviCryptEngine::EncryptError
@@ -339,19 +349,18 @@ void KviDccChat::ownMessage(const QString &text)
 				QString tmp = text.right(text.length() - 1);
 				m_pSlaveThread->sendRawData(buf.ptr(),buf.len());
 				m_pFrm->firstConsole()->outputPrivmsg(this,KVI_OUT_OWNPRIVMSG,
-					m_pDescriptor->szLocalNick.ptr(),m_pDescriptor->szLocalUser.ptr(),
-					m_pDescriptor->szLocalHost.ptr(),tmp,KviConsole::NoNotifications);
+					m_pDescriptor->szLocalNick.utf8().data(),m_pDescriptor->szLocalUser.utf8().data(),
+					m_pDescriptor->szLocalHost.utf8().data(),tmp,KviConsole::NoNotifications);
 				return;
 			}
 		}
 	}
 #endif
 	KviStr buf(KviStr::Format,"%s\r\n",d);
-
 	m_pSlaveThread->sendRawData(buf.ptr(),buf.len());
 	m_pFrm->firstConsole()->outputPrivmsg(this,KVI_OUT_OWNPRIVMSG,
-		m_pDescriptor->szLocalNick.ptr(),m_pDescriptor->szLocalUser.ptr(),
-		m_pDescriptor->szLocalHost.ptr(),text,KviConsole::NoNotifications);
+		m_pDescriptor->szLocalNick.utf8().data(),m_pDescriptor->szLocalUser.utf8().data(),
+		m_pDescriptor->szLocalHost.utf8().data(),text,KviConsole::NoNotifications);
 }
 
 const QString & KviDccChat::localNick()
@@ -368,9 +377,9 @@ void KviDccChat::ownAction(const QString &text)
 		QCString szData = encodeText(text);
 		const char * d = szData.data();
 		if(!d)return;
-		KviStr buf(KviStr::Format,"%cACTION %s%c\r\n",d);
+		KviStr buf(KviStr::Format,"%cACTION %s%c\r\n",0x01,d,0x01);
 		m_pSlaveThread->sendRawData(buf.ptr(),buf.len());
-		output(KVI_OUT_ACTION,"%s %Q",m_pDescriptor->szLocalNick.ptr(),&text);
+		output(KVI_OUT_ACTION,"%Q %Q",&(m_pDescriptor->szLocalNick),&text);
 	} else {
 		output(KVI_OUT_SYSTEMWARNING,__tr2qs_ctx("Cannot send data: No active connection","dcc"));
 	}
@@ -386,23 +395,24 @@ bool KviDccChat::event(QEvent *e)
 			{
 				int * err = ((KviThreadDataEvent<int> *)e)->getData();
 				QString szErr = KviError::getDescription(*err);
-				if(!TRIGGER_EVENT_2PARAM_RETVALUE(KviEvent_OnDCCChatError,this,szErr.latin1(),m_pDescriptor->idString().latin1()))
+				if(!KVS_TRIGGER_EVENT_2_HALTED(KviEvent_OnDCCChatError,this,szErr,m_pDescriptor->idString()))
 					output(KVI_OUT_DCCERROR,__tr2qs_ctx("ERROR: %Q","dcc"),&szErr);
-				TRIGGER_EVENT_1PARAM(KviEvent_OnDCCChatDisconnected,this,m_pDescriptor->idString().latin1());
+				KVS_TRIGGER_EVENT_1(KviEvent_OnDCCChatDisconnected,this,m_pDescriptor->idString());
 				delete err;
 				return true;
 			}
 			break;
 			case KVI_DCC_THREAD_EVENT_DATA:
 			{
-				KviStr * d = ((KviThreadDataEvent<KviStr> *)e)->getData();
-				if(d->firstCharIs(0x01))
+				KviStr * encoded = ((KviThreadDataEvent<KviStr> *)e)->getData();
+				KviStr d=KviStr(decodeText(encoded->ptr()));
+				if(d.firstCharIs(0x01))
 				{
-					d->cutLeft(1);
-					if(d->lastCharIs(0x01))d->cutRight(1);
-					if(kvi_strEqualCIN("ACTION",d->ptr(),6))d->cutLeft(6);
-					d->stripLeftWhiteSpace();
-					output(KVI_OUT_ACTION,"%s %s",m_pDescriptor->szNick.ptr(),d->ptr());
+					d.cutLeft(1);
+					if(d.lastCharIs(0x01))d.cutRight(1);
+					if(kvi_strEqualCIN("ACTION",d.ptr(),6))d.cutLeft(6);
+					d.stripLeftWhiteSpace();
+					output(KVI_OUT_ACTION,"%Q %s",&(m_pDescriptor->szNick),d.ptr());
 				} else {
 
 #ifdef COMPILE_CRYPT_SUPPORT
@@ -411,18 +421,18 @@ bool KviDccChat::event(QEvent *e)
 						if(cinf->bDoDecrypt)
 						{
 							KviStr decryptedStuff;
-							switch(cinf->pEngine->decrypt(d->ptr(),decryptedStuff))
+							switch(cinf->pEngine->decrypt(d.ptr(),decryptedStuff))
 							{
 								case KviCryptEngine::DecryptOkWasEncrypted:
 								case KviCryptEngine::DecryptOkWasEncoded:
 								case KviCryptEngine::DecryptOkWasPlainText:
-									if(!TRIGGER_EVENT_2PARAM_RETVALUE(KviEvent_OnDCCChatMessage,this,decryptedStuff.ptr(),m_pDescriptor->idString().latin1()))
+									if(!KVS_TRIGGER_EVENT_2_HALTED(KviEvent_OnDCCChatMessage,this,QString(decryptedStuff.ptr()),m_pDescriptor->idString()))
 									{
 										m_pFrm->firstConsole()->outputPrivmsg(this,KVI_OUT_DCCCHATMSG,
-											m_pDescriptor->szNick.ptr(),m_pDescriptor->szUser.ptr(),
-											m_pDescriptor->szHost.ptr(),decryptedStuff.ptr());
+											m_pDescriptor->szNick.utf8().data(),m_pDescriptor->szUser.utf8().data(),
+											m_pDescriptor->szHost.utf8().data(),decryptedStuff.ptr());
 									}
-									delete d;
+									delete encoded;
 									return true;
 								break;
 								
@@ -436,15 +446,15 @@ bool KviDccChat::event(QEvent *e)
 					} else {
 #endif
 						// FIXME!
-						if(!TRIGGER_EVENT_2PARAM_RETVALUE(KviEvent_OnDCCChatMessage,this,d->ptr(),m_pDescriptor->idString().latin1()))
+						if(!KVS_TRIGGER_EVENT_2_HALTED(KviEvent_OnDCCChatMessage,this,QString(d.ptr()),m_pDescriptor->idString()))
 							m_pFrm->firstConsole()->outputPrivmsg(this,KVI_OUT_DCCCHATMSG,
-								m_pDescriptor->szNick.ptr(),m_pDescriptor->szUser.ptr(),
-								m_pDescriptor->szHost.ptr(),d->ptr());
+								m_pDescriptor->szNick.utf8().data(),m_pDescriptor->szUser.utf8().data(),
+								m_pDescriptor->szHost.utf8().data(),d.ptr());
 #ifdef COMPILE_CRYPT_SUPPORT
 					}
 #endif
 				}
-				delete d;
+				delete encoded;
 				return true;
 			}
 			break;
@@ -472,8 +482,8 @@ QSize KviDccChat::sizeHint() const
 void KviDccChat::handleMarshalError(int err)
 {
 	QString szErr = KviError::getDescription(err);
-	if(!TRIGGER_EVENT_2PARAM_RETVALUE(KviEvent_OnDCCChatError,this,szErr.latin1(),m_pDescriptor->idString().latin1()))
-		output(KVI_OUT_DCCERROR,__tr2qs_ctx("DCC %s failed: %Q","dcc"),m_pDescriptor->szType.ptr(),&szErr);
+	if(!KVS_TRIGGER_EVENT_2_HALTED(KviEvent_OnDCCChatError,this,szErr,m_pDescriptor->idString()))
+		output(KVI_OUT_DCCERROR,__tr2qs_ctx("DCC %Q failed: %Q","dcc"),&(m_pDescriptor->szType),&szErr);
 }
 
 void KviDccChat::connected()
@@ -499,12 +509,12 @@ void KviDccChat::connected()
 #endif
 	m_pSlaveThread->start();
 
-	if(!TRIGGER_EVENT_1PARAM_RETVALUE(KviEvent_OnDCCChatConnected,this,m_pDescriptor->idString().latin1()))
+	if(!KVS_TRIGGER_EVENT_1_HALTED(KviEvent_OnDCCChatConnected,this,m_pDescriptor->idString()))
 	{
-		output(KVI_OUT_DCCMSG,__tr2qs_ctx("Connected to %s:%s","dcc"),
-			m_pMarshal->remoteIp(),m_pMarshal->remotePort());
-		output(KVI_OUT_DCCMSG,__tr2qs_ctx("Local end is %s:%s","dcc"),
-			m_pMarshal->localIp(),m_pMarshal->localPort());
+		output(KVI_OUT_DCCMSG,__tr2qs_ctx("Connected to %Q:%Q","dcc"),
+			&(m_pMarshal->remoteIp()),&(m_pMarshal->remotePort()));
+		output(KVI_OUT_DCCMSG,__tr2qs_ctx("Local end is %Q:%Q","dcc"),
+			&(m_pMarshal->localIp()),&(m_pMarshal->localPort()));
 	}
 }
 
