@@ -34,7 +34,7 @@
 #include "kvi_netutils.h"
 #include "kvi_frame.h"
 #include "kvi_console.h"
-#include "kvi_uparser.h"
+//#include "kvi_uparser.h"
 #include "kvi_error.h"
 #include "kvi_options.h"
 #include "kvi_defaults.h"
@@ -50,6 +50,8 @@
 #include "utils.h"
 #include "send.h"
 
+#include <qfileinfo.h>
+
 #ifdef COMPILE_ON_WINDOWS
 	// Ugly Windoze compiler...
 	#include "dialogs.h"
@@ -60,25 +62,26 @@
 extern KVIRC_API KviSharedFilesManager * g_pSharedFilesManager;
 extern KviDccBroker * g_pDccBroker;
 
-static void dcc_module_reply_errmsg(KviDccRequest * dcc,const char * text)
+static void dcc_module_reply_errmsg(KviDccRequest * dcc,const QString& errText)
 {
 	dcc->ctcpMsg->msg->console()->connection()->sendFmtData(
-		"NOTICE %Q :%cERRMSG %s%c",
-		&(dcc->ctcpMsg->pSource->nick()),0x01,text,0x01);
+		"NOTICE %s :%cERRMSG %s%c",
+		dcc->ctcpMsg->msg->console()->connection()->encodeText(dcc->ctcpMsg->pSource->nick()).data(),0x01,
+		dcc->ctcpMsg->msg->console()->connection()->encodeText(errText).data()
+		,0x01);
 }
 
-static void dcc_module_request_error(KviDccRequest * dcc,const char * errText)
+static void dcc_module_request_error(KviDccRequest * dcc,const QString& errText)
 {
 	dcc->ctcpMsg->msg->console()->output(KVI_OUT_DCCERROR,
-			__tr2qs_ctx("Unable to process the above request: %s, %Q","dcc"),
-			errText,
+			__tr2qs_ctx("Unable to process the above request: %Q, %Q","dcc"),
+			&errText,
 			KVI_OPTION_BOOL(KviOption_boolNotifyFailedDccHandshakes) ? &(__tr2qs_ctx("Ignoring and notifying failure","dcc")) : &(__tr2qs_ctx("Ignoring","dcc")));
 
 	if(KVI_OPTION_BOOL(KviOption_boolNotifyFailedDccHandshakes))
 	{
-		KviStr szError(KviStr::Format,"Sorry, your DCC %s request can't be satisfied: %s",
-			dcc->szType.ptr(),errText);
-		dcc_module_reply_errmsg(dcc,szError.ptr());
+		QString szError = QString("Sorry, your DCC %1 request can't be satisfied: %2").arg(dcc->szType.ptr()).arg(errText);
+		dcc_module_reply_errmsg(dcc,szError);
 	}
 }
 
@@ -161,7 +164,7 @@ static bool dcc_module_normalize_target_data(KviDccRequest * dcc,KviStr &ipaddr,
 	if(ipaddr.isUnsignedNum())
 	{
 		addr.s_addr = htonl((unsigned long)ipaddr.toULong());
-		KviStr tmp;
+		QString tmp;
 		if(!kvi_binaryIpToStringIp(addr,tmp))
 		{
 			if(!dcc->ctcpMsg->msg->haltOutput())
@@ -219,6 +222,27 @@ static void dccModuleParseDccChat(KviDccRequest *dcc)
 	// the receiving side should respond with a DCC CHAT offer
 	// with the same random number appended, and then should listen for a connection.
 	//
+	// when a zero port request is initiated by another party we get
+	// 
+	//      DCC CHAT chat <fakeipaddress> 0 <tag>
+	//
+	// and we reply with
+	//
+	//      DCC CHAT chat <ourip> <ourport> <tag>
+	//
+	// when a zero port request is initiated by us we send out
+	//
+	//      DCC CHAT chat <fakeipaddress> 0 <tag>
+	//
+	// and we get
+	//
+	//      DCC CHAT chat <remoteip> <remoteport> <tag>
+	//
+	// Thus if there is a <tag> and the port is 0, then the remote party
+	// wanted to estabilish a dcc with us and wants us to listen, but if the port is nonzero then
+	// we have sent out a zero port request and the remote party acked it
+	// thus we have to connect instead!
+	//
 
 	// First of all we check the dcc slot limits
 	if(!dcc_module_check_limits(dcc))return;
@@ -259,25 +283,56 @@ static void dccModuleParseDccChat(KviDccRequest *dcc)
 	d->szIp              = dcc->szParam2.ptr();
 	d->szPort            = dcc->szParam3.ptr();
 
-	if(kvi_strEqualCS(d->szPort.ptr(),"0") && dcc->szParam4.hasData())
+
+	if(dcc->szParam4.hasData())
 	{
-		if(KVI_OPTION_BOOL(KviOption_boolDccSendFakeAddressByDefault))
+		// zero port tag ?
+		if(d->szPort == "0")
 		{
-			d->szFakeIp = KVI_OPTION_STRING(KviOption_stringDefaultDccFakeAddress);
-			if(d->szFakeIp.isEmpty())KVI_OPTION_BOOL(KviOption_boolDccSendFakeAddressByDefault) = false;
+			// zero port request
+			if(KVI_OPTION_BOOL(KviOption_boolDccSendFakeAddressByDefault))
+			{
+				d->szFakeIp = KVI_OPTION_STRING(KviOption_stringDefaultDccFakeAddress);
+				if(d->szFakeIp.isEmpty())KVI_OPTION_BOOL(KviOption_boolDccSendFakeAddressByDefault) = false;
+			}
+			d->setZeroPortRequestTag(dcc->szParam4.ptr());
+			QString tmp;
+			if(!dcc_module_get_listen_ip_address(0,d->console(),tmp))d->szListenIp = "0.0.0.0";
+			else d->szListenIp=tmp;
+			d->szListenPort = "0"; // any port is OK
+			d->bAutoAccept       = KVI_OPTION_BOOL(KviOption_boolAutoAcceptDccChat);
+			d->bActive           = false; // we must listen then...
+		} else {
+			// zero port acknowledge
+			// check if this is a tag that we have sent out
+			QString szTag = QString(dcc->szParam4.ptr());
+			KviDccZeroPortTag * t = g_pDccBroker->findZeroPortTag(szTag);
+			if(!t)
+			{
+				// hum.. not our tag
+				dcc->ctcpMsg->msg->console()->output(KVI_OUT_DCCMSG,
+					__tr2qs_ctx("The above request is broken: it looks like a zero port tag acknowledge but I have either never seen this tag or it was sent more than 120 seconds ago","dcc"));
+				dcc_module_request_error(dcc,__tr2qs_ctx("It seems that I haven't requested this dcc chat","dcc"));
+				delete d;
+				return;
+			} else {
+				g_pDccBroker->removeZeroPortTag(szTag);
+			}
+
+			d->bAutoAccept       = true; // auto-accept it (we have sent it out)
+			d->bActive           = true;
 		}
-		d->setZeroPortRequestTag(dcc->szParam4.ptr());
-		if(!dcc_module_get_listen_ip_address(0,d->console(),d->szListenIp))d->szListenIp = "0.0.0.0";
-		d->szListenPort = "0"; // any port is OK
+	} else {
+		d->bAutoAccept       = KVI_OPTION_BOOL(KviOption_boolAutoAcceptDccChat);
+		d->bActive           = true; // we have to connct (standard active chat)
 	}
 
-	d->bActive           = !d->isZeroPortRequest(); // we have to connect unless it is a zero port request
 #ifdef COMPILE_SSL_SUPPORT
 	d->bIsSSL            = bSSLExtension;
 #endif
-	d->bAutoAccept       = KVI_OPTION_BOOL(KviOption_boolAutoAcceptDccChat);
 
 	dcc_module_set_dcc_type(d,"CHAT");
+	d->triggerCreationEvent();
 
 	g_pDccBroker->handleChatRequest(d);
 }
@@ -288,6 +343,8 @@ static void dccModuleParseDccChat(KviDccRequest *dcc)
 // SEND
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
 
+static void dccModuleParseDccRecv(KviDccRequest * dcc);
+
 static void dccModuleParseDccSend(KviDccRequest *dcc)
 {
 //#warning "Ignore files depending on file type ? (MediaType ?)"
@@ -311,8 +368,59 @@ static void dccModuleParseDccSend(KviDccRequest *dcc)
 	// with the same random number appended, listen for a connection, and receive the file
 	// instead of sending it.
 	//
+	// when a zero port request is initiated by another party we get
+	//      DCC SEND <filename> <fakeipaddress> 0 <filesize> <tag>
+	// if (and only if) we want to resume we reply with
+	//      DCC RESUME <filename> 0 <resumesize> <tag>
+	// in this case the remote part replies again with
+	//      DCC ACCEPT <filename> 0 <resumesize> <tag>
+	// and we finally reply with
+	//      DCC SEND <filename> <ourip> <ourport> <filesize> <tag>
+	//
+	// when a zero port request is initiated by us we send out
+	//      DCC SEND <filename> <fakeipaddress> 0 <filesize> <tag>
+	// and if the remote party wants to resume then we get
+	//      DCC RESUME <filename> 0 <resumesize> <tag>
+	// and we eventually reply with
+	//      DCC ACCEPT <filename> 0 <resumesize> <tag>
+	// and we finally get
+	//      DCC SEND <filename> <remoteip> <remoteport> <filesize> <tag>
+	//
+	// Thus if there is a <tag> and the port is 0, then the remote party
+	// is trying to send a file to us, but if the port is nonzero then
+	// we have sent out a zero port request and the remote party acked it
+	//
+
+	if((!kvi_strEqualCS(dcc->szParam3.ptr(),"0")) && dcc->szParam5.hasData())
+	{
+		// DCC SEND <filename> <remoteip> <remoteport> <filesize> <tag>
+		// zero port acknowledge: treat as a RECV that should look like
+		// DCC [TS]RECV <filename> <remoteip> <remoteport> <resume-filesize>
+		// but since we have stored the sharedfile with the name <tag>
+		// we do exchange the params :)
+
+		KviDccZeroPortTag * t = g_pDccBroker->findZeroPortTag(dcc->szParam5.ptr());
+		if(t)
+		{
+			dcc->szParam4.sprintf("%u",t->m_uResumePosition);
+			g_pDccBroker->removeZeroPortTag(dcc->szParam5.ptr());
+		} else {
+			// this should never happen since we always add
+			// a zero port tag for out outgoing requests
+			// but well... maybe the user did something behing our back...
+			dcc->szParam4 = "0"; // no resume possible in this case
+		}
+
+		// swap the tag and the filename (we have added a fileoffer with this tag)
+		dcc->szParam1 = dcc->szParam5;
+		dcc->szParam5 = "";
+		
+		dccModuleParseDccRecv(dcc);
+		return;
+	}
 
 	// First of all we check the transfer limits
+	dcc->szParam1=dcc->pConsole->decodeText(dcc->szParam1).data();
 	if(!dcc_module_check_limits(dcc))return;
 	if(!dcc_module_check_concurrent_transfers_limit(dcc))return;
 
@@ -364,7 +472,7 @@ static void dccModuleParseDccSend(KviDccRequest *dcc)
 	d->szFileName        = dcc->szParam1.ptr();
 	d->szFileSize        = dcc->szParam4.ptr();
 
-	if(kvi_strEqualCS(d->szPort.ptr(),"0") && dcc->szParam5.hasData())
+	if(d->szPort=="0" && dcc->szParam5.hasData())
 	{
 		if(KVI_OPTION_BOOL(KviOption_boolDccSendFakeAddressByDefault))
 		{
@@ -372,7 +480,9 @@ static void dccModuleParseDccSend(KviDccRequest *dcc)
 			if(d->szFakeIp.isEmpty())KVI_OPTION_BOOL(KviOption_boolDccSendFakeAddressByDefault) = false;
 		}
 		d->setZeroPortRequestTag(dcc->szParam5.ptr());
-		if(!dcc_module_get_listen_ip_address(0,d->console(),d->szListenIp))d->szListenIp = "0.0.0.0";
+		QString tmp;
+		if(!dcc_module_get_listen_ip_address(0,d->console(),tmp))d->szListenIp = "0.0.0.0";
+		else d->szListenIp=QString(tmp);
 		d->szListenPort = "0"; // any port is OK
 		d->bSendRequest = true;
 		d->szLocalFileSize = d->szFileSize;
@@ -389,24 +499,25 @@ static void dccModuleParseDccSend(KviDccRequest *dcc)
 #endif
 	d->bOverrideMinimize = false;
 	d->bAutoAccept       = KVI_OPTION_BOOL(KviOption_boolAutoAcceptDccSend);
-	d->bIsIncomingAvatar = g_pApp->findPendingAvatarChange(dcc->pConsole,d->szNick.ptr(),d->szFileName.ptr());
+
+	d->bIsIncomingAvatar = g_pApp->findPendingAvatarChange(dcc->pConsole,d->szNick,d->szFileName);
 	dcc_module_set_dcc_type(d,"RECV");
 	if(KVI_OPTION_BOOL(KviOption_boolAutoAcceptIncomingAvatars))d->bAutoAccept = d->bAutoAccept || d->bIsIncomingAvatar;
+	d->triggerCreationEvent();
 
 	g_pDccBroker->recvFileManage(d);
 }
 
 static void dccModuleParseDccAccept(KviDccRequest *dcc)
 {
-	if(!g_pDccBroker->handleResumeAccepted(dcc->szParam1.ptr(),dcc->szParam2.ptr()))
+	// this is usually DCC ACCEPT <filename> <port> <resumesize>
+	// but may be also
+	// DCC ACCEPT <filename> 0 <resumesize> <tag>
+	if(!g_pDccBroker->handleResumeAccepted(dcc->szParam1.ptr(),dcc->szParam2.ptr(),dcc->szParam4.ptr()))
 	{
 //#warning "IF KviOption_boolReplyCtcpErrmsgOnInvalidAccept..."
 		if(!dcc->ctcpMsg->msg->haltOutput())
 		{
-//			dcc->ctcpMsg->msg->console()->output(KVI_OUT_DCCERROR,
-//				__tr("Can't proceed with dcc recv: transfer not initiated for file %s on port %s"),
-//					dcc->szParam1.ptr(),dcc->szParam2.ptr());
-
 			KviStr szError(KviStr::Format,__tr2qs_ctx("Can't proceed with DCC RECV: Transfer not initiated for file %s on port %s","dcc"),dcc->szParam1.ptr(),dcc->szParam2.ptr());
 			dcc_module_request_error(dcc,szError.ptr());
 		}
@@ -415,6 +526,17 @@ static void dccModuleParseDccAccept(KviDccRequest *dcc)
 
 static void dccModuleParseDccResume(KviDccRequest *dcc)
 {
+	// This is usually RESUME <filename> <port> <resumesize>
+	
+	// when a zero port request is initiated by us we send out
+	//      DCC SEND <filename> <fakeipaddress> 0 <filesize> <tag>
+	// and if the remote party wants to resume then we get
+	//      DCC RESUME <filename> 0 <resumesize> <tag>
+	// and we eventually reply with
+	//      DCC ACCEPT <filename> 0 <resumesize> <tag>
+	// and we finally get
+	//      DCC SEND <filename> <remoteip> <remoteport> <filesize> <tag>
+
 	bool bOk;
 	unsigned int filePos = dcc->szParam3.toUInt(&bOk);
 	if(!bOk)
@@ -423,18 +545,11 @@ static void dccModuleParseDccResume(KviDccRequest *dcc)
 		{
 			KviStr szError(KviStr::Format,__tr2qs_ctx("Invalid resume position argument '%s'","dcc"),dcc->szParam3.ptr());
 			dcc_module_request_error(dcc,szError.ptr());
-
-//			dcc->ctcpMsg->msg->console()->output(KVI_OUT_DCCERROR,
-//				__tr("Unable to process the above request: invalid resume position argument '%s'"),dcc->szParam3.ptr());
-//			if(KVI_OPTION_BOOL(KviOption_boolNotifyFailedDccHandshakes))
-//			{
-//				dcc_module_reply_errmsg(dcc,"Sorry, your resume request seems to be invalid");
-//			}
 		}
 		return;
 	}
 
-	if(!g_pDccBroker->handleResumeRequest(dcc->szParam1.ptr(),dcc->szParam2.ptr(),filePos))
+	if(!g_pDccBroker->handleResumeRequest(dcc,dcc->szParam1.ptr(),dcc->szParam2.ptr(),filePos,dcc->szParam4.ptr()))
 	{
 //#warning "IF KviOption_boolReplyCtcpErrmsgOnInvalidResume..."
 		if(!dcc->ctcpMsg->msg->haltOutput())
@@ -550,6 +665,7 @@ static void dccModuleParseDccRecv(KviDccRequest * dcc)
 
 		d->bActive           = true;
 		dcc_module_set_dcc_type(d,"SEND");
+		d->triggerCreationEvent();
 		g_pDccBroker->sendFileExecute(0,d);
 
 		return;
@@ -593,7 +709,7 @@ static void dccModuleParseDccRSend(KviDccRequest *dcc)
 	//
 	//      DCC [ST]RSEND <filename> <filesize>
 	//
-
+	dcc->szParam1 = dcc->pConsole->decodeText(dcc->szParam1).data();
 	if(!dcc_module_check_limits(dcc))return;
 	if(!dcc_module_check_concurrent_transfers_limit(dcc))return;
 
@@ -639,13 +755,14 @@ static void dccModuleParseDccRSend(KviDccRequest *dcc)
 	d->szHost            = dcc->ctcpMsg->pSource->host();
 	d->szIp              = __tr2qs_ctx("(unknown)","dcc");
 	d->szPort            = d->szIp;
-
-	if(!dcc_module_get_listen_ip_address(0,d->console(),d->szListenIp))
+	QString tmp;
+	if(!dcc_module_get_listen_ip_address(0,d->console(),tmp))
 	{
 		d->console()->output(KVI_OUT_DCCMSG,
 			__tr2qs_ctx("No suitable interface to listen on, trying to continue anyway...","dcc"));
 		d->szListenIp = "0.0.0.0";
-	}
+	} else
+		d->szListenIp=QString(tmp);
 
 	d->szListenPort      = "0";
 	dcc_fill_local_nick_user_host(d,dcc);
@@ -665,7 +782,7 @@ static void dccModuleParseDccRSend(KviDccRequest *dcc)
 	d->bNoAcks           = d->bIsTdcc;
 	d->bOverrideMinimize = false;
 	d->bAutoAccept       = KVI_OPTION_BOOL(KviOption_boolAutoAcceptDccSend);
-	d->bIsIncomingAvatar = g_pApp->findPendingAvatarChange(dcc->pConsole,d->szNick.ptr(),d->szFileName.ptr());
+	d->bIsIncomingAvatar = g_pApp->findPendingAvatarChange(dcc->pConsole,d->szNick.utf8().data(),d->szFileName.utf8().data());
 
 	if(KVI_OPTION_BOOL(KviOption_boolDccSendFakeAddressByDefault))
 	{
@@ -676,6 +793,7 @@ static void dccModuleParseDccRSend(KviDccRequest *dcc)
 	if(KVI_OPTION_BOOL(KviOption_boolAutoAcceptIncomingAvatars))d->bAutoAccept = d->bAutoAccept || d->bIsIncomingAvatar;
 
 	dcc_module_set_dcc_type(d,"RECV");
+	d->triggerCreationEvent();
 	g_pDccBroker->recvFileManage(d);
 }
 
@@ -690,7 +808,7 @@ static void dccModuleParseDccGet(KviDccRequest *dcc)
 	// DCC [TS]GET <filename> [filesize]
 	// -> DCC [TS]SEND <filename> <ipaddr> <port> <filesize>
 	// ...
-
+	dcc->szParam1=dcc->pConsole->decodeText(dcc->szParam1);
 	bool bOk;
 	unsigned int uSize = dcc->szParam2.toUInt(&bOk);
 	if(!bOk)uSize = 0;
@@ -740,6 +858,26 @@ static void dccModuleParseDccGet(KviDccRequest *dcc)
 		KviStr szSubproto("RSEND");
 		szSubproto.prepend(szExtensions);
 
+		
+		QString szFileName = QFileInfo(o->absFilePath()).fileName();
+		if(o->name() != szFileName)
+		{
+			// BUG
+			//   If the file offer was added with a name that is senseless (like "mediaXYZ" for an *.mp3 file)
+			//   then we would be going to RSEND that name here: the remote user woulnd't be
+			//   able to recognize the file.
+			//   Here we add another temporary offer with the right filename.
+
+			// now add a file offer , so he we will accept it automatically
+			// 120 secs is a reasonable timeout
+			QString szMask;
+			dcc->ctcpMsg->pSource->mask(szMask,KviIrcMask::NickUserHost);
+
+			KviSharedFile * pOld = o;
+			o = g_pSharedFilesManager->addSharedFile(szFileName,o->absFilePath(),szMask,120);
+			if(!o)o = pOld; // give up (FIXME: should we notify that ?)
+		}
+
 		if(!dcc->ctcpMsg->msg->haltOutput())
 		{
 			dcc->ctcpMsg->msg->console()->output(KVI_OUT_DCCMSG,
@@ -750,9 +888,10 @@ static void dccModuleParseDccGet(KviDccRequest *dcc)
 				&(o->absFilePath()),szSubproto.ptr());
 		}
 
-		dcc->pConsole->connection()->sendFmtData("PRIVMSG %Q :%cDCC %s %s %u%c",
-			&(dcc->ctcpMsg->pSource->nick()),
-			0x01,szSubproto.ptr(),dcc->szParam1.ptr(),o->fileSize(),0x01);
+		dcc->pConsole->connection()->sendFmtData("PRIVMSG %s :%cDCC %s %s %u%c",
+			dcc->pConsole->connection()->encodeText(dcc->ctcpMsg->pSource->nick()).data(),
+			0x01,szSubproto.ptr(),
+			dcc->pConsole->connection()->encodeText(dcc->szParam1.ptr()).data(),o->fileSize(),0x01);
 		return;
 	}
 
@@ -764,14 +903,15 @@ static void dccModuleParseDccGet(KviDccRequest *dcc)
 	d->szHost            = dcc->ctcpMsg->pSource->host();
 	d->bRecvFile         = false;
 	dcc_fill_local_nick_user_host(d,dcc);
-
-	if(!dcc_module_get_listen_ip_address(0,d->console(),d->szListenIp))
+	
+	QString tmp;
+	if(!dcc_module_get_listen_ip_address(0,d->console(),tmp))
 	{
 		d->console()->output(KVI_OUT_DCCMSG,
 			__tr2qs_ctx("No suitable interface to listen on, trying to continue anyway...","dcc"));
 		d->szListenIp = "0.0.0.0";
-	}
-
+	} else
+		d->szListenIp=QString(tmp);
 //#warning "DO STH WITH THIS PORT (HOW TO SPECIFY IT ?)"
 	d->szListenPort      = "0"; // any port is ok
 
@@ -798,13 +938,14 @@ static void dccModuleParseDccGet(KviDccRequest *dcc)
 	if(!dcc->ctcpMsg->msg->haltOutput())
 	{
 		dcc->ctcpMsg->msg->console()->output(KVI_OUT_DCCMSG,
-			__tr2qs_ctx("Accepting file request from %Q [%Q@%Q] for '%s' (real file: %Q), offering DCC %s","dcc"),
+			__tr2qs_ctx("Accepting file request from %Q [%Q@%Q] for '%s' (real file: %Q), offering DCC %Q","dcc"),
 			&(dcc->ctcpMsg->pSource->nick()),
 			&(dcc->ctcpMsg->pSource->username()),
 			&(dcc->ctcpMsg->pSource->host()),
 			dcc->szParam1.ptr(),
-			&(o->absFilePath()),d->szType.ptr());
+			&(o->absFilePath()),&(d->szType));
 	}
+	d->triggerCreationEvent();
 	g_pDccBroker->sendFileExecute(0,d);
 }
 
@@ -877,6 +1018,7 @@ static void dccModuleParseDccVoice(KviDccRequest *dcc)
 	d->bOverrideMinimize = false;
 	d->bAutoAccept       = KVI_OPTION_BOOL(KviOption_boolAutoAcceptDccVoice);
 	dcc_module_set_dcc_type(d,"VOICE");
+	d->triggerCreationEvent();
 	g_pDccBroker->activeVoiceManage(d);
 }
 
@@ -924,6 +1066,7 @@ static void dccModuleParseDccCanvas(KviDccRequest *dcc)
 	d->bOverrideMinimize = false;
 	d->bAutoAccept       = KVI_OPTION_BOOL(KviOption_boolAutoAcceptDccCanvas);
 	dcc_module_set_dcc_type(d,"CANVAS");
+	d->triggerCreationEvent();
 	g_pDccBroker->activeCanvasManage(d);
 }
 
