@@ -26,6 +26,8 @@
 #include "broker.h"
 #include "marshal.h"
 #include "broker.h"
+#include "window.h"
+#include "kvi_styled_controls.h"
 
 #ifdef COMPILE_ON_WINDOWS
 	// Ugly Windoze compiler...
@@ -48,13 +50,15 @@
 #include "kvi_memmove.h"
 #include "kvi_thread.h"
 #include "kvi_ircsocket.h"
-#include "kvi_uparser.h"
+//#include "kvi_uparser.h"
 #include "kvi_mediatype.h"
 #include "kvi_socket.h"
-#include "kvi_event.h"
+#include "kvi_kvs_eventtriggers.h"
 #include "kvi_parameterlist.h"
 #include "kvi_ircconnection.h"
 #include "kvi_ircconnectionuserinfo.h"
+#include "kvi_sparser.h"
+#include "kvi_kvs_script.h"
 
 #include <qevent.h>
 #include <qfile.h>
@@ -185,7 +189,7 @@ void KviDccRecvThread::run()
 
 	int iProbableTerminationTime = 0;
 
-	m_pFile = new QFile(m_pOpt->szFileName.ptr());
+	m_pFile = new QFile(QString::fromUtf8(m_pOpt->szFileName.ptr()));
 
 	if(m_pOpt->bResume)
 	{
@@ -482,7 +486,7 @@ void KviDccSendThread::run()
 	if(m_pOpt->iPacketSize < 32)m_pOpt->iPacketSize = 32;
 	char * buffer = (char *)kvi_malloc(m_pOpt->iPacketSize * sizeof(char));
 
-	QFile * pFile = new QFile(m_pOpt->szFileName.ptr());
+	QFile * pFile = new QFile(QString::fromUtf8(m_pOpt->szFileName.ptr()));
 
 	if(!pFile->open(IO_ReadOnly))
 	{
@@ -804,38 +808,62 @@ void KviDccFileTransfer::startConnection()
 		// PASSIVE CONNECTION
 		m_szStatusString = __tr2qs_ctx("Attempting a passive DCC %1 connection","dcc").arg(m_szDccType.ptr());
 		outputAndLog(m_szStatusString);
-
-		int ret = m_pMarshal->dccListen(m_pDescriptor->szListenIp.ptr(),m_pDescriptor->szListenPort.ptr(),m_pDescriptor->bDoTimeout);
-		if(ret != KviError_success)handleMarshalError(ret);
-
 	} else {
 		// ACTIVE CONNECTION
 		m_szStatusString = __tr2qs_ctx("Attempting an active DCC %1 connection","dcc").arg(m_szDccType.ptr());
 		outputAndLog(m_szStatusString);
+	}
 
-		if(m_pDescriptor->bResume && m_pDescriptor->bRecvFile)
+
+	if(m_pDescriptor->bResume && m_pDescriptor->bRecvFile)
+	{
+		QString fName;
+		KviServerParser::encodeCtcpParameter(m_pDescriptor->szFileName.utf8().data(),fName);
+		if(m_pDescriptor->isZeroPortRequest())
 		{
-			KviStr fName  = m_pDescriptor->szFileName.ptr();
-			fName.replaceAll(' ',"\\040"); // be cool :)
-			m_pDescriptor->console()->connection()->sendFmtData("PRIVMSG %s :%cDCC RESUME %s %s %s%c",
-				m_pDescriptor->szNick.ptr(),0x01,fName.ptr(),m_pDescriptor->szPort.ptr(),
-				m_pDescriptor->szLocalFileSize.ptr(),0x01);
-			m_szStatusString = __tr2qs_ctx("Sent DCC RESUME request to %1, waiting for ACCEPT","dcc").arg(m_pDescriptor->szNick.ptr());
-			outputAndLog(m_szStatusString);
-
-			// setup the resume timer: we don't want to wait forever
-
-			if(KVI_OPTION_UINT(KviOption_uintDccSocketTimeout) < 5)
-				KVI_OPTION_UINT(KviOption_uintDccSocketTimeout) = 5;
-
-			if(m_pResumeTimer)delete m_pResumeTimer;
-			m_pResumeTimer = new QTimer(this);
-			connect(m_pResumeTimer,SIGNAL(timeout()),this,SLOT(resumeTimedOut()));
-			m_pResumeTimer->start(KVI_OPTION_UINT(KviOption_uintDccSocketTimeout) * 1000,true);
+		m_pDescriptor->console()->connection()->sendFmtData("PRIVMSG %s :%cDCC RESUME %s %s %s %s%c",
+			m_pDescriptor->console()->connection()->encodeText(m_pDescriptor->szNick).data(),
+			0x01,
+			m_pDescriptor->console()->connection()->encodeText(fName).data(),
+			m_pDescriptor->szPort.utf8().data(),
+			m_pDescriptor->szLocalFileSize.utf8().data(),
+			m_pDescriptor->zeroPortRequestTag(),0x01);
 		} else {
-			int ret = m_pMarshal->dccConnect(m_pDescriptor->szIp.ptr(),m_pDescriptor->szPort.ptr(),m_pDescriptor->bDoTimeout);
-			if(ret != KviError_success)handleMarshalError(ret);
+			m_pDescriptor->console()->connection()->sendFmtData("PRIVMSG %s :%cDCC RESUME %s %s %s%c",
+				m_pDescriptor->console()->connection()->encodeText(m_pDescriptor->szNick).data(),
+				0x01,
+				m_pDescriptor->console()->connection()->encodeText(fName).data(),
+				m_pDescriptor->szPort.utf8().data(),
+				m_pDescriptor->szLocalFileSize.utf8().data(),0x01);
 		}
+		m_szStatusString = __tr2qs_ctx("Sent DCC RESUME request to %1, waiting for ACCEPT","dcc").arg(m_pDescriptor->szNick);
+		outputAndLog(m_szStatusString);
+
+		// setup the resume timer: we don't want to wait forever
+
+		if(KVI_OPTION_UINT(KviOption_uintDccSocketTimeout) < 5)
+			KVI_OPTION_UINT(KviOption_uintDccSocketTimeout) = 5;
+
+		if(m_pResumeTimer)delete m_pResumeTimer;
+		m_pResumeTimer = new QTimer(this);
+		connect(m_pResumeTimer,SIGNAL(timeout()),this,SLOT(resumeTimedOut()));
+		m_pResumeTimer->start(KVI_OPTION_UINT(KviOption_uintDccSocketTimeout) * 1000,true);
+	} else {
+		listenOrConnect();
+	}
+
+	displayUpdate();
+}
+
+void KviDccFileTransfer::listenOrConnect()
+{
+	if(!(m_pDescriptor->bActive))
+	{
+		int ret = m_pMarshal->dccListen(m_pDescriptor->szListenIp,m_pDescriptor->szListenPort,m_pDescriptor->bDoTimeout);
+		if(ret != KviError_success)handleMarshalError(ret);
+	} else {
+		int ret = m_pMarshal->dccConnect(m_pDescriptor->szIp.utf8().data(),m_pDescriptor->szPort.utf8().data(),m_pDescriptor->bDoTimeout);
+		if(ret != KviError_success)handleMarshalError(ret);
 	}
 
 	displayUpdate();
@@ -858,7 +886,7 @@ KviWindow * KviDccFileTransfer::dccMarshalOutputWindow()
 
 const char * KviDccFileTransfer::dccMarshalOutputContextString()
 {
-	return m_szTransferIdString.latin1();
+	return m_szTransferIdString.utf8().data();
 }
 
 void KviDccFileTransfer::die()
@@ -868,10 +896,9 @@ void KviDccFileTransfer::die()
 
 QString KviDccFileTransfer::localFileName()
 {
-	return QString(m_pDescriptor->szLocalFileName.ptr());
+	return m_pDescriptor->szLocalFileName;
 }
 
-
 void KviDccFileTransfer::abort()
 {
 	if(m_pSlaveRecvThread)m_pSlaveRecvThread->terminate();
@@ -879,20 +906,22 @@ void KviDccFileTransfer::abort()
 	if(m_pMarshal)m_pMarshal->abort();
 
 	if(m_pDescriptor->bRecvFile)
-			g_pApp->fileDownloadTerminated(false,m_pDescriptor->szFileName.ptr(),m_pDescriptor->szLocalFileName.ptr(),m_pDescriptor->szNick.ptr(),__tr_ctx("Aborted","dcc"));
+			g_pApp->fileDownloadTerminated(false,m_pDescriptor->szFileName.utf8().data(),m_pDescriptor->szLocalFileName.utf8().data(),m_pDescriptor->szNick.utf8().data(),__tr_ctx("Aborted","dcc"));
 
 	KviStr tmp;
 	
 	if(m_pSlaveRecvThread)tmp.setNum(m_pSlaveRecvThread->receivedBytes());
 	else if(m_pSlaveSendThread)tmp.setNum(m_pSlaveSendThread->sentBytes());
 	else tmp = '0';
-	TRIGGER_EVENT_3PARAM(KviEvent_OnDCCFileTransferFailed,eventWindow(),"Aborted by user",tmp.ptr(),m_pDescriptor->idString().latin1());
 
-	m_szStatusString = __tr2qs_ctx("Transfer failed: ","dcc");
-	m_szStatusString += __tr2qs_ctx("Aborted","dcc");
-	outputAndLog(KVI_OUT_DCCERROR,m_szStatusString);
 	m_eGeneralStatus = Failure;
 	m_tTransferEndTime = kvi_unixTime();
+	m_szStatusString = __tr2qs_ctx("Transfer failed: ","dcc");
+	m_szStatusString += __tr2qs_ctx("Aborted","dcc");
+
+	KVS_TRIGGER_EVENT_3(KviEvent_OnDCCFileTransferFailed,eventWindow(),QString("Aborted by user"),QString(tmp.ptr()),m_pDescriptor->idString());
+
+	outputAndLog(KVI_OUT_DCCERROR,m_szStatusString);
 	displayUpdate();
 }
 
@@ -900,6 +929,13 @@ void KviDccFileTransfer::abort()
 void KviDccFileTransfer::fillContextPopup(QPopupMenu * m,int column)
 {
 	m->insertItem(__tr2qs_ctx("Configure Bandwidth...","dcc"),this,SLOT(configureBandwidth()));
+	m->insertSeparator();
+	m->insertItem(__tr2qs_ctx("Resend DCC","dcc"),this,SLOT(retryDCC()));
+	m->insertItem(__tr2qs_ctx("Resend TDCC","dcc"),this,SLOT(retryTDCC()));
+	m->insertItem(__tr2qs_ctx("Resend RevDCC","dcc"),this,SLOT(retryRevDCC()));
+	/* FIX ME credo che il problema sia che se riavvio un trasferimento, a sua volta gia'
+		   avviato, questo non ha irc contex, perche' la finestra "in cui e' nato"e' sta 
+	   quella della dcc. Conservarsi l'id della finestra? */
 	int id = m->insertItem(__tr2qs_ctx("Abort","dcc"),this,SLOT(abort()));
 	if(!active())m->setItemEnabled(id,false);
 }
@@ -913,7 +949,59 @@ void KviDccFileTransfer::configureBandwidth()
 	m_pBandwidthDialog->show();
 }
 
+void KviDccFileTransfer::retryDCC()
+{
+	abort();
+	QString szRemoteNick = m_pDescriptor->remoteNick();
+	QString szFileName = m_pDescriptor->localFileName();
+	QString szId;
+	szId.setNum(m_pDescriptor->id());
+	QString szCommand = "dcc.send -r=" + szId + " " + szRemoteNick + " " + "\"" + szFileName + "\"";
+	KviKvsScript::run(szCommand,g_pActiveWindow);
+}
 
+void KviDccFileTransfer::retryTDCC()
+{
+	abort();
+	QString szRemoteNick = m_pDescriptor->remoteNick();
+	QString szFileName = m_pDescriptor->localFileName();
+	QString szId;
+	szId.setNum(m_pDescriptor->id());
+	QString szCommand = "dcc.send -r=" + szId + " -t " + szRemoteNick + " " + "\"" + szFileName + "\"";
+	KviKvsScript::run(szCommand,g_pActiveWindow);
+}
+void KviDccFileTransfer::retryRevDCC()
+{
+	abort();
+	QString szRemoteNick = m_pDescriptor->remoteNick();
+	QString szFileName = m_pDescriptor->localFileName();
+	QString szId;
+	szId.setNum(m_pDescriptor->id());
+	QString szCommand = "dcc.rsend -r=" + szId + " " + szRemoteNick + " " + "\"" + szFileName + "\"";
+	KviKvsScript::run(szCommand,g_pActiveWindow);
+}
+
+void KviDccFileTransfer::fillStatusString(QString &szBuffer)
+{
+	switch(m_eGeneralStatus)
+	{
+		case Connecting:
+			szBuffer = "connecting";
+			break;
+		case Transferring:
+			szBuffer = "transferring";
+			break;
+		case Failure:
+			szBuffer = "failure";
+			break;
+		case Success:
+			szBuffer = "success";
+			break;
+		default:
+			szBuffer = "unknown";
+			break;
+	}
+}
 
 bool KviDccFileTransfer::active()
 {
@@ -1051,15 +1139,15 @@ void KviDccFileTransfer::displayPaint(QPainter * p,int column,int width,int heig
 
 			p->setPen(Qt::black);
 
-			KviStr szRemote(KviStr::Format,"dcc://%s@%s:%s/%s",m_pDescriptor->szNick.ptr(),m_pDescriptor->szIp.ptr(),m_pDescriptor->szPort.ptr(),
-					m_pDescriptor->szFileName.ptr());
+			KviStr szRemote(KviStr::Format,"dcc://%s@%s:%s/%s",m_pDescriptor->szNick.utf8().data(),m_pDescriptor->szIp.utf8().data(),m_pDescriptor->szPort.utf8().data(),
+					m_pDescriptor->szFileName.utf8().data());
 
 			p->drawText(4 + daW1,iY,width - (8 + daW1),height - 8,Qt::AlignTop | Qt::AlignLeft,
-					m_pDescriptor->bRecvFile ? szRemote.ptr() : m_pDescriptor->szLocalFileName.ptr());
+					m_pDescriptor->bRecvFile ? szRemote.ptr() : m_pDescriptor->szLocalFileName.utf8().data());
 			iY += iLineSpacing;
 
 			p->drawText(4 + daW1,iY,width - (8 + daW1),height - 8,Qt::AlignTop | Qt::AlignLeft,
-					m_pDescriptor->bRecvFile ? m_pDescriptor->szLocalFileName.ptr() : szRemote.ptr());
+					m_pDescriptor->bRecvFile ? m_pDescriptor->szLocalFileName.utf8().data() : szRemote.ptr());
 			iY += iLineSpacing;
 
 
@@ -1322,13 +1410,13 @@ unsigned int KviDccFileTransfer::runningTransfersCount()
 	return cnt;
 }
 
-bool KviDccFileTransfer::handleResumeAccepted(const char * filename,const char * port)
+bool KviDccFileTransfer::handleResumeAccepted(const char * filename,const char * port,const char * szZeroPortTag)
 {
 	if(!g_pDccFileTransfers)return false;
 
 	for(KviDccFileTransfer * t = g_pDccFileTransfers->first();t;t = g_pDccFileTransfers->next())
 	{
-		if(t->resumeAccepted(filename,port))return true;
+		if(t->resumeAccepted(filename,port,szZeroPortTag))return true;
 	}
 
 	return false;
@@ -1384,7 +1472,7 @@ void KviDccFileTransfer::connectionInProgress()
 //			if(TRIGGER_EVENT_5PARAM_RETVALUE(KviEvent_OnDCCSendConnected,this,m_pDescriptor->szPort.ptr(),m_pDescriptor->szFileName.ptr(),m_pDescriptor->szNick.ptr(),m_pDescriptor->szUser.ptr(),m_pDescriptor->szHost.ptr()));
 //		}
 //
-		m_szStatusString = __tr2qs_ctx("Contacting host %1 on port %2","dcc").arg(m_pDescriptor->szIp.ptr()).arg(m_pDescriptor->szPort.ptr());
+		m_szStatusString = __tr2qs_ctx("Contacting host %1 on port %2","dcc").arg(m_pDescriptor->szIp).arg(m_pDescriptor->szPort);
 		outputAndLog(m_szStatusString);
 		displayUpdate();
 		return;
@@ -1396,8 +1484,8 @@ void KviDccFileTransfer::connectionInProgress()
 
 	if(m_pDescriptor->bSendRequest)
 	{
-		KviStr ip;
-		if(m_pDescriptor->szFakeIp.hasData())
+		QString ip;
+		if(!m_pDescriptor->szFakeIp.isEmpty())
 		{
 			ip = m_pDescriptor->szFakeIp;
 		} else {
@@ -1405,16 +1493,16 @@ void KviDccFileTransfer::connectionInProgress()
 
 			if(KVI_OPTION_BOOL(KviOption_boolDccGuessIpFromServerWhenLocalIsUnroutable))
 			{
-				if(!kvi_isRoutableIpString(ip.ptr()))
+				if(!KviNetUtils::isRoutableIpString(ip))
 				{
 					// try to get the IP that the IRC server can see
 					if(m_pDescriptor->console())
 					{
-						KviStr tmp = m_pDescriptor->console()->connection() ? m_pDescriptor->console()->connection()->userInfo()->hostIp().latin1() : "";
-						if(tmp.hasData())
+						QString tmp = m_pDescriptor->console()->connection() ? m_pDescriptor->console()->connection()->userInfo()->hostIp() : "";
+						if(!tmp.isEmpty())
 						{
 							ip = tmp;
-							outputAndLog(__tr2qs_ctx("The local IP address is private, determining from IRC server: %1","dcc").arg(QString(ip.ptr())));
+							outputAndLog(__tr2qs_ctx("The local IP address is private, determining from IRC server: %1","dcc").arg(ip));
 						} else {
 							outputAndLog(__tr2qs_ctx("The local IP address is private, but unable to determine it from the IRC server","dcc"));
 						}
@@ -1425,38 +1513,52 @@ void KviDccFileTransfer::connectionInProgress()
 			}
 		}
 		
-		KviStr port   = m_pDescriptor->szFakePort.hasData() ? m_pDescriptor->szFakePort.ptr() : m_pMarshal->localPort();
+		KviStr port   = !m_pDescriptor->szFakePort.isEmpty() ? m_pDescriptor->szFakePort.utf8().data() : m_pMarshal->localPort();
 		//#warning "OPTION FOR SENDING 127.0.0.1 and so on (not an unsigned nuumber)"
 		struct in_addr a;
-		if(kvi_stringIpToBinaryIp(ip.ptr(),&a))ip.setNum(htonl(a.s_addr));
+		if(KviNetUtils::stringIpToBinaryIp(ip,&a))ip.setNum(htonl(a.s_addr));
 
-		KviStr fName  = m_pDescriptor->szFileName.ptr();
-		fName.cutToLast('/'); // just to be sure...
-		fName.cutToLast('\\');
-		fName.replaceAll(' ',"\\040"); // be cool :)
+		QString tmp = m_pDescriptor->szFileName;
+		// just to be sure
+		KviQString::cutToLast(tmp,'/');
+		KviQString::cutToLast(tmp,'\\');
 
+		QString fName;
+
+		// BUG-TO-BUG mIrc compatibility
+		if(KVI_OPTION_BOOL(KviOption_boolDCCFileTransferReplaceOutgoingSpacesWithUnderscores))
+			tmp.replace(" ","_");
+
+		KviServerParser::encodeCtcpParameter(tmp.utf8().data(),fName);
 		// Zero port requests want DCC SEND as back-request
 		KviStr szReq;
+		
 		if(m_pDescriptor->isZeroPortRequest())
 		{
 			szReq = "SEND";
 			m_pDescriptor->console()->connection()->sendFmtData("PRIVMSG %s :%cDCC %s %s %s %s %s %s%c",
-					m_pDescriptor->szNick.ptr(),0x01,szReq.ptr(),
-					fName.ptr(),ip.ptr(),port.ptr(),
-					m_pDescriptor->szFileSize.ptr(),m_pDescriptor->zeroPortRequestTag(),0x01);
+					m_pDescriptor->console()->connection()->encodeText(m_pDescriptor->szNick).data(),
+					0x01,
+					m_pDescriptor->console()->connection()->encodeText(szReq.ptr()).data(),
+					m_pDescriptor->console()->connection()->encodeText(fName).data(),
+					ip.utf8().data(),port.ptr(),
+					m_pDescriptor->szFileSize.utf8().data(),m_pDescriptor->zeroPortRequestTag(),0x01);
 		} else {
 			szReq = m_szDccType;
-			m_pDescriptor->console()->connection()->sendFmtData("PRIVMSG %s :%cDCC %s %s %s %s %s%c",
-					m_pDescriptor->szNick.ptr(),0x01,szReq.ptr(),
-					fName.ptr(),ip.ptr(),port.ptr(),
-					m_pDescriptor->szLocalFileSize.ptr(),0x01);
+			m_pDescriptor->console()->connection()->sendFmtData("PRIVMSG %s :%cDCC %s %s %s %s %Q%c",
+					m_pDescriptor->console()->connection()->encodeText(m_pDescriptor->szNick).data(),
+					0x01,
+					m_pDescriptor->console()->connection()->encodeText(szReq.ptr()).data(),
+					m_pDescriptor->console()->connection()->encodeText(fName).data(),
+					ip.utf8().data(),port.ptr(),
+					&(m_pDescriptor->szLocalFileSize),0x01);
 		}
-		outputAndLog(__tr2qs_ctx("Sent DCC %1 request to %2, waiting for remote client to connect...","dcc").arg(szReq.ptr()).arg(m_pDescriptor->szNick.ptr()));
+		outputAndLog(__tr2qs_ctx("Sent DCC %1 request to %2, waiting for remote client to connect...","dcc").arg(szReq.ptr()).arg(m_pDescriptor->szNick));
 	} else {
 		outputAndLog(__tr2qs_ctx("DCC %1 request not sent, awaiting manual connection","dcc").arg(m_szDccType.ptr()));
 	}
 
-	TRIGGER_EVENT_1PARAM(KviEvent_OnDCCFileTransferConnectionInProgress,eventWindow(),m_pDescriptor->idString().latin1());
+	KVS_TRIGGER_EVENT_1(KviEvent_OnDCCFileTransferConnectionInProgress,eventWindow(),m_pDescriptor->idString());
 
 	displayUpdate();
 }
@@ -1491,16 +1593,20 @@ bool KviDccFileTransfer::event(QEvent *e)
 				QString szErrorString = KviError::getDescription(*err);
 				delete err;
 				if(m_pDescriptor->bRecvFile)
-					g_pApp->fileDownloadTerminated(false,m_pDescriptor->szFileName.ptr(),m_pDescriptor->szLocalFileName.ptr(),m_pDescriptor->szNick.ptr(),szErrorString.latin1());
-
-				KviStr tmp(KviStr::Format,"%d",m_pSlaveRecvThread ? m_pSlaveRecvThread->receivedBytes() : m_pSlaveSendThread->sentBytes());
-				TRIGGER_EVENT_3PARAM(KviEvent_OnDCCFileTransferFailed,eventWindow(),szErrorString.latin1(),tmp.ptr(),m_pDescriptor->idString().latin1());
+					g_pApp->fileDownloadTerminated(false,m_pDescriptor->szFileName.utf8().data(),m_pDescriptor->szLocalFileName.utf8().data(),m_pDescriptor->szNick.utf8().data(),szErrorString.utf8().data());
 
 				m_szStatusString = __tr2qs_ctx("Transfer failed: ","dcc");
 				m_szStatusString += szErrorString;
-				outputAndLog(KVI_OUT_DCCERROR,m_szStatusString);
 				m_eGeneralStatus = Failure;
 				m_tTransferEndTime = kvi_unixTime();
+
+				KVS_TRIGGER_EVENT_3(KviEvent_OnDCCFileTransferFailed,
+						eventWindow(),
+						szErrorString,
+						(kvs_int_t)(m_pSlaveRecvThread ? m_pSlaveRecvThread->receivedBytes() : m_pSlaveSendThread->sentBytes()),
+						m_pDescriptor->idString());
+
+				outputAndLog(KVI_OUT_DCCERROR,m_szStatusString);
 				displayUpdate();
 				return true;
 			}
@@ -1511,10 +1617,10 @@ bool KviDccFileTransfer::event(QEvent *e)
 				// File Upload/Download terminated, or something like this
 				if(KVI_OPTION_BOOL(KviOption_boolNotifyDccSendSuccessInConsole))
 				{
-					g_pApp->activeConsole()->output(KVI_OUT_DCCMSG,__tr2qs_ctx("DCC %s transfer with %s@%s:%s completed: \r![!dbl]play $0\r%s\r","dcc"),
+					g_pApp->activeConsole()->output(KVI_OUT_DCCMSG,__tr2qs_ctx("DCC %s transfer with %Q@%Q:%Q completed: \r![!dbl]play $0\r%s\r","dcc"),
 						m_pDescriptor->bIsTdcc ? (m_pDescriptor->bRecvFile ? "TRECV" : "TSEND") : (m_pDescriptor->bRecvFile ? "RECV" : "SEND"),
-						m_pDescriptor->szNick.ptr(),m_pDescriptor->szIp.ptr(),m_pDescriptor->szPort.ptr(),
-						m_pDescriptor->szLocalFileName.ptr());
+						&(m_pDescriptor->szNick),&(m_pDescriptor->szIp),&(m_pDescriptor->szPort),
+						&(m_pDescriptor->szLocalFileName));
 				}
 				/*
 				// Also add an optional message to the notifier, unless it is an AVATAR download!
@@ -1525,7 +1631,7 @@ bool KviDccFileTransfer::event(QEvent *e)
 					g_pApp->notifierMessage(0,KVI_SMALLICON_DCCMSG,szMsg,30);
 				}
 				*/
-				if(m_pDescriptor->bRecvFile)g_pApp->fileDownloadTerminated(true,m_pDescriptor->szFileName.ptr(),m_pDescriptor->szLocalFileName.ptr(),m_pDescriptor->szNick.ptr());
+				if(m_pDescriptor->bRecvFile)g_pApp->fileDownloadTerminated(true,m_pDescriptor->szFileName.utf8().data(),m_pDescriptor->szLocalFileName.utf8().data(),m_pDescriptor->szNick.utf8().data());
 /*
 				FIXME: Autoclear ?
 				if(KVI_OPTION_BOOL(KviOption_boolAutoCloseDccSendOnSuccess))close();
@@ -1535,8 +1641,10 @@ bool KviDccFileTransfer::event(QEvent *e)
 				m_eGeneralStatus = Success;
 				m_tTransferEndTime = kvi_unixTime();
 
-				KviStr tmp(KviStr::Format,"%d",m_pSlaveRecvThread ? m_pSlaveRecvThread->receivedBytes() : m_pSlaveSendThread->sentBytes());
-				TRIGGER_EVENT_2PARAM(KviEvent_OnDCCFileTransferSuccess,eventWindow(),tmp.ptr(),m_pDescriptor->idString().latin1());
+				KVS_TRIGGER_EVENT_2(KviEvent_OnDCCFileTransferSuccess,
+					eventWindow(),
+					(kvs_int_t)(m_pSlaveRecvThread ? m_pSlaveRecvThread->receivedBytes() : m_pSlaveSendThread->sentBytes()),
+					m_pDescriptor->idString());
 
 				displayUpdate();
 				return true;
@@ -1567,7 +1675,7 @@ void KviDccFileTransfer::handleMarshalError(int err)
 	m_szStatusString = __tr2qs_ctx("Transfer failed: ","dcc");
 	m_szStatusString += szErr;
 	outputAndLog(m_szStatusString);
-	TRIGGER_EVENT_3PARAM(KviEvent_OnDCCFileTransferFailed,eventWindow(),szErr.latin1(),"0",m_pDescriptor->idString().latin1());
+	KVS_TRIGGER_EVENT_3(KviEvent_OnDCCFileTransferFailed,eventWindow(),szErr,(kvs_int_t)0,m_pDescriptor->idString());
 	displayUpdate();
 }
 
@@ -1588,7 +1696,7 @@ void KviDccFileTransfer::connected()
 	if(m_pDescriptor->bRecvFile)
 	{
 		KviDccRecvThreadOptions * o = new KviDccRecvThreadOptions;
-		o->szFileName      = m_pDescriptor->szLocalFileName.ptr();
+		o->szFileName      = m_pDescriptor->szLocalFileName.utf8().data();
 		bool bOk;
 		o->iTotalFileSize  = m_pDescriptor->szFileSize.toInt(&bOk);
 		if(!bOk)o->iTotalFileSize = -1;
@@ -1602,7 +1710,7 @@ void KviDccFileTransfer::connected()
 		m_pSlaveRecvThread->start();
 	} else {
 		KviDccSendThreadOptions * o = new KviDccSendThreadOptions;
-		o->szFileName      = m_pDescriptor->szLocalFileName.ptr();
+		o->szFileName      = m_pDescriptor->szLocalFileName.utf8().data();
 		o->bFastSend       = KVI_OPTION_BOOL(KviOption_boolUseFastDccSend);
 		o->iIdleStepLengthInMSec = KVI_OPTION_BOOL(KviOption_boolDccSendForceIdleStep) ? KVI_OPTION_UINT(KviOption_uintDccSendIdleStepInMSec) : 0;
 		bool bOk;
@@ -1617,48 +1725,58 @@ void KviDccFileTransfer::connected()
 		m_pSlaveSendThread->start();
 	}
 
-	TRIGGER_EVENT_1PARAM(KviEvent_OnDCCFileTransferBegin,eventWindow(),m_pDescriptor->idString().latin1());
-
-	m_szStatusString = __tr2qs_ctx("Transferring data","dcc");
-	outputAndLog(m_szStatusString);
 	m_eGeneralStatus = Transferring;
+	m_szStatusString = __tr2qs_ctx("Transferring data","dcc");
+
+	KVS_TRIGGER_EVENT_1(KviEvent_OnDCCFileTransferBegin,eventWindow(),m_pDescriptor->idString());
+
+	outputAndLog(m_szStatusString);
 	displayUpdate();
 }
 
-bool KviDccFileTransfer::resumeAccepted(const char *filename,const char* port)
+bool KviDccFileTransfer::resumeAccepted(const char *filename,const char *port,const char *szZeroPortTag)
 {
-	if(kvi_strEqualCI(filename,m_pDescriptor->szFileName.ptr()) || KVI_OPTION_BOOL(KviOption_boolAcceptBrokenFileNameDccResumeRequests))
+	if(!(kvi_strEqualCI(filename,m_pDescriptor->szFileName.utf8().data()) || KVI_OPTION_BOOL(KviOption_boolAcceptBrokenFileNameDccResumeRequests)))
+		return false;
+
+	if(!(kvi_strEqualCI(port,m_pDescriptor->szPort.utf8().data()) &&
+			(!m_pSlaveRecvThread) && m_pDescriptor->bResume && m_pDescriptor->bRecvFile && m_pResumeTimer))
+		return false;
+
+	if(kvi_strEqualCI(port,"0"))
 	{
-		if(kvi_strEqualCI(port,m_pDescriptor->szPort.ptr()) &&
-			(!m_pSlaveRecvThread) && m_pDescriptor->bResume && m_pDescriptor->bRecvFile && m_pResumeTimer)
-		{
-			delete m_pResumeTimer;
-			m_pResumeTimer = 0;
-
-			outputAndLog(__tr2qs_ctx("RESUME accepted, transfer will begin at position %1","dcc").arg(m_pDescriptor->szLocalFileSize.ptr()));
-
-			int ret = m_pMarshal->dccConnect(m_pDescriptor->szIp.ptr(),
-							m_pDescriptor->szPort.ptr(),m_pDescriptor->bDoTimeout);
-
-			if(ret != KviError_success)handleMarshalError(ret);
-			else {
-				m_szStatusString = __tr2qs_ctx("Contacting host %1 on port %2","dcc").arg(m_pDescriptor->szIp.ptr()).arg(m_pDescriptor->szPort.ptr());
-				outputAndLog(m_szStatusString);
-				displayUpdate();
-			}
-	
-			return true;
-		}
+		if(!kvi_strEqualCI(szZeroPortTag,m_pDescriptor->zeroPortRequestTag()))
+			return false;
 	}
-	return false;
+
+	delete m_pResumeTimer;
+	m_pResumeTimer = 0;
+
+	outputAndLog(__tr2qs_ctx("RESUME accepted, transfer will begin at position %1","dcc").arg(m_pDescriptor->szLocalFileSize));
+
+	listenOrConnect();
+
+	/*
+	int ret = m_pMarshal->dccConnect(m_pDescriptor->szIp.utf8().data(),
+					m_pDescriptor->szPort.utf8().data(),m_pDescriptor->bDoTimeout);
+
+	if(ret != KviError_success)handleMarshalError(ret);
+	else {
+		m_szStatusString = __tr2qs_ctx("Contacting host %1 on port %2","dcc").arg(m_pDescriptor->szIp).arg(m_pDescriptor->szPort);
+		outputAndLog(m_szStatusString);
+		displayUpdate();
+	}
+	*/
+
+	return true;
 }
 
 bool KviDccFileTransfer::doResume(const char * filename,const char * port,unsigned int filePos)
 {
-	if(kvi_strEqualCI(port,m_pMarshal->dccPort()) &&
+	if(KviQString::equalCI(port,m_pMarshal->dccPort()) &&
 		(!m_pSlaveRecvThread) && (!m_pDescriptor->bRecvFile))
 	{
-		if(kvi_strEqualCI(filename,m_pDescriptor->szFileName.ptr()) || KVI_OPTION_BOOL(KviOption_boolAcceptBrokenFileNameDccResumeRequests))
+		if(KviQString::equalCI(filename,m_pDescriptor->szFileName) || KVI_OPTION_BOOL(KviOption_boolAcceptBrokenFileNameDccResumeRequests))
 		{
 			bool bOk;
 			unsigned int iLocalFileSize = m_pDescriptor->szLocalFileSize.toUInt(&bOk);
@@ -1677,8 +1795,16 @@ bool KviDccFileTransfer::doResume(const char * filename,const char * port,unsign
 			outputAndLog(KVI_OUT_DCCERROR,__tr2qs_ctx("Accepting RESUME request, transfer will begin at position %1","dcc").arg(filePos));
 
 			m_pDescriptor->szFileSize.setNum(filePos);
+			
+			
+			KviStr szBuffy;
+			KviServerParser::encodeCtcpParameter(filename,szBuffy);
+			
 			m_pDescriptor->console()->connection()->sendFmtData("PRIVMSG %s :%cDCC ACCEPT %s %s %u%c",
-				m_pDescriptor->szNick.ptr(),0x01,filename,port,filePos,0x01);
+				m_pDescriptor->console()->connection()->encodeText(m_pDescriptor->szNick).data(),
+				0x01,
+				m_pDescriptor->console()->connection()->encodeText(szBuffy.ptr()).data(),
+				port,filePos,0x01);
 			return true;
 		}
 	}
@@ -1703,7 +1829,7 @@ KviDccFileTransferBandwidthDialog::KviDccFileTransferBandwidthDialog(QWidget * p
 
 	szText = t->isFileUpload() ? __tr2qs_ctx("Limit upload bandwidth to","dcc") : __tr2qs_ctx("Limit download bandwidth to","dcc");
 	
-	m_pEnableLimitCheck = new QCheckBox(szText,this);
+	m_pEnableLimitCheck = new KviStyledCheckBox(szText,this);
 	g->addWidget(m_pEnableLimitCheck,0,0);
 	
 	m_pEnableLimitCheck->setChecked((iVal >= 0) && (iVal < MAX_DCC_BANDWIDTH_LIMIT));
