@@ -22,12 +22,17 @@
 //   Inc. ,59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 //
 
+#include "kvi_settings.h"
 #include "kvi_string.h"
 #include "kvi_command.h"
 #include "kvi_console.h"
+#include "kvi_kvs_moduleinterface.h"
 
 #ifndef _UTILS_CPP_
-	extern bool dcc_module_get_listen_ip_address(KviCommand *c,KviConsole * pConsole,KviStr &szListenIp);
+	extern bool dcc_module_get_listen_ip_address(KviCommand *c,KviConsole * pConsole,QString &szListenIp);
+#ifdef COMPILE_NEW_KVS
+	extern bool dcc_kvs_get_listen_ip_address(KviKvsModuleCommandCall *c,KviConsole * pConsole,QString &szListenIp);
+#endif
 #endif
 
 #endif //_UTILS_H_
