@@ -744,12 +744,12 @@ void KviDccVoice::startConnection()
 	{
 		// PASSIVE CONNECTION
 		output(KVI_OUT_DCCMSG,__tr2qs_ctx("Attempting a passive DCC VOICE connection","dcc"));
-		int ret = m_pMarshal->dccListen(m_pDescriptor->szListenIp.ptr(),m_pDescriptor->szListenPort.ptr(),m_pDescriptor->bDoTimeout);
+		int ret = m_pMarshal->dccListen(m_pDescriptor->szListenIp,m_pDescriptor->szListenPort,m_pDescriptor->bDoTimeout);
 		if(ret != KviError_success)handleMarshalError(ret);
 	} else {
 		// ACTIVE CONNECTION
 		output(KVI_OUT_DCCMSG,__tr2qs_ctx("Attempting an active DCC VOICE connection","dcc"));
-		int ret = m_pMarshal->dccConnect(m_pDescriptor->szIp.ptr(),m_pDescriptor->szPort.ptr(),m_pDescriptor->bDoTimeout);
+		int ret = m_pMarshal->dccConnect(m_pDescriptor->szIp.utf8().data(),m_pDescriptor->szPort.utf8().data(),m_pDescriptor->bDoTimeout);
 		if(ret != KviError_success)handleMarshalError(ret);
 	}
 }
@@ -758,25 +758,26 @@ void KviDccVoice::connectionInProgress()
 {
 	if(m_pDescriptor->bActive)
 	{
-		output(KVI_OUT_DCCMSG,__tr2qs_ctx("Contacting host %s on port %s","dcc"),m_pDescriptor->szIp.ptr(),m_pDescriptor->szPort.ptr());
+		output(KVI_OUT_DCCMSG,__tr2qs_ctx("Contacting host %Q on port %Q","dcc"),&(m_pDescriptor->szIp),&(m_pDescriptor->szPort));
 	} else {
 		
-		output(KVI_OUT_DCCMSG,__tr2qs_ctx("Listening on interface %s port %s","dcc"),
-			m_pMarshal->localIp(),m_pMarshal->localPort());
+		output(KVI_OUT_DCCMSG,__tr2qs_ctx("Listening on interface %Q port %Q","dcc"),
+			&(m_pMarshal->localIp()),&(m_pMarshal->localPort()));
 
 		if(m_pDescriptor->bSendRequest)
 		{
-			KviStr ip     = m_pDescriptor->szFakeIp.hasData() ? m_pDescriptor->szFakeIp : m_pDescriptor->szListenIp;
-			KviStr port   = m_pDescriptor->szFakePort.hasData() ? m_pDescriptor->szFakePort.ptr() : m_pMarshal->localPort();
+			KviStr ip     = !m_pDescriptor->szFakeIp.isEmpty() ? m_pDescriptor->szFakeIp : m_pDescriptor->szListenIp;
+			KviStr port   = !m_pDescriptor->szFakePort.isEmpty() ? m_pDescriptor->szFakePort.utf8().data() : m_pMarshal->localPort();
 //#warning "OPTION FOR SENDING 127.0.0.1 and so on (not an unsigned nuumber)"
 			struct in_addr a;
 			if(kvi_stringIpToBinaryIp(ip.ptr(),&a))ip.setNum(htonl(a.s_addr));
 
 			m_pDescriptor->console()->connection()->sendFmtData("PRIVMSG %s :%cDCC VOICE %s %s %s %d%c",
-					m_pDescriptor->szNick.ptr(),0x01,m_pDescriptor->szCodec.ptr(),
+					m_pDescriptor->console()->connection()->encodeText(m_pDescriptor->szNick).data(),
+					0x01,m_pDescriptor->szCodec.ptr(),
 					ip.ptr(),port.ptr(),m_pDescriptor->iSampleRate,0x01);
-			output(KVI_OUT_DCCMSG,__tr2qs_ctx("Sent DCC VOICE (%s) request to %s, waiting for the remote client to connect...","dcc"),
-					m_pDescriptor->szCodec.ptr(),m_pDescriptor->szNick.ptr());
+			output(KVI_OUT_DCCMSG,__tr2qs_ctx("Sent DCC VOICE (%s) request to %Q, waiting for the remote client to connect...","dcc"),
+					m_pDescriptor->szCodec.ptr(),&(m_pDescriptor->szNick));
 		} else output(KVI_OUT_DCCMSG,__tr2qs_ctx("DCC VOICE request not sent: awaiting manual connections","dcc"));
 	}
 }
@@ -785,20 +786,20 @@ const QString & KviDccVoice::target()
 {
 	// This may change on the fly...
 	m_szTarget.sprintf("%s@%s:%s",
-		m_pDescriptor->szNick.ptr(),m_pDescriptor->szIp.ptr(),m_pDescriptor->szPort.ptr());
+		m_pDescriptor->szNick.utf8().data(),m_pDescriptor->szIp.utf8().data(),m_pDescriptor->szPort.utf8().data());
 	return m_szTarget;
 }
 
 void KviDccVoice::getBaseLogFileName(KviStr &buffer)
 {
-	buffer.sprintf("dccvoice_%s_%s_%s",m_pDescriptor->szNick.ptr(),m_pDescriptor->szLocalFileName.ptr(),m_pDescriptor->szPort.ptr());
+	buffer.sprintf("dccvoice_%s_%s_%s",m_pDescriptor->szNick.utf8().data(),m_pDescriptor->szLocalFileName.utf8().data(),m_pDescriptor->szPort.utf8().data());
 }
 
 void KviDccVoice::fillCaptionBuffers()
 {
 	KviStr tmp(KviStr::Format,"DCC Voice %s@%s:%s %s",
-		m_pDescriptor->szNick.ptr(),m_pDescriptor->szIp.ptr(),m_pDescriptor->szPort.ptr(),
-		m_pDescriptor->szLocalFileName.ptr());
+		m_pDescriptor->szNick.utf8().data(),m_pDescriptor->szIp.utf8().data(),m_pDescriptor->szPort.utf8().data(),
+		m_pDescriptor->szLocalFileName.utf8().data());
 
 	m_szPlainTextCaption = tmp;
 
@@ -915,10 +916,10 @@ void KviDccVoice::handleMarshalError(int err)
 
 void KviDccVoice::connected()
 {
-	output(KVI_OUT_DCCMSG,__tr2qs_ctx("Connected to %s:%s","dcc"),
-		m_pMarshal->remoteIp(),m_pMarshal->remotePort());
-	output(KVI_OUT_DCCMSG,__tr2qs_ctx("Local end is %s:%s","dcc"),
-		m_pMarshal->localIp(),m_pMarshal->localPort());
+	output(KVI_OUT_DCCMSG,__tr2qs_ctx("Connected to %Q:%Q","dcc"),
+		&(m_pMarshal->remoteIp()),&(m_pMarshal->remotePort()));
+	output(KVI_OUT_DCCMSG,__tr2qs_ctx("Local end is %Q:%Q","dcc"),
+		&(m_pMarshal->localIp()),&(m_pMarshal->localPort()));
 	if(!(m_pDescriptor->bActive))
 	{
 		m_pDescriptor->szIp   = m_pMarshal->remoteIp();
@@ -940,7 +941,7 @@ void KviDccVoice::connected()
 	opt->bForceHalfDuplex = KVI_OPTION_BOOL(KviOption_boolDccVoiceForceHalfDuplex);
 //	opt->bForceDummyReadTrigger = false;
 	opt->iPreBufferSize = KVI_OPTION_UINT(KviOption_uintDccVoicePreBufferSize);
-	opt->szSoundDevice = KVI_OPTION_STRING(KviOption_stringDccVoiceSoundDevice).latin1();
+	opt->szSoundDevice = KVI_OPTION_STRING(KviOption_stringDccVoiceSoundDevice).utf8().data();
 	opt->iSampleRate = m_pDescriptor->iSampleRate;
 
 	m_pSlaveThread = new KviDccVoiceThread(this,m_pMarshal->releaseSocket(),opt);
@@ -978,7 +979,7 @@ int KviDccVoice::getMixerVolume(void) const
 	int left; //, right;
 	int req;
 
-	if((fd = ::open(KVI_OPTION_STRING(KviOption_stringDccVoiceMixerDevice).latin1(), O_RDONLY)) == -1)
+	if((fd = ::open(KVI_OPTION_STRING(KviOption_stringDccVoiceMixerDevice).utf8().data(), O_RDONLY)) == -1)
 	{
 		return 0;
 	}
@@ -1009,7 +1010,7 @@ void KviDccVoice::setMixerVolume(int vol)
 	int val;
 	int req;
 
-	if((fd = ::open(KVI_OPTION_STRING(KviOption_stringDccVoiceMixerDevice).latin1(), O_WRONLY)) == -1)
+	if((fd = ::open(KVI_OPTION_STRING(KviOption_stringDccVoiceMixerDevice).utf8().data(), O_WRONLY)) == -1)
 		return;
 
 	req = KVI_OPTION_BOOL(KviOption_boolDccVoiceVolumeSliderControlsPCM) ? SOUND_MIXER_WRITE_PCM : SOUND_MIXER_WRITE_VOLUME;
