@@ -1,9 +1,10 @@
+//=============================================================================
 //
 //   File : libkvidialog.cpp
 //   Creation date : Sat Sep 15 2001 01:13:25 by Szymon Stefanek
 //
 //   This file is part of the KVirc irc client distribution
-//   Copyright (C) 2001 Szymon Stefanek (pragma at kvirc dot net)
+//   Copyright (C) 2001-2005 Szymon Stefanek (pragma at kvirc dot net)
 //
 //   This program is FREE software. You can redistribute it and/or
 //   modify it under the terms of the GNU General Public License
@@ -19,6 +20,7 @@
 //   along with this program. If not, write to the Free Software Foundation,
 //   Inc. ,59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 //
+//=============================================================================
 
 
 #include "libkvidialog.h"
@@ -32,7 +34,7 @@
 #include <qpushbutton.h>
 
 #include "kvi_locale.h"
-#include "kvi_uparser.h"
+//#include "kvi_uparser.h"
 #include "kvi_module.h"
 #include "kvi_modulemanager.h"
 #include "kvi_command.h"
@@ -41,80 +43,54 @@
 #include "kvi_iconmanager.h"
 #include "kvi_console.h"
 #include "kvi_iconmanager.h"
+#include "kvi_kvs_script.h"
+#include "kvi_msgbox.h"
 
-static KviPtrList<KviScriptCallbackDialog> * g_pDialogModuleDialogList = 0;
 
-//#warning "Other dialogs!"
-
-KviScriptCallbackDialog::KviScriptCallbackDialog(KviWindow * pWnd,KviStr &code,KviParameterList * parms)
-{
-	g_pDialogModuleDialogList->append(this);
-	m_pWindow = pWnd;
-	m_szCode = code;
-	m_pParams = parms;
-	if(!m_pParams)
-	{
-		m_pParams = new KviParameterList();
-		m_pParams->setAutoDelete(true);
-	}
-}
-
-KviScriptCallbackDialog::~KviScriptCallbackDialog()
-{
-	g_pDialogModuleDialogList->removeRef(this);
-	if(m_pParams)delete m_pParams;
-}
-
-void KviScriptCallbackDialog::executeCallback()
-{
-	if(!g_pApp->windowExists(m_pWindow))m_pWindow = g_pApp->activeConsole();
-
-	KviCommand cmd(m_szCode.ptr(),m_pWindow);
-	if(m_pParams)cmd.setParams(m_pParams);
-
-	m_pParams = 0;
-
-	ENTER_STACK_FRAME((&cmd),"dialog_callback");
-
-	if(!g_pUserParser->parseCommand(&cmd))
-	{
-		if(cmd.hasError())g_pUserParser->printError(&cmd);
-	}
-}
-
-//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-
-KviScriptCallbackMessageBox::KviScriptCallbackMessageBox(KviStr &caption,KviStr &text,KviStr &icon,KviStr &b1,KviStr &b2,KviStr &b3,KviParameterList * parms,KviWindow * pWnd,KviStr &code)
-: QMessageBox(caption.ptr(),text.ptr(),QMessageBox::NoIcon,
-		b1.hasData() ? QMessageBox::Ok | QMessageBox::Default : QMessageBox::NoButton,
-		b2.hasData() ? (b3.hasData() ? QMessageBox::No : QMessageBox::No | QMessageBox::Escape) : QMessageBox::NoButton,
-		b3.hasData() ? QMessageBox::Cancel | QMessageBox::Escape : QMessageBox::NoButton,
-		0,0,false) ,
-	KviScriptCallbackDialog(pWnd,code,parms)
+KviKvsCallbackMessageBox::KviKvsCallbackMessageBox(
+		const QString &szCaption,
+		const QString &szText,
+		const QString &szIcon,
+		const QString &szButton0,
+		const QString &szButton1,
+		const QString &szButton2,
+		const QString &szCode,
+		KviKvsVariantList * pMagicParams,
+		KviWindow * pWindow,bool modal)
+	: QMessageBox(
+		szCaption,
+		szText,
+		QMessageBox::NoIcon,
+		szButton0.isEmpty() ? QMessageBox::NoButton : QMessageBox::Ok | QMessageBox::Default,
+		szButton1.isEmpty() ? QMessageBox::NoButton : (szButton2.isEmpty() ? QMessageBox::No | QMessageBox::Escape : QMessageBox::No),
+		szButton2.isEmpty() ? QMessageBox::NoButton : QMessageBox::Cancel | QMessageBox::Escape,
+		0,0,modal) ,
+	KviKvsCallbackObject("dialog.message",pWindow,szCode,pMagicParams,0)
 {
 	setIcon(*(g_pIconManager->getSmallIcon(KVI_SMALLICON_KVIRC)));
 
-	QPixmap * pix = g_pIconManager->getImage(icon.ptr());
+	QPixmap * pix = g_pIconManager->getImage(szIcon);
+
 	if(pix)setIconPixmap(*pix);
 	else {
-		if(kvi_strEqualCI(icon.ptr(),"information"))setIcon(QMessageBox::Information);
-		else if(kvi_strEqualCI(icon.ptr(),"warning"))setIcon(QMessageBox::Warning);
-		else if(kvi_strEqualCI(icon.ptr(),"critical"))setIcon(QMessageBox::Critical);
+		if(KviQString::equalCI(szIcon,"information"))setIcon(QMessageBox::Information);
+		else if(KviQString::equalCI(szIcon,"warning"))setIcon(QMessageBox::Warning);
+		else if(KviQString::equalCI(szIcon,"critical"))setIcon(QMessageBox::Critical);
 	}
-	if(b1.hasData())setButtonText(QMessageBox::Ok,b1.ptr());
-	if(b2.hasData())setButtonText(QMessageBox::No,b2.ptr());
-	if(b3.hasData())setButtonText(QMessageBox::Cancel,b3.ptr());
+	if(!szButton0.isEmpty())setButtonText(QMessageBox::Ok,szButton0);
+	if(!szButton1.isEmpty())setButtonText(QMessageBox::No,szButton1);
+	if(!szButton2.isEmpty())setButtonText(QMessageBox::Cancel,szButton2);
 }
 
-KviScriptCallbackMessageBox::~KviScriptCallbackMessageBox()
+KviKvsCallbackMessageBox::~KviKvsCallbackMessageBox()
 {
 }
 
-void KviScriptCallbackMessageBox::done(int code)
+void KviKvsCallbackMessageBox::done(int code)
 {
 	QMessageBox::done(code);
 
-	int iVal = 0;
+	kvs_int_t iVal = 0;
 	
 	switch(code)
 	{
@@ -122,14 +98,15 @@ void KviScriptCallbackMessageBox::done(int code)
 		case QMessageBox::Cancel: iVal = 2; break;
 	}
 
-	m_pParams->prepend(new KviStr(KviStr::Format,"%d",iVal));
-	executeCallback();
+	KviKvsVariantList params;
+	params.append(new KviKvsVariant(iVal));
+
+	execute(&params);
 
 	delete this;
 }
 
 
-
 /*
 	@doc: dialog.message
 	@type:
@@ -139,7 +116,7 @@ void KviScriptCallbackMessageBox::done(int code)
 	@short:
 		Shows a message box
 	@syntax:
-		dialog.message(<caption>,<message_text>,<icon>,<button0>[,<button1>[,<button2>[,<magic1>[,<magic2>[...]]]]])
+		dialog.message [-b] (<caption>,<message_text>,<icon>,<button0>[,<button1>[,<button2>[,<magic1>[,<magic2>[...]]]]])
 		{
 			<callback_command>
 		}
@@ -154,8 +131,13 @@ void KviScriptCallbackMessageBox::done(int code)
 		<button0> is the text of the first button (on the left).[br]
 		<button1> is the text of the second button (if empty or not given at all, only one button will appear in the dialog).[br]
 		<button2> is the text of the third button (if empty or not given, only two buttons will appear in the dialog).[br]
+		The first button is always the default button: it is activated when the user presses the
+		enter key. The thirs, or the second if the third is not present, is the escape button
+		and is activated when the user presses the Esc key.[br]
 		<magic1>,<magic2>... are the magic parameters: evaluated at dialog.message call time and passed
 		to the <callback_command> as positional parameters.[br]
+		If the -b or -modal switch is specified the dialog will have non-blocking modal behaviour:
+		it will appear above its parent widget and block its input until it's closed.[br] 
 		Once the dialog has been shown , the user will click one of the buttons. At this point the dialog
 		is hidden and the <callback_command> is executed passing the number of the button clicked
 		as $0 and the magic parameters as positional parameters $1 , $2 , $3....[br]
@@ -164,7 +146,7 @@ void KviScriptCallbackMessageBox::done(int code)
 	@examples:
 		[example]
 		[comment]# Just a warning dialog[/comment]
-		dialog.message("Warning","You're being <b>warned</b>",warning,Ok){ echo The user clicked OK; }
+		dialog.message("Warning","You're being <b>warned</b>",warning,"Ok"){ echo The user clicked OK; }
 		[comment]# A question[/comment]
 		dialog.message("And now ?","What do you want to do ?",information,"Go home","Watch TV","Scream")
 		{
@@ -175,47 +157,55 @@ void KviScriptCallbackMessageBox::done(int code)
 		[/example]
 */
 
-static bool dialog_module_cmd_message(KviModule *m,KviCommand *c)
+static bool dialog_kvs_cmd_message(KviKvsModuleCallbackCommandCall * c)
 {
-	ENTER_STACK_FRAME(c,"dialog_module_cmd_message");
+	QString szCaption,szMessage,szIcon,szButton0,szButton1,szButton2;
+	KviKvsVariantList params;
 
-	KviParameterList paramList;
-	paramList.setAutoDelete(true);
+	KVSM_PARAMETERS_BEGIN(c)
+		KVSM_PARAMETER("caption",KVS_PT_STRING,0,szCaption)
+		KVSM_PARAMETER("message",KVS_PT_STRING,0,szMessage)
+		KVSM_PARAMETER("icon",KVS_PT_STRING,0,szIcon)
+		KVSM_PARAMETER("button0",KVS_PT_STRING,KVS_PF_OPTIONAL,szButton0)
+		KVSM_PARAMETER("button1",KVS_PT_STRING,KVS_PF_OPTIONAL,szButton1)
+		KVSM_PARAMETER("button2",KVS_PT_STRING,KVS_PF_OPTIONAL,szButton2)
+		KVSM_PARAMETER("magic",KVS_PT_VARIANTLIST,KVS_PF_OPTIONAL,params)
+	KVSM_PARAMETERS_END(c)
+	bool modal;
+	if(c->hasSwitch('b',"modal")) modal=true;
+	else modal=false;
+	QString szCmd = c->callback()->code();
 
-	KviStr cmd;
-
-	if(!g_pUserParser->parseCallbackCommand(c,&paramList,&cmd))return false;
-
-	KviStr caption = paramList.safeFirstParam();
-	KviStr message = paramList.safeNextParam();
-	KviStr icon    = paramList.safeNextParam();
-	KviStr button1 = paramList.safeNextParam();
-	KviStr button2 = paramList.safeNextParam();
-	KviStr button3 = paramList.safeNextParam();
-
-	KviParameterList * parms = new KviParameterList();
-	parms->setAutoDelete(true);
-	while(KviStr * n = paramList.next())parms->append(new KviStr(*n));
-
-	KviScriptCallbackMessageBox * box = new KviScriptCallbackMessageBox(caption,message,icon,button1,button2,button3,parms,c->window(),cmd);
+	KviKvsCallbackMessageBox * box = new KviKvsCallbackMessageBox(
+			szCaption,szMessage,szIcon,szButton0,szButton1,szButton2,szCmd,&params,c->window(),modal);
 	box->show();
 
-	return c->leaveStackFrame();
+	return true;
 }
 
 
-//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
 
-KviScriptCallbackTextInput::KviScriptCallbackTextInput(KviStr &caption,KviStr &label,KviStr &deftext,KviStr &icon,bool bMultiLine,KviStr &b1,KviStr &b2,KviStr &b3,KviParameterList * parms,KviWindow * pWnd,KviStr &code)
-: QDialog() , KviScriptCallbackDialog(pWnd,code,parms)
+KviKvsCallbackTextInput::KviKvsCallbackTextInput(
+		const QString &szCaption,
+		const QString &szLabel,
+		const QString &szDefaultText,
+		const QString &szIcon,
+		bool bMultiLine,
+		const QString &szButton0,
+		const QString &szButton1,
+		const QString &szButton2,
+		const QString &szCode,
+		KviKvsVariantList * pMagicParams,
+		KviWindow * pWindow,bool modal)
+	: QDialog(), KviKvsCallbackObject("dialog.textinput",pWindow,szCode,pMagicParams,0)
 {
 	setIcon(*(g_pIconManager->getSmallIcon(KVI_SMALLICON_KVIRC)));
-
-	setCaption(caption.ptr());
+	setModal(modal);
+	setCaption(szCaption);
 
 	QGridLayout * g = new QGridLayout(this,2,3,5,5);
 
-	QPixmap * pix = g_pIconManager->getImage(icon.ptr());
+	QPixmap * pix = g_pIconManager->getImage(szIcon);
 
 	if(pix)
 	{
@@ -223,10 +213,10 @@ KviScriptCallbackTextInput::KviScriptCallbackTextInput(KviStr &caption,KviStr &l
 		il->setPixmap(*pix);
 		il->setAlignment(AlignCenter);
 		g->addWidget(il,0,0);
-		QLabel * tl = new QLabel(label.ptr(),this);
+		QLabel * tl = new QLabel(szLabel,this);
 		g->addWidget(tl,0,1);
 	} else {
-		QLabel * tl = new QLabel(label.ptr(),this);
+		QLabel * tl = new QLabel(szLabel,this);
 		g->addMultiCellWidget(tl,0,0,0,1);
 	}
 
@@ -237,10 +227,10 @@ KviScriptCallbackTextInput::KviScriptCallbackTextInput(KviStr &caption,KviStr &l
 	if(m_bMultiLine)
 	{
 		m_pEdit = new QMultiLineEdit(this);
-		((QMultiLineEdit *)m_pEdit)->setText(deftext.ptr());
+		((QMultiLineEdit *)m_pEdit)->setText(szDefaultText);
 	} else {
 		m_pEdit = new QLineEdit(this);
-		((QLineEdit *)m_pEdit)->setText(deftext.ptr());
+		((QLineEdit *)m_pEdit)->setText(szDefaultText);
 	}
 
 	g->addMultiCellWidget(m_pEdit,1,1,0,1);
@@ -248,78 +238,138 @@ KviScriptCallbackTextInput::KviScriptCallbackTextInput(KviStr &caption,KviStr &l
 	QHBox * box = new QHBox(this);
 	g->addMultiCellWidget(box,2,2,0,1);
 
-	if(b1.hasData())
+	m_iEscapeButton = 0;
+	m_iDefaultButton = 0;
+
+	if(!szButton0.isEmpty())
 	{
-		QPushButton * pb1 = new QPushButton(b1.ptr(),box);
-		connect(pb1,SIGNAL(clicked()),this,SLOT(b1Clicked()));
+		QString szB = szButton0;
+		bool bDef = false;
+		if(KviQString::equalCIN(szB,"default=",8))
+		{
+			bDef = true;
+			szB.remove(0,8);
+			m_iDefaultButton = 0;
+		} else if(KviQString::equalCIN(szB,"escape=",7))
+		{
+			szB.remove(0,7);
+			m_iEscapeButton = 0;
+		}
+		QPushButton * pb1 = new QPushButton(szB,box);
+		if(bDef)pb1->setDefault(true);
+		connect(pb1,SIGNAL(clicked()),this,SLOT(b0Clicked()));
 	}
 
-	if(b2.hasData())
+	if(!szButton1.isEmpty())
 	{
-		QPushButton * pb2 = new QPushButton(b2.ptr(),box);
-		connect(pb2,SIGNAL(clicked()),this,SLOT(b2Clicked()));
-		
+		QString szB = szButton1;
+		bool bDef = false;
+		if(KviQString::equalCIN(szB,"default=",8))
+		{
+			bDef = true;
+			szB.remove(0,8);
+			m_iDefaultButton = 1;
+		} else if(KviQString::equalCIN(szB,"escape=",7))
+		{
+			szB.remove(0,7);
+			m_iEscapeButton = 1;
+		}
+		QPushButton * pb2 = new QPushButton(szB,box);
+		if(bDef)pb2->setDefault(true);
+		connect(pb2,SIGNAL(clicked()),this,SLOT(b1Clicked()));
 	}
 
-	if(b3.hasData())
+	if(!szButton2.isEmpty())
 	{
-		QPushButton * pb3 = new QPushButton(b3.ptr(),box);
-		connect(pb3,SIGNAL(clicked()),this,SLOT(b3Clicked()));
+		QString szB = szButton2;
+		bool bDef = false;
+		if(KviQString::equalCIN(szB,"default=",8))
+		{
+			bDef = true;
+			szB.remove(0,8);
+			m_iDefaultButton = 2;
+		} else if(KviQString::equalCIN(szB,"escape=",7))
+		{
+			szB.remove(0,7);
+			m_iEscapeButton = 2;
+		}
+		QPushButton * pb3 = new QPushButton(szB,box);
+		if(bDef)pb3->setDefault(true);
+		connect(pb3,SIGNAL(clicked()),this,SLOT(b2Clicked()));
 	}
+
 }
 
-KviScriptCallbackTextInput::~KviScriptCallbackTextInput()
+KviKvsCallbackTextInput::~KviKvsCallbackTextInput()
 {
 }
 
-void KviScriptCallbackTextInput::b1Clicked()
+void KviKvsCallbackTextInput::b0Clicked()
 {
-	done(0);
+	done(0+10);
 }
 
-void KviScriptCallbackTextInput::b2Clicked()
+void KviKvsCallbackTextInput::b1Clicked()
 {
-	done(1);
+	done(1+10);
 }
 
-void KviScriptCallbackTextInput::b3Clicked()
+void KviKvsCallbackTextInput::b2Clicked()
 {
-	done(2);
+	done(2+10);
 }
 
-void KviScriptCallbackTextInput::closeEvent(QCloseEvent *e)
+void KviKvsCallbackTextInput::closeEvent(QCloseEvent *e)
 {
 	e->ignore();
-	done(2);
+	done(m_iEscapeButton+10);
 }
 
-void KviScriptCallbackTextInput::done(int code)
+void KviKvsCallbackTextInput::done(int code)
 {
-	KviStr * txt = new KviStr();
+	if(code >= 10)
+	{
+		code -= 10;
+	} else {
+		switch(code)
+		{
+			case QDialog::Accepted:
+				code = m_iDefaultButton;
+			break;
+			default:
+				code = m_iEscapeButton;
+			break;
+		}
+	}
 
+	QString txt;
+	
 	if(m_bMultiLine)
 	{
-		*txt = ((QMultiLineEdit *)m_pEdit)->text();
+		txt = ((QMultiLineEdit *)m_pEdit)->text();
 	} else {
-		*txt = ((QLineEdit *)m_pEdit)->text();
+		txt = ((QLineEdit *)m_pEdit)->text();
 	}
-	m_pParams->prepend(txt);
 
-	m_pParams->prepend(new KviStr(KviStr::Format,"%d",code));
+	KviKvsVariantList params;
+	params.append(new KviKvsVariant((kvs_int_t)code));
+	params.append(new KviKvsVariant(txt));
 
-	executeCallback();
+	execute(&params);
 
 	//QDialog::done(code);
 
 	delete this;
 }
 
-void KviScriptCallbackTextInput::showEvent(QShowEvent *e)
+void KviKvsCallbackTextInput::showEvent(QShowEvent *e)
 {
 	move((g_pApp->desktop()->width() - width())/2,(g_pApp->desktop()->height() - height()) / 2);
 	QDialog::showEvent(e);
 }
 
+
+
 /*
 	@doc: dialog.textinput
 	@type:
@@ -329,10 +379,17 @@ void KviScriptCallbackTextInput::showEvent(QShowEvent *e)
 	@short:
 		Shows a dialog that accepts user input as text
 	@syntax:
-		dialog.textinput [-d=<default text>] [-i=<icon>] [-m] (<caption>,<info_text>,<button0>[,<button1>[,<button2>[,<magic1>[,<magic2>[...]]]]])
+		dialog.textinput [-d=<default text>] [-i=<icon>] [-m] [-b] (<caption>,<info_text>,<button0>[,<button1>[,<button2>[,<magic1>[,<magic2>[...]]]]])
 		{
 			<callback_command>
 		}
+	@switches:
+		!sw: -d=<default_text> | --default=<default_text>
+		Set the initial text input value to <default_text>
+		!sw: -i=<icon> | --icon=<icon>
+		Display the specified icon, just of the left of the informational text
+		!sw: -m | --multiline
+		Input multiline text instead of single line
 	@description:
 		Shows a text input dialog box with the specified <caption> , <info_text> , <icon> and
 		buttons.[br]
@@ -341,10 +398,14 @@ void KviScriptCallbackTextInput::showEvent(QShowEvent *e)
 		<button0> is the text of the first button (on the left).[br]
 		<button1> is the text of the second button (if empty or not given at all, only one button will appear in the dialog).[br]
 		<button2> is the text of the third button (if empty or not given, only two buttons will appear in the dialog).[br]
+		If one of the text strings starts with a "default=" prefix then the button is assumed
+		to be the default button of the dialog and will be also activated when the user presses enter.
 		If the -m switch is used , the dialog will be a multi-line text input, otherwise the user will be able to
 		input only a single line of text.[br]
 		If the -d switch is used , the initial text input value is set to <default text>.[br]
-		If the -i switch is used , the dialog displays also the icon <icon> , just on the left ot the <info_text>
+		If the -i switch is used , the dialog displays also the icon <icon> , just on the left ot the <info_text>[br]
+		If the -b or -modal switch is specified the dialog will have non-blocking modal behaviour:
+		it will appear above its parent widget and block its input until it's closed.[br] 
 		In that case <icon> is an [doc:image_id]image identifier[/doc] (can be a relative or absolute
 		path to an image file or a signed number (in that case it defines an internal KVIrc image).[br]
 		<magic1>,<magic2>... are the magic parameters: evaluated at dialog.textinput call time and passed
@@ -372,36 +433,33 @@ void KviScriptCallbackTextInput::showEvent(QShowEvent *e)
 		[/example]
 */
 
-static bool dialog_module_cmd_textinput(KviModule *m,KviCommand *c)
+static bool dialog_kvs_cmd_textinput(KviKvsModuleCallbackCommandCall * c)
 {
-	ENTER_STACK_FRAME(c,"dialog_module_cmd_input");
+	QString szCaption,szInfoText,szIcon,szDefaultText,szButton0,szButton1,szButton2;
+	KviKvsVariantList params;
 
-	KviParameterList paramList;
-	paramList.setAutoDelete(true);
+	KVSM_PARAMETERS_BEGIN(c)
+		KVSM_PARAMETER("caption",KVS_PT_STRING,0,szCaption)
+		KVSM_PARAMETER("info_text",KVS_PT_STRING,0,szInfoText)
+		KVSM_PARAMETER("button0",KVS_PT_STRING,KVS_PF_OPTIONAL,szButton0)
+		KVSM_PARAMETER("button1",KVS_PT_STRING,KVS_PF_OPTIONAL,szButton1)
+		KVSM_PARAMETER("button2",KVS_PT_STRING,KVS_PF_OPTIONAL,szButton2)
+		KVSM_PARAMETER("magic",KVS_PT_VARIANTLIST,KVS_PF_OPTIONAL,params)
+	KVSM_PARAMETERS_END(c)
 
-	KviStr cmd;
+	QString szCmd = c->callback()->code();
 
-	if(!g_pUserParser->parseCallbackCommand(c,&paramList,&cmd))return false;
-
-	KviStr caption  = paramList.safeFirstParam();
-	KviStr infotext = paramList.safeNextParam();
-	KviStr defaultv;
-	KviStr icon;
-	KviStr button1  = paramList.safeNextParam();
-	KviStr button2  = paramList.safeNextParam();
-	KviStr button3  = paramList.safeNextParam();
-
-	if(c->hasSwitch('i'))c->getSwitchValue('i',icon);
-	if(c->hasSwitch('d'))c->getSwitchValue('d',defaultv);
-
-	KviParameterList * parms = new KviParameterList();
-	parms->setAutoDelete(true);
-	while(KviStr * n = paramList.next())parms->append(new KviStr(*n));
-
-	KviScriptCallbackTextInput * box = new KviScriptCallbackTextInput(caption,infotext,defaultv,icon,c->hasSwitch('m'),button1,button2,button3,parms,c->window(),cmd);
+	c->switches()->getAsStringIfExisting('i',"icon",szIcon);
+	c->switches()->getAsStringIfExisting('d',"default",szDefaultText);
+	bool modal;
+	if(c->hasSwitch('b',"modal")) modal=true;
+	else modal=false;
+	KviKvsCallbackTextInput * box = new KviKvsCallbackTextInput(
+			szCaption,szInfoText,szDefaultText,szIcon,c->switches()->find('m',"multiline"),
+			szButton0,szButton1,szButton2,szCmd,&params,c->window(),modal);
 	box->show();
 
-	return c->leaveStackFrame();
+	return true;
 }
 
 
@@ -409,20 +467,27 @@ static bool dialog_module_cmd_textinput(KviModule *m,KviCommand *c)
 //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
 
 
-KviScriptCallbackFileDialog::KviScriptCallbackFileDialog(KviStr &caption,KviStr &initial,KviStr &filter,KviParameterList * parms,KviWindow * pWnd,KviStr &code)
- : KviFileDialog(initial.ptr(),filter.ptr()) , KviScriptCallbackDialog(pWnd,code,parms)
+// New KVS
+KviKvsCallbackFileDialog::KviKvsCallbackFileDialog(
+		const QString &szCaption,
+		const QString &szInitialSelection,
+		const QString &szFilter,
+		const QString &szCode,
+		KviKvsVariantList * pMagicParams,
+		KviWindow * pWindow,bool modal)
+	: KviFileDialog(szInitialSelection,szFilter,0,0,modal), KviKvsCallbackObject("dialog.file",pWindow,szCode,pMagicParams,0)
 {
-	setCaption(caption.ptr());
+	setCaption(szCaption);
 }
 
-
-KviScriptCallbackFileDialog::~KviScriptCallbackFileDialog()
+KviKvsCallbackFileDialog::~KviKvsCallbackFileDialog()
 {
 }
 
-void KviScriptCallbackFileDialog::done(int code)
+void KviKvsCallbackFileDialog::done(int code)
 {
 	KviFileDialog::done(code);
+	KviKvsVariantList params;
 
 	if(code == QDialog::Accepted)
 	{
@@ -432,13 +497,20 @@ void KviScriptCallbackFileDialog::done(int code)
 		if(mode() == QFileDialog::ExistingFiles)
 #endif
 		{
-			KviStr joined = selectedFiles().join(",");
-			m_pParams->prepend(new KviStr(joined));
+			KviKvsArray * a = new KviKvsArray();
+			QStringList sl = selectedFiles();
+			int idx = 0;
+			for(QStringList::Iterator it = sl.begin();it != sl.end();++it)
+			{
+				a->set(idx,new KviKvsVariant(*it));
+				idx++;
+			}
+			params.append(new KviKvsVariant(a));
 		} else {
-			m_pParams->prepend(new KviStr(selectedFile()));
+			params.append(new KviKvsVariant(selectedFile()));
 		}
 	} else {
-		m_pParams->prepend(new KviStr());
+		params.append(new KviKvsVariant(QString("")));
 	}
 
 	hide(); // ensure we're hidden
@@ -454,10 +526,9 @@ void KviScriptCallbackFileDialog::done(int code)
 	// with a lot of qt versions that have the ugly file dialog "accept before this reference" bug)
 	// to avoid it, we can execute the callback triggered by a timer...
 	// ... umpf ...
-	executeCallback();
+	execute(&params);
 }
 
-
 /*
 	@doc: dialog.file
 	@type:
@@ -467,7 +538,7 @@ void KviScriptCallbackFileDialog::done(int code)
 	@short:
 		Shows a file dialog
 	@syntax:
-		dialog.file(<mode>,<caption>,<initial_selection>,<file_filter>[,<magic1>[,<magic2>[...]]]]])
+		dialog.file [-b] (<mode>,<caption>[,<initial_selection[,<file_filter>[,<magic1>[,<magic2>[...]]]]]])
 		{
 			<callback_command>
 		}
@@ -487,6 +558,8 @@ void KviScriptCallbackFileDialog::done(int code)
 		header files, the filter string could look like "C++ Files (*.cpp *.cc *.C *.cxx *.c++);;Header Files (*.h *.hxx *.h++)" 
 		<magic1>,<magic2>... are the magic parameters: evaluated at dialog.message call time and passed
 		to the <callback_command> as positional parameters.[br]
+		If the -b or -modal switch is specified the dialog will have non-blocking modal behaviour:
+		it will appear above its parent widget and block its input until it's closed.[br] 
 		Once the dialog has been shown , the user will select an EXISTING file and click either
 		Ok or Cancel. At this point the dialog is hidden and the <callback_command> is executed passing the selected file(s) as $0
 		and the magic parameters as positional parameters $1 , $2 , $3....[br]
@@ -502,66 +575,71 @@ void KviScriptCallbackFileDialog::done(int code)
 
 //#warning "Examples for these dialogs!"
 
-static bool dialog_module_cmd_file(KviModule *m,KviCommand *c)
+
+static bool dialog_kvs_cmd_file(KviKvsModuleCallbackCommandCall * c)
 {
-	ENTER_STACK_FRAME(c,"dialog_module_cmd_file");
+	QString szMode,szCaption,szInitialSelection,szFilter;
+	KviKvsVariantList params;
 
-	KviParameterList paramList;
-	paramList.setAutoDelete(true);
+	KVSM_PARAMETERS_BEGIN(c)
+		KVSM_PARAMETER("mode",KVS_PT_STRING,0,szMode)
+		KVSM_PARAMETER("caption",KVS_PT_STRING,0,szCaption)
+		KVSM_PARAMETER("initial_selection",KVS_PT_STRING,KVS_PF_OPTIONAL,szInitialSelection)
+		KVSM_PARAMETER("filter",KVS_PT_STRING,KVS_PF_OPTIONAL,szFilter)
+		KVSM_PARAMETER("magic",KVS_PT_VARIANTLIST,KVS_PF_OPTIONAL,params)
+	KVSM_PARAMETERS_END(c)
+	bool modal;
+	if(c->hasSwitch('b',"modal")) modal=true;
+	else modal=false;
+	QString szCmd = c->callback()->code();
 
-	KviStr cmd;
-
-	if(!g_pUserParser->parseCallbackCommand(c,&paramList,&cmd))return false;
-
-	KviStr mode    = paramList.safeFirstParam();
-	KviStr caption = paramList.safeNextParam();
-	KviStr initial = paramList.safeNextParam();
-	KviStr filter  = paramList.safeNextParam();
-
-	KviParameterList * parms = new KviParameterList();
-	parms->setAutoDelete(true);
-	while(KviStr * n = paramList.next())parms->append(new KviStr(*n));
-
-	KviScriptCallbackFileDialog * box = new KviScriptCallbackFileDialog(caption,initial,filter,parms,c->window(),cmd);
+	KviKvsCallbackFileDialog * box = new KviKvsCallbackFileDialog(szCaption,szInitialSelection,szFilter,szCmd,&params,c->window(),modal);
 
 	KviFileDialog::FileMode md = KviFileDialog::ExistingFile;
 
-	if(kvi_strEqualCI(mode.ptr(),"openm"))md = KviFileDialog::ExistingFiles;
-	else if(kvi_strEqualCI(mode.ptr(),"save"))md = KviFileDialog::AnyFile;
-	else if(kvi_strEqualCI(mode.ptr(),"dir"))md = KviFileDialog::DirectoryOnly;
+	if(KviQString::equalCI(szMode,"openm"))md = KviFileDialog::ExistingFiles;
+	else if(KviQString::equalCI(szMode,"save"))md = KviFileDialog::AnyFile;
+	else if(KviQString::equalCI(szMode,"dir"))md = KviFileDialog::DirectoryOnly;
 
 	box->setFileMode(md);
 
 	box->show();
 
-	return c->leaveStackFrame();
+	return true;
 }
 
 
 
-
 //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
 
 
-KviScriptCallbackImageDialog::KviScriptCallbackImageDialog(KviStr &caption,KviStr &szInitial,int iType,int iMaxSize,KviParameterList * parms,KviWindow * pWnd,KviStr &code)
- : KviImageDialog(0,caption.ptr(),iType,0,szInitial.ptr(),iMaxSize) , KviScriptCallbackDialog(pWnd,code,parms)
+// New KVS
+KviKvsCallbackImageDialog::KviKvsCallbackImageDialog(
+		const QString &szCaption,
+		const QString &szInitialSelection,
+		int iType,
+		int iMaxSize,
+		const QString &szCode,
+		KviKvsVariantList * pMagicParams,
+		KviWindow * pWindow,bool modal)
+	: KviImageDialog(0,szCaption,iType,0,szInitialSelection,iMaxSize,modal), KviKvsCallbackObject("dialog.image",pWindow,szCode,pMagicParams,0)
 {
 }
 
-
-KviScriptCallbackImageDialog::~KviScriptCallbackImageDialog()
+KviKvsCallbackImageDialog::~KviKvsCallbackImageDialog()
 {
 }
 
-void KviScriptCallbackImageDialog::done(int code)
+void KviKvsCallbackImageDialog::done(int code)
 {
 	KviImageDialog::done(code);
+	KviKvsVariantList params;
 
 	if(code == QDialog::Accepted)
 	{
-		m_pParams->prepend(new KviStr(selectedImage()));
+		params.append(new KviKvsVariant(selectedImage()));
 	} else {
-		m_pParams->prepend(new KviStr());
+		params.append(new KviKvsVariant(QString("")));
 	}
 
 	hide(); // ensure we're hidden
@@ -577,7 +655,7 @@ void KviScriptCallbackImageDialog::done(int code)
 	// with a lot of qt versions that have the ugly file dialog "accept before this reference" bug)
 	// to avoid it, we can execute the callback triggered by a timer...
 	// ... umpf ...
-	executeCallback();
+	execute(&params);
 }
 
 
@@ -590,7 +668,7 @@ void KviScriptCallbackImageDialog::done(int code)
 	@short:
 		Shows a image dialog
 	@syntax:
-		dialog.image(<type>,<caption>,<initial_directory>,[<maxsize>,[,<magic1>[,<magic2>[...]]]]]])
+		dialog.image [-b] (<type>,<caption>,<initial_directory>,[<maxsize>,[,<magic1>[,<magic2>[...]]]]]])
 		{
 			<callback_command>
 		}
@@ -609,6 +687,8 @@ void KviScriptCallbackImageDialog::done(int code)
 		the thumbnails for bigger images (and it can eat a considerable amount of memory).<br>
 		<magic1>,<magic2>... are the magic parameters: evaluated at dialog.image call time and passed
 		to the <callback_command> as positional parameters.[br]
+		If the -b or -modal switch is specified the dialog will have non-blocking modal behaviour:
+		it will appear above its parent widget and block its input until it's closed.[br] 
 		Once the dialog has been shown , the user will select an EXISTING file and click either
 		Ok or Cancel. At this point the dialog is hidden and the <callback_command> is executed passing the selected file(s) as $0
 		and the magic parameters as positional parameters $1 , $2 , $3....[br]
@@ -624,44 +704,73 @@ void KviScriptCallbackImageDialog::done(int code)
 
 //#warning "Examples for these dialogs!"
 
-static bool dialog_module_cmd_image(KviModule *m,KviCommand *c)
+
+static bool dialog_kvs_cmd_image(KviKvsModuleCallbackCommandCall * c)
 {
-	ENTER_STACK_FRAME(c,"dialog_module_cmd_image");
+	QString szType,szCaption,szInitialSelection;
+	kvs_uint_t iMaxSize;
+	KviKvsVariantList params;
 
-	KviParameterList paramList;
-	paramList.setAutoDelete(true);
-
-	KviStr cmd;
-
-	if(!g_pUserParser->parseCallbackCommand(c,&paramList,&cmd))return false;
-
-	KviStr type    = paramList.safeFirstParam();
-	KviStr caption = paramList.safeNextParam();
-	KviStr initial = paramList.safeNextParam();
-	KviStr maxsize = paramList.safeNextParam();
-
-	KviParameterList * parms = new KviParameterList();
-	parms->setAutoDelete(true);
-	while(KviStr * n = paramList.next())parms->append(new KviStr(*n));
+	KVSM_PARAMETERS_BEGIN(c)
+		KVSM_PARAMETER("mode",KVS_PT_STRING,0,szType)
+		KVSM_PARAMETER("caption",KVS_PT_STRING,0,szCaption)
+		KVSM_PARAMETER("initial_directory",KVS_PT_STRING,0,szInitialSelection)
+		KVSM_PARAMETER("maxsize",KVS_PT_UINT,KVS_PF_OPTIONAL,iMaxSize)
+		KVSM_PARAMETER("magic",KVS_PT_VARIANTLIST,KVS_PF_OPTIONAL,params)
+	KVSM_PARAMETERS_END(c)
+bool modal;
+	if(c->hasSwitch('b',"modal")) modal=true;
+	else modal=false;
+	QString szCmd = c->callback()->code();
 
 	int iType = 0;
 
-	if(type.contains('s'))iType |= KID_TYPE_BUILTIN_IMAGES_SMALL;
-	if(type.contains('f'))iType |= KID_TYPE_FULL_PATH;
-	if(type.isEmpty())iType = KID_TYPE_ALL;
+	if(szType.contains('s'))iType |= KID_TYPE_BUILTIN_IMAGES_SMALL;
+	if(szType.contains('f'))iType |= KID_TYPE_FULL_PATH;
+	if(szType.isEmpty())iType = KID_TYPE_ALL;
 
-	bool bOk;
-	int iSize = maxsize.toInt(&bOk);
-	if(!bOk || (iSize < 1))iSize = 256000;
+	if(iMaxSize < 1)iMaxSize = 256000;
 
-	KviScriptCallbackImageDialog * box = new KviScriptCallbackImageDialog(caption,initial,iType,iSize,parms,c->window(),cmd);
+	KviKvsCallbackImageDialog * box = new KviKvsCallbackImageDialog(szCaption,szInitialSelection,iType,iMaxSize,szCmd,&params,c->window(),modal);
 
 	box->show();
 
-	return c->leaveStackFrame();
+	return true;
 }
 
+/*
+	@doc: dialog.yesno
+	@type:
+		function
+	@title:
+		$dialog.yesno
+	@short:
+		Shows a simple yes/no dialog
+	@syntax:
+		$dialog.yesno(<caption:string>,<szText:string>)
+	@description:
+		Shows a simple yes/no dialog. Returns 1 if user clicks "Yes" and 0 if (s)he clicks "No".
+		Please note that this dialog is BLOCKING: it blocks execution of the script
+		until the user has selected either YES or NO.
+	@examples:
+	@seealso:
+*/
 
+static bool dialog_kvs_fnc_yesno(KviKvsModuleFunctionCall * c)
+{ 
+	QString szCaption;
+	QString szText;
+	KVSM_PARAMETERS_BEGIN(c)
+		KVSM_PARAMETER("caption",KVS_PT_STRING,0,szCaption)
+		KVSM_PARAMETER("text",KVS_PT_STRING,0,szText)
+	KVSM_PARAMETERS_END(c)
+	
+	c->enterBlockingSection();
+	bool yes=KviMessageBox::yesNo(szCaption,szText);
+	if(!c->leaveBlockingSection())return true; // just die
+	c->returnValue()->setBoolean(yes);
+	return true;
+}
 
 /*
 
@@ -753,15 +862,16 @@ static bool dialog_module_fnc_textline(KviModule *m,KviCommand *c,KviParameterLi
 
 static bool dialog_module_init(KviModule * m)
 {
-	g_pDialogModuleDialogList = new KviPtrList<KviScriptCallbackDialog>;
-	g_pDialogModuleDialogList->setAutoDelete(false);
+//	g_pDialogModuleDialogList = new KviPtrList<KviScriptCallbackDialog>;
+//	g_pDialogModuleDialogList->setAutoDelete(false);
 
-//	m->registerFunction("textline",dialog_module_fnc_textline);
 
-	m->registerCommand("message",dialog_module_cmd_message);
-	m->registerCommand("file",dialog_module_cmd_file);
-	m->registerCommand("textinput",dialog_module_cmd_textinput);
-	m->registerCommand("image",dialog_module_cmd_image);
+	KVSM_REGISTER_CALLBACK_COMMAND(m,"message",dialog_kvs_cmd_message);
+	KVSM_REGISTER_CALLBACK_COMMAND(m,"textinput",dialog_kvs_cmd_textinput);
+	KVSM_REGISTER_CALLBACK_COMMAND(m,"file",dialog_kvs_cmd_file);
+	KVSM_REGISTER_CALLBACK_COMMAND(m,"image",dialog_kvs_cmd_image);
+
+	KVSM_REGISTER_FUNCTION(m,"yesno",dialog_kvs_fnc_yesno);
 
     return true;
 }
@@ -769,20 +879,15 @@ static bool dialog_module_init(KviModule * m)
 static bool dialog_module_cleanup(KviModule *m)
 {
 	// Here we get a tragedy if g_iLocalEventLoops > 0!
-	while(g_pDialogModuleDialogList->first())delete g_pDialogModuleDialogList->first();
-	delete g_pDialogModuleDialogList;
-    g_pDialogModuleDialogList = 0;
-
-	m->unregisterMetaObject("KviScriptCallbackMessageBox");
-	m->unregisterMetaObject("KviScriptCallbackFileDialog");
-	m->unregisterMetaObject("KviScriptCallbackTextInput");
-	m->unregisterMetaObject("KviScriptCallbackImageDialog");
+//	while(g_pDialogModuleDialogList->first())delete g_pDialogModuleDialogList->first();
+//	delete g_pDialogModuleDialogList;
+//    g_pDialogModuleDialogList = 0;
 	return true;
 }
 
 static bool dialog_module_can_unload(KviModule *m)
 {
-	return g_pDialogModuleDialogList->isEmpty();
+	return 1;
 }
 
 
