@@ -26,16 +26,20 @@
 #include "kvi_module.h"
 #include "kvi_locale.h"
 #include "kvi_memmove.h"
-#include "kvi_uparser.h"
+//#include "kvi_uparser.h"
 #include "kvi_taskbar.h"
 #include "kvi_window.h"
 #include "kvi_dynamictooltip.h"
 #include "kvi_iconmanager.h"
 #include "kvi_internalcmd.h"
-
+#include "kvi_console.h"
+#include "kvi_ircconnection.h"
+#include "kvi_ircconnectionuserinfo.h"
 #include "kvi_imagelib.h"
+#include "kvi_options.h"
 
 #include "libkvidockwidget.h"
+#include "kvi_doublebuffer.h"
 
 #ifdef COMPILE_KDE_SUPPORT
 	#include <kwin.h>
@@ -47,6 +51,7 @@
 
 #include <qpixmap.h>
 #include <qpainter.h>
+#include <qtimer.h>
 
 #include <qregexp.h>
 
@@ -61,8 +66,10 @@
 
 	#define ID_DOCKWIDGET_TASKBAR_ICON 0xdeadbeef
 	#define WM_KVIRC_NOTIFY_ICON_MESSAGE (WM_USER + 0xbeef)
-#else
 
+	static UINT WM_KVIRC_TASKBAR_CREATED = 0;
+#else
+#ifndef Q_OS_MACX
 	#include <X11/Xlib.h>
 
 	//const int XFocusOut = FocusOut;
@@ -77,7 +84,9 @@
 		#undef Bool
 	#endif
 #endif
+#endif
 
+extern KVIRC_API QAsciiDict<KviWindow> * g_pGlobalWindowDict;
 static KviPtrList<KviDockWidget> * g_pDockWidgetList = 0;
 
 static QPixmap * g_pDock1 = 0;
@@ -85,19 +94,29 @@ static QPixmap * g_pDock2 = 0;
 static QPixmap * g_pDock3 = 0;
 
 #ifdef COMPILE_ON_WINDOWS
-	static HICON g_hDock[6];
-	static int   g_iCurLevel = -1;
+	static HICON g_pCurrentIcon = 0;
+	static QPixmap * g_pCurrentPixmap = 0;
 #endif
 
 
 KviDockWidget::KviDockWidget(KviFrame * frm,const char * name)
 : QWidget(0,name)
 {
+	m_iConsoles = 0;
+	m_iChannels = 0;
+	m_iQueries  = 0;
+	m_iOther    = 0;
+	
+	m_pFlashingTimer = new QTimer(this,"flashing_timer");
+	connect( m_pFlashingTimer, SIGNAL(timeout()), this, SLOT(flashingTimerShot()) );
+	m_bFlashed=0;
 	g_pDockWidgetList->append(this);
 	m_pFrm = frm;
 	m_pFrm->setDockExtension(this);
 
 #ifdef COMPILE_ON_WINDOWS
+	// kode54
+	if(!WM_KVIRC_TASKBAR_CREATED)WM_KVIRC_TASKBAR_CREATED = RegisterWindowMessage(TEXT("TaskbarCreated"));
 	createTaskbarIcon();
 #else //!COMPILE_ON_WINDOWS
 	setMinimumSize(24,24);
@@ -108,7 +127,9 @@ KviDockWidget::KviDockWidget(KviFrame * frm,const char * name)
 
 	m_pTip = new KviDynamicToolTip(this,"dock_tooltip");
 	connect(m_pTip,SIGNAL(tipRequest(KviDynamicToolTip *,const QPoint &)),this,SLOT(tipRequest(KviDynamicToolTip *,const QPoint &)));
-
+	
+	m_pAwayPopup = new QPopupMenu(this);
+	
 #ifdef COMPILE_KDE_SUPPORT
 	m_pContextPopup = new KPopupMenu(this);
 	m_pContextPopup->insertTitle(*(g_pIconManager->getSmallIcon(KVI_SMALLICON_KVIRC)),__tr2qs("KVIrc"));
@@ -119,6 +140,9 @@ KviDockWidget::KviDockWidget(KviFrame * frm,const char * name)
 	m_pContextPopup->insertItem(l);
 #endif
 	m_pContextPopup->setCaption(__tr2qs("Context"));
+	m_iAwayMenuId = m_pContextPopup->insertItem ( __tr2qs("Away"), m_pAwayPopup);
+	m_pContextPopup->changeItem(m_iAwayMenuId,*(g_pIconManager->getSmallIcon(KVI_SMALLICON_AWAY)),__tr2qs("Away"));
+	
 	int id = m_pContextPopup->insertItem(*(g_pIconManager->getSmallIcon(KVI_SMALLICON_OPTIONS)),__tr2qs("&Configure KVIrc..."),m_pFrm,SLOT(executeInternalCommand(int)));
 	m_pContextPopup->setItemParameter(id,KVI_INTERNALCOMMAND_OPTIONS_DIALOG);
 	id = m_pContextPopup->insertItem(*(g_pIconManager->getSmallIcon(KVI_SMALLICON_KVIRC)),__tr2qs("&About KVIrc"),m_pFrm,SLOT(executeInternalCommand(int)));
@@ -147,11 +171,26 @@ void KviDockWidget::die()
 {
 	delete this;
 }
-
+void KviDockWidget::flashingTimerShot()
+{
+	m_bFlashed=!m_bFlashed;
+#ifdef COMPILE_ON_WINDOWS
+	updateTaskbarIcon();
+#else
+	update();
+#endif
+}
 
 #ifdef COMPILE_ON_WINDOWS
 void KviDockWidget::createTaskbarIcon()
 {
+	ICONINFO inf;
+	g_pCurrentPixmap=new QPixmap(*g_pDock1);
+	inf.hbmColor = g_pCurrentPixmap->hbm();
+	if(!g_pCurrentPixmap->mask())g_pCurrentPixmap->setMask(g_pCurrentPixmap->createHeuristicMask());
+	inf.hbmMask = g_pCurrentPixmap->mask()->hbm();
+	g_pCurrentIcon=CreateIconIndirect(&inf);
+
 	NOTIFYICONDATA nid;
 	nid.cbSize = sizeof(nid);
 	nid.hWnd = winId();
@@ -159,9 +198,8 @@ void KviDockWidget::createTaskbarIcon()
 	nid.uFlags = NIF_ICON | NIF_MESSAGE | NIF_TIP;
 	nid.uCallbackMessage = WM_KVIRC_NOTIFY_ICON_MESSAGE;
 	kvi_memmove(nid.szTip,"KVIrc",6);
-	nid.hIcon  = g_hDock[0];
+	nid.hIcon  = g_pCurrentIcon;
 	Shell_NotifyIcon(NIM_ADD,&nid);
-	g_iCurLevel = 0;
 }
 
 void KviDockWidget::destroyTaskbarIcon()
@@ -174,15 +212,24 @@ void KviDockWidget::destroyTaskbarIcon()
 	Shell_NotifyIcon(NIM_DELETE,&nid);
 }
 
-void KviDockWidget::setTaskbarIcon(void * hIcon)
+void KviDockWidget::updateTaskbarIcon()
 {
+	ICONINFO inf;
+	QPixmap* pix = m_bFlashed ? g_pIconManager->getSmallIcon(KVI_SMALLICON_MESSAGE) : g_pCurrentPixmap;
+	inf.hbmColor = pix->hbm();
+	if(!pix->mask())pix->setMask(pix->createHeuristicMask());
+	inf.hbmMask = pix->mask()->hbm();
+	HICON hIcon=CreateIconIndirect(&inf);
+
 	NOTIFYICONDATA nid;
 	nid.cbSize = sizeof(nid);
 	nid.hWnd = winId();
 	nid.uID = ID_DOCKWIDGET_TASKBAR_ICON;
 	nid.uFlags = NIF_ICON;
-	nid.hIcon = (HICON)hIcon;
+	nid.hIcon = hIcon;
 	Shell_NotifyIcon(NIM_MODIFY,&nid);
+	DestroyIcon(g_pCurrentIcon);
+	g_pCurrentIcon=hIcon;
 }
 
 bool KviDockWidget::winEvent(MSG * m)
@@ -205,6 +252,13 @@ bool KviDockWidget::winEvent(MSG * m)
 			}
 			return true;
 		}
+	} else if(m->message == WM_KVIRC_TASKBAR_CREATED)
+	{
+		// kode54 : when the window shell dies and is restarted
+		// it sends the "TaskbarCreated" message to all the toplevel windows
+		// We recreate the taskbar icon then
+		createTaskbarIcon();
+		refresh();
 	}
 	return false;
 }
@@ -271,15 +325,18 @@ void KviDockWidget::tipRequest(KviDynamicToolTip *tip,const QPoint &pnt)
 	{
 		if(b->highlightLevel() > 0)
 		{
-			line = b->window()->lastLineOfText();
-			line.replace(QChar('&'),"&amp;");
-			line.replace(QChar('<'),"&lt;");
-			line.replace(QChar('>'),"&gt;");
-			tmp += "<b>";
-			tmp += b->window()->plainTextCaption();
-			tmp += "</b><br>";
-			tmp += line;
-			tmp += "<br><br>\n";
+			line = b->window()->lastMessageText();
+			if(!line.isEmpty())
+			{
+				line.replace(QChar('&'),"&amp;");
+				line.replace(QChar('<'),"&lt;");
+				line.replace(QChar('>'),"&gt;");
+				tmp += "<b>";
+				tmp += b->window()->plainTextCaption();
+				tmp += "</b><br>";
+				tmp += line;
+				tmp += "<br><br>\n";
+			}
 		}
 	}
 
@@ -307,63 +364,159 @@ void KviDockWidget::tipRequest(KviDynamicToolTip *tip,const QPoint &pnt)
 void KviDockWidget::mousePressEvent(QMouseEvent *e)
 {
 	if(e->button() & LeftButton)toggleParentFrame();
-	else if(e->button() & RightButton)m_pContextPopup->popup(mapToGlobal(e->pos()));
+	else if(e->button() & RightButton)
+		m_pContextPopup->popup(mapToGlobal(e->pos()));
+}
+
+void KviDockWidget::doAway(int id)
+{
+	if(id<0)
+	{
+		QAsciiDictIterator<KviWindow> it(*g_pGlobalWindowDict);
+		while(KviWindow * wnd = it.current())
+		{
+			if(wnd->type()==KVI_WINDOW_TYPE_CONSOLE)
+			{
+				KviConsole* pConsole=(KviConsole*)wnd;
+				if(pConsole->isConnected())
+					if(id==-2)
+						pConsole->connection()->sendFmtData("AWAY");
+					else
+						pConsole->connection()->sendFmtData("AWAY :%s",
+							pConsole->connection()->encodeText(KVI_OPTION_STRING(KviOption_stringAwayMessage)).data()
+							);
+			}
+ 			++it;
+		}
+	} else {
+		KviConsole* pConsole=g_pApp->findConsole((unsigned int)id);
+		if(pConsole)
+			if(pConsole->isConnected())
+				if(pConsole->connection()->userInfo()->isAway())
+					pConsole->connection()->sendFmtData("AWAY");
+				else
+					pConsole->connection()->sendFmtData("AWAY :%s",
+						pConsole->connection()->encodeText(KVI_OPTION_STRING(KviOption_stringAwayMessage)).data()
+						);
+	}
 }
 
 void KviDockWidget::fillContextPopup()
 {
 	m_pContextPopup->changeItem(m_iToggleFrame,m_pFrm->isVisible() ? __tr2qs("Hide Window") : __tr2qs("Show Window"));
+	if(g_pApp->topmostConnectedConsole())
+	{
+		m_pContextPopup->setItemVisible(m_iAwayMenuId,true);
+		m_pAwayPopup->clear();
+		
+		int iAllAway=m_pAwayPopup->insertItem(*(g_pIconManager->getSmallIcon(KVI_SMALLICON_CONSOLE)),__tr2qs("Away on all"),this,SLOT(doAway(int)));
+		m_pAwayPopup->setItemParameter(iAllAway,-1);
+		
+		int iAllUnaway=m_pAwayPopup->insertItem(*(g_pIconManager->getSmallIcon(KVI_SMALLICON_CONSOLE)),__tr2qs("Back on all"),this,SLOT(doAway(int)));
+		m_pAwayPopup->setItemParameter(iAllUnaway,-2);
+		
+		int iSeparator=m_pAwayPopup->insertSeparator();
+		
+		QAsciiDictIterator<KviWindow> it(*g_pGlobalWindowDict);
+		bool bAllAway=1;
+		bool bAllUnaway=1;
+		int iNetCount=0;
+		while(KviWindow * wnd = it.current())
+		{
+			if(wnd->type()==KVI_WINDOW_TYPE_CONSOLE)
+			{
+				KviConsole* pConsole=(KviConsole*)wnd;
+				if(pConsole->isConnected())
+				{
+					int id;
+					if(pConsole->connection()->userInfo()->isAway())
+					{
+						id=m_pAwayPopup->insertItem(*(g_pIconManager->getSmallIcon(KVI_SMALLICON_CONSOLE)),__tr2qs("Back on %1").arg(pConsole->currentNetworkName()),this,SLOT(doAway(int)));
+						bAllUnaway=0;
+					} else {
+						id=m_pAwayPopup->insertItem(*(g_pIconManager->getSmallIcon(KVI_SMALLICON_CONSOLE)),__tr2qs("Away on %1").arg(pConsole->currentNetworkName()),this,SLOT(doAway(int)));
+						bAllAway=0;
+					}
+					m_pAwayPopup->setItemParameter(id,pConsole->ircContextId());
+					iNetCount++;
+				}
+			}
+ 			++it;
+		}
+		if(iNetCount==1)
+		{
+			m_pAwayPopup->setItemVisible(iAllAway,false);
+			m_pAwayPopup->setItemVisible(iAllUnaway,false);
+			m_pAwayPopup->setItemVisible(iSeparator,false);
+		} else {
+			m_pAwayPopup->setItemVisible(iAllAway,!bAllAway);
+			m_pAwayPopup->setItemVisible(iAllUnaway,!bAllUnaway);
+		}
+	} else {
+		m_pContextPopup->setItemVisible(m_iAwayMenuId,false);
+	}
 }
 
 void KviDockWidget::toggleParentFrame()
 {
-	if(m_pFrm->isVisible())m_pFrm->hide();
+	if(m_pFrm->isVisible())
+	{
+#if QT_VERSION > 0x030201
+		setPrevWindowState(m_pFrm->windowState());
+#endif
+		m_pFrm->hide();
+	}
 	else
 	{
 		m_pFrm->show();
+#if QT_VERSION > 0x030201
+		m_pFrm->setWindowState(getPrevWindowState());
+#endif
 		m_pFrm->raise();
 		m_pFrm->setFocus();
 	}
+
+	if( m_pFrm->isMinimized() )
+	{
+	    if( m_pFrm->isMaximized() )
+		m_pFrm->showMaximized();
+	    else
+		m_pFrm->showNormal();
+	}
 }
 
 void KviDockWidget::refresh()
 {
+	grabActivityInfo();
+
 #ifdef COMPILE_ON_WINDOWS
-	int max = 0;
-
-
-	KviTaskBarBase * t = m_pFrm->taskBar();
-
-	for(KviTaskBarItem * b = t->firstItem();b;b = t->nextItem())
+	bitBlt(g_pCurrentPixmap,0,0,m_iQueries ? ((m_iQueries == 2) ? g_pDock3 : g_pDock2) : g_pDock1,0,0,16,8,Qt::CopyROP,false);
+	bitBlt(g_pCurrentPixmap,0,8,m_iChannels ? ((m_iChannels == 2) ? g_pDock3 : g_pDock2) : g_pDock1,0,8,16,8,Qt::CopyROP,false);
+#endif	
+	
+	if( (m_iChannels == 2) || (m_iQueries == 2) ) 
 	{
-		int iLevel = b->highlightLevel();
-		if(iLevel > max)max = iLevel;
+		if(!m_pFlashingTimer->isActive() && !KVI_OPTION_BOOL(KviOption_boolDisableTrayIconFlashing) ) m_pFlashingTimer->start(1000);
+	} else {
+		if(m_pFlashingTimer->isActive()) m_pFlashingTimer->stop();
+		m_bFlashed=false;
 	}
-
-	if(max > 5)max = 5;
-
-	if(g_iCurLevel != max)
-	{
-		setTaskbarIcon((void *)(g_hDock[max]));
-		g_iCurLevel = max;
-	}
-
+	
+#ifdef COMPILE_ON_WINDOWS
+	updateTaskbarIcon();
 #else
 	update();
 #endif
 }
 
-
-void KviDockWidget::paintEvent(QPaintEvent *)
+void KviDockWidget::grabActivityInfo()
 {
-	int consoles = 0; // 0=nothing , 1=new text , 2=alt text
-	int channels = 0;
-	int queries  = 0;
-	int other    = 0;
-
 	KviTaskBarBase * t = m_pFrm->taskBar();
 
-	// 0=nothing , 1=new text , 2=alt text
+	m_iConsoles = 0;
+	m_iChannels = 0;
+	m_iQueries  = 0;
+	m_iOther    = 0;
 
 	for(KviTaskBarItem * b = t->firstItem();b;b = t->nextItem())
 	{
@@ -371,24 +524,34 @@ void KviDockWidget::paintEvent(QPaintEvent *)
 		switch(b->window()->type())
 		{
 			case KVI_WINDOW_TYPE_CONSOLE:
-				if((consoles < 2) && (iLevel > 0))consoles = iLevel > 3 ? 2 : 1;
+				if((m_iConsoles < 2) && (iLevel > 0))m_iConsoles = iLevel > 3 ? 2 : 1;
 			break;
 			case KVI_WINDOW_TYPE_CHANNEL:
-				if((channels < 2) && (iLevel > 0))channels = iLevel > 3 ? 2 : 1;
+				if((m_iChannels < 2) && (iLevel > 0))m_iChannels = iLevel > 3 ? 2 : 1;
 			break;
 			case KVI_WINDOW_TYPE_QUERY:
-				if((queries < 2) && (iLevel > 0))queries = iLevel > 3 ? 2 : 1;
+				if((m_iQueries < 2) && (iLevel > 0))m_iQueries = iLevel > 3 ? 2 : 1;
 			break;
 			default:
-				if((other < 2) && (iLevel > 0))other = iLevel > 3 ? 2 : 1;
+				if((m_iOther < 2) && (iLevel > 0))m_iOther = iLevel > 3 ? 2 : 1;
 			break;
 		}
 	}
+}
 
-	bitBlt(this,0,0,other ? ((other == 2) ? g_pDock3 : g_pDock2) : g_pDock1,0,0,12,12,Qt::CopyROP,true);
-	bitBlt(this,0,12,consoles ? ((consoles == 2) ? g_pDock3 : g_pDock2) : g_pDock1,0,12,12,12,Qt::CopyROP,true);
-	bitBlt(this,12,0,queries ? ((queries == 2) ? g_pDock3 : g_pDock2) : g_pDock1,12,0,12,12,Qt::CopyROP,true);
-	bitBlt(this,12,12,channels ? ((channels == 2) ? g_pDock3 : g_pDock2) : g_pDock1,12,12,12,12,Qt::CopyROP,true);
+void KviDockWidget::paintEvent(QPaintEvent * event)
+{
+	if(m_bFlashed)
+	{
+		erase();
+		bitBlt(this,4,4,g_pIconManager->getSmallIcon(KVI_SMALLICON_MESSAGE),0,0,16,16);
+	} else {
+
+		bitBlt(this,0,0,m_iOther ? ((m_iOther == 2) ? g_pDock3 : g_pDock2) : g_pDock1,0,0,12,12,Qt::CopyROP,false);
+		bitBlt(this,0,12,m_iConsoles ? ((m_iConsoles == 2) ? g_pDock3 : g_pDock2) : g_pDock1,0,12,12,12,Qt::CopyROP,false);
+		bitBlt(this,12,0,m_iQueries ? ((m_iQueries == 2) ? g_pDock3 : g_pDock2) : g_pDock1,12,0,12,12,Qt::CopyROP,false);
+		bitBlt(this,12,12,m_iChannels ? ((m_iChannels == 2) ? g_pDock3 : g_pDock2) : g_pDock1,12,12,12,12,Qt::CopyROP,false);
+	}
 }
 
 
@@ -436,13 +599,8 @@ static KviDockWidget * dockwidget_find(KviFrame *f)
 		[cmd]dockwidget.hide[/doc]
 */
 
-static bool dockwidget_module_command_show(KviModule *m,KviCommand *c)
-{
-	ENTER_STACK_FRAME(c,"dockwidget_module_cmd_show");
-
-	KviStr dummy;
-	if(!g_pUserParser->parseCmdFinalPart(c,dummy))return false;
-
+static bool dockwidget_kvs_cmd_show(KviKvsModuleCommandCall * c)
+{ 
 	if(!(dockwidget_find(c->window()->frame())))
 	{
 		KviDockWidget * w = new KviDockWidget(c->window()->frame(),"dock_widget");
@@ -452,8 +610,7 @@ static bool dockwidget_module_command_show(KviModule *m,KviCommand *c)
 		w->hide();
 #endif
 	}
-
-    return c->leaveStackFrame();
+	return true;
 }
 
 /*
@@ -472,13 +629,8 @@ static bool dockwidget_module_command_show(KviModule *m,KviCommand *c)
 		[cmd]dockwidget.show[/doc]
 */
 
-static bool dockwidget_module_command_hide(KviModule *m,KviCommand *c)
-{
-	ENTER_STACK_FRAME(c,"dockwidget_module_cmd_hide");
-
-	KviStr dummy;
-	if(!g_pUserParser->parseCmdFinalPart(c,dummy))return false;
-
+static bool dockwidget_kvs_cmd_hide(KviKvsModuleCommandCall * c)
+{ 
 	KviDockWidget * w= dockwidget_find(c->window()->frame());
 	if(w)delete w;
 	// show the parent frame.. otherwise there will be no way to get it back
@@ -486,8 +638,36 @@ static bool dockwidget_module_command_hide(KviModule *m,KviCommand *c)
 	{
 		c->window()->frame()->show();
 	}
+	return true;
+}
 
-    return c->leaveStackFrame();
+/*
+	@doc: dockwidget.hidewindow
+	@type:
+		command
+	@title:
+		dockwidget.hidewindow
+	@short:
+		Returns the state of the dock widget
+	@syntax:
+		dockwidget.hidewindow
+	@description:
+		Hides the window, assotiated with dockwidget
+	@seealso:
+		[cmd]dockwidget.show[/cmd] [cmd]dockwidget.hide[/cmd]
+*/
+
+static bool dockwidget_kvs_cmd_hidewindow(KviKvsModuleCommandCall * c)
+{ 
+	KviDockWidget * w= dockwidget_find(c->window()->frame());
+	if(w)
+	{
+#if QT_VERSION > 0x030201
+		w->setPrevWindowState(c->window()->frame()->windowState());
+#endif
+		c->window()->frame()->hide();
+	}
+	return true;
 }
 
 /*
@@ -506,48 +686,44 @@ static bool dockwidget_module_command_hide(KviModule *m,KviCommand *c)
 		[cmd]dockwidget.show[/cmd]
 */
 
-static bool dockwidget_module_function_isvisible(KviModule *m,KviCommand *c,KviParameterList * parms,KviStr &buffer)
-{
-//	ENTER_STACK_FRAME(c,"dockwidget_module_fnc_isVisible");
-	buffer.append(dockwidget_find(c->window()->frame()) ? '1' : '0');
+static bool dockwidget_kvs_fnc_isvisible(KviKvsModuleFunctionCall * c)
+{ 
+	c->returnValue()->setBoolean(dockwidget_find(c->window()->frame()));
 	return true;
 }
 
 
+
 // =======================================
 // init routine
 // =======================================
 static bool dockwidget_module_init(KviModule * m)
 {
 	KviStr buffer;
+#ifdef COMPILE_ON_WINDOWS
+	g_pApp->findImage(buffer,"kvi_dock_win32.png");
+	KviImageLibrary l1(buffer.ptr(),16,16);
+#else
 	g_pApp->findImage(buffer,"kvi_dock.png");
-
 	KviImageLibrary l1(buffer.ptr(),24,24);
-
+#endif
 	g_pDock1 = new QPixmap(l1.getImage(0));
 	g_pDock2 = new QPixmap(l1.getImage(1));
 	g_pDock3 = new QPixmap(l1.getImage(2));
 
 #ifdef COMPILE_ON_WINDOWS
-	ICONINFO inf;
-	for(int i=0;i<6;i++)
-	{
-		inf.fIcon = true;
-		QPixmap * pix = g_pIconManager->getSmallIcon(KVI_SMALLICON_IRC0 + i);
-		inf.hbmColor = pix->hbm();
-		if(!pix->mask())pix->setMask(pix->createHeuristicMask());
-		inf.hbmMask = pix->mask()->hbm();
-		g_hDock[i]  = CreateIconIndirect(&inf);
-	}
+	
 #endif
 
 
 	g_pDockWidgetList = new KviPtrList<KviDockWidget>;
 	g_pDockWidgetList->setAutoDelete(false);
 
-	m->registerCommand("hide",dockwidget_module_command_hide);
- 	m->registerCommand("show",dockwidget_module_command_show);
- 	m->registerFunction("isVisible",dockwidget_module_function_isvisible);
+
+	KVSM_REGISTER_SIMPLE_COMMAND(m,"hide",dockwidget_kvs_cmd_hide);
+	KVSM_REGISTER_SIMPLE_COMMAND(m,"hidewindow",dockwidget_kvs_cmd_hidewindow);
+	KVSM_REGISTER_SIMPLE_COMMAND(m,"show",dockwidget_kvs_cmd_show);
+	KVSM_REGISTER_FUNCTION(m,"isVisible",dockwidget_kvs_fnc_isvisible);
 
 	return true;
 }
@@ -559,7 +735,7 @@ static bool dockwidget_module_cleanup(KviModule *m)
     g_pDockWidgetList = 0;
 
 #ifdef COMPILE_ON_WINDOWS
-	for(int i=0;i<6;i++)DestroyIcon(g_hDock[i]);
+	DestroyIcon(g_pCurrentIcon);
 #endif
 
 	delete g_pDock1;
@@ -569,8 +745,6 @@ static bool dockwidget_module_cleanup(KviModule *m)
 	delete g_pDock3;
     g_pDock3 = 0;
 
-	m->unregisterMetaObject("KviDockWidget");
-
 	return true;
 }
 
