@@ -175,6 +175,7 @@ KviFileTransferWindow::KviFileTransferWindow(KviModuleExtensionDescriptor * d,Kv
 	m_pIrcView = new KviIrcView(m_pVertSplitter,lpFrm,this);
 
 	connect(m_pListView,SIGNAL(rightButtonPressed(QListViewItem *,const QPoint &,int)),this,SLOT(rightButtonPressed(QListViewItem *,const QPoint &,int)));
+	connect(m_pListView,SIGNAL(doubleClicked(QListViewItem *,const QPoint &,int)),this,SLOT(doubleClicked(QListViewItem *,const QPoint &,int)));
 	fillTransferView();
 
 	connect(KviFileTransferManager::instance(),SIGNAL(transferRegistered(KviFileTransfer *)),this,SLOT(transferRegistered(KviFileTransfer *)));
@@ -247,6 +248,12 @@ void KviFileTransferWindow::transferUnregistering(KviFileTransfer * t)
     it = 0;
 }
 
+void KviFileTransferWindow::doubleClicked(QListViewItem *it,const QPoint &pnt,int col)
+{
+	if(it)
+		openLocalFile();
+}
+
 void KviFileTransferWindow::rightButtonPressed(QListViewItem *it,const QPoint &pnt,int col)
 {
 	if(!m_pContextPopup)m_pContextPopup = new QPopupMenu(this);
@@ -370,8 +377,12 @@ void KviFileTransferWindow::rightButtonPressed(QListViewItem *it,const QPoint &p
 	id = m_pContextPopup->insertItem(__tr2qs_ctx("&Clear Terminated","filetransferwindow"),this,SLOT(clearTerminated()));
 	m_pContextPopup->setItemEnabled(id,bHaveTerminated);
 	
+	bool bAreTransfersActive = false;
+	if(m_pListView->childCount() >= 1)
+		bAreTransfersActive = true;
+
 	id = m_pContextPopup->insertItem(__tr2qs_ctx("Clear &All","filetransferwindow"),this,SLOT(clearAll()));
-	m_pContextPopup->setItemEnabled(id,it);
+	m_pContextPopup->setItemEnabled(id,bAreTransfersActive);
 
 	m_pContextPopup->popup(pnt);
 }
@@ -431,7 +442,7 @@ void KviFileTransferWindow::openLocalFileTerminal()
 /*FIX ME " this is not a solution ...because if the drive isn't system's drive the command
           'cd' naturaly doesn't work"*/
 	tmp.prepend("cmd.exe /k cd \"");
-	system(tmp);
+	system(tmp.local8Bit().data());
 #else //COMPILE_ON_WINDOWS
 // G&N end
 	#ifdef COMPILE_KDE_SUPPORT
@@ -483,7 +494,7 @@ void KviFileTransferWindow::openLocalFile()
 	QString tmp = t->localFileName();
 	if(tmp.isEmpty())return;
 	tmp.replace("/","\\");
-	ShellExecute(0,"open",tmp,NULL,NULL,SW_SHOWNORMAL);  //You have to link the shell32.lib
+	ShellExecute(0,"open",tmp.local8Bit().data(),NULL,NULL,SW_SHOWNORMAL);  //You have to link the shell32.lib
 #else //!COMPILE_ON_WINDOWS
 // G&N end
 	#ifdef COMPILE_KDE_SUPPORT
@@ -519,7 +530,7 @@ void KviFileTransferWindow::openLocalFileWith()
 	if(tmp.isEmpty())return;
 	tmp.replace("/","\\");
 	tmp.prepend("rundll32.exe shell32.dll,OpenAs_RunDLL "); // this if to show the 'open with...' window
-	WinExec(tmp,SW_SHOWNORMAL);
+	WinExec(tmp.local8Bit().data(),SW_SHOWNORMAL);
 #else //!COMPILE_ON_WINDOWS
 // G&N end
 	#ifdef COMPILE_KDE_SUPPORT
@@ -557,11 +568,10 @@ void KviFileTransferWindow::openLocalFileFolder()
 	if(!t)return;
 	QString tmp = t->localFileName();
 	if(tmp.isEmpty())return;
-	int idx = tmp.findRev("/");
-	if(idx == -1)idx = tmp.findRev("\\");
-	tmp = tmp.left(idx);
+	tmp=QFileInfo(tmp).dirPath(TRUE);
+	tmp.replace('/','\\');
 	tmp.prepend("explorer.exe ");
-    WinExec(tmp, SW_MAXIMIZE);
+	WinExec(tmp.local8Bit().data(), SW_MAXIMIZE);
 #else //!COMPILE_ON_WINDOWS
 // G&N end
 	#ifdef COMPILE_KDE_SUPPORT
@@ -616,12 +626,14 @@ void KviFileTransferWindow::heartbeat()
 
 void KviFileTransferWindow::clearAll()
 {
-	if(QMessageBox::information(this,__tr2qs_ctx("Clear All Transfers? - KVIrc","filetransferwindow"),
-				__tr2qs_ctx("Clear all transfers, including any in progress?","filetransferwindow"),
-				QMessageBox::Yes|QMessageBox::Default,
-				QMessageBox::No|QMessageBox::Escape,
-				QMessageBox::NoButton) == QMessageBox::Yes)
-		KviFileTransferManager::instance()->killAllTransfers();
+	QString tmp;
+
+	KviQString::sprintf(tmp,__tr2qs_ctx("Clear all transfers, including any in progress?","filetransferwindow"));
+
+	if(QMessageBox::warning(this,__tr2qs_ctx("Clear All Transfers? - KVIrc","filetransferwindow"),
+			tmp,__tr2qs_ctx("Yes","filetransferwindow"),__tr2qs_ctx("No","filetransferwindow")) != 0)
+		return;
+	KviFileTransferManager::instance()->killAllTransfers();
 }
 
 void KviFileTransferWindow::clearTerminated()
