@@ -4,7 +4,7 @@
 //   Creation date : Tue Jul 6 2004 20:25:12 CEST by Szymon Stefanek
 //
 //   This file is part of the KVirc irc client distribution
-//   Copyright (C) 2004 Iacopo Palazzi < iakko(at)siena(dot)linux(dot)it >
+//   Copyright (C) 2004 Iacopo Palazzi < iakko(at)siena.linux.it >
 //
 //   This program is FREE software. You can redistribute it and/or
 //   modify it under the linkss of the GNU General Public License
@@ -29,11 +29,12 @@
 #include "kvi_config.h"
 #include "kvi_app.h"
 #include "kvi_window.h"
-#include "kvi_uparser.h"
+//#include "kvi_uparser.h"
 #include "kvi_locale.h"
 #include "kvi_frame.h"
 #include "kvi_mirccntrl.h"
 #include "kvi_options.h"
+#include "kvi_userinput.h"
 
 #include <qpopupmenu.h>
 #include <qapplication.h>
@@ -45,11 +46,14 @@
 #include <qpainter.h>
 #include <qpen.h>
 #include <qfontmetrics.h> 
+#include <qregexp.h> 
 
 #include "notifierwindowtabs.h"
 #include "notifierwindowbody.h"
 #include "notifierwindowborder.h"
 
+extern KviNotifierWindow * g_pNotifierWindow;
+
 KviNotifierWindow::KviNotifierWindow()
 : QWidget(0,"kvirc_notifier_window",
 #ifdef COMPILE_ON_WINDOWS
@@ -60,19 +64,25 @@ KviNotifierWindow::KviNotifierWindow()
 #else
 			Qt::WStyle_Customize |
 			Qt::WStyle_NoBorder |
+#ifndef Q_OS_MACX
 			Qt::WStyle_Tool |
 			Qt::WX11BypassWM |
+#endif
 			Qt::WStyle_StaysOnTop)
 #endif
 {
 
+	g_pNotifierWindow = this;
+
 	m_eState = Hidden;
 	m_dOpacity = 0.0;
 	m_pShowHideTimer = 0;
 	m_pBlinkTimer = 0;
 	m_tAutoHideAt = 0;
+	m_tStartedAt=0;
 	m_pAutoHideTimer = 0;
-
+	m_pProgressTimer = 0;
+	
 	m_pWndBorder = new KviNotifierWindowBorder();
 
 	reloadImages();
@@ -110,7 +120,6 @@ KviNotifierWindow::KviNotifierWindow()
 	//setCursor(m_cursor);
 	
 	hide();
-
 	m_pLineEdit = new QLineEdit(this);
 	m_pLineEdit->setGeometry(0,0,0,0);
 	m_pLineEdit->hide();
@@ -154,7 +163,8 @@ KviNotifierWindow::KviNotifierWindow()
 	//m_pWndBorder = new KviNotifierWindowBorder();
 	m_pWndTabs = new KviNotifierWindowTabs(m_pWndBorder->tabsRect());
 	m_pWndBody = new KviNotifierWindowBody(m_pWndBorder->bodyRect());
-
+	
+	m_pProgressBar = new KviNotifierWindowProgressBar(m_pWndBorder->progressRect());
 	
 	// FIXME: Ugly global-font initialization..
 	QFontMetrics titleFontMetrics(*m_pTitleFont);
@@ -193,10 +203,12 @@ void KviNotifierWindow::reloadImages()
 void KviNotifierWindow::addMessage(KviWindow * pWnd,const QString &szImageId,const QString &szText,unsigned int uMessageTime)
 {
 	QPixmap * pIcon;
+	QString szMessage=szText;
+	szMessage.replace( QRegExp("\r([^\r])*\r([^\r])+\r"), "\\2" );
 	if(szImageId.isEmpty())pIcon = 0;
 	else pIcon = g_pIconManager->getImage(szImageId);
 
-	KviNotifierMessage * m = new KviNotifierMessage(this,pIcon ? new QPixmap(*pIcon) : 0,szText);
+	KviNotifierMessage * m = new KviNotifierMessage(this,pIcon ? new QPixmap(*pIcon) : 0,szMessage);
 
 	m_pWndTabs->addMessage(pWnd, m); // Adds a tab about the sender pWnd
 
@@ -248,9 +260,13 @@ void KviNotifierWindow::stopBlinkTimer()
 
 void KviNotifierWindow::stopAutoHideTimer()
 {
+	if(m_pProgressTimer) delete m_pProgressTimer;
+	m_pProgressTimer = 0;
+	
 	if(!m_pAutoHideTimer)return;
 	delete m_pAutoHideTimer;
 	m_pAutoHideTimer = 0;
+	m_pProgressBar->setProgress(0);
 }
 
 void KviNotifierWindow::computeRect()
@@ -455,23 +471,31 @@ void KviNotifierWindow::startBlinking()
 {
 	stopBlinkTimer();
 	m_bBlinkOn = false;
-	m_pBlinkTimer = new QTimer();
-	connect(m_pBlinkTimer,SIGNAL(timeout()),this,SLOT(blink()));
-	m_iBlinkCount = 0;
-	m_pBlinkTimer->start(m_iBlinkTimeout);
+	if(!KVI_OPTION_BOOL(KviOption_boolDisableNotifierFlashing))
+	{
+		m_pBlinkTimer = new QTimer();
+		connect(m_pBlinkTimer,SIGNAL(timeout()),this,SLOT(blink()));
+		m_iBlinkCount = 0;
+		m_pBlinkTimer->start(m_iBlinkTimeout);
+	}
 }
 
 void KviNotifierWindow::startAutoHideTimer()
 {
 	stopAutoHideTimer();
-	kvi_time_t tNow = kvi_unixTime();
-	if(m_tAutoHideAt <= tNow)return;
+	m_tStartedAt = kvi_unixTime();
+	m_qtStartedAt.start();
+	if(m_tAutoHideAt <= m_tStartedAt)return;
 
-	int nSecs = m_tAutoHideAt - tNow;
+	int nSecs = m_tAutoHideAt - m_tStartedAt;
 	if(nSecs < 5)nSecs = 5;
 	m_pAutoHideTimer = new QTimer();
 	connect(m_pAutoHideTimer,SIGNAL(timeout()),this,SLOT(autoHide()));
 	m_pAutoHideTimer->start(nSecs * 1000);
+	
+	m_pProgressTimer= new QTimer();
+	connect(m_pProgressTimer,SIGNAL(timeout()),this,SLOT(progressUpdate()));
+	m_pProgressTimer->start(60);
 }
 
 void KviNotifierWindow::autoHide()
@@ -738,15 +762,7 @@ void KviNotifierWindow::mousePressEvent(QMouseEvent * e)
 
 	if(m_pWndTabs->rect().contains(e->pos()))
 	{
-		if(m_pWndTabs->currentTab())
-		{
-			m_pWndTabs->mousePressEvent(e);
-
-			if (m_pWndTabs->rctCloseTabIconHotArea().contains(e->pos())) {
-				m_pWndTabs->setCloseTabIcon(WDG_ICON_CLICKED);
-				//m_pWndTabs->closeCurrentTab();
-			}
-		}
+		m_pWndTabs->mousePressEvent(e);
 		goto sartelo;
 	}
 
@@ -836,14 +852,7 @@ void KviNotifierWindow::mouseReleaseEvent(QMouseEvent * e)
 
 	if(m_pWndTabs->rect().contains(e->pos()))
 	{
-		if(m_pWndTabs->currentTab())
-		{
-			if (m_pWndTabs->rctCloseTabIconHotArea().contains(e->pos()))
-			{
-				m_pWndTabs->closeCurrentTab();
-				update();
-			}
-		}
+		m_pWndTabs->mouseReleaseEvent(e);
 	}
 	
 	setCursor(-1);
@@ -1054,6 +1063,7 @@ void KviNotifierWindow::redrawWindow()
 			m_pWndBorder->draw(&paint,true);
 			m_pWndTabs->draw(&paint);
 			m_pWndBody->draw(&paint);
+			m_pProgressBar->draw(&paint);
 		paint.end();
 		bitBlt(&m_pixForeground, 0,0, &m_pixBackgroundHighlighted);
 		m_pWndBorder->touch();
@@ -1064,6 +1074,7 @@ void KviNotifierWindow::redrawWindow()
 			m_pWndBorder->draw(&paint);
 			m_pWndTabs->draw(&paint);
 			m_pWndBody->draw(&paint);
+			m_pProgressBar->draw(&paint);
 		paint.end();
 		bitBlt(&m_pixForeground, 0,0, &m_pixBackground);
 	}
@@ -1272,12 +1283,16 @@ void KviNotifierWindow::returnPressed()
 
 	addMessage(tab->window(),tmp.ptr(),html,0); //m_pCurrentFocusedWindowTab on place of m_pCurrentMessage->window()
 	m_pLineEdit->setText("");
-	if(!g_pUserParser->parseUserCommandFromTextInput(txt,tab->window()))
-	{
-		tmp.sprintf("%d",KVI_SMALLICON_PARSERERROR);
-		addMessage(m_pWndTabs->currentTab()->window(),tmp.ptr(),__tr2qs_ctx("The command parser raised an error","notifier"),0);
-	}
+	KviUserInput::parse(txt,tab->window(),QString::null,1);
 }
 
+void KviNotifierWindow::progressUpdate()
+{
+	double dProgress = m_qtStartedAt.elapsed()/(m_tAutoHideAt - m_tStartedAt);
+	dProgress/=1000;
+	m_pProgressBar->setProgress(dProgress);
+	QPainter p(this);
+	m_pProgressBar->draw(&p);
+}
 #include "m_notifierwindow.moc"
 
