@@ -25,8 +25,9 @@
 #include "kvi_optionswidget.h"
 
 #define KVI_OPTIONS_WIDGET_ICON_KviCtcpOptionsWidget KVI_SMALLICON_CTCPREQUESTREPLIED
-#define KVI_OPTIONS_WIDGET_NAME_KviCtcpOptionsWidget __tr_no_lookup("CTCP")
+#define KVI_OPTIONS_WIDGET_NAME_KviCtcpOptionsWidget __tr2qs_no_lookup("CTCP")
 #define KVI_OPTIONS_WIDGET_PARENT_KviCtcpOptionsWidget KviIrcOptionsWidget
+#define KVI_OPTIONS_WIDGET_PRIORITY_KviCtcpOptionsWidget 70000
 
 class KviCtcpOptionsWidget : public KviOptionsWidget
 {
