@@ -25,16 +25,34 @@
 #include "kvi_optionswidget.h"
 
 #define KVI_OPTIONS_WIDGET_ICON_KviIrcOptionsWidget KVI_SMALLICON_IRC
-#define KVI_OPTIONS_WIDGET_NAME_KviIrcOptionsWidget __tr_no_lookup("IRC")
+#define KVI_OPTIONS_WIDGET_NAME_KviIrcOptionsWidget __tr2qs_no_lookup("IRC")
 #define KVI_OPTIONS_WIDGET_PRIORITY_KviIrcOptionsWidget 80000
 
+class QComboBox;
 class KviIrcOptionsWidget : public KviOptionsWidget
 {
 	Q_OBJECT
 public:
 	KviIrcOptionsWidget(QWidget * parent);
 	~KviIrcOptionsWidget();
+
+public:
+	virtual void commit();
+	
+protected:
+	QComboBox	* m_pBanTypeCombo;
 };
 
+#define KVI_OPTIONS_WIDGET_ICON_KviIrcAdvancedOptionsWidget KVI_SMALLICON_IRC
+#define KVI_OPTIONS_WIDGET_NAME_KviIrcAdvancedOptionsWidget __tr2qs_no_lookup("Advanced")
+#define KVI_OPTIONS_WIDGET_PARENT_KviIrcAdvancedOptionsWidget KviIrcOptionsWidget
+//#define KVI_OPTIONS_WIDGET_HIDDEN_KviIrcAdvancedOptionsWidget true
 
+class KviIrcAdvancedOptionsWidget : public KviOptionsWidget
+{
+	Q_OBJECT
+public:
+	KviIrcAdvancedOptionsWidget(QWidget * parent);
+	~KviIrcAdvancedOptionsWidget();
+};
 #endif //_OPTW_IRC_H_
