@@ -25,8 +25,11 @@
 #include "kvi_optionswidget.h"
 
 #define KVI_OPTIONS_WIDGET_ICON_KviPrivmsgOptionsWidget KVI_SMALLICON_RAW
-#define KVI_OPTIONS_WIDGET_NAME_KviPrivmsgOptionsWidget __tr_no_lookup("Message Format")
-#define KVI_OPTIONS_WIDGET_PARENT_KviPrivmsgOptionsWidget KviIrcOutputOptionsWidget
+#define KVI_OPTIONS_WIDGET_NAME_KviPrivmsgOptionsWidget __tr2qs_no_lookup("Output Style")
+#define KVI_OPTIONS_WIDGET_PARENT_KviPrivmsgOptionsWidget KviIrcOptionsWidget
+#define KVI_OPTIONS_WIDGET_PRIORITY_KviPrivmsgOptionsWidget 85000
+#define KVI_OPTIONS_WIDGET_GROUP_KviPrivmsgOptionsWidget "theme"
+#define KVI_OPTIONS_WIDGET_KEYWORDS_KviPrivmsgOptionsWidget __tr2qs_no_lookup("theme,privmsg,format")
 
 class KviPrivmsgOptionsWidget : public KviOptionsWidget
 {
