@@ -25,8 +25,10 @@
 #include "kvi_optionswidget.h"
 
 #define KVI_OPTIONS_WIDGET_ICON_KviQueryOptionsWidget KVI_SMALLICON_QUERY
-#define KVI_OPTIONS_WIDGET_NAME_KviQueryOptionsWidget __tr_no_lookup("Query")
+#define KVI_OPTIONS_WIDGET_NAME_KviQueryOptionsWidget __tr2qs_no_lookup("Query")
 #define KVI_OPTIONS_WIDGET_PARENT_KviQueryOptionsWidget KviIrcOptionsWidget
+#define KVI_OPTIONS_WIDGET_KEYWORDS_KviQueryOptionsWidget __tr2qs_no_lookup("chat")
+#define KVI_OPTIONS_WIDGET_PRIORITY_KviQueryOptionsWidget 80000
 
 class KviQueryOptionsWidget : public KviOptionsWidget
 {
