@@ -26,16 +26,4 @@
 #include "kvi_optionswidget.h"
 #include <qlistview.h>
 
-#define KVI_OPTIONS_WIDGET_ICON_KviTransportOptionsWidget KVI_SMALLICON_SOCKETERROR
-#define KVI_OPTIONS_WIDGET_NAME_KviTransportOptionsWidget __tr_no_lookup("Transport")
-#define KVI_OPTIONS_WIDGET_PARENT_KviTransportOptionsWidget KviConnectionOptionsWidget
-
-class KviTransportOptionsWidget : public KviOptionsWidget
-{
-	Q_OBJECT
-public:
-	KviTransportOptionsWidget(QWidget * parent);
-	~KviTransportOptionsWidget();
-};
-
 #endif //!_OPTW_TRANSPORT_H_
