@@ -25,7 +25,9 @@
 #include "kvi_optionswidget.h"
 
 #define KVI_OPTIONS_WIDGET_ICON_KviUParserOptionsWidget KVI_SMALLICON_PARSERWARNING
-#define KVI_OPTIONS_WIDGET_NAME_KviUParserOptionsWidget __tr_no_lookup("Parser")
+#define KVI_OPTIONS_WIDGET_NAME_KviUParserOptionsWidget __tr2qs_no_lookup("Scripting Engine")
+#define KVI_OPTIONS_WIDGET_MINIMAL_USER_EXPERIENCE_LEVEL_KviUParserOptionsWidget 3
+#define KVI_OPTIONS_WIDGET_KEYWORDS_KviUParserOptionsWidget __tr2qs_no_lookup("scripting,kvs,debug")
 
 class KviUParserOptionsWidget : public KviOptionsWidget
 {
