@@ -4,7 +4,7 @@
 //   Creation date : Tue Jul 13 13:03:31 2004 GMT by Szymon Stefanek
 //
 //   This file is part of the KVirc irc client distribution
-//   Copyright (C) 2001 Szymon Stefanek (pragma at kvirc dot net)
+//   Copyright (C) 2004-2005 Szymon Stefanek (pragma at kvirc dot net)
 //
 //   This program is FREE software. You can redistribute it and/or
 //   modify it under the terms of the GNU General Public License
@@ -24,10 +24,9 @@
 
 #include "kvi_settings.h"
 #include "kvi_module.h"
-#include "kvi_uparser.h"
+//#include "kvi_uparser.h"
 #include "kvi_fileutils.h"
 #include "kvi_locale.h"
-#include "kvi_command.h"
 #include "kvi_app.h"
 #include "kvi_options.h"
 #include "kvi_modulemanager.h"
@@ -40,28 +39,28 @@
 #endif // COMPILE_PERL_SUPPORT
 
 #ifdef COMPILE_PERL_SUPPORT
-	#define CHECK_PERLCORE(_m,_c) \
+	#define KVS_CHECK_PERLCORE(_m,_c) \
 		g_pPerlCoreModule = g_pModuleManager->getModule("perlcore"); \
 		if(!g_pPerlCoreModule) \
 		{ \
-			if(!_c->hasSwitch('q')) \
+			if(!_c->switches()->find('q',"quiet")) \
 			{ \
 				_c->warning(__tr2qs_ctx("The perlcore module can't be loaded: perl support not available","perl")); \
 				_c->warning(__tr2qs_ctx("To see more details about loading failure try /perlcore.load","perl")); \
-				return _c->leaveStackFrame(); \
+				return true; \
 			} \
 		}
 #else // !COMPILE_PERL_SUPPORT
-	#define CHECK_PERLCORE(_m,_c)
+	#define KVS_CHECK_PERLCORE(_m,_c)
 #endif // !COMPILE_PERL_SUPPORT
 
 #ifdef COMPILE_PERL_SUPPORT
-	#define CHECK_MODULE_STATE(_m,_c) CHECK_PERLCORE(_m,_c)
+	#define KVS_CHECK_MODULE_STATE(_m,_c) KVS_CHECK_PERLCORE(_m,_c)
 #else // !COMPILE_PERL_SUPPORT
-	#define CHECK_MODULE_STATE(_m,_c) \
-		if(!_c->hasSwitch('q')) \
+	#define KVS_CHECK_MODULE_STATE(_m,_c) \
+		if(!_c->switches()->find('q',"quiet")) \
 			_c->warning(__tr2qs_ctx("This KVIrc executable has been compiled without perl scripting support","perl")); \
-		return _c->leaveStackFrame();
+		return true;
 #endif // !COMPILE_PERL_SUPPORT
 
 /*
@@ -319,7 +318,7 @@
 		}
 		sub handler_for_event_2
 		{
-			do_complet_perl_stuff_here
+			do_complex_perl_stuff_here
 		}
 		[cmd]perl.end[/cmd]
 		[/example]
@@ -362,11 +361,14 @@
 		<perl code>
 		perl.end
 	@switches:
-		!sw: -q
+		!sw: -q | --quiet
 		Prevents the command from printing any warnings.
-		!sw: -n
+		!sw: -n | --no-return
 		Prevents the perl script return value to be propagated
 		to the current context.
+		!sw: -f | --fail-on-error
+		Treat perl errors as KVS errors and abort execution of the
+		current script. Incompatible with -q
 	@description:
 		Indicates the beginning of a snipped of perl code.
 		The whole code part between perl.begin and perl.end
@@ -422,97 +424,74 @@
 		[cmd]perl.begin[/cmd]
 */
 
-
-static bool perl_module_cmd_begin(KviModule *m,KviCommand *c)
+static bool perl_kvs_cmd_begin(KviKvsModuleCommandCall * c)
 {
+	// This command is somewhat special in the fact that has a dedicated
+	// parsing routine in the KVS core parser.
+	// The parser sets the perl code as the first parameter of our call,
+	// the remaining params are the context name and the arguments
 
-	// THE SKIPPING ROUTINE FOR THIS COMMAND IS HARDWIRED IN
-	// THE USER PARSER!
-	// see: bool KviUserParser::skipSingleCommand(KviCommand *c)
-	// in kvi_up_skip.cpp
+	QString szCode,szContext;
+	KviKvsVariantList vList;
+	KVSM_PARAMETERS_BEGIN(c)
+		KVSM_PARAMETER("code",KVS_PT_STRING,0,szCode)
+		KVSM_PARAMETER("context",KVS_PT_STRING,KVS_PF_OPTIONAL,szContext)
+		KVSM_PARAMETER("args",KVS_PT_VARIANTLIST,KVS_PF_OPTIONAL,vList)
+	KVSM_PARAMETERS_END(c)
 
-	ENTER_STACK_FRAME(c,"perl.begin");
-	KviStr szSrc,szDst;
-
-	c->skipSpace();
-	KviParameterList par;
-
-	g_pUserParser->extractFunctionParameters(c,&par);
-	c->skipSpace();
-
-	char * p = c->ptr();
-	if(*p == ';')p++;
-
-	char * begin = p;
-	char * end = begin;
-	
-	while(*p)
-	{
-		if(*p=='p')
-		{
-			if(kvi_strEqualCIN("perl.end",p,8))
-			{
-				end = p;
-				p += 8;
-				if(*p)
-				{
-					if((*p == ' ') || (*p == '\n') ||
-						(*p == ';') || (*p == '\t') || (*p == '\r'))
-						break;
-				} else {
-					break;
-				}
-			}
-		}
-		p++;
-	}
-
-	if(*p)
-	{
-		while(*(p) && (*p != '\n') && (*p != ';'))p++;
-		if(*p)p++;
-	}
-	
-	c->setPtr(p);
-
-	CHECK_MODULE_STATE(m,c)
+	KVS_CHECK_MODULE_STATE(m,c)
 
 #ifdef COMPILE_PERL_SUPPORT
-	KviStr szCode(begin,end - begin);
 	
 	KviPerlCoreCtrlCommand_execute ex;
 	ex.uSize = sizeof(KviPerlCoreCtrlCommand_execute);
-	ex.pCommand = c;
-	if(par.count() > 0)
-		ex.szContext = par.safeFirst()->ptr();
-	else
-		ex.szContext = "";
-	ex.szCode = szCode.ptr();
-	while(KviStr * str = par.next())
-		ex.lArgs.append(str->ptr());
-	ex.bQuiet = c->hasSwitch('q');
+	ex.pKvsContext = c->context();
+	ex.szContext = szContext;
+	ex.szCode = szCode;
+	for(KviKvsVariant * v = vList.first();v;v = vList.next())
+	{
+		QString tmp;
+		v->asString(tmp);
+		ex.lArgs.append(tmp);
+	}
+	ex.bQuiet = c->switches()->find('q',"quiet");
 	
 	if(!g_pPerlCoreModule->ctrl(KVI_PERLCORECTRLCOMMAND_EXECUTE,&ex))
 	{
-		if(!c->hasSwitch('q'))
+		if(!c->switches()->find('q',"quiet"))
 			c->warning(__tr2qs_ctx("The perlcore module failed to execute the code: something is wrong with the perl support","perl"));
-		return c->leaveStackFrame();
+		return true;
+	}
+
+	if(!ex.lWarnings.isEmpty())
+	{
+		for(QStringList::Iterator it = ex.lWarnings.begin();it != ex.lWarnings.end();++it)
+			c->warning(*it);
 	}
 
 	if(!ex.bExitOk)
 	{
-		if(!c->hasSwitch('q'))
+		if(!c->switches()->find('q',"quiet"))
 		{
-			c->warning(__tr2qs_ctx("Perl execution error:","perl"));
-			c->warning(ex.szError.latin1());
+			
+			if(c->switches()->find('f',"fail-on-error"))
+			{
+				c->warning(__tr2qs_ctx("Perl execution error:","perl"));
+				c->warning(ex.szError);
+				return false;
+			} else {
+				c->warning(__tr2qs_ctx("Perl execution error:","perl"));
+				c->error(ex.szError);
+			}
 		}
 	}
 
-	if(!c->hasSwitch('n'))
-		c->setReturnValue(ex.szRetVal.utf8().data());
+	if(!c->switches()->find('n',"no-return"))
+		c->context()->returnValue()->setString(ex.szRetVal);
 
 #endif //COMPILE_PERL_SUPPORT
-	return c->leaveStackFrame();
+
+	return true;
 }
 
 /*
@@ -524,7 +503,7 @@ static bool perl_module_cmd_begin(KviModule *m,KviCommand *c)
 	@short:
 		Destroys a perl execution context
 	@syntax:
-		perl.destroy [-q] <context_name>
+		perl.destroy [-q] <context_name:string>
 	@description:
 		Destroys the perl execution context <context_name>.
 		If the context is not existing then a warning is printed unless the
@@ -537,14 +516,14 @@ static bool perl_module_cmd_begin(KviModule *m,KviCommand *c)
 		[cmd]perl.begin[/cmd]
 */
 
-static bool perl_module_cmd_destroy(KviModule *m,KviCommand *c)
+static bool perl_kvs_cmd_destroy(KviKvsModuleCommandCall * c)
 {
-	ENTER_STACK_FRAME(c,"perl.destroy");
-	KviStr szContext,szDst;
-	if(!g_pUserParser->parseCmdSingleToken(c,szContext))return false;
-	if(!g_pUserParser->parseCmdFinalPart(c,szDst))return false;
+	QString szContext;
+	KVSM_PARAMETERS_BEGIN(c)
+		KVSM_PARAMETER("context",KVS_PT_NONEMPTYSTRING,0,szContext)
+	KVSM_PARAMETERS_END(c)
 
-	CHECK_MODULE_STATE(m,c)
+	KVS_CHECK_MODULE_STATE(m,c)
 
 #ifdef COMPILE_PERL_SUPPORT
 	KviPerlCoreCtrlCommand_destroy ex;
@@ -553,12 +532,12 @@ static bool perl_module_cmd_destroy(KviModule *m,KviCommand *c)
 	
 	if(!g_pPerlCoreModule->ctrl(KVI_PERLCORECTRLCOMMAND_DESTROY,&ex))
 	{
-		if(!c->hasSwitch('q'))
+		if(!c->switches()->find('q',"quiet"))
 			c->warning(__tr2qs_ctx("The perlcore module failed to execute the code: something is wrong with the perl support","perl"));
 	}
 #endif //COMPILE_PERL_SUPPORT
 
-	return c->leaveStackFrame();
+	return true;
 }
 
 
@@ -576,24 +555,25 @@ static bool perl_module_cmd_destroy(KviModule *m,KviCommand *c)
 		Returns 1 if the perl scripting support is available and 0 otherwise.
 */
 
-static bool perl_module_fnc_isAvailable(KviModule *m,KviCommand *c,KviParameterList * parms,KviStr &buffer)
-{
+static bool perl_kvs_fnc_isAvailable(KviKvsModuleFunctionCall * c)
+{ 
 #ifdef COMPILE_PERL_SUPPORT
 	g_pPerlCoreModule = g_pModuleManager->getModule("perlcore");
-	buffer.append(g_pPerlCoreModule ? '1' : '0');
+	c->returnValue()->setBoolean(g_pPerlCoreModule ? true : false);
 #else //!COMPILE_PERL_SUPPORT
-	buffer.append('0');
+	c->returnValue()->setBoolean(false);
 #endif //!COMPILE_PERL_SUPPORT
 	return true;
 }
 
-
 static bool perl_module_init(KviModule * m)
 {
 	// register the command anyway
-	m->registerCommand("begin",perl_module_cmd_begin);
-	m->registerCommand("destroy",perl_module_cmd_destroy);
-	m->registerFunction("isAvailable",perl_module_fnc_isAvailable);
+	KVSM_REGISTER_SIMPLE_COMMAND(m,"begin",perl_kvs_cmd_begin);
+	KVSM_REGISTER_SIMPLE_COMMAND(m,"destroy",perl_kvs_cmd_destroy);
+	
+	KVSM_REGISTER_FUNCTION(m,"isAvailable",perl_kvs_fnc_isAvailable);
+
 	// FIXME: perl.isSupported()
 #ifdef COMPILE_PERL_SUPPORT
 	g_pPerlCoreModule = g_pModuleManager->getModule("perlcore");
