@@ -1,3 +1,4 @@
+//=============================================================================
 //
 //   File : popupeditor.cpp
 //   Creation date : Mon Dec 23 2002 20:28:18 by Szymon Stefanek
@@ -19,6 +20,8 @@
 //   along with this program. If not, write to the Free Software Foundation,
 //   Inc. ,59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 //
+//=============================================================================
+
 #include "popupeditor.h"
 
 #include "kvi_iconmanager.h"
@@ -32,8 +35,10 @@
 #include "kvi_debug.h"
 #include "kvi_locale.h"
 #include "kvi_app.h"
-#include "kvi_popupmanager.h"
-#include "kvi_parameterlist.h"
+#include "kvi_kvs_popupmanager.h"
+#include "kvi_kvs_popupmenu.h"
+#include "kvi_kvs_variantlist.h"
+//#include "kvi_parameterlist.h"
 
 #include <qmessagebox.h>
 #include <qdir.h>
@@ -154,10 +159,11 @@ void KviPopupListViewItem::setIcon(const QString & szIcon)
 	{
 		case Item:
 		case Menu:
+		case Label:
 		case ExtMenu:
 			m_szIcon = szIcon;
 			{
-				QPixmap * pix = g_pIconManager->getImage(szIcon.latin1());
+				QPixmap * pix = g_pIconManager->getImage(szIcon);
 				if(pix)setPixmap(0,*pix);
 				else setPixmap(0,QPixmap());
 			}
@@ -278,25 +284,54 @@ KviSinglePopupEditor::~KviSinglePopupEditor()
 
 void KviSinglePopupEditor::testPopup()
 {
+
 	if(m_pTestPopup)delete m_pTestPopup;
 	m_pTestPopup = getMenu();
 	if(!m_pTestPopup)return;
-	connect(m_pTestPopup,SIGNAL(testModeItemClicked(KviPopupMenuItem *)),this,SLOT(testModeMenuItemClicked(KviPopupMenuItem *)));
+	connect(m_pTestPopup,SIGNAL(testModeItemClicked(KviKvsPopupMenuItem *)),this,SLOT(testModeMenuItemClicked(KviKvsPopupMenuItem *)));
 	QPoint pnt = m_pMenuButton->mapToGlobal(QPoint(0,m_pMenuButton->height()));
-	m_pTestPopup->doPopup(pnt,g_pActiveWindow,new KviParameterList(),true);
+	KviKvsVariantList * parms = new KviKvsVariantList();
+	parms->append(new KviKvsVariant(QString("test1")));
+	parms->append(new KviKvsVariant(QString("test2")));
+	parms->append(new KviKvsVariant(QString("test3")));
+	parms->append(new KviKvsVariant(QString("test4")));
+	m_pTestPopup->doPopup(pnt,g_pActiveWindow,parms,true);
 }
 
-KviPopupListViewItem * KviSinglePopupEditor::findMatchingItem(KviPopupMenuItem * it,KviPopupListViewItem * item)
+KviPopupListViewItem * KviSinglePopupEditor::findMatchingItem(KviKvsPopupMenuItem * it,KviPopupListViewItem * item)
 {
-	if((it->type() == KviPopupMenuItem::Item) &&
-		(item->m_type == KviPopupListViewItem::Item) &&
-		(it->text() == item->m_szText) &&
-		(it->code() == item->m_szCode) &&
-		(it->icon() == item->m_szIcon) &&
-		(it->expr() == item->m_szCondition) &&
-		(it->id() == item->m_szId))
-			return item;
-		
+	
+	if(it->type() != KviKvsPopupMenuItem::Item)goto not_this_one;
+	if(item->m_type != KviPopupListViewItem::Item)goto not_this_one;
+	if(it->name() != item->m_szId)goto not_this_one;
+	if(it->kvsText())
+	{
+		if(it->kvsText()->code() != item->m_szText)goto not_this_one;
+	} else {
+		if(!item->m_szText.isEmpty())goto not_this_one;
+	}
+	if(it->kvsCode())
+	{
+		if(it->kvsCode()->code() != item->m_szCode)goto not_this_one;
+	} else {
+		if(!item->m_szCode.isEmpty())goto not_this_one;
+	}
+	if(it->kvsIcon())
+	{
+		if(it->kvsIcon()->code() != item->m_szIcon)goto not_this_one;
+	} else {
+		if(!item->m_szIcon.isEmpty())goto not_this_one;
+	}
+	if(it->kvsCondition())
+	{
+		if(it->kvsCondition()->code() != item->m_szCondition)goto not_this_one;
+	} else {
+		if(!item->m_szCondition.isEmpty())goto not_this_one;
+	}
+
+	return item;
+			
+not_this_one:
 	item = (KviPopupListViewItem *)item->firstChild();
 	while(item)
 	{
@@ -304,11 +339,13 @@ KviPopupListViewItem * KviSinglePopupEditor::findMatchingItem(KviPopupMenuItem *
 		if(found)return found;
 		item = (KviPopupListViewItem *)item->nextSibling();
 	}
+
 	return 0;
 }
 
-void KviSinglePopupEditor::testModeMenuItemClicked(KviPopupMenuItem * it)
+void KviSinglePopupEditor::testModeMenuItemClicked(KviKvsPopupMenuItem * it)
 {
+
 	saveLastSelectedItem(); // that's the first thingie
 	// find the matching item and set it as current
 	KviPopupListViewItem * item = (KviPopupListViewItem *)m_pListView->firstChild();
@@ -325,6 +362,7 @@ void KviSinglePopupEditor::testModeMenuItemClicked(KviPopupMenuItem * it)
 		}
 		item = (KviPopupListViewItem *)item->nextSibling();
 	}
+
 }
 
 
@@ -592,7 +630,7 @@ void KviSinglePopupEditor::contextCopy()
 {
 	if(!m_pLastSelectedItem)return;
 	if(m_pClipboard)delete m_pClipboard;
-	m_pClipboard = new KviPopupMenu("clipboard");
+	m_pClipboard = new KviKvsPopupMenu("clipboard");
 	addItemToMenu(m_pClipboard,m_pLastSelectedItem);
 }
 
@@ -692,68 +730,53 @@ void KviSinglePopupEditor::saveLastSelectedItem()
 	}
 }
 
-void KviSinglePopupEditor::addItemToMenu(KviPopupMenu * p,KviPopupListViewItem *it)
+void KviSinglePopupEditor::addItemToMenu(KviKvsPopupMenu * p,KviPopupListViewItem *it)
 {
+	it->m_szId.stripWhiteSpace();
 	switch(it->m_type)
 	{
 		case KviPopupListViewItem::Prologue:
-		{
 			it->m_szCode.stripWhiteSpace();
-			KviPopupMenuCodeSnippet * snip = new KviPopupMenuCodeSnippet;
-			snip->m_szCode = it->m_szCode;
-			snip->m_szId = it->m_szId;
-			p->addPrologue(snip);
-		}
+			p->addPrologue(it->m_szId,it->m_szCode);
 		break;
 		case KviPopupListViewItem::Epilogue:
-		{
 			it->m_szCode.stripWhiteSpace();
-			KviPopupMenuCodeSnippet * snip = new KviPopupMenuCodeSnippet;
-			snip->m_szCode = it->m_szCode;
-			snip->m_szId = it->m_szId;
-			p->addEpilogue(snip);
-		}
+			p->addEpilogue(it->m_szId,it->m_szCode);
 		break;
 		case KviPopupListViewItem::Separator:
-			p->addItem(new KviPopupMenuItem(KviPopupMenuItem::Separator,
-				QString::null,QString::null,QString::null,QString::null,0,it->m_szId));
+			it->m_szCondition.stripWhiteSpace();
+			p->addSeparator(it->m_szId,it->m_szCondition);
 		break;
 		case KviPopupListViewItem::Label:
 			it->m_szText.stripWhiteSpace();
 			it->m_szCondition.stripWhiteSpace();
-			p->addItem(new KviPopupMenuItem(KviPopupMenuItem::Label,it->m_szText,
-					QString::null,
-					it->m_szCondition,
-					QString::null,
-					0,
-					it->m_szId));
+			it->m_szIcon.stripWhiteSpace();
+			p->addLabel(it->m_szId,it->m_szText,it->m_szIcon,it->m_szCondition);
 		break;
 		case KviPopupListViewItem::Item:
 			it->m_szText.stripWhiteSpace();
 			it->m_szIcon.stripWhiteSpace();
 			it->m_szCondition.stripWhiteSpace();
 			it->m_szCode.stripWhiteSpace();
-			p->addItem(new KviPopupMenuItem(KviPopupMenuItem::Item,
-					it->m_szText,it->m_szIcon,it->m_szCondition,it->m_szCode,0,it->m_szId));
+			p->addItem(it->m_szId,it->m_szCode,it->m_szText,it->m_szIcon,it->m_szCondition);
 		break;
 		case KviPopupListViewItem::ExtMenu:
 			it->m_szText.stripWhiteSpace();
 			it->m_szIcon.stripWhiteSpace();
 			it->m_szCondition.stripWhiteSpace();
-			it->m_szCode.stripWhiteSpace();
-			p->addItem(new KviPopupMenuItem(KviPopupMenuItem::ExtMenu,it->m_szText,it->m_szIcon,it->m_szCondition,it->m_szCode,0,it->m_szId));
+			it->m_szCode.stripWhiteSpace();// <-- this is the ext name in fact
+			p->addExtPopup(it->m_szId,it->m_szCode,it->m_szText,it->m_szIcon,it->m_szCondition);
 		break;
 		case KviPopupListViewItem::Menu:
 		{
-			KviPopupMenu * menu = new KviPopupMenu("submenu");
+			it->m_szText.stripWhiteSpace();
+			it->m_szIcon.stripWhiteSpace();
+			it->m_szCondition.stripWhiteSpace();
+			KviKvsPopupMenu * menu = p->addPopup(it->m_szId,it->m_szText,it->m_szIcon,it->m_szCondition);
 			for(KviPopupListViewItem * item = (KviPopupListViewItem *)it->firstChild();item;item = (KviPopupListViewItem *)item->nextSibling())
 			{
 				addItemToMenu(menu,item);
 			}
-			it->m_szText.stripWhiteSpace();
-			it->m_szIcon.stripWhiteSpace();
-			it->m_szCondition.stripWhiteSpace();
-			p->addItem(new KviPopupMenuItem(KviPopupMenuItem::Menu,it->m_szText,it->m_szIcon,it->m_szCondition,QString::null,menu,it->m_szId));
 		}
 		break;
 		default:
@@ -761,14 +784,14 @@ void KviSinglePopupEditor::addItemToMenu(KviPopupMenu * p,KviPopupListViewItem *
 	}
 }
 
-KviPopupMenu * KviSinglePopupEditor::getMenu()
+KviKvsPopupMenu * KviSinglePopupEditor::getMenu()
 {
 	saveLastSelectedItem();
 
 	QString tmp = m_pNameEditor->text();
 	tmp.stripWhiteSpace();
 
-	KviPopupMenu * p = new KviPopupMenu(tmp);
+	KviKvsPopupMenu * p = new KviKvsPopupMenu(tmp);
 	for(KviPopupListViewItem * it = (KviPopupListViewItem *)m_pListView->firstChild();it;it = (KviPopupListViewItem *)it->nextSibling())
 	{
 		addItemToMenu(p,it);
@@ -806,6 +829,7 @@ void KviSinglePopupEditor::selectionChanged(QListViewItem * it)
 		{
 			case KviPopupListViewItem::Menu:
 			case KviPopupListViewItem::Item:
+			case KviPopupListViewItem::Label:
 			case KviPopupListViewItem::ExtMenu:
 				m_pIconEditor->setText(((KviPopupListViewItem *)it)->m_szIcon);
 				bIconEditorEnabled = true;
@@ -851,72 +875,73 @@ void KviSinglePopupEditor::selectionChanged(QListViewItem * it)
 	m_pIdEditor->setEnabled(it);
 }
 
-void KviSinglePopupEditor::populateMenu(KviPopupMenu * pop,KviPopupListViewItem * par,KviPopupListViewItem * theItem)
+void KviSinglePopupEditor::populateMenu(KviKvsPopupMenu * pop,KviPopupListViewItem * par,KviPopupListViewItem * theItem)
 {
 	if(!pop)return;
 
-	for(KviPopupMenuCodeSnippet * sp = pop->prologues()->first();sp;sp = pop->prologues()->next())
+	for(KviKvsScript * sp = pop->prologues()->first();sp;sp = pop->prologues()->next())
 	{
 		if(par)theItem = new KviPopupListViewItem(par,theItem,KviPopupListViewItem::Prologue);
 		else theItem = new KviPopupListViewItem(m_pListView,theItem,KviPopupListViewItem::Prologue);
-		theItem->setCode(sp->m_szCode);
-		theItem->setId(sp->m_szId);
+		theItem->setCode(sp->code());
+		theItem->setId(sp->name());
 	}
 
-	for(KviPopupMenuItem * item = pop->m_pItemList->first();item;item = pop->m_pItemList->next())
+	for(KviKvsPopupMenuItem * item = pop->m_pItemList->first();item;item = pop->m_pItemList->next())
 	{
 		switch(item->type())
 		{
-			case KviPopupMenuItem::Item:
+			case KviKvsPopupMenuItem::Item:
 				if(par)theItem = new KviPopupListViewItem(par,theItem,KviPopupListViewItem::Item);
 				else theItem = new KviPopupListViewItem(m_pListView,theItem,KviPopupListViewItem::Item);
-				theItem->setIcon(item->icon());
-				theItem->setItemText(item->text());
-				theItem->setCondition(item->expr());
-				theItem->setCode(item->code());
-				theItem->setId(item->id());
+				theItem->setIcon(item->kvsIcon() ? item->kvsIcon()->code() : QString::null);
+				theItem->setItemText(item->kvsText() ? item->kvsText()->code() : QString::null);
+				theItem->setCondition(item->kvsCondition() ? item->kvsCondition()->code() : QString::null);
+				theItem->setCode(item->kvsCode() ? item->kvsCode()->code() : QString::null);
+				theItem->setId(item->name());
 			break;
-			case KviPopupMenuItem::ExtMenu:
+			case KviKvsPopupMenuItem::ExtMenu:
 				if(par)theItem = new KviPopupListViewItem(par,theItem,KviPopupListViewItem::ExtMenu);
 				else theItem = new KviPopupListViewItem(m_pListView,theItem,KviPopupListViewItem::ExtMenu);
-				theItem->setIcon(item->icon());
-				theItem->setItemText(item->text());
-				theItem->setCondition(item->expr());
-				theItem->setCode(item->code());
-				theItem->setId(item->id());
+				theItem->setIcon(item->kvsIcon() ? item->kvsIcon()->code() : QString::null);
+				theItem->setItemText(item->kvsText() ? item->kvsText()->code() : QString::null);
+				theItem->setCondition(item->kvsCondition() ? item->kvsCondition()->code() : QString::null);
+				theItem->setCode(((KviKvsPopupMenuItemExtMenu *)item)->extName());
+				theItem->setId(item->name());
 			break;
-			case KviPopupMenuItem::Label:
+			case KviKvsPopupMenuItem::Label:
 				if(par)theItem = new KviPopupListViewItem(par,theItem,KviPopupListViewItem::Label);
 				else theItem = new KviPopupListViewItem(m_pListView,theItem,KviPopupListViewItem::Label);
-				theItem->setItemText(item->text());
-				theItem->setCondition(item->expr());
-				theItem->setId(item->id());
+				theItem->setIcon(item->kvsIcon() ? item->kvsIcon()->code() : QString::null);
+				theItem->setItemText(item->kvsText() ? item->kvsText()->code() : QString::null);
+				theItem->setCondition(item->kvsCondition() ? item->kvsCondition()->code() : QString::null);
+				theItem->setId(item->name());
 			break;
-			case KviPopupMenuItem::Separator:
+			case KviKvsPopupMenuItem::Separator:
 				if(par)theItem = new KviPopupListViewItem(par,theItem,KviPopupListViewItem::Separator);
 				else theItem = new KviPopupListViewItem(m_pListView,theItem,KviPopupListViewItem::Separator);
-				theItem->setId(item->id());
+				theItem->setId(item->name());
 			break;
-			case KviPopupMenuItem::Menu:
+			case KviKvsPopupMenuItem::Menu:
 				if(par)theItem = new KviPopupListViewItem(par,theItem,KviPopupListViewItem::Menu);
 				else theItem = new KviPopupListViewItem(m_pListView,theItem,KviPopupListViewItem::Menu);
-				theItem->setIcon(item->icon());
-				theItem->setItemText(item->text());
-				theItem->setCondition(item->expr());
-				populateMenu(item->menu(),theItem,0);
-				theItem->setId(item->id());
+				theItem->setIcon(item->kvsIcon() ? item->kvsIcon()->code() : QString::null);
+				theItem->setItemText(item->kvsText() ? item->kvsText()->code() : QString::null);
+				theItem->setCondition(item->kvsCondition() ? item->kvsCondition()->code() : QString::null);
+				theItem->setId(item->name());
+				populateMenu(((KviKvsPopupMenuItemMenu *)item)->menu(),theItem,0);
 			break;
 			default:
 			break;
 		}
 	}
 
-	for(KviPopupMenuCodeSnippet * se = pop->epilogues()->first();se;se = pop->epilogues()->next())
+	for(KviKvsScript * se = pop->epilogues()->first();se;se = pop->epilogues()->next())
 	{
 		if(par)theItem = new KviPopupListViewItem(par,theItem,KviPopupListViewItem::Epilogue);
 		else theItem = new KviPopupListViewItem(m_pListView,theItem,KviPopupListViewItem::Epilogue);
-		theItem->setCode(se->m_szCode);
-		theItem->setId(se->m_szId);
+		theItem->setCode(se->code());
+		theItem->setId(se->name());
 	}
 }
 
@@ -934,7 +959,7 @@ void KviSinglePopupEditor::edit(KviMenuListViewItem * it)
 
 	if(it)
 	{
-		m_pNameEditor->setText(it->m_pPopup->name());
+		m_pNameEditor->setText(it->m_pPopup->popupName());
 
 		populateMenu(it->m_pPopup,0,0);
 
@@ -975,7 +1000,7 @@ void KviSinglePopupEditor::edit(KviMenuListViewItem * it)
 
 
 
-KviMenuListViewItem::KviMenuListViewItem(QListView * par,KviPopupMenu * popup)
+KviMenuListViewItem::KviMenuListViewItem(QListView * par,KviKvsPopupMenu * popup)
 : QListViewItem(par)
 {
 	setPixmap(0,*(g_pIconManager->getSmallIcon(KVI_SMALLICON_POPUP)));
@@ -988,7 +1013,7 @@ KviMenuListViewItem::~KviMenuListViewItem()
 	delete m_pPopup;
 }
 
-void KviMenuListViewItem::replacePopup(KviPopupMenu * popup)
+void KviMenuListViewItem::replacePopup(KviKvsPopupMenu * popup)
 {
 	delete m_pPopup;
 	m_pPopup = popup;
@@ -1007,11 +1032,13 @@ KviPopupEditor::KviPopupEditor(QWidget * par)
 
 	m_pListView = new QListView(box);
 	m_pListView->addColumn(__tr2qs("Popup"));
-	m_pListView->setMultiSelection(false);
+	m_pListView->setSelectionMode(QListView::Extended);
 	m_pListView->setShowSortIndicator(true);
 
 	QPushButton * pb = new QPushButton(__tr2qs("&Export All To..."),box);
-	connect(pb,SIGNAL(clicked()),this,SLOT(exportAllPopups()));
+	connect(pb,SIGNAL(clicked()),this,SLOT(exportAll()));
+	QPushButton * gn = new QPushButton(__tr2qs("&Export selected To..."),box);
+	connect(gn,SIGNAL(clicked()),this,SLOT(exportSelected()));
 
 	m_pEditor = new KviSinglePopupEditor(spl);
 
@@ -1020,7 +1047,7 @@ KviPopupEditor::KviPopupEditor(QWidget * par)
 
 	m_pContextPopup = new QPopupMenu(this);
 
-	selectionChanged(0);
+	currentItemChanged(0);
 }
 
 KviPopupEditor::~KviPopupEditor()
@@ -1032,23 +1059,23 @@ void KviPopupEditor::oneTimeSetup()
 	if(m_bOneTimeSetupDone)return;
 	m_bOneTimeSetupDone = true;
 
-	const QDict<KviPopupMenu> * a = g_pPopupManager->popupDict();
+	const QDict<KviKvsPopupMenu> * a = KviKvsPopupManager::instance()->popupDict();
 	if(!a)return;
 
-	QDictIterator<KviPopupMenu> it(*a);
+	QDictIterator<KviKvsPopupMenu> it(*a);
 
 	KviMenuListViewItem * item;
 
 	while(it.current())
 	{
-		KviPopupMenu * popup = it.current();
-		KviPopupMenu * copy = new KviPopupMenu(popup->name());
+		KviKvsPopupMenu * popup = it.current();
+		KviKvsPopupMenu * copy = new KviKvsPopupMenu(popup->popupName());
 		copy->copyFrom(popup);
 		item = new KviMenuListViewItem(m_pListView,copy);
 		++it;
 	}
 
-	connect(m_pListView,SIGNAL(selectionChanged(QListViewItem *)),this,SLOT(selectionChanged(QListViewItem *)));
+	connect(m_pListView,SIGNAL(currentChanged(QListViewItem *)),this,SLOT(currentItemChanged(QListViewItem *)));
 	connect(m_pListView,SIGNAL(rightButtonPressed(QListViewItem *,const QPoint &,int)),
 		this,SLOT(itemPressed(QListViewItem *,const QPoint &,int)));
 
@@ -1095,7 +1122,7 @@ void KviPopupEditor::exportCurrentPopup()
 	
 	QString szFile;
 	
-	if(!KviFileDialog::askForSaveFileName(szFile,__tr2qs("Choose a Filename - KVIrc"),szName,QString::null,true,true))return;
+	if(!KviFileDialog::askForSaveFileName(szFile,__tr2qs("Choose a Filename - KVIrc"),szName,"*.kvs",true,true,true))return;
 
 	QString szOut;
 	m_pLastEditedItem->popup()->generateDefPopup(szOut);
@@ -1106,7 +1133,19 @@ void KviPopupEditor::exportCurrentPopup()
 	}
 }
 
-void KviPopupEditor::exportAllPopups()
+void KviPopupEditor::exportAll()
+{
+	bool bSelectedOnly=true;
+	exportPopups(bSelectedOnly);
+}
+
+void KviPopupEditor::exportSelected()
+{
+	bool bSelectedOnly=false;
+	exportPopups(bSelectedOnly);
+}
+
+void KviPopupEditor::exportPopups(bool bSelectedOnly)
 {
 	saveLastEditedItem();
 
@@ -1114,14 +1153,22 @@ void KviPopupEditor::exportAllPopups()
 
 	QString out;
 
-	while(it)
+	int count=0;
+	QListViewItemIterator itv( m_pListView );
+	while( itv.current() )
 	{
-		QString tmp;
-		it->popup()->generateDefPopup(tmp);
-		out += tmp;
-		out += "\n";
+		if ( (itv.current()->isSelected()) || (bSelectedOnly == true) )
+		{
+			count ++;
+			QString tmp;
+			it->popup()->generateDefPopup(tmp);
+			out += tmp;
+			out += "\n";
+		}
 		it = (KviMenuListViewItem *)it->nextSibling();
+		++itv;
 	}
+	if (!count && bSelectedOnly==false) return;
 
 	QString szName = QDir::homeDirPath();
 	if(!szName.endsWith(QString(KVI_PATH_SEPARATOR)))szName += KVI_PATH_SEPARATOR;
@@ -1129,7 +1176,7 @@ void KviPopupEditor::exportAllPopups()
 	
 	QString szFile;
 	
-	if(!KviFileDialog::askForSaveFileName(szFile,__tr2qs("Choose a Filename - KVIrc"),szName,QString::null,true,true))return;
+	if(!KviFileDialog::askForSaveFileName(szFile,__tr2qs("Choose a Filename - KVIrc"),szName,"*.kvs",true,true,true))return;
 
 	if(!KviFileUtils::writeFile(szFile,out))
 	{
@@ -1145,15 +1192,15 @@ void KviPopupEditor::removeCurrentPopup()
 		KviMenuListViewItem * it = m_pLastEditedItem;
 		m_pLastEditedItem = 0;
 		delete it;
-		if(!m_pLastEditedItem)selectionChanged(0);
+		if(!m_pLastEditedItem)currentItemChanged(0);
 	}
 }
 
 void KviPopupEditor::newPopup()
 {
-	KviStr newName;
+	QString newName;
 	getUniquePopupName(0,newName);
-	KviMenuListViewItem * it = new KviMenuListViewItem(m_pListView,new KviPopupMenu(newName.ptr()));
+	KviMenuListViewItem * it = new KviMenuListViewItem(m_pListView,new KviKvsPopupMenu(newName));
 	m_pListView->setCurrentItem(it);
 }
 
@@ -1161,20 +1208,20 @@ void KviPopupEditor::saveLastEditedItem()
 {
 	if(m_pLastEditedItem == 0)return;
 
-	KviPopupMenu * m = m_pEditor->getMenu();
-	KviStr tmp = m->name();
-	KviStr old = m_pLastEditedItem->m_pPopup->name();
-	if(!kvi_strEqualCI(old.ptr(),tmp.ptr()))
+	KviKvsPopupMenu * m = m_pEditor->getMenu();
+	QString tmp = m->popupName();
+	QString old = m_pLastEditedItem->m_pPopup->popupName();
+	if(!KviQString::equalCI(tmp,old))
 	{
 		getUniquePopupName(m_pLastEditedItem,tmp);
-		m->setName(tmp.ptr());
+		m->setPopupName(tmp);
 	}
 
 	m_pLastEditedItem->replacePopup(m);
-	m_pLastEditedItem->setText(0,m->name());
+	m_pLastEditedItem->setText(0,m->popupName());
 }
 
-void KviPopupEditor::selectionChanged(QListViewItem *it)
+void KviPopupEditor::currentItemChanged(QListViewItem *it)
 {
 	saveLastEditedItem();
 
@@ -1195,31 +1242,31 @@ void KviPopupEditor::commit()
 
 	saveLastEditedItem();
 
-	//g_pPopupManager->clear();
+	//KviKvsPopupManager::instance()->clear();
 
 	KviMenuListViewItem * it = (KviMenuListViewItem *)m_pListView->firstChild();
 
 	// Copy the original popup dict
-	QDict<KviPopupMenu> copy(*(g_pPopupManager->popupDict()));
+	QDict<KviKvsPopupMenu> copy(*(KviKvsPopupManager::instance()->popupDict()));
 	copy.setAutoDelete(false);
 
 	while(it)
 	{
-		KviPopupMenu * p = g_pPopupManager->getPopup(it->m_pPopup->name());
+		KviKvsPopupMenu * p = KviKvsPopupManager::instance()->get(it->m_pPopup->popupName());
 		p->doClear();
 		p->copyFrom(it->m_pPopup);
 		it = (KviMenuListViewItem *)it->nextSibling();
 		// remove it from the original copy
-		copy.remove(p->name());
+		copy.remove(p->popupName());
 	}
 
 	// the remaining elements in the copy need to be removed from
 	// the "new" dictionary (they are no longer used)
-	QDictIterator<KviPopupMenu> iter(copy);
+	QDictIterator<KviKvsPopupMenu> iter(copy);
 
 	while(iter.current())
 	{
-		g_pPopupManager->removePopup(iter.currentKey());
+		KviKvsPopupManager::instance()->remove(iter.currentKey());
 		++iter;
 	}
 	
@@ -1228,12 +1275,12 @@ void KviPopupEditor::commit()
 }
 
 
-void KviPopupEditor::getUniquePopupName(KviMenuListViewItem *item,KviStr &buffer)
+void KviPopupEditor::getUniquePopupName(KviMenuListViewItem *item,QString &buffer)
 {
 	__range_valid(m_bOneTimeSetupDone);
 
-	if(buffer.isEmpty())buffer = __tr("unnamed");
-	KviStr newName = buffer;
+	if(buffer.isEmpty())buffer = __tr2qs("unnamed");
+	QString newName = buffer;
 
 	bool bFound = true;
 	int idx = 1;
@@ -1244,10 +1291,10 @@ void KviPopupEditor::getUniquePopupName(KviMenuListViewItem *item,KviStr &buffer
 
 		for(KviMenuListViewItem * ch = (KviMenuListViewItem *)(m_pListView->firstChild());ch;ch = (KviMenuListViewItem *)ch->nextSibling())
 		{
-			if(kvi_strEqualCI(newName.ptr(),ch->m_pPopup->name()) && (ch != item))
+			if(KviQString::equalCI(newName,ch->m_pPopup->popupName()) && (ch != item))
 			{
 				bFound = true;
-				newName.sprintf("%s.%d",buffer.ptr(),idx);
+				KviQString::sprintf(newName,"%Q.%d",&buffer,idx);
 				idx++;
 				break;
 			}
