@@ -21,9 +21,9 @@
 //   Inc. ,59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 //
 //=============================================================================
+#define _WANT_OPTION_FLAGS_
 
 #include "edituser.h"
-
 #include "kvi_regusersdb.h"
 
 #include "kvi_locale.h"
@@ -37,7 +37,8 @@
 #include "kvi_msgbox.h"
 #include "kvi_fileutils.h"
 #include "kvi_settings.h"
-
+#include "kvi_stringconversion.h"
+#include "kvi_options.h"
 
 #include <qlayout.h>
 #include <qlabel.h>
@@ -52,6 +53,7 @@
 #include <qpainter.h>
 #include <qhbox.h>
 #include <qvbox.h>
+#include <qinputdialog.h>
 
 #include "wizard.h"
 
@@ -67,6 +69,7 @@ extern KviRegisteredUsersDialog * g_pRegisteredUsersDialog;
 static KviRegisteredUserDataBase * g_pLocalRegisteredUserDataBase = 0; // local copy!
 
 
+
 KviReguserPropertiesDialog::KviReguserPropertiesDialog(QWidget * p,QDict<QString> * dict)
 : QDialog(p,"property_editor",true)
 {
@@ -124,6 +127,11 @@ KviReguserPropertiesDialog::~KviReguserPropertiesDialog()
 {
 }
 
+void KviReguserPropertiesDialog::closeEvent(QCloseEvent *e)
+{
+	e->accept();
+	//delete this;
+}
 
 void KviReguserPropertiesDialog::fillData()
 {
@@ -266,6 +274,11 @@ KviReguserMaskDialog::~KviReguserMaskDialog()
 {
 }
 
+void KviReguserMaskDialog::closeEvent(QCloseEvent *e)
+{
+	e->accept();
+	//delete this;
+}
 
 void KviReguserMaskDialog::okClicked()
 {
@@ -291,7 +304,14 @@ KviRegisteredUserEntryDialog::KviRegisteredUserEntryDialog(QWidget *p,KviRegiste
 : QTabDialog(p,"reguser_entry_editor",bModal)
 {
 	m_pUser = r;
-
+	m_pCustomColor = new QColor();
+	
+	if(r)
+	{
+		QString col=r->getProperty("customColor");
+		KviStringConversion::fromString(col,(*m_pCustomColor));
+	}
+	
 	m_pPropertyDict = new QDict<QString>(17,false);
 	m_pPropertyDict->setAutoDelete(true);
 
@@ -302,7 +322,7 @@ KviRegisteredUserEntryDialog::KviRegisteredUserEntryDialog(QWidget *p,KviRegiste
 
 	QWidget * p1 = new QWidget(this);
 
-	QGridLayout * g = new QGridLayout(p1,5,2,4,4);
+	QGridLayout * g = new QGridLayout(p1,6,2,4,4);
 
 	QLabel * l = new QLabel(__tr2qs("Name:"),p1);
 	g->addWidget(l,0,0);
@@ -310,22 +330,27 @@ KviRegisteredUserEntryDialog::KviRegisteredUserEntryDialog(QWidget *p,KviRegiste
 	m_pNameEdit = new QLineEdit(p1);
 	g->addWidget(m_pNameEdit,0,1);
 
+	l = new QLabel(__tr2qs("Comment:"),p1);
+	g->addWidget(l,1,0);
+	
+	m_pCommentEdit = new QLineEdit(p1);
+	g->addWidget(m_pCommentEdit,1,1);
 
 	QFrame * f = new QFrame(p1);
-	g->addMultiCellWidget(f,1,1,0,1);
+	g->addMultiCellWidget(f,2,2,0,1);
 	f->setFrameStyle(QFrame::HLine | QFrame::Sunken);
 
 	l = new QLabel(__tr2qs("Masks:"),p1);
-	g->addMultiCellWidget(l,2,2,0,1);
+	g->addMultiCellWidget(l,3,3,0,1);
 
 	m_pMaskListBox = new QListBox(p1);
 	connect(m_pMaskListBox,SIGNAL(currentChanged(QListBoxItem *)),this,SLOT(maskCurrentChanged(QListBoxItem *)));
 	m_pMaskListBox->setMinimumSize(300,200);
 
-	g->addMultiCellWidget(m_pMaskListBox,3,3,0,1);
+	g->addMultiCellWidget(m_pMaskListBox,4,4,0,1);
 
 	QHBox * b = new QHBox(p1);
-	g->addMultiCellWidget(b,4,4,0,1);
+	g->addMultiCellWidget(b,5,5,0,1);
 	b->setSpacing(4);
 
 	m_pAddMaskButton = new QPushButton(__tr2qs("&Add..."),b);
@@ -342,7 +367,7 @@ KviRegisteredUserEntryDialog::KviRegisteredUserEntryDialog(QWidget *p,KviRegiste
 	connect(m_pEditMaskButton,SIGNAL(clicked()),this,SLOT(editMaskClicked()));
 	m_pEditMaskButton->setIconSet(*(g_pIconManager->getSmallIcon(KVI_SMALLICON_EDITITEM)));
 
-	g->setRowStretch(3,1);
+	g->setRowStretch(4,1);
 	g->setColStretch(1,1);
 
 	addTab(p1,__tr2qs("Identity"));
@@ -353,7 +378,7 @@ KviRegisteredUserEntryDialog::KviRegisteredUserEntryDialog(QWidget *p,KviRegiste
 
 	g = new QGridLayout(p2,6,3,5,2);
 
-	m_pNotifyCheck = new QCheckBox(__tr2qs("Notify when user is online"),p2);
+	m_pNotifyCheck = new KviStyledCheckBox(__tr2qs("Notify when user is online"),p2);
 	g->addMultiCellWidget(m_pNotifyCheck,0,0,0,2);
 
 	l = new QLabel(__tr2qs("Notify nicknames:"),p2);
@@ -393,9 +418,15 @@ KviRegisteredUserEntryDialog::KviRegisteredUserEntryDialog(QWidget *p,KviRegiste
 	f->setFrameStyle(QFrame::HLine | QFrame::Sunken);
 	g->addMultiCellWidget(f,4,4,0,2);
 
+	m_pCustomColorCheck = new KviStyledCheckBox(__tr2qs("Use custom color in userlist"),p2);
+	g->addMultiCellWidget(m_pCustomColorCheck,5,5,0,1);
+
+	m_pCustomColorSelector = new KviColorSelector(p2,QString::null,m_pCustomColor,1);
+	g->addWidget(m_pCustomColorSelector,5,2);
+	
 	QPushButton * pb = new QPushButton(__tr2qs("All Properties..."),p2);
 	connect(pb,SIGNAL(clicked()),this,SLOT(editAllPropertiesClicked()));
-	g->addWidget(pb,5,2);
+	g->addWidget(pb,6,2);
 
 	g->setColStretch(1,1);
 	g->setRowStretch(3,1);
@@ -410,6 +441,7 @@ KviRegisteredUserEntryDialog::KviRegisteredUserEntryDialog(QWidget *p,KviRegiste
 	if(r)
 	{
 		m_pNameEdit->setText(r->name());
+		m_pCommentEdit->setText(r->getProperty("comment"));
 		for(KviIrcMask * m = r->maskList()->first();m;m = r->maskList()->next())
 		{
 			QString mk = m->nick();
@@ -427,6 +459,11 @@ KviRegisteredUserEntryDialog::KviRegisteredUserEntryDialog(QWidget *p,KviRegiste
 			m_pNotifyNick->setText(notp);
 			m_pNotifyNick->setEnabled(true);
 		}
+		
+		if( r->getBoolProperty("useCustomColor"))
+		{
+			m_pCustomColorCheck->setChecked(1);
+		}
 
 		if(r->propertyDict())
 		{
@@ -440,10 +477,18 @@ KviRegisteredUserEntryDialog::KviRegisteredUserEntryDialog(QWidget *p,KviRegiste
 	}
 }
 
+void KviRegisteredUserEntryDialog::closeEvent(QCloseEvent *e)
+{
+	e->accept();
+	okClicked();
+	//delete this;
+}
+
 KviRegisteredUserEntryDialog::~KviRegisteredUserEntryDialog()
 {
 	delete m_pAvatar;
 	delete m_pPropertyDict;
+	delete m_pCustomColor;
 }
 
 void KviRegisteredUserEntryDialog::maskCurrentChanged(QListBoxItem *it)
@@ -454,8 +499,10 @@ void KviRegisteredUserEntryDialog::maskCurrentChanged(QListBoxItem *it)
 
 void KviRegisteredUserEntryDialog::okClicked()
 {
+	QString szGroup;
 	if(m_pUser)
 	{
+		szGroup=m_pUser->group();
 		g_pLocalRegisteredUserDataBase->removeUser(m_pUser->name());
 	}
 
@@ -480,7 +527,8 @@ void KviRegisteredUserEntryDialog::okClicked()
 
 
 	u = g_pLocalRegisteredUserDataBase->addUser(szNameOk);
-
+	u->setGroup(szGroup);
+	
 	if(!u)
 	{
 		// ops... no way
@@ -501,13 +549,14 @@ void KviRegisteredUserEntryDialog::okClicked()
 		cnt--;
 		idx++;
 	}
-
+	u->setProperty("comment",m_pCommentEdit->text());
+	
 	m_pAvatarSelector->commit();
 
 	if(!m_pAvatar->isNull())
 	{
-		KviStr szPath = m_pAvatar->path();
-		u->setProperty("avatar",szPath.ptr());
+		QString szPath = m_pAvatar->path();
+		u->setProperty("avatar",szPath);
 	}
 
 	if(m_pNotifyCheck->isChecked())
@@ -519,7 +568,7 @@ void KviRegisteredUserEntryDialog::okClicked()
 			u->setProperty("notify",szNicks);
 		}
 	}
-
+	
 	m_pPropertyDict->remove("notify");
 	m_pPropertyDict->remove("avatar");
 
@@ -530,7 +579,20 @@ void KviRegisteredUserEntryDialog::okClicked()
 		++it;
 	}
 
+	if(m_pCustomColorCheck->isChecked())
+	{
+		u->setProperty("useCustomColor","1");
+	} else {
+		u->setProperty("useCustomColor","0");
+	}
+	
+	QString col;
+	KviStringConversion::toString(m_pCustomColorSelector->getColor(),col);
+//	debug(col);
+	u->setProperty("customColor",col.utf8().data());
+
 	accept();
+	g_pApp->optionResetUpdate(KviOption_resetUpdateGui);
 }
 
 
@@ -633,11 +695,17 @@ void KviRegisteredUserEntryDialog::editAllPropertiesClicked()
 	if(avatar)
 	{
 		if(!avatar->isEmpty())
-			m_pAvatarSelector->setImagePath(avatar->latin1());
+			m_pAvatarSelector->setImagePath(*avatar);
 	}
 
 }
 
+KviRegisteredUsersDialogItem::KviRegisteredUsersDialogItem(QListViewItem * par,KviRegisteredUser * u)
+: KviRegisteredUsersDialogItemBase(User,par), m_pUser(u)
+{
+	setText(0,u->name());
+}
+
 void KviRegisteredUsersDialogItem::paintCell(QPainter * p,const QColorGroup &cg,int column,int width,int align)
 {
 	if(column == 1)
@@ -695,6 +763,7 @@ KviRegisteredUsersDialog::KviRegisteredUsersDialog(QWidget * par)
 	m_pListView->setAllColumnsShowFocus(true);
 
 	m_pListView->setSelectionMode(QListView::Extended);
+	m_pListView->setRootIsDecorated(TRUE);
 
 #ifdef COMPILE_INFO_TIPS
 	QToolTip::add(m_pListView,__tr2qs("<center>This is the list of registered users. " \
@@ -727,6 +796,13 @@ KviRegisteredUsersDialog::KviRegisteredUsersDialog(QWidget * par)
 	QToolTip::add(m_pAddButton,__tr2qs("Open the edit dialog to create a new user entry."));
 #endif // COMPILE_INFO_TIPS
 	m_pAddButton->setIconSet(*(g_pIconManager->getSmallIcon(KVI_SMALLICON_NEWITEM)));
+	
+	m_pAddGroupButton = new QPushButton(__tr2qs("&Add Group..."),vbox);
+	connect(m_pAddGroupButton,SIGNAL(clicked()),this,SLOT(addGroupClicked()));
+#ifdef COMPILE_INFO_TIPS
+	QToolTip::add(m_pAddGroupButton,__tr2qs("Adds a new group"));
+#endif // COMPILE_INFO_TIPS
+	m_pAddGroupButton->setIconSet(*(g_pIconManager->getSmallIcon(KVI_SMALLICON_NEWITEM)));
 
 	m_pRemoveButton = new QPushButton(__tr2qs("Re&move"),vbox);
 	connect(m_pRemoveButton,SIGNAL(clicked()),this,SLOT(removeClicked()));
@@ -789,6 +865,7 @@ KviRegisteredUsersDialog::KviRegisteredUsersDialog(QWidget * par)
 	g->setRowStretch(1,1);
 
 	connect(m_pListView,SIGNAL(selectionChanged()),this,SLOT(selectionChanged()));
+	connect(m_pListView,SIGNAL(rightButtonClicked ( QListViewItem *, const QPoint &, int ) ),this,SLOT(listViewRightButtonClicked ( QListViewItem *, const QPoint &, int )));
 
 	fillList();
 
@@ -824,63 +901,179 @@ KviRegisteredUsersDialog::~KviRegisteredUsersDialog()
 void KviRegisteredUsersDialog::itemPressed(QListViewItem *it,const QPoint &pnt,int c)
 {
 	if(!it)return;
-	KviRegisteredUsersDialogItem *i = (KviRegisteredUsersDialogItem *)it;
-
-	QRect r = m_pListView->itemRect(i);
-	int daw = m_pListView->columnWidth(0);
-
-	QPoint ppp = m_pListView->mapFromGlobal(pnt);
-
-	if((c == 1) && (ppp.x() < (r.height() + 5 + daw)))
+	KviRegisteredUsersDialogItemBase* b=(KviRegisteredUsersDialogItemBase*)it;
+	if(b->type()==KviRegisteredUsersDialogItemBase::User)
 	{
-		// notify list toggle
-		if(i->user()->getProperty("notify"))
+		KviRegisteredUsersDialogItem *i = (KviRegisteredUsersDialogItem *)it;
+	
+		QRect r = m_pListView->itemRect(i);
+		int daw = m_pListView->columnWidth(0);
+	
+		QPoint ppp = m_pListView->mapFromGlobal(pnt);
+	
+		if((c == 1) && (ppp.x() < (r.height() + 5 + daw)))
 		{
-			i->user()->setProperty("notify",""); // kill that
-		} else {
-			// try to find the nicknames to be notified
-			QString szMask;
-
-			for(KviIrcMask * m = i->user()->maskList()->first();m;m = i->user()->maskList()->next())
+			// notify list toggle
+			if(i->user()->getProperty("notify"))
 			{
-				QString tmp = m->nick();
-				if((tmp.find('*') == -1) && (tmp.find('?') == -1) && (!tmp.isEmpty()))
+				i->user()->setProperty("notify",""); // kill that
+			} else {
+				// try to find the nicknames to be notified
+				QString szMask;
+	
+				for(KviIrcMask * m = i->user()->maskList()->first();m;m = i->user()->maskList()->next())
 				{
-					if(!szMask.isEmpty())szMask.append(' ');
-					szMask.append(tmp);
+					QString tmp = m->nick();
+					if((tmp.find('*') == -1) && (tmp.find('?') == -1) && (!tmp.isEmpty()))
+					{
+						if(!szMask.isEmpty())szMask.append(' ');
+						szMask.append(tmp);
+					}
 				}
+				// if the nickname list is still empty , build a dummy nick to notify
+				szMask = i->user()->name();
+				szMask.replace(" ","");
+				szMask.replace("'","");
+				szMask.replace("&","");
+				szMask.replace(",","");
+	
+				i->user()->setProperty("notify",szMask);
 			}
-			// if the nickname list is still empty , build a dummy nick to notify
-			szMask = i->user()->name();
-			szMask.replace(" ","");
-			szMask.replace("'","");
-			szMask.replace("&","");
-			szMask.replace(",","");
-
-			i->user()->setProperty("notify",szMask);
+			m_pListView->update();
 		}
-		m_pListView->update();
 	}
 }
 
 void KviRegisteredUsersDialog::itemDoubleClicked(QListViewItem *it)
 {
 	if(!it)return;
-	editItem((KviRegisteredUsersDialogItem *)it);
+	KviRegisteredUsersDialogItemBase* b=(KviRegisteredUsersDialogItemBase*)it;
+	if(b->type()==KviRegisteredUsersDialogItemBase::User)
+	{
+		editItem((KviRegisteredUsersDialogItem *)it);
+	} else	{
+		KviRegisteredUsersGroupItem *i = (KviRegisteredUsersGroupItem *)b;
+		editGroup(i->group());
+	}
 }
 
+void KviRegisteredUsersDialog::addGroupClicked()
+{
+	bool ok;
+	QString text = QInputDialog::getText(
+		"KVIrc", __tr("Group name:"), QLineEdit::Normal,
+		QString::null, &ok, this );
+	if ( ok && !text.isEmpty() ) {
+		g_pLocalRegisteredUserDataBase->addGroup(text);
+		fillList();
+	}
+}
+void KviRegisteredUsersDialog::editGroup(KviRegisteredUserGroup* group)
+{
+	bool ok;
+	
+	QString text = QInputDialog::getText(
+		"KVIrc", __tr("Group name:"), QLineEdit::Normal,
+		group->name(), &ok, this );
+	if ( ok && !text.isEmpty() ) {
+		QString szOldGroup=group->name();
+		g_pLocalRegisteredUserDataBase->groupDict()->setAutoDelete(0);
+		g_pLocalRegisteredUserDataBase->groupDict()->remove(szOldGroup);
+		g_pLocalRegisteredUserDataBase->groupDict()->setAutoDelete(1);
+		group->setName(text);
+		g_pLocalRegisteredUserDataBase->groupDict()->insert(text,group);
+		
+		QDict<KviRegisteredUser> * d = g_pLocalRegisteredUserDataBase->userDict();
+		QDictIterator<KviRegisteredUser> it(*d);
+	
+		while(KviRegisteredUser * u = it.current())
+		{
+			if(u->group()==szOldGroup)
+				u->setGroup(text);
+			++it;
+		}
+		
+		
+		fillList();
+	}
+}
+
+void KviRegisteredUsersDialog::listViewRightButtonClicked ( QListViewItem * pItem, const QPoint & point, int )
+{
+	if(pItem)
+	{
+		KviRegisteredUsersDialogItemBase* b=(KviRegisteredUsersDialogItemBase*)pItem;
+		if(b->type()==KviRegisteredUsersDialogItemBase::User)
+		{
+			QPopupMenu *groups = new QPopupMenu;
+			
+			QDict<KviRegisteredUserGroup> * pGroups = g_pLocalRegisteredUserDataBase->groupDict();
+			QDictIterator<KviRegisteredUserGroup> git(*pGroups);
+			m_TmpDict.clear();
+			while(KviRegisteredUserGroup * g = git.current())
+			{
+				int id=groups->insertItem(git.currentKey());
+				m_TmpDict.replace(id,g);
+				++git;
+			}
+			
+			connect(groups,SIGNAL(activated ( int )),this,SLOT(moveToGroupMenuClicked(int)));
+			
+			QPopupMenu *mainPopup = new QPopupMenu;
+			mainPopup->insertItem(__tr("Move to group"),groups);
+			mainPopup->exec(point);
+		}
+	}
+}
+
+void KviRegisteredUsersDialog::moveToGroupMenuClicked(int id)
+{
+	QString szGroup=m_TmpDict.find(id)->name();
+	QListViewItemIterator it( m_pListView,  QListViewItemIterator::Selected );
+	while ( it.current() ) {
+		KviRegisteredUsersDialogItemBase* b=(KviRegisteredUsersDialogItemBase*)(it.current());
+		if(b->type()==KviRegisteredUsersDialogItemBase::User)
+		{
+			((KviRegisteredUsersDialogItem *)(it.current()))->user()->setGroup(szGroup);
+		} 
+		++it;
+	}
+	fillList();
+}
 
 void KviRegisteredUsersDialog::fillList()
 {
 	m_pListView->clear();
-
+	QDict<KviRegisteredUsersGroupItem> groupItems(5,false);
+	
+	QDict<KviRegisteredUserGroup> * pGroups = g_pLocalRegisteredUserDataBase->groupDict();
+	QDictIterator<KviRegisteredUserGroup> git(*pGroups);
+	while(KviRegisteredUserGroup * g = git.current())
+	{
+		KviRegisteredUsersGroupItem* pCur = new KviRegisteredUsersGroupItem(m_pListView,g);
+		groupItems.insert(g->name(),pCur);
+		pCur->setOpen(TRUE);
+		++git;
+	}
+	
 	QDict<KviRegisteredUser> * d = g_pLocalRegisteredUserDataBase->userDict();
 	QDictIterator<KviRegisteredUser> it(*d);
 	KviRegisteredUsersDialogItem * item;
 
 	while(KviRegisteredUser * u = it.current())
 	{
-		item = new KviRegisteredUsersDialogItem(m_pListView,u);
+		if(u->group().isEmpty())
+			u->setGroup(__tr("Default"));
+		if(groupItems.find(u->group()))
+			item = new KviRegisteredUsersDialogItem(groupItems.find(u->group()),u);
+		else if(groupItems.find(__tr("Default")))
+			item = new KviRegisteredUsersDialogItem(groupItems.find(__tr("Default")),u);
+		else { //should never be called
+			KviRegisteredUserGroup* pGroup = g_pLocalRegisteredUserDataBase->addGroup(__tr("Default"));
+			KviRegisteredUsersGroupItem* pCur = new KviRegisteredUsersGroupItem(m_pListView,pGroup);
+			groupItems.insert(__tr("Default"),pCur);
+			item = new KviRegisteredUsersDialogItem(pCur,u);
+		}
 		item->setPixmap(0,*(g_pIconManager->getSmallIcon(KVI_SMALLICON_LINUX)));
 		++it;
 	}
@@ -893,12 +1086,15 @@ void KviRegisteredUsersDialog::fillList()
 
 void KviRegisteredUsersDialog::closeEvent(QCloseEvent *e)
 {
+	m_pListView->clear();
 	e->accept();
-	delete this;
+	okClicked();
+	//delete this;
 }
 
 void KviRegisteredUsersDialog::okClicked()
 {
+	m_pListView->clear();
 	g_pRegisteredUserDataBase->copyFrom(g_pLocalRegisteredUserDataBase);
 	g_pApp->restartNotifyLists();
 	delete this;
@@ -906,6 +1102,7 @@ void KviRegisteredUsersDialog::okClicked()
 
 void KviRegisteredUsersDialog::cancelClicked()
 {
+	m_pListView->clear();
 	delete this;
 }
 
@@ -937,28 +1134,52 @@ void KviRegisteredUsersDialog::addWizardClicked()
 
 void KviRegisteredUsersDialog::removeClicked()
 {
-	KviRegisteredUsersDialogItem *it = (KviRegisteredUsersDialogItem *)m_pListView->firstChild();
-	KviPtrList<KviRegisteredUsersDialogItem> l;
-	l.setAutoDelete(false);
-	while(it)
-	{
-		if(it->isSelected())l.append(it);
-		it = (KviRegisteredUsersDialogItem *)it->nextSibling();
-	}
-
-	for(KviRegisteredUsersDialogItem * i = l.first();i;i = l.next())
-	{
-		g_pLocalRegisteredUserDataBase->removeUser(i->user()->name());
-		delete i;
+	QListViewItemIterator it( m_pListView,  QListViewItemIterator::Selected );
+	while ( it.current() ) {
+		KviRegisteredUsersDialogItemBase* b=(KviRegisteredUsersDialogItemBase*)(it.current());
+		if(b->type()==KviRegisteredUsersDialogItemBase::User)
+		{
+			g_pLocalRegisteredUserDataBase->removeUser(((KviRegisteredUsersDialogItem *)(it.current()))->user()->name());
+		} else {
+			g_pLocalRegisteredUserDataBase->removeGroup(((KviRegisteredUsersGroupItem *)(it.current()))->group()->name());
+		}
+		++it;
 	}
+	fillList();
+// 	KviRegisteredUsersDialogItem *it = (KviRegisteredUsersDialogItem *)m_pListView->firstChild();
+// 	KviRegisteredUsersDialogItemBase* b=(KviRegisteredUsersDialogItemBase*)it;
+// 	if(b->type()==KviRegisteredUsersDialogItemBase::User)
+// 	{
+// 		KviPtrList<KviRegisteredUsersDialogItem> l;
+// 		l.setAutoDelete(false);
+// 		while(it)
+// 		{
+// 			if(it->isSelected())l.append(it);
+// 			it = (KviRegisteredUsersDialogItem *)it->nextSibling();
+// 		}
+// 	
+// 		for(KviRegisteredUsersDialogItem * i = l.first();i;i = l.next())
+// 		{
+// 			//g_pLocalRegisteredUserDataBase->removeUser(i->user()->name());
+// 			delete i;
+// 		}
+// 	} else {
+// 		
+// 	}
 }
 
 void KviRegisteredUsersDialog::editClicked()
 {
-	KviRegisteredUsersDialogItem *i = (KviRegisteredUsersDialogItem *)(m_pListView->currentItem());
-	if(!i)return;
-
-	editItem(i);
+	KviRegisteredUsersDialogItemBase* b=(KviRegisteredUsersDialogItemBase*)(m_pListView->currentItem());
+	if(!b)return;
+	if(b->type()==KviRegisteredUsersDialogItemBase::User)
+	{
+		KviRegisteredUsersDialogItem *i = (KviRegisteredUsersDialogItem *)b;
+		editItem(i);
+	} else {
+		KviRegisteredUsersGroupItem *i = (KviRegisteredUsersGroupItem *)b;
+		editGroup(i->group());
+	}
 }
 
 void KviRegisteredUsersDialog::editItem(KviRegisteredUsersDialogItem * i)
@@ -1003,17 +1224,7 @@ void KviRegisteredUsersDialog::editItem(KviRegisteredUsersDialogItem * i)
 
 void KviRegisteredUsersDialog::selectionChanged()
 {
-	bool bHaveSelected = false;
-	QListViewItem * it = m_pListView->firstChild();
-	while(it)
-	{
-		if(it->isSelected())
-		{
-			bHaveSelected = true;
-			break;
-		}
-		it = it->nextSibling();
-	}
+	bool bHaveSelected = !m_pListView->selectedItem();
 
 	m_pEditButton->setEnabled(bHaveSelected);
 	m_pRemoveButton->setEnabled(bHaveSelected);
@@ -1044,7 +1255,7 @@ void KviRegisteredUsersDialog::exportClicked()
 
 	if(nEntries < 1)
 	{
-		kvi_warningBox(__tr("No entries selected."));
+		KviMessageBox::warning(__tr("No entries selected."));
 		return;
 	}
 
@@ -1057,7 +1268,7 @@ void KviRegisteredUsersDialog::exportClicked()
 	KviFile f(buffer.ptr());
 	if(!f.open(IO_WriteOnly | IO_Truncate))
 	{
-		kvi_warningBox(__tr("Can't open file %s for writing."),buffer.ptr());
+		KviMessageBox::warning(__tr("Can't open file %s for writing."),buffer.ptr());
 		return;
 	}
 
@@ -1110,7 +1321,7 @@ void KviRegisteredUsersDialog::exportClicked()
 				QString avatar;
 				if(u->getProperty("avatar",avatar))
 				{
-					KviAvatar * av = g_pIconManager->getAvatar(QString::null,avatar.latin1());
+					KviAvatar * av = g_pIconManager->getAvatar(QString::null,avatar);
 					if(av)
 					{
 						if(!av->pixmap()->isNull())
@@ -1138,7 +1349,7 @@ void KviRegisteredUsersDialog::exportClicked()
 
 
 write_error:
-	kvi_warningBox(__tr("Can't export the registered users database: Write error."));
+	KviMessageBox::warning(__tr("Can't export the registered users database: Write error."));
 	f.close();
 	return;
 
@@ -1149,16 +1360,17 @@ succesfull_export:
 
 void KviRegisteredUsersDialog::importClicked()
 {
-	KviStr buffer;
+	//KviStr buffer;
+	QString buffer;
 
 	if(!KviFileDialog::askForOpenFileName(buffer,__tr("Choose a Filename - KVIrc")))return;
 
 	if(!g_pRegisteredUsersDialog)return; // we have been deleted!
 
-	KviFile f(buffer.ptr());
+	KviFile f(buffer);
 	if(!f.open(IO_ReadOnly))
 	{
-		kvi_warningBox(__tr("Can't open file %s for reading."),buffer.ptr());
+		KviMessageBox::warning(__tr2qs("Can't open file %s for reading."),&buffer);
 		return;
 	}
 
@@ -1169,14 +1381,14 @@ void KviRegisteredUsersDialog::importClicked()
 
 	if(hf.magic != KVI_REGUSER_DB_FILE_MAGIC)
 	{
-		kvi_warningBox(__tr("The file %s doesn't appear to be a valid registered users database."),buffer.ptr());
+		KviMessageBox::warning(__tr2qs("The file %s doesn't appear to be a valid registered users database."),&buffer);
 		f.close();
 		return;
 	}
 
 	if(hf.version != KVI_REGUSER_DB_FILE_VERSION)
 	{
-		kvi_warningBox(__tr("The file %s contains an invalid registered users database version."),buffer.ptr());
+		KviMessageBox::warning(__tr2qs("The file %s contains an invalid registered users database version."),&buffer);
 		f.close();
 		return;
 	}
@@ -1231,10 +1443,10 @@ void KviRegisteredUsersDialog::importClicked()
 			int rnm = 0 ;
 			do
 			{
-				g_pApp->getLocalKvircDirectory(fPath,KviApp::Avatars,fName);
+				g_pApp->getLocalKvircDirectory(fPath,KviApp::Avatars,fName.ptr());
 				fPath.append(KviStr::Format,"%d.png",rnm);
 				rnm++;
-			} while(kvi_fileExists(fPath.ptr()));
+			} while(KviFileUtils::fileExists(fPath.ptr()));
 
 			if(!img.save(fPath.ptr(),"PNG"))
 			{
@@ -1249,7 +1461,7 @@ void KviRegisteredUsersDialog::importClicked()
 
 
 read_error:
-	kvi_warningBox(__tr("Can't import the registered users database: Read error."));
+	KviMessageBox::warning(__tr("Can't import the registered users database: Read error."));
 	f.close();
 	return;
 
