@@ -23,7 +23,7 @@
 //=============================================================================
 
 #include "kvi_module.h"
-#include "kvi_uparser.h"
+//#include "kvi_uparser.h"
 #include "kvi_regusersdb.h"
 #include "kvi_command.h"
 #include "kvi_out.h"
@@ -88,6 +88,9 @@ KviRegisteredUsersDialog * g_pRegisteredUsersDialog = 0;
 		Opens the registeredusers dialog
 	@syntax:
 		reguser.edit [-t]
+	@switches:
+		!sw: -t | --toplevel
+		the editor is created as a toplevel window (dialog)
 	@description:
 		Shows the registered user database editor.[br]
 		If the [-t] switch is passed, the editor is created as a toplevel window (dialog), otherwise
@@ -102,21 +105,14 @@ KviRegisteredUsersDialog * g_pRegisteredUsersDialog = 0;
 		[fnc]$reguser.list[/fnc]
 */
 
-static bool reguser_module_cmd_edit(KviModule *m,KviCommand *c)
+static bool reguser_kvs_cmd_edit(KviKvsModuleCommandCall * c)
 {
-	ENTER_STACK_FRAME(c,"reguser.edit");
-
-	KviStr dummy;
-	if(!g_pUserParser->parseCmdFinalPart(c,dummy))return false;
-
-	///
 	if(g_pRegisteredUsersDialog)
 	{
-		if(c->hasSwitch('t'))
+		if(c->hasSwitch('t',"toplevel"))
 		{
 			if(g_pRegisteredUsersDialog->parent())
 			{
-//#warning "CENTER THE DIALOG ? Remember its position ?"
 				g_pRegisteredUsersDialog->reparent(0,QPoint(0,0),true);
 			}
 		} else {
@@ -126,7 +122,7 @@ static bool reguser_module_cmd_edit(KviModule *m,KviCommand *c)
 			}
 		}
 	} else {
-		if(c->hasSwitch('t'))
+		if(c->hasSwitch('t',"toplevel"))
 		{
 			g_pRegisteredUsersDialog = new KviRegisteredUsersDialog(0);
 		} else {
@@ -137,17 +133,9 @@ static bool reguser_module_cmd_edit(KviModule *m,KviCommand *c)
 	g_pRegisteredUsersDialog->show();
 	g_pRegisteredUsersDialog->raise();
 	g_pRegisteredUsersDialog->setFocus();
-	///
-
-//	if(name.hasData())
-//	{
-//		g_pRegisteredUserEditor->editUser(name.ptr(),c->hasSwitch('f'));
-//	}
-
-	return c->leaveStackFrame();
+	return true;
 }
 
-
 /*
 	@doc: reguser.add
 	@type:
@@ -185,52 +173,49 @@ static bool reguser_module_cmd_edit(KviModule *m,KviCommand *c)
 		[fnc]$reguser.list[/fnc]
 */
 
-static bool reguser_module_cmd_add(KviModule *m,KviCommand *c)
+static bool reguser_kvs_cmd_add(KviKvsModuleCommandCall * c)
 {
-	ENTER_STACK_FRAME(c,"reguser.add");
-
-	KviStr name,mask;
-	if(!g_pUserParser->parseCmdSingleToken(c,name))return false;
-	if(!g_pUserParser->parseCmdFinalPart(c,mask))return false;
-
-	if(name.isEmpty())
+	QString szName; 
+	QString szMask;
+	KVSM_PARAMETERS_BEGIN(c) 
+		KVSM_PARAMETER("name",KVS_PT_STRING,0,szName) 
+		KVSM_PARAMETER("mask",KVS_PT_STRING,0,szMask) 
+	KVSM_PARAMETERS_END(c) 
+	if(szName.isEmpty())
 	{
-		if(!c->hasSwitch('q'))c->warning(__tr2qs("No name specified"));
-		return c->leaveStackFrame();
+		if(!c->hasSwitch('q',"quiet"))c->warning(__tr2qs("No name specified"));
+		return true;
 	}
 
-	if(c->hasSwitch('r'))g_pRegisteredUserDataBase->removeUser(name.ptr());
+	if(c->hasSwitch('r',"replace"))g_pRegisteredUserDataBase->removeUser(szName);
 
 
-	KviRegisteredUser * u = g_pRegisteredUserDataBase->addUser(name.ptr());
+	KviRegisteredUser * u = g_pRegisteredUserDataBase->addUser(szName);
 
 	if(u == 0)
 	{
-		if(c->hasSwitch('f'))
+		if(c->hasSwitch('f',"force"))
 		{
-			u = g_pRegisteredUserDataBase->findUserByName(name.ptr());
+			u = g_pRegisteredUserDataBase->findUserByName(szName);
 		} else {
-			if(!c->hasSwitch('q'))c->warning(__tr2qs("User already registered: found exact name match"));
+			if(!c->hasSwitch('q',"quiet"))c->warning(__tr2qs("User already registered: found exact name match"));
 		}
 	}
 
 
 	if(u)
 	{
-		if(mask.hasData())
+		if(!szMask.isEmpty())
 		{
-			KviIrcMask * m = new KviIrcMask(mask.ptr());
+			KviIrcMask * m = new KviIrcMask(szMask);
 			u = g_pRegisteredUserDataBase->addMask(u,m);
 			if(u != 0)
 			{
-				if(!c->hasSwitch('q'))c->warning(__tr2qs("Mask %s is already used to identify user %s"),mask.ptr(),u->name().latin1());
+				if(!c->hasSwitch('q',"quiet"))c->warning(__tr2qs("Mask %Q is already used to identify user %s"),&szMask,u->name().utf8().data());
 			}
 		}
 	}
-
-	// no need to restart notify list for now
-
-	return c->leaveStackFrame();
+	return true;
 }
 
 /*
@@ -264,27 +249,26 @@ static bool reguser_module_cmd_add(KviModule *m,KviCommand *c)
 		[fnc]$reguser.list[/fnc]
 */
 
-static bool reguser_module_cmd_remove(KviModule *m,KviCommand *c)
+static bool reguser_kvs_cmd_remove(KviKvsModuleCommandCall * c)
 {
-	ENTER_STACK_FRAME(c,"reguser.remove");
-
-	KviStr name;
-	if(!g_pUserParser->parseCmdFinalPart(c,name))return false;
-
-	if(name.isEmpty())
+	QString szName; 
+	KVSM_PARAMETERS_BEGIN(c) 
+		KVSM_PARAMETER("name",KVS_PT_STRING,0,szName) 
+	KVSM_PARAMETERS_END(c) 
+	
+	if(szName.isEmpty())
 	{
-		if(!c->hasSwitch('q'))c->warning(__tr2qs("No name specified"));
-		return c->leaveStackFrame();
+		if(!c->hasSwitch('q',"quiet"))c->warning(__tr2qs("No name specified"));
+		return true;
 	}
 
-	if(!(g_pRegisteredUserDataBase->removeUser(name.ptr())))
+	if(!(g_pRegisteredUserDataBase->removeUser(szName)))
 	{
-		c->warning(__tr2qs("User not found (%s)"),name.ptr());
+		c->warning(__tr2qs("User not found (%Q)"),&szName);
 	} else {
-		if(c->hasSwitch('n'))g_pApp->restartNotifyLists();
+		if(c->hasSwitch('n',"restartnotifylists"))g_pApp->restartNotifyLists();
 	}
-
-	return c->leaveStackFrame();
+	return true;
 }
 
 /*
@@ -326,45 +310,44 @@ static bool reguser_module_cmd_remove(KviModule *m,KviCommand *c)
 		[fnc]reguser.exactMatch[/fnc]
 */
 
-static bool reguser_module_cmd_addmask(KviModule *m,KviCommand *c)
+static bool reguser_kvs_cmd_addmask(KviKvsModuleCommandCall * c)
 {
-	ENTER_STACK_FRAME(c,"reguser.addmask");
-
-	KviStr name,mask;
-	if(!g_pUserParser->parseCmdSingleToken(c,name))return false;
-	if(!g_pUserParser->parseCmdFinalPart(c,mask))return false;
-
-	if(name.isEmpty())
+	QString szName; 
+	QString szMask;
+	KVSM_PARAMETERS_BEGIN(c) 
+		KVSM_PARAMETER("name",KVS_PT_STRING,0,szName) 
+		KVSM_PARAMETER("mask",KVS_PT_STRING,0,szMask) 
+	KVSM_PARAMETERS_END(c) 
+	
+	if(szName.isEmpty())
 	{
 		c->warning(__tr2qs("No name specified"));
-		return c->leaveStackFrame();
+		return true;
 	}
 
-	if(mask.isEmpty())
+	if(szMask.isEmpty())
 	{
 		c->warning(__tr2qs("No mask specified"));
-		return c->leaveStackFrame();
+		return true;
 	}
 
-	KviRegisteredUser * u = g_pRegisteredUserDataBase->findUserByName(name.ptr());
+	KviRegisteredUser * u = g_pRegisteredUserDataBase->findUserByName(szName);
 	if(!u)
 	{
-		c->warning(__tr2qs("User %s not found"),name.ptr());
-		return c->leaveStackFrame();
+		c->warning(__tr2qs("User %Q not found"),&szName);
+		return true;
 	}
 
-	KviIrcMask * mk = new KviIrcMask(mask.ptr());
+	KviIrcMask * mk = new KviIrcMask(szMask);
 
-	if(c->hasSwitch('f'))g_pRegisteredUserDataBase->removeMask(*mk);
+	if(c->hasSwitch('f',"force"))g_pRegisteredUserDataBase->removeMask(*mk);
 
 	u = g_pRegisteredUserDataBase->addMask(u,mk);
 
-	if(u != 0)c->warning(__tr2qs("Mask %s already used to identify user %s"),mask.ptr(),u->name().latin1());
-
-	return c->leaveStackFrame();
+	if(u != 0)c->warning(__tr2qs("Mask %Q already used to identify user %Q"),&szMask,&(u->name()));
+	return true;
 }
 
-
 /*
 	@doc: reguser.delmask
 	@type:
@@ -400,29 +383,28 @@ static bool reguser_module_cmd_addmask(KviModule *m,KviCommand *c)
 		[fnc]reguser.exactMatch[/fnc]
 */
 
-static bool reguser_module_cmd_delmask(KviModule *m,KviCommand *c)
+static bool reguser_kvs_cmd_delmask(KviKvsModuleCommandCall * c)
 {
-	ENTER_STACK_FRAME(c,"reguser.delmask");
-
-	KviStr mask;
-	if(!g_pUserParser->parseCmdFinalPart(c,mask))return false;
-
-	if(mask.isEmpty())
+	QString szMask;
+	KVSM_PARAMETERS_BEGIN(c) 
+		KVSM_PARAMETER("mask",KVS_PT_STRING,0,szMask) 
+	KVSM_PARAMETERS_END(c) 
+	
+	if(szMask.isEmpty())
 	{
 		c->warning(__tr2qs("No mask specified"));
-		return c->leaveStackFrame();
+		return true;
 	}
 
-	KviIrcMask mk(mask.ptr());
+	KviIrcMask mk(szMask);
 	if(!g_pRegisteredUserDataBase->removeMask(mk))
 	{
-		c->warning(__tr2qs("Mask %s not found"),mask.ptr());
+		c->warning(__tr2qs("Mask %Q not found"),&szMask);
 	}
 
-	return c->leaveStackFrame();
+	return true;
 }
 
-
 /*
 	@doc: reguser.setproperty
 	@type:
@@ -458,40 +440,40 @@ static bool reguser_module_cmd_delmask(KviModule *m,KviCommand *c)
 		[fnc]$reguser.property[/fnc]
 */
 
-static bool reguser_module_cmd_setproperty(KviModule *m,KviCommand *c)
+static bool reguser_kvs_cmd_setproperty(KviKvsModuleCommandCall * c)
 {
-	ENTER_STACK_FRAME(c,"reguser.setproperty");
-
-	KviStr name,property,value;
-	if(!g_pUserParser->parseCmdSingleToken(c,name))return false;
-	if(!g_pUserParser->parseCmdSingleToken(c,property))return false;
-	if(!g_pUserParser->parseCmdFinalPart(c,value))return false;
-
-	if(name.isEmpty())
+	QString szName;
+	QString szProperty;
+	QString szValue;
+	KVSM_PARAMETERS_BEGIN(c) 
+		KVSM_PARAMETER("mask",KVS_PT_STRING,0,szName) 
+		KVSM_PARAMETER("property",KVS_PT_STRING,0,szProperty) 
+		KVSM_PARAMETER("value",KVS_PT_STRING,KVS_PF_OPTIONAL,szValue) 
+	KVSM_PARAMETERS_END(c) 
+	
+	if(szName.isEmpty())
 	{
-		if(!c->hasSwitch('q'))c->warning(__tr2qs("No name specified"));
-		return c->leaveStackFrame();
+		if(!c->hasSwitch('q',"quiet"))c->warning(__tr2qs("No name specified"));
+		return true;
 	}
 
-	if(property.isEmpty())
+	if(szProperty.isEmpty())
 	{
-		if(!c->hasSwitch('q'))c->warning(__tr2qs("No property specified"));
-		return c->leaveStackFrame();
+		if(!c->hasSwitch('q',"quiet"))c->warning(__tr2qs("No property specified"));
+		return true;
 	}
 
-	KviRegisteredUser * u = g_pRegisteredUserDataBase->findUserByName(name.ptr());
+	KviRegisteredUser * u = g_pRegisteredUserDataBase->findUserByName(szName);
 	if(!u)
 	{
-		if(!c->hasSwitch('q'))c->warning(__tr2qs("User %s not found"),name.ptr());
+		if(!c->hasSwitch('q',"quiet"))c->warning(__tr2qs("User %Q not found"),&szName);
 	} else {
-		u->setProperty(property.ptr(),value.ptr());
-		if(c->hasSwitch('n'))g_pApp->restartNotifyLists();
-		if(c->hasSwitch('a'))g_pApp->resetAvatarForMatchingUsers(u);
+		u->setProperty(szProperty,szValue);
+		if(c->hasSwitch('n',"restartnotifylists"))g_pApp->restartNotifyLists();
+		if(c->hasSwitch('a',"resetavatar"))g_pApp->resetAvatarForMatchingUsers(u);
 	}
-
-	return c->leaveStackFrame();
+	return true;
 }
-
 /*
 	@doc: reguser.list
 	@type:
@@ -518,14 +500,18 @@ static bool reguser_module_cmd_setproperty(KviModule *m,KviCommand *c)
 		[fnc]$reguser.exactMatch[/fnc]
 */
 
-static bool reguser_module_fnc_list(KviModule *m,KviCommand *c,KviParameterList * parms,KviStr &buffer)
-{
-	ENTER_STACK_FRAME(c,"reguser.list");
-	KviIrcMask mask(parms->safeFirstParam());
-
-
-	c->beginListArrayOrDictionaryReturnIdentifier();
-
+static bool reguser_kvs_fnc_list(KviKvsModuleFunctionCall * c)
+{ 
+	QString szMask; 
+	
+	KVSM_PARAMETERS_BEGIN(c)
+		KVSM_PARAMETER("user_mask",KVS_PT_STRING,KVS_PF_OPTIONAL,szMask)
+	KVSM_PARAMETERS_END(c)
+	
+	KviIrcMask mask(szMask);
+	KviKvsArray* pArray = new KviKvsArray();
+	int aid=0;
+	
 	int cnt = 0;
 
 	QDict<KviRegisteredUser> * d = g_pRegisteredUserDataBase->userDict();
@@ -536,15 +522,15 @@ static bool reguser_module_fnc_list(KviModule *m,KviCommand *c,KviParameterList
 		KviPtrList<KviIrcMask> * ml = u->maskList();
 		if(u->matches(mask) || (ml->count() == 0))
 		{
-			c->addListArrayOrDictionaryReturnValue(cnt++,u->name().utf8().data(),buffer);
+			pArray->set(aid,new KviKvsVariant(u->name()));
+			aid++;
 		}
 		++it;
 	}
-
-	return c->leaveStackFrame();
+	c->returnValue()->setArray(pArray);
+	return true;
 }
 
-
 /*
 	@doc: reguser.showlist
 	@type:
@@ -572,17 +558,14 @@ static bool reguser_module_fnc_list(KviModule *m,KviCommand *c,KviParameterList
 		[fnc]reguser.exactMatch[/fnc]
 */
 
-static bool reguser_module_cmd_showlist(KviModule *m,KviCommand *c)
+static bool reguser_kvs_cmd_showlist(KviKvsModuleCommandCall * c)
 {
-	ENTER_STACK_FRAME(c,"reguser.showlist");
+	QString szMask;
+	KVSM_PARAMETERS_BEGIN(c) 
+		KVSM_PARAMETER("mask",KVS_PT_STRING,KVS_PF_OPTIONAL,szMask) 
+	KVSM_PARAMETERS_END(c) 
 
-	KviStr usermask;
-	if(!g_pUserParser->parseCmdFinalPart(c,usermask))return false;
-
-	KviIrcMask mask(usermask.ptr());
-
-
-//#warning "FIND A BETTER KVI_OUT_*"
+	KviIrcMask mask(szMask);
 	c->window()->outputNoFmt(KVI_OUT_SYSTEMMESSAGE,__tr2qs("Registered users database dump:"));
 
 	int count = 0;
@@ -623,15 +606,11 @@ static bool reguser_module_cmd_showlist(KviModule *m,KviCommand *c)
 	}
 
 	c->window()->output(KVI_OUT_SYSTEMMESSAGE,__tr2qs("Total: %d matching users (of %d in the database)"),count,d->count());
-
-	return c->leaveStackFrame();
+	return true;
 }
 
 
 
-
-
-
 /*
 	@doc: reguser.match
 	@type:
@@ -655,16 +634,19 @@ static bool reguser_module_cmd_showlist(KviModule *m,KviCommand *c)
 		[fnc]reguser.exactMatch[/fnc]
 */
 
-static bool reguser_module_fnc_match(KviModule *m,KviCommand *c,KviParameterList * parms,KviStr &buffer)
-{
-	ENTER_STACK_FRAME(c,"reguser.match");
-	KviIrcMask mask(parms->safeFirstParam());
+static bool reguser_kvs_fnc_match(KviKvsModuleFunctionCall * c)
+{ 
+	QString szMask; 
+	
+	KVSM_PARAMETERS_BEGIN(c)
+		KVSM_PARAMETER("user_mask",KVS_PT_STRING,0,szMask)
+	KVSM_PARAMETERS_END(c)
+	KviIrcMask mask(szMask);
 	KviRegisteredUser * u = g_pRegisteredUserDataBase->findMatchingUser(mask.nick(),mask.user(),mask.host());
-	if(u)buffer.append(u->name());
-	return c->leaveStackFrame();
+	if(u) c->returnValue()->setString(u->name());
+	return true;
 }
 
-
 /*
 	@doc: reguser.exactmatch
 	@type:
@@ -688,13 +670,17 @@ static bool reguser_module_fnc_match(KviModule *m,KviCommand *c,KviParameterList
 		[fnc]reguser.match[/fnc]
 */
 
-static bool reguser_module_fnc_exactMatch(KviModule *m,KviCommand *c,KviParameterList * parms,KviStr &buffer)
-{
-	ENTER_STACK_FRAME(c,"reguser.exactmatch");
-	KviIrcMask mask(parms->safeFirstParam());
+static bool reguser_kvs_fnc_exactMatch(KviKvsModuleFunctionCall * c)
+{ 
+	QString szMask; 
+	
+	KVSM_PARAMETERS_BEGIN(c)
+		KVSM_PARAMETER("user_mask",KVS_PT_STRING,0,szMask)
+	KVSM_PARAMETERS_END(c)
+	KviIrcMask mask(szMask);
 	KviRegisteredUser * u = g_pRegisteredUserDataBase->findUserWithMask(mask);
-	if(u)buffer.append(u->name());
-	return c->leaveStackFrame();
+	if(u) c->returnValue()->setString(u->name());
+	return true;
 }
 
 ///*
@@ -784,30 +770,37 @@ static bool reguser_module_fnc_exactMatch(KviModule *m,KviCommand *c,KviParamete
 		[fnc]$reguser.matchProperty[/fnc]
 */
 
-static bool reguser_module_fnc_mask(KviModule *m,KviCommand *c,KviParameterList * parms,KviStr &buffer)
-{
-	ENTER_STACK_FRAME(c,"reguser.mask");
-	KviRegisteredUser * u = g_pRegisteredUserDataBase->findUserByName(parms->safeFirstParam());
+static bool reguser_kvs_fnc_mask(KviKvsModuleFunctionCall * c)
+{ 
+	QString szName; 
+	QString szN;
+	KVSM_PARAMETERS_BEGIN(c)
+		KVSM_PARAMETER("name",KVS_PT_STRING,0,szName)
+		KVSM_PARAMETER("N",KVS_PT_STRING,KVS_PF_OPTIONAL,szN)
+	KVSM_PARAMETERS_END(c)
+	
+	KviKvsArray* pArray = new KviKvsArray();
+	int aid=0;
+	KviRegisteredUser * u = g_pRegisteredUserDataBase->findUserByName(szName);
 	if(u)
 	{
-		KviStr n = parms->safeNextParam();
+		KviStr n = szName;
 		if(n.hasData() && n.isUnsignedNum())
 		{
 			KviIrcMask * m = u->maskList()->at(n.toInt());
-			if(m)buffer.append(KviStr::Format,"%Q!%Q@%Q",&(m->nick()),&(m->user()),&(m->host()));
+			if(m) c->returnValue()->setString(m->nick()+"!"+m->user()+"@"+m->host());
 		} else {
-			c->beginListArrayOrDictionaryReturnIdentifier();
 			int id=0;
 			for(KviIrcMask * m = u->maskList()->first();m;m = u->maskList()->next())
 			{
-				KviStr tmp(KviStr::Format,"%Q!%Q@%Q",&(m->nick()),&(m->user()),&(m->host()));
-				c->addListArrayOrDictionaryReturnValue(id++,tmp,buffer);
+				pArray->set(aid,new KviKvsVariant(QString(m->nick()+"!"+m->user()+"@"+m->host())));
+				aid++;
 			}
+			c->returnValue()->setArray(pArray);
 		}
-	} //else c->warning(__tr2qs("User %s not found"),parms->safeFirstParam());
-	return c->leaveStackFrame();
+	}
+	return true;
 }
-
 /*
 	@doc: reguser.property
 	@type:
@@ -840,20 +833,24 @@ static bool reguser_module_fnc_mask(KviModule *m,KviCommand *c,KviParameterList
 		[fnc]$reguser.matchProperty[/fnc]
 */
 
-static bool reguser_module_fnc_property(KviModule *m,KviCommand *c,KviParameterList * parms,KviStr &buffer)
-{
-	ENTER_STACK_FRAME(c,"reguser.property");
-	KviRegisteredUser * u = g_pRegisteredUserDataBase->findUserByName(parms->safeFirstParam());
+static bool reguser_kvs_fnc_property(KviKvsModuleFunctionCall * c)
+{ 
+	QString szName; 
+	QString szProperty;
+	KVSM_PARAMETERS_BEGIN(c)
+		KVSM_PARAMETER("user_name",KVS_PT_STRING,0,szName)
+		KVSM_PARAMETER("property_name",KVS_PT_STRING,KVS_PF_OPTIONAL,szProperty)
+	KVSM_PARAMETERS_END(c)
+	
+	KviRegisteredUser * u = g_pRegisteredUserDataBase->findUserByName(szName);
 	if(u)
 	{
 		QString tmp;
-		u->getProperty(parms->safeNextParam(),tmp);
-		buffer.append(tmp);
-	} //else c->warning(__tr2qs("User %s not found"),parms->safeFirstParam());
-	return c->leaveStackFrame();
+		u->getProperty(szProperty,tmp);
+		c->returnValue()->setString(tmp);
+	}
+	return true;
 }
-
-
 /*
 	@doc: reguser.matchproperty
 	@type:
@@ -889,22 +886,26 @@ static bool reguser_module_fnc_property(KviModule *m,KviCommand *c,KviParameterL
 		[fnc]reguser.property[/fnc]
 */
 
-static bool reguser_module_fnc_matchProperty(KviModule *m,KviCommand *c,KviParameterList * parms,KviStr &buffer)
-{
-	ENTER_STACK_FRAME(c,"reguser.matchProperty");
-	KviIrcMask mask(parms->safeFirstParam());
-
+static bool reguser_kvs_fnc_matchProperty(KviKvsModuleFunctionCall * c)
+{ 
+	QString szName; 
+	QString szProperty;
+	KVSM_PARAMETERS_BEGIN(c)
+		KVSM_PARAMETER("user_name",KVS_PT_STRING,0,szName)
+		KVSM_PARAMETER("property_name",KVS_PT_STRING,KVS_PF_OPTIONAL,szProperty)
+	KVSM_PARAMETERS_END(c)
+	
+	KviIrcMask mask(szName);
 	KviRegisteredUser * u = g_pRegisteredUserDataBase->findMatchingUser(mask.nick(),mask.user(),mask.host());
 	if(u)
 	{
 		QString tmp;
-		u->getProperty(parms->safeNextParam(),tmp);
-		buffer.append(tmp);
+		u->getProperty(szProperty,tmp);
+		c->returnValue()->setString(tmp);
 	}
-	return c->leaveStackFrame();
+	return true;
 }
 
-
 /*
 	@doc: reguser.wizard
 	@type:
@@ -926,42 +927,38 @@ static bool reguser_module_fnc_matchProperty(KviModule *m,KviCommand *c,KviParam
 		[cmd]reguser.add[cmd],
 */
 
-static bool reguser_module_cmd_wizard(KviModule *m,KviCommand *c)
+static bool reguser_kvs_cmd_wizard(KviKvsModuleCommandCall * c)
 {
-	ENTER_STACK_FRAME(c,"reguser.wizard");
+	QString szMask;
+	KVSM_PARAMETERS_BEGIN(c) 
+		KVSM_PARAMETER("mask",KVS_PT_STRING,KVS_PF_OPTIONAL,szMask) 
+	KVSM_PARAMETERS_END(c) 
 
-	KviStr name;
-
-	if(!g_pUserParser->parseCmdFinalPart(c,name))return false;
-
-	KviRegistrationWizard * w = new KviRegistrationWizard(name.ptr());
+	KviRegistrationWizard * w = new KviRegistrationWizard(szMask);
 	w->show();
-
-	return c->leaveStackFrame();
+	return true;
 }
 
-
 static bool reguser_module_init(KviModule * m)
 {
 	g_pRegistrationWizardList = new KviPtrList<KviRegistrationWizard>;
 	g_pRegistrationWizardList->setAutoDelete(true);
 
-	m->registerCommand("add",reguser_module_cmd_add);
-	m->registerCommand("remove",reguser_module_cmd_remove);
-	m->registerCommand("addmask",reguser_module_cmd_addmask);
-	m->registerCommand("delmask",reguser_module_cmd_delmask);
-	m->registerCommand("edit",reguser_module_cmd_edit);
-	m->registerCommand("showlist",reguser_module_cmd_showlist);
-	m->registerCommand("setproperty",reguser_module_cmd_setproperty); // setproperty <mask> <name> <value>
-	m->registerCommand("wizard",reguser_module_cmd_wizard);
-
-
-	m->registerFunction("match",reguser_module_fnc_match);
-	m->registerFunction("list",reguser_module_fnc_list);
-	m->registerFunction("exactmatch",reguser_module_fnc_exactMatch);
-	m->registerFunction("mask",reguser_module_fnc_mask);
-	m->registerFunction("property",reguser_module_fnc_property);
-	m->registerFunction("matchProperty",reguser_module_fnc_matchProperty);
+	KVSM_REGISTER_SIMPLE_COMMAND(m,"add",reguser_kvs_cmd_add);
+	KVSM_REGISTER_SIMPLE_COMMAND(m,"remove",reguser_kvs_cmd_remove);
+	KVSM_REGISTER_SIMPLE_COMMAND(m,"addmask",reguser_kvs_cmd_addmask);
+	KVSM_REGISTER_SIMPLE_COMMAND(m,"delmask",reguser_kvs_cmd_delmask);
+	KVSM_REGISTER_SIMPLE_COMMAND(m,"edit",reguser_kvs_cmd_edit);
+	KVSM_REGISTER_SIMPLE_COMMAND(m,"showlist",reguser_kvs_cmd_showlist);
+	KVSM_REGISTER_SIMPLE_COMMAND(m,"setproperty",reguser_kvs_cmd_setproperty);
+	KVSM_REGISTER_SIMPLE_COMMAND(m,"wizard",reguser_kvs_cmd_wizard);
+	
+	KVSM_REGISTER_FUNCTION(m,"match",reguser_kvs_fnc_match);
+	KVSM_REGISTER_FUNCTION(m,"list",reguser_kvs_fnc_list);
+	KVSM_REGISTER_FUNCTION(m,"exactMatch",reguser_kvs_fnc_exactMatch);
+	KVSM_REGISTER_FUNCTION(m,"mask",reguser_kvs_fnc_mask);
+	KVSM_REGISTER_FUNCTION(m,"property",reguser_kvs_fnc_property);
+	KVSM_REGISTER_FUNCTION(m,"matchProperty",reguser_kvs_fnc_matchProperty);
 
 	return true;
 }
