@@ -223,7 +223,18 @@
 		}
 
 		if(*inBuffer != KVI_TEXT_CRYPTESCAPE)
+		{
+			plainText = inBuffer;
 			return KviCryptEngine::DecryptOkWasPlainText;
+		}
+
+		inBuffer++;
+		
+		if(!*inBuffer)
+		{
+			plainText = inBuffer;
+			return KviCryptEngine::DecryptOkWasPlainText; // empty buffer
+		}
 
 		int len;
 		char * binary;
@@ -808,16 +819,6 @@ static bool rijndael_module_cleanup(KviModule *m)
 	delete g_pEngineList;
     g_pEngineList = 0;
 	m->unregisterCryptEngines();
-	m->unregisterMetaObject("KviRijndaelEngine");
-	m->unregisterMetaObject("KviRijndaelHexEngine");
-	m->unregisterMetaObject("KviRijndael128HexEngine");
-	m->unregisterMetaObject("KviRijndael192HexEngine");
-	m->unregisterMetaObject("KviRijndael256HexEngine");
-	m->unregisterMetaObject("KviRijndaelBase64Engine");
-	m->unregisterMetaObject("KviRijndael128Base64Engine");
-	m->unregisterMetaObject("KviRijndael192Base64Engine");
-	m->unregisterMetaObject("KviRijndael256Base64Engine");
-	m->unregisterMetaObject("KviMircryptionEngine");
 	return true;
 #else
 	return false;
