@@ -39,6 +39,8 @@
 #include <qdir.h>
 #include <qpushbutton.h>
 #include <qvalidator.h>
+#include <qtextcodec.h>
+#include <qhbox.h> 
 
 #ifdef COMPILE_ON_WINDOWS
 	#include <windows.h>
@@ -55,6 +57,11 @@ extern int g_iThemeToApply;
 extern QString g_szChoosenDefaultNick;
 
 
+#ifdef COMPILE_ON_WINDOWS
+	#define KVI_LOCAL_KVIRC_SUBDIRECTORY_NAME "KVIrc"
+#else
+	#define KVI_LOCAL_KVIRC_SUBDIRECTORY_NAME ".kvirc"
+#endif
 
 KviSetupPage::KviSetupPage(KviSetupWizard * w)
 : QHBox(w)
@@ -82,7 +89,7 @@ KviSetupPage::KviSetupPage(KviSetupWizard * w)
 	QLabel * l = new QLabel(m_pVBox);
 	l->setAlignment(Qt::AlignAuto | Qt::AlignTop);
 	QString szHeader = "<table width=\"100%\"><tr><td bgcolor=\"#303030\">" \
-			"<h1><font color=\"#FFFFFF\">KVIrc " VERSION "</font></h1>" \
+			"<h1><font color=\"#FFFFFF\">KVIrc " KVI_VERSION "</font></h1>" \
 			"</td></tr></table>";
 	l->setText(szHeader);
 
@@ -138,7 +145,7 @@ KviSetupWizard::KviSetupWizard()
 
 	/////////////////////////////////////////////////////////////////////////////////////////////////////////////
 	// License
-
+#ifndef COMPILE_ON_WINDOWS //it have been already shown by installer
 	KviSetupPage * m_pLicense = new KviSetupPage(this);
 	m_pLicense->m_pTextLabel->setText(__tr2qs( \
 			"<p>All of the files in this distribution are covered by the GPL. " \
@@ -155,13 +162,14 @@ KviSetupWizard::KviSetupWizard()
 	KviStr szLicense;
 	KviStr szLicensePath;
 	g_pApp->getGlobalKvircDirectory(szLicensePath,KviApp::License,"COPYING");
-	
-	if(!kvi_loadFile(szLicensePath.ptr(),szLicense))
+	QString buf;
+	if(!KviFileUtils::loadFile(szLicensePath.ptr(),buf))
 	{
 		szLicense = __tr("Oops... can't find the license file.\n" \
 						"It MUST be included in the distribution...\n" \
 						"Please report to <pragma at kvirc dot net>");
 	}
+	szLicense=buf;
 	ed->setText(szLicense.ptr());
 	
 	m_pLicense->m_pVBox->setStretchFactor(ed,1);
@@ -171,7 +179,9 @@ KviSetupWizard::KviSetupWizard()
 	setHelpEnabled(m_pLicense,false);
 
 	setCaption(__tr2qs("KVIrc Setup"));
-
+#else
+	m_pLicense = 0;
+#endif
 	/////////////////////////////////////////////////////////////////////////////////////////////////////////////
 	// Identity
 
@@ -179,7 +189,7 @@ KviSetupWizard::KviSetupWizard()
 
 	m_pIdentity->m_pTextLabel->setText(__tr2qs("Please choose a Nickname.<br><br>" \
 		"Your nickname is the name that other IRC users will know you by. " \
-		"It can't contain spaces or punctuation. Some IRC networks will shorten your nickname if it is more than 9 characters " \
+		"It can't contain spaces or punctuation. Some IRC networks will shorten your nickname if it is more than 32 characters " \
 		"long.<br><br>"
 		"If in doubt, just enter the first nick that comes to mind. " \
 		"You will be able to change it later in the Identity properties, or with the /NICK command."));
@@ -226,7 +236,7 @@ KviSetupWizard::KviSetupWizard()
 
 	addPage(m_pDirectory,__tr2qs("Application Folders"));
 
-	KviStr tmp;
+	QString tmp;
 
 	l = new QLabel(m_pDirectory->m_pVBox);
 	l->setText(__tr2qs("<b>Store configuration in folder:</b>"));
@@ -241,11 +251,17 @@ KviSetupWizard::KviSetupWizard()
 	hb->setSpacing(3);
 	hb->setStretchFactor(m_pDataPathEdit,1);
 
+#ifdef COMPILE_ON_WINDOWS
+	tmp = QTextCodec::codecForLocale()->toUnicode(getenv( "APPDATA" ));
+	if(tmp.isEmpty())
+		tmp = QDir::homeDirPath();
+#else 
 	tmp = QDir::homeDirPath();
-	tmp.ensureLastCharIs(KVI_PATH_SEPARATOR_CHAR);
+#endif //COMPILE_ON_WINDOWS
+	KviQString::ensureLastCharIs(tmp,KVI_PATH_SEPARATOR_CHAR);
 	tmp.append(KVI_LOCAL_KVIRC_SUBDIRECTORY_NAME);
-	kvi_adjustFilePath(tmp);
-	m_pDataPathEdit->setText(tmp.ptr());
+	KviFileUtils::adjustFilePath(tmp);
+	m_pDataPathEdit->setText(tmp);
 
 
 
@@ -264,10 +280,10 @@ KviSetupWizard::KviSetupWizard()
 	hb->setStretchFactor(m_pIncomingPathEdit,1);
 
 	tmp = QDir::homeDirPath();
-	tmp.ensureLastCharIs(KVI_PATH_SEPARATOR_CHAR);
+	KviQString::ensureLastCharIs(tmp,KVI_PATH_SEPARATOR_CHAR);
 	tmp.append(KVI_DEFAULT_INCOMING_SUBDIRECTORY_NAME);
-	kvi_adjustFilePath(tmp);
-	m_pIncomingPathEdit->setText(tmp.ptr());
+	KviFileUtils::adjustFilePath(tmp);
+	m_pIncomingPathEdit->setText(tmp);
 
 	l = new QLabel(m_pDirectory->m_pVBox,"<b> </b>");
 
@@ -288,10 +304,50 @@ KviSetupWizard::KviSetupWizard()
 		"If you don't know what to choose, just use the default.</p>"));
 
 	m_pThemeButtonGroup = new QVButtonGroup(m_pTheme->m_pVBox);
+	
+	QHBox* pThemesHb = new QHBox(m_pThemeButtonGroup);
 
-	m_pThemeHiRes = new QRadioButton(__tr2qs("&Fancy Theme"),m_pThemeButtonGroup);
-	m_pThemeLoRes  = new QRadioButton(__tr2qs("&Minimalist Theme"),m_pThemeButtonGroup);
+	QVBox* pThemesVb = new QVBox(pThemesHb);
+	
+	QString szThemeImagePath;
+	g_pApp->getGlobalKvircDirectory(szThemeImagePath,KviApp::Pics,"kvi_setup_theme_hires.png");
+	
+	QPixmap* pHiResPixmap = new QPixmap(szThemeImagePath);
+	if(pHiResPixmap->isNull())
+	{
+		delete pHiResPixmap;
+		pHiResPixmap= new QPixmap(250,200);
+	}
+	
+	QLabel* pPixmapLabelHiRes = new QLabel(pThemesVb);
+
+	pPixmapLabelHiRes->setPixmap(*pHiResPixmap);
+	pPixmapLabelHiRes->setFixedSize(pHiResPixmap->size());
+	pPixmapLabelHiRes->setFrameStyle(QFrame::Sunken | QFrame::StyledPanel);
+	pPixmapLabelHiRes->setMargin(0);
+	
+	m_pThemeHiRes = new QRadioButton(__tr2qs("&Fancy Theme"),pThemesVb);
+	
+	pThemesVb = new QVBox(pThemesHb);
+	g_pApp->getGlobalKvircDirectory(szThemeImagePath,KviApp::Pics,"kvi_setup_theme_lowres.png");
+	QPixmap* pLowResPixmap = new QPixmap(szThemeImagePath);
+	if(pLowResPixmap->isNull())
+	{
+		delete pLowResPixmap;
+		pLowResPixmap= new QPixmap(250,200);
+	}
+	
+	QLabel* pPixmapLabelLowRes = new QLabel(pThemesVb);
+
+	pPixmapLabelLowRes->setPixmap(*pLowResPixmap);
+	pPixmapLabelLowRes->setFixedSize(pLowResPixmap->size());
+	pPixmapLabelLowRes->setFrameStyle(QFrame::Sunken | QFrame::StyledPanel);
+	pPixmapLabelLowRes->setMargin(0);
+	
+	m_pThemeLoRes  = new QRadioButton(__tr2qs("&Minimalist Theme"),pThemesVb);
 	m_pThemeNone = new QRadioButton(__tr2qs("&Don't apply any theme"),m_pThemeButtonGroup);
+	m_pThemeButtonGroup->insert(m_pThemeHiRes);
+	m_pThemeButtonGroup->insert(m_pThemeLoRes);
 	
 	m_pThemeHiRes->setChecked(true);
 
@@ -299,17 +355,7 @@ KviSetupWizard::KviSetupWizard()
 
 	setHelpEnabled(m_pTheme,false);
 
-#undef WANT_LINK_ON_DESKTOP
-
-#ifdef COMPILE_ON_WINDOWS
-	#define WANT_LINK_ON_DESKTOP
-#else
-	#ifdef COMPILE_KDE_SUPPORT
-		#define WANT_LINK_ON_DESKTOP
-	#endif
-#endif
-
-#ifdef WANT_LINK_ON_DESKTOP
+#ifdef COMPILE_KDE_SUPPORT
 	/////////////////////////////////////////////////////////////////////////////////////////////////////////////
 	// Link
 	m_pLink = new KviSetupPage(this);
@@ -454,10 +500,10 @@ void KviSetupWizard::makeLink()
     }
 
 	// Build our paths
-	KviStr szLinkTarget = szLink;
+	QString szLinkTarget = szLink;
 	szLinkTarget.append("\\kvirc.lnk");
 
-	KviStr szKvircExec = g_pApp->m_szGlobalKvircDir;
+	QString szKvircExec = g_pApp->m_szGlobalKvircDir;
 	szKvircExec.append("\\kvirc.exe");
 
 	// Trigger a horrible machinery
@@ -480,13 +526,13 @@ void KviSetupWizard::makeLink()
 		{
 			WORD wsz[MAX_PATH];
 			// Set the path to the shell link target.
-			psl->SetPath(szKvircExec.ptr());
+			psl->SetPath(QTextCodec::codecForLocale()->fromUnicode(szKvircExec).data());
 			// Set the description of the shell link.
 			psl->SetDescription("kvirc");
 	        // Ensure string is ANSI.
-			MultiByteToWideChar(CP_ACP,0,szLinkTarget.ptr(),-1,wsz,MAX_PATH);
+			MultiByteToWideChar(CP_ACP,0,QTextCodec::codecForLocale()->fromUnicode(szLinkTarget).data(),-1,(LPWSTR)wsz,MAX_PATH);
 			// Save the link via the IPersistFile::Save method.
-			ppf->Save(wsz,true);    
+			ppf->Save((LPCOLESTR)wsz,true);    
 			ppf->Release();
 		}
 		psl->Release();
@@ -496,13 +542,13 @@ void KviSetupWizard::makeLink()
 #endif //COMPILE_ON_WINDOWS
 
 #ifdef COMPILE_KDE_SUPPORT
-	KviStr tmp = QDir::homeDirPath();
-	tmp.ensureLastCharIs(KVI_PATH_SEPARATOR_CHAR);
+	QString tmp = QDir::homeDirPath();
+	KviQString::ensureLastCharIs(tmp,KVI_PATH_SEPARATOR_CHAR);
 	tmp.append("Desktop");
-	tmp.ensureLastCharIs(KVI_PATH_SEPARATOR_CHAR);
+	KviQString::ensureLastCharIs(tmp,KVI_PATH_SEPARATOR_CHAR);
 	tmp.append("kvirc.desktop");
 
-	KviStr contents = "[Desktop Entry]\n" \
+	QString contents = "[Desktop Entry]\n" \
 		"GenericName=IRC Client\n" \
 		"Comment=Connect to Internet Relay Chat\n" \
 		"Exec=kvirc -m %u\n" \
@@ -513,7 +559,7 @@ void KviSetupWizard::makeLink()
 		"Type=Application\n" \
 		"X-KDE-SubstituteUID=false\n";
 
-	kvi_writeFile(tmp.ptr(),contents,false);
+	KviFileUtils::writeFile(tmp,contents,false);
 #endif //COMPILE_KDE_SUPPORT
 
 }
@@ -521,7 +567,7 @@ void KviSetupWizard::makeLink()
 void KviSetupWizard::setUrlHandlers()
 {
 #ifdef COMPILE_ON_WINDOWS
-	KviStr szReg = "REGEDIT4\r\n" \
+	QString szReg = "REGEDIT4\r\n" \
 		"\r\n" \
 		"[HKEY_CLASSES_ROOT\\irc]\r\n" \
 		"@=\"URL:IRC Protocol\"\r\n" \
@@ -559,22 +605,22 @@ void KviSetupWizard::setUrlHandlers()
 		"[HKEY_CLASSES_ROOT\\irc6\\shell\\open\\command]\r\n" \
 		"@=\"\\\"@KVIRCEXECUTABLE@\\\" %1\"\r\n";
 
-	KviStr szKvircExec = g_pApp->m_szGlobalKvircDir;
+	QString szKvircExec = g_pApp->m_szGlobalKvircDir;
 	szKvircExec.append("\\kvirc.exe");
-	szKvircExec.replaceAll("\\","\\\\");
+	szKvircExec.replace("\\","\\\\");
 
-	szReg.replaceAll("@KVIRCEXECUTABLE@",szKvircExec.ptr());
+	szReg.replace("@KVIRCEXECUTABLE@",QTextCodec::codecForLocale()->fromUnicode(szKvircExec));
 
-	KviStr szRegFile = g_pApp->m_szGlobalKvircDir;
+	QString szRegFile = g_pApp->m_szGlobalKvircDir;
 	szRegFile.append("\\kvirc.reg");
 
-	kvi_writeFile(szRegFile.ptr(),szReg,false);
+	KviFileUtils::writeFile(szRegFile,szReg,false);
 
-	KviStr szCmd = "regedit /S \"";
+	QString szCmd = "regedit /S \"";
 	szCmd += szRegFile;
 	szCmd += "\"";
 
-	WinExec(szCmd.ptr(),SW_SHOW);
+	WinExec(QTextCodec::codecForLocale()->fromUnicode(szCmd).data(),SW_SHOW);
 #endif
 }
 
@@ -589,13 +635,13 @@ void KviSetupWizard::reject()
 
 void KviSetupWizard::accept()
 {
-	KviStr szDir = m_pDataPathEdit->text();
-	kvi_adjustFilePath(szDir);
-	if(!kvi_directoryExists(szDir.ptr()))
+	QString szDir = m_pDataPathEdit->text();
+	KviFileUtils::adjustFilePath(szDir);
+	if(!KviFileUtils::directoryExists(szDir))
 	{
-		if(!kvi_makeDir(szDir.ptr()))
+		if(!KviFileUtils::makeDir(szDir))
 		{
-			kvi_warningBox(__tr("Cannot create directory %s.\n" \
+			KviMessageBox::warning(__tr("Cannot create directory %s.\n" \
 				"You may not have write permission " \
 				"for that path. Please go back and choose another directory."));
 			showPage(m_pDirectory);
@@ -610,15 +656,15 @@ void KviSetupWizard::accept()
 	}*/
 
 	g_pApp->m_szLocalKvircDir = szDir;
-	g_pApp->m_szLocalKvircDir.ensureLastCharIs(KVI_PATH_SEPARATOR_CHAR);
+	KviFileUtils::adjustFilePath(g_pApp->m_szLocalKvircDir);
 
 	szDir = m_pIncomingPathEdit->text();
-	kvi_adjustFilePath(szDir);
-	if(!kvi_directoryExists(szDir.ptr()))
+	KviFileUtils::adjustFilePath(szDir);
+	if(!KviFileUtils::directoryExists(szDir))
 	{
-		if(!kvi_makeDir(szDir.ptr()))
+		if(!KviFileUtils::makeDir(szDir))
 		{
-			kvi_warningBox(__tr("Cannot create directory %s.\n" \
+			KviMessageBox::warning(__tr("Cannot create directory %s.\n" \
 				"You may not have write permission " \
 				"for that path. Please go back and choose another directory."));
 			showPage(m_pDirectory);
@@ -630,9 +676,9 @@ void KviSetupWizard::accept()
 
 #ifndef COMPILE_ON_WINDOWS
 	// Make local->global link
-	KviStr localPath(KviStr::Format,"%s/global",g_pApp->m_szLocalKvircDir.ptr());
-	unlink(localPath.ptr());
-	symlink(g_pApp->m_szGlobalKvircDir.ptr(),localPath.ptr());
+	QString localPath = QString("%1/global").arg(g_pApp->m_szLocalKvircDir);
+	unlink(QTextCodec::codecForLocale()->fromUnicode(localPath).data());
+	symlink(QTextCodec::codecForLocale()->fromUnicode(g_pApp->m_szGlobalKvircDir).data(),QTextCodec::codecForLocale()->fromUnicode(localPath).data());
 #endif
 
 	if(m_pLink)
