@@ -35,6 +35,7 @@
 #include "kvi_themedlabel.h"
 #include "kvi_input.h"
 #include "kvi_filedialog.h"
+#include "kvi_styled_controls.h"
 
 #include <qdatetimeedit.h>
 #include <qlabel.h>
@@ -80,7 +81,7 @@ KviSharedFileEditDialog::KviSharedFileEditDialog(QWidget * par,KviSharedFile * f
 	m_pUserMaskEdit = new QLineEdit(this);
 	g->addMultiCellWidget( m_pUserMaskEdit, 2, 2, 1, 3 );
 
-	m_pExpireCheckBox = new QCheckBox(__tr2qs_ctx("Expire at:","sharedfileswindow"),this);
+	m_pExpireCheckBox = new KviStyledCheckBox(__tr2qs_ctx("Expire at:","sharedfileswindow"),this);
 	g->addWidget(m_pExpireCheckBox,3,0);
 
 	m_pExpireDateTimeEdit = new QDateTimeEdit(this);
