@@ -18,12 +18,13 @@
 //
 
 
-
+#include "kvi_styled_controls.h"
 #include "kvi_module.h"
 #include "libkviurl.h"
 #include "kvi_app.h"
 #include "kvi_frame.h"
-#include "kvi_event.h"
+#include "kvi_kvs_eventmanager.h"
+//#include "kvi_uparser.h"
 #include "kvi_menubar.h"
 #include "kvi_internalcmd.h"
 #include "kvi_iconmanager.h"
@@ -65,6 +66,15 @@ bool urllist();
 void url_module_help();
 
 
+#define KVI_URL_EXTENSION_NAME "URL module extenstion"
+
+static KviModuleExtension * url_extension_alloc(KviModuleExtensionAllocStruct * s)
+{
+	urllist();
+	return 0;
+}
+
+
 KviUrlAction::KviUrlAction(QObject * pParent)
 : KviKvsAction(pParent,
 	"url.list",
@@ -199,7 +209,7 @@ void UrlDialog::remove()
 	}
 	for(KviUrl *tmp=g_pList->first();tmp;tmp=g_pList->next())
 	{
-		if (tmp->url == KviStr(m_pUrlList->currentItem()->text(0))) {
+		if (tmp->url == m_pUrlList->currentItem()->text(0)) {
 			g_pList->find(tmp);
 			g_pList->remove();
 			m_pUrlList->takeItem(m_pUrlList->currentItem());
@@ -236,8 +246,9 @@ void UrlDialog::findtext()
 
 void UrlDialog::dblclk_url(QListViewItem *item)
 {
-	KviStr cmd(KviStr::Format,"openurl %s",item->text(0).latin1());
-	g_pUserParser->parseCommandBuffer(cmd.ptr(),this);
+	QString cmd="openurl ";
+	cmd.append(item->text(0));
+	KviKvsScript::run(cmd,this);
 }
 
 void UrlDialog::popup(QListViewItem *item, const QPoint &point, int col)
@@ -264,12 +275,10 @@ void UrlDialog::popup(QListViewItem *item, const QPoint &point, int col)
 
 void UrlDialog::sayToWin(int itemID)
 {
-	KviStr say="PRIVMSG %1 %2";
-	KviWindow *wnd = g_pApp->findWindowByCaption(m_pListPopup->text(itemID).latin1());
-	say.replaceAll("%1",wnd->windowName().latin1());
-	say.replaceAll("%2",m_szUrl.ptr());
+	KviWindow *wnd = g_pApp->findWindowByCaption(m_pListPopup->text(itemID).utf8().data());
+	QString say=QString("PRIVMSG %1 %2").arg(wnd->windowName()).arg(m_szUrl.ptr());
 	if (wnd) {
-		g_pUserParser->parseCommandBuffer(say.ptr(),wnd);
+		KviKvsScript::run(say,wnd);
 		wnd->raise();
 		wnd->setActiveWindow();
 		wnd->setFocus();
@@ -335,11 +344,11 @@ ConfigDialog::ConfigDialog()
 	KviConfig *cfg = new KviConfig(szConfigPath.ptr(),KviConfig::Read);
 	cfg->setGroup("ConfigDialog");
 
-	cb[0] = new QCheckBox(__tr2qs("Save URL list on module unload"),this);
+	cb[0] = new KviStyledCheckBox(__tr2qs("Save URL list on module unload"),this);
 	cb[0]->setChecked(cfg->readBoolEntry("SaveUrlListOnUnload",false));
 	g->addMultiCellWidget(cb[0],0,0,0,1);
 
-	cb[1] = new QCheckBox(__tr2qs("Save columns width on URL list close"),this);
+	cb[1] = new KviStyledCheckBox(__tr2qs("Save columns width on URL list close"),this);
 	cb[1]->setChecked(cfg->readBoolEntry("SaveColumnWidthOnClose",false));
 	g->addMultiCellWidget(cb[1],1,1,0,1);
 
@@ -402,7 +411,7 @@ BanFrame::BanFrame(QWidget *parent, const char *name, bool banEnabled)
 
 	QGridLayout *g = new QGridLayout(this,2,2,10,10);
 
-	m_pEnable = new QCheckBox(__tr2qs("Enable URL ban list"),this);
+	m_pEnable = new KviStyledCheckBox(__tr2qs("Enable URL ban list"),this);
 	connect(m_pEnable,SIGNAL(clicked()),this,SLOT(enableClicked()));
 	m_pEnable->setChecked(banEnabled);
 	g->addMultiCellWidget(m_pEnable,0,0,0,1);
@@ -450,7 +459,7 @@ void BanFrame::removeBan()
 		QMessageBox::warning(0,__tr2qs("Warning - KVIrc"),__tr2qs("Select a ban."),QMessageBox::Ok,QMessageBox::NoButton,QMessageBox::NoButton);
 		return;
 	}
-	KviStr item(m_pBanList->text(i).latin1());
+	KviStr item(m_pBanList->text(i).utf8().data());
 	for(KviStr *tmp=g_pBanList->first();tmp;tmp=g_pBanList->next())
 	{
 		if (*tmp == item) g_pBanList->remove();
@@ -478,23 +487,23 @@ BanFrame::~BanFrame()
 
 void saveUrlList()
 {
-	KviStr urllist;
+	QString urllist;
 	g_pApp->getLocalKvircDirectory(urllist,KviApp::ConfigPlugins);
 	urllist += g_pUrlListFilename;
 	QFile file;
-	file.setName(urllist.ptr());
+	file.setName(urllist);
 	file.open(IO_WriteOnly);
 
 	QTextStream stream(&file);
-
+	
 	stream << g_pList->count() << endl;
 
 	for(KviUrl *tmp=g_pList->first();tmp;tmp=g_pList->next())
 	{
-		stream << tmp->url.ptr() << endl;
-		stream << tmp->window.ptr() << endl;
+		stream << tmp->url << endl;
+		stream << tmp->window << endl;
 		stream << tmp->count << endl;
-		stream << tmp->timestamp.ptr() << endl;
+		stream << tmp->timestamp << endl;
 	}
 	file.flush();
 	file.close();
@@ -506,7 +515,7 @@ void loadUrlList()
 	g_pApp->getLocalKvircDirectory(urllist,KviApp::ConfigPlugins);
 	urllist += g_pUrlListFilename;
 	QFile file;
-	file.setName(urllist.ptr());
+	file.setName(QString::fromUtf8(urllist.ptr()));
 	if (!file.open(IO_ReadOnly))return;
 	QTextStream stream(&file);
 
@@ -531,7 +540,7 @@ void loadUrlList()
 			if (tmpitem->dlg) {
 				QString tmpCount;
 				tmpCount.setNum(tmp->count);
-				tmpitem->dlg->addUrl(QString(tmp->url.ptr()), QString(tmp->window.ptr()), tmpCount, QString(tmp->timestamp.ptr()));
+				tmpitem->dlg->addUrl(QString(tmp->url), QString(tmp->window), tmpCount, QString(tmp->timestamp));
 			}
 		}
 		i++;
@@ -545,7 +554,7 @@ void saveBanList()
 	g_pApp->getLocalKvircDirectory(banlist,KviApp::ConfigPlugins);
 	banlist += g_pBanListFilename;
 	QFile file;
-	file.setName(banlist.ptr());
+	file.setName(QString::fromUtf8(banlist.ptr()));
 	file.open(IO_WriteOnly);
 
 	QTextStream stream(&file);
@@ -565,7 +574,7 @@ void loadBanList()
 	g_pApp->getLocalKvircDirectory(banlist,KviApp::ConfigPlugins);
 	banlist += g_pBanListFilename;
 	QFile file;
-	file.setName(banlist.ptr());
+	file.setName(QString::fromUtf8(banlist.ptr()));
 	if (!file.open(IO_ReadOnly))return;
 	QTextStream stream(&file);
 
@@ -614,6 +623,15 @@ bool urllist_command(KviModule *m,KviCommand *c)
 	return c->leaveStackFrame();
 }
 
+#ifdef COMPILE_NEW_KVS
+static bool url_kvs_cmd_list(KviKvsModuleCommandCall * c)
+{
+	urllist();
+	return true;
+}
+#endif
+
+
 UrlDlgList *findFrame()
 {
 	UrlDlgList *tmpitem = g_pUrlDlgList->first();
@@ -639,7 +657,7 @@ bool urllist()
 	{
 		QString tmpCount;
 		tmpCount.setNum(tmp->count);
-		tmpitem->dlg->addUrl(QString(tmp->url.ptr()), QString(tmp->window.ptr()), tmpCount, QString(tmp->timestamp.ptr()));
+		tmpitem->dlg->addUrl(QString(tmp->url), QString(tmp->window), tmpCount, QString(tmp->timestamp));
 	}
 	return true;
 }
@@ -675,19 +693,27 @@ bool urllistconfig(KviModule *m,KviCommand *c)
 	return c->leaveStackFrame();
 }
 
-int check_url(KviWindow *w,KviParameterList *p)		// return 0 if no occurence of the url were found
+#ifdef COMPILE_NEW_KVS
+static bool url_kvs_cmd_config(KviKvsModuleCommandCall * c)
+{
+	if (!g_pConfigDialog) g_pConfigDialog = new ConfigDialog();
+	return true;
+}
+#endif
+
+int check_url(KviWindow *w,const QString &szUrl) // return 0 if no occurence of the url were found
 {
 	int tmp = 0;
 
 	for(KviStr *tmpi=g_pBanList->first();tmpi;tmpi=g_pBanList->next())
 	{
-		if (p->safeFirst()->contains(tmpi->ptr())) tmp++;
+		if (szUrl.find(QString(tmpi->ptr())) != -1) tmp++;
 	}
 	if (tmp > 0) return tmp;
 
 	for(KviUrl *u = g_pList->first();u;u=g_pList->next())
 	{
-		if (u->url == p->safeFirst()->ptr()) {
+		if (u->url == szUrl) {
 			u->window = w->plainTextCaption();
 			u->count++;
 			tmp++;
@@ -699,7 +725,7 @@ int check_url(KviWindow *w,KviParameterList *p)		// return 0 if no occurence of
 			QListViewItemIterator lvi(tmpitem->dlg->m_pUrlList);
 			for(;lvi.current();++lvi)
 			{
-				if (lvi.current()->text(0) == p->safeFirst()->ptr()) {
+				if (lvi.current()->text(0) == szUrl) {
 					int tmpCount = lvi.current()->text(2).toInt();
 					tmpCount++;
 					QString tmpStr;
@@ -713,9 +739,14 @@ int check_url(KviWindow *w,KviParameterList *p)		// return 0 if no occurence of
 	return tmp;
 }
 
-bool urllist_module_event_onUrl(KviModule *m,KviWindow *w,KviParameterList * p)
+bool urllist_module_event_onUrl(KviKvsModuleEventCall * c)
 {
-	if (check_url(w,p) == 0) {
+	KviKvsVariant * vUrl = c->firstParam();
+	QString szUrl;
+	if(vUrl)vUrl->asString(szUrl);
+
+	if (check_url(c->window(),szUrl) == 0)
+	{
 
 		KviUrl *tmp = new KviUrl;
 		KviStr tmpTimestamp;
@@ -723,8 +754,8 @@ bool urllist_module_event_onUrl(KviModule *m,KviWindow *w,KviParameterList * p)
 		KviStr date(KviStr::Format,"%d-%d%d-%d%d",d.year(),d.month() / 10,d.month() % 10,d.day() / 10,d.day() % 10);
 		tmpTimestamp = "["+date+"]"+" [";
 		tmpTimestamp += QTime::currentTime().toString()+"]";
-		tmp->url = *(p->safeFirst());
-		tmp->window = w->plainTextCaption();
+		tmp->url = szUrl;
+		tmp->window = c->window()->plainTextCaption();
 		tmp->count = 1;
 		tmp->timestamp = tmpTimestamp;
 
@@ -733,7 +764,7 @@ bool urllist_module_event_onUrl(KviModule *m,KviWindow *w,KviParameterList * p)
 			if (tmpitem->dlg) {
 				QString tmpCount;
 				tmpCount.setNum(tmp->count);
-				tmpitem->dlg->addUrl(QString(tmp->url.ptr()), QString(tmp->window.ptr()), tmpCount, QString(tmp->timestamp.ptr()));
+				tmpitem->dlg->addUrl(QString(tmp->url), QString(tmp->window), tmpCount, QString(tmp->timestamp));
 				tmpitem->dlg->taskBarItem()->highlight(false);
 			}
 		}
@@ -758,7 +789,11 @@ bool urllist_module_event_onUrl(KviModule *m,KviWindow *w,KviParameterList * p)
 
 static bool url_module_init(KviModule *m)
 {
-
+	KviModuleExtensionDescriptor * d = m->registerExtension("tool",
+							KVI_URL_EXTENSION_NAME,
+							__tr2qs("View URL list"),
+							url_extension_alloc);
+	if(d)d->setIcon(*(g_pIconManager->getSmallIcon(KVI_SMALLICON_URL)));
 
 	g_pList = new KviPtrList<KviUrl>;
 	g_pList->setAutoDelete(true);
@@ -771,13 +806,12 @@ static bool url_module_init(KviModule *m)
 
 	g_pUrlIconPixmap = new QPixmap(url_icon_xpm);
 
-	m->registerCommand("list",urllist_command);
-	m->registerCommand("config",urllistconfig);
-
+	KVSM_REGISTER_SIMPLE_COMMAND(m,"list",url_kvs_cmd_list);
+	KVSM_REGISTER_SIMPLE_COMMAND(m,"config",url_kvs_cmd_config);
 	g_pUrlAction = new KviUrlAction(KviActionManager::instance());
 	KviActionManager::instance()->registerAction(g_pUrlAction);
 
-	m->registerEventHandler(KviEvent_OnUrl,urllist_module_event_onUrl);
+	m->kvsRegisterAppEventHandler(KviEvent_OnUrl,urllist_module_event_onUrl);
 
 	g_pApp->getLocalKvircDirectory(szConfigPath,KviApp::ConfigPlugins,"url.conf");
 
@@ -787,6 +821,7 @@ static bool url_module_init(KviModule *m)
 	UrlDlgList *udl = new UrlDlgList();
 	udl->dlg = 0;
 	g_pUrlDlgList->append(udl);
+	
 	return true;
 }
 
@@ -809,12 +844,6 @@ static bool url_module_cleanup(KviModule *m)
 	delete g_pUrlAction;
 	g_pUrlAction = 0;
 
-	m->unregisterAllEventHandlers();
-	m->unregisterAllCommands();
-
-	m->unregisterMetaObject("UrlDialog");
-	m->unregisterMetaObject("BanFrame");
-	m->unregisterMetaObject("ConfigDialog");
 
 	delete g_pUrlIconPixmap;
     g_pUrlIconPixmap = 0;
